# -*- coding: utf-8 -*-
# pylint: disable=chained-comparison,invalid-name,line-too-long,protected-access,too-many-arguments,too-many-boolean-expressions,too-many-branches,too-many-lines,too-many-locals,too-many-statements,unneeded-not,unused-import
# pylint: disable=simplifiable-if-statement
u"""
Interfaces for the NAG Mark 27.3 `lapackeig` Chapter.

``lapackeig`` - Least Squares and Eigenvalue Problems (LAPACK)

This module provides functions for the solution of linear least squares
problems, eigenvalue problems and singular value problems, as well as
associated computations.
It provides functions for:

    solution of linear least squares problems;

    solution of symmetric eigenvalue problems;

    solution of nonsymmetric eigenvalue problems;

    solution of singular value problems;

    solution of generalized linear least squares problems;

    solution of generalized symmetric-definite eigenvalue problems;

    solution of generalized nonsymmetric eigenvalue problems;

    solution of generalized singular value problems;

    matrix factorizations associated with the above problems;

    estimating condition numbers of eigenvalue and eigenvector problems;

    estimating the numerical rank of a matrix;

    solution of the Sylvester matrix equation.

Functions are provided for both `real` and `complex` data.

For a general introduction to the solution of linear least squares
problems, you should turn first to submodule ``linsys``.
The decision trees, at the end of submodule ``linsys``, direct you to
the most appropriate functions in submodule ``linsys`` or submodule
``lapackeig``. Submodule ``linsys`` and submodule ``lapackeig`` contain
`Black Box` (or `driver`) functions which enable standard linear least
squares problems to be solved by a call to a single function.

For a general introduction to eigenvalue and singular value problems,
you should turn first to submodule ``eigen``.
The decision trees, at the end of submodule ``eigen``, direct you to the
most appropriate functions in submodule ``eigen`` or submodule
``lapackeig``. Submodule ``eigen`` and submodule ``lapackeig`` contain
`Black Box` (or `driver`) functions which enable standard types of
problem to be solved by a call to a single function.
Often functions in submodule ``eigen`` call submodule ``lapackeig``
functions to perform the necessary computational tasks.

The functions in this module (submodule ``lapackeig``) handle only
`dense`, `band`, `tridiagonal` and `Hessenberg` matrices (not matrices
with more specialised structures, or general sparse matrices).
The tables in [Available Functions] and the decision trees in [Decision
Trees] direct you to the most appropriate functions in submodule
``lapackeig``.

The functions in this module have all been derived from the LAPACK
project (see Anderson `et al.` (1999)).
They have been designed to be efficient on a wide range of
high-performance computers, without compromising efficiency on
conventional serial machines.

For full information please refer to the NAG Library document

https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08intro.html
"""

# NAG Copyright 2017-2021.

import collections as _collections

from ..base import lapackeig as _b_lapackeig
from ..base import utils as _b_utils
from ..library import _utils

DgelsReturnData = _collections.namedtuple(
    'DgelsReturnData',
    ['a', 'b'],
)

def dgels(trans, a, b):
    u"""
    Solves a real linear least squares problem of full rank.

    ``dgels`` solves linear least squares problems of the form

        min_x||b-Ax||_2 or min_x||b-A^Tx||_2,

    where A is an m*n real matrix of full rank, using a QR or LQ
    factorization of A.

    For full information please refer to the NAG Library document for
    f08aa

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08aaf.html

    Parameters
    ----------
    trans : str, length 1
        If `trans` = 'N', the linear system involves A.

        If `trans` = 'T', the linear system involves A^T.

    a : float, array-like, shape (m, n)
        The m*n matrix A.

    b : float, array-like, shape (max(m, n), nrhs)
        The matrix B of right-hand side vectors, stored in columns; `b`
        is m*r if `trans` = 'N', or n*r if `trans` = 'T'.

    Returns
    -------
    a : float, ndarray, shape (m, n)
        If m >= n, `a` is overwritten by details of its QR
        factorization, as returned by ``dgeqrf``.

        If m < n, `a` is overwritten by details of its LQ factorization,
        as returned by ``dgelqf``.

    b : float, ndarray, shape (max(m, n), nrhs)
        `b` is overwritten by the solution vectors, x, stored in
        columns:

            if `trans` = 'N' and m >= n, or `trans` = 'T' and m < n,
            elements 1 to min(m, n) in each column of `b` contain the
            least squares solution vectors; the residual sum of squares
            for the solution is given by the sum of squares of the
            modulus of elements (min(m, n)+1) to max(m, n) in that
            column;

            otherwise, elements 1 to max(m, n) in each column of `b`
            contain the minimum norm solution vectors.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'T'.

        (`errno` -2)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -4)
            On entry, error in parameter nrhs.

            Constraint: nrhs >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            Diagonal element *<value>* of the triangular factor of A is
            zero, so that A does not have full rank; the least squares
            solution could not be computed.

    Notes
    -----
    The following options are provided:

    (1) If `trans` = 'N' and m >= n: find the least squares solution of
        an overdetermined system, i.e., solve the least squares problem

            min_x||b-Ax||_2.

    (#) If `trans` = 'N' and m < n: find the minimum norm solution of an
        underdetermined system Ax = b.

    (#) If `trans` = 'T' and m >= n: find the minimum norm solution of
        an undetermined system A^Tx = b.

    (#) If `trans` = 'T' and m < n: find the least squares solution of
        an overdetermined system, i.e., solve the least squares problem

            min_x||b-A^Tx||_2.

    Several right-hand side vectors b and solution vectors x can be
    handled in a single call; they are stored as the columns of the m*r
    right-hand side matrix B and the n*r solution matrix X.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dgels'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _nrhs_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[1]
    _nrhs = _b_utils._EngineIntScalarType(
        _nrhs_py, {'fun_name': fun_name, 'entity': '_nrhs'},
    ).to_ctypes()
    try:
        _b_lapackeig.dgels(
            trans,
            _m,
            _n,
            _nrhs,
            _a,
            _b,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _a_py = _a
    _b_py = _b
    _return_data = DgelsReturnData(
        _a_py,
        _b_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

DgeqrtReturnData = _collections.namedtuple(
    'DgeqrtReturnData',
    ['a', 't'],
)

def dgeqrt(nb, a):
    u"""
    Performs a QR factorization of real general rectangular matrix, with
    explicit blocking.

    ``dgeqrt`` recursively computes, with explicit blocking, the QR
    factorization of a real m*n matrix.

    For full information please refer to the NAG Library document for
    f08ab

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08abf.html

    Parameters
    ----------
    nb : int
        The explicitly chosen block size to be used in computing the QR
        factorization. See [Further Comments] for details.

    a : float, array-like, shape (m, n)
        The m*n matrix A.

    Returns
    -------
    a : float, ndarray, shape (m, n)
        If m >= n, the elements below the diagonal are overwritten by
        details of the orthogonal matrix Q and the upper triangle is
        overwritten by the corresponding elements of the n*n upper
        triangular matrix R.

        If m < n, the strictly lower triangular part is overwritten by
        details of the orthogonal matrix Q and the remaining elements
        are overwritten by the corresponding elements of the m*n upper
        trapezoidal matrix R.

    t : float, ndarray, shape (`nb`, min(m, n))
        Further details of the orthogonal matrix Q. The number of blocks
        is b = ceiling(k/`nb`), where k = min(m, n) and each block is of
        order `nb` except for the last block, which is of order
        k-(b-1)*`nb`. For each of the blocks, an upper triangular block
        reflector factor is computed: T_1,T_2,...,T_b. These are stored
        in the `nb`*n matrix T as T = [T_1|T_2|...|T_b].

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -3)
            On entry, error in parameter `nb`.

            Constraint: 1 <= `nb` <= min(m, n).

    Notes
    -----
    ``dgeqrt`` forms the QR factorization of an arbitrary rectangular
    real m*n matrix.
    No pivoting is performed.

    It differs from ``dgeqrf`` in that it: requires an explicit block
    size; stores reflector factors that are upper triangular matrices of
    the chosen block size (rather than scalars); and recursively
    computes the QR factorization based on the algorithm of Elmroth and
    Gustavson (2000).

    If m >= n, the factorization is given by:

        A = Q(R;0),

    where R is an n*n upper triangular matrix and Q is an m*m orthogonal
    matrix.
    It is sometimes more convenient to write the factorization as

        A = (Q_1 Q_2)(R;0),

    which reduces to

        A = Q_1 R,

    where Q_1 consists of the first n columns of Q, and Q_2 the
    remaining m-n columns.

    If m < n, R is upper trapezoidal, and the factorization can be
    written

        A = Q(R_1 R_2),

    where R_1 is upper triangular and R_2 is rectangular.

    The matrix Q is not formed explicitly but is represented as a
    product of min(m, n) elementary reflectors (see [the F08
    Introduction] for details). Functions are provided to work with Q in
    this representation (see [Further Comments]).

    Note also that for any k < n, the information returned represents a
    QR factorization of the first k columns of the original matrix A.

    References
    ----------
    Elmroth, E and Gustavson, F, 2000, `Applying Recursion to Serial and
    Parallel QR Factorization Leads to Better Performance`, IBM Journal
    of Research and Development. (Volume 44) (4), 605--624

    Golub, G H and Van Loan, C F, 2012, `Matrix Computations`, (4th
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dgeqrt'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'),),
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _nb = _b_utils._EngineIntScalarType(
        nb, {'fun_name': fun_name, 'entity': 'nb'},
    ).to_ctypes()
    _nb_py = _b_utils._EngineIntScalarType.to_py(_nb)
    _t = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_nb_py, min(_m_py, _n_py)),
        sorder=_m_sorder,
    )
    _b_lapackeig.dgeqrt(
        _m,
        _n,
        _nb,
        _a,
        _t,
    )
    _a_py = _a
    _t_py = _t
    _return_data = DgeqrtReturnData(
        _a_py,
        _t_py,
    )
    return _return_data

def dgemqrt(side, trans, v, t, c):
    u"""
    Applies the orthogonal transformation determined by ``dgeqrt``.

    ``dgemqrt`` multiplies an arbitrary real matrix C by the real
    orthogonal matrix Q from a QR factorization computed by ``dgeqrt``.

    For full information please refer to the NAG Library document for
    f08ac

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08acf.html

    Parameters
    ----------
    side : str, length 1
        Indicates how Q or Q^T is to be applied to C.

        `side` = 'L'

            Q or Q^T is applied to C from the left.

        `side` = 'R'

            Q or Q^T is applied to C from the right.

    trans : str, length 1
        Indicates whether Q or Q^T is to be applied to C.

        `trans` = 'N'

            Q is applied to C.

        `trans` = 'T'

            Q^T is applied to C.

    v : float, array-like, shape (:, k)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `side`='L': m; if `side`='R': n;
        otherwise: 0.

        Details of the vectors which define the elementary reflectors,
        as returned by ``dgeqrt`` in the first k columns of its array
        argument a.

    t : float, array-like, shape (nb, k)
        Further details of the orthogonal matrix Q as returned by
        ``dgeqrt``. The number of blocks is b = ceiling(k/nb), where k =
        min(m, n) and each block is of order nb except for the last
        block, which is of order k-(b-1)*nb. For the b blocks the upper
        triangular block reflector factors T_1,T_2,...,T_b are stored in
        the nb*n matrix T as T = [T_1|T_2|...|T_b].

    c : float, array-like, shape (m, n)
        The m*n matrix C.

    Returns
    -------
    c : float, ndarray, shape (m, n)
        `c` is overwritten by QC or Q^TC or CQ or CQ^T as specified by
        `side` and `trans`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'T'.

        (`errno` -3)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -5)
            On entry, error in parameter k.

            Constraint: m >= k >= 0.

        (`errno` -5)
            On entry, error in parameter k.

            Constraint: n >= k >= 0.

        (`errno` -6)
            On entry, error in parameter nb.

            Constraint: nb >= 1.

        (`errno` -6)
            On entry, error in parameter nb.

            Constraint: nb <= k.

    Notes
    -----
    ``dgemqrt`` is intended to be used after a call to ``dgeqrt`` which
    performs a QR factorization of a real matrix A.
    The orthogonal matrix Q is represented as a product of elementary
    reflectors.

    This function may be used to form one of the matrix products

        QC,Q^TC,CQ or CQ^T,

    overwriting the result on C (which may be any real rectangular
    matrix).

    A common application of this function is in solving linear least
    squares problems, as described in [the F08 Introduction].

    References
    ----------
    Golub, G H and Van Loan, C F, 2012, `Matrix Computations`, (4th
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dgemqrt'

    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _v = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'v'},
        v,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _k_py = _b_utils._EngineFloat64ArrayType.get_shape(_v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2)[1]
    _k = _b_utils._EngineIntScalarType(
        _k_py, {'fun_name': fun_name, 'entity': '_k'},
    ).to_ctypes()
    _t = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 't'},
        t,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _nb_py = _b_utils._EngineFloat64ArrayType.get_shape(_t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=2)[0]
    _nb = _b_utils._EngineIntScalarType(
        _nb_py, {'fun_name': fun_name, 'entity': '_nb'},
    ).to_ctypes()
    _b_lapackeig.dgemqrt(
        side,
        trans,
        _m,
        _n,
        _k,
        _nb,
        _v,
        _t,
        _c,
    )
    _c_py = _c
    return _c_py

DgeqrfReturnData = _collections.namedtuple(
    'DgeqrfReturnData',
    ['a', 'tau'],
)

def dgeqrf(a):
    u"""
    Performs a QR factorization of real general rectangular matrix.

    ``dgeqrf`` computes the QR factorization of a real m*n matrix.

    For full information please refer to the NAG Library document for
    f08ae

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08aef.html

    Parameters
    ----------
    a : float, array-like, shape (m, n)
        The m*n matrix A.

    Returns
    -------
    a : float, ndarray, shape (m, n)
        If m >= n, the elements below the diagonal are overwritten by
        details of the orthogonal matrix Q and the upper triangle is
        overwritten by the corresponding elements of the n*n upper
        triangular matrix R.

        If m < n, the strictly lower triangular part is overwritten by
        details of the orthogonal matrix Q and the remaining elements
        are overwritten by the corresponding elements of the m*n upper
        trapezoidal matrix R.

    tau : float, ndarray, shape (min(m, n))
        Further details of the orthogonal matrix Q.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= 0.

    Notes
    -----
    ``dgeqrf`` forms the QR factorization of an arbitrary rectangular
    real m*n matrix.
    No pivoting is performed.

    If m >= n, the factorization is given by:

        A = Q(R;0),

    where R is an n*n upper triangular matrix and Q is an m*m orthogonal
    matrix.
    It is sometimes more convenient to write the factorization as

        A = (Q_1 Q_2)(R;0),

    which reduces to

        A = Q_1 R,

    where Q_1 consists of the first n columns of Q, and Q_2 the
    remaining m-n columns.

    If m < n, R is trapezoidal, and the factorization can be written

        A = Q(R_1 R_2),

    where R_1 is upper triangular and R_2 is rectangular.

    The matrix Q is not formed explicitly but is represented as a
    product of min(m, n) elementary reflectors (see [the F08
    Introduction] for details). Functions are provided to work with Q in
    this representation (see [Further Comments]).

    Note also that for any k < n, the information returned in the first
    k columns of the array `a` represents a QR factorization of the
    first k columns of the original matrix A.

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dgeqrf'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _tau = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (min(_m_py, _n_py),),
    )
    _b_lapackeig.dgeqrf(
        _m,
        _n,
        _a,
        _tau,
    )
    _a_py = _a
    _tau_py = _tau
    _return_data = DgeqrfReturnData(
        _a_py,
        _tau_py,
    )
    return _return_data

def dorgqr(a, tau):
    u"""
    Forms all or part of orthogonal Q from QR factorization determined
    by ``dgeqrf``, ``dgeqpf`` or ``dgeqp3``.

    ``dorgqr`` generates all or part of the real orthogonal matrix Q
    from a QR factorization computed by ``dgeqrf`` or ``dgeqp3``.

    For full information please refer to the NAG Library document for
    f08af

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08aff.html

    Parameters
    ----------
    a : float, array-like, shape (m, n)
        Details of the vectors which define the elementary reflectors,
        as returned by ``dgeqrf`` or ``dgeqp3``.

    tau : float, array-like, shape (max(1, k))
        Further details of the elementary reflectors, as returned by
        ``dgeqrf`` or ``dgeqp3``.

    Returns
    -------
    a : float, ndarray, shape (m, n)
        The m*n matrix Q.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: m >= n >= 0.

        (`errno` -3)
            On entry, error in parameter k.

            Constraint: n >= k >= 0.

    Notes
    -----
    ``dorgqr`` is intended to be used after a call to ``dgeqrf`` or
    ``dgeqp3``. which perform a QR factorization of a real matrix A.
    The orthogonal matrix Q is represented as a product of elementary
    reflectors.

    This function may be used to generate Q explicitly as a square
    matrix, or to form only its leading columns.

    Usually Q is determined from the QR factorization of an m*p matrix A
    with m >= p.
    The whole of Q may be computed by calling ``dorgqr`` with n set to m
    and k set to p or its leading p columns by calling ``dorgqr`` with n
    and k set to p.

    The columns of Q returned by the last call form an orthonormal basis
    for the space spanned by the columns of A; thus ``dgeqrf`` followed
    by ``dorgqr`` can be used to orthogonalize the columns of A.

    The information returned by the QR factorization functions also
    yields the QR factorization of the leading k columns of A, where k <
    p.
    The orthogonal matrix arising from this factorization can be
    computed by calling ``dorgqr`` with n set to m or its leading k
    columns by calling ``dorgqr`` with n set to k.

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dorgqr'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _tau = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'tau'},
        tau,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _k_py = _b_utils._EngineFloat64ArrayType.get_shape(_tau, {'fun_name': fun_name, 'entity': 'tau'}, exp_rank=1)[0]
    _k = _b_utils._EngineIntScalarType(
        _k_py, {'fun_name': fun_name, 'entity': '_k'},
    ).to_ctypes()
    _b_lapackeig.dorgqr(
        _m,
        _n,
        _k,
        _a,
        _tau,
    )
    _a_py = _a
    return _a_py

def dormqr(side, trans, a, tau, c):
    u"""
    Applies an orthogonal transformation determined by ``dgeqrf``,
    ``dgeqpf`` or ``dgeqp3``.

    ``dormqr`` multiplies an arbitrary real matrix C by the real
    orthogonal matrix Q from a QR factorization computed by ``dgeqrf``
    or ``dgeqp3``.

    For full information please refer to the NAG Library document for
    f08ag

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08agf.html

    Parameters
    ----------
    side : str, length 1
        Indicates how Q or Q^T is to be applied to C.

        `side` = 'L'

            Q or Q^T is applied to C from the left.

        `side` = 'R'

            Q or Q^T is applied to C from the right.

    trans : str, length 1
        Indicates whether Q or Q^T is to be applied to C.

        `trans` = 'N'

            Q is applied to C.

        `trans` = 'T'

            Q^T is applied to C.

    a : float, array-like, shape (:, k)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `side`='L': m; if `side`='R': n;
        otherwise: 0.

        Details of the vectors which define the elementary reflectors,
        as returned by ``dgeqrf`` or ``dgeqp3``.

    tau : float, array-like, shape (k)
        Further details of the elementary reflectors, as returned by
        ``dgeqrf`` or ``dgeqp3``.

    c : float, array-like, shape (m, n)
        The m*n matrix C.

    Returns
    -------
    c : float, ndarray, shape (m, n)
        `c` is overwritten by QC or Q^TC or CQ or CQ^T as specified by
        `side` and `trans`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'T'.

        (`errno` -3)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -5)
            On entry, error in parameter k.

            Constraint: m >= k >= 0.

        (`errno` -5)
            On entry, error in parameter k.

            Constraint: n >= k >= 0.

    Notes
    -----
    ``dormqr`` is intended to be used after a call to ``dgeqrf`` or
    ``dgeqp3`` which perform a QR factorization of a real matrix A.
    The orthogonal matrix Q is represented as a product of elementary
    reflectors.

    This function may be used to form one of the matrix products

        QC,Q^TC,CQ or CQ^T,

    overwriting the result on `c` (which may be any real rectangular
    matrix).

    A common application of this function is in solving linear least
    squares problems, as described in [the F08 Introduction].

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dormqr'

    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _k_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _k = _b_utils._EngineIntScalarType(
        _k_py, {'fun_name': fun_name, 'entity': '_k'},
    ).to_ctypes()
    _tau = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'tau'},
        tau,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b_lapackeig.dormqr(
        side,
        trans,
        _m,
        _n,
        _k,
        _a,
        _tau,
        _c,
    )
    _c_py = _c
    return _c_py

DgelqfReturnData = _collections.namedtuple(
    'DgelqfReturnData',
    ['a', 'tau'],
)

def dgelqf(a):
    u"""
    Performs a LQ factorization of real general rectangular matrix.

    ``dgelqf`` computes the LQ factorization of a real m*n matrix.

    For full information please refer to the NAG Library document for
    f08ah

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08ahf.html

    Parameters
    ----------
    a : float, array-like, shape (m, n)
        The m*n matrix A.

    Returns
    -------
    a : float, ndarray, shape (m, n)
        If m <= n, the elements above the diagonal are overwritten by
        details of the orthogonal matrix Q and the lower triangle is
        overwritten by the corresponding elements of the m*m lower
        triangular matrix L.

        If m > n, the strictly upper triangular part is overwritten by
        details of the orthogonal matrix Q and the remaining elements
        are overwritten by the corresponding elements of the m*n lower
        trapezoidal matrix L.

    tau : float, ndarray, shape (min(m, n))
        Further details of the orthogonal matrix Q.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= 0.

    Notes
    -----
    ``dgelqf`` forms the LQ factorization of an arbitrary rectangular
    real m*n matrix.
    No pivoting is performed.

    If m <= n, the factorization is given by:

        A = (L 0)Q

    where L is an m*m lower triangular matrix and Q is an n*n orthogonal
    matrix.
    It is sometimes more convenient to write the factorization as

        A = (L 0)(Q_1;Q_2)

    which reduces to

        A = LQ_1,

    where Q_1 consists of the first m rows of Q, and Q_2 the remaining
    n-m rows.

    If m > n, L is trapezoidal, and the factorization can be written

        A = (L_1;L_2)Q

    where L_1 is lower triangular and L_2 is rectangular.

    The LQ factorization of A is essentially the same as the QR
    factorization of A^T, since

        A = (L 0)Q⇔A^T = Q^T(L^T;0).

    The matrix Q is not formed explicitly but is represented as a
    product of min(m, n) elementary reflectors (see [the F08
    Introduction] for details). Functions are provided to work with Q in
    this representation (see [Further Comments]).

    Note also that for any k < m, the information returned in the first
    k rows of the array `a` represents an LQ factorization of the first
    k rows of the original matrix A.
    """

    fun_name = 'naginterfaces.library.lapackeig.dgelqf'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _tau = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (min(_m_py, _n_py),),
    )
    _b_lapackeig.dgelqf(
        _m,
        _n,
        _a,
        _tau,
    )
    _a_py = _a
    _tau_py = _tau
    _return_data = DgelqfReturnData(
        _a_py,
        _tau_py,
    )
    return _return_data

def dorglq(a, tau):
    u"""
    Forms all or part of orthogonal Q from LQ factorization determined
    by ``dgelqf``.

    ``dorglq`` generates all or part of the real orthogonal matrix Q
    from an LQ factorization computed by ``dgelqf``.

    For full information please refer to the NAG Library document for
    f08aj

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08ajf.html

    Parameters
    ----------
    a : float, array-like, shape (m, n)
        Details of the vectors which define the elementary reflectors,
        as returned by ``dgelqf``.

    tau : float, array-like, shape (k)
        Further details of the elementary reflectors, as returned by
        ``dgelqf``.

    Returns
    -------
    a : float, ndarray, shape (m, n)
        The m*n matrix Q.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= m.

        (`errno` -3)
            On entry, error in parameter k.

            Constraint: m >= k >= 0.

    Notes
    -----
    ``dorglq`` is intended to be used after a call to ``dgelqf``, which
    performs an LQ factorization of a real matrix A.
    The orthogonal matrix Q is represented as a product of elementary
    reflectors.

    This function may be used to generate Q explicitly as a square
    matrix, or to form only its leading rows.

    Usually Q is determined from the LQ factorization of a p*n matrix A
    with p <= n.
    The whole of Q may be computed by calling ``dorglq`` with m set to n
    and k set to p or its leading p rows by calling ``dorglq`` with m
    and k set to p.

    The rows of Q returned by the last call form an orthonormal basis
    for the space spanned by the rows of A; thus ``dgelqf`` followed by
    ``dorglq`` can be used to orthogonalize the rows of A.

    The information returned by the LQ factorization functions also
    yields the LQ factorization of the leading k rows of A, where k < p.
    The orthogonal matrix arising from this factorization can be
    computed by calling ``dorglq`` with m set to n or its leading k rows
    by calling ``dorglq`` with m set to k.

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dorglq'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _tau = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'tau'},
        tau,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _k_py = _b_utils._EngineFloat64ArrayType.get_shape(_tau, {'fun_name': fun_name, 'entity': 'tau'}, exp_rank=1)[0]
    _k = _b_utils._EngineIntScalarType(
        _k_py, {'fun_name': fun_name, 'entity': '_k'},
    ).to_ctypes()
    _b_lapackeig.dorglq(
        _m,
        _n,
        _k,
        _a,
        _tau,
    )
    _a_py = _a
    return _a_py

def dormlq(side, trans, a, tau, c):
    u"""
    Applies the orthogonal transformation determined by ``dgelqf``.

    ``dormlq`` multiplies an arbitrary real matrix C by the real
    orthogonal matrix Q from an LQ factorization computed by ``dgelqf``.

    For full information please refer to the NAG Library document for
    f08ak

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08akf.html

    Parameters
    ----------
    side : str, length 1
        Indicates how Q or Q^T is to be applied to C.

        `side` = 'L'

            Q or Q^T is applied to C from the left.

        `side` = 'R'

            Q or Q^T is applied to C from the right.

    trans : str, length 1
        Indicates whether Q or Q^T is to be applied to C.

        `trans` = 'N'

            Q is applied to C.

        `trans` = 'T'

            Q^T is applied to C.

    a : float, array-like, shape (k, :)
        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `side`='L': m; if `side`='R': n;
        otherwise: 0.

        Details of the vectors which define the elementary reflectors,
        as returned by ``dgelqf``.

    tau : float, array-like, shape (k)
        Further details of the elementary reflectors, as returned by
        ``dgelqf``.

    c : float, array-like, shape (m, n)
        The m*n matrix C.

    Returns
    -------
    c : float, ndarray, shape (m, n)
        `c` is overwritten by QC or Q^TC or CQ or CQ^T as specified by
        `side` and `trans`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'T'.

        (`errno` -3)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -5)
            On entry, error in parameter k.

            Constraint: m >= k >= 0.

        (`errno` -5)
            On entry, error in parameter k.

            Constraint: n >= k >= 0.

    Notes
    -----
    ``dormlq`` is intended to be used after a call to ``dgelqf``, which
    performs an LQ factorization of a real matrix A.
    The orthogonal matrix Q is represented as a product of elementary
    reflectors.

    This function may be used to form one of the matrix products

        QC,Q^TC,CQ or CQ^T,

    overwriting the result on C (which may be any real rectangular
    matrix).

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dormlq'

    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _tau = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'tau'},
        tau,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _k_py = _b_utils._EngineFloat64ArrayType.get_shape(_tau, {'fun_name': fun_name, 'entity': 'tau'}, exp_rank=1)[0]
    _k = _b_utils._EngineIntScalarType(
        _k_py, {'fun_name': fun_name, 'entity': '_k'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b_lapackeig.dormlq(
        side,
        trans,
        _m,
        _n,
        _k,
        _a,
        _tau,
        _c,
    )
    _c_py = _c
    return _c_py

ZgelsReturnData = _collections.namedtuple(
    'ZgelsReturnData',
    ['a', 'b'],
)

def zgels(trans, a, b):
    u"""
    Solves a complex linear least problem of full rank.

    ``zgels`` solves linear least squares problems of the form

        min_x||b-Ax||_2 or min_x||b-A^Hx||_2,

    where A is an m*n complex matrix of full rank, using a QR or LQ
    factorization of A.

    For full information please refer to the NAG Library document for
    f08an

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08anf.html

    Parameters
    ----------
    trans : str, length 1
        If `trans` = 'N', the linear system involves A.

        If `trans` = 'C', the linear system involves A^H.

    a : complex, array-like, shape (m, n)
        The m*n matrix A.

    b : complex, array-like, shape (max(m, n), nrhs)
        The matrix B of right-hand side vectors, stored in columns; `b`
        is m*r if `trans` = 'N', or n*r if `trans` = 'C'.

    Returns
    -------
    a : complex, ndarray, shape (m, n)
        If m >= n, `a` is overwritten by details of its QR
        factorization, as returned by ``zgeqrf``.

        If m < n, `a` is overwritten by details of its LQ factorization,
        as returned by ``zgelqf``.

    b : complex, ndarray, shape (max(m, n), nrhs)
        `b` is overwritten by the solution vectors, x, stored in
        columns:

            if `trans` = 'N' and m >= n, or `trans` = 'C' and m < n,
            elements 1 to min(m, n) in each column of `b` contain the
            least squares solution vectors; the residual sum of squares
            for the solution is given by the sum of squares of the
            modulus of elements (min(m, n)+1) to max(m, n) in that
            column;

            otherwise, elements 1 to max(m, n) in each column of `b`
            contain the minimum norm solution vectors.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'C'.

        (`errno` -2)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -4)
            On entry, error in parameter nrhs.

            Constraint: nrhs >= 0.

        (`errno` i > 0)
            Diagonal element *<value>* of the triangular factor of A is
            zero, so that A does not have full rank; the least squares
            solution could not be computed.

    Notes
    -----
    The following options are provided:

    (1) If `trans` = 'N' and m >= n: find the least squares solution of
        an overdetermined system, i.e., solve the least squares problem

            min_x||b-Ax||_2.

    (#) If `trans` = 'N' and m < n: find the minimum norm solution of an
        underdetermined system Ax = b.

    (#) If `trans` = 'C' and m >= n: find the minimum norm solution of
        an undetermined system A^Hx = b.

    (#) If `trans` = 'C' and m < n: find the least squares solution of
        an overdetermined system, i.e., solve the least squares problem

            min_x||b-A^Hx||_2.

    Several right-hand side vectors b and solution vectors x can be
    handled in a single call; they are stored as the columns of the m*r
    right-hand side matrix B and the n*r solution matrix X.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zgels'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
    )
    _nrhs_py = _b_utils._EngineComplex128ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[1]
    _nrhs = _b_utils._EngineIntScalarType(
        _nrhs_py, {'fun_name': fun_name, 'entity': '_nrhs'},
    ).to_ctypes()
    _b_lapackeig.zgels(
        trans,
        _m,
        _n,
        _nrhs,
        _a,
        _b,
    )
    _a_py = _a
    _b_py = _b
    _return_data = ZgelsReturnData(
        _a_py,
        _b_py,
    )
    return _return_data

ZgeqrtReturnData = _collections.namedtuple(
    'ZgeqrtReturnData',
    ['a', 't'],
)

def zgeqrt(nb, a):
    u"""
    Performs a QR factorization of complex general rectangular matrix
    using recursive algorithm.

    ``zgeqrt`` recursively computes, with explicit blocking, the QR
    factorization of a complex m*n matrix.

    For full information please refer to the NAG Library document for
    f08ap

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08apf.html

    Parameters
    ----------
    nb : int
        The explicitly chosen block size to be used in computing the QR
        factorization. See [Further Comments] for details.

    a : complex, array-like, shape (m, n)
        The m*n matrix A.

    Returns
    -------
    a : complex, ndarray, shape (m, n)
        If m >= n, the elements below the diagonal are overwritten by
        details of the unitary matrix Q and the upper triangle is
        overwritten by the corresponding elements of the n*n upper
        triangular matrix R.

        If m < n, the strictly lower triangular part is overwritten by
        details of the unitary matrix Q and the remaining elements are
        overwritten by the corresponding elements of the m*n upper
        trapezoidal matrix R.

        The diagonal elements of R are real.

    t : complex, ndarray, shape (`nb`, min(m, n))
        Further details of the unitary matrix Q. The number of blocks is
        b = ceiling(k/`nb`), where k = min(m, n) and each block is of
        order `nb` except for the last block, which is of order
        k-(b-1)*`nb`. For each of the blocks, an upper triangular block
        reflector factor is computed: T_1,T_2,...,T_b. These are stored
        in the `nb`*n matrix T as T = [T_1|T_2|...|T_b].

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -3)
            On entry, error in parameter `nb`.

            Constraint: 1 <= `nb` <= min(m, n).

    Notes
    -----
    ``zgeqrt`` forms the QR factorization of an arbitrary rectangular
    complex m*n matrix.
    No pivoting is performed.

    It differs from ``zgeqrf`` in that it: requires an explicit block
    size; stores reflector factors that are upper triangular matrices of
    the chosen block size (rather than scalars); and recursively
    computes the QR factorization based on the algorithm of Elmroth and
    Gustavson (2000).

    If m >= n, the factorization is given by:

        A = Q(R;0),

    where R is an n*n upper triangular matrix (with real diagonal
    elements) and Q is an m*m unitary matrix.
    It is sometimes more convenient to write the factorization as

        A = (Q_1 Q_2)(R;0),

    which reduces to

        A = Q_1 R,

    where Q_1 consists of the first n columns of Q, and Q_2 the
    remaining m-n columns.

    If m < n, R is upper trapezoidal, and the factorization can be
    written

        A = Q(R_1 R_2),

    where R_1 is upper triangular and R_2 is rectangular.

    The matrix Q is not formed explicitly but is represented as a
    product of min(m, n) elementary reflectors (see [the F08
    Introduction] for details). Functions are provided to work with Q in
    this representation (see [Further Comments]).

    Note also that for any k < n, the information returned represents a
    QR factorization of the first k columns of the original matrix A.

    References
    ----------
    Elmroth, E and Gustavson, F, 2000, `Applying Recursion to Serial and
    Parallel QR Factorization Leads to Better Performance`, IBM Journal
    of Research and Development. (Volume 44) (4), 605--624

    Golub, G H and Van Loan, C F, 2012, `Matrix Computations`, (4th
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zgeqrt'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'),),
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _nb = _b_utils._EngineIntScalarType(
        nb, {'fun_name': fun_name, 'entity': 'nb'},
    ).to_ctypes()
    _nb_py = _b_utils._EngineIntScalarType.to_py(_nb)
    _t = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_nb_py, min(_m_py, _n_py)),
        sorder=_m_sorder,
    )
    _b_lapackeig.zgeqrt(
        _m,
        _n,
        _nb,
        _a,
        _t,
    )
    _a_py = _a
    _t_py = _t
    _return_data = ZgeqrtReturnData(
        _a_py,
        _t_py,
    )
    return _return_data

def zgemqrt(side, trans, v, t, c):
    u"""
    Applies the unitary transformation determined by ``zgeqrt``.

    ``zgemqrt`` multiplies an arbitrary complex matrix C by the complex
    unitary matrix Q from a QR factorization computed by ``zgeqrt``.

    For full information please refer to the NAG Library document for
    f08aq

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08aqf.html

    Parameters
    ----------
    side : str, length 1
        Indicates how Q or Q^H is to be applied to C.

        `side` = 'L'

            Q or Q^H is applied to C from the left.

        `side` = 'R'

            Q or Q^H is applied to C from the right.

    trans : str, length 1
        Indicates whether Q or Q^H is to be applied to C.

        `trans` = 'N'

            Q is applied to C.

        `trans` = 'C'

            Q^H is applied to C.

    v : complex, array-like, shape (:, k)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `side`='L': m; if `side`='R': n;
        otherwise: 0.

        Details of the vectors which define the elementary reflectors,
        as returned by ``zgeqrt`` in the first k columns of its array
        argument a.

    t : complex, array-like, shape (nb, k)
        Further details of the unitary matrix Q as returned by
        ``zgeqrt``. The number of blocks is b = ceiling(k/nb), where k =
        min(m, n) and each block is of order nb except for the last
        block, which is of order k-(b-1)*nb. For the b blocks the upper
        triangular block reflector factors T_1,T_2,...,T_b are stored in
        the nb*n matrix T as T = [T_1|T_2|...|T_b].

    c : complex, array-like, shape (m, n)
        The m*n matrix C.

    Returns
    -------
    c : complex, ndarray, shape (m, n)
        `c` is overwritten by QC or Q^HC or CQ or CQ^H as specified by
        `side` and `trans`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'C'.

        (`errno` -3)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -5)
            On entry, error in parameter k.

            Constraint: m >= k >= 0.

        (`errno` -5)
            On entry, error in parameter k.

            Constraint: n >= k >= 0.

        (`errno` -6)
            On entry, error in parameter nb.

            Constraint: nb >= 1.

        (`errno` -6)
            On entry, error in parameter nb.

            Constraint: nb <= k.

    Notes
    -----
    ``zgemqrt`` is intended to be used after a call to ``zgeqrt``, which
    performs a QR factorization of a complex matrix A.
    The unitary matrix Q is represented as a product of elementary
    reflectors.

    This function may be used to form one of the matrix products

        QC,Q^HC,CQ or CQ^H,

    overwriting the result on C (which may be any complex rectangular
    matrix).

    A common application of this function is in solving linear least
    squares problems, as described in [the F08 Introduction].

    References
    ----------
    Golub, G H and Van Loan, C F, 2012, `Matrix Computations`, (4th
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zgemqrt'

    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=True,
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _v = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'v'},
        v,
        copy=True,
    )
    _k_py = _b_utils._EngineComplex128ArrayType.get_shape(_v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2)[1]
    _k = _b_utils._EngineIntScalarType(
        _k_py, {'fun_name': fun_name, 'entity': '_k'},
    ).to_ctypes()
    _t = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 't'},
        t,
        copy=False,
    )
    _nb_py = _b_utils._EngineComplex128ArrayType.get_shape(_t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=2)[0]
    _nb = _b_utils._EngineIntScalarType(
        _nb_py, {'fun_name': fun_name, 'entity': '_nb'},
    ).to_ctypes()
    _b_lapackeig.zgemqrt(
        side,
        trans,
        _m,
        _n,
        _k,
        _nb,
        _v,
        _t,
        _c,
    )
    _c_py = _c
    return _c_py

ZgeqrfReturnData = _collections.namedtuple(
    'ZgeqrfReturnData',
    ['a', 'tau'],
)

def zgeqrf(a):
    u"""
    Performs a QR factorization of complex general rectangular matrix.

    ``zgeqrf`` computes the QR factorization of a complex m*n matrix.

    For full information please refer to the NAG Library document for
    f08as

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08asf.html

    Parameters
    ----------
    a : complex, array-like, shape (m, n)
        The m*n matrix A.

    Returns
    -------
    a : complex, ndarray, shape (m, n)
        If m >= n, the elements below the diagonal are overwritten by
        details of the unitary matrix Q and the upper triangle is
        overwritten by the corresponding elements of the n*n upper
        triangular matrix R.

        If m < n, the strictly lower triangular part is overwritten by
        details of the unitary matrix Q and the remaining elements are
        overwritten by the corresponding elements of the m*n upper
        trapezoidal matrix R.

        The diagonal elements of R are real.

    tau : complex, ndarray, shape (min(m, n))
        Further details of the unitary matrix Q.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= 0.

    Notes
    -----
    ``zgeqrf`` forms the QR factorization of an arbitrary rectangular
    complex m*n matrix.
    No pivoting is performed.

    If m >= n, the factorization is given by:

        A = Q(R;0),

    where R is an n*n upper triangular matrix (with real diagonal
    elements) and Q is an m*m unitary matrix.
    It is sometimes more convenient to write the factorization as

        A = (Q_1 Q_2)(R;0),

    which reduces to

        A = Q_1 R,

    where Q_1 consists of the first n columns of Q, and Q_2 the
    remaining m-n columns.

    If m < n, R is trapezoidal, and the factorization can be written

        A = Q(R_1 R_2),

    where R_1 is upper triangular and R_2 is rectangular.

    The matrix Q is not formed explicitly but is represented as a
    product of min(m, n) elementary reflectors (see [the F08
    Introduction] for details). Functions are provided to work with Q in
    this representation (see [Further Comments]).

    Note also that for any k < n, the information returned in the first
    k columns of the array `a` represents a QR factorization of the
    first k columns of the original matrix A.

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zgeqrf'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _tau = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (min(_m_py, _n_py),),
    )
    _b_lapackeig.zgeqrf(
        _m,
        _n,
        _a,
        _tau,
    )
    _a_py = _a
    _tau_py = _tau
    _return_data = ZgeqrfReturnData(
        _a_py,
        _tau_py,
    )
    return _return_data

def zungqr(a, tau):
    u"""
    Forms all or part of unitary Q from QR factorization determined by
    ``zgeqrf``, ``zgeqpf`` or ``zgeqp3``.

    ``zungqr`` generates all or part of the complex unitary matrix Q
    from a QR factorization computed by ``zgeqrf`` or ``zgeqp3``.

    For full information please refer to the NAG Library document for
    f08at

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08atf.html

    Parameters
    ----------
    a : complex, array-like, shape (m, n)
        Details of the vectors which define the elementary reflectors,
        as returned by ``zgeqrf`` or ``zgeqp3``.

    tau : complex, array-like, shape (max(1, k))
        Further details of the elementary reflectors, as returned by
        ``zgeqrf`` or ``zgeqp3``.

    Returns
    -------
    a : complex, ndarray, shape (m, n)
        The m*n matrix Q.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: m >= n >= 0.

        (`errno` -3)
            On entry, error in parameter k.

            Constraint: n >= k >= 0.

    Notes
    -----
    ``zungqr`` is intended to be used after a call to ``zgeqrf`` or
    ``zgeqp3``, which perform a QR factorization of a complex matrix A.
    The unitary matrix Q is represented as a product of elementary
    reflectors.

    This function may be used to generate Q explicitly as a square
    matrix, or to form only its leading columns.

    Usually Q is determined from the QR factorization of an m*p matrix A
    with m >= p.
    The whole of Q may be computed by calling ``zungqr`` with n set to m
    and k set to p or its leading p columns by calling ``zungqr`` with n
    and k set to p.

    The columns of Q returned by the last call form an orthonormal basis
    for the space spanned by the columns of A; thus ``zgeqrf`` followed
    by ``zungqr`` can be used to orthogonalize the columns of A.

    The information returned by the QR factorization functions also
    yields the QR factorization of the leading k columns of A, where k <
    p.
    The unitary matrix arising from this factorization can be computed
    by calling ``zungqr`` with n set to m or its leading k columns by
    calling ``zungqr`` with n set to k.

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zungqr'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _tau = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'tau'},
        tau,
        copy=False,
    )
    _k_py = _b_utils._EngineComplex128ArrayType.get_shape(_tau, {'fun_name': fun_name, 'entity': 'tau'}, exp_rank=1)[0]
    _k = _b_utils._EngineIntScalarType(
        _k_py, {'fun_name': fun_name, 'entity': '_k'},
    ).to_ctypes()
    _b_lapackeig.zungqr(
        _m,
        _n,
        _k,
        _a,
        _tau,
    )
    _a_py = _a
    return _a_py

def zunmqr(side, trans, a, tau, c):
    u"""
    Applies a unitary transformation determined by ``zgeqrf``,
    ``zgeqpf`` or ``zgeqp3``.

    ``zunmqr`` multiplies an arbitrary complex matrix C by the complex
    unitary matrix Q from a QR factorization computed by ``zgeqrf`` or
    ``zgeqp3``.

    For full information please refer to the NAG Library document for
    f08au

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08auf.html

    Parameters
    ----------
    side : str, length 1
        Indicates how Q or Q^H is to be applied to C.

        `side` = 'L'

            Q or Q^H is applied to C from the left.

        `side` = 'R'

            Q or Q^H is applied to C from the right.

    trans : str, length 1
        Indicates whether Q or Q^H is to be applied to C.

        `trans` = 'N'

            Q is applied to C.

        `trans` = 'C'

            Q^H is applied to C.

    a : complex, array-like, shape (:, k)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `side`='L': m; if `side`='R': n;
        otherwise: 0.

        Details of the vectors which define the elementary reflectors,
        as returned by ``zgeqrf`` or ``zgeqp3``.

    tau : complex, array-like, shape (k)
        Further details of the elementary reflectors, as returned by
        ``zgeqrf`` or ``zgeqp3``.

    c : complex, array-like, shape (m, n)
        The m*n matrix C.

    Returns
    -------
    c : complex, ndarray, shape (m, n)
        `c` is overwritten by QC or Q^HC or CQ or CQ^H as specified by
        `side` and `trans`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'C'.

        (`errno` -3)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -5)
            On entry, error in parameter k.

            Constraint: m >= k >= 0.

        (`errno` -5)
            On entry, error in parameter k.

            Constraint: n >= k >= 0.

    Notes
    -----
    ``zunmqr`` is intended to be used after a call to ``zgeqrf`` or
    ``zgeqp3``, which perform a QR factorization of a complex matrix A.
    The unitary matrix Q is represented as a product of elementary
    reflectors.

    This function may be used to form one of the matrix products

        QC,Q^HC,CQ or CQ^H,

    overwriting the result on `c` (which may be any complex rectangular
    matrix).

    A common application of this function is in solving linear least
    squares problems, as described in [the F08 Introduction].

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zunmqr'

    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=True,
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _k_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _k = _b_utils._EngineIntScalarType(
        _k_py, {'fun_name': fun_name, 'entity': '_k'},
    ).to_ctypes()
    _tau = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'tau'},
        tau,
        copy=False,
    )
    _b_lapackeig.zunmqr(
        side,
        trans,
        _m,
        _n,
        _k,
        _a,
        _tau,
        _c,
    )
    _c_py = _c
    return _c_py

ZgelqfReturnData = _collections.namedtuple(
    'ZgelqfReturnData',
    ['a', 'tau'],
)

def zgelqf(a):
    u"""
    Performs a LQ factorization of complex general rectangular matrix.

    ``zgelqf`` computes the LQ factorization of a complex m*n matrix.

    For full information please refer to the NAG Library document for
    f08av

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08avf.html

    Parameters
    ----------
    a : complex, array-like, shape (m, n)
        The m*n matrix A.

    Returns
    -------
    a : complex, ndarray, shape (m, n)
        If m <= n, the elements above the diagonal are overwritten by
        details of the unitary matrix Q and the lower triangle is
        overwritten by the corresponding elements of the m*m lower
        triangular matrix L.

        If m > n, the strictly upper triangular part is overwritten by
        details of the unitary matrix Q and the remaining elements are
        overwritten by the corresponding elements of the m*n lower
        trapezoidal matrix L.

        The diagonal elements of L are real.

    tau : complex, ndarray, shape (min(m, n))
        Further details of the unitary matrix Q.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= 0.

    Notes
    -----
    ``zgelqf`` forms the LQ factorization of an arbitrary rectangular
    complex m*n matrix.
    No pivoting is performed.

    If m <= n, the factorization is given by:

        A = (L 0)Q

    where L is an m*m lower triangular matrix (with real diagonal
    elements) and Q is an n*n unitary matrix.
    It is sometimes more convenient to write the factorization as

        A = (L 0)(Q_1;Q_2)

    which reduces to

        A = LQ_1,

    where Q_1 consists of the first m rows of Q, and Q_2 the remaining
    n-m rows.

    If m > n, L is trapezoidal, and the factorization can be written

        A = (L_1;L_2)Q

    where L_1 is lower triangular and L_2 is rectangular.

    The LQ factorization of A is essentially the same as the QR
    factorization of A^H, since

        A = (L 0)Q⇔A^H = Q^H(L^H;0).

    The matrix Q is not formed explicitly but is represented as a
    product of min(m, n) elementary reflectors (see [the F08
    Introduction] for details). Functions are provided to work with Q in
    this representation (see [Further Comments]).

    Note also that for any k < m, the information returned in the first
    k rows of the array `a` represents an LQ factorization of the first
    k rows of the original matrix A.
    """

    fun_name = 'naginterfaces.library.lapackeig.zgelqf'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _tau = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (min(_m_py, _n_py),),
    )
    _b_lapackeig.zgelqf(
        _m,
        _n,
        _a,
        _tau,
    )
    _a_py = _a
    _tau_py = _tau
    _return_data = ZgelqfReturnData(
        _a_py,
        _tau_py,
    )
    return _return_data

def zunglq(a, tau):
    u"""
    Forms all or part of unitary Q from LQ factorization determined by
    ``zgelqf``.

    ``zunglq`` generates all or part of the complex unitary matrix Q
    from an LQ factorization computed by ``zgelqf``.

    For full information please refer to the NAG Library document for
    f08aw

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08awf.html

    Parameters
    ----------
    a : complex, array-like, shape (m, n)
        Details of the vectors which define the elementary reflectors,
        as returned by ``zgelqf``.

    tau : complex, array-like, shape (k)
        Further details of the elementary reflectors, as returned by
        ``zgelqf``.

    Returns
    -------
    a : complex, ndarray, shape (m, n)
        The m*n matrix Q.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= m.

        (`errno` -3)
            On entry, error in parameter k.

            Constraint: m >= k >= 0.

    Notes
    -----
    ``zunglq`` is intended to be used after a call to ``zgelqf``, which
    performs an LQ factorization of a complex matrix A.
    The unitary matrix Q is represented as a product of elementary
    reflectors.

    This function may be used to generate Q explicitly as a square
    matrix, or to form only its leading rows.

    Usually Q is determined from the LQ factorization of a p*n matrix A
    with p <= n.
    The whole of Q may be computed by calling ``zunglq`` with m set to n
    and k set to p or its leading p rows by calling ``zunglq`` with m
    and k set to p.

    The rows of Q returned by the last call form an orthonormal basis
    for the space spanned by the rows of A; thus ``zgelqf`` followed by
    ``zunglq`` can be used to orthogonalize the rows of A.

    The information returned by the LQ factorization functions also
    yields the LQ factorization of the leading k rows of A, where k < p.
    The unitary matrix arising from this factorization can be computed
    by calling ``zunglq`` with m set to n or its leading k rows by
    calling ``zunglq`` with m set to k.

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zunglq'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _tau = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'tau'},
        tau,
        copy=False,
    )
    _k_py = _b_utils._EngineComplex128ArrayType.get_shape(_tau, {'fun_name': fun_name, 'entity': 'tau'}, exp_rank=1)[0]
    _k = _b_utils._EngineIntScalarType(
        _k_py, {'fun_name': fun_name, 'entity': '_k'},
    ).to_ctypes()
    _b_lapackeig.zunglq(
        _m,
        _n,
        _k,
        _a,
        _tau,
    )
    _a_py = _a
    return _a_py

def zunmlq(side, trans, a, tau, c):
    u"""
    Applies the unitary transformation determined by ``zgelqf``.

    ``zunmlq`` multiplies an arbitrary complex matrix C by the complex
    unitary matrix Q from an LQ factorization computed by ``zgelqf``.

    For full information please refer to the NAG Library document for
    f08ax

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08axf.html

    Parameters
    ----------
    side : str, length 1
        Indicates how Q or Q^H is to be applied to C.

        `side` = 'L'

            Q or Q^H is applied to C from the left.

        `side` = 'R'

            Q or Q^H is applied to C from the right.

    trans : str, length 1
        Indicates whether Q or Q^H is to be applied to C.

        `trans` = 'N'

            Q is applied to C.

        `trans` = 'C'

            Q^H is applied to C.

    a : complex, array-like, shape (k, :)
        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `side`='L': m; if `side`='R': n;
        otherwise: 0.

        Details of the vectors which define the elementary reflectors,
        as returned by ``zgelqf``.

    tau : complex, array-like, shape (k)
        Further details of the elementary reflectors, as returned by
        ``zgelqf``.

    c : complex, array-like, shape (m, n)
        The m*n matrix C.

    Returns
    -------
    c : complex, ndarray, shape (m, n)
        `c` is overwritten by QC or Q^HC or CQ or CQ^H as specified by
        `side` and `trans`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'C'.

        (`errno` -3)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -5)
            On entry, error in parameter k.

            Constraint: m >= k >= 0.

        (`errno` -5)
            On entry, error in parameter k.

            Constraint: n >= k >= 0.

    Notes
    -----
    ``zunmlq`` is intended to be used after a call to ``zgelqf``, which
    performs an LQ factorization of a complex matrix A.
    The unitary matrix Q is represented as a product of elementary
    reflectors.

    This function may be used to form one of the matrix products

        QC,Q^HC,CQ or CQ^H,

    overwriting the result on C (which may be any complex rectangular
    matrix).

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zunmlq'

    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=True,
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _tau = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'tau'},
        tau,
        copy=False,
    )
    _k_py = _b_utils._EngineComplex128ArrayType.get_shape(_tau, {'fun_name': fun_name, 'entity': 'tau'}, exp_rank=1)[0]
    _k = _b_utils._EngineIntScalarType(
        _k_py, {'fun_name': fun_name, 'entity': '_k'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _b_lapackeig.zunmlq(
        side,
        trans,
        _m,
        _n,
        _k,
        _a,
        _tau,
        _c,
    )
    _c_py = _c
    return _c_py

DgelsyReturnData = _collections.namedtuple(
    'DgelsyReturnData',
    ['a', 'b', 'jpvt', 'rank'],
)

def dgelsy(a, b, jpvt, rcond):
    u"""
    Computes the minimum-norm solution to a real linear least squares
    problem.

    ``dgelsy`` computes the minimum norm solution to a real linear least
    squares problem

        min_x||b-Ax||_2

    using a complete orthogonal factorization of A. A is an m*n matrix
    which may be rank-deficient.
    Several right-hand side vectors b and solution vectors x can be
    handled in a single call.

    For full information please refer to the NAG Library document for
    f08ba

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08baf.html

    Parameters
    ----------
    a : float, array-like, shape (m, n)
        The m*n matrix A.

    b : float, array-like, shape (max(m, n), nrhs)
        The m*r right-hand side matrix B.

    jpvt : int, array-like, shape (n)
        If `jpvt`[i-1] != 0, the ith column of A is permuted to the
        front of AP, otherwise column i is a free column.

    rcond : float
        Used to determine the effective rank of A, which is defined as
        the order of the largest leading triangular sub-matrix R_11 in
        the QR factorization of A, whose estimated condition number is <
        1/`rcond`.

        Suggested value: if the condition number of `a` is not known
        then `rcond` = sqrt((epsilon)/2) (where epsilon is machine
        precision, see ``machine.precision``) is a good choice. Negative
        values or values less than machine precision should be avoided
        since this will cause `a` to have an effective rank = min(m, n)
        that could be larger than its actual rank, leading to
        meaningless results.

    Returns
    -------
    a : float, ndarray, shape (m, n)
        `a` has been overwritten by details of its complete orthogonal
        factorization.

    b : float, ndarray, shape (max(m, n), nrhs)
        The n*r solution matrix X.

    jpvt : int, ndarray, shape (n)
        If `jpvt`[i-1] = k, the ith column of AP was the kth column of
        A.

    rank : int
        The effective rank of A, i.e., the order of the sub-matrix R_11.
        This is the same as the order of the sub-matrix T_11 in the
        complete orthogonal factorization of A.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -3)
            On entry, error in parameter nrhs.

            Constraint: nrhs >= 0.

    Notes
    -----
    The right-hand side vectors are stored as the columns of the m*r
    matrix B and the solution vectors in the n*r matrix X.

    ``dgelsy`` first computes a QR factorization with column pivoting

        AP = Q[table omitted],

    with R_11 defined as the largest leading sub-matrix whose estimated
    condition number is less than 1/`rcond`.
    The order of R_11, `rank`, is the effective rank of A.

    Then, R_22 is considered to be negligible, and R_12 is annihilated
    by orthogonal transformations from the right, arriving at the
    complete orthogonal factorization

        AP = Q[table omitted]Z.

    The minimum norm solution is then

        X = PZ^T(T_11^-1 Q_1^T b ;0)

    where Q_1 consists of the first `rank` columns of Q.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dgelsy'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _nrhs_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[1]
    _nrhs = _b_utils._EngineIntScalarType(
        _nrhs_py, {'fun_name': fun_name, 'entity': '_nrhs'},
    ).to_ctypes()
    _jpvt = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'jpvt'},
        jpvt,
        copy=True,
    )
    _return_scalar = _b_lapackeig.dgelsy(
        _m,
        _n,
        _nrhs,
        _a,
        _b,
        _jpvt,
        rcond,
    )
    _a_py = _a
    _b_py = _b
    _jpvt_py = _jpvt
    _rank_py = _return_scalar
    _return_data = DgelsyReturnData(
        _a_py,
        _b_py,
        _jpvt_py,
        _rank_py,
    )
    return _return_data

DtpqrtReturnData = _collections.namedtuple(
    'DtpqrtReturnData',
    ['a', 'b', 't'],
)

def dtpqrt(l, nb, a, b):
    u"""
    QR factorization of real general triangular-pentagonal matrix.

    ``dtpqrt`` computes the QR factorization of a real (m+n)*n
    triangular-pentagonal matrix.

    For full information please refer to the NAG Library document for
    f08bb

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08bbf.html

    Parameters
    ----------
    l : int
        l, the number of rows of the trapezoidal part of B (i.e., B_2).

    nb : int
        The explicitly chosen block-size to be used in the algorithm for
        computing the QR factorization. See [Further Comments] for
        details.

    a : float, array-like, shape (n, n)
        The n*n upper triangular matrix A.

    b : float, array-like, shape (m, n)
        The m*n pentagonal matrix B composed of an (m-l)*n rectangular
        matrix B_1 above an l*n upper trapezoidal matrix B_2.

    Returns
    -------
    a : float, ndarray, shape (n, n)
        The upper triangle is overwritten by the corresponding elements
        of the n*n upper triangular matrix R.

    b : float, ndarray, shape (m, n)
        Details of the orthogonal matrix Q.

    t : float, ndarray, shape (`nb`, n)
        Further details of the orthogonal matrix Q. The number of blocks
        is b = ceiling(k/`nb`), where k = min(m, n) and each block is of
        order `nb` except for the last block, which is of order
        k-(b-1)*`nb`. For each of the blocks, an upper triangular block
        reflector factor is computed: T_1,T_2,...,T_b. These are stored
        in the `nb`*n matrix T as T = [T_1|T_2|...|T_b].

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -3)
            On entry, error in parameter `l`.

            Constraint: 0 <= `l` <= min(m, n).

        (`errno` -4)
            On entry, error in parameter `nb`.

            Constraint: `nb` >= 1.

        (`errno` -4)
            On entry, error in parameter `nb`.

            Constraint: `nb` <= n.

    Notes
    -----
    ``dtpqrt`` forms the QR factorization of a real (m+n)*n
    triangular-pentagonal matrix C,

        C = (A;B)

    where A is an upper triangular n*n matrix and B is an m*n pentagonal
    matrix consisting of an (m-l)*n rectangular matrix B_1 on top of an
    l*n upper trapezoidal matrix B_2:

        B = (B_1;B_2).

    The upper trapezoidal matrix B_2 consists of the first l rows of an
    n*n upper triangular matrix, where 0≤l≤min(m, n).
    If l = 0, B is m*n rectangular; if l = n and m = n, B is upper
    triangular.

    A recursive, explicitly blocked, QR factorization (see ``dgeqrt``)
    is performed on the matrix C.
    The upper triangular matrix R, details of the orthogonal matrix Q,
    and further details (the block reflector factors) of Q are returned.

    Typically the matrix A or B_2 contains the matrix R from the QR
    factorization of a subproblem and ``dtpqrt`` performs the QR update
    operation from the inclusion of matrix B_1.

    For example, consider the QR factorization of an l*n matrix B^ with
    l < n: B^= Q^R^, R^= (R_1^R_2^), where R_1^ is l*l upper triangular
    and R_2^ is (n-l)*n rectangular (this can be performed by
    ``dgeqrt``).
    Given an initial least squares problem B^X^= Y^ where X and Y are
    l*nrhs matrices, we have R^X^= Q^^TY^.

    Now, adding an additional m-l rows to the original system gives the
    augmented least squares problem

        BX = Y

    where B is an m*n matrix formed by adding m-l rows on top of R^ and
    Y is an m*nrhs matrix formed by adding m-l rows on top of Q^^TY^.

    ``dtpqrt`` can then be used to perform the QR factorization of the
    pentagonal matrix B; the n*n matrix A will be zero on input and
    contain R on output.

    In the case where B^ is r*n, r >= n, R^ is n*n upper triangular
    (forming A) on top of r-n rows of zeros (forming first r-n rows of
    B).
    Augmentation is then performed by adding rows to the bottom of B
    with l = 0.

    References
    ----------
    Elmroth, E and Gustavson, F, 2000, `Applying Recursion to Serial and
    Parallel QR Factorization Leads to Better Performance`, IBM Journal
    of Research and Development. (Volume 44) (4), 605--624

    Golub, G H and Van Loan, C F, 2012, `Matrix Computations`, (4th
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dtpqrt'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'), (_b, 'b')),
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _nb = _b_utils._EngineIntScalarType(
        nb, {'fun_name': fun_name, 'entity': 'nb'},
    ).to_ctypes()
    _nb_py = _b_utils._EngineIntScalarType.to_py(_nb)
    _t = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_nb_py, _n_py),
        sorder=_m_sorder,
    )
    _b_lapackeig.dtpqrt(
        _m,
        _n,
        l,
        _nb,
        _a,
        _b,
        _t,
    )
    _a_py = _a
    _b_py = _b
    _t_py = _t
    _return_data = DtpqrtReturnData(
        _a_py,
        _b_py,
        _t_py,
    )
    return _return_data

DtpmqrtReturnData = _collections.namedtuple(
    'DtpmqrtReturnData',
    ['c1', 'c2'],
)

def dtpmqrt(side, trans, l, v, t, c1, c2):
    u"""
    Applies the orthogonal transformation determined by ``dtpqrt``.

    ``dtpmqrt`` multiplies an arbitrary real matrix C by the real
    orthogonal matrix Q from a QR factorization computed by ``dtpqrt``.

    For full information please refer to the NAG Library document for
    f08bc

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08bcf.html

    Parameters
    ----------
    side : str, length 1
        Indicates how Q or Q^T is to be applied to C.

        `side` = 'L'

            Q or Q^T is applied to C from the left.

        `side` = 'R'

            Q or Q^T is applied to C from the right.

    trans : str, length 1
        Indicates whether Q or Q^T is to be applied to C.

        `trans` = 'N'

            Q is applied to C.

        `trans` = 'T'

            Q^T is applied to C.

    l : int
        l, the number of rows of the upper trapezoidal part of the
        pentagonal composite matrix V, passed (as b) in a previous call
        to ``dtpqrt``. This must be the same value used in the previous
        call to ``dtpqrt`` (see l in ``dtpqrt``).

    v : float, array-like, shape (:, k)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `side`='L': m; if `side`='R': n;
        otherwise: 0.

        The m_v*n_v matrix V; this should remain unchanged from the
        array b returned by a previous call to ``dtpqrt``.

    t : float, array-like, shape (nb, k)
        This must remain unchanged from a previous call to ``dtpqrt``
        (see t in ``dtpqrt``).

    c1 : float, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `side`='L': k; if `side`='R': m;
        otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `side`='L': n; if `side`='R': k;
        otherwise: 0.

        C_1, the first part of the composite matrix C:

        if `side` = 'L'

            then `c1` contains the first k rows of C;

        if `side` = 'R'

            then `c1` contains the first k columns of C.

    c2 : float, array-like, shape (m, n)
        C_2, the second part of the composite matrix C.

        if `side` = 'L'

            then `c2` contains the remaining m_v rows of C;

        if `side` = 'R'

            then `c2` contains the remaining m_v columns of C;

    Returns
    -------
    c1 : float, ndarray, shape (:, :)
        `c1` is overwritten by the corresponding block of QC or Q^TC or
        CQ or CQ^T.

    c2 : float, ndarray, shape (m, n)
        `c2` is overwritten by the corresponding block of QC or Q^TC or
        CQ or CQ^T.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'T'.

        (`errno` -3)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -5)
            On entry, error in parameter k.

            Constraint: k >= 0.

        (`errno` -6)
            On entry, error in parameter `l`.

            Constraint: 0 <= `l` <= k.

        (`errno` -7)
            On entry, error in parameter nb.

            Constraint: nb >= 1.

        (`errno` -7)
            On entry, error in parameter nb.

            Constraint: nb <= k.

    Notes
    -----
    ``dtpmqrt`` is intended to be used after a call to ``dtpqrt`` which
    performs a QR factorization of a triangular-pentagonal matrix
    containing an upper triangular matrix A over a pentagonal matrix B.
    The orthogonal matrix Q is represented as a product of elementary
    reflectors.

    This function may be used to form the matrix products

        QC,Q^TC,CQ or CQ^T,

    where the real rectangular m_c*n_c matrix C is split into component
    matrices C_1 and C_2.

    If Q is being applied from the left (QC or Q^TC) then

        C = (C_1;C_2)

    where C_1 is k*n_c, C_2 is m_v*n_c, m_c = k+m_v is fixed and m_v is
    the number of rows of the matrix V containing the elementary
    reflectors (i.e., m as passed to ``dtpqrt``); the number of columns
    of V is n_v (i.e., n as passed to ``dtpqrt``).

    If Q is being applied from the right (CQ or CQ^T) then

        C = (C_1 C_2)

    where C_1 is m_c*k, and C_2 is m_c*m_v and n_c = k+m_v is fixed.

    The matrices C_1 and C_2 are overwriten by the result of the matrix
    product.

    A common application of this routine is in updating the solution of
    a linear least squares problem.

    References
    ----------
    Golub, G H and Van Loan, C F, 2012, `Matrix Computations`, (4th
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dtpmqrt'

    _c2 = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c2'},
        c2,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_c2, {'fun_name': fun_name, 'entity': 'c2'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_c2, {'fun_name': fun_name, 'entity': 'c2'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _v = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'v'},
        v,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _k_py = _b_utils._EngineFloat64ArrayType.get_shape(_v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2)[1]
    _k = _b_utils._EngineIntScalarType(
        _k_py, {'fun_name': fun_name, 'entity': '_k'},
    ).to_ctypes()
    _t = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 't'},
        t,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _nb_py = _b_utils._EngineFloat64ArrayType.get_shape(_t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=2)[0]
    _nb = _b_utils._EngineIntScalarType(
        _nb_py, {'fun_name': fun_name, 'entity': '_nb'},
    ).to_ctypes()
    _c1 = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c1'},
        c1,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b_lapackeig.dtpmqrt(
        side,
        trans,
        _m,
        _n,
        _k,
        l,
        _nb,
        _v,
        _t,
        _c1,
        _c2,
    )
    _c1_py = _c1
    _c2_py = _c2
    _return_data = DtpmqrtReturnData(
        _c1_py,
        _c2_py,
    )
    return _return_data

DgeqpfReturnData = _collections.namedtuple(
    'DgeqpfReturnData',
    ['a', 'jpvt', 'tau'],
)

@_b_utils._withdrawal(
    'naginterfaces.library.lapackeig.dgeqpf',
    mkwdrn=None,
    advice=[
        'Please use dgeqp3 instead.',
        'See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html',
    ]
)
def dgeqpf(a, jpvt):
    u"""
    QR factorization, with column pivoting, of real general rectangular
    matrix.

    ``dgeqpf`` computes the QR factorization, with column pivoting, of a
    real m*n matrix. ``dgeqpf`` is marked as `deprecated` by LAPACK; the
    replacement routine is ``dgeqp3`` which makes better use of Level 3
    BLAS.

    Note: ``dgeqpf`` is deprecated.
    Please use ``dgeqp3`` instead.
    See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html

    For full information please refer to the NAG Library document for
    f08be

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08bef.html

    Parameters
    ----------
    a : float, array-like, shape (m, n)
        The m*n matrix A.

    jpvt : int, array-like, shape (n)
        If `jpvt`[i-1] != 0, the i th column of A is moved to the
        beginning of AP before the decomposition is computed and is
        fixed in place during the computation. Otherwise, the i th
        column of A is a free column (i.e., one which may be
        interchanged during the computation with any other free column).

    Returns
    -------
    a : float, ndarray, shape (m, n)
        If m >= n, the elements below the diagonal are overwritten by
        details of the orthogonal matrix Q and the upper triangle is
        overwritten by the corresponding elements of the n*n upper
        triangular matrix R.

        If m < n, the strictly lower triangular part is overwritten by
        details of the orthogonal matrix Q and the remaining elements
        are overwritten by the corresponding elements of the m*n upper
        trapezoidal matrix R.

    jpvt : int, ndarray, shape (n)
        Details of the permutation matrix P. More precisely, if
        `jpvt`[i-1] = k, the kth column of A is moved to become the i th
        column of AP; in other words, the columns of AP are the columns
        of A in the order `jpvt`[0],`jpvt`[1],...,`jpvt`[n-1].

    tau : float, ndarray, shape (min(m, n))
        Further details of the orthogonal matrix Q.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= 0.

    Notes
    -----
    ``dgeqpf`` forms the QR factorization, with column pivoting, of an
    arbitrary rectangular real m*n matrix.

    If m >= n, the factorization is given by:

        AP = Q(R;0),

    where R is an n*n upper triangular matrix, Q is an m*m orthogonal
    matrix and P is an n*n permutation matrix.
    It is sometimes more convenient to write the factorization as

        AP = (Q_1 Q_2)(R;0),

    which reduces to

        AP = Q_1 R,

    where Q_1 consists of the first n columns of Q, and Q_2 the
    remaining m-n columns.

    If m < n, R is trapezoidal, and the factorization can be written

        AP = Q(R_1 R_2),

    where R_1 is upper triangular and R_2 is rectangular.

    The matrix Q is not formed explicitly but is represented as a
    product of min(m, n) elementary reflectors (see [the F08
    Introduction] for details). Functions are provided to work with Q in
    this representation (see [Further Comments]).

    Note also that for any k < n, the information returned in the first
    k columns of the array `a` represents a QR factorization of the
    first k columns of the permuted matrix AP.

    The function allows specified columns of A to be moved to the
    leading columns of AP at the start of the factorization and fixed
    there.
    The remaining columns are free to be interchanged so that at the ith
    stage the pivot column is chosen to be the column which maximizes
    the 2-norm of elements i to m over columns i to n.

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dgeqpf'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _jpvt = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'jpvt'},
        jpvt,
        copy=True,
    )
    _tau = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (min(_m_py, _n_py),),
    )
    _b_lapackeig.dgeqpf(
        _m,
        _n,
        _a,
        _jpvt,
        _tau,
    )
    _a_py = _a
    _jpvt_py = _jpvt
    _tau_py = _tau
    _return_data = DgeqpfReturnData(
        _a_py,
        _jpvt_py,
        _tau_py,
    )
    return _return_data

Dgeqp3ReturnData = _collections.namedtuple(
    'Dgeqp3ReturnData',
    ['a', 'jpvt', 'tau'],
)

def dgeqp3(a, jpvt):
    u"""
    QR factorization, with column pivoting, using BLAS-3, of real
    general rectangular matrix.

    ``dgeqp3`` computes the QR factorization, with column pivoting, of a
    real m*n matrix.

    For full information please refer to the NAG Library document for
    f08bf

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08bff.html

    Parameters
    ----------
    a : float, array-like, shape (m, n)
        The m*n matrix A.

    jpvt : int, array-like, shape (n)
        If `jpvt`[j-1] != 0, the j th column of A is moved to the
        beginning of AP before the decomposition is computed and is
        fixed in place during the computation. Otherwise, the j th
        column of A is a free column (i.e., one which may be
        interchanged during the computation with any other free column).

    Returns
    -------
    a : float, ndarray, shape (m, n)
        If m >= n, the elements below the diagonal are overwritten by
        details of the orthogonal matrix Q and the upper triangle is
        overwritten by the corresponding elements of the n*n upper
        triangular matrix R.

        If m < n, the strictly lower triangular part is overwritten by
        details of the orthogonal matrix Q and the remaining elements
        are overwritten by the corresponding elements of the m*n upper
        trapezoidal matrix R.

    jpvt : int, ndarray, shape (n)
        Details of the permutation matrix P. More precisely, if
        `jpvt`[j-1] = k, the kth column of A is moved to become the j th
        column of AP; in other words, the columns of AP are the columns
        of A in the order `jpvt`[0],`jpvt`[1],...,`jpvt`[n-1].

    tau : float, ndarray, shape (min(m, n))
        The scalar factors of the elementary reflectors.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= 0.

    Notes
    -----
    ``dgeqp3`` forms the QR factorization, with column pivoting, of an
    arbitrary rectangular real m*n matrix.

    If m >= n, the factorization is given by:

        AP = Q(R;0),

    where R is an n*n upper triangular matrix, Q is an m*m orthogonal
    matrix and P is an n*n permutation matrix.
    It is sometimes more convenient to write the factorization as

        AP = (Q_1 Q_2)(R;0),

    which reduces to

        AP = Q_1 R,

    where Q_1 consists of the first n columns of Q, and Q_2 the
    remaining m-n columns.

    If m < n, R is trapezoidal, and the factorization can be written

        AP = Q(R_1 R_2),

    where R_1 is upper triangular and R_2 is rectangular.

    The matrix Q is not formed explicitly but is represented as a
    product of min(m, n) elementary reflectors (see [the F08
    Introduction] for details). Functions are provided to work with Q in
    this representation (see [Further Comments]).

    Note also that for any k < n, the information returned in the first
    k columns of the array `a` represents a QR factorization of the
    first k columns of the permuted matrix AP.

    The function allows specified columns of A to be moved to the
    leading columns of AP at the start of the factorization and fixed
    there.
    The remaining columns are free to be interchanged so that at the ith
    stage the pivot column is chosen to be the column which maximizes
    the 2-norm of elements i to m over columns i to n.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dgeqp3'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _jpvt = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'jpvt'},
        jpvt,
        copy=True,
    )
    _tau = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (min(_m_py, _n_py),),
    )
    _b_lapackeig.dgeqp3(
        _m,
        _n,
        _a,
        _jpvt,
        _tau,
    )
    _a_py = _a
    _jpvt_py = _jpvt
    _tau_py = _tau
    _return_data = Dgeqp3ReturnData(
        _a_py,
        _jpvt_py,
        _tau_py,
    )
    return _return_data

DtzrzfReturnData = _collections.namedtuple(
    'DtzrzfReturnData',
    ['a', 'tau'],
)

def dtzrzf(a):
    u"""
    Reduces a real upper trapezoidal matrix to upper triangular form.

    ``dtzrzf`` reduces the m*n (m <= n) real upper trapezoidal matrix A
    to upper triangular form by means of orthogonal transformations.

    For full information please refer to the NAG Library document for
    f08bh

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08bhf.html

    Parameters
    ----------
    a : float, array-like, shape (m, n)
        The leading m*n upper trapezoidal part of the array `a` must
        contain the matrix to be factorized.

    Returns
    -------
    a : float, ndarray, shape (m, n)
        The leading m*m upper triangular part of `a` contains the upper
        triangular matrix R, and elements m+1 to n of the first m rows
        of `a`, with the array `tau`, represent the orthogonal matrix Z
        as a product of m elementary reflectors (see [the F08
        Introduction]).

    tau : float, ndarray, shape (m)
        The scalar factors of the elementary reflectors.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= m.

    Notes
    -----
    The m*n (m <= n) real upper trapezoidal matrix A given by

        A = (R_1 R_2),

    where R_1 is an m*m upper triangular matrix and R_2 is an m*(n-m)
    matrix, is factorized as

        A = (R 0)Z,

    where R is also an m*m upper triangular matrix and Z is an n*n
    orthogonal matrix.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug
    """

    fun_name = 'naginterfaces.library.lapackeig.dtzrzf'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _tau = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py,),
    )
    _b_lapackeig.dtzrzf(
        _m,
        _n,
        _a,
        _tau,
    )
    _a_py = _a
    _tau_py = _tau
    _return_data = DtzrzfReturnData(
        _a_py,
        _tau_py,
    )
    return _return_data

def dormrz(side, trans, l, a, tau, c):
    u"""
    Applies the orthogonal transformation determined by ``dtzrzf``.

    ``dormrz`` multiplies a general real m*n matrix C by the real
    orthogonal matrix Z from an RZ factorization computed by ``dtzrzf``.

    For full information please refer to the NAG Library document for
    f08bk

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08bkf.html

    Parameters
    ----------
    side : str, length 1
        Indicates how Z or Z^T is to be applied to C.

        `side` = 'L'

            Z or Z^T is applied to C from the left.

        `side` = 'R'

            Z or Z^T is applied to C from the right.

    trans : str, length 1
        Indicates whether Z or Z^T is to be applied to C.

        `trans` = 'N'

            Z is applied to C.

        `trans` = 'T'

            Z^T is applied to C.

    l : int
        l, the number of columns of the matrix A containing the
        meaningful part of the Householder reflectors.

    a : float, array-like, shape (k, :)
        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `side`='L': m; if `side`='R': n;
        otherwise: 0.

        The ith row of `a` must contain the vector which defines the
        elementary reflector H_i, for i = 1,2,...,k, as returned by
        ``dtzrzf``.

    tau : float, array-like, shape (k)
        `tau`[i-1] must contain the scalar factor of the elementary
        reflector H_i, as returned by ``dtzrzf``.

    c : float, array-like, shape (m, n)
        The m*n matrix C.

    Returns
    -------
    c : float, ndarray, shape (m, n)
        `c` is overwritten by ZC or Z^TC or CZ or Z^TC as specified by
        `side` and `trans`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'T'.

        (`errno` -3)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -5)
            On entry, error in parameter k.

            Constraint: m >= k >= 0.

        (`errno` -5)
            On entry, error in parameter k.

            Constraint: n >= k >= 0.

        (`errno` -6)
            On entry, error in parameter `l`.

            Constraint: m >= `l` >= 0.

        (`errno` -6)
            On entry, error in parameter `l`.

            Constraint: n >= `l` >= 0.

    Notes
    -----
    ``dormrz`` is intended to be used following a call to ``dtzrzf``,
    which performs an RZ factorization of a real upper trapezoidal
    matrix A and represents the orthogonal matrix Z as a product of
    elementary reflectors.

    This function may be used to form one of the matrix products

        ZC, Z^TC, CZ, CZ^T,

    overwriting the result on C, which may be any real rectangular m*n
    matrix.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug
    """

    fun_name = 'naginterfaces.library.lapackeig.dormrz'

    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _tau = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'tau'},
        tau,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _k_py = _b_utils._EngineFloat64ArrayType.get_shape(_tau, {'fun_name': fun_name, 'entity': 'tau'}, exp_rank=1)[0]
    _k = _b_utils._EngineIntScalarType(
        _k_py, {'fun_name': fun_name, 'entity': '_k'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b_lapackeig.dormrz(
        side,
        trans,
        _m,
        _n,
        _k,
        l,
        _a,
        _tau,
        _c,
    )
    _c_py = _c
    return _c_py

ZgelsyReturnData = _collections.namedtuple(
    'ZgelsyReturnData',
    ['a', 'b', 'jpvt', 'rank'],
)

def zgelsy(a, b, jpvt, rcond):
    u"""
    Computes the minimum-norm solution to a complex linear least squares
    problem.

    ``zgelsy`` computes the minimum norm solution to a complex linear
    least squares problem

        min_x||b-Ax||_2

    using a complete orthogonal factorization of A. A is an m*n matrix
    which may be rank-deficient.
    Several right-hand side vectors b and solution vectors x can be
    handled in a single call.

    For full information please refer to the NAG Library document for
    f08bn

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08bnf.html

    Parameters
    ----------
    a : complex, array-like, shape (m, n)
        The m*n matrix A.

    b : complex, array-like, shape (max(m, n), nrhs)
        The m*r right-hand side matrix B.

    jpvt : int, array-like, shape (n)
        If `jpvt`[i-1] != 0, the ith column of A is permuted to the
        front of AP, otherwise column i is a free column.

    rcond : float
        Used to determine the effective rank of A, which is defined as
        the order of the largest leading triangular sub-matrix R_11 in
        the QR factorization of A, whose estimated condition number is <
        1/`rcond`.

        Suggested value: if the condition number of `a` is not known
        then `rcond` = sqrt((epsilon)/2) (where epsilon is machine
        precision, see ``machine.precision``) is a good choice. Negative
        values or values less than machine precision should be avoided
        since this will cause `a` to have an effective rank = min(m, n)
        that could be larger than its actual rank, leading to
        meaningless results.

    Returns
    -------
    a : complex, ndarray, shape (m, n)
        `a` has been overwritten by details of its complete orthogonal
        factorization.

    b : complex, ndarray, shape (max(m, n), nrhs)
        The n*r solution matrix X.

    jpvt : int, ndarray, shape (n)
        If `jpvt`[i-1] = k, the ith column of AP was the kth column of
        A.

    rank : int
        The effective rank of A, i.e., the order of the sub-matrix R_11.
        This is the same as the order of the sub-matrix T_11 in the
        complete orthogonal factorization of A.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -3)
            On entry, error in parameter nrhs.

            Constraint: nrhs >= 0.

    Notes
    -----
    The right-hand side vectors are stored as the columns of the m*r
    matrix B and the solution vectors in the n*r matrix X.

    ``zgelsy`` first computes a QR factorization with column pivoting

        AP = Q[table omitted],

    with R_11 defined as the largest leading sub-matrix whose estimated
    condition number is less than 1/`rcond`.
    The order of R_11, `rank`, is the effective rank of A.

    Then, R_22 is considered to be negligible, and R_12 is annihilated
    by orthogonal transformations from the right, arriving at the
    complete orthogonal factorization

        AP = Q[table omitted]Z.

    The minimum norm solution is then

        X = PZ^H(T_11^-1 Q_1^H b ;0)

    where Q_1 consists of the first `rank` columns of Q.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zgelsy'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
    )
    _nrhs_py = _b_utils._EngineComplex128ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[1]
    _nrhs = _b_utils._EngineIntScalarType(
        _nrhs_py, {'fun_name': fun_name, 'entity': '_nrhs'},
    ).to_ctypes()
    _jpvt = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'jpvt'},
        jpvt,
        copy=True,
    )
    _return_scalar = _b_lapackeig.zgelsy(
        _m,
        _n,
        _nrhs,
        _a,
        _b,
        _jpvt,
        rcond,
    )
    _a_py = _a
    _b_py = _b
    _jpvt_py = _jpvt
    _rank_py = _return_scalar
    _return_data = ZgelsyReturnData(
        _a_py,
        _b_py,
        _jpvt_py,
        _rank_py,
    )
    return _return_data

ZtpqrtReturnData = _collections.namedtuple(
    'ZtpqrtReturnData',
    ['a', 'b', 't'],
)

def ztpqrt(l, nb, a, b):
    u"""
    QR factorization of complex triangular-pentagonal matrix.

    ``ztpqrt`` computes the QR factorization of a complex (m+n)*n
    triangular-pentagonal matrix.

    For full information please refer to the NAG Library document for
    f08bp

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08bpf.html

    Parameters
    ----------
    l : int
        l, the number of rows of the trapezoidal part of B (i.e., B_2).

    nb : int
        The explicitly chosen block-size to be used in the algorithm for
        computing the QR factorization. See [Further Comments] for
        details.

    a : complex, array-like, shape (n, n)
        The n*n upper triangular matrix A.

    b : complex, array-like, shape (m, n)
        The m*n pentagonal matrix B composed of an (m-l)*n rectangular
        matrix B_1 above an l*n upper trapezoidal matrix B_2.

    Returns
    -------
    a : complex, ndarray, shape (n, n)
        The upper triangle is overwritten by the corresponding elements
        of the n*n upper triangular matrix R.

    b : complex, ndarray, shape (m, n)
        Details of the unitary matrix Q.

    t : complex, ndarray, shape (`nb`, n)
        Further details of the unitary matrix Q. The number of blocks is
        b = ceiling(k/`nb`), where k = min(m, n) and each block is of
        order `nb` except for the last block, which is of order
        k-(b-1)*`nb`. For each of the blocks, an upper triangular block
        reflector factor is computed: T_1,T_2,...,T_b. These are stored
        in the `nb`*n matrix T as T = [T_1|T_2|...|T_b].

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -3)
            On entry, error in parameter `l`.

            Constraint: 0 <= `l` <= min(m, n).

        (`errno` -4)
            On entry, error in parameter `nb`.

            Constraint: `nb` >= 1.

        (`errno` -4)
            On entry, error in parameter `nb`.

            Constraint: `nb` <= n.

    Notes
    -----
    ``ztpqrt`` forms the QR factorization of a complex (m+n)*n
    triangular-pentagonal matrix C,

        C = (A;B)

    where A is an upper triangular n*n matrix and B is an m*n pentagonal
    matrix consisting of an (m-l)*n rectangular matrix B_1 on top of an
    l*n upper trapezoidal matrix B_2:

        B = (B_1;B_2).

    The upper trapezoidal matrix B_2 consists of the first l rows of an
    n*n upper triangular matrix, where 0≤l≤min(m, n).
    If l = 0, B is m*n rectangular; if l = n and m = n, B is upper
    triangular.

    A recursive, explicitly blocked, QR factorization (see ``zgeqrt``)
    is performed on the matrix C.
    The upper triangular matrix R, details of the unitary matrix Q, and
    further details (the block reflector factors) of Q are returned.

    Typically the matrix A or B_2 contains the matrix R from the QR
    factorization of a subproblem and ``ztpqrt`` performs the QR update
    operation from the inclusion of matrix B_1.

    For example, consider the QR factorization of an l*n matrix B^ with
    l < n: B^= Q^R^, R^= (R_1^R_2^), where R_1^ is l*l upper triangular
    and R_2^ is (n-l)*n rectangular (this can be performed by
    ``zgeqrt``).
    Given an initial least squares problem B^X^= Y^ where X and Y are
    l*nrhs matrices, we have R^X^= Q^^HY^.

    Now, adding an additional m-l rows to the original system gives the
    augmented least squares problem

        BX = Y

    where B is an m*n matrix formed by adding m-l rows on top of R^ and
    Y is an m*nrhs matrix formed by adding m-l rows on top of Q^^HY^.

    ``ztpqrt`` can then be used to perform the QR factorization of the
    pentagonal matrix B; the n*n matrix A will be zero on input and
    contain R on output.

    In the case where B^ is r*n, r >= n, R^ is n*n upper triangular
    (forming A) on top of r-n rows of zeros (forming first r-n rows of
    B).
    Augmentation is then performed by adding rows to the bottom of B
    with l = 0.

    References
    ----------
    Elmroth, E and Gustavson, F, 2000, `Applying Recursion to Serial and
    Parallel QR Factorization Leads to Better Performance`, IBM Journal
    of Research and Development. (Volume 44) (4), 605--624

    Golub, G H and Van Loan, C F, 2012, `Matrix Computations`, (4th
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.ztpqrt'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'), (_b, 'b')),
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _nb = _b_utils._EngineIntScalarType(
        nb, {'fun_name': fun_name, 'entity': 'nb'},
    ).to_ctypes()
    _nb_py = _b_utils._EngineIntScalarType.to_py(_nb)
    _t = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_nb_py, _n_py),
        sorder=_m_sorder,
    )
    _b_lapackeig.ztpqrt(
        _m,
        _n,
        l,
        _nb,
        _a,
        _b,
        _t,
    )
    _a_py = _a
    _b_py = _b
    _t_py = _t
    _return_data = ZtpqrtReturnData(
        _a_py,
        _b_py,
        _t_py,
    )
    return _return_data

ZtpmqrtReturnData = _collections.namedtuple(
    'ZtpmqrtReturnData',
    ['c1', 'c2'],
)

def ztpmqrt(side, trans, l, v, t, c1, c2):
    u"""
    Applies the unitary transformation determined by ``ztpqrt``.

    ``ztpmqrt`` multiplies an arbitrary complex matrix C by the complex
    unitary matrix Q from a QR factorization computed by ``ztpqrt``.

    For full information please refer to the NAG Library document for
    f08bq

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08bqf.html

    Parameters
    ----------
    side : str, length 1
        Indicates how Q or Q^H is to be applied to C.

        `side` = 'L'

            Q or Q^H is applied to C from the left.

        `side` = 'R'

            Q or Q^H is applied to C from the right.

    trans : str, length 1
        Indicates whether Q or Q^H is to be applied to C.

        `trans` = 'N'

            Q is applied to C.

        `trans` = 'C'

            Q^H is applied to C.

    l : int
        l, the number of rows of the upper trapezoidal part of the
        pentagonal composite matrix V, passed (as b) in a previous call
        to ``ztpqrt``. This must be the same value used in the previous
        call to ``ztpqrt`` (see l in ``ztpqrt``).

    v : complex, array-like, shape (:, k)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `side`='L': m; if `side`='R': n;
        otherwise: 0.

        The m_v*n_v matrix V; this should remain unchanged from the
        array b returned by a previous call to ``ztpqrt``.

    t : complex, array-like, shape (nb, k)
        This must remain unchanged from a previous call to ``ztpqrt``
        (see t in ``ztpqrt``).

    c1 : complex, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `side`='L': k; if `side`='R': m;
        otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `side`='L': n; if `side`='R': k;
        otherwise: 0.

        C_1, the first part of the composite matrix C:

        if `side` = 'L'

            then `c1` contains the first k rows of C;

        if `side` = 'R'

            then `c1` contains the first k columns of C.

    c2 : complex, array-like, shape (m, n)
        C_2, the second part of the composite matrix C.

        if `side` = 'L'

            then `c2` contains the remaining m_v rows of C;

        if `side` = 'R'

            then `c2` contains the remaining m_v columns of C;

    Returns
    -------
    c1 : complex, ndarray, shape (:, :)
        `c1` is overwritten by the corresponding block of QC or Q^HC or
        CQ or CQ^H.

    c2 : complex, ndarray, shape (m, n)
        `c2` is overwritten by the corresponding block of QC or Q^HC or
        CQ or CQ^H.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'C'.

        (`errno` -3)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -5)
            On entry, error in parameter k.

            Constraint: k >= 0.

        (`errno` -6)
            On entry, error in parameter `l`.

            Constraint: 0 <= `l` <= k.

        (`errno` -7)
            On entry, error in parameter nb.

            Constraint: nb >= 1.

        (`errno` -7)
            On entry, error in parameter nb.

            Constraint: nb <= k.

    Notes
    -----
    ``ztpmqrt`` is intended to be used after a call to ``ztpqrt`` which
    performs a QR factorization of a triangular-pentagonal matrix
    containing an upper triangular matrix A over a pentagonal matrix B.
    The unitary matrix Q is represented as a product of elementary
    reflectors.

    This function may be used to form the matrix products

        QC,Q^HC,CQ or CQ^H,

    where the complex rectangular m_c*n_c matrix C is split into
    component matrices C_1 and C_2.

    If Q is being applied from the left (QC or Q^HC) then

        C = (C_1;C_2)

    where C_1 is k*n_c, C_2 is m_v*n_c, m_c = k+m_v is fixed and m_v is
    the number of rows of the matrix V containing the elementary
    reflectors (i.e., m as passed to ``ztpqrt``); the number of columns
    of V is n_v (i.e., n as passed to ``ztpqrt``).

    If Q is being applied from the right (CQ or CQ^H) then

        C = (C_1 C_2)

    where C_1 is m_c*k, and C_2 is m_c*m_v and n_c = k+m_v is fixed.

    The matrices C_1 and C_2 are overwriten by the result of the matrix
    product.

    A common application of this routine is in updating the solution of
    a linear least squares problem.

    References
    ----------
    Golub, G H and Van Loan, C F, 2012, `Matrix Computations`, (4th
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.ztpmqrt'

    _c2 = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c2'},
        c2,
        copy=True,
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_c2, {'fun_name': fun_name, 'entity': 'c2'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_c2, {'fun_name': fun_name, 'entity': 'c2'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _v = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'v'},
        v,
        copy=False,
    )
    _k_py = _b_utils._EngineComplex128ArrayType.get_shape(_v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2)[1]
    _k = _b_utils._EngineIntScalarType(
        _k_py, {'fun_name': fun_name, 'entity': '_k'},
    ).to_ctypes()
    _t = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 't'},
        t,
        copy=False,
    )
    _nb_py = _b_utils._EngineComplex128ArrayType.get_shape(_t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=2)[0]
    _nb = _b_utils._EngineIntScalarType(
        _nb_py, {'fun_name': fun_name, 'entity': '_nb'},
    ).to_ctypes()
    _c1 = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c1'},
        c1,
        copy=True,
    )
    _b_lapackeig.ztpmqrt(
        side,
        trans,
        _m,
        _n,
        _k,
        l,
        _nb,
        _v,
        _t,
        _c1,
        _c2,
    )
    _c1_py = _c1
    _c2_py = _c2
    _return_data = ZtpmqrtReturnData(
        _c1_py,
        _c2_py,
    )
    return _return_data

ZgeqpfReturnData = _collections.namedtuple(
    'ZgeqpfReturnData',
    ['a', 'jpvt', 'tau'],
)

@_b_utils._withdrawal(
    'naginterfaces.library.lapackeig.zgeqpf',
    mkwdrn=None,
    advice=[
        'Please use zgeqp3 instead.',
        'See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html',
    ]
)
def zgeqpf(a, jpvt):
    u"""
    QR factorization, with column pivoting, of complex general
    rectangular matrix.

    ``zgeqpf`` computes the QR factorization, with column pivoting, of a
    complex m*n matrix. ``zgeqpf`` is marked as `deprecated` by LAPACK;
    the replacement routine is ``zgeqp3`` which makes better use of
    Level 3 BLAS.

    Note: ``zgeqpf`` is deprecated.
    Please use ``zgeqp3`` instead.
    See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html

    For full information please refer to the NAG Library document for
    f08bs

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08bsf.html

    Parameters
    ----------
    a : complex, array-like, shape (m, n)
        The m*n matrix A.

    jpvt : int, array-like, shape (n)
        If `jpvt`[i-1] != 0, the i th column of A is moved to the
        beginning of AP before the decomposition is computed and is
        fixed in place during the computation. Otherwise, the i th
        column of A is a free column (i.e., one which may be
        interchanged during the computation with any other free column).

    Returns
    -------
    a : complex, ndarray, shape (m, n)
        If m >= n, the elements below the diagonal are overwritten by
        details of the unitary matrix Q and the upper triangle is
        overwritten by the corresponding elements of the n*n upper
        triangular matrix R.

        If m < n, the strictly lower triangular part is overwritten by
        details of the unitary matrix Q and the remaining elements are
        overwritten by the corresponding elements of the m*n upper
        trapezoidal matrix R.

        The diagonal elements of R are real.

    jpvt : int, ndarray, shape (n)
        Details of the permutation matrix P. More precisely, if
        `jpvt`[i-1] = k, the kth column of A is moved to become the i th
        column of AP; in other words, the columns of AP are the columns
        of A in the order `jpvt`[0],`jpvt`[1],...,`jpvt`[n-1].

    tau : complex, ndarray, shape (min(m, n))
        Further details of the unitary matrix Q.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= 0.

    Notes
    -----
    ``zgeqpf`` forms the QR factorization, with column pivoting, of an
    arbitrary rectangular complex m*n matrix.

    If m >= n, the factorization is given by:

        AP = Q(R;0),

    where R is an n*n upper triangular matrix (with real diagonal
    elements), Q is an m*m unitary matrix and P is an n*n permutation
    matrix.
    It is sometimes more convenient to write the factorization as

        AP = (Q_1 Q_2)(R;0),

    which reduces to

        AP = Q_1 R,

    where Q_1 consists of the first n columns of Q, and Q_2 the
    remaining m-n columns.

    If m < n, R is trapezoidal, and the factorization can be written

        AP = Q(R_1 R_2),

    where R_1 is upper triangular and R_2 is rectangular.

    The matrix Q is not formed explicitly but is represented as a
    product of min(m, n) elementary reflectors (see [the F08
    Introduction] for details). Functions are provided to work with Q in
    this representation (see [Further Comments]).

    Note also that for any k < n, the information returned in the first
    k columns of the array `a` represents a QR factorization of the
    first k columns of the permuted matrix AP.

    The function allows specified columns of A to be moved to the
    leading columns of AP at the start of the factorization and fixed
    there.
    The remaining columns are free to be interchanged so that at the ith
    stage the pivot column is chosen to be the column which maximizes
    the 2-norm of elements i to m over columns i to n.

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zgeqpf'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _jpvt = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'jpvt'},
        jpvt,
        copy=True,
    )
    _tau = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (min(_m_py, _n_py),),
    )
    _b_lapackeig.zgeqpf(
        _m,
        _n,
        _a,
        _jpvt,
        _tau,
    )
    _a_py = _a
    _jpvt_py = _jpvt
    _tau_py = _tau
    _return_data = ZgeqpfReturnData(
        _a_py,
        _jpvt_py,
        _tau_py,
    )
    return _return_data

Zgeqp3ReturnData = _collections.namedtuple(
    'Zgeqp3ReturnData',
    ['a', 'jpvt', 'tau'],
)

def zgeqp3(a, jpvt):
    u"""
    QR factorization, with column pivoting, using BLAS-3, of complex
    general rectangular matrix.

    ``zgeqp3`` computes the QR factorization, with column pivoting, of a
    complex m*n matrix.

    For full information please refer to the NAG Library document for
    f08bt

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08btf.html

    Parameters
    ----------
    a : complex, array-like, shape (m, n)
        The m*n matrix A.

    jpvt : int, array-like, shape (n)
        If `jpvt`[j-1] != 0, the j th column of A is moved to the
        beginning of AP before the decomposition is computed and is
        fixed in place during the computation. Otherwise, the j th
        column of A is a free column (i.e., one which may be
        interchanged during the computation with any other free column).

    Returns
    -------
    a : complex, ndarray, shape (m, n)
        If m >= n, the elements below the diagonal are overwritten by
        details of the unitary matrix Q and the upper triangle is
        overwritten by the corresponding elements of the n*n upper
        triangular matrix R.

        If m < n, the strictly lower triangular part is overwritten by
        details of the unitary matrix Q and the remaining elements are
        overwritten by the corresponding elements of the m*n upper
        trapezoidal matrix R.

        The diagonal elements of R are real.

    jpvt : int, ndarray, shape (n)
        Details of the permutation matrix P. More precisely, if
        `jpvt`[j-1] = k, the kth column of A is moved to become the j th
        column of AP; in other words, the columns of AP are the columns
        of A in the order `jpvt`[0],`jpvt`[1],...,`jpvt`[n-1].

    tau : complex, ndarray, shape (min(m, n))
        Further details of the unitary matrix Q.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= 0.

    Notes
    -----
    ``zgeqp3`` forms the QR factorization, with column pivoting, of an
    arbitrary rectangular complex m*n matrix.

    If m >= n, the factorization is given by:

        AP = Q(R;0),

    where R is an n*n upper triangular matrix (with real diagonal
    elements), Q is an m*m unitary matrix and P is an n*n permutation
    matrix.
    It is sometimes more convenient to write the factorization as

        AP = (Q_1 Q_2)(R;0),

    which reduces to

        AP = Q_1 R,

    where Q_1 consists of the first n columns of Q, and Q_2 the
    remaining m-n columns.

    If m < n, R is trapezoidal, and the factorization can be written

        AP = Q(R_1 R_2),

    where R_1 is upper triangular and R_2 is rectangular.

    The matrix Q is not formed explicitly but is represented as a
    product of min(m, n) elementary reflectors (see [the F08
    Introduction] for details). Functions are provided to work with Q in
    this representation (see [Further Comments]).

    Note also that for any k < n, the information returned in the first
    k columns of the array `a` represents a QR factorization of the
    first k columns of the permuted matrix AP.

    The function allows specified columns of A to be moved to the
    leading columns of AP at the start of the factorization and fixed
    there.
    The remaining columns are free to be interchanged so that at the ith
    stage the pivot column is chosen to be the column which maximizes
    the 2-norm of elements i to m over columns i to n.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zgeqp3'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _jpvt = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'jpvt'},
        jpvt,
        copy=True,
    )
    _tau = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (min(_m_py, _n_py),),
    )
    _b_lapackeig.zgeqp3(
        _m,
        _n,
        _a,
        _jpvt,
        _tau,
    )
    _a_py = _a
    _jpvt_py = _jpvt
    _tau_py = _tau
    _return_data = Zgeqp3ReturnData(
        _a_py,
        _jpvt_py,
        _tau_py,
    )
    return _return_data

ZtzrzfReturnData = _collections.namedtuple(
    'ZtzrzfReturnData',
    ['a', 'tau'],
)

def ztzrzf(a):
    u"""
    Reduces a complex upper trapezoidal matrix to upper triangular form.

    ``ztzrzf`` reduces the m*n (m <= n) complex upper trapezoidal matrix
    A to upper triangular form by means of unitary transformations.

    For full information please refer to the NAG Library document for
    f08bv

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08bvf.html

    Parameters
    ----------
    a : complex, array-like, shape (m, n)
        The leading m*n upper trapezoidal part of the array `a` must
        contain the matrix to be factorized.

    Returns
    -------
    a : complex, ndarray, shape (m, n)
        The leading m*m upper triangular part of `a` contains the upper
        triangular matrix R, and elements m+1 to n of the first m rows
        of `a`, with the array `tau`, represent the unitary matrix Z as
        a product of m elementary reflectors (see [the F08
        Introduction]).

    tau : complex, ndarray, shape (m)
        The scalar factors of the elementary reflectors.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= m.

    Notes
    -----
    The m*n (m <= n) complex upper trapezoidal matrix A given by

        A = (R_1 R_2),

    where R_1 is an m*m upper triangular matrix and R_2 is an m*(n-m)
    matrix, is factorized as

        A = (R 0)Z,

    where R is also an m*m upper triangular matrix and Z is an n*n
    unitary matrix.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug
    """

    fun_name = 'naginterfaces.library.lapackeig.ztzrzf'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _tau = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_m_py,),
    )
    _b_lapackeig.ztzrzf(
        _m,
        _n,
        _a,
        _tau,
    )
    _a_py = _a
    _tau_py = _tau
    _return_data = ZtzrzfReturnData(
        _a_py,
        _tau_py,
    )
    return _return_data

def zunmrz(side, trans, l, a, tau, c):
    u"""
    Applies the unitary transformation determined by ``ztzrzf``.

    ``zunmrz`` multiplies a general complex m*n matrix C by the complex
    unitary matrix Z from an RZ factorization computed by ``ztzrzf``.

    For full information please refer to the NAG Library document for
    f08bx

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08bxf.html

    Parameters
    ----------
    side : str, length 1
        Indicates how Z or Z^H is to be applied to C.

        `side` = 'L'

            Z or Z^H is applied to C from the left.

        `side` = 'R'

            Z or Z^H is applied to C from the right.

    trans : str, length 1
        Indicates whether Z or Z^H is to be applied to C.

        `trans` = 'N'

            Z is applied to C.

        `trans` = 'C'

            Z^H is applied to C.

    l : int
        l, the number of columns of the matrix A containing the
        meaningful part of the Householder reflectors.

    a : complex, array-like, shape (k, :)
        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `side`='L': m; if `side`='R': n;
        otherwise: 0.

        The ith row of `a` must contain the vector which defines the
        elementary reflector H_i, for i = 1,2,...,k, as returned by
        ``ztzrzf``.

    tau : complex, array-like, shape (k)
        `tau`[i-1] must contain the scalar factor of the elementary
        reflector H_i, as returned by ``ztzrzf``.

    c : complex, array-like, shape (m, n)
        The m*n matrix C.

    Returns
    -------
    c : complex, ndarray, shape (m, n)
        `c` is overwritten by ZC or Z^HC or CZ or Z^HC as specified by
        `side` and `trans`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'C'.

        (`errno` -3)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -5)
            On entry, error in parameter k.

            Constraint: m >= k >= 0.

        (`errno` -5)
            On entry, error in parameter k.

            Constraint: n >= k >= 0.

        (`errno` -6)
            On entry, error in parameter `l`.

            Constraint: m >= `l` >= 0.

        (`errno` -6)
            On entry, error in parameter `l`.

            Constraint: n >= `l` >= 0.

    Notes
    -----
    ``zunmrz`` is intended to be used following a call to ``ztzrzf``,
    which performs an RZ factorization of a real upper trapezoidal
    matrix A and represents the unitary matrix Z as a product of
    elementary reflectors.

    This function may be used to form one of the matrix products

        ZC, Z^HC, CZ, CZ^H,

    overwriting the result on C, which may be any complex rectangular
    m*n matrix.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug
    """

    fun_name = 'naginterfaces.library.lapackeig.zunmrz'

    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=True,
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _tau = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'tau'},
        tau,
        copy=False,
    )
    _k_py = _b_utils._EngineComplex128ArrayType.get_shape(_tau, {'fun_name': fun_name, 'entity': 'tau'}, exp_rank=1)[0]
    _k = _b_utils._EngineIntScalarType(
        _k_py, {'fun_name': fun_name, 'entity': '_k'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _b_lapackeig.zunmrz(
        side,
        trans,
        _m,
        _n,
        _k,
        l,
        _a,
        _tau,
        _c,
    )
    _c_py = _c
    return _c_py

DgeqlfReturnData = _collections.namedtuple(
    'DgeqlfReturnData',
    ['a', 'tau'],
)

def dgeqlf(a):
    u"""
    QL factorization of real general rectangular matrix.

    ``dgeqlf`` computes a QL factorization of a real m*n matrix A.

    For full information please refer to the NAG Library document for
    f08ce

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08cef.html

    Parameters
    ----------
    a : float, array-like, shape (m, n)
        The m*n matrix A.

    Returns
    -------
    a : float, ndarray, shape (m, n)
        If m >= n, the lower triangle of the subarray `a`[m-n:m,0:n]
        contains the n*n lower triangular matrix L.

        If m <= n, the elements on and below the (n-m)th superdiagonal
        contain the m*n lower trapezoidal matrix L.

        The remaining elements, with the array `tau`, represent the
        orthogonal matrix Q as a product of elementary reflectors (see
        [the F08 Introduction]).

    tau : float, ndarray, shape (min(m, n))
        The scalar factors of the elementary reflectors (see [Further
        Comments]).

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= 0.

    Notes
    -----
    ``dgeqlf`` forms the QL factorization of an arbitrary rectangular
    real m*n matrix.

    If m >= n, the factorization is given by:

        A = Q(0;L),

    where L is an n*n lower triangular matrix and Q is an m*m orthogonal
    matrix.
    If m < n the factorization is given by

        A = QL,

    where L is an m*n lower trapezoidal matrix and Q is again an m*m
    orthogonal matrix.
    In the case where m > n the factorization can be expressed as

        A = (Q_1 Q_2)(0;L) = Q_2 L,

    where Q_1 consists of the first m-n columns of Q, and Q_2 the
    remaining n columns.

    The matrix Q is not formed explicitly but is represented as a
    product of min(m, n) elementary reflectors (see [the F08
    Introduction] for details). Functions are provided to work with Q in
    this representation (see [Further Comments]).

    Note also that for any k < n, the information returned in the last k
    columns of the array `a` represents a QL factorization of the last k
    columns of the original matrix A.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dgeqlf'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _tau = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (min(_m_py, _n_py),),
    )
    _b_lapackeig.dgeqlf(
        _m,
        _n,
        _a,
        _tau,
    )
    _a_py = _a
    _tau_py = _tau
    _return_data = DgeqlfReturnData(
        _a_py,
        _tau_py,
    )
    return _return_data

def dorgql(a, tau):
    u"""
    Form all or part of orthogonal Q from QL factorization determined by
    ``dgeqlf``.

    ``dorgql`` generates all or part of the real m*m orthogonal matrix Q
    from a QL factorization computed by ``dgeqlf``.

    For full information please refer to the NAG Library document for
    f08cf

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08cff.html

    Parameters
    ----------
    a : float, array-like, shape (m, n)
        Details of the vectors which define the elementary reflectors,
        as returned by ``dgeqlf``.

    tau : float, array-like, shape (k)
        Further details of the elementary reflectors, as returned by
        ``dgeqlf``.

    Returns
    -------
    a : float, ndarray, shape (m, n)
        The m*n matrix Q.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: m >= n >= 0.

        (`errno` -3)
            On entry, error in parameter k.

            Constraint: n >= k >= 0.

    Notes
    -----
    ``dorgql`` is intended to be used after a call to ``dgeqlf``, which
    performs a QL factorization of a real matrix A.
    The orthogonal matrix Q is represented as a product of elementary
    reflectors.

    This function may be used to generate Q explicitly as a square
    matrix, or to form only its trailing columns.

    Usually Q is determined from the QL factorization of an m*p matrix A
    with m >= p.
    The whole of Q may be computed by calling ``dorgql`` with n set to m
    and k set to p or its trailing p columns by calling ``dorgql`` with
    n and k set to p.

    The columns of Q returned by the last call form an orthonormal basis
    for the space spanned by the columns of A; thus ``dgeqlf`` followed
    by ``dorgql`` can be used to orthogonalize the columns of A.

    The information returned by ``dgeqlf`` also yields the QL
    factorization of the trailing k columns of A, where k < p.
    The orthogonal matrix arising from this factorization can be
    computed by calling ``dorgql`` with n set to m or its trailing k
    columns by calling ``dorgql`` with n set to k.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dorgql'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _tau = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'tau'},
        tau,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _k_py = _b_utils._EngineFloat64ArrayType.get_shape(_tau, {'fun_name': fun_name, 'entity': 'tau'}, exp_rank=1)[0]
    _k = _b_utils._EngineIntScalarType(
        _k_py, {'fun_name': fun_name, 'entity': '_k'},
    ).to_ctypes()
    _b_lapackeig.dorgql(
        _m,
        _n,
        _k,
        _a,
        _tau,
    )
    _a_py = _a
    return _a_py

def dormql(side, trans, a, tau, c):
    u"""
    Applies the orthogonal transformation determined by ``dgeqlf``.

    ``dormql`` multiplies a general real m*n matrix C by the real
    orthogonal matrix Q from a QL factorization computed by ``dgeqlf``.

    For full information please refer to the NAG Library document for
    f08cg

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08cgf.html

    Parameters
    ----------
    side : str, length 1
        Indicates how Q or Q^T is to be applied to C.

        `side` = 'L'

            Q or Q^T is applied to C from the left.

        `side` = 'R'

            Q or Q^T is applied to C from the right.

    trans : str, length 1
        Indicates whether Q or Q^T is to be applied to C.

        `trans` = 'N'

            Q is applied to C.

        `trans` = 'T'

            Q^T is applied to C.

    a : float, array-like, shape (:, k)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `side`='L': m; if `side`='R': n;
        otherwise: 0.

        `On entry`: details of the vectors which define the elementary
        reflectors, as returned by ``dgeqlf``.

        `On exit`: is modified by ``dormql`` but restored on exit.

    tau : float, array-like, shape (k)
        Further details of the elementary reflectors, as returned by
        ``dgeqlf``.

    c : float, array-like, shape (m, n)
        The m*n matrix C.

    Returns
    -------
    c : float, ndarray, shape (m, n)
        `c` is overwritten by QC or Q^TC or CQ or CQ^T as specified by
        `side` and `trans`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'T'.

        (`errno` -3)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -5)
            On entry, error in parameter k.

            Constraint: m >= k >= 0.

        (`errno` -5)
            On entry, error in parameter k.

            Constraint: n >= k >= 0.

    Notes
    -----
    ``dormql`` is intended to be used following a call to ``dgeqlf``,
    which performs a QL factorization of a real matrix A and represents
    the orthogonal matrix Q as a product of elementary reflectors.

    This function may be used to form one of the matrix products

        QC, Q^TC, CQ, CQ^T,

    overwriting the result on C, which may be any real rectangular m*n
    matrix.

    A common application of this function is in solving linear least
    squares problems, as described in [the F08 Introduction].

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug
    """

    fun_name = 'naginterfaces.library.lapackeig.dormql'

    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _k_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _k = _b_utils._EngineIntScalarType(
        _k_py, {'fun_name': fun_name, 'entity': '_k'},
    ).to_ctypes()
    _tau = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'tau'},
        tau,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b_lapackeig.dormql(
        side,
        trans,
        _m,
        _n,
        _k,
        _a,
        _tau,
        _c,
    )
    _c_py = _c
    return _c_py

DgerqfReturnData = _collections.namedtuple(
    'DgerqfReturnData',
    ['a', 'tau'],
)

def dgerqf(a):
    u"""
    RQ factorization of real general rectangular matrix.

    ``dgerqf`` computes an RQ factorization of a real m*n matrix A.

    For full information please refer to the NAG Library document for
    f08ch

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08chf.html

    Parameters
    ----------
    a : float, array-like, shape (m, n)
        The m*n matrix A.

    Returns
    -------
    a : float, ndarray, shape (m, n)
        If m <= n, the upper triangle of the subarray `a`[0:m,n-m:n]
        contains the m*m upper triangular matrix R.

        If m >= n, the elements on and above the (m-n)th subdiagonal
        contain the m*n upper trapezoidal matrix R; the remaining
        elements, with the array `tau`, represent the orthogonal matrix
        Q as a product of min(m, n) elementary reflectors (see [the F08
        Introduction]).

    tau : float, ndarray, shape (min(m, n))
        The scalar factors of the elementary reflectors.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= 0.

    Notes
    -----
    ``dgerqf`` forms the RQ factorization of an arbitrary rectangular
    real m*n matrix.
    If m <= n, the factorization is given by

        A = (0 R)Q,

    where R is an m*m lower triangular matrix and Q is an n*n orthogonal
    matrix.
    If m > n the factorization is given by

        A = RQ,

    where R is an m*n upper trapezoidal matrix and Q is again an n*n
    orthogonal matrix.
    In the case where m < n the factorization can be expressed as

        A = (0 R)(Q_1;Q_2) = RQ_2,

    where Q_1 consists of the first (n-m) rows of Q and Q_2 the
    remaining m rows.

    The matrix Q is not formed explicitly, but is represented as a
    product of min(m, n) elementary reflectors (see [the F08
    Introduction] for details). Functions are provided to work with Q in
    this representation (see [Further Comments]).

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dgerqf'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _tau = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (min(_m_py, _n_py),),
    )
    _b_lapackeig.dgerqf(
        _m,
        _n,
        _a,
        _tau,
    )
    _a_py = _a
    _tau_py = _tau
    _return_data = DgerqfReturnData(
        _a_py,
        _tau_py,
    )
    return _return_data

def dorgrq(a, tau):
    u"""
    Form all or part of orthogonal Q from RQ factorization determined by
    ``dgerqf``.

    ``dorgrq`` generates all or part of the real n*n orthogonal matrix Q
    from an RQ factorization computed by ``dgerqf``.

    For full information please refer to the NAG Library document for
    f08cj

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08cjf.html

    Parameters
    ----------
    a : float, array-like, shape (m, n)
        Details of the vectors which define the elementary reflectors,
        as returned by ``dgerqf``.

    tau : float, array-like, shape (k)
        `tau`[i-1] must contain the scalar factor of the elementary
        reflector H_i, as returned by ``dgerqf``.

    Returns
    -------
    a : float, ndarray, shape (m, n)
        The m*n matrix Q.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= m.

        (`errno` -3)
            On entry, error in parameter k.

            Constraint: m >= k >= 0.

    Notes
    -----
    ``dorgrq`` is intended to be used following a call to ``dgerqf``,
    which performs an RQ factorization of a real matrix A and represents
    the orthogonal matrix Q as a product of k elementary reflectors of
    order n.

    This function may be used to generate Q explicitly as a square
    matrix, or to form only its trailing rows.

    Usually Q is determined from the RQ factorization of a p*n matrix A
    with p <= n.
    The whole of Q may be computed by calling ``dorgrq`` with m set to n
    and k set to p or its trailing p rows by calling ``dorgrq`` with m
    and k set to p.

    The rows of Q returned by the last call form an orthonormal basis
    for the space spanned by the rows of A; thus ``dgerqf`` followed by
    ``dorgrq`` can be used to orthogonalize the rows of A.

    The information returned by ``dgerqf`` also yields the RQ
    factorization of the trailing k rows of A, where k < p.
    The orthogonal matrix arising from this factorization can be
    computed by calling ``dorgrq`` with m set to n or its leading k
    columns by calling ``dorgrq`` with m set to k.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dorgrq'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _tau = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'tau'},
        tau,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _k_py = _b_utils._EngineFloat64ArrayType.get_shape(_tau, {'fun_name': fun_name, 'entity': 'tau'}, exp_rank=1)[0]
    _k = _b_utils._EngineIntScalarType(
        _k_py, {'fun_name': fun_name, 'entity': '_k'},
    ).to_ctypes()
    _b_lapackeig.dorgrq(
        _m,
        _n,
        _k,
        _a,
        _tau,
    )
    _a_py = _a
    return _a_py

DormrqReturnData = _collections.namedtuple(
    'DormrqReturnData',
    ['a', 'c'],
)

def dormrq(side, trans, a, tau, c):
    u"""
    Applies the orthogonal transformation determined by ``dgerqf``.

    ``dormrq`` multiplies a general real m*n matrix C by the real
    orthogonal matrix Q from an RQ factorization computed by ``dgerqf``.

    For full information please refer to the NAG Library document for
    f08ck

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08ckf.html

    Parameters
    ----------
    side : str, length 1
        Indicates how Q or Q^T is to be applied to C.

        `side` = 'L'

            Q or Q^T is applied to C from the left.

        `side` = 'R'

            Q or Q^T is applied to C from the right.

    trans : str, length 1
        Indicates whether Q or Q^T is to be applied to C.

        `trans` = 'N'

            Q is applied to C.

        `trans` = 'T'

            Q^T is applied to C.

    a : float, array-like, shape (k, :)
        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `side`='L': m; if `side`='R': n;
        otherwise: 0.

        The ith row of `a` must contain the vector which defines the
        elementary reflector H_i, for i = 1,2,...,k, as returned by
        ``dgerqf``.

    tau : float, array-like, shape (k)
        `tau`[i-1] must contain the scalar factor of the elementary
        reflector H_i, as returned by ``dgerqf``.

    c : float, array-like, shape (m, n)
        The m*n matrix C.

    Returns
    -------
    a : float, ndarray, shape (k, :)
        Is modified by ``dormrq`` but restored on exit.

    c : float, ndarray, shape (m, n)
        `c` is overwritten by QC or Q^TC or CQ or CQ^T as specified by
        `side` and `trans`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'T'.

        (`errno` -3)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -5)
            On entry, error in parameter k.

            Constraint: m >= k >= 0.

        (`errno` -5)
            On entry, error in parameter k.

            Constraint: n >= k >= 0.

    Notes
    -----
    ``dormrq`` is intended to be used following a call to ``dgerqf``,
    which performs an RQ factorization of a real matrix A and represents
    the orthogonal matrix Q as a product of elementary reflectors.

    This function may be used to form one of the matrix products

        QC, Q^TC, CQ, CQ^T,

    overwriting the result on C, which may be any real rectangular m*n
    matrix.

    A common application of this function is in solving underdetermined
    linear least squares problems, as described in [the F08
    Introduction].

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dormrq'

    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _tau = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'tau'},
        tau,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _k_py = _b_utils._EngineFloat64ArrayType.get_shape(_tau, {'fun_name': fun_name, 'entity': 'tau'}, exp_rank=1)[0]
    _k = _b_utils._EngineIntScalarType(
        _k_py, {'fun_name': fun_name, 'entity': '_k'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b_lapackeig.dormrq(
        side,
        trans,
        _m,
        _n,
        _k,
        _a,
        _tau,
        _c,
    )
    _a_py = _a
    _c_py = _c
    _return_data = DormrqReturnData(
        _a_py,
        _c_py,
    )
    return _return_data

ZgeqlfReturnData = _collections.namedtuple(
    'ZgeqlfReturnData',
    ['a', 'tau'],
)

def zgeqlf(a):
    u"""
    QL factorization of complex general rectangular matrix.

    ``zgeqlf`` computes a QL factorization of a complex m*n matrix A.

    For full information please refer to the NAG Library document for
    f08cs

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08csf.html

    Parameters
    ----------
    a : complex, array-like, shape (m, n)
        The m*n matrix A.

    Returns
    -------
    a : complex, ndarray, shape (m, n)
        If m >= n, the lower triangle of the subarray `a`[m-n:m,0:n]
        contains the n*n lower triangular matrix L.

        If m <= n, the elements on and below the (n-m)th superdiagonal
        contain the m*n lower trapezoidal matrix L.

        The remaining elements, with the array `tau`, represent the
        unitary matrix Q as a product of elementary reflectors (see [the
        F08 Introduction]).

    tau : complex, ndarray, shape (min(m, n))
        The scalar factors of the elementary reflectors (see [Further
        Comments]).

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= 0.

    Notes
    -----
    ``zgeqlf`` forms the QL factorization of an arbitrary rectangular
    complex m*n matrix.

    If m >= n, the factorization is given by:

        A = Q(0;L),

    where L is an n*n lower triangular matrix and Q is an m*m unitary
    matrix.
    If m < n the factorization is given by

        A = QL,

    where L is an m*n lower trapezoidal matrix and Q is again an m*m
    unitary matrix.
    In the case where m > n the factorization can be expressed as

        A = (Q_1 Q_2)(0;L) = Q_2 L,

    where Q_1 consists of the first m-n columns of Q, and Q_2 the
    remaining n columns.

    The matrix Q is not formed explicitly but is represented as a
    product of min(m, n) elementary reflectors (see [the F08
    Introduction] for details). Functions are provided to work with Q in
    this representation (see [Further Comments]).

    Note also that for any k < n, the information returned in the last k
    columns of the array `a` represents a QL factorization of the last k
    columns of the original matrix A.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zgeqlf'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _tau = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (min(_m_py, _n_py),),
    )
    _b_lapackeig.zgeqlf(
        _m,
        _n,
        _a,
        _tau,
    )
    _a_py = _a
    _tau_py = _tau
    _return_data = ZgeqlfReturnData(
        _a_py,
        _tau_py,
    )
    return _return_data

def zungql(a, tau):
    u"""
    Form all or part of unitary Q from QL factorization determined by
    ``zgeqlf``.

    ``zungql`` generates all or part of the complex m*m unitary matrix Q
    from a QL factorization computed by ``zgeqlf``.

    For full information please refer to the NAG Library document for
    f08ct

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08ctf.html

    Parameters
    ----------
    a : complex, array-like, shape (m, n)
        Details of the vectors which define the elementary reflectors,
        as returned by ``zgeqlf``.

    tau : complex, array-like, shape (k)
        Further details of the elementary reflectors, as returned by
        ``zgeqlf``.

    Returns
    -------
    a : complex, ndarray, shape (m, n)
        The m*n matrix Q.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: m >= n >= 0.

        (`errno` -3)
            On entry, error in parameter k.

            Constraint: n >= k >= 0.

    Notes
    -----
    ``zungql`` is intended to be used after a call to ``zgeqlf``, which
    performs a QL factorization of a complex matrix A.
    The unitary matrix Q is represented as a product of elementary
    reflectors.

    This function may be used to generate Q explicitly as a square
    matrix, or to form only its trailing columns.

    Usually Q is determined from the QL factorization of an m*p matrix A
    with m >= p.
    The whole of Q may be computed by calling ``zungql`` with n set to m
    or its trailing p columns by calling ``zungql`` with n and k set to
    p.

    The columns of Q returned by the last call form an orthonormal basis
    for the space spanned by the columns of A; thus ``zgeqlf`` followed
    by ``zungql`` can be used to orthogonalize the columns of A.

    The information returned by ``zgeqlf`` also yields the QL
    factorization of the trailing k columns of A, where k < p.
    The unitary matrix arising from this factorization can be computed
    by calling ``zungql`` with n set to m or its trailing k columns by
    calling ``zungql`` with n set to k.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zungql'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _tau = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'tau'},
        tau,
        copy=False,
    )
    _k_py = _b_utils._EngineComplex128ArrayType.get_shape(_tau, {'fun_name': fun_name, 'entity': 'tau'}, exp_rank=1)[0]
    _k = _b_utils._EngineIntScalarType(
        _k_py, {'fun_name': fun_name, 'entity': '_k'},
    ).to_ctypes()
    _b_lapackeig.zungql(
        _m,
        _n,
        _k,
        _a,
        _tau,
    )
    _a_py = _a
    return _a_py

def zunmql(side, trans, a, tau, c):
    u"""
    Applies the unitary transformation determined by ``zgeqlf``.

    ``zunmql`` multiplies a general complex m*n matrix C by the complex
    unitary matrix Q from a QL factorization computed by ``zgeqlf``.

    For full information please refer to the NAG Library document for
    f08cu

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08cuf.html

    Parameters
    ----------
    side : str, length 1
        Indicates how Q or Q^H is to be applied to C.

        `side` = 'L'

            Q or Q^H is applied to C from the left.

        `side` = 'R'

            Q or Q^H is applied to C from the right.

    trans : str, length 1
        Indicates whether Q or Q^H is to be applied to C.

        `trans` = 'N'

            Q is applied to C.

        `trans` = 'C'

            Q^H is applied to C.

    a : complex, array-like, shape (:, k)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `side`='L': m; if `side`='R': n;
        otherwise: 0.

        `On entry`: details of the vectors which define the elementary
        reflectors, as returned by ``zgeqlf``.

        `On exit`: is modified by ``zunmql`` but restored on exit.

    tau : complex, array-like, shape (k)
        Further details of the elementary reflectors, as returned by
        ``zgeqlf``.

    c : complex, array-like, shape (m, n)
        The m*n matrix C.

    Returns
    -------
    c : complex, ndarray, shape (m, n)
        `c` is overwritten by QC or Q^HC or CQ or CQ^H as specified by
        `side` and `trans`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'C'.

        (`errno` -3)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -5)
            On entry, error in parameter k.

            Constraint: m >= k >= 0.

        (`errno` -5)
            On entry, error in parameter k.

            Constraint: n >= k >= 0.

    Notes
    -----
    ``zunmql`` is intended to be used following a call to ``zgeqlf``,
    which performs a QL factorization of a complex matrix A and
    represents the unitary matrix Q as a product of elementary
    reflectors.

    This function may be used to form one of the matrix products

        QC, Q^HC, CQ, CQ^H,

    overwriting the result on C, which may be any complex rectangular
    m*n matrix.

    A common application of this function is in solving linear least
    squares problems, as described in [the F08 Introduction].

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug
    """

    fun_name = 'naginterfaces.library.lapackeig.zunmql'

    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=True,
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _k_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _k = _b_utils._EngineIntScalarType(
        _k_py, {'fun_name': fun_name, 'entity': '_k'},
    ).to_ctypes()
    _tau = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'tau'},
        tau,
        copy=False,
    )
    _b_lapackeig.zunmql(
        side,
        trans,
        _m,
        _n,
        _k,
        _a,
        _tau,
        _c,
    )
    _c_py = _c
    return _c_py

ZgerqfReturnData = _collections.namedtuple(
    'ZgerqfReturnData',
    ['a', 'tau'],
)

def zgerqf(a):
    u"""
    RQ factorization of complex general rectangular matrix.

    ``zgerqf`` computes an RQ factorization of a complex m*n matrix A.

    For full information please refer to the NAG Library document for
    f08cv

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08cvf.html

    Parameters
    ----------
    a : complex, array-like, shape (m, n)
        The m*n matrix A.

    Returns
    -------
    a : complex, ndarray, shape (m, n)
        If m <= n, the upper triangle of the subarray `a`[0:m,n-m:n]
        contains the m*m upper triangular matrix R.

        If m >= n, the elements on and above the (m-n)th subdiagonal
        contain the m*n upper trapezoidal matrix R; the remaining
        elements, with the array `tau`, represent the unitary matrix Q
        as a product of min(m, n) elementary reflectors (see [the F08
        Introduction]).

    tau : complex, ndarray, shape (min(m, n))
        The scalar factors of the elementary reflectors.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= 0.

    Notes
    -----
    ``zgerqf`` forms the RQ factorization of an arbitrary rectangular
    real m*n matrix.
    If m <= n, the factorization is given by

        A = (0 R)Q,

    where R is an m*m lower triangular matrix and Q is an n*n unitary
    matrix.
    If m > n the factorization is given by

        A = RQ,

    where R is an m*n upper trapezoidal matrix and Q is again an n*n
    unitary matrix.
    In the case where m < n the factorization can be expressed as

        A = (0 R)(Q_1;Q_2) = RQ_2,

    where Q_1 consists of the first (n-m) rows of Q and Q_2 the
    remaining m rows.

    The matrix Q is not formed explicitly, but is represented as a
    product of min(m, n) elementary reflectors (see [the F08
    Introduction] for details). Functions are provided to work with Q in
    this representation (see [Further Comments]).

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zgerqf'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _tau = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (min(_m_py, _n_py),),
    )
    _b_lapackeig.zgerqf(
        _m,
        _n,
        _a,
        _tau,
    )
    _a_py = _a
    _tau_py = _tau
    _return_data = ZgerqfReturnData(
        _a_py,
        _tau_py,
    )
    return _return_data

def zungrq(a, tau):
    u"""
    Form all or part of unitary Q from RQ factorization determined by
    ``zgerqf``.

    ``zungrq`` generates all or part of the complex n*n unitary matrix Q
    from an RQ factorization computed by ``zgerqf``.

    For full information please refer to the NAG Library document for
    f08cw

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08cwf.html

    Parameters
    ----------
    a : complex, array-like, shape (m, n)
        Details of the vectors which define the elementary reflectors,
        as returned by ``zgerqf``.

    tau : complex, array-like, shape (k)
        `tau`[i-1] must contain the scalar factor of the elementary
        reflector H_i, as returned by ``zgerqf``.

    Returns
    -------
    a : complex, ndarray, shape (m, n)
        The m*n matrix Q.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= m.

        (`errno` -3)
            On entry, error in parameter k.

            Constraint: m >= k >= 0.

    Notes
    -----
    ``zungrq`` is intended to be used following a call to ``zgerqf``,
    which performs an RQ factorization of a complex matrix A and
    represents the unitary matrix Q as a product of k elementary
    reflectors of order n.

    This function may be used to generate Q explicitly as a square
    matrix, or to form only its trailing rows.

    Usually Q is determined from the RQ factorization of a p*n matrix A
    with p <= n.
    The whole of Q may be computed by calling ``zungrq`` with m set to n
    and k set to p or its trailing p rows by calling ``zungrq`` with m
    and k set to p.

    The rows of Q returned by the last call form an orthonormal basis
    for the space spanned by the rows of A; thus ``zgerqf`` followed by
    ``zungrq`` can be used to orthogonalize the rows of A.

    The information returned by ``zgerqf`` also yields the RQ
    factorization of the trailing k rows of A, where k < p.
    The unitary matrix arising from this factorization can be computed
    by calling ``zungrq`` with m set to n or its leading k columns by
    calling ``zungrq`` with m set to k.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zungrq'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _tau = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'tau'},
        tau,
        copy=False,
    )
    _k_py = _b_utils._EngineComplex128ArrayType.get_shape(_tau, {'fun_name': fun_name, 'entity': 'tau'}, exp_rank=1)[0]
    _k = _b_utils._EngineIntScalarType(
        _k_py, {'fun_name': fun_name, 'entity': '_k'},
    ).to_ctypes()
    _b_lapackeig.zungrq(
        _m,
        _n,
        _k,
        _a,
        _tau,
    )
    _a_py = _a
    return _a_py

ZunmrqReturnData = _collections.namedtuple(
    'ZunmrqReturnData',
    ['a', 'c'],
)

def zunmrq(side, trans, a, tau, c):
    u"""
    Applies the unitary transformation determined by ``zgerqf``.

    ``zunmrq`` multiplies a general complex m*n matrix C by the complex
    unitary matrix Q from an RQ factorization computed by ``zgerqf``.

    For full information please refer to the NAG Library document for
    f08cx

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08cxf.html

    Parameters
    ----------
    side : str, length 1
        Indicates how Q or Q^H is to be applied to C.

        `side` = 'L'

            Q or Q^H is applied to C from the left.

        `side` = 'R'

            Q or Q^H is applied to C from the right.

    trans : str, length 1
        Indicates whether Q or Q^H is to be applied to C.

        `trans` = 'N'

            Q is applied to C.

        `trans` = 'C'

            Q^H is applied to C.

    a : complex, array-like, shape (k, :)
        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `side`='L': m; if `side`='R': n;
        otherwise: 0.

        The ith row of `a` must contain the vector which defines the
        elementary reflector H_i, for i = 1,2,...,k, as returned by
        ``zgerqf``.

    tau : complex, array-like, shape (k)
        `tau`[i-1] must contain the scalar factor of the elementary
        reflector H_i, as returned by ``zgerqf``.

    c : complex, array-like, shape (m, n)
        The m*n matrix C.

    Returns
    -------
    a : complex, ndarray, shape (k, :)
        Is modified by ``zunmrq`` but restored on exit.

    c : complex, ndarray, shape (m, n)
        `c` is overwritten by QC or Q^HC or CQ or CQ^H as specified by
        `side` and `trans`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'C'.

        (`errno` -3)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -5)
            On entry, error in parameter k.

            Constraint: m >= k >= 0.

        (`errno` -5)
            On entry, error in parameter k.

            Constraint: n >= k >= 0.

    Notes
    -----
    ``zunmrq`` is intended to be used following a call to ``zgerqf``,
    which performs an RQ factorization of a complex matrix A and
    represents the unitary matrix Q as a product of elementary
    reflectors.

    This function may be used to form one of the matrix products

        QC, Q^HC, CQ, CQ^H,

    overwriting the result on C, which may be any complex rectangular
    m*n matrix.

    A common application of this function is in solving underdetermined
    linear least squares problems, as described in [the F08
    Introduction].

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zunmrq'

    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=True,
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _tau = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'tau'},
        tau,
        copy=False,
    )
    _k_py = _b_utils._EngineComplex128ArrayType.get_shape(_tau, {'fun_name': fun_name, 'entity': 'tau'}, exp_rank=1)[0]
    _k = _b_utils._EngineIntScalarType(
        _k_py, {'fun_name': fun_name, 'entity': '_k'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _b_lapackeig.zunmrq(
        side,
        trans,
        _m,
        _n,
        _k,
        _a,
        _tau,
        _c,
    )
    _a_py = _a
    _c_py = _c
    _return_data = ZunmrqReturnData(
        _a_py,
        _c_py,
    )
    return _return_data

DsyevReturnData = _collections.namedtuple(
    'DsyevReturnData',
    ['a', 'w'],
)

def dsyev(jobz, uplo, a):
    u"""
    Computes all eigenvalues and, optionally, eigenvectors of a real
    symmetric matrix.

    ``dsyev`` computes all the eigenvalues and, optionally, all the
    eigenvectors of a real n*n symmetric matrix A.

    For full information please refer to the NAG Library document for
    f08fa

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08faf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

            Only eigenvalues are computed.

        `jobz` = 'V'

            Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangular part of A is stored.

        If `uplo` = 'L', the lower triangular part of A is stored.

    a : float, array-like, shape (n, n)
        The n*n symmetric matrix A.

    Returns
    -------
    a : None or float, ndarray, shape (n, n)
        If `jobz` = 'V', `a` contains the orthonormal eigenvectors of
        the matrix A.

    w : float, ndarray, shape (n)
        The eigenvalues in ascending order.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` i > 0)
            The algorithm failed to converge; *<value>* off-diagonal
            elements of an intermediate tridiagonal form did not
            converge to zero.

    Notes
    -----
    The symmetric matrix A is first reduced to tridiagonal form, using
    orthogonal similarity transformations, and then the QR algorithm is
    applied to the tridiagonal matrix to compute the eigenvalues and
    (optionally) the eigenvectors.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dsyev'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _b_lapackeig.dsyev(
        jobz,
        uplo,
        _n,
        _a,
        _w,
    )
    _jobz = _b_utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = _b_utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py in ('V', 'v'):
        _a_py = _a
    else:
        _a_py = None
    _w_py = _w
    _return_data = DsyevReturnData(
        _a_py,
        _w_py,
    )
    return _return_data

DsyevxReturnData = _collections.namedtuple(
    'DsyevxReturnData',
    ['a', 'm', 'w', 'z', 'jfail'],
)

def dsyevx(jobz, erange, uplo, a, vl, vu, il, iu, abstol):
    u"""
    Computes selected eigenvalues and, optionally, eigenvectors of a
    real symmetric matrix.

    ``dsyevx`` computes selected eigenvalues and, optionally,
    eigenvectors of a real n*n symmetric matrix A.
    Eigenvalues and eigenvectors can be selected by specifying either a
    range of values or a range of indices for the desired eigenvalues.

    For full information please refer to the NAG Library document for
    f08fb

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08fbf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

            Only eigenvalues are computed.

        `jobz` = 'V'

            Eigenvalues and eigenvectors are computed.

    erange : str, length 1
        If `erange` = 'A', all eigenvalues will be found.

        If `erange` = 'V', all eigenvalues in the half-open interval
        (`vl`,`vu`] will be found.

        If `erange` = 'I', the `il`th to `iu`th eigenvalues will be
        found.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangular part of A is stored.

        If `uplo` = 'L', the lower triangular part of A is stored.

    a : float, array-like, shape (n, n)
        The n*n symmetric matrix A.

    vl : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    vu : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    il : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    iu : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    abstol : float
        The absolute error tolerance for the eigenvalues. An approximate
        eigenvalue is accepted as converged when it is determined to lie
        in an interval [a,b] of width less than or equal to

            `abstol`+epsilon max(|a|, |b|),

        where epsilon is the machine precision. If `abstol` is less than
        or equal to zero, then epsilon||T||_1 will be used in its place,
        where T is the tridiagonal matrix obtained by reducing A to
        tridiagonal form. Eigenvalues will be computed most accurately
        when `abstol` is set to twice the underflow threshold
        2*machine.real_safe(), not zero. If this function returns with
        `errno` > 0, indicating that some eigenvectors did not converge,
        try setting `abstol` to 2*machine.real_safe(). See Demmel and
        Kahan (1990).

    Returns
    -------
    a : float, ndarray, shape (n, n)
        The lower triangle (if `uplo` = 'L') or the upper triangle (if
        `uplo` = 'U') of `a`, including the diagonal, is overwritten.

    m : int
        The total number of eigenvalues found. 0 <= `m` <= n.

        If `erange` = 'A', `m` = n.

        If `erange` = 'I', `m` = `iu`-`il`+1.

    w : float, ndarray, shape (n)
        The first `m` elements contain the selected eigenvalues in
        ascending order.

    z : float, ndarray, shape (:, :)
        If `jobz` = 'V', then

            if no exception or warning is raised, the first `m` columns
            of Z contain the orthonormal eigenvectors of the matrix A
            corresponding to the selected eigenvalues, with the ith
            column of Z holding the eigenvector associated with
            `w`[i-1];

            if an eigenvector fails to converge (`errno` > 0), then that
            column of Z contains the latest approximation to the
            eigenvector, and the index of the eigenvector is returned in
            `jfail`.

        If `jobz` = 'N', `z` is not referenced.

        **Note:** you must ensure that at least max(1, `m`) columns are
        supplied in the array `z`; if `erange` = 'V', the exact value of
        `m` is not known in advance and an upper bound of at least n
        must be used.

    jfail : int, ndarray, shape (n)
        If `jobz` = 'V', then

            if no exception or warning is raised, the first `m` elements
            of `jfail` are zero;

            if `errno` > 0, `jfail` contains the indices of the
            eigenvectors that failed to converge.

        If `jobz` = 'N', `jfail` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `erange`.

            Constraint: `erange` = 'A', 'V' or 'I'.

        (`errno` -3)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -8)
            On entry, error in parameter `vu`.

            Constraint: `vl` < `vu`.

        (`errno` -9)
            On entry, error in parameter `il`.

        (`errno` -10)
            On entry, error in parameter `iu`.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            The algorithm failed to converge; *<value>* eigenvectors did
            not converge. Their indices are stored in array `jfail`.

    Notes
    -----
    The symmetric matrix A is first reduced to tridiagonal form, using
    orthogonal similarity transformations.
    The required eigenvalues and eigenvectors are then computed from the
    tridiagonal matrix; the method used depends upon whether all, or
    selected, eigenvalues and eigenvectors are required.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Demmel, J W and Kahan, W, 1990, `Accurate singular values of
    bidiagonal matrices`, SIAM J. Sci. Statist. Comput. (11), 873--912

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dsyevx'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'),),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobz = _b_utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = _b_utils._EngineCharScalarType.to_py(_jobz)
    _erange = _b_utils._EngineCharScalarType(
        erange, {'fun_name': fun_name, 'entity': 'erange'}, exp_chrlen=1,
    ).to_ctypes()
    _erange_py = _b_utils._EngineCharScalarType.to_py(_erange)
    _iu = _b_utils._EngineIntScalarType(
        iu, {'fun_name': fun_name, 'entity': 'iu'},
    ).to_ctypes()
    _iu_py = _b_utils._EngineIntScalarType.to_py(_iu)
    _il = _b_utils._EngineIntScalarType(
        il, {'fun_name': fun_name, 'entity': 'il'},
    ).to_ctypes()
    _il_py = _b_utils._EngineIntScalarType.to_py(_il)
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        if _erange_py in ('I', 'i'):
            _z_dim2 = _iu_py-_il_py+1
        else:
            _z_dim2 = _n_py
    else:
        _z_dim2 = 0
    exp_shape_z = (_z_dim1, _z_dim2)
    _z = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_z,
        sorder=_m_sorder,
    )
    _jfail = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        _return_scalar = _b_lapackeig.dsyevx(
            jobz,
            erange,
            uplo,
            _n,
            _a,
            vl,
            vu,
            _il,
            _iu,
            abstol,
            _w,
            _z,
            _jfail,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _a_py = _a
    _m_py = _return_scalar
    _w_py = _w
    if _jobz_py in ('V', 'v'):
        _z_dim2 = _m_py
    else:
        _z_dim2 = 0
    _z_py = _z[:, :_z_dim2]
    _jfail_py = _jfail
    _return_data = DsyevxReturnData(
        _a_py,
        _m_py,
        _w_py,
        _z_py,
        _jfail_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

DsyevdReturnData = _collections.namedtuple(
    'DsyevdReturnData',
    ['a', 'w'],
)

def dsyevd(job, uplo, a):
    u"""
    Computes all eigenvalues and, optionally, all eigenvectors of real
    symmetric matrix (divide-and-conquer).

    ``dsyevd`` computes all the eigenvalues and, optionally, all the
    eigenvectors of a real symmetric matrix. If the eigenvectors are
    requested, then it uses a divide-and-conquer algorithm to compute
    eigenvalues and eigenvectors.
    However, if only eigenvalues are required, then it uses the
    Pal--Walker--Kahan variant of the QL or QR algorithm.

    For full information please refer to the NAG Library document for
    f08fc

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08fcf.html

    Parameters
    ----------
    job : str, length 1
        Indicates whether eigenvectors are computed.

        `job` = 'N'

            Only eigenvalues are computed.

        `job` = 'V'

            Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        Indicates whether the upper or lower triangular part of A is
        stored.

        `uplo` = 'U'

            The upper triangular part of A is stored.

        `uplo` = 'L'

            The lower triangular part of A is stored.

    a : float, array-like, shape (n, n)
        The n*n symmetric matrix A.

    Returns
    -------
    a : float, ndarray, shape (n, n)
        If `job` = 'V', `a` is overwritten by the orthogonal matrix Z
        which contains the eigenvectors of A.

    w : float, ndarray, shape (n)
        The eigenvalues of the matrix A in ascending order.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` i > 0)
            If errno = *<value>* and `job` = 'N', the algorithm failed
            to converge; *<value>* elements of an intermediate
            tridiagonal form did not converge to zero; if errno =
            *<value>* and `job` = 'V', then the algorithm failed to
            compute an eigenvalue while working on the submatrix lying
            in rows and column *<value>*/(n+1) through mod(*<value>*,
            n+1).

    Notes
    -----
    ``dsyevd`` computes all the eigenvalues and, optionally, all the
    eigenvectors of a real symmetric matrix A. In other words, it can
    compute the spectral factorization of A as

        A = Z Lambda Z^T,

    where Lambda is a diagonal matrix whose diagonal elements are the
    eigenvalues lambda_i, and Z is the orthogonal matrix whose columns
    are the eigenvectors z_i.
    Thus

        Az_i = lambda_i z_i, i = 1,2,...,n.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dsyevd'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _b_lapackeig.dsyevd(
        job,
        uplo,
        _n,
        _a,
        _w,
    )
    _a_py = _a
    _w_py = _w
    _return_data = DsyevdReturnData(
        _a_py,
        _w_py,
    )
    return _return_data

DsyevrReturnData = _collections.namedtuple(
    'DsyevrReturnData',
    ['a', 'm', 'w', 'z', 'isuppz'],
)

def dsyevr(jobz, erange, uplo, a, vl, vu, il, iu, abstol):
    u"""
    Computes selected eigenvalues and, optionally, eigenvectors of a
    real symmetric matrix (Relatively Robust Representations).

    ``dsyevr`` computes selected eigenvalues and, optionally,
    eigenvectors of a real n*n symmetric matrix A.
    Eigenvalues and eigenvectors can be selected by specifying either a
    range of values or a range of indices for the desired eigenvalues.

    For full information please refer to the NAG Library document for
    f08fd

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08fdf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

            Only eigenvalues are computed.

        `jobz` = 'V'

            Eigenvalues and eigenvectors are computed.

    erange : str, length 1
        If `erange` = 'A', all eigenvalues will be found.

        If `erange` = 'V', all eigenvalues in the half-open interval
        (`vl`,`vu`] will be found.

        If `erange` = 'I', the `il`th to `iu`th eigenvalues will be
        found.

        For `erange` = 'V' or 'I' and `iu`-`il` < n-1, ``dstebz`` and
        ``dstein`` are called.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangular part of A is stored.

        If `uplo` = 'L', the lower triangular part of A is stored.

    a : float, array-like, shape (n, n)
        The n*n symmetric matrix A.

    vl : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    vu : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    il : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    iu : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    abstol : float
        The absolute error tolerance for the eigenvalues. An approximate
        eigenvalue is accepted as converged when it is determined to lie
        in an interval [a,b] of width less than or equal to

            `abstol`+epsilon max(|a|, |b|),

        where epsilon is the machine precision. If `abstol` is less than
        or equal to zero, then epsilon||T||_1 will be used in its place,
        where T is the tridiagonal matrix obtained by reducing A to
        tridiagonal form. See Demmel and Kahan (1990).

        If high relative accuracy is important, set `abstol` to
        machine.real_safe(), although doing so does not currently
        guarantee that eigenvalues are computed to high relative
        accuracy.

        See Barlow and Demmel (1990) for a discussion of which matrices
        can define their eigenvalues to high relative accuracy.

    Returns
    -------
    a : float, ndarray, shape (n, n)
        The lower triangle (if `uplo` = 'L') or the upper triangle (if
        `uplo` = 'U') of `a`, including the diagonal, is overwritten.

    m : int
        The total number of eigenvalues found. 0 <= `m` <= n.

        If `erange` = 'A', `m` = n.

        If `erange` = 'I', `m` = `iu`-`il`+1.

    w : float, ndarray, shape (n)
        The first `m` elements contain the selected eigenvalues in
        ascending order.

    z : float, ndarray, shape (:, :)
        If `jobz` = 'V', the first `m` columns of Z contain the
        orthonormal eigenvectors of the matrix A corresponding to the
        selected eigenvalues, with the ith column of Z holding the
        eigenvector associated with `w`[i-1].

        If `jobz` = 'N', `z` is not referenced.

        **Note:** you must ensure that at least max(1, `m`) columns are
        supplied in the array `z`; if `erange` = 'V', the exact value of
        `m` is not known in advance and an upper bound of at least n
        must be used.

    isuppz : int, ndarray, shape (:)
        The support of the eigenvectors in `z`, i.e., the indices
        indicating the nonzero elements in `z`. The ith eigenvector is
        nonzero only in elements `isuppz`[2*i-2] through
        `isuppz`[2*i-1]. Implemented only for `erange` = 'A' or `erange`
        = 'I' and `iu`-`il` = n-1.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `erange`.

            Constraint: `erange` = 'A', 'V' or 'I'.

        (`errno` -3)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -8)
            On entry, error in parameter `vu`.

            Constraint: `vl` < `vu`.

        (`errno` -9)
            On entry, error in parameter `il`.

        (`errno` -10)
            On entry, error in parameter `iu`.

        (`errno` i > 0)
            ``dsyevr`` failed to converge.

    Notes
    -----
    The symmetric matrix is first reduced to a tridiagonal matrix T,
    using orthogonal similarity transformations.
    Then whenever possible, ``dsyevr`` computes the eigenspectrum using
    Relatively Robust Representations. ``dsyevr`` computes eigenvalues
    by the dqds algorithm, while orthogonal eigenvectors are computed
    from various 'good' LDL^T representations (also known as Relatively
    Robust Representations).
    Gram--Schmidt orthogonalization is avoided as far as possible.
    More specifically, the various steps of the algorithm are as
    follows.
    For the ith unreduced block of T:

    (a) compute T-sigma_i I = L_i D_i L_i^T, such that L_i D_i L_i^T is
        a relatively robust representation,

    (#) compute the eigenvalues, lambda_j, of L_i D_i L_i^T to high
        relative accuracy by the dqds algorithm,

    (#) if there is a cluster of close eigenvalues, 'choose' sigma_i
        close to the cluster, and go to (a),

    (#) given the approximate eigenvalue lambda_j of L_i D_i L_i^T,
        compute the corresponding eigenvector by forming a
        rank-revealing twisted factorization.

    The desired accuracy of the output can be specified by the argument
    `abstol`.
    For more details, see Dhillon (1997) and Parlett and Dhillon (2000).

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Barlow, J and Demmel, J W, 1990, `Computing accurate eigensystems of
    scaled diagonally dominant matrices`, SIAM J. Numer. Anal. (27),
    762--791

    Demmel, J W and Kahan, W, 1990, `Accurate singular values of
    bidiagonal matrices`, SIAM J. Sci. Statist. Comput. (11), 873--912

    Dhillon, I, 1997, `A new O(n^2) algorithm for the symmetric
    tridiagonal eigenvalue/eigenvector problem`, Computer Science
    Division Technical Report No. UCB//CSD-97-971, UC Berkeley

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore

    Parlett, B N and Dhillon, I S, 2000, `Relatively robust
    representations of symmetric tridiagonals`, Linear Algebra Appl.
    (309), 121--151
    """

    fun_name = 'naginterfaces.library.lapackeig.dsyevr'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'),),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobz = _b_utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = _b_utils._EngineCharScalarType.to_py(_jobz)
    _erange = _b_utils._EngineCharScalarType(
        erange, {'fun_name': fun_name, 'entity': 'erange'}, exp_chrlen=1,
    ).to_ctypes()
    _erange_py = _b_utils._EngineCharScalarType.to_py(_erange)
    _iu = _b_utils._EngineIntScalarType(
        iu, {'fun_name': fun_name, 'entity': 'iu'},
    ).to_ctypes()
    _iu_py = _b_utils._EngineIntScalarType.to_py(_iu)
    _il = _b_utils._EngineIntScalarType(
        il, {'fun_name': fun_name, 'entity': 'il'},
    ).to_ctypes()
    _il_py = _b_utils._EngineIntScalarType.to_py(_il)
    if _jobz_py in ('V', 'v'):
        if _erange_py in ('I', 'i'):
            _vl_est_m_py = _iu_py-_il_py+1
        else:
            _vl_est_m_py = _n_py
    else:
        _vl_est_m_py = 0
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    exp_shape_z = (_z_dim1, _vl_est_m_py)
    _z = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_z,
        sorder=_m_sorder,
    )
    if _jobz_py in ('V', 'v'):
        _isuppz_dim1 = 2*max(1, _vl_est_m_py)
    else:
        _isuppz_dim1 = 0
    exp_shape_isuppz = (_isuppz_dim1,)
    _isuppz = _b_utils._EngineIntArrayType.empty_ndarray(
        exp_shape_isuppz,
    )
    _return_scalar = _b_lapackeig.dsyevr(
        jobz,
        erange,
        uplo,
        _n,
        _a,
        vl,
        vu,
        _il,
        _iu,
        abstol,
        _w,
        _z,
        _isuppz,
    )
    _a_py = _a
    _m_py = _return_scalar
    _w_py = _w
    if _jobz_py in ('V', 'v'):
        _z_dim2 = _m_py
    else:
        _z_dim2 = 0
    _z_py = _z[:, :_z_dim2]
    if _jobz_py in ('V', 'v'):
        _isuppz_dim1 = 2*_m_py
    else:
        _isuppz_dim1 = 0
    _isuppz_py = _isuppz[:_isuppz_dim1]
    _return_data = DsyevrReturnData(
        _a_py,
        _m_py,
        _w_py,
        _z_py,
        _isuppz_py,
    )
    return _return_data

DsytrdReturnData = _collections.namedtuple(
    'DsytrdReturnData',
    ['a', 'd', 'e', 'tau'],
)

def dsytrd(uplo, a):
    u"""
    Orthogonal reduction of real symmetric matrix to symmetric
    tridiagonal form.

    ``dsytrd`` reduces a real symmetric matrix to tridiagonal form.

    For full information please refer to the NAG Library document for
    f08fe

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08fef.html

    Parameters
    ----------
    uplo : str, length 1
        Indicates whether the upper or lower triangular part of A is
        stored.

        `uplo` = 'U'

            The upper triangular part of A is stored.

        `uplo` = 'L'

            The lower triangular part of A is stored.

    a : float, array-like, shape (n, n)
        The n*n symmetric matrix A.

    Returns
    -------
    a : float, ndarray, shape (n, n)
        `a` is overwritten by the tridiagonal matrix T and details of
        the orthogonal matrix Q as specified by `uplo`.

    d : float, ndarray, shape (n)
        The diagonal elements of the tridiagonal matrix T.

    e : float, ndarray, shape (max(1, n-1))
        The off-diagonal elements of the tridiagonal matrix T.

    tau : float, ndarray, shape (max(1, n-1))
        Further details of the orthogonal matrix Q.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= 0.

    Notes
    -----
    ``dsytrd`` reduces a real symmetric matrix A to symmetric
    tridiagonal form T by an orthogonal similarity transformation: A =
    QTQ^T.

    The matrix Q is not formed explicitly but is represented as a
    product of n-1 elementary reflectors (see [the F08 Introduction] for
    details). Functions are provided to work with Q in this
    representation (see [Further Comments]).

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dsytrd'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _d = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _e = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (max(1, _n_py-1),),
    )
    _tau = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (max(1, _n_py-1),),
    )
    _b_lapackeig.dsytrd(
        uplo,
        _n,
        _a,
        _d,
        _e,
        _tau,
    )
    _a_py = _a
    _d_py = _d
    _e_py = _e
    _tau_py = _tau
    _return_data = DsytrdReturnData(
        _a_py,
        _d_py,
        _e_py,
        _tau_py,
    )
    return _return_data

def dorgtr(uplo, a, tau):
    u"""
    Generate orthogonal transformation matrix from reduction to
    tridiagonal form determined by ``dsytrd``.

    ``dorgtr`` generates the real orthogonal matrix Q, which was
    determined by ``dsytrd`` when reducing a symmetric matrix to
    tridiagonal form.

    For full information please refer to the NAG Library document for
    f08ff

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08fff.html

    Parameters
    ----------
    uplo : str, length 1
        This **must** be the same argument `uplo` as supplied to
        ``dsytrd``.

    a : float, array-like, shape (n, n)
        Details of the vectors which define the elementary reflectors,
        as returned by ``dsytrd``.

    tau : float, array-like, shape (max(1, n-1))
        Further details of the elementary reflectors, as returned by
        ``dsytrd``.

    Returns
    -------
    a : float, ndarray, shape (n, n)
        The n*n orthogonal matrix Q.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= 0.

    Notes
    -----
    ``dorgtr`` is intended to be used after a call to ``dsytrd``, which
    reduces a real symmetric matrix A to symmetric tridiagonal form T by
    an orthogonal similarity transformation: A = QTQ^T. ``dsytrd``
    represents the orthogonal matrix Q as a product of n-1 elementary
    reflectors.

    This function may be used to generate Q explicitly as a square
    matrix.

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dorgtr'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _tau = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'tau'},
        tau,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b_lapackeig.dorgtr(
        uplo,
        _n,
        _a,
        _tau,
    )
    _a_py = _a
    return _a_py

def dormtr(side, uplo, trans, a, tau, c):
    u"""
    Applies the orthogonal transformation determined by ``dsytrd``.

    ``dormtr`` multiplies an arbitrary real matrix C by the real
    orthogonal matrix Q which was determined by ``dsytrd`` when reducing
    a real symmetric matrix to tridiagonal form.

    For full information please refer to the NAG Library document for
    f08fg

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08fgf.html

    Parameters
    ----------
    side : str, length 1
        Indicates how Q or Q^T is to be applied to C.

        `side` = 'L'

            Q or Q^T is applied to C from the left.

        `side` = 'R'

            Q or Q^T is applied to C from the right.

    uplo : str, length 1
        This **must** be the same argument `uplo` as supplied to
        ``dsytrd``.

    trans : str, length 1
        Indicates whether Q or Q^T is to be applied to C.

        `trans` = 'N'

            Q is applied to C.

        `trans` = 'T'

            Q^T is applied to C.

    a : float, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `side`='L': m; if `side`='R': n;
        otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `side`='L': m; if `side`='R': n;
        otherwise: 0.

        Details of the vectors which define the elementary reflectors,
        as returned by ``dsytrd``.

    tau : float, array-like, shape (:)
        Note: the required length for this argument is determined as
        follows: if `side`='L': m-1; if `side`='R': n-1; otherwise: 0.

        Further details of the elementary reflectors, as returned by
        ``dsytrd``.

    c : float, array-like, shape (m, n)
        The m*n matrix C.

    Returns
    -------
    c : float, ndarray, shape (m, n)
        `c` is overwritten by QC or Q^TC or CQ or CQ^T as specified by
        `side` and `trans`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'T'.

        (`errno` -4)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -5)
            On entry, error in parameter n.

            Constraint: n >= 0.

    Notes
    -----
    ``dormtr`` is intended to be used after a call to ``dsytrd``, which
    reduces a real symmetric matrix A to symmetric tridiagonal form T by
    an orthogonal similarity transformation: A = QTQ^T. ``dsytrd``
    represents the orthogonal matrix Q as a product of elementary
    reflectors.

    This function may be used to form one of the matrix products

        QC,Q^TC,CQ or CQ^T,

    overwriting the result on C (which may be any real rectangular
    matrix).

    A common application of this function is to transform a matrix Z of
    eigenvectors of T to the matrix QZ of eigenvectors of A.

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dormtr'

    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _tau = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'tau'},
        tau,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b_lapackeig.dormtr(
        side,
        uplo,
        trans,
        _m,
        _n,
        _a,
        _tau,
        _c,
    )
    _c_py = _c
    return _c_py

def ddisna(job, m, n, d):
    u"""
    Computes the reciprocal condition numbers for the eigenvectors of a
    real symmetric or complex Hermitian matrix or for the left or right
    singular vectors of a general matrix.

    ``ddisna`` computes the reciprocal condition numbers for the
    eigenvectors of a real symmetric or complex Hermitian m*m matrix A,
    or for the left or right singular vectors of a general m*n matrix A.

    For full information please refer to the NAG Library document for
    f08fl

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08flf.html

    Parameters
    ----------
    job : str, length 1
        Specifies for which problem the reciprocal condition number
        should be computed.

        `job` = 'E'

            The eigenvectors of a symmetric or Hermitian matrix.

        `job` = 'L'

            The left singular vectors of a general matrix.

        `job` = 'R'

            The right singular vectors of a general matrix.

    m : int
        m, the number of rows of the matrix A.

    n : int
        n, the number of columns of the matrix when `job` = 'L' or 'R'.

        If `job` = 'E', `n` is not referenced.

    d : float, array-like, shape (:)
        Note: the required length for this argument is determined as
        follows: if `job`='E': `m`; if `job` in ('L', 'R'): min(`m`,
        `n`); otherwise: 0.

        The eigenvalues if `job` = 'E', or singular values if `job` =
        'L' or 'R' of the matrix A.

    Returns
    -------
    sep : float, ndarray, shape (:)
        The reciprocal condition numbers of the vectors.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'E', 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `d`.

            Constraint: the elements of the array must be in increasing
            or decreasing order.

        (`errno` -4)
            On entry, error in parameter `d`.

            Constraint: the elements of the array must not be negative.

    Notes
    -----
    The bound on the error, measured by the angle in radians, for the
    ith computed vector is given by epsilon||A||_2/sep_i, where epsilon
    is the machine precision and sep_i is the reciprocal condition
    number for the vectors, returned in the array element `sep`[i-1].
    `sep`[i-1] is restricted to be at least epsilon||A||_2 in order to
    limit the size of the error bound.

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.ddisna'

    _d = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'd'},
        d,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _job = _b_utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _job_py = _b_utils._EngineCharScalarType.to_py(_job)
    _m = _b_utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = _b_utils._EngineIntScalarType.to_py(_m)
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    if _job_py in ('E', 'e'):
        _sep_dim1 = _m_py
    elif _job_py in ('L', 'l', 'R', 'r'):
        _sep_dim1 = min(_m_py, _n_py)
    else:
        _sep_dim1 = 0
    exp_shape_sep = (_sep_dim1,)
    _sep = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_sep,
    )
    _b_lapackeig.ddisna(
        job,
        _m,
        _n,
        _d,
        _sep,
    )
    _sep_py = _sep
    return _sep_py

ZheevReturnData = _collections.namedtuple(
    'ZheevReturnData',
    ['a', 'w'],
)

def zheev(jobz, uplo, a):
    u"""
    Computes all eigenvalues and, optionally, eigenvectors of a complex
    Hermitian matrix.

    ``zheev`` computes all the eigenvalues and, optionally, all the
    eigenvectors of a complex n*n Hermitian matrix A.

    For full information please refer to the NAG Library document for
    f08fn

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08fnf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

            Only eigenvalues are computed.

        `jobz` = 'V'

            Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangular part of A is stored.

        If `uplo` = 'L', the lower triangular part of A is stored.

    a : complex, array-like, shape (n, n)
        The n*n Hermitian matrix A.

    Returns
    -------
    a : None or complex, ndarray, shape (n, n)
        If `jobz` = 'V', `a` contains the orthonormal eigenvectors of
        the matrix A.

    w : float, ndarray, shape (n)
        The eigenvalues in ascending order.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            The algorithm failed to converge; *<value>* off-diagonal
            elements of an intermediate tridiagonal form did not
            converge to zero.

    Notes
    -----
    The Hermitian matrix A is first reduced to real tridiagonal form,
    using unitary similarity transformations, and then the QR algorithm
    is applied to the tridiagonal matrix to compute the eigenvalues and
    (optionally) the eigenvectors.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zheev'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        _b_lapackeig.zheev(
            jobz,
            uplo,
            _n,
            _a,
            _w,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _jobz = _b_utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = _b_utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py in ('V', 'v'):
        _a_py = _a
    else:
        _a_py = None
    _w_py = _w
    _return_data = ZheevReturnData(
        _a_py,
        _w_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

ZheevxReturnData = _collections.namedtuple(
    'ZheevxReturnData',
    ['a', 'm', 'w', 'z', 'jfail'],
)

def zheevx(jobz, erange, uplo, a, vl, vu, il, iu, abstol):
    u"""
    Computes selected eigenvalues and, optionally, eigenvectors of a
    complex Hermitian matrix.

    ``zheevx`` computes selected eigenvalues and, optionally,
    eigenvectors of a complex n*n Hermitian matrix A.
    Eigenvalues and eigenvectors can be selected by specifying either a
    range of values or a range of indices for the desired eigenvalues.

    For full information please refer to the NAG Library document for
    f08fp

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08fpf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

            Only eigenvalues are computed.

        `jobz` = 'V'

            Eigenvalues and eigenvectors are computed.

    erange : str, length 1
        If `erange` = 'A', all eigenvalues will be found.

        If `erange` = 'V', all eigenvalues in the half-open interval
        (`vl`,`vu`] will be found.

        If `erange` = 'I', the `il`th to `iu`th eigenvalues will be
        found.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangular part of A is stored.

        If `uplo` = 'L', the lower triangular part of A is stored.

    a : complex, array-like, shape (n, n)
        The n*n Hermitian matrix A.

    vl : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    vu : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    il : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    iu : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    abstol : float
        The absolute error tolerance for the eigenvalues. An approximate
        eigenvalue is accepted as converged when it is determined to lie
        in an interval [a,b] of width less than or equal to

            `abstol`+epsilon max(|a|, |b|),

        where epsilon is the machine precision. If `abstol` is less than
        or equal to zero, then epsilon||T||_1 will be used in its place,
        where T is the tridiagonal matrix obtained by reducing A to
        tridiagonal form. Eigenvalues will be computed most accurately
        when `abstol` is set to twice the underflow threshold
        2*machine.real_safe(), not zero. If this function returns with
        `errno` > 0, indicating that some eigenvectors did not converge,
        try setting `abstol` to 2*machine.real_safe(). See Demmel and
        Kahan (1990).

    Returns
    -------
    a : complex, ndarray, shape (n, n)
        The lower triangle (if `uplo` = 'L') or the upper triangle (if
        `uplo` = 'U') of `a`, including the diagonal, is overwritten.

    m : int
        The total number of eigenvalues found. 0 <= `m` <= n.

        If `erange` = 'A', `m` = n.

        If `erange` = 'I', `m` = `iu`-`il`+1.

    w : float, ndarray, shape (n)
        The first `m` elements contain the selected eigenvalues in
        ascending order.

    z : complex, ndarray, shape (:, :)
        If `jobz` = 'V', then

            if no exception or warning is raised, the first `m` columns
            of Z contain the orthonormal eigenvectors of the matrix A
            corresponding to the selected eigenvalues, with the ith
            column of Z holding the eigenvector associated with
            `w`[i-1];

            if an eigenvector fails to converge (`errno` > 0), then that
            column of Z contains the latest approximation to the
            eigenvector, and the index of the eigenvector is returned in
            `jfail`.

        If `jobz` = 'N', `z` is not referenced.

        **Note:** you must ensure that at least max(1, `m`) columns are
        supplied in the array `z`; if `erange` = 'V', the exact value of
        `m` is not known in advance and an upper bound of at least n
        must be used.

    jfail : int, ndarray, shape (n)
        If `jobz` = 'V', then

            if no exception or warning is raised, the first `m` elements
            of `jfail` are zero;

            if `errno` > 0, `jfail` contains the indices of the
            eigenvectors that failed to converge.

        If `jobz` = 'N', `jfail` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `erange`.

            Constraint: `erange` = 'A', 'V' or 'I'.

        (`errno` -3)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -8)
            On entry, error in parameter `vu`.

            Constraint: `vl` < `vu`.

        (`errno` -9)
            On entry, error in parameter `il`.

        (`errno` -10)
            On entry, error in parameter `iu`.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            The algorithm failed to converge; *<value>* eigenvectors did
            not converge. Their indices are stored in array `jfail`.

    Notes
    -----
    The Hermitian matrix A is first reduced to real tridiagonal form,
    using unitary similarity transformations.
    The required eigenvalues and eigenvectors are then computed from the
    tridiagonal matrix; the method used depends upon whether all, or
    selected, eigenvalues and eigenvectors are required.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Demmel, J W and Kahan, W, 1990, `Accurate singular values of
    bidiagonal matrices`, SIAM J. Sci. Statist. Comput. (11), 873--912

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zheevx'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'),),
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobz = _b_utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = _b_utils._EngineCharScalarType.to_py(_jobz)
    _erange = _b_utils._EngineCharScalarType(
        erange, {'fun_name': fun_name, 'entity': 'erange'}, exp_chrlen=1,
    ).to_ctypes()
    _erange_py = _b_utils._EngineCharScalarType.to_py(_erange)
    _iu = _b_utils._EngineIntScalarType(
        iu, {'fun_name': fun_name, 'entity': 'iu'},
    ).to_ctypes()
    _iu_py = _b_utils._EngineIntScalarType.to_py(_iu)
    _il = _b_utils._EngineIntScalarType(
        il, {'fun_name': fun_name, 'entity': 'il'},
    ).to_ctypes()
    _il_py = _b_utils._EngineIntScalarType.to_py(_il)
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        if _erange_py in ('I', 'i'):
            _z_dim2 = _iu_py-_il_py+1
        else:
            _z_dim2 = _n_py
    else:
        _z_dim2 = 0
    exp_shape_z = (_z_dim1, _z_dim2)
    _z = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_z,
        sorder=_m_sorder,
    )
    _jfail = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        _return_scalar = _b_lapackeig.zheevx(
            jobz,
            erange,
            uplo,
            _n,
            _a,
            vl,
            vu,
            _il,
            _iu,
            abstol,
            _w,
            _z,
            _jfail,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _a_py = _a
    _m_py = _return_scalar
    _w_py = _w
    if _jobz_py in ('V', 'v'):
        _z_dim2 = _m_py
    else:
        _z_dim2 = 0
    _z_py = _z[:, :_z_dim2]
    _jfail_py = _jfail
    _return_data = ZheevxReturnData(
        _a_py,
        _m_py,
        _w_py,
        _z_py,
        _jfail_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

ZheevdReturnData = _collections.namedtuple(
    'ZheevdReturnData',
    ['a', 'w'],
)

def zheevd(job, uplo, a):
    u"""
    Computes all eigenvalues and, optionally, all eigenvectors of
    complex Hermitian matrix (divide-and-conquer).

    ``zheevd`` computes all the eigenvalues and, optionally, all the
    eigenvectors of a complex Hermitian matrix. If the eigenvectors are
    requested, then it uses a divide-and-conquer algorithm to compute
    eigenvalues and eigenvectors.
    However, if only eigenvalues are required, then it uses the
    Pal--Walker--Kahan variant of the QL or QR algorithm.

    For full information please refer to the NAG Library document for
    f08fq

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08fqf.html

    Parameters
    ----------
    job : str, length 1
        Indicates whether eigenvectors are computed.

        `job` = 'N'

            Only eigenvalues are computed.

        `job` = 'V'

            Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        Indicates whether the upper or lower triangular part of A is
        stored.

        `uplo` = 'U'

            The upper triangular part of A is stored.

        `uplo` = 'L'

            The lower triangular part of A is stored.

    a : complex, array-like, shape (n, n)
        The n*n Hermitian matrix A.

    Returns
    -------
    a : complex, ndarray, shape (n, n)
        If `job` = 'V', `a` is overwritten by the unitary matrix Z which
        contains the eigenvectors of A.

    w : float, ndarray, shape (n)
        The eigenvalues of the matrix A in ascending order.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` i > 0)
            If errno = *<value>* and `job` = 'N', the algorithm failed
            to converge; *<value>* elements of an intermediate
            tridiagonal form did not converge to zero; if errno =
            *<value>* and `job` = 'V', then the algorithm failed to
            compute an eigenvalue while working on the submatrix lying
            in rows and column *<value>*/(n+1) through mod(*<value>*,
            n+1).

    Notes
    -----
    ``zheevd`` computes all the eigenvalues and, optionally, all the
    eigenvectors of a complex Hermitian matrix A. In other words, it can
    compute the spectral factorization of A as

        A = Z Lambda Z^H,

    where Lambda is a real diagonal matrix whose diagonal elements are
    the eigenvalues lambda_i, and Z is the (complex) unitary matrix
    whose columns are the eigenvectors z_i.
    Thus

        Az_i = lambda_i z_i, i = 1,2,...,n.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zheevd'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _b_lapackeig.zheevd(
        job,
        uplo,
        _n,
        _a,
        _w,
    )
    _a_py = _a
    _w_py = _w
    _return_data = ZheevdReturnData(
        _a_py,
        _w_py,
    )
    return _return_data

ZheevrReturnData = _collections.namedtuple(
    'ZheevrReturnData',
    ['a', 'm', 'w', 'z', 'isuppz'],
)

def zheevr(jobz, erange, uplo, a, vl, vu, il, iu, abstol):
    u"""
    Computes selected eigenvalues and, optionally, eigenvectors of a
    complex Hermitian matrix (Relatively Robust Representations).

    ``zheevr`` computes selected eigenvalues and, optionally,
    eigenvectors of a complex n*n Hermitian matrix A.
    Eigenvalues and eigenvectors can be selected by specifying either a
    range of values or a range of indices for the desired eigenvalues.

    For full information please refer to the NAG Library document for
    f08fr

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08frf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

            Only eigenvalues are computed.

        `jobz` = 'V'

            Eigenvalues and eigenvectors are computed.

    erange : str, length 1
        If `erange` = 'A', all eigenvalues will be found.

        If `erange` = 'V', all eigenvalues in the half-open interval
        (`vl`,`vu`] will be found.

        If `erange` = 'I', the `il`th to `iu`th eigenvalues will be
        found.

        For `erange` = 'V' or 'I' and `iu`-`il` < n-1, ``dstebz`` and
        ``zstein`` are called.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangular part of A is stored.

        If `uplo` = 'L', the lower triangular part of A is stored.

    a : complex, array-like, shape (n, n)
        The n*n Hermitian matrix A.

    vl : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    vu : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    il : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    iu : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    abstol : float
        The absolute error tolerance for the eigenvalues. An approximate
        eigenvalue is accepted as converged when it is determined to lie
        in an interval [a,b] of width less than or equal to

            `abstol`+epsilon max(|a|, |b|),

        where epsilon is the machine precision. If `abstol` is less than
        or equal to zero, then epsilon||T||_1 will be used in its place,
        where T is the real tridiagonal matrix obtained by reducing A to
        tridiagonal form. See Demmel and Kahan (1990).

        If high relative accuracy is important, set `abstol` to
        machine.real_safe(), although doing so does not currently
        guarantee that eigenvalues are computed to high relative
        accuracy.

        See Barlow and Demmel (1990) for a discussion of which matrices
        can define their eigenvalues to high relative accuracy.

    Returns
    -------
    a : complex, ndarray, shape (n, n)
        The lower triangle (if `uplo` = 'L') or the upper triangle (if
        `uplo` = 'U') of `a`, including the diagonal, is overwritten.

    m : int
        The total number of eigenvalues found. 0 <= `m` <= n.

        If `erange` = 'A', `m` = n.

        If `erange` = 'I', `m` = `iu`-`il`+1.

    w : float, ndarray, shape (n)
        The first `m` elements contain the selected eigenvalues in
        ascending order.

    z : complex, ndarray, shape (:, :)
        If `jobz` = 'V', the first `m` columns of Z contain the
        orthonormal eigenvectors of the matrix A corresponding to the
        selected eigenvalues, with the ith column of Z holding the
        eigenvector associated with `w`[i-1].

        If `jobz` = 'N', `z` is not referenced.

        **Note:** you must ensure that at least max(1, `m`) columns are
        supplied in the array `z`; if `erange` = 'V', the exact value of
        `m` is not known in advance and an upper bound of at least n
        must be used.

    isuppz : int, ndarray, shape (:)
        The support of the eigenvectors in `z`, i.e., the indices
        indicating the nonzero elements in `z`. The ith eigenvector is
        nonzero only in elements `isuppz`[2*i-2] through
        `isuppz`[2*i-1]. Implemented only for `erange` = 'A' or `erange`
        = 'I' and `iu`-`il` = n-1.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `erange`.

            Constraint: `erange` = 'A', 'V' or 'I'.

        (`errno` -3)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -8)
            On entry, error in parameter `vu`.

            Constraint: `vl` < `vu`.

        (`errno` -9)
            On entry, error in parameter `il`.

        (`errno` -10)
            On entry, error in parameter `iu`.

        (`errno` i > 0)
            ``zheevr`` failed to converge.

    Notes
    -----
    The Hermitian matrix is first reduced to a real tridiagonal matrix
    T, using unitary similarity transformations.
    Then whenever possible, ``zheevr`` computes the eigenspectrum using
    Relatively Robust Representations. ``zheevr`` computes eigenvalues
    by the dqds algorithm, while orthogonal eigenvectors are computed
    from various 'good' LDL^T representations (also known as Relatively
    Robust Representations).
    Gram--Schmidt orthogonalization is avoided as far as possible.
    More specifically, the various steps of the algorithm are as
    follows.
    For the ith unreduced block of T:

    (a) compute T-sigma_i I = L_i D_i L_i^T, such that L_i D_i L_i^T is
        a relatively robust representation,

    (#) compute the eigenvalues, lambda_j, of L_i D_i L_i^T to high
        relative accuracy by the dqds algorithm,

    (#) if there is a cluster of close eigenvalues, 'choose' sigma_i
        close to the cluster, and go to (a),

    (#) given the approximate eigenvalue lambda_j of L_i D_i L_i^T,
        compute the corresponding eigenvector by forming a
        rank-revealing twisted factorization.

    The desired accuracy of the output can be specified by the argument
    `abstol`.
    For more details, see Dhillon (1997) and Parlett and Dhillon (2000).

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Barlow, J and Demmel, J W, 1990, `Computing accurate eigensystems of
    scaled diagonally dominant matrices`, SIAM J. Numer. Anal. (27),
    762--791

    Demmel, J W and Kahan, W, 1990, `Accurate singular values of
    bidiagonal matrices`, SIAM J. Sci. Statist. Comput. (11), 873--912

    Dhillon, I, 1997, `A new O(n^2) algorithm for the symmetric
    tridiagonal eigenvalue/eigenvector problem`, Computer Science
    Division Technical Report No. UCB//CSD-97-971, UC Berkeley

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore

    Parlett, B N and Dhillon, I S, 2000, `Relatively robust
    representations of symmetric tridiagonals`, Linear Algebra Appl.
    (309), 121--151
    """

    fun_name = 'naginterfaces.library.lapackeig.zheevr'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'),),
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _erange = _b_utils._EngineCharScalarType(
        erange, {'fun_name': fun_name, 'entity': 'erange'}, exp_chrlen=1,
    ).to_ctypes()
    _erange_py = _b_utils._EngineCharScalarType.to_py(_erange)
    _iu = _b_utils._EngineIntScalarType(
        iu, {'fun_name': fun_name, 'entity': 'iu'},
    ).to_ctypes()
    _iu_py = _b_utils._EngineIntScalarType.to_py(_iu)
    _il = _b_utils._EngineIntScalarType(
        il, {'fun_name': fun_name, 'entity': 'il'},
    ).to_ctypes()
    _il_py = _b_utils._EngineIntScalarType.to_py(_il)
    if _erange_py in ('I', 'i'):
        _vl_est_m_py = _iu_py-_il_py+1
    else:
        _vl_est_m_py = _n_py
    _jobz = _b_utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = _b_utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        _z_dim2 = _vl_est_m_py
    else:
        _z_dim2 = 1
    exp_shape_z = (_z_dim1, _z_dim2)
    _z = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_z,
        sorder=_m_sorder,
    )
    _isuppz = _b_utils._EngineIntArrayType.empty_ndarray(
        (2*max(1, _vl_est_m_py),),
    )
    _return_scalar = _b_lapackeig.zheevr(
        jobz,
        erange,
        uplo,
        _n,
        _a,
        vl,
        vu,
        _il,
        _iu,
        abstol,
        _w,
        _z,
        _isuppz,
    )
    _a_py = _a
    _m_py = _return_scalar
    _w_py = _w
    if _jobz_py in ('V', 'v'):
        _z_dim2 = _m_py
    else:
        _z_dim2 = 0
    _z_py = _z[:, :_z_dim2]
    if _jobz_py in ('V', 'v'):
        _isuppz_dim1 = 2*_m_py
    else:
        _isuppz_dim1 = 0
    _isuppz_py = _isuppz[:_isuppz_dim1]
    _return_data = ZheevrReturnData(
        _a_py,
        _m_py,
        _w_py,
        _z_py,
        _isuppz_py,
    )
    return _return_data

ZhetrdReturnData = _collections.namedtuple(
    'ZhetrdReturnData',
    ['a', 'd', 'e', 'tau'],
)

def zhetrd(uplo, a):
    u"""
    Unitary reduction of complex Hermitian matrix to real symmetric
    tridiagonal form.

    ``zhetrd`` reduces a complex Hermitian matrix to tridiagonal form.

    For full information please refer to the NAG Library document for
    f08fs

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08fsf.html

    Parameters
    ----------
    uplo : str, length 1
        Indicates whether the upper or lower triangular part of A is
        stored.

        `uplo` = 'U'

            The upper triangular part of A is stored.

        `uplo` = 'L'

            The lower triangular part of A is stored.

    a : complex, array-like, shape (n, n)
        The n*n Hermitian matrix A.

    Returns
    -------
    a : complex, ndarray, shape (n, n)
        `a` is overwritten by the tridiagonal matrix T and details of
        the unitary matrix Q as specified by `uplo`.

    d : float, ndarray, shape (n)
        The diagonal elements of the tridiagonal matrix T.

    e : float, ndarray, shape (max(1, n-1))
        The off-diagonal elements of the tridiagonal matrix T.

    tau : complex, ndarray, shape (max(1, n-1))
        Further details of the unitary matrix Q.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= 0.

    Notes
    -----
    ``zhetrd`` reduces a complex Hermitian matrix A to real symmetric
    tridiagonal form T by a unitary similarity transformation: A =
    QTQ^H.

    The matrix Q is not formed explicitly but is represented as a
    product of n-1 elementary reflectors (see [the F08 Introduction] for
    details). Functions are provided to work with Q in this
    representation (see [Further Comments]).

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zhetrd'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _d = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _e = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (max(1, _n_py-1),),
    )
    _tau = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (max(1, _n_py-1),),
    )
    _b_lapackeig.zhetrd(
        uplo,
        _n,
        _a,
        _d,
        _e,
        _tau,
    )
    _a_py = _a
    _d_py = _d
    _e_py = _e
    _tau_py = _tau
    _return_data = ZhetrdReturnData(
        _a_py,
        _d_py,
        _e_py,
        _tau_py,
    )
    return _return_data

def zungtr(uplo, a, tau):
    u"""
    Generate unitary transformation matrix from reduction to tridiagonal
    form determined by ``zhetrd``.

    ``zungtr`` generates the complex unitary matrix Q, which was
    determined by ``zhetrd`` when reducing a Hermitian matrix to
    tridiagonal form.

    For full information please refer to the NAG Library document for
    f08ft

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08ftf.html

    Parameters
    ----------
    uplo : str, length 1
        This **must** be the same argument `uplo` as supplied to
        ``zhetrd``.

    a : complex, array-like, shape (n, n)
        Details of the vectors which define the elementary reflectors,
        as returned by ``zhetrd``.

    tau : complex, array-like, shape (max(1, n-1))
        Further details of the elementary reflectors, as returned by
        ``zhetrd``.

    Returns
    -------
    a : complex, ndarray, shape (n, n)
        The n*n unitary matrix Q.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= 0.

    Notes
    -----
    ``zungtr`` is intended to be used after a call to ``zhetrd``, which
    reduces a complex Hermitian matrix A to real symmetric tridiagonal
    form T by a unitary similarity transformation: A = QTQ^H. ``zhetrd``
    represents the unitary matrix Q as a product of n-1 elementary
    reflectors.

    This function may be used to generate Q explicitly as a square
    matrix.

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zungtr'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _tau = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'tau'},
        tau,
        copy=False,
    )
    _b_lapackeig.zungtr(
        uplo,
        _n,
        _a,
        _tau,
    )
    _a_py = _a
    return _a_py

def zunmtr(side, uplo, trans, a, tau, c):
    u"""
    Applies the unitary transformation matrix determined by ``zhetrd``.

    ``zunmtr`` multiplies an arbitrary complex matrix C by the complex
    unitary matrix Q which was determined by ``zhetrd`` when reducing a
    complex Hermitian matrix to tridiagonal form.

    For full information please refer to the NAG Library document for
    f08fu

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08fuf.html

    Parameters
    ----------
    side : str, length 1
        Indicates how Q or Q^H is to be applied to C.

        `side` = 'L'

            Q or Q^H is applied to C from the left.

        `side` = 'R'

            Q or Q^H is applied to C from the right.

    uplo : str, length 1
        This **must** be the same argument `uplo` as supplied to
        ``zhetrd``.

    trans : str, length 1
        Indicates whether Q or Q^H is to be applied to C.

        `trans` = 'N'

            Q is applied to C.

        `trans` = 'C'

            Q^H is applied to C.

    a : complex, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `side`='L': m; if `side`='R': n;
        otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `side`='L': m; if `side`='R': n;
        otherwise: 0.

        Details of the vectors which define the elementary reflectors,
        as returned by ``zhetrd``.

    tau : complex, array-like, shape (:)
        Note: the required length for this argument is determined as
        follows: if `side`='L': m-1; if `side`='R': n-1; otherwise: 0.

        Further details of the elementary reflectors, as returned by
        ``zhetrd``.

    c : complex, array-like, shape (m, n)
        The m*n matrix C.

    Returns
    -------
    c : complex, ndarray, shape (m, n)
        `c` is overwritten by QC or Q^HC or CQ or CQ^H as specified by
        `side` and `trans`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'C'.

        (`errno` -4)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -5)
            On entry, error in parameter n.

            Constraint: n >= 0.

    Notes
    -----
    ``zunmtr`` is intended to be used after a call to ``zhetrd``, which
    reduces a complex Hermitian matrix A to real symmetric tridiagonal
    form T by a unitary similarity transformation: A = QTQ^H. ``zhetrd``
    represents the unitary matrix Q as a product of elementary
    reflectors.

    This function may be used to form one of the matrix products

        QC,Q^HC,CQ or CQ^H,

    overwriting the result on C (which may be any complex rectangular
    matrix).

    A common application of this function is to transform a matrix Z of
    eigenvectors of T to the matrix QZ of eigenvectors of A.

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zunmtr'

    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=True,
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _tau = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'tau'},
        tau,
        copy=False,
    )
    _b_lapackeig.zunmtr(
        side,
        uplo,
        trans,
        _m,
        _n,
        _a,
        _tau,
        _c,
    )
    _c_py = _c
    return _c_py

DspevReturnData = _collections.namedtuple(
    'DspevReturnData',
    ['ap', 'w', 'z'],
)

def dspev(jobz, uplo, n, ap):
    u"""
    Computes all eigenvalues and, optionally, eigenvectors of a real
    symmetric matrix, packed storage.

    ``dspev`` computes all the eigenvalues and, optionally, all the
    eigenvectors of a real n*n symmetric matrix A in packed storage.

    For full information please refer to the NAG Library document for
    f08ga

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08gaf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

            Only eigenvalues are computed.

        `jobz` = 'V'

            Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangular part of A is stored.

        If `uplo` = 'L', the lower triangular part of A is stored.

    n : int
        n, the order of the matrix A.

    ap : float, array-like, shape (`n`*(`n`+1)/2)
        The upper or lower triangle of the n*n symmetric matrix A,
        packed by columns.

    Returns
    -------
    ap : float, ndarray, shape (`n`*(`n`+1)/2)
        `ap` is overwritten by the values generated during the reduction
        to tridiagonal form. The elements of the diagonal and the
        off-diagonal of the tridiagonal matrix overwrite the
        corresponding elements of A.

    w : float, ndarray, shape (`n`)
        The eigenvalues in ascending order.

    z : float, ndarray, shape (:, :)
        If `jobz` = 'V', `z` contains the orthonormal eigenvectors of
        the matrix A, with the ith column of Z holding the eigenvector
        associated with `w`[i-1].

        If `jobz` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` i > 0)
            The algorithm failed to converge; *<value>* off-diagonal
            elements of an intermediate tridiagonal form did not
            converge to zero.

    Notes
    -----
    The symmetric matrix A is first reduced to tridiagonal form, using
    orthogonal similarity transformations, and then the QR algorithm is
    applied to the tridiagonal matrix to compute the eigenvalues and
    (optionally) the eigenvectors.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dspev'

    _ap = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ap'},
        ap,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobz = _b_utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = _b_utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        _z_dim2 = _n_py
    else:
        _z_dim2 = 1
    exp_shape_z = (_z_dim1, _z_dim2)
    _z = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_z,
        sorder=_b_utils._EngineData.default_sorder,
    )
    _b_lapackeig.dspev(
        jobz,
        uplo,
        _n,
        _ap,
        _w,
        _z,
    )
    _ap_py = _ap
    _w_py = _w
    _z_py = _z
    _return_data = DspevReturnData(
        _ap_py,
        _w_py,
        _z_py,
    )
    return _return_data

DspevxReturnData = _collections.namedtuple(
    'DspevxReturnData',
    ['ap', 'm', 'w', 'z', 'jfail'],
)

def dspevx(jobz, erange, uplo, n, ap, vl, vu, il, iu, abstol):
    u"""
    Computes selected eigenvalues and, optionally, eigenvectors of a
    real symmetric matrix, packed storage.

    ``dspevx`` computes selected eigenvalues and, optionally,
    eigenvectors of a real n*n symmetric matrix A in packed storage.
    Eigenvalues and eigenvectors can be selected by specifying either a
    range of values or a range of indices for the desired eigenvalues.

    For full information please refer to the NAG Library document for
    f08gb

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08gbf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

            Only eigenvalues are computed.

        `jobz` = 'V'

            Eigenvalues and eigenvectors are computed.

    erange : str, length 1
        If `erange` = 'A', all eigenvalues will be found.

        If `erange` = 'V', all eigenvalues in the half-open interval
        (`vl`,`vu`] will be found.

        If `erange` = 'I', the `il`th to `iu`th eigenvalues will be
        found.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangular part of A is stored.

        If `uplo` = 'L', the lower triangular part of A is stored.

    n : int
        n, the order of the matrix A.

    ap : float, array-like, shape (`n`*(`n`+1)/2)
        The upper or lower triangle of the n*n symmetric matrix A,
        packed by columns.

    vl : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    vu : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    il : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    iu : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    abstol : float
        The absolute error tolerance for the eigenvalues. An approximate
        eigenvalue is accepted as converged when it is determined to lie
        in an interval [a,b] of width less than or equal to

            `abstol`+epsilon max(|a|, |b|),

        where epsilon is the machine precision. If `abstol` is less than
        or equal to zero, then epsilon||T||_1 will be used in its place,
        where T is the tridiagonal matrix obtained by reducing A to
        tridiagonal form. Eigenvalues will be computed most accurately
        when `abstol` is set to twice the underflow threshold
        2*machine.real_safe(), not zero. If this function returns with
        `errno` > 0, indicating that some eigenvectors did not converge,
        try setting `abstol` to 2*machine.real_safe(). See Demmel and
        Kahan (1990).

    Returns
    -------
    ap : float, ndarray, shape (`n`*(`n`+1)/2)
        `ap` is overwritten by the values generated during the reduction
        to tridiagonal form. The elements of the diagonal and the
        off-diagonal of the tridiagonal matrix overwrite the
        corresponding elements of A.

    m : int
        The total number of eigenvalues found. 0 <= `m` <= `n`.

        If `erange` = 'A', `m` = `n`.

        If `erange` = 'I', `m` = `iu`-`il`+1.

    w : float, ndarray, shape (`n`)
        The selected eigenvalues in ascending order.

    z : float, ndarray, shape (:, :)
        If `jobz` = 'V', then

            if no exception or warning is raised, the first `m` columns
            of Z contain the orthonormal eigenvectors of the matrix A
            corresponding to the selected eigenvalues, with the ith
            column of Z holding the eigenvector associated with
            `w`[i-1];

            if an eigenvector fails to converge (`errno` > 0), then that
            column of Z contains the latest approximation to the
            eigenvector, and the index of the eigenvector is returned in
            `jfail`.

        If `jobz` = 'N', `z` is not referenced.

        **Note:** you must ensure that at least max(1, `m`) columns are
        supplied in the array `z`; if `erange` = 'V', the exact value of
        `m` is not known in advance and an upper bound of at least `n`
        must be used.

    jfail : int, ndarray, shape (`n`)
        If `jobz` = 'V', then

            if no exception or warning is raised, the first `m` elements
            of `jfail` are zero;

            if `errno` > 0, `jfail` contains the indices of the
            eigenvectors that failed to converge.

        If `jobz` = 'N', `jfail` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `erange`.

            Constraint: `erange` = 'A', 'V' or 'I'.

        (`errno` -3)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -7)
            On entry, error in parameter `vu`.

            Constraint: `vl` < `vu`.

        (`errno` -8)
            On entry, error in parameter `il`.

        (`errno` -9)
            On entry, error in parameter `iu`.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            The algorithm failed to converge; *<value>* eigenvectors did
            not converge. Their indices are stored in array `jfail`.

    Notes
    -----
    The symmetric matrix A is first reduced to tridiagonal form, using
    orthogonal similarity transformations.
    The required eigenvalues and eigenvectors are then computed from the
    tridiagonal matrix; the method used depends upon whether all, or
    selected, eigenvalues and eigenvectors are required.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Demmel, J W and Kahan, W, 1990, `Accurate singular values of
    bidiagonal matrices`, SIAM J. Sci. Statist. Comput. (11), 873--912

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dspevx'

    _ap = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ap'},
        ap,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobz = _b_utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = _b_utils._EngineCharScalarType.to_py(_jobz)
    _erange = _b_utils._EngineCharScalarType(
        erange, {'fun_name': fun_name, 'entity': 'erange'}, exp_chrlen=1,
    ).to_ctypes()
    _erange_py = _b_utils._EngineCharScalarType.to_py(_erange)
    _iu = _b_utils._EngineIntScalarType(
        iu, {'fun_name': fun_name, 'entity': 'iu'},
    ).to_ctypes()
    _iu_py = _b_utils._EngineIntScalarType.to_py(_iu)
    _il = _b_utils._EngineIntScalarType(
        il, {'fun_name': fun_name, 'entity': 'il'},
    ).to_ctypes()
    _il_py = _b_utils._EngineIntScalarType.to_py(_il)
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        if _erange_py in ('I', 'i'):
            _z_dim2 = _iu_py-_il_py+1
        else:
            _z_dim2 = _n_py
    else:
        _z_dim2 = 0
    exp_shape_z = (_z_dim1, _z_dim2)
    _z = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_z,
        sorder=_b_utils._EngineData.default_sorder,
    )
    _jfail = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        _return_scalar = _b_lapackeig.dspevx(
            jobz,
            erange,
            uplo,
            _n,
            _ap,
            vl,
            vu,
            _il,
            _iu,
            abstol,
            _w,
            _z,
            _jfail,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _ap_py = _ap
    _m_py = _return_scalar
    _w_py = _w
    if _jobz_py in ('V', 'v'):
        _z_dim2 = _m_py
    else:
        _z_dim2 = 0
    _z_py = _z[:, :_z_dim2]
    _jfail_py = _jfail
    _return_data = DspevxReturnData(
        _ap_py,
        _m_py,
        _w_py,
        _z_py,
        _jfail_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

DspevdReturnData = _collections.namedtuple(
    'DspevdReturnData',
    ['ap', 'w', 'z'],
)

def dspevd(job, uplo, n, ap):
    u"""
    Computes all eigenvalues and, optionally, all eigenvectors of real
    symmetric matrix, packed storage (divide-and-conquer or
    Pal--Walker--Kahan variant of the QL or QR algorithm).

    ``dspevd`` computes all the eigenvalues and, optionally, all the
    eigenvectors of a real symmetric matrix held in packed storage. If
    the eigenvectors are requested, then it uses a divide-and-conquer
    algorithm to compute eigenvalues and eigenvectors.
    However, if only eigenvalues are required, then it uses the
    Pal--Walker--Kahan variant of the QL or QR algorithm.

    For full information please refer to the NAG Library document for
    f08gc

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08gcf.html

    Parameters
    ----------
    job : str, length 1
        Indicates whether eigenvectors are computed.

        `job` = 'N'

            Only eigenvalues are computed.

        `job` = 'V'

            Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        Indicates whether the upper or lower triangular part of A is
        stored.

        `uplo` = 'U'

            The upper triangular part of A is stored.

        `uplo` = 'L'

            The lower triangular part of A is stored.

    n : int
        n, the order of the matrix A.

    ap : float, array-like, shape (`n`*(`n`+1)/2)
        The upper or lower triangle of the n*n symmetric matrix A,
        packed by columns.

    Returns
    -------
    ap : float, ndarray, shape (`n`*(`n`+1)/2)
        `ap` is overwritten by the values generated during the reduction
        to tridiagonal form. The elements of the diagonal and the
        off-diagonal of the tridiagonal matrix overwrite the
        corresponding elements of A.

    w : float, ndarray, shape (`n`)
        The eigenvalues of the matrix A in ascending order.

    z : float, ndarray, shape (:, :)
        If `job` = 'V', `z` is overwritten by the orthogonal matrix Z
        which contains the eigenvectors of A.

        If `job` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` i > 0)
            If errno = *<value>* and `job` = 'N', the algorithm failed
            to converge; *<value>* elements of an intermediate
            tridiagonal form did not converge to zero; if errno =
            *<value>* and `job` = 'V', then the algorithm failed to
            compute an eigenvalue while working on the submatrix lying
            in rows and column *<value>*/(`n`+1) through mod(*<value>*,
            `n`+1).

    Notes
    -----
    ``dspevd`` computes all the eigenvalues and, optionally, all the
    eigenvectors of a real symmetric matrix A (held in packed storage).
    In other words, it can compute the spectral factorization of A as

        A = Z Lambda Z^T,

    where Lambda is a diagonal matrix whose diagonal elements are the
    eigenvalues lambda_i, and Z is the orthogonal matrix whose columns
    are the eigenvectors z_i.
    Thus

        Az_i = lambda_i z_i, i = 1,2,...,n.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dspevd'

    _ap = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ap'},
        ap,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _job = _b_utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _job_py = _b_utils._EngineCharScalarType.to_py(_job)
    if _job_py in ('V', 'v'):
        _z_dim1 = _n_py
    elif _job_py in ('N', 'n'):
        _z_dim1 = 1
    else:
        _z_dim1 = 0
    if _job_py in ('V', 'v'):
        _z_dim2 = _n_py
    elif _job_py in ('N', 'n'):
        _z_dim2 = 1
    else:
        _z_dim2 = 0
    exp_shape_z = (_z_dim1, _z_dim2)
    _z = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_z,
        sorder=_b_utils._EngineData.default_sorder,
    )
    _b_lapackeig.dspevd(
        job,
        uplo,
        _n,
        _ap,
        _w,
        _z,
    )
    _ap_py = _ap
    _w_py = _w
    _z_py = _z
    _return_data = DspevdReturnData(
        _ap_py,
        _w_py,
        _z_py,
    )
    return _return_data

DsptrdReturnData = _collections.namedtuple(
    'DsptrdReturnData',
    ['ap', 'd', 'e', 'tau'],
)

def dsptrd(uplo, n, ap):
    u"""
    Orthogonal reduction of real symmetric matrix to symmetric
    tridiagonal form, packed storage.

    ``dsptrd`` reduces a real symmetric matrix to tridiagonal form,
    using packed storage.

    For full information please refer to the NAG Library document for
    f08ge

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08gef.html

    Parameters
    ----------
    uplo : str, length 1
        Indicates whether the upper or lower triangular part of A is
        stored.

        `uplo` = 'U'

            The upper triangular part of A is stored.

        `uplo` = 'L'

            The lower triangular part of A is stored.

    n : int
        n, the order of the matrix A.

    ap : float, array-like, shape (`n`*(`n`+1)/2)
        The upper or lower triangle of the n*n symmetric matrix A,
        packed by columns.

    Returns
    -------
    ap : float, ndarray, shape (`n`*(`n`+1)/2)
        `ap` is overwritten by the tridiagonal matrix T and details of
        the orthogonal matrix Q.

    d : float, ndarray, shape (`n`)
        The diagonal elements of the tridiagonal matrix T.

    e : float, ndarray, shape (max(1, `n`-1))
        The off-diagonal elements of the tridiagonal matrix T.

    tau : float, ndarray, shape (max(1, `n`-1))
        Further details of the orthogonal matrix Q.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Notes
    -----
    ``dsptrd`` reduces a real symmetric matrix A, held in packed
    storage, to symmetric tridiagonal form T by an orthogonal similarity
    transformation: A = QTQ^T.

    The matrix Q is not formed explicitly but is represented as a
    product of n-1 elementary reflectors (see [the F08 Introduction] for
    details). Functions are provided to work with Q in this
    representation (see [Further Comments]).

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dsptrd'

    _ap = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ap'},
        ap,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _d = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _e = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (max(1, _n_py-1),),
    )
    _tau = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (max(1, _n_py-1),),
    )
    _b_lapackeig.dsptrd(
        uplo,
        _n,
        _ap,
        _d,
        _e,
        _tau,
    )
    _ap_py = _ap
    _d_py = _d
    _e_py = _e
    _tau_py = _tau
    _return_data = DsptrdReturnData(
        _ap_py,
        _d_py,
        _e_py,
        _tau_py,
    )
    return _return_data

def dopgtr(uplo, n, ap, tau):
    u"""
    Generate orthogonal transformation matrix from reduction to
    tridiagonal form determined by ``dsptrd``.

    ``dopgtr`` generates the real orthogonal matrix Q, which was
    determined by ``dsptrd`` when reducing a symmetric matrix to
    tridiagonal form.

    For full information please refer to the NAG Library document for
    f08gf

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08gff.html

    Parameters
    ----------
    uplo : str, length 1
        This **must** be the same argument `uplo` as supplied to
        ``dsptrd``.

    n : int
        n, the order of the matrix Q.

    ap : float, array-like, shape (`n`*(`n`+1)/2)
        Details of the vectors which define the elementary reflectors,
        as returned by ``dsptrd``.

    tau : float, array-like, shape (max(1, `n`-1))
        Further details of the elementary reflectors, as returned by
        ``dsptrd``.

    Returns
    -------
    q : float, ndarray, shape (`n`, `n`)
        The n*n orthogonal matrix Q.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Notes
    -----
    ``dopgtr`` is intended to be used after a call to ``dsptrd``, which
    reduces a real symmetric matrix A to symmetric tridiagonal form T by
    an orthogonal similarity transformation: A = QTQ^T. ``dsptrd``
    represents the orthogonal matrix Q as a product of n-1 elementary
    reflectors.

    This function may be used to generate Q explicitly as a square
    matrix.

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dopgtr'

    _ap = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ap'},
        ap,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _tau = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'tau'},
        tau,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _q = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _b_lapackeig.dopgtr(
        uplo,
        _n,
        _ap,
        _tau,
        _q,
    )
    _q_py = _q
    return _q_py

DopmtrReturnData = _collections.namedtuple(
    'DopmtrReturnData',
    ['ap', 'c'],
)

def dopmtr(side, uplo, trans, ap, tau, c):
    u"""
    Applies the orthogonal transformation determined by ``dsptrd``.

    ``dopmtr`` multiplies an arbitrary real matrix C by the real
    orthogonal matrix Q which was determined by ``dsptrd`` when reducing
    a real symmetric matrix to tridiagonal form.

    For full information please refer to the NAG Library document for
    f08gg

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08ggf.html

    Parameters
    ----------
    side : str, length 1
        Indicates how Q or Q^T is to be applied to C.

        `side` = 'L'

            Q or Q^T is applied to C from the left.

        `side` = 'R'

            Q or Q^T is applied to C from the right.

    uplo : str, length 1
        This **must** be the same argument `uplo` as supplied to
        ``dsptrd``.

    trans : str, length 1
        Indicates whether Q or Q^T is to be applied to C.

        `trans` = 'N'

            Q is applied to C.

        `trans` = 'T'

            Q^T is applied to C.

    ap : float, array-like, shape (:)
        Note: the required length for this argument is determined as
        follows: if `side`='L': m*(m+1)/2; if `side`='R': n*(n+1)/2;
        otherwise: 0.

        Details of the vectors which define the elementary reflectors,
        as returned by ``dsptrd``.

    tau : float, array-like, shape (:)
        Note: the required length for this argument is determined as
        follows: if `side`='L': m-1; if `side`='R': n-1; otherwise: 0.

        Further details of the elementary reflectors, as returned by
        ``dsptrd``.

    c : float, array-like, shape (m, n)
        The m*n matrix C.

    Returns
    -------
    ap : float, ndarray, shape (:)
        Is used as internal workspace prior to being restored and hence
        is unchanged.

    c : float, ndarray, shape (m, n)
        `c` is overwritten by QC or Q^TC or CQ or CQ^T as specified by
        `side` and `trans`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'T'.

        (`errno` -4)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -5)
            On entry, error in parameter n.

            Constraint: n >= 0.

    Notes
    -----
    ``dopmtr`` is intended to be used after a call to ``dsptrd``, which
    reduces a real symmetric matrix A to symmetric tridiagonal form T by
    an orthogonal similarity transformation: A = QTQ^T. ``dsptrd``
    represents the orthogonal matrix Q as a product of elementary
    reflectors.

    This function may be used to form one of the matrix products

        QC,Q^TC,CQ or CQ^T,

    overwriting the result on C (which may be any real rectangular
    matrix).

    A common application of this function is to transform a matrix Z of
    eigenvectors of T to the matrix QZ of eigenvectors of A.

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dopmtr'

    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _ap = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ap'},
        ap,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _tau = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'tau'},
        tau,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b_lapackeig.dopmtr(
        side,
        uplo,
        trans,
        _m,
        _n,
        _ap,
        _tau,
        _c,
    )
    _ap_py = _ap
    _c_py = _c
    _return_data = DopmtrReturnData(
        _ap_py,
        _c_py,
    )
    return _return_data

ZhpevReturnData = _collections.namedtuple(
    'ZhpevReturnData',
    ['ap', 'w', 'z'],
)

def zhpev(jobz, uplo, n, ap):
    u"""
    Computes all eigenvalues and, optionally, eigenvectors of a complex
    Hermitian matrix, packed storage.

    ``zhpev`` computes all the eigenvalues and, optionally, all the
    eigenvectors of a complex n*n Hermitian matrix A in packed storage.

    For full information please refer to the NAG Library document for
    f08gn

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08gnf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

            Only eigenvalues are computed.

        `jobz` = 'V'

            Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangular part of A is stored.

        If `uplo` = 'L', the lower triangular part of A is stored.

    n : int
        n, the order of the matrix A.

    ap : complex, array-like, shape (`n`*(`n`+1)/2)
        The upper or lower triangle of the n*n Hermitian matrix A,
        packed by columns.

    Returns
    -------
    ap : complex, ndarray, shape (`n`*(`n`+1)/2)
        `ap` is overwritten by the values generated during the reduction
        to tridiagonal form. The elements of the diagonal and the
        off-diagonal of the tridiagonal matrix overwrite the
        corresponding elements of A.

    w : float, ndarray, shape (`n`)
        The eigenvalues in ascending order.

    z : complex, ndarray, shape (:, :)
        If `jobz` = 'V', `z` contains the orthonormal eigenvectors of
        the matrix A, with the ith column of Z holding the eigenvector
        associated with `w`[i-1].

        If `jobz` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` i > 0)
            The algorithm failed to converge; *<value>* off-diagonal
            elements of an intermediate tridiagonal form did not
            converge to zero.

    Notes
    -----
    The Hermitian matrix A is first reduced to real tridiagonal form,
    using unitary similarity transformations, and then the QR algorithm
    is applied to the tridiagonal matrix to compute the eigenvalues and
    (optionally) the eigenvectors.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zhpev'

    _ap = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ap'},
        ap,
        copy=True,
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobz = _b_utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = _b_utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        _z_dim2 = _n_py
    else:
        _z_dim2 = 1
    exp_shape_z = (_z_dim1, _z_dim2)
    _z = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_z,
        sorder=_b_utils._EngineData.default_sorder,
    )
    _b_lapackeig.zhpev(
        jobz,
        uplo,
        _n,
        _ap,
        _w,
        _z,
    )
    _ap_py = _ap
    _w_py = _w
    _z_py = _z
    _return_data = ZhpevReturnData(
        _ap_py,
        _w_py,
        _z_py,
    )
    return _return_data

ZhpevxReturnData = _collections.namedtuple(
    'ZhpevxReturnData',
    ['ap', 'm', 'w', 'z', 'jfail'],
)

def zhpevx(jobz, erange, uplo, n, ap, vl, vu, il, iu, abstol):
    u"""
    Computes selected eigenvalues and, optionally, eigenvectors of a
    complex Hermitian matrix, packed storage.

    ``zhpevx`` computes selected eigenvalues and, optionally,
    eigenvectors of a complex n*n Hermitian matrix A in packed storage.
    Eigenvalues and eigenvectors can be selected by specifying either a
    range of values or a range of indices for the desired eigenvalues.

    For full information please refer to the NAG Library document for
    f08gp

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08gpf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

            Only eigenvalues are computed.

        `jobz` = 'V'

            Eigenvalues and eigenvectors are computed.

    erange : str, length 1
        If `erange` = 'A', all eigenvalues will be found.

        If `erange` = 'V', all eigenvalues in the half-open interval
        (`vl`,`vu`] will be found.

        If `erange` = 'I', the `il`th to `iu`th eigenvalues will be
        found.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangular part of A is stored.

        If `uplo` = 'L', the lower triangular part of A is stored.

    n : int
        n, the order of the matrix A.

    ap : complex, array-like, shape (`n`*(`n`+1)/2)
        The upper or lower triangle of the n*n Hermitian matrix A,
        packed by columns.

    vl : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    vu : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    il : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    iu : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    abstol : float
        The absolute error tolerance for the eigenvalues. An approximate
        eigenvalue is accepted as converged when it is determined to lie
        in an interval [a,b] of width less than or equal to

            `abstol`+epsilon max(|a|, |b|),

        where epsilon is the machine precision. If `abstol` is less than
        or equal to zero, then epsilon||T||_1 will be used in its place,
        where T is the tridiagonal matrix obtained by reducing A to
        tridiagonal form. Eigenvalues will be computed most accurately
        when `abstol` is set to twice the underflow threshold
        2*machine.real_safe(), not zero. If this function returns with
        `errno` > 0, indicating that some eigenvectors did not converge,
        try setting `abstol` to 2*machine.real_safe(). See Demmel and
        Kahan (1990).

    Returns
    -------
    ap : complex, ndarray, shape (`n`*(`n`+1)/2)
        `ap` is overwritten by the values generated during the reduction
        to tridiagonal form. The elements of the diagonal and the
        off-diagonal of the tridiagonal matrix overwrite the
        corresponding elements of A.

    m : int
        The total number of eigenvalues found. 0 <= `m` <= `n`.

        If `erange` = 'A', `m` = `n`.

        If `erange` = 'I', `m` = `iu`-`il`+1.

    w : float, ndarray, shape (`n`)
        The selected eigenvalues in ascending order.

    z : complex, ndarray, shape (:, :)
        If `jobz` = 'V', then

            if no exception or warning is raised, the first `m` columns
            of Z contain the orthonormal eigenvectors of the matrix A
            corresponding to the selected eigenvalues, with the ith
            column of Z holding the eigenvector associated with
            `w`[i-1];

            if an eigenvector fails to converge (`errno` > 0), then that
            column of Z contains the latest approximation to the
            eigenvector, and the index of the eigenvector is returned in
            `jfail`.

        If `jobz` = 'N', `z` is not referenced.

        **Note:** you must ensure that at least max(1, `m`) columns are
        supplied in the array `z`; if `erange` = 'V', the exact value of
        `m` is not known in advance and an upper bound of at least `n`
        must be used.

    jfail : int, ndarray, shape (`n`)
        If `jobz` = 'V', then

            if no exception or warning is raised, the first `m` elements
            of `jfail` are zero;

            if `errno` > 0, `jfail` contains the indices of the
            eigenvectors that failed to converge.

        If `jobz` = 'N', `jfail` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `erange`.

            Constraint: `erange` = 'A', 'V' or 'I'.

        (`errno` -3)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -7)
            On entry, error in parameter `vu`.

            Constraint: `vl` < `vu`.

        (`errno` -8)
            On entry, error in parameter `il`.

        (`errno` -9)
            On entry, error in parameter `iu`.

        (`errno` i > 0)
            The algorithm failed to converge; *<value>* eigenvectors did
            not converge. Their indices are stored in array `jfail`.

    Notes
    -----
    The Hermitian matrix A is first reduced to real tridiagonal form,
    using unitary similarity transformations.
    The required eigenvalues and eigenvectors are then computed from the
    tridiagonal matrix; the method used depends upon whether all, or
    selected, eigenvalues and eigenvectors are required.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Demmel, J W and Kahan, W, 1990, `Accurate singular values of
    bidiagonal matrices`, SIAM J. Sci. Statist. Comput. (11), 873--912

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zhpevx'

    _ap = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ap'},
        ap,
        copy=True,
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobz = _b_utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = _b_utils._EngineCharScalarType.to_py(_jobz)
    _erange = _b_utils._EngineCharScalarType(
        erange, {'fun_name': fun_name, 'entity': 'erange'}, exp_chrlen=1,
    ).to_ctypes()
    _erange_py = _b_utils._EngineCharScalarType.to_py(_erange)
    _iu = _b_utils._EngineIntScalarType(
        iu, {'fun_name': fun_name, 'entity': 'iu'},
    ).to_ctypes()
    _iu_py = _b_utils._EngineIntScalarType.to_py(_iu)
    _il = _b_utils._EngineIntScalarType(
        il, {'fun_name': fun_name, 'entity': 'il'},
    ).to_ctypes()
    _il_py = _b_utils._EngineIntScalarType.to_py(_il)
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        if _erange_py in ('I', 'i'):
            _z_dim2 = _iu_py-_il_py+1
        else:
            _z_dim2 = _n_py
    else:
        _z_dim2 = 0
    exp_shape_z = (_z_dim1, _z_dim2)
    _z = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_z,
        sorder=_b_utils._EngineData.default_sorder,
    )
    _jfail = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    _return_scalar = _b_lapackeig.zhpevx(
        jobz,
        erange,
        uplo,
        _n,
        _ap,
        vl,
        vu,
        _il,
        _iu,
        abstol,
        _w,
        _z,
        _jfail,
    )
    _ap_py = _ap
    _m_py = _return_scalar
    _w_py = _w
    if _jobz_py in ('V', 'v'):
        _z_dim2 = _m_py
    else:
        _z_dim2 = 0
    _z_py = _z[:, :_z_dim2]
    _jfail_py = _jfail
    _return_data = ZhpevxReturnData(
        _ap_py,
        _m_py,
        _w_py,
        _z_py,
        _jfail_py,
    )
    return _return_data

ZhpevdReturnData = _collections.namedtuple(
    'ZhpevdReturnData',
    ['ap', 'w', 'z'],
)

def zhpevd(job, uplo, n, ap):
    u"""
    Computes all eigenvalues and, optionally, all eigenvectors of
    complex Hermitian matrix, packed storage (divide-and-conquer or
    Pal--Walker--Kahan variant of the QL or QR algorithm).

    ``zhpevd`` computes all the eigenvalues and, optionally, all the
    eigenvectors of a complex Hermitian matrix held in packed storage.
    If the eigenvectors are requested, then it uses a divide-and-conquer
    algorithm to compute eigenvalues and eigenvectors.
    However, if only eigenvalues are required, then it uses the
    Pal--Walker--Kahan variant of the QL or QR algorithm.

    For full information please refer to the NAG Library document for
    f08gq

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08gqf.html

    Parameters
    ----------
    job : str, length 1
        Indicates whether eigenvectors are computed.

        `job` = 'N'

            Only eigenvalues are computed.

        `job` = 'V'

            Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        Indicates whether the upper or lower triangular part of A is
        stored.

        `uplo` = 'U'

            The upper triangular part of A is stored.

        `uplo` = 'L'

            The lower triangular part of A is stored.

    n : int
        n, the order of the matrix A.

    ap : complex, array-like, shape (`n`*(`n`+1)/2)
        The upper or lower triangle of the n*n Hermitian matrix A,
        packed by columns.

    Returns
    -------
    ap : complex, ndarray, shape (`n`*(`n`+1)/2)
        `ap` is overwritten by the values generated during the reduction
        to tridiagonal form. The elements of the diagonal and the
        off-diagonal of the tridiagonal matrix overwrite the
        corresponding elements of A.

    w : float, ndarray, shape (`n`)
        The eigenvalues of the matrix A in ascending order.

    z : complex, ndarray, shape (:, :)
        If `job` = 'V', `z` is overwritten by the unitary matrix Z which
        contains the eigenvectors of A.

        If `job` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` i > 0)
            If errno = *<value>* and `job` = 'N', the algorithm failed
            to converge; *<value>* elements of an intermediate
            tridiagonal form did not converge to zero; if errno =
            *<value>* and `job` = 'V', then the algorithm failed to
            compute an eigenvalue while working on the submatrix lying
            in rows and column *<value>*/(`n`+1) through mod(*<value>*,
            `n`+1).

    Notes
    -----
    ``zhpevd`` computes all the eigenvalues and, optionally, all the
    eigenvectors of a complex Hermitian matrix A (held in packed
    storage). In other words, it can compute the spectral factorization
    of A as

        A = Z Lambda Z^H,

    where Lambda is a real diagonal matrix whose diagonal elements are
    the eigenvalues lambda_i, and Z is the (complex) unitary matrix
    whose columns are the eigenvectors z_i.
    Thus

        Az_i = lambda_i z_i, i = 1,2,...,n.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zhpevd'

    _ap = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ap'},
        ap,
        copy=True,
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _job = _b_utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _job_py = _b_utils._EngineCharScalarType.to_py(_job)
    if _job_py in ('V', 'v'):
        _z_dim1 = _n_py
    elif _job_py in ('N', 'n'):
        _z_dim1 = 1
    else:
        _z_dim1 = 0
    if _job_py in ('V', 'v'):
        _z_dim2 = _n_py
    elif _job_py in ('N', 'n'):
        _z_dim2 = 1
    else:
        _z_dim2 = 0
    exp_shape_z = (_z_dim1, _z_dim2)
    _z = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_z,
        sorder=_b_utils._EngineData.default_sorder,
    )
    _b_lapackeig.zhpevd(
        job,
        uplo,
        _n,
        _ap,
        _w,
        _z,
    )
    _ap_py = _ap
    _w_py = _w
    _z_py = _z
    _return_data = ZhpevdReturnData(
        _ap_py,
        _w_py,
        _z_py,
    )
    return _return_data

ZhptrdReturnData = _collections.namedtuple(
    'ZhptrdReturnData',
    ['ap', 'd', 'e', 'tau'],
)

def zhptrd(uplo, n, ap):
    u"""
    Performs a unitary reduction of complex Hermitian matrix to real
    symmetric tridiagonal form, packed storage.

    ``zhptrd`` reduces a complex Hermitian matrix to tridiagonal form,
    using packed storage.

    For full information please refer to the NAG Library document for
    f08gs

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08gsf.html

    Parameters
    ----------
    uplo : str, length 1
        Indicates whether the upper or lower triangular part of A is
        stored.

        `uplo` = 'U'

            The upper triangular part of A is stored.

        `uplo` = 'L'

            The lower triangular part of A is stored.

    n : int
        n, the order of the matrix A.

    ap : complex, array-like, shape (`n`*(`n`+1)/2)
        The upper or lower triangle of the n*n Hermitian matrix A,
        packed by columns.

    Returns
    -------
    ap : complex, ndarray, shape (`n`*(`n`+1)/2)
        `ap` is overwritten by the tridiagonal matrix T and details of
        the unitary matrix Q.

    d : float, ndarray, shape (`n`)
        The diagonal elements of the tridiagonal matrix T.

    e : float, ndarray, shape (max(1, `n`-1))
        The off-diagonal elements of the tridiagonal matrix T.

    tau : complex, ndarray, shape (max(1, `n`-1))
        Further details of the unitary matrix Q.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Notes
    -----
    ``zhptrd`` reduces a complex Hermitian matrix A, held in packed
    storage, to real symmetric tridiagonal form T by a unitary
    similarity transformation: A = QTQ^H.

    The matrix Q is not formed explicitly but is represented as a
    product of n-1 elementary reflectors (see [the F08 Introduction] for
    details). Functions are provided to work with Q in this
    representation (see [Further Comments]).

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zhptrd'

    _ap = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ap'},
        ap,
        copy=True,
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _d = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _e = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (max(1, _n_py-1),),
    )
    _tau = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (max(1, _n_py-1),),
    )
    _b_lapackeig.zhptrd(
        uplo,
        _n,
        _ap,
        _d,
        _e,
        _tau,
    )
    _ap_py = _ap
    _d_py = _d
    _e_py = _e
    _tau_py = _tau
    _return_data = ZhptrdReturnData(
        _ap_py,
        _d_py,
        _e_py,
        _tau_py,
    )
    return _return_data

def zupgtr(uplo, n, ap, tau):
    u"""
    Generates a unitary transformation matrix from reduction to
    tridiagonal form determined by ``zhptrd``.

    ``zupgtr`` generates the complex unitary matrix Q, which was
    determined by ``zhptrd`` when reducing a Hermitian matrix to
    tridiagonal form.

    For full information please refer to the NAG Library document for
    f08gt

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08gtf.html

    Parameters
    ----------
    uplo : str, length 1
        This **must** be the same argument `uplo` as supplied to
        ``zhptrd``.

    n : int
        n, the order of the matrix Q.

    ap : complex, array-like, shape (`n`*(`n`+1)/2)
        Details of the vectors which define the elementary reflectors,
        as returned by ``zhptrd``.

    tau : complex, array-like, shape (max(1, `n`-1))
        Further details of the elementary reflectors, as returned by
        ``zhptrd``.

    Returns
    -------
    q : complex, ndarray, shape (`n`, `n`)
        The n*n unitary matrix Q.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Notes
    -----
    ``zupgtr`` is intended to be used after a call to ``zhptrd``, which
    reduces a complex Hermitian matrix A to real symmetric tridiagonal
    form T by a unitary similarity transformation: A = QTQ^H. ``zhptrd``
    represents the unitary matrix Q as a product of n-1 elementary
    reflectors.

    This function may be used to generate Q explicitly as a square
    matrix.

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zupgtr'

    _ap = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ap'},
        ap,
        copy=False,
    )
    _tau = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'tau'},
        tau,
        copy=False,
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _q = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_n_py, _n_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _b_lapackeig.zupgtr(
        uplo,
        _n,
        _ap,
        _tau,
        _q,
    )
    _q_py = _q
    return _q_py

ZupmtrReturnData = _collections.namedtuple(
    'ZupmtrReturnData',
    ['ap', 'c'],
)

def zupmtr(side, uplo, trans, ap, tau, c):
    u"""
    Applies the unitary transformation matrix determined by ``zhptrd``.

    ``zupmtr`` multiplies an arbitrary complex matrix C by the complex
    unitary matrix Q which was determined by ``zhptrd`` when reducing a
    complex Hermitian matrix to tridiagonal form.

    For full information please refer to the NAG Library document for
    f08gu

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08guf.html

    Parameters
    ----------
    side : str, length 1
        Indicates how Q or Q^H is to be applied to C.

        `side` = 'L'

            Q or Q^H is applied to C from the left.

        `side` = 'R'

            Q or Q^H is applied to C from the right.

    uplo : str, length 1
        This **must** be the same argument `uplo` as supplied to
        ``zhptrd``.

    trans : str, length 1
        Indicates whether Q or Q^H is to be applied to C.

        `trans` = 'N'

            Q is applied to C.

        `trans` = 'C'

            Q^H is applied to C.

    ap : complex, array-like, shape (:)
        Note: the required length for this argument is determined as
        follows: if `side`='L': m*(m+1)/2; if `side`='R': n*(n+1)/2;
        otherwise: 0.

        Details of the vectors which define the elementary reflectors,
        as returned by ``zhptrd``.

    tau : complex, array-like, shape (:)
        Note: the required length for this argument is determined as
        follows: if `side`='L': max(1, m-1); if `side`='R': max(1, n-1);
        otherwise: 0.

        Further details of the elementary reflectors, as returned by
        ``zhptrd``.

    c : complex, array-like, shape (m, n)
        The m*n matrix C.

    Returns
    -------
    ap : complex, ndarray, shape (:)
        Is used as internal workspace prior to being restored and hence
        is unchanged.

    c : complex, ndarray, shape (m, n)
        `c` is overwritten by QC or Q^HC or CQ or CQ^H as specified by
        `side` and `trans`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'C'.

        (`errno` -4)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -5)
            On entry, error in parameter n.

            Constraint: n >= 0.

    Notes
    -----
    ``zupmtr`` is intended to be used after a call to ``zhptrd``, which
    reduces a complex Hermitian matrix A to real symmetric tridiagonal
    form T by a unitary similarity transformation: A = QTQ^H. ``zhptrd``
    represents the unitary matrix Q as a product of elementary
    reflectors.

    This function may be used to form one of the matrix products

        QC,Q^HC,CQ or CQ^H,

    overwriting the result on C (which may be any complex rectangular
    matrix).

    A common application of this function is to transform a matrix Z of
    eigenvectors of T to the matrix QZ of eigenvectors of A.

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zupmtr'

    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=True,
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _ap = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ap'},
        ap,
        copy=True,
    )
    _tau = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'tau'},
        tau,
        copy=False,
    )
    _b_lapackeig.zupmtr(
        side,
        uplo,
        trans,
        _m,
        _n,
        _ap,
        _tau,
        _c,
    )
    _ap_py = _ap
    _c_py = _c
    _return_data = ZupmtrReturnData(
        _ap_py,
        _c_py,
    )
    return _return_data

DsbevReturnData = _collections.namedtuple(
    'DsbevReturnData',
    ['ab', 'w', 'z'],
)

def dsbev(jobz, uplo, kd, ab):
    u"""
    Computes all eigenvalues and, optionally, eigenvectors of a real
    symmetric band matrix.

    ``dsbev`` computes all the eigenvalues and, optionally, all the
    eigenvectors of a real n*n symmetric band matrix A of bandwidth
    (2k_d+1).

    For full information please refer to the NAG Library document for
    f08ha

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08haf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

            Only eigenvalues are computed.

        `jobz` = 'V'

            Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangular part of A is stored.

        If `uplo` = 'L', the lower triangular part of A is stored.

    kd : int
        If `uplo` = 'U', the number of superdiagonals, k_d, of the
        matrix A.

        If `uplo` = 'L', the number of subdiagonals, k_d, of the matrix
        A.

    ab : float, array-like, shape (`kd`+1, n)
        The upper or lower triangle of the n*n symmetric band matrix A.

    Returns
    -------
    ab : float, ndarray, shape (`kd`+1, n)
        `ab` is overwritten by values generated during the reduction to
        tridiagonal form.

        The first superdiagonal or subdiagonal and the diagonal of the
        tridiagonal matrix T are returned in `ab` using the same storage
        format as described above.

    w : float, ndarray, shape (n)
        The eigenvalues in ascending order.

    z : float, ndarray, shape (:, :)
        If `jobz` = 'V', `z` contains the orthonormal eigenvectors of
        the matrix A, with the ith column of Z holding the eigenvector
        associated with `w`[i-1].

        If `jobz` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -4)
            On entry, error in parameter `kd`.

            Constraint: `kd` >= 0.

        (`errno` i > 0)
            The algorithm failed to converge; *<value>* off-diagonal
            elements of an intermediate tridiagonal form did not
            converge to zero.

    Notes
    -----
    The symmetric band matrix A is first reduced to tridiagonal form,
    using orthogonal similarity transformations, and then the QR
    algorithm is applied to the tridiagonal matrix to compute the
    eigenvalues and (optionally) the eigenvectors.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dsbev'

    _ab = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ab'},
        ab,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_ab, 'ab'),),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobz = _b_utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = _b_utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        _z_dim2 = _n_py
    else:
        _z_dim2 = 1
    exp_shape_z = (_z_dim1, _z_dim2)
    _z = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_z,
        sorder=_m_sorder,
    )
    _b_lapackeig.dsbev(
        jobz,
        uplo,
        _n,
        kd,
        _ab,
        _w,
        _z,
    )
    _ab_py = _ab
    _w_py = _w
    _z_py = _z
    _return_data = DsbevReturnData(
        _ab_py,
        _w_py,
        _z_py,
    )
    return _return_data

DsbevxReturnData = _collections.namedtuple(
    'DsbevxReturnData',
    ['ab', 'q', 'm', 'w', 'z', 'jfail'],
)

def dsbevx(jobz, erange, uplo, kd, ab, vl, vu, il, iu, abstol):
    u"""
    Computes selected eigenvalues and, optionally, eigenvectors of a
    real symmetric band matrix.

    ``dsbevx`` computes selected eigenvalues and, optionally,
    eigenvectors of a real n*n symmetric band matrix A of bandwidth
    (2k_d+1).
    Eigenvalues and eigenvectors can be selected by specifying either a
    range of values or a range of indices for the desired eigenvalues.

    For full information please refer to the NAG Library document for
    f08hb

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08hbf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

            Only eigenvalues are computed.

        `jobz` = 'V'

            Eigenvalues and eigenvectors are computed.

    erange : str, length 1
        If `erange` = 'A', all eigenvalues will be found.

        If `erange` = 'V', all eigenvalues in the half-open interval
        (`vl`,`vu`] will be found.

        If `erange` = 'I', the `il`th to `iu`th eigenvalues will be
        found.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangular part of A is stored.

        If `uplo` = 'L', the lower triangular part of A is stored.

    kd : int
        If `uplo` = 'U', the number of superdiagonals, k_d, of the
        matrix A.

        If `uplo` = 'L', the number of subdiagonals, k_d, of the matrix
        A.

    ab : float, array-like, shape (`kd`+1, n)
        The upper or lower triangle of the n*n symmetric band matrix A.

    vl : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    vu : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    il : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    iu : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    abstol : float
        The absolute error tolerance for the eigenvalues. An approximate
        eigenvalue is accepted as converged when it is determined to lie
        in an interval [a,b] of width less than or equal to

            `abstol`+epsilon max(|a|, |b|),

        where epsilon is the machine precision. If `abstol` is less than
        or equal to zero, then epsilon||T||_1 will be used in its place,
        where T is the tridiagonal matrix obtained by reducing A to
        tridiagonal form. Eigenvalues will be computed most accurately
        when `abstol` is set to twice the underflow threshold
        2*machine.real_safe(), not zero. If this function returns with
        `errno` > 0, indicating that some eigenvectors did not converge,
        try setting `abstol` to 2*machine.real_safe(). See Demmel and
        Kahan (1990).

    Returns
    -------
    ab : float, ndarray, shape (`kd`+1, n)
        `ab` is overwritten by values generated during the reduction to
        tridiagonal form.

        The first superdiagonal or subdiagonal and the diagonal of the
        tridiagonal matrix T are returned in `ab` using the same storage
        format as described above.

    q : float, ndarray, shape (:, :)
        If `jobz` = 'V', the n*n orthogonal matrix used in the reduction
        to tridiagonal form.

        If `jobz` = 'N', `q` is not referenced.

    m : int
        The total number of eigenvalues found. 0 <= `m` <= n.

        If `erange` = 'A', `m` = n.

        If `erange` = 'I', `m` = `iu`-`il`+1.

    w : float, ndarray, shape (n)
        The first `m` elements contain the selected eigenvalues in
        ascending order.

    z : float, ndarray, shape (:, :)
        If `jobz` = 'V', then

            if no exception or warning is raised, the first `m` columns
            of Z contain the orthonormal eigenvectors of the matrix A
            corresponding to the selected eigenvalues, with the ith
            column of Z holding the eigenvector associated with
            `w`[i-1];

            if an eigenvector fails to converge (`errno` > 0), then that
            column of Z contains the latest approximation to the
            eigenvector, and the index of the eigenvector is returned in
            `jfail`.

        If `jobz` = 'N', `z` is not referenced.

        **Note:** you must ensure that at least max(1, `m`) columns are
        supplied in the array `z`; if `erange` = 'V', the exact value of
        `m` is not known in advance and an upper bound of at least n
        must be used.

    jfail : int, ndarray, shape (n)
        If `jobz` = 'V', then

            if no exception or warning is raised, the first `m` elements
            of `jfail` are zero;

            if `errno` > 0, `jfail` contains the indices of the
            eigenvectors that failed to converge.

        If `jobz` = 'N', `jfail` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `erange`.

            Constraint: `erange` = 'A', 'V' or 'I'.

        (`errno` -3)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -5)
            On entry, error in parameter `kd`.

            Constraint: `kd` >= 0.

        (`errno` -11)
            On entry, error in parameter `vu`.

            Constraint: `vl` < `vu`.

        (`errno` -12)
            On entry, error in parameter `il`.

        (`errno` -13)
            On entry, error in parameter `iu`.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            The algorithm failed to converge; *<value>* eigenvectors did
            not converge. Their indices are stored in array `jfail`.

    Notes
    -----
    The symmetric band matrix A is first reduced to tridiagonal form,
    using orthogonal similarity transformations.
    The required eigenvalues and eigenvectors are then computed from the
    tridiagonal matrix; the method used depends upon whether all, or
    selected, eigenvalues and eigenvectors are required.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Demmel, J W and Kahan, W, 1990, `Accurate singular values of
    bidiagonal matrices`, SIAM J. Sci. Statist. Comput. (11), 873--912

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dsbevx'

    _ab = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ab'},
        ab,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_ab, 'ab'),),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _jobz = _b_utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = _b_utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py in ('V', 'v'):
        _q_dim1 = _n_py
    else:
        _q_dim1 = 1
    if _jobz_py in ('V', 'v'):
        _q_dim2 = _n_py
    else:
        _q_dim2 = 1
    exp_shape_q = (_q_dim1, _q_dim2)
    _q = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_q,
        sorder=_m_sorder,
    )
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _erange = _b_utils._EngineCharScalarType(
        erange, {'fun_name': fun_name, 'entity': 'erange'}, exp_chrlen=1,
    ).to_ctypes()
    _erange_py = _b_utils._EngineCharScalarType.to_py(_erange)
    _iu = _b_utils._EngineIntScalarType(
        iu, {'fun_name': fun_name, 'entity': 'iu'},
    ).to_ctypes()
    _iu_py = _b_utils._EngineIntScalarType.to_py(_iu)
    _il = _b_utils._EngineIntScalarType(
        il, {'fun_name': fun_name, 'entity': 'il'},
    ).to_ctypes()
    _il_py = _b_utils._EngineIntScalarType.to_py(_il)
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        if _erange_py in ('I', 'i'):
            _z_dim2 = _iu_py-_il_py+1
        else:
            _z_dim2 = _n_py
    else:
        _z_dim2 = 0
    exp_shape_z = (_z_dim1, _z_dim2)
    _z = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_z,
        sorder=_m_sorder,
    )
    _jfail = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        _return_scalar = _b_lapackeig.dsbevx(
            jobz,
            erange,
            uplo,
            _n,
            kd,
            _ab,
            _q,
            vl,
            vu,
            _il,
            _iu,
            abstol,
            _w,
            _z,
            _jfail,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _ab_py = _ab
    _q_py = _q
    _m_py = _return_scalar
    _w_py = _w
    if _jobz_py in ('V', 'v'):
        _z_dim2 = _m_py
    else:
        _z_dim2 = 0
    _z_py = _z[:, :_z_dim2]
    _jfail_py = _jfail
    _return_data = DsbevxReturnData(
        _ab_py,
        _q_py,
        _m_py,
        _w_py,
        _z_py,
        _jfail_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

DsbevdReturnData = _collections.namedtuple(
    'DsbevdReturnData',
    ['ab', 'w', 'z'],
)

def dsbevd(job, uplo, kd, ab):
    u"""
    Computes all eigenvalues and, optionally, all eigenvectors of real
    symmetric band matrix (divide-and-conquer or Pal--Walker--Kahan
    variant of the QL or QR algorithm).

    ``dsbevd`` computes all the eigenvalues and, optionally, all the
    eigenvectors of a real symmetric band matrix. If the eigenvectors
    are requested, then it uses a divide-and-conquer algorithm to
    compute eigenvalues and eigenvectors.
    However, if only eigenvalues are required, then it uses the
    Pal--Walker--Kahan variant of the QL or QR algorithm.

    For full information please refer to the NAG Library document for
    f08hc

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08hcf.html

    Parameters
    ----------
    job : str, length 1
        Indicates whether eigenvectors are computed.

        `job` = 'N'

            Only eigenvalues are computed.

        `job` = 'V'

            Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        Indicates whether the upper or lower triangular part of A is
        stored.

        `uplo` = 'U'

            The upper triangular part of A is stored.

        `uplo` = 'L'

            The lower triangular part of A is stored.

    kd : int
        If `uplo` = 'U', the number of superdiagonals, k_d, of the
        matrix A.

        If `uplo` = 'L', the number of subdiagonals, k_d, of the matrix
        A.

    ab : float, array-like, shape (`kd`+1, n)
        The upper or lower triangle of the n*n symmetric band matrix A.

    Returns
    -------
    ab : float, ndarray, shape (`kd`+1, n)
        `ab` is overwritten by values generated during the reduction to
        tridiagonal form.

        The first superdiagonal or subdiagonal and the diagonal of the
        tridiagonal matrix T are returned in `ab` using the same storage
        format as described above.

    w : float, ndarray, shape (n)
        The eigenvalues of the matrix A in ascending order.

    z : float, ndarray, shape (:, :)
        If `job` = 'V', `z` is overwritten by the orthogonal matrix Z
        which contains the eigenvectors of A. The ith column of Z
        contains the eigenvector which corresponds to the eigenvalue
        `w`[i-1].

        If `job` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -4)
            On entry, error in parameter `kd`.

            Constraint: `kd` >= 0.

        (`errno` i > 0)
            If errno = *<value>* and `job` = 'N', the algorithm failed
            to converge; *<value>* elements of an intermediate
            tridiagonal form did not converge to zero; if errno =
            *<value>* and `job` = 'V', then the algorithm failed to
            compute an eigenvalue while working on the submatrix lying
            in rows and column *<value>*/(n+1) through mod(*<value>*,
            n+1).

    Notes
    -----
    ``dsbevd`` computes all the eigenvalues and, optionally, all the
    eigenvectors of a real symmetric band matrix A. In other words, it
    can compute the spectral factorization of A as

        A = Z Lambda Z^T,

    where Lambda is a diagonal matrix whose diagonal elements are the
    eigenvalues lambda_i, and Z is the orthogonal matrix whose columns
    are the eigenvectors z_i.
    Thus

        Az_i = lambda_i z_i, i = 1,2,...,n.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dsbevd'

    _ab = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ab'},
        ab,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_ab, 'ab'),),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _job = _b_utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _job_py = _b_utils._EngineCharScalarType.to_py(_job)
    if _job_py in ('V', 'v'):
        _z_dim1 = _n_py
    elif _job_py in ('N', 'n'):
        _z_dim1 = 1
    else:
        _z_dim1 = 0
    if _job_py in ('V', 'v'):
        _z_dim2 = _n_py
    elif _job_py in ('N', 'n'):
        _z_dim2 = 1
    else:
        _z_dim2 = 0
    exp_shape_z = (_z_dim1, _z_dim2)
    _z = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_z,
        sorder=_m_sorder,
    )
    _b_lapackeig.dsbevd(
        job,
        uplo,
        _n,
        kd,
        _ab,
        _w,
        _z,
    )
    _ab_py = _ab
    _w_py = _w
    _z_py = _z
    _return_data = DsbevdReturnData(
        _ab_py,
        _w_py,
        _z_py,
    )
    return _return_data

DsbtrdReturnData = _collections.namedtuple(
    'DsbtrdReturnData',
    ['ab', 'd', 'e', 'q'],
)

def dsbtrd(vect, uplo, kd, ab, q=None):
    u"""
    Performs an orthogonal reduction of real symmetric band matrix to
    symmetric tridiagonal form.

    ``dsbtrd`` reduces a real symmetric band matrix to tridiagonal form.

    For full information please refer to the NAG Library document for
    f08he

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08hef.html

    Parameters
    ----------
    vect : str, length 1
        Indicates whether Q is to be returned.

        `vect` = 'V'

            Q is returned.

        `vect` = 'U'

            Q is updated (and the array `q` must contain a matrix on
            entry).

        `vect` = 'N'

            Q is not required.

    uplo : str, length 1
        Indicates whether the upper or lower triangular part of A is
        stored.

        `uplo` = 'U'

            The upper triangular part of A is stored.

        `uplo` = 'L'

            The lower triangular part of A is stored.

    kd : int
        If `uplo` = 'U', the number of superdiagonals, k_d, of the
        matrix A.

        If `uplo` = 'L', the number of subdiagonals, k_d, of the matrix
        A.

    ab : float, array-like, shape (`kd`+1, n)
        The upper or lower triangle of the n*n symmetric band matrix A.

    q : None or float, array-like, shape (:, :), optional
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `vect` in ('V', 'U'): max(1, n); if
        `vect`='N': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `vect` in ('V', 'U'): n; if
        `vect`='N': 1; otherwise: 0.

        If `vect` = 'U', `q` must contain the matrix formed in a
        previous stage of the reduction (for example, the reduction of a
        banded symmetric-definite generalized eigenproblem); otherwise
        `q` need not be set.

    Returns
    -------
    ab : float, ndarray, shape (`kd`+1, n)
        `ab` is overwritten by values generated during the reduction to
        tridiagonal form.

        The first superdiagonal or subdiagonal and the diagonal of the
        tridiagonal matrix T are returned in `ab` using the same storage
        format as described above.

    d : float, ndarray, shape (n)
        The diagonal elements of the tridiagonal matrix T.

    e : float, ndarray, shape (max(1, n-1))
        The off-diagonal elements of the tridiagonal matrix T.

    q : None or float, ndarray, shape (:, :)
        If `vect` = 'V' or 'U', the n*n matrix Q.

        If `vect` = 'N', `q` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `vect`.

            Constraint: `vect` = 'V', 'U' or 'N'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -4)
            On entry, error in parameter `kd`.

            Constraint: `kd` >= 0.

    Notes
    -----
    ``dsbtrd`` reduces a symmetric band matrix A to symmetric
    tridiagonal form T by an orthogonal similarity transformation:

        T = Q^TAQ.

    The orthogonal matrix Q is determined as a product of Givens
    rotation matrices, and may be formed explicitly by the function if
    required.

    The function uses a vectorizable form of the reduction, due to
    Kaufman (1984).

    References
    ----------
    Kaufman, L, 1984, `Banded eigenvalue solvers on vector machines`,
    ACM Trans. Math. Software (10), 73--86

    Parlett, B N, 1998, `The Symmetric Eigenvalue Problem`, SIAM,
    Philadelphia
    """

    fun_name = 'naginterfaces.library.lapackeig.dsbtrd'

    _ab = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ab'},
        ab,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _q = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'q'},
        q,
        copy=True,
        allow_none=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_ab, 'ab'), (_q, 'q')),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _d = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _e = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (max(1, _n_py-1),),
    )
    _vect = _b_utils._EngineCharScalarType(
        vect, {'fun_name': fun_name, 'entity': 'vect'}, exp_chrlen=1,
    ).to_ctypes()
    _vect_py = _b_utils._EngineCharScalarType.to_py(_vect)
    if _vect_py in ('U', 'u'):
        if q is None:
            raise _b_utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'q'},
                'must not be None when vect in (\'U\', \'u\')',
            )
    if q is None:
        if _vect_py in ('V', 'v', 'U', 'u'):
            _q_dim1 = max(1, _n_py)
        elif _vect_py in ('N', 'n'):
            _q_dim1 = 1
        else:
            _q_dim1 = 0
        if _vect_py in ('V', 'v', 'U', 'u'):
            _q_dim2 = _n_py
        elif _vect_py in ('N', 'n'):
            _q_dim2 = 1
        else:
            _q_dim2 = 0
        exp_shape_q = (_q_dim1, _q_dim2)
        _q = _b_utils._EngineFloat64ArrayType.empty_ndarray(
            exp_shape_q,
            sorder=_m_sorder,
        )
    _b_lapackeig.dsbtrd(
        vect,
        uplo,
        _n,
        kd,
        _ab,
        _d,
        _e,
        _q,
    )
    _ab_py = _ab
    if _vect_py not in ('N', 'n'):
        _q_py = _q
    else:
        _q_py = None
    _d_py = _d
    _e_py = _e
    _return_data = DsbtrdReturnData(
        _ab_py,
        _d_py,
        _e_py,
        _q_py,
    )
    return _return_data

ZhbevReturnData = _collections.namedtuple(
    'ZhbevReturnData',
    ['ab', 'w', 'z'],
)

def zhbev(jobz, uplo, n, kd, ab):
    u"""
    Computes all eigenvalues and, optionally, eigenvectors of a complex
    Hermitian band matrix.

    ``zhbev`` computes all the eigenvalues and, optionally, all the
    eigenvectors of a complex n*n Hermitian band matrix A of bandwidth
    (2k_d+1).

    For full information please refer to the NAG Library document for
    f08hn

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08hnf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

            Only eigenvalues are computed.

        `jobz` = 'V'

            Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangular part of A is stored.

        If `uplo` = 'L', the lower triangular part of A is stored.

    n : int
        n, the order of the matrix A.

    kd : int
        If `uplo` = 'U', the number of superdiagonals, k_d, of the
        matrix A.

        If `uplo` = 'L', the number of subdiagonals, k_d, of the matrix
        A.

    ab : complex, array-like, shape (`kd`+1, `n`)
        The upper or lower triangle of the n*n Hermitian band matrix A.

    Returns
    -------
    ab : complex, ndarray, shape (`kd`+1, `n`)
        `ab` is overwritten by values generated during the reduction to
        tridiagonal form.

        The first superdiagonal or subdiagonal and the diagonal of the
        tridiagonal matrix T are returned in `ab` using the same storage
        format as described above.

    w : float, ndarray, shape (`n`)
        The eigenvalues in ascending order.

    z : complex, ndarray, shape (:, :)
        If `jobz` = 'V', `z` contains the orthonormal eigenvectors of
        the matrix A, with the ith column of Z holding the eigenvector
        associated with `w`[i-1].

        If `jobz` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `kd`.

            Constraint: `kd` >= 0.

        (`errno` i > 0)
            The algorithm failed to converge; *<value>* off-diagonal
            elements of an intermediate tridiagonal form did not
            converge to zero.

    Notes
    -----
    The Hermitian band matrix A is first reduced to real tridiagonal
    form, using unitary similarity transformations, and then the QR
    algorithm is applied to the tridiagonal matrix to compute the
    eigenvalues and (optionally) the eigenvectors.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zhbev'

    _ab = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ab'},
        ab,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_ab, 'ab'),),
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobz = _b_utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = _b_utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        _z_dim2 = _n_py
    else:
        _z_dim2 = 1
    exp_shape_z = (_z_dim1, _z_dim2)
    _z = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_z,
        sorder=_m_sorder,
    )
    _b_lapackeig.zhbev(
        jobz,
        uplo,
        _n,
        kd,
        _ab,
        _w,
        _z,
    )
    _ab_py = _ab
    _w_py = _w
    _z_py = _z
    _return_data = ZhbevReturnData(
        _ab_py,
        _w_py,
        _z_py,
    )
    return _return_data

ZhbevxReturnData = _collections.namedtuple(
    'ZhbevxReturnData',
    ['ab', 'q', 'm', 'w', 'z', 'jfail'],
)

def zhbevx(jobz, erange, uplo, kd, ab, vl, vu, il, iu, abstol):
    u"""
    Computes selected eigenvalues and, optionally, eigenvectors of a
    complex Hermitian band matrix.

    ``zhbevx`` computes selected eigenvalues and, optionally,
    eigenvectors of a complex n*n Hermitian band matrix A of bandwidth
    (2k_d+1).
    Eigenvalues and eigenvectors can be selected by specifying either a
    range of values or a range of indices for the desired eigenvalues.

    For full information please refer to the NAG Library document for
    f08hp

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08hpf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

            Only eigenvalues are computed.

        `jobz` = 'V'

            Eigenvalues and eigenvectors are computed.

    erange : str, length 1
        If `erange` = 'A', all eigenvalues will be found.

        If `erange` = 'V', all eigenvalues in the half-open interval
        (`vl`,`vu`] will be found.

        If `erange` = 'I', the `il`th to `iu`th eigenvalues will be
        found.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangular part of A is stored.

        If `uplo` = 'L', the lower triangular part of A is stored.

    kd : int
        If `uplo` = 'U', the number of superdiagonals, k_d, of the
        matrix A.

        If `uplo` = 'L', the number of subdiagonals, k_d, of the matrix
        A.

    ab : complex, array-like, shape (`kd`+1, n)
        The upper or lower triangle of the n*n Hermitian band matrix A.

    vl : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    vu : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    il : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    iu : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    abstol : float
        The absolute error tolerance for the eigenvalues. An approximate
        eigenvalue is accepted as converged when it is determined to lie
        in an interval [a,b] of width less than or equal to

            `abstol`+epsilon max(|a|, |b|),

        where epsilon is the machine precision. If `abstol` is less than
        or equal to zero, then epsilon||T||_1 will be used in its place,
        where T is the tridiagonal matrix obtained by reducing A to
        tridiagonal form. Eigenvalues will be computed most accurately
        when `abstol` is set to twice the underflow threshold
        2*machine.real_safe(), not zero. If this function returns with
        `errno` > 0, indicating that some eigenvectors did not converge,
        try setting `abstol` to 2*machine.real_safe(). See Demmel and
        Kahan (1990).

    Returns
    -------
    ab : complex, ndarray, shape (`kd`+1, n)
        `ab` is overwritten by values generated during the reduction to
        tridiagonal form.

        The first superdiagonal or subdiagonal and the diagonal of the
        tridiagonal matrix T are returned in `ab` using the same storage
        format as described above.

    q : complex, ndarray, shape (:, :)
        If `jobz` = 'V', the n*n unitary matrix used in the reduction to
        tridiagonal form.

        If `jobz` = 'N', `q` is not referenced.

    m : int
        The total number of eigenvalues found. 0 <= `m` <= n.

        If `erange` = 'A', `m` = n.

        If `erange` = 'I', `m` = `iu`-`il`+1.

    w : float, ndarray, shape (n)
        The first `m` elements contain the selected eigenvalues in
        ascending order.

    z : complex, ndarray, shape (:, :)
        If `jobz` = 'V', then

            if no exception or warning is raised, the first `m` columns
            of Z contain the orthonormal eigenvectors of the matrix A
            corresponding to the selected eigenvalues, with the ith
            column of Z holding the eigenvector associated with
            `w`[i-1];

            if an eigenvector fails to converge (`errno` > 0), then that
            column of Z contains the latest approximation to the
            eigenvector, and the index of the eigenvector is returned in
            `jfail`.

        If `jobz` = 'N', `z` is not referenced.

        **Note:** you must ensure that at least max(1, `m`) columns are
        supplied in the array `z`; if `erange` = 'V', the exact value of
        `m` is not known in advance and an upper bound of at least n
        must be used.

    jfail : int, ndarray, shape (n)
        If `jobz` = 'V', then

            if no exception or warning is raised, the first `m` elements
            of `jfail` are zero;

            if `errno` > 0, `jfail` contains the indices of the
            eigenvectors that failed to converge.

        If `jobz` = 'N', `jfail` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `erange`.

            Constraint: `erange` = 'A', 'V' or 'I'.

        (`errno` -3)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -5)
            On entry, error in parameter `kd`.

            Constraint: `kd` >= 0.

        (`errno` -11)
            On entry, error in parameter `vu`.

            Constraint: `vl` < `vu`.

        (`errno` -12)
            On entry, error in parameter `il`.

        (`errno` -13)
            On entry, error in parameter `iu`.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            The algorithm failed to converge; *<value>* eigenvectors did
            not converge. Their indices are stored in array `jfail`.

    Notes
    -----
    The Hermitian band matrix A is first reduced to real tridiagonal
    form, using unitary similarity transformations.
    The required eigenvalues and eigenvectors are then computed from the
    tridiagonal matrix; the method used depends upon whether all, or
    selected, eigenvalues and eigenvectors are required.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Demmel, J W and Kahan, W, 1990, `Accurate singular values of
    bidiagonal matrices`, SIAM J. Sci. Statist. Comput. (11), 873--912

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zhbevx'

    _ab = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ab'},
        ab,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_ab, 'ab'),),
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _jobz = _b_utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = _b_utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py in ('V', 'v'):
        _q_dim1 = _n_py
    else:
        _q_dim1 = 1
    if _jobz_py in ('V', 'v'):
        _q_dim2 = _n_py
    else:
        _q_dim2 = 1
    exp_shape_q = (_q_dim1, _q_dim2)
    _q = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_q,
        sorder=_m_sorder,
    )
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _erange = _b_utils._EngineCharScalarType(
        erange, {'fun_name': fun_name, 'entity': 'erange'}, exp_chrlen=1,
    ).to_ctypes()
    _erange_py = _b_utils._EngineCharScalarType.to_py(_erange)
    _iu = _b_utils._EngineIntScalarType(
        iu, {'fun_name': fun_name, 'entity': 'iu'},
    ).to_ctypes()
    _iu_py = _b_utils._EngineIntScalarType.to_py(_iu)
    _il = _b_utils._EngineIntScalarType(
        il, {'fun_name': fun_name, 'entity': 'il'},
    ).to_ctypes()
    _il_py = _b_utils._EngineIntScalarType.to_py(_il)
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        if _erange_py in ('I', 'i'):
            _z_dim2 = _iu_py-_il_py+1
        else:
            _z_dim2 = _n_py
    else:
        _z_dim2 = 0
    exp_shape_z = (_z_dim1, _z_dim2)
    _z = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_z,
        sorder=_m_sorder,
    )
    _jfail = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        _return_scalar = _b_lapackeig.zhbevx(
            jobz,
            erange,
            uplo,
            _n,
            kd,
            _ab,
            _q,
            vl,
            vu,
            _il,
            _iu,
            abstol,
            _w,
            _z,
            _jfail,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _ab_py = _ab
    _q_py = _q
    _m_py = _return_scalar
    _w_py = _w
    if _jobz_py in ('V', 'v'):
        _z_dim2 = _m_py
    else:
        _z_dim2 = 0
    _z_py = _z[:, :_z_dim2]
    _jfail_py = _jfail
    _return_data = ZhbevxReturnData(
        _ab_py,
        _q_py,
        _m_py,
        _w_py,
        _z_py,
        _jfail_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

ZhbevdReturnData = _collections.namedtuple(
    'ZhbevdReturnData',
    ['ab', 'w', 'z'],
)

def zhbevd(job, uplo, kd, ab):
    u"""
    Computes all eigenvalues and, optionally, all eigenvectors of
    complex Hermitian band matrix (divide-and-conquer).

    ``zhbevd`` computes all the eigenvalues and, optionally, all the
    eigenvectors of a complex Hermitian band matrix. If the eigenvectors
    are requested, then it uses a divide-and-conquer algorithm to
    compute eigenvalues and eigenvectors.
    However, if only eigenvalues are required, then it uses the
    Pal--Walker--Kahan variant of the QL or QR algorithm.

    For full information please refer to the NAG Library document for
    f08hq

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08hqf.html

    Parameters
    ----------
    job : str, length 1
        Indicates whether eigenvectors are computed.

        `job` = 'N'

            Only eigenvalues are computed.

        `job` = 'V'

            Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        Indicates whether the upper or lower triangular part of A is
        stored.

        `uplo` = 'U'

            The upper triangular part of A is stored.

        `uplo` = 'L'

            The lower triangular part of A is stored.

    kd : int
        If `uplo` = 'U', the number of superdiagonals, k_d, of the
        matrix A.

        If `uplo` = 'L', the number of subdiagonals, k_d, of the matrix
        A.

    ab : complex, array-like, shape (`kd`+1, n)
        The upper or lower triangle of the n*n Hermitian band matrix A.

    Returns
    -------
    ab : complex, ndarray, shape (`kd`+1, n)
        `ab` is overwritten by values generated during the reduction to
        tridiagonal form.

        The first superdiagonal or subdiagonal and the diagonal of the
        tridiagonal matrix T are returned in `ab` using the same storage
        format as described above.

    w : float, ndarray, shape (n)
        The eigenvalues of the matrix A in ascending order.

    z : complex, ndarray, shape (:, :)
        If `job` = 'V', `z` is overwritten by the unitary matrix Z which
        contains the eigenvectors of A. The ith column of Z contains the
        eigenvector which corresponds to the eigenvalue `w`[i-1].

        If `job` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -4)
            On entry, error in parameter `kd`.

            Constraint: `kd` >= 0.

        (`errno` i > 0)
            If errno = *<value>* and `job` = 'N', the algorithm failed
            to converge; *<value>* elements of an intermediate
            tridiagonal form did not converge to zero; if errno =
            *<value>* and `job` = 'V', then the algorithm failed to
            compute an eigenvalue while working on the submatrix lying
            in rows and column *<value>*/(n+1) through mod(*<value>*,
            n+1).

    Notes
    -----
    ``zhbevd`` computes all the eigenvalues and, optionally, all the
    eigenvectors of a complex Hermitian band matrix A. In other words,
    it can compute the spectral factorization of A as

        A = Z Lambda Z^H,

    where Lambda is a real diagonal matrix whose diagonal elements are
    the eigenvalues lambda_i, and Z is the (complex) unitary matrix
    whose columns are the eigenvectors z_i.
    Thus

        Az_i = lambda_i z_i, i = 1,2,...,n.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zhbevd'

    _ab = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ab'},
        ab,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_ab, 'ab'),),
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _job = _b_utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _job_py = _b_utils._EngineCharScalarType.to_py(_job)
    if _job_py in ('V', 'v'):
        _z_dim1 = _n_py
    elif _job_py in ('N', 'n'):
        _z_dim1 = 1
    else:
        _z_dim1 = 0
    if _job_py in ('V', 'v'):
        _z_dim2 = _n_py
    elif _job_py in ('N', 'n'):
        _z_dim2 = 1
    else:
        _z_dim2 = 0
    exp_shape_z = (_z_dim1, _z_dim2)
    _z = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_z,
        sorder=_m_sorder,
    )
    _b_lapackeig.zhbevd(
        job,
        uplo,
        _n,
        kd,
        _ab,
        _w,
        _z,
    )
    _ab_py = _ab
    _w_py = _w
    _z_py = _z
    _return_data = ZhbevdReturnData(
        _ab_py,
        _w_py,
        _z_py,
    )
    return _return_data

ZhbtrdReturnData = _collections.namedtuple(
    'ZhbtrdReturnData',
    ['ab', 'd', 'e', 'q'],
)

def zhbtrd(vect, uplo, kd, ab, q=None):
    u"""
    Performs a unitary reduction of complex Hermitian band matrix to
    real symmetric tridiagonal form.

    ``zhbtrd`` reduces a complex Hermitian band matrix to tridiagonal
    form.

    For full information please refer to the NAG Library document for
    f08hs

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08hsf.html

    Parameters
    ----------
    vect : str, length 1
        Indicates whether Q is to be returned.

        `vect` = 'V'

            Q is returned.

        `vect` = 'U'

            Q is updated (and the array `q` must contain a matrix on
            entry).

        `vect` = 'N'

            Q is not required.

    uplo : str, length 1
        Indicates whether the upper or lower triangular part of A is
        stored.

        `uplo` = 'U'

            The upper triangular part of A is stored.

        `uplo` = 'L'

            The lower triangular part of A is stored.

    kd : int
        If `uplo` = 'U', the number of superdiagonals, k_d, of the
        matrix A.

        If `uplo` = 'L', the number of subdiagonals, k_d, of the matrix
        A.

    ab : complex, array-like, shape (`kd`+1, n)
        The upper or lower triangle of the n*n Hermitian band matrix A.

    q : None or complex, array-like, shape (:, :), optional
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `vect` in ('V', 'U'): max(1, n); if
        `vect`='N': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `vect` in ('V', 'U'): n; if
        `vect`='N': 1; otherwise: 0.

        If `vect` = 'U', `q` must contain the matrix formed in a
        previous stage of the reduction (for example, the reduction of a
        banded Hermitian-definite generalized eigenproblem); otherwise
        `q` need not be set.

    Returns
    -------
    ab : complex, ndarray, shape (`kd`+1, n)
        `ab` is overwritten by values generated during the reduction to
        tridiagonal form.

        The first superdiagonal or subdiagonal and the diagonal of the
        tridiagonal matrix T are returned in `ab` using the same storage
        format as described above.

    d : float, ndarray, shape (n)
        The diagonal elements of the tridiagonal matrix T.

    e : float, ndarray, shape (max(1, n-1))
        The off-diagonal elements of the tridiagonal matrix T.

    q : None or complex, ndarray, shape (:, :)
        If `vect` = 'V' or 'U', the n*n matrix Q.

        If `vect` = 'N', `q` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `vect`.

            Constraint: `vect` = 'V', 'U' or 'N'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -4)
            On entry, error in parameter `kd`.

            Constraint: `kd` >= 0.

    Notes
    -----
    ``zhbtrd`` reduces a Hermitian band matrix A to real symmetric
    tridiagonal form T by a unitary similarity transformation:

        T = Q^HAQ.

    The unitary matrix Q is determined as a product of Givens rotation
    matrices, and may be formed explicitly by the function if required.

    The function uses a vectorizable form of the reduction, due to
    Kaufman (1984).

    References
    ----------
    Kaufman, L, 1984, `Banded eigenvalue solvers on vector machines`,
    ACM Trans. Math. Software (10), 73--86

    Parlett, B N, 1998, `The Symmetric Eigenvalue Problem`, SIAM,
    Philadelphia
    """

    fun_name = 'naginterfaces.library.lapackeig.zhbtrd'

    _ab = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ab'},
        ab,
        copy=True,
    )
    _q = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'q'},
        q,
        copy=True,
        allow_none=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_ab, 'ab'), (_q, 'q')),
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _d = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _e = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (max(1, _n_py-1),),
    )
    _vect = _b_utils._EngineCharScalarType(
        vect, {'fun_name': fun_name, 'entity': 'vect'}, exp_chrlen=1,
    ).to_ctypes()
    _vect_py = _b_utils._EngineCharScalarType.to_py(_vect)
    if _vect_py in ('U', 'u'):
        if q is None:
            raise _b_utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'q'},
                'must not be None when vect in (\'U\', \'u\')',
            )
    if q is None:
        if _vect_py in ('V', 'v', 'U', 'u'):
            _q_dim1 = max(1, _n_py)
        elif _vect_py in ('N', 'n'):
            _q_dim1 = 1
        else:
            _q_dim1 = 0
        if _vect_py in ('V', 'v', 'U', 'u'):
            _q_dim2 = _n_py
        elif _vect_py in ('N', 'n'):
            _q_dim2 = 1
        else:
            _q_dim2 = 0
        exp_shape_q = (_q_dim1, _q_dim2)
        _q = _b_utils._EngineComplex128ArrayType.empty_ndarray(
            exp_shape_q,
            sorder=_m_sorder,
        )
    _b_lapackeig.zhbtrd(
        vect,
        uplo,
        _n,
        kd,
        _ab,
        _d,
        _e,
        _q,
    )
    _ab_py = _ab
    if _vect_py not in ('N', 'n'):
        _q_py = _q
    else:
        _q_py = None
    _d_py = _d
    _e_py = _e
    _return_data = ZhbtrdReturnData(
        _ab_py,
        _d_py,
        _e_py,
        _q_py,
    )
    return _return_data

DstevReturnData = _collections.namedtuple(
    'DstevReturnData',
    ['d', 'e', 'z'],
)

def dstev(jobz, d, e):
    u"""
    Computes all eigenvalues and, optionally, eigenvectors of a real
    symmetric tridiagonal matrix.

    ``dstev`` computes all the eigenvalues and, optionally, all the
    eigenvectors of a real n*n symmetric tridiagonal matrix A.

    For full information please refer to the NAG Library document for
    f08ja

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08jaf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

            Only eigenvalues are computed.

        `jobz` = 'V'

            Eigenvalues and eigenvectors are computed.

    d : float, array-like, shape (n)
        The n diagonal elements of the tridiagonal matrix A.

    e : float, array-like, shape (n-1)
        The (n-1) subdiagonal elements of the tridiagonal matrix A.

    Returns
    -------
    d : float, ndarray, shape (n)
        If no exception or warning is raised, the eigenvalues in
        ascending order.

    e : float, ndarray, shape (n-1)
        The contents of `e` are destroyed.

    z : float, ndarray, shape (:, :)
        If `jobz` = 'V', then if no exception or warning is raised, `z`
        contains the orthonormal eigenvectors of the matrix A, with the
        ith column of Z holding the eigenvector associated with
        `d`[i-1].

        If `jobz` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` i > 0)
            The algorithm failed to converge; *<value>* off-diagonal
            elements of `e` did not converge to zero.

    Notes
    -----
    ``dstev`` computes all the eigenvalues and, optionally, all the
    eigenvectors of A using a combination of the QR and QL algorithms,
    with an implicit shift.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dstev'

    _d = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'd'},
        d,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_d, {'fun_name': fun_name, 'entity': 'd'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _e = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'e'},
        e,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _jobz = _b_utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = _b_utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        _z_dim2 = _n_py
    else:
        _z_dim2 = 1
    exp_shape_z = (_z_dim1, _z_dim2)
    _z = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_z,
        sorder=_b_utils._EngineData.default_sorder,
    )
    _b_lapackeig.dstev(
        jobz,
        _n,
        _d,
        _e,
        _z,
    )
    _d_py = _d
    _e_py = _e
    _z_py = _z
    _return_data = DstevReturnData(
        _d_py,
        _e_py,
        _z_py,
    )
    return _return_data

DstevxReturnData = _collections.namedtuple(
    'DstevxReturnData',
    ['d', 'e', 'm', 'w', 'z', 'jfail'],
)

def dstevx(jobz, erange, d, e, vl, vu, il, iu, abstol):
    u"""
    Computes selected eigenvalues and, optionally, eigenvectors of a
    real symmetric tridiagonal matrix.

    ``dstevx`` computes selected eigenvalues and, optionally,
    eigenvectors of a real symmetric tridiagonal matrix A.
    Eigenvalues and eigenvectors can be selected by specifying either a
    range of values or a range of indices for the desired eigenvalues.

    For full information please refer to the NAG Library document for
    f08jb

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08jbf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

            Only eigenvalues are computed.

        `jobz` = 'V'

            Eigenvalues and eigenvectors are computed.

    erange : str, length 1
        If `erange` = 'A', all eigenvalues will be found.

        If `erange` = 'V', all eigenvalues in the half-open interval
        (`vl`,`vu`] will be found.

        If `erange` = 'I', the `il`th to `iu`th eigenvalues will be
        found.

    d : float, array-like, shape (n)
        The n diagonal elements of the tridiagonal matrix A.

    e : float, array-like, shape (max(1, n-1))
        The (n-1) subdiagonal elements of the tridiagonal matrix A.

    vl : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    vu : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    il : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    iu : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    abstol : float
        The absolute error tolerance for the eigenvalues. An approximate
        eigenvalue is accepted as converged when it is determined to lie
        in an interval [a,b] of width less than or equal to

            `abstol`+epsilon max(|a|, |b|),

        where epsilon is the machine precision. If `abstol` is less than
        or equal to zero, then epsilon||A||_1 will be used in its place.
        Eigenvalues will be computed most accurately when `abstol` is
        set to twice the underflow threshold 2*machine.real_safe(), not
        zero. If this function returns with `errno` > 0, indicating that
        some eigenvectors did not converge, try setting `abstol` to
        2*machine.real_safe(). See Demmel and Kahan (1990).

    Returns
    -------
    d : float, ndarray, shape (n)
        May be multiplied by a constant factor chosen to avoid
        over/underflow in computing the eigenvalues.

    e : float, ndarray, shape (max(1, n-1))
        May be multiplied by a constant factor chosen to avoid
        over/underflow in computing the eigenvalues.

    m : int
        The total number of eigenvalues found. 0 <= `m` <= n.

        If `erange` = 'A', `m` = n.

        If `erange` = 'I', `m` = `iu`-`il`+1.

    w : float, ndarray, shape (n)
        The first `m` elements contain the selected eigenvalues in
        ascending order.

    z : float, ndarray, shape (:, :)
        If `jobz` = 'V', then

            if no exception or warning is raised, the first `m` columns
            of Z contain the orthonormal eigenvectors of the matrix A
            corresponding to the selected eigenvalues, with the ith
            column of Z holding the eigenvector associated with
            `w`[i-1];

            if an eigenvector fails to converge (`errno` > 0), then that
            column of Z contains the latest approximation to the
            eigenvector, and the index of the eigenvector is returned in
            `jfail`.

        If `jobz` = 'N', `z` is not referenced.

        **Note:** you must ensure that at least max(1, `m`) columns are
        supplied in the array `z`; if `erange` = 'V', the exact value of
        `m` is not known in advance and an upper bound of at least n
        must be used.

    jfail : int, ndarray, shape (n)
        If `jobz` = 'V', then

            if no exception or warning is raised, the first `m` elements
            of `jfail` are zero;

            if `errno` > 0, `jfail` contains the indices of the
            eigenvectors that failed to converge.

        If `jobz` = 'N', `jfail` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `erange`.

            Constraint: `erange` = 'A', 'V' or 'I'.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -7)
            On entry, error in parameter `vu`.

            Constraint: `vl` < `vu`.

        (`errno` -8)
            On entry, error in parameter `il`.

        (`errno` -9)
            On entry, error in parameter `iu`.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            The algorithm failed to converge; *<value>* eigenvectors did
            not converge. Their indices are stored in array `jfail`.

    Notes
    -----
    ``dstevx`` computes the required eigenvalues and eigenvectors of A
    by reducing the tridiagonal matrix to diagonal form using the QR
    algorithm.
    Bisection is used to determine selected eigenvalues.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Demmel, J W and Kahan, W, 1990, `Accurate singular values of
    bidiagonal matrices`, SIAM J. Sci. Statist. Comput. (11), 873--912

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dstevx'

    _d = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'd'},
        d,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_d, {'fun_name': fun_name, 'entity': 'd'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _e = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'e'},
        e,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobz = _b_utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = _b_utils._EngineCharScalarType.to_py(_jobz)
    _erange = _b_utils._EngineCharScalarType(
        erange, {'fun_name': fun_name, 'entity': 'erange'}, exp_chrlen=1,
    ).to_ctypes()
    _erange_py = _b_utils._EngineCharScalarType.to_py(_erange)
    _iu = _b_utils._EngineIntScalarType(
        iu, {'fun_name': fun_name, 'entity': 'iu'},
    ).to_ctypes()
    _iu_py = _b_utils._EngineIntScalarType.to_py(_iu)
    _il = _b_utils._EngineIntScalarType(
        il, {'fun_name': fun_name, 'entity': 'il'},
    ).to_ctypes()
    _il_py = _b_utils._EngineIntScalarType.to_py(_il)
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        if _erange_py in ('I', 'i'):
            _z_dim2 = _iu_py-_il_py+1
        else:
            _z_dim2 = _n_py
    else:
        _z_dim2 = 0
    exp_shape_z = (_z_dim1, _z_dim2)
    _z = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_z,
        sorder=_b_utils._EngineData.default_sorder,
    )
    _jfail = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        _return_scalar = _b_lapackeig.dstevx(
            jobz,
            erange,
            _n,
            _d,
            _e,
            vl,
            vu,
            _il,
            _iu,
            abstol,
            _w,
            _z,
            _jfail,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _d_py = _d
    _e_py = _e
    _m_py = _return_scalar
    _w_py = _w
    if _jobz_py in ('V', 'v'):
        _z_dim2 = _m_py
    else:
        _z_dim2 = 0
    _z_py = _z[:, :_z_dim2]
    _jfail_py = _jfail
    _return_data = DstevxReturnData(
        _d_py,
        _e_py,
        _m_py,
        _w_py,
        _z_py,
        _jfail_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

DstevdReturnData = _collections.namedtuple(
    'DstevdReturnData',
    ['d', 'e', 'z'],
)

def dstevd(job, d, e):
    u"""
    Computes all eigenvalues and, optionally, all eigenvectors of real
    symmetric tridiagonal matrix (divide-and-conquer).

    ``dstevd`` computes all the eigenvalues and, optionally, all the
    eigenvectors of a real symmetric tridiagonal matrix. If the
    eigenvectors are requested, then it uses a divide-and-conquer
    algorithm to compute eigenvalues and eigenvectors.
    However, if only eigenvalues are required, then it uses the
    Pal--Walker--Kahan variant of the QL or QR algorithm.

    For full information please refer to the NAG Library document for
    f08jc

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08jcf.html

    Parameters
    ----------
    job : str, length 1
        Indicates whether eigenvectors are computed.

        `job` = 'N'

            Only eigenvalues are computed.

        `job` = 'V'

            Eigenvalues and eigenvectors are computed.

    d : float, array-like, shape (n)
        The n diagonal elements of the tridiagonal matrix T.

    e : float, array-like, shape (n)
        The n-1 off-diagonal elements of the tridiagonal matrix T. The
        nth element of this array is used as workspace.

    Returns
    -------
    d : float, ndarray, shape (n)
        The eigenvalues of the matrix T in ascending order.

    e : float, ndarray, shape (n)
        `e` is overwritten with intermediate results.

    z : float, ndarray, shape (:, :)
        If `job` = 'V', `z` is overwritten by the orthogonal matrix Z
        which contains the eigenvectors of T.

        If `job` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` i > 0)
            The algorithm failed to converge; *<value>* off-diagonal
            elements of `e` did not converge to zero.

    Notes
    -----
    ``dstevd`` computes all the eigenvalues and, optionally, all the
    eigenvectors of a real symmetric tridiagonal matrix T. In other
    words, it can compute the spectral factorization of T as

        T = Z Lambda Z^T,

    where Lambda is a diagonal matrix whose diagonal elements are the
    eigenvalues lambda_i, and Z is the orthogonal matrix whose columns
    are the eigenvectors z_i.
    Thus

        Tz_i = lambda_i z_i, i = 1,2,...,n.

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dstevd'

    _d = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'd'},
        d,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_d, {'fun_name': fun_name, 'entity': 'd'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _e = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'e'},
        e,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _job = _b_utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _job_py = _b_utils._EngineCharScalarType.to_py(_job)
    if _job_py in ('V', 'v'):
        _z_dim1 = _n_py
    elif _job_py in ('N', 'n'):
        _z_dim1 = 1
    else:
        _z_dim1 = 0
    if _job_py in ('V', 'v'):
        _z_dim2 = _n_py
    elif _job_py in ('N', 'n'):
        _z_dim2 = 1
    else:
        _z_dim2 = 0
    exp_shape_z = (_z_dim1, _z_dim2)
    _z = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_z,
        sorder=_b_utils._EngineData.default_sorder,
    )
    _b_lapackeig.dstevd(
        job,
        _n,
        _d,
        _e,
        _z,
    )
    _d_py = _d
    _e_py = _e
    _z_py = _z
    _return_data = DstevdReturnData(
        _d_py,
        _e_py,
        _z_py,
    )
    return _return_data

DstevrReturnData = _collections.namedtuple(
    'DstevrReturnData',
    ['d', 'e', 'm', 'w', 'z', 'isuppz'],
)

def dstevr(jobz, erange, d, e, vl, vu, il, iu, abstol):
    u"""
    Computes selected eigenvalues and, optionally, eigenvectors of a
    real symmetric tridiagonal matrix (Relatively Robust
    Representations).

    ``dstevr`` computes selected eigenvalues and, optionally,
    eigenvectors of a real n*n symmetric tridiagonal matrix T.
    Eigenvalues and eigenvectors can be selected by specifying either a
    range of values or a range of indices for the desired eigenvalues.

    For full information please refer to the NAG Library document for
    f08jd

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08jdf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

            Only eigenvalues are computed.

        `jobz` = 'V'

            Eigenvalues and eigenvectors are computed.

    erange : str, length 1
        If `erange` = 'A', all eigenvalues will be found.

        If `erange` = 'V', all eigenvalues in the half-open interval
        (`vl`,`vu`] will be found.

        If `erange` = 'I', the `il`th to `iu`th eigenvalues will be
        found.

    d : float, array-like, shape (n)
        The n diagonal elements of the tridiagonal matrix T.

    e : float, array-like, shape (n-1)
        The (n-1) subdiagonal elements of the tridiagonal matrix T.

    vl : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    vu : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    il : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    iu : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    abstol : float
        The absolute error tolerance for the eigenvalues. An approximate
        eigenvalue is accepted as converged when it is determined to lie
        in an interval [a,b] of width less than or equal to

            `abstol`+epsilon max(|a|, |b|),

        where epsilon is the machine precision. If `abstol` is less than
        or equal to zero, then epsilon||T||_1 will be used in its place.
        See Demmel and Kahan (1990).

        If high relative accuracy is important, set `abstol` to
        machine.real_safe(), although doing so does not currently
        guarantee that eigenvalues are computed to high relative
        accuracy.

        See Barlow and Demmel (1990) for a discussion of which matrices
        can define their eigenvalues to high relative accuracy.

    Returns
    -------
    d : float, ndarray, shape (n)
        May be multiplied by a constant factor chosen to avoid
        over/underflow in computing the eigenvalues.

    e : float, ndarray, shape (n-1)
        May be multiplied by a constant factor chosen to avoid
        over/underflow in computing the eigenvalues.

    m : int
        The total number of eigenvalues found. 0 <= `m` <= n.

        If `erange` = 'A', `m` = n.

        If `erange` = 'I', `m` = `iu`-`il`+1.

    w : float, ndarray, shape (n)
        The first `m` elements contain the selected eigenvalues in
        ascending order.

    z : float, ndarray, shape (:, :)
        If `jobz` = 'V', the first `m` columns of Z contain the
        orthonormal eigenvectors of the matrix A corresponding to the
        selected eigenvalues, with the ith column of Z holding the
        eigenvector associated with `w`[i-1].

        If `jobz` = 'N', `z` is not referenced.

        **Note:** you must ensure that at least max(1, `m`) columns are
        supplied in the array `z`; if `erange` = 'V', the exact value of
        `m` is not known in advance and an upper bound of at least n
        must be used.

    isuppz : int, ndarray, shape (:)
        The support of the eigenvectors in `z`, i.e., the indices
        indicating the nonzero elements in `z`. The ith eigenvector is
        nonzero only in elements `isuppz`[2*i-2] through
        `isuppz`[2*i-1]. Implemented only for `erange` = 'A' or `erange`
        = 'I' and `iu`-`il` = n-1.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `erange`.

            Constraint: `erange` = 'A', 'V' or 'I'.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -7)
            On entry, error in parameter `vu`.

            Constraint: `vl` < `vu`.

        (`errno` -8)
            On entry, error in parameter `il`.

        (`errno` -9)
            On entry, error in parameter `iu`.

        (`errno` i > 0)
            An internal error has occurred in this function. Please
            refer to errno in ``dstebz``.

    Notes
    -----
    Whenever possible ``dstevr`` computes the eigenspectrum using
    Relatively Robust Representations. ``dstevr`` computes eigenvalues
    by the dqds algorithm, while orthogonal eigenvectors are computed
    from various 'good' LDL^T representations (also known as Relatively
    Robust Representations).
    Gram--Schmidt orthogonalization is avoided as far as possible.
    More specifically, the various steps of the algorithm are as
    follows.
    For the ith unreduced block of T:

    (a) compute T-sigma_i I = L_i D_i L_i^T, such that L_i D_i L_i^T is
        a relatively robust representation,

    (#) compute the eigenvalues, lambda_j, of L_i D_i L_i^T to high
        relative accuracy by the dqds algorithm,

    (#) if there is a cluster of close eigenvalues, 'choose' sigma_i
        close to the cluster, and go to (a),

    (#) given the approximate eigenvalue lambda_j of L_i D_i L_i^T,
        compute the corresponding eigenvector by forming a
        rank-revealing twisted factorization.

    The desired accuracy of the output can be specified by the argument
    `abstol`.
    For more details, see Dhillon (1997) and Parlett and Dhillon (2000).

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Barlow, J and Demmel, J W, 1990, `Computing accurate eigensystems of
    scaled diagonally dominant matrices`, SIAM J. Numer. Anal. (27),
    762--791

    Demmel, J W and Kahan, W, 1990, `Accurate singular values of
    bidiagonal matrices`, SIAM J. Sci. Statist. Comput. (11), 873--912

    Dhillon, I, 1997, `A new O(n^2) algorithm for the symmetric
    tridiagonal eigenvalue/eigenvector problem`, Computer Science
    Division Technical Report No. UCB//CSD-97-971, UC Berkeley

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore

    Parlett, B N and Dhillon, I S, 2000, `Relatively robust
    representations of symmetric tridiagonals`, Linear Algebra Appl.
    (309), 121--151
    """

    fun_name = 'naginterfaces.library.lapackeig.dstevr'

    _d = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'd'},
        d,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_d, {'fun_name': fun_name, 'entity': 'd'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _e = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'e'},
        e,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobz = _b_utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = _b_utils._EngineCharScalarType.to_py(_jobz)
    _erange = _b_utils._EngineCharScalarType(
        erange, {'fun_name': fun_name, 'entity': 'erange'}, exp_chrlen=1,
    ).to_ctypes()
    _erange_py = _b_utils._EngineCharScalarType.to_py(_erange)
    _iu = _b_utils._EngineIntScalarType(
        iu, {'fun_name': fun_name, 'entity': 'iu'},
    ).to_ctypes()
    _iu_py = _b_utils._EngineIntScalarType.to_py(_iu)
    _il = _b_utils._EngineIntScalarType(
        il, {'fun_name': fun_name, 'entity': 'il'},
    ).to_ctypes()
    _il_py = _b_utils._EngineIntScalarType.to_py(_il)
    if _jobz_py in ('V', 'v'):
        if _erange_py in ('I', 'i'):
            _vl_est_m_py = _iu_py-_il_py+1
        else:
            _vl_est_m_py = _n_py
    else:
        _vl_est_m_py = 0
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    exp_shape_z = (_z_dim1, _vl_est_m_py)
    _z = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_z,
        sorder=_b_utils._EngineData.default_sorder,
    )
    if _jobz_py in ('V', 'v'):
        _isuppz_dim1 = 2*max(1, _vl_est_m_py)
    else:
        _isuppz_dim1 = 0
    exp_shape_isuppz = (_isuppz_dim1,)
    _isuppz = _b_utils._EngineIntArrayType.empty_ndarray(
        exp_shape_isuppz,
    )
    _return_scalar = _b_lapackeig.dstevr(
        jobz,
        erange,
        _n,
        _d,
        _e,
        vl,
        vu,
        _il,
        _iu,
        abstol,
        _w,
        _z,
        _isuppz,
    )
    _d_py = _d
    _e_py = _e
    _m_py = _return_scalar
    _w_py = _w
    if _jobz_py in ('V', 'v'):
        _z_dim2 = _m_py
    else:
        _z_dim2 = 0
    _z_py = _z[:, :_z_dim2]
    if _jobz_py in ('V', 'v'):
        _isuppz_dim1 = 2*_m_py
    else:
        _isuppz_dim1 = 0
    _isuppz_py = _isuppz[:_isuppz_dim1]
    _return_data = DstevrReturnData(
        _d_py,
        _e_py,
        _m_py,
        _w_py,
        _z_py,
        _isuppz_py,
    )
    return _return_data

DsteqrReturnData = _collections.namedtuple(
    'DsteqrReturnData',
    ['d', 'e', 'z'],
)

def dsteqr(compz, d, e, z=None):
    u"""
    Computes all eigenvalues and eigenvectors of real symmetric
    tridiagonal matrix, reduced from real symmetric matrix using the
    implicit QL or QR algorithm.

    ``dsteqr`` computes all the eigenvalues and, optionally, all the
    eigenvectors of a real symmetric tridiagonal matrix, or of a real
    symmetric matrix which has been reduced to tridiagonal form.

    For full information please refer to the NAG Library document for
    f08je

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08jef.html

    Parameters
    ----------
    compz : str, length 1
        Indicates whether the eigenvectors are to be computed.

        `compz` = 'N'

            Only the eigenvalues are computed (and the array `z` is not
            referenced).

        `compz` = 'V'

            The eigenvalues and eigenvectors of A are computed (and the
            array `z` must contain the matrix Q on entry).

        `compz` = 'I'

            The eigenvalues and eigenvectors of T are computed (and the
            array `z` is initialized by the function).

    d : float, array-like, shape (n)
        The diagonal elements of the tridiagonal matrix T.

    e : float, array-like, shape (max(1, n-1))
        The off-diagonal elements of the tridiagonal matrix T.

    z : None or float, array-like, shape (:, :), optional
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `compz` in ('V', 'I'): max(1, n); if
        `compz`='N': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compz` in ('V', 'I'): n; otherwise:
        0.

        If `compz` = 'V', `z` must contain the orthogonal matrix Q from
        the reduction to tridiagonal form.

        If `compz` = 'I', `z` need not be set.

    Returns
    -------
    d : float, ndarray, shape (n)
        The n eigenvalues in ascending order, unless `errno` > 0 (in
        which case see [Raises/Warns]).

    e : float, ndarray, shape (max(1, n-1))
        `e` is overwritten.

    z : None or float, ndarray, shape (:, :)
        If `compz` = 'V' or 'I', the n required orthonormal eigenvectors
        stored as columns of Z; the ith column corresponds to the ith
        eigenvalue, where i = 1,2,...,n, unless `errno` > 0.

        If `compz` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `compz`.

            Constraint: `compz` = 'N', 'V' or 'I'.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            The algorithm has failed to find all the eigenvalues after a
            total of 30*n iterations. In this case, `d` and `e` contain
            on exit the diagonal and off-diagonal elements,
            respectively, of a tridiagonal matrix orthogonally similar
            to T. *<value>* off-diagonal elements have not converged to
            zero.

    Notes
    -----
    ``dsteqr`` computes all the eigenvalues and, optionally, all the
    eigenvectors of a real symmetric tridiagonal matrix T. In other
    words, it can compute the spectral factorization of T as

        T = Z Lambda Z^T,

    where Lambda is a diagonal matrix whose diagonal elements are the
    eigenvalues lambda_i, and Z is the orthogonal matrix whose columns
    are the eigenvectors z_i.
    Thus

        Tz_i = lambda_i z_i, i = 1,2,...,n.

    The function may also be used to compute all the eigenvalues and
    eigenvectors of a real symmetric matrix A which has been reduced to
    tridiagonal form T:

        [table omitted]

    In this case, the matrix Q must be formed explicitly and passed to
    ``dsteqr``, which must be called with `compz` = 'V'.
    The functions which must be called to perform the reduction to
    tridiagonal form and form Q are:

    [table omitted]

    ``dsteqr`` uses the implicitly shifted QR algorithm, switching
    between the QR and QL variants in order to handle graded matrices
    effectively (see Greenbaum and Dongarra (1980)).
    The eigenvectors are normalized so that ||z_i||_2 = 1, but are
    determined only to within a factor +/-1.

    If only the eigenvalues of T are required, it is more efficient to
    call ``dsterf`` instead.
    If T is positive definite, small eigenvalues can be computed more
    accurately by ``dpteqr``.

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore

    Greenbaum, A and Dongarra, J J, 1980, `Experiments with QR/QL
    methods for the symmetric triangular eigenproblem`, LAPACK Working
    Note No. 17 (Technical Report CS-89-92), University of Tennessee,
    Knoxville, https://www.netlib.org/lapack/lawnspdf/lawn17.pdf

    Parlett, B N, 1998, `The Symmetric Eigenvalue Problem`, SIAM,
    Philadelphia
    """

    fun_name = 'naginterfaces.library.lapackeig.dsteqr'

    _z = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'z'},
        z,
        copy=True,
        allow_none=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_z, 'z'),),
    )
    _d = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'd'},
        d,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_d, {'fun_name': fun_name, 'entity': 'd'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _e = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'e'},
        e,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _compz = _b_utils._EngineCharScalarType(
        compz, {'fun_name': fun_name, 'entity': 'compz'}, exp_chrlen=1,
    ).to_ctypes()
    _compz_py = _b_utils._EngineCharScalarType.to_py(_compz)
    if _compz_py in ('V', 'v'):
        if z is None:
            raise _b_utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'z'},
                'must not be None when compz in (\'V\', \'v\')',
            )
    if z is None:
        if _compz_py in ('V', 'v', 'I', 'i'):
            _z_dim1 = max(1, _n_py)
        elif _compz_py in ('N', 'n'):
            _z_dim1 = 1
        else:
            _z_dim1 = 0
        if _compz_py in ('V', 'v', 'I', 'i'):
            _z_dim2 = _n_py
        else:
            _z_dim2 = 0
        exp_shape_z = (_z_dim1, _z_dim2)
        _z = _b_utils._EngineFloat64ArrayType.empty_ndarray(
            exp_shape_z,
            sorder=_m_sorder,
        )
    try:
        _b_lapackeig.dsteqr(
            compz,
            _n,
            _d,
            _e,
            _z,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _d_py = _d
    _e_py = _e
    if _compz_py not in ('N', 'n'):
        _z_py = _z
    else:
        _z_py = None
    _return_data = DsteqrReturnData(
        _d_py,
        _e_py,
        _z_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

DsterfReturnData = _collections.namedtuple(
    'DsterfReturnData',
    ['d', 'e'],
)

def dsterf(d, e):
    u"""
    Computes all eigenvalues of real symmetric tridiagonal matrix,
    root-free variant of the QL or QR algorithm.

    ``dsterf`` computes all the eigenvalues of a real symmetric
    tridiagonal matrix.

    For full information please refer to the NAG Library document for
    f08jf

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08jff.html

    Parameters
    ----------
    d : float, array-like, shape (n)
        The diagonal elements of the tridiagonal matrix T.

    e : float, array-like, shape (max(1, n-1))
        The off-diagonal elements of the tridiagonal matrix T.

    Returns
    -------
    d : float, ndarray, shape (n)
        The n eigenvalues in ascending order, unless `errno` > 0 (in
        which case see [Raises/Warns]).

    e : float, ndarray, shape (max(1, n-1))
        `e` is overwritten.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter n.

            Constraint: n >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            The algorithm has failed to find all the eigenvalues after a
            total of 30*n iterations; *<value>* elements of `e` have not
            converged to zero.

    Notes
    -----
    ``dsterf`` computes all the eigenvalues of a real symmetric
    tridiagonal matrix, using a square-root-free variant of the QR
    algorithm.

    The function uses an explicit shift, and, like ``dsteqr``, switches
    between the QR and QL variants in order to handle graded matrices
    effectively (see Greenbaum and Dongarra (1980)).

    References
    ----------
    Greenbaum, A and Dongarra, J J, 1980, `Experiments with QR/QL
    methods for the symmetric triangular eigenproblem`, LAPACK Working
    Note No. 17 (Technical Report CS-89-92), University of Tennessee,
    Knoxville, https://www.netlib.org/lapack/lawnspdf/lawn17.pdf

    Parlett, B N, 1998, `The Symmetric Eigenvalue Problem`, SIAM,
    Philadelphia
    """

    fun_name = 'naginterfaces.library.lapackeig.dsterf'

    _d = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'd'},
        d,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_d, {'fun_name': fun_name, 'entity': 'd'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _e = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'e'},
        e,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    try:
        _b_lapackeig.dsterf(
            _n,
            _d,
            _e,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _d_py = _d
    _e_py = _e
    _return_data = DsterfReturnData(
        _d_py,
        _e_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

DpteqrReturnData = _collections.namedtuple(
    'DpteqrReturnData',
    ['d', 'e', 'z'],
)

def dpteqr(compz, d, e, z):
    u"""
    Computes all eigenvalues and eigenvectors of real symmetric positive
    definite tridiagonal matrix, reduced from real symmetric positive
    definite matrix.

    ``dpteqr`` computes all the eigenvalues and, optionally, all the
    eigenvectors of a real symmetric positive definite tridiagonal
    matrix, or of a real symmetric positive definite matrix which has
    been reduced to tridiagonal form.

    For full information please refer to the NAG Library document for
    f08jg

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08jgf.html

    Parameters
    ----------
    compz : str, length 1
        Indicates whether the eigenvectors are to be computed.

        `compz` = 'N'

            Only the eigenvalues are computed (and the array `z` is not
            referenced).

        `compz` = 'V'

            The eigenvalues and eigenvectors of A are computed (and the
            array `z` must contain the matrix Q on entry).

        `compz` = 'I'

            The eigenvalues and eigenvectors of T are computed (and the
            array `z` is initialized by the function).

    d : float, array-like, shape (n)
        The diagonal elements of the tridiagonal matrix T.

    e : float, array-like, shape (max(1, n-1))
        The off-diagonal elements of the tridiagonal matrix T.

    z : float, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `compz` in ('V', 'I'): max(1, n); if
        `compz`='N': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compz` in ('V', 'I'): n; if
        `compz`='N': 1; otherwise: 0.

        If `compz` = 'V', `z` must contain the orthogonal matrix Q from
        the reduction to tridiagonal form.

        If `compz` = 'I', `z` need not be set.

    Returns
    -------
    d : float, ndarray, shape (n)
        The n eigenvalues in descending order, unless `errno` > 0, in
        which case `d` is overwritten.

    e : float, ndarray, shape (max(1, n-1))
        `e` is overwritten.

    z : float, ndarray, shape (:, :)
        If `compz` = 'V' or 'I', the n required orthonormal eigenvectors
        stored as columns of Z; the ith column corresponds to the ith
        eigenvalue, where i = 1,2,...,n, unless `errno` > 0.

        If `compz` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `compz`.

            Constraint: `compz` = 'N', 'V' or 'I'.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` i>0 and i<=n)
            The leading minor of order *<value>* is not positive
            definite and the Cholesky factorization of T could not be
            completed. Hence T itself is not positive definite.

        (`errno` i > n)
            The algorithm to compute the singular values of the Cholesky
            factor B failed to converge; *<value>* off-diagonal elements
            did not converge to zero.

    Notes
    -----
    ``dpteqr`` computes all the eigenvalues and, optionally, all the
    eigenvectors of a real symmetric positive definite tridiagonal
    matrix T. In other words, it can compute the spectral factorization
    of T as

        T = Z Lambda Z^T,

    where Lambda is a diagonal matrix whose diagonal elements are the
    eigenvalues lambda_i, and Z is the orthogonal matrix whose columns
    are the eigenvectors z_i.
    Thus

        Tz_i = lambda_i z_i, i = 1,2,...,n.

    The function may also be used to compute all the eigenvalues and
    eigenvectors of a real symmetric positive definite matrix A which
    has been reduced to tridiagonal form T:

        [table omitted]

    In this case, the matrix Q must be formed explicitly and passed to
    ``dpteqr``, which must be called with `compz` = 'V'.
    The functions which must be called to perform the reduction to
    tridiagonal form and form Q are:

    [table omitted]

    ``dpteqr`` first factorizes T as LDL^T where L is unit lower
    bidiagonal and D is diagonal.
    It forms the bidiagonal matrix B = LD^(1/2), and then calls
    ``dbdsqr`` to compute the singular values of B which are the same as
    the eigenvalues of T.
    The method used by the function allows high relative accuracy to be
    achieved in the small eigenvalues of T.
    The eigenvectors are normalized so that ||z_i||_2 = 1, but are
    determined only to within a factor +/-1.

    References
    ----------
    Barlow, J and Demmel, J W, 1990, `Computing accurate eigensystems of
    scaled diagonally dominant matrices`, SIAM J. Numer. Anal. (27),
    762--791
    """

    fun_name = 'naginterfaces.library.lapackeig.dpteqr'

    _d = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'd'},
        d,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_d, {'fun_name': fun_name, 'entity': 'd'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _e = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'e'},
        e,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _z = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'z'},
        z,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b_lapackeig.dpteqr(
        compz,
        _n,
        _d,
        _e,
        _z,
    )
    _d_py = _d
    _e_py = _e
    _z_py = _z
    _return_data = DpteqrReturnData(
        _d_py,
        _e_py,
        _z_py,
    )
    return _return_data

DstedcReturnData = _collections.namedtuple(
    'DstedcReturnData',
    ['d', 'e', 'z'],
)

def dstedc(compz, d, e, z):
    u"""
    Computes all eigenvalues and, optionally, eigenvectors of a real
    symmetric tridiagonal matrix or a matrix reduced to this form
    (divide-and-conquer).

    ``dstedc`` computes all the eigenvalues and, optionally, all the
    eigenvectors of a real n*n symmetric tridiagonal matrix, or of a
    real full or banded symmetric matrix which has been reduced to
    tridiagonal form.

    For full information please refer to the NAG Library document for
    f08jh

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08jhf.html

    Parameters
    ----------
    compz : str, length 1
        Indicates whether the eigenvectors are to be computed.

        `compz` = 'N'

            Only the eigenvalues are computed (and the array `z` is not
            referenced).

        `compz` = 'V'

            The eigenvalues and eigenvectors of A are computed (and the
            array `z` must contain the matrix Q on entry).

        `compz` = 'I'

            The eigenvalues and eigenvectors of T are computed (and the
            array `z` is initialized by the function).

    d : float, array-like, shape (n)
        The diagonal elements of the tridiagonal matrix.

    e : float, array-like, shape (max(1, n-1))
        The subdiagonal elements of the tridiagonal matrix.

    z : float, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `compz` in ('V', 'I'): n; otherwise:
        1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compz` in ('V', 'I'): n; otherwise:
        1.

        If `compz` = 'V', `z` must contain the orthogonal matrix Q used
        in the reduction to tridiagonal form.

    Returns
    -------
    d : float, ndarray, shape (n)
        If no exception or warning is raised, the eigenvalues in
        ascending order.

    e : float, ndarray, shape (max(1, n-1))
        `e` is overwritten.

    z : float, ndarray, shape (:, :)
        If `compz` = 'V', `z` contains the orthonormal eigenvectors of
        the original symmetric matrix A, and if `compz` = 'I', `z`
        contains the orthonormal eigenvectors of the symmetric
        tridiagonal matrix T.

        If `compz` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `compz`.

            Constraint: `compz` = 'N', 'V' or 'I'.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            The algorithm failed to compute an eigenvalue while working
            on the submatrix lying in rows and columns *<value>*/(n+1)
            through mod(*<value>*, n+1).

    Notes
    -----
    ``dstedc`` computes all the eigenvalues and, optionally, the
    eigenvectors of a real symmetric tridiagonal matrix T.
    That is, the function computes the spectral factorization of T given
    by

        T = Z Lambda Z^T,

    where Lambda is a diagonal matrix whose diagonal elements are the
    eigenvalues, lambda_i, of T and Z is an orthogonal matrix whose
    columns are the eigenvectors, z_i, of T.
    Thus

        Tz_i = lambda_i z_i, i = 1,2,...,n.

    The function may also be used to compute all the eigenvalues and
    vectors of a real full, or banded, symmetric matrix A which has been
    reduced to tridiagonal form T as

        A = QTQ^T,

    where Q is orthogonal.
    The spectral factorization of A is then given by

        A = (QZ)Lambda(QZ)^T.

    In this case Q must be formed explicitly and passed to ``dstedc`` in
    the array `z`, and the function called with `compz` = 'V'. Functions
    which may be called to form T and Q are

    [table omitted]

    When only eigenvalues are required then this function calls
    ``dsterf`` to compute the eigenvalues of the tridiagonal matrix T,
    but when eigenvectors of T are also required and the matrix is not
    too small, then a divide and conquer method is used, which can be
    much faster than ``dsteqr``, although more storage is required.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dstedc'

    _d = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'd'},
        d,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_d, {'fun_name': fun_name, 'entity': 'd'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _e = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'e'},
        e,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _z = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'z'},
        z,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    try:
        _b_lapackeig.dstedc(
            compz,
            _n,
            _d,
            _e,
            _z,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _d_py = _d
    _e_py = _e
    _z_py = _z
    _return_data = DstedcReturnData(
        _d_py,
        _e_py,
        _z_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

DstebzReturnData = _collections.namedtuple(
    'DstebzReturnData',
    ['m', 'nsplit', 'w', 'iblock', 'isplit'],
)

def dstebz(erange, order, vl, vu, il, iu, abstol, d, e):
    u"""
    Computes selected eigenvalues of real symmetric tridiagonal matrix
    by bisection.

    ``dstebz`` computes some (or all) of the eigenvalues of a real
    symmetric tridiagonal matrix, by bisection.

    For full information please refer to the NAG Library document for
    f08jj

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08jjf.html

    Parameters
    ----------
    erange : str, length 1
        Indicates which eigenvalues are required.

        `erange` = 'A'

            All the eigenvalues are required.

        `erange` = 'V'

            All the eigenvalues in the half-open interval (`vl`,`vu`]
            are required.

        `erange` = 'I'

            Eigenvalues with indices `il` to `iu` are required.

    order : str, length 1
        Indicates the order in which the eigenvalues and their block
        numbers are to be stored.

        `order` = 'B'

            The eigenvalues are to be grouped by split-off block and
            ordered from smallest to largest within each block.

        `order` = 'E'

            The eigenvalues for the entire matrix are to be ordered from
            smallest to largest.

    vl : float
        If `erange` = 'V', the lower and upper bounds, respectively, of
        the half-open interval (`vl`,`vu`] within which the required
        eigenvalues lie.

        If `erange` = 'A' or 'I', `vl` is not referenced.

    vu : float
        If `erange` = 'V', the lower and upper bounds, respectively, of
        the half-open interval (`vl`,`vu`] within which the required
        eigenvalues lie.

        If `erange` = 'A' or 'I', `vu` is not referenced.

    il : int
        If `erange` = 'I', the indices of the first and last
        eigenvalues, respectively, to be computed (assuming that the
        eigenvalues are in ascending order).

        If `erange` = 'A' or 'V', `il` is not referenced.

    iu : int
        If `erange` = 'I', the indices of the first and last
        eigenvalues, respectively, to be computed (assuming that the
        eigenvalues are in ascending order).

        If `erange` = 'A' or 'V', `iu` is not referenced.

    abstol : float
        The absolute tolerance to which each eigenvalue is required. An
        eigenvalue (or cluster) is considered to have converged if it
        lies in an interval of width <= `abstol`. If `abstol` <= 0.0,
        the tolerance is taken as machine precision*||T||_1.

    d : float, array-like, shape (n)
        The diagonal elements of the tridiagonal matrix T.

    e : float, array-like, shape (max(1, n-1))
        The off-diagonal elements of the tridiagonal matrix T.

    Returns
    -------
    m : int
        m, the actual number of eigenvalues found.

    nsplit : int
        The number of diagonal blocks which constitute the tridiagonal
        matrix T.

    w : float, ndarray, shape (n)
        The required eigenvalues of the tridiagonal matrix T stored in
        `w`[0] to `w`[m-1].

    iblock : int, ndarray, shape (n)
        At each row/column j where `e`[j-1] is zero or negligible, T is
        considered to split into a block diagonal matrix and
        `iblock`[i-1] contains the block number of the eigenvalue stored
        in `w`[i-1], for i = 1,2,...,m. Note that `iblock`[i-1] < 0 for
        some i whenever `errno` = 1 or 3 (see [Raises/Warns]) and
        `erange` = 'A' or 'V'.

    isplit : int, ndarray, shape (n)
        The leading `nsplit` elements contain the points at which T
        splits up into sub-matrices as follows. The first sub-matrix
        consists of rows/columns 1 to `isplit`[0], the second sub-matrix
        consists of rows/columns `isplit`[0]+1 to `isplit`[1], ..., and
        the `nsplit`(th) sub-matrix consists of rows/columns
        `isplit`[`nsplit`-2]+1 to `isplit`[`nsplit`-1] (= n).

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `erange`.

            Constraint: `erange` = 'A', 'V' or 'I'.

        (`errno` -2)
            On entry, error in parameter `order`.

            Constraint: `order` = 'B' or 'E'.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -5)
            On entry, error in parameter `vu`.

            Constraint: `vl` < `vu`.

        (`errno` -6)
            On entry, error in parameter `il`.

            Constraint: `il` >= 1 and `il` <= max(1, n).

        (`errno` -7)
            On entry, error in parameter `iu`.

            Constraint: `iu` <= n and `iu` >= min(n, `il`).

        (`errno` 2)
            If `erange` = 'I', the algorithm failed to compute some (or
            all) of the required eigenvalues. Try calling the function
            again with `erange` = 'A'.

        (`errno` 3)
            If `erange` = 'I', the algorithm failed to compute some (or
            all) of the required eigenvalues. Try calling the function
            again with `erange` = 'A'. If `erange` = 'A' or 'V', the
            algorithm failed to compute some (or all) of the required
            eigenvalues to the required accuracy. More precisely,
            `iblock`[*<value>*] < 0 indicates that eigenvalue *<value>*
            (stored in `w`[*<value>*]) failed to converge.

        (`errno` 4)
            No eigenvalues have been computed. The floating-point
            arithmetic on the computer is not behaving as expected.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            If `erange` = 'A' or 'V', the algorithm failed to compute
            some (or all) of the required eigenvalues to the required
            accuracy. More precisely, `iblock`[*<value>*] < 0 indicates
            that eigenvalue *<value>* (stored in `w`[*<value>*]) failed
            to converge.

    Notes
    -----
    ``dstebz`` uses bisection to compute some or all of the eigenvalues
    of a real symmetric tridiagonal matrix T.

    It searches for zero or negligible off-diagonal elements of T to see
    if the matrix splits into block diagonal form:

        T = [table omitted].

    It performs bisection on each of the blocks T_i and returns the
    block index of each computed eigenvalue, so that a subsequent call
    to ``dstein`` to compute eigenvectors can also take advantage of the
    block structure.

    References
    ----------
    Kahan, W, 1966, `Accurate eigenvalues of a symmetric tridiagonal
    matrix`, Report CS41, Stanford University
    """

    fun_name = 'naginterfaces.library.lapackeig.dstebz'

    _d = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'd'},
        d,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_d, {'fun_name': fun_name, 'entity': 'd'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _e = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'e'},
        e,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _iblock = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    _isplit = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        _return_scalars = _b_lapackeig.dstebz(
            erange,
            order,
            _n,
            vl,
            vu,
            il,
            iu,
            abstol,
            _d,
            _e,
            _w,
            _iblock,
            _isplit,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _m_py = _return_scalars.m
    _nsplit_py = _return_scalars.nsplit
    _w_py = _w
    _iblock_py = _iblock
    _isplit_py = _isplit
    _return_data = DstebzReturnData(
        _m_py,
        _nsplit_py,
        _w_py,
        _iblock_py,
        _isplit_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

DsteinReturnData = _collections.namedtuple(
    'DsteinReturnData',
    ['z', 'ifailv'],
)

def dstein(d, e, m, w, iblock, isplit):
    u"""
    Computes selected eigenvectors of real symmetric tridiagonal matrix
    by inverse iteration, storing eigenvectors in real array.

    ``dstein`` computes the eigenvectors of a real symmetric tridiagonal
    matrix corresponding to specified eigenvalues, by inverse iteration.

    For full information please refer to the NAG Library document for
    f08jk

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08jkf.html

    Parameters
    ----------
    d : float, array-like, shape (n)
        The diagonal elements of the tridiagonal matrix T.

    e : float, array-like, shape (n-1)
        The off-diagonal elements of the tridiagonal matrix T.

    m : int
        m, the number of eigenvectors to be returned.

    w : float, array-like, shape (n)
        The eigenvalues of the tridiagonal matrix T stored in `w`[0] to
        `w`[m-1], as returned by ``dstebz`` with order = 'B'.
        Eigenvalues associated with the first sub-matrix must be
        supplied first, in nondecreasing order; then those associated
        with the second sub-matrix, again in nondecreasing order; and so
        on.

    iblock : int, array-like, shape (n)
        The first m elements must contain the sub-matrix indices
        associated with the specified eigenvalues, as returned by
        ``dstebz`` with order = 'B'. If the eigenvalues were not
        computed by ``dstebz`` with order = 'B', set `iblock`[i-1] to 1,
        for i = 1,2,...,m.

    isplit : int, array-like, shape (n)
        The points at which T breaks up into sub-matrices, as returned
        by ``dstebz`` with order = 'B'. If the eigenvalues were not
        computed by ``dstebz`` with order = 'B', set `isplit`[0] to n.

    Returns
    -------
    z : float, ndarray, shape (n, `m`)
        The m eigenvectors, stored as columns of Z; the ith column
        corresponds to the ith specified eigenvalue, unless `errno` > 0
        (in which case see [Raises/Warns]).

    ifailv : int, ndarray, shape (`m`)
        If errno = i > 0, the first i elements of `ifailv` contain the
        indices of any eigenvectors which have failed to converge. The
        rest of the first `m` elements of `ifailv` are set to 0.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -4)
            On entry, error in parameter `m`.

            Constraint: 0 <= `m` <= n.

        (`errno` -5)
            On entry, error in parameter `w`.

            Constraint: `w`[i] <= `w`[i+1].

        (`errno` -6)
            On entry, error in parameter `iblock`.

            Constraint: `iblock`[i] <= `iblock`[i+1].

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            *<value>* eigenvectors (as indicated by argument `ifailv`)
            each failed to converge in five iterations. The current
            iterate after five iterations is stored in the corresponding
            column of `z`.

    Notes
    -----
    ``dstein`` computes the eigenvectors of a real symmetric tridiagonal
    matrix T corresponding to specified eigenvalues, by inverse
    iteration (see Jessup and Ipsen (1992)).
    It is designed to be used in particular after the specified
    eigenvalues have been computed by ``dstebz`` with order = 'B', but
    may also be used when the eigenvalues have been computed by other
    functions in submodule ``lapackeig`` or submodule ``eigen``.

    If T has been formed by reduction of a full real symmetric matrix A
    to tridiagonal form, then eigenvectors of T may be transformed to
    eigenvectors of A by a call to ``dormtr`` or ``dopmtr``.

    ``dstebz`` determines whether the matrix T splits into block
    diagonal form:

        T = [table omitted]

    and passes details of the block structure to this function in the
    arrays `iblock` and `isplit`.
    This function can then take advantage of the block structure by
    performing inverse iteration on each block T_i separately, which is
    more efficient than using the whole matrix.

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore

    Jessup, E and Ipsen, I C F, 1992, `Improving the accuracy of inverse
    iteration`, SIAM J. Sci. Statist. Comput. (13), 550--572
    """

    fun_name = 'naginterfaces.library.lapackeig.dstein'

    _d = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'd'},
        d,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_d, {'fun_name': fun_name, 'entity': 'd'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _e = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'e'},
        e,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _w = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'w'},
        w,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _iblock = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'iblock'},
        iblock,
        copy=False,
    )
    _isplit = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'isplit'},
        isplit,
        copy=False,
    )
    _m = _b_utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = _b_utils._EngineIntScalarType.to_py(_m)
    _z = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py, _m_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _ifailv = _b_utils._EngineIntArrayType.empty_ndarray(
        (_m_py,),
    )
    try:
        _b_lapackeig.dstein(
            _n,
            _d,
            _e,
            _m,
            _w,
            _iblock,
            _isplit,
            _z,
            _ifailv,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _z_py = _z
    _ifailv_py = _ifailv
    _return_data = DsteinReturnData(
        _z_py,
        _ifailv_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

DstegrReturnData = _collections.namedtuple(
    'DstegrReturnData',
    ['d', 'e', 'm', 'w', 'z', 'isuppz'],
)

def dstegr(jobz, erange, d, e, vl, vu, il, iu):
    u"""
    Computes selected eigenvalues and, optionally, the corresponding
    eigenvectors of a real symmetric tridiagonal matrix or a symmetric
    matrix reduced to this form (Relatively Robust Representations).

    ``dstegr`` computes selected eigenvalues and, optionally, the
    corresponding eigenvectors of a real n*n symmetric tridiagonal
    matrix.

    For full information please refer to the NAG Library document for
    f08jl

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08jlf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

            Only eigenvalues are computed.

        `jobz` = 'V'

            Eigenvalues and eigenvectors are computed.

    erange : str, length 1
        Indicates which eigenvalues should be returned.

        `erange` = 'A'

            All eigenvalues will be found.

        `erange` = 'V'

            All eigenvalues in the half-open interval (`vl`,`vu`] will
            be found.

        `erange` = 'I'

            The `il`th through `iu`th eigenvectors will be found.

    d : float, array-like, shape (n)
        The n diagonal elements of the tridiagonal matrix T.

    e : float, array-like, shape (n)
        `e`[0] to `e`[n-2] are the subdiagonal elements of the
        tridiagonal matrix T. `e`[n-1] need not be set.

    vl : float
        If `erange` = 'V', `vl` and `vu` contain the lower and upper
        bounds respectively of the interval to be searched for
        eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    vu : float
        If `erange` = 'V', `vl` and `vu` contain the lower and upper
        bounds respectively of the interval to be searched for
        eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    il : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    iu : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    Returns
    -------
    d : float, ndarray, shape (n)
        `d` is overwritten.

    e : float, ndarray, shape (n)
        `e` is overwritten.

    m : int
        The total number of eigenvalues found. 0 <= `m` <= n.

        If `erange` = 'A', `m` = n.

        If `erange` = 'I', `m` = `iu`-`il`+1.

    w : float, ndarray, shape (n)
        The eigenvalues in ascending order.

    z : float, ndarray, shape (:, :)
        If `jobz` = 'V', then if no exception or warning is raised, the
        columns of `z` contain the orthonormal eigenvectors of the
        matrix T, with the ith column of Z holding the eigenvector
        associated with `w`[i-1].

        If `jobz` = 'N', `z` is not referenced.

        **Note:** you must ensure that at least max(1, `m`) columns are
        supplied in the array `z`; if `erange` = 'V', the exact value of
        `m` is not known in advance and an upper bound of at least n
        must be used.

    isuppz : int, ndarray, shape (:)
        The support of the eigenvectors in Z, i.e., the indices
        indicating the nonzero elements in Z. The ith eigenvector is
        nonzero only in elements `isuppz`[2*i-2] through
        `isuppz`[2*i-1].

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `erange`.

            Constraint: `erange` = 'A', 'V' or 'I'.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -7)
            On entry, error in parameter `vu`.

            Constraint: `vl` < `vu`.

        (`errno` -9)
            On entry, error in parameter `iu`.

            Constraint: 1 <= `il` <= `iu` <= n.

        (`errno` -9)
            On entry, error in parameter `iu`.

            Constraint: `il` = 1 and `iu` = 0.

        (`errno` i = 1)
            The dqds algorithm failed to converge.

        (`errno` i = 2)
            Inverse iteration failed to converge.

    Notes
    -----
    ``dstegr`` computes selected eigenvalues and, optionally, the
    corresponding eigenvectors, of a real symmetric tridiagonal matrix
    T.
    That is, the function computes the (partial) spectral factorization
    of T given by

        Z Lambda Z^T,

    where Lambda is a diagonal matrix whose diagonal elements are the
    selected eigenvalues, lambda_i, of T and Z is an orthogonal matrix
    whose columns are the corresponding eigenvectors, z_i, of T.
    Thus

        Tz_i = lambda_i z_i, i = 1,2,...,m

    where m is the number of selected eigenvalues computed.

    The function may also be used to compute selected eigenvalues and
    eigenvectors of a real symmetric matrix A which has been reduced to
    tridiagonal form T:

        (QZ)Lambda(QZ)^T, where Q is orthogonal.

    In this case, the matrix Q must be explicitly applied to the output
    matrix Z.
    The functions which must be called to perform the reduction to
    tridiagonal form and apply Q are:

    [table omitted]

    This function uses the dqds and the Relatively Robust Representation
    algorithms to compute the eigenvalues and eigenvectors respectively;
    see for example Parlett and Dhillon (2000) and Dhillon and Parlett
    (2004) for further details. ``dstegr`` can usually compute all the
    eigenvalues and eigenvectors in O(n^2) floating-point operations and
    so, for large matrices, is often considerably faster than the other
    symmetric tridiagonal functions in this module when all the
    eigenvectors are required, particularly so compared to those
    functions that are based on the QR algorithm.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Barlow, J and Demmel, J W, 1990, `Computing accurate eigensystems of
    scaled diagonally dominant matrices`, SIAM J. Numer. Anal. (27),
    762--791

    Dhillon, I S and Parlett, B N, 2004, `Orthogonal eigenvectors and
    relative gaps`, SIAM J. Appl. Math. (25), 858--899

    Parlett, B N and Dhillon, I S, 2000, `Relatively robust
    representations of symmetric tridiagonals`, Linear Algebra Appl.
    (309), 121--151
    """

    fun_name = 'naginterfaces.library.lapackeig.dstegr'

    _d = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'd'},
        d,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_d, {'fun_name': fun_name, 'entity': 'd'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _e = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'e'},
        e,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobz = _b_utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = _b_utils._EngineCharScalarType.to_py(_jobz)
    _erange = _b_utils._EngineCharScalarType(
        erange, {'fun_name': fun_name, 'entity': 'erange'}, exp_chrlen=1,
    ).to_ctypes()
    _erange_py = _b_utils._EngineCharScalarType.to_py(_erange)
    _iu = _b_utils._EngineIntScalarType(
        iu, {'fun_name': fun_name, 'entity': 'iu'},
    ).to_ctypes()
    _iu_py = _b_utils._EngineIntScalarType.to_py(_iu)
    _il = _b_utils._EngineIntScalarType(
        il, {'fun_name': fun_name, 'entity': 'il'},
    ).to_ctypes()
    _il_py = _b_utils._EngineIntScalarType.to_py(_il)
    if _jobz_py in ('V', 'v'):
        if _erange_py in ('I', 'i'):
            _vl_est_m_py = _iu_py-_il_py+1
        else:
            _vl_est_m_py = _n_py
    else:
        _vl_est_m_py = 0
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    exp_shape_z = (_z_dim1, _vl_est_m_py)
    _z = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_z,
        sorder=_b_utils._EngineData.default_sorder,
    )
    if _jobz_py in ('V', 'v'):
        _isuppz_dim1 = 2*max(1, _vl_est_m_py)
    else:
        _isuppz_dim1 = 0
    exp_shape_isuppz = (_isuppz_dim1,)
    _isuppz = _b_utils._EngineIntArrayType.empty_ndarray(
        exp_shape_isuppz,
    )
    _return_scalar = _b_lapackeig.dstegr(
        jobz,
        erange,
        _n,
        _d,
        _e,
        vl,
        vu,
        _il,
        _iu,
        _w,
        _z,
        _isuppz,
    )
    _d_py = _d
    _e_py = _e
    _m_py = _return_scalar
    _w_py = _w
    if _jobz_py in ('V', 'v'):
        _z_dim2 = _m_py
    else:
        _z_dim2 = 0
    _z_py = _z[:, :_z_dim2]
    if _jobz_py in ('V', 'v'):
        _isuppz_dim1 = 2*_m_py
    else:
        _isuppz_dim1 = 0
    _isuppz_py = _isuppz[:_isuppz_dim1]
    _return_data = DstegrReturnData(
        _d_py,
        _e_py,
        _m_py,
        _w_py,
        _z_py,
        _isuppz_py,
    )
    return _return_data

ZsteqrReturnData = _collections.namedtuple(
    'ZsteqrReturnData',
    ['d', 'e', 'z'],
)

def zsteqr(compz, d, e, z):
    u"""
    Computes all eigenvalues and eigenvectors of real symmetric
    tridiagonal matrix, reduced from complex Hermitian matrix, using the
    implicit QL or QR algorithm.

    ``zsteqr`` computes all the eigenvalues and, optionally, all the
    eigenvectors of a complex Hermitian matrix which has been reduced to
    tridiagonal form.

    For full information please refer to the NAG Library document for
    f08js

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08jsf.html

    Parameters
    ----------
    compz : str, length 1
        Indicates whether the eigenvectors are to be computed.

        `compz` = 'N'

            Only the eigenvalues are computed (and the array `z` is not
            referenced).

        `compz` = 'V'

            The eigenvalues and eigenvectors of A are computed (and the
            array `z` must contain the matrix Q on entry).

        `compz` = 'I'

            The eigenvalues and eigenvectors of T are computed (and the
            array `z` is initialized by the function).

    d : float, array-like, shape (n)
        The diagonal elements of the tridiagonal matrix T.

    e : float, array-like, shape (max(1, n-1))
        The off-diagonal elements of the tridiagonal matrix T.

    z : complex, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `compz` in ('V', 'I'): max(1, n); if
        `compz`='N': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compz` in ('V', 'I'): n; if
        `compz`='N': 1; otherwise: 0.

        If `compz` = 'V', `z` must contain the unitary matrix Q from the
        reduction to tridiagonal form.

        If `compz` = 'I', `z` need not be set.

    Returns
    -------
    d : float, ndarray, shape (n)
        The n eigenvalues in ascending order, unless `errno` > 0 (in
        which case see [Raises/Warns]).

    e : float, ndarray, shape (max(1, n-1))
        `e` is overwritten.

    z : complex, ndarray, shape (:, :)
        If `compz` = 'V' or 'I', the n required orthonormal eigenvectors
        stored as columns of Z; the ith column corresponds to the ith
        eigenvalue, where i = 1,2,...,n, unless `errno` > 0.

        If `compz` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `compz`.

            Constraint: `compz` = 'N', 'V' or 'I'.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            The algorithm has failed to find all the eigenvalues after a
            total of 30*n iterations. In this case, `d` and `e` contain
            on exit the diagonal and off-diagonal elements,
            respectively, of a tridiagonal matrix unitarily similar to
            T. *<value>* off-diagonal elements have not converged to
            zero.

    Notes
    -----
    ``zsteqr`` computes all the eigenvalues and, optionally, all the
    eigenvectors of a real symmetric tridiagonal matrix T. In other
    words, it can compute the spectral factorization of T as

        T = Z Lambda Z^T,

    where Lambda is a diagonal matrix whose diagonal elements are the
    eigenvalues lambda_i, and Z is the orthogonal matrix whose columns
    are the eigenvectors z_i.
    Thus

        Tz_i = lambda_i z_i, i = 1,2,...,n.

    The function stores the real orthogonal matrix Z in a complex array,
    so that it may also be used to compute all the eigenvalues and
    eigenvectors of a complex Hermitian matrix A which has been reduced
    to tridiagonal form T:

        [table omitted]

    In this case, the matrix Q must be formed explicitly and passed to
    ``zsteqr``, which must be called with `compz` = 'V'.
    The functions which must be called to perform the reduction to
    tridiagonal form and form Q are:

    [table omitted]

    ``zsteqr`` uses the implicitly shifted QR algorithm, switching
    between the QR and QL variants in order to handle graded matrices
    effectively (see Greenbaum and Dongarra (1980)).
    The eigenvectors are normalized so that ||z_i||_2 = 1, but are
    determined only to within a complex factor of absolute value 1.

    If only the eigenvalues of T are required, it is more efficient to
    call ``dsterf`` instead.
    If T is positive definite, small eigenvalues can be computed more
    accurately by ``zpteqr``.

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore

    Greenbaum, A and Dongarra, J J, 1980, `Experiments with QR/QL
    methods for the symmetric triangular eigenproblem`, LAPACK Working
    Note No. 17 (Technical Report CS-89-92), University of Tennessee,
    Knoxville, https://www.netlib.org/lapack/lawnspdf/lawn17.pdf

    Parlett, B N, 1998, `The Symmetric Eigenvalue Problem`, SIAM,
    Philadelphia
    """

    fun_name = 'naginterfaces.library.lapackeig.zsteqr'

    _d = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'd'},
        d,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_d, {'fun_name': fun_name, 'entity': 'd'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _e = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'e'},
        e,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _z = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'z'},
        z,
        copy=True,
    )
    try:
        _b_lapackeig.zsteqr(
            compz,
            _n,
            _d,
            _e,
            _z,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _d_py = _d
    _e_py = _e
    _z_py = _z
    _return_data = ZsteqrReturnData(
        _d_py,
        _e_py,
        _z_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

ZpteqrReturnData = _collections.namedtuple(
    'ZpteqrReturnData',
    ['d', 'e', 'z'],
)

def zpteqr(compz, d, e, z):
    u"""
    Computes all eigenvalues and eigenvectors of real symmetric positive
    definite tridiagonal matrix, reduced from complex Hermitian positive
    definite matrix.

    ``zpteqr`` computes all the eigenvalues and, optionally, all the
    eigenvectors of a complex Hermitian positive definite matrix which
    has been reduced to tridiagonal form.

    For full information please refer to the NAG Library document for
    f08ju

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08juf.html

    Parameters
    ----------
    compz : str, length 1
        Indicates whether the eigenvectors are to be computed.

        `compz` = 'N'

            Only the eigenvalues are computed (and the array `z` is not
            referenced).

        `compz` = 'V'

            The eigenvalues and eigenvectors of A are computed (and the
            array `z` must contain the matrix Q on entry).

        `compz` = 'I'

            The eigenvalues and eigenvectors of T are computed (and the
            array `z` is initialized by the function).

    d : float, array-like, shape (n)
        The diagonal elements of the tridiagonal matrix T.

    e : float, array-like, shape (max(1, n-1))
        The off-diagonal elements of the tridiagonal matrix T.

    z : complex, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `compz` in ('V', 'I'): max(1, n); if
        `compz`='N': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compz` in ('V', 'I'): n; if
        `compz`='N': 1; otherwise: 0.

        If `compz` = 'V', `z` must contain the unitary matrix Q from the
        reduction to tridiagonal form.

        If `compz` = 'I', `z` need not be set.

    Returns
    -------
    d : float, ndarray, shape (n)
        The n eigenvalues in descending order, unless `errno` > 0, in
        which case `d` is overwritten.

    e : float, ndarray, shape (max(1, n-1))
        `e` is overwritten.

    z : complex, ndarray, shape (:, :)
        If `compz` = 'V' or 'I', the n required orthonormal eigenvectors
        stored as columns of Z; the ith column corresponds to the ith
        eigenvalue, where i = 1,2,...,n, unless `errno` > 0.

        If `compz` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `compz`.

            Constraint: `compz` = 'N', 'V' or 'I'.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` i>0 and i<=n)
            The leading minor of order *<value>* is not positive
            definite and the Cholesky factorization of T could not be
            completed. Hence T itself is not positive definite.

        (`errno` i > n)
            The algorithm to compute the singular values of the Cholesky
            factor B failed to converge; *<value>* off-diagonal elements
            did not converge to zero.

    Notes
    -----
    ``zpteqr`` computes all the eigenvalues and, optionally, all the
    eigenvectors of a real symmetric positive definite tridiagonal
    matrix T. In other words, it can compute the spectral factorization
    of T as

        T = Z Lambda Z^T,

    where Lambda is a diagonal matrix whose diagonal elements are the
    eigenvalues lambda_i, and Z is the orthogonal matrix whose columns
    are the eigenvectors z_i.
    Thus

        Tz_i = lambda_i z_i, i = 1,2,...,n.

    The function stores the real orthogonal matrix Z in a complex array,
    so that it may be used to compute all the eigenvalues and
    eigenvectors of a complex Hermitian positive definite matrix A which
    has been reduced to tridiagonal form T:

        [table omitted]

    In this case, the matrix Q must be formed explicitly and passed to
    ``zpteqr``, which must be called with `compz` = 'V'.
    The functions which must be called to perform the reduction to
    tridiagonal form and form Q are:

    [table omitted]

    ``zpteqr`` first factorizes T as LDL^H where L is unit lower
    bidiagonal and D is diagonal.
    It forms the bidiagonal matrix B = LD^(1/2), and then calls
    ``zbdsqr`` to compute the singular values of B which are the same as
    the eigenvalues of T.
    The method used by the function allows high relative accuracy to be
    achieved in the small eigenvalues of T.
    The eigenvectors are normalized so that ||z_i||_2 = 1, but are
    determined only to within a complex factor of absolute value 1.

    References
    ----------
    Barlow, J and Demmel, J W, 1990, `Computing accurate eigensystems of
    scaled diagonally dominant matrices`, SIAM J. Numer. Anal. (27),
    762--791
    """

    fun_name = 'naginterfaces.library.lapackeig.zpteqr'

    _d = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'd'},
        d,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_d, {'fun_name': fun_name, 'entity': 'd'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _e = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'e'},
        e,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _z = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'z'},
        z,
        copy=True,
    )
    _b_lapackeig.zpteqr(
        compz,
        _n,
        _d,
        _e,
        _z,
    )
    _d_py = _d
    _e_py = _e
    _z_py = _z
    _return_data = ZpteqrReturnData(
        _d_py,
        _e_py,
        _z_py,
    )
    return _return_data

ZstedcReturnData = _collections.namedtuple(
    'ZstedcReturnData',
    ['d', 'e', 'z'],
)

def zstedc(compz, d, e, z):
    u"""
    Computes all eigenvalues and, optionally, eigenvectors of a real
    symmetric tridiagonal matrix or a complex Hermitian matrix reduced
    to this form (divide-and-conquer).

    ``zstedc`` computes all the eigenvalues and, optionally, all the
    eigenvectors of a real n*n symmetric tridiagonal matrix, or of a
    complex full or banded Hermitian matrix which has been reduced to
    tridiagonal form.

    For full information please refer to the NAG Library document for
    f08jv

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08jvf.html

    Parameters
    ----------
    compz : str, length 1
        Indicates whether the eigenvectors are to be computed.

        `compz` = 'N'

            Only the eigenvalues are computed (and the array `z` is not
            referenced).

        `compz` = 'V'

            The eigenvalues and eigenvectors of A are computed (and the
            array `z` must contain the matrix Q on entry).

        `compz` = 'I'

            The eigenvalues and eigenvectors of T are computed (and the
            array `z` is initialized by the function).

    d : float, array-like, shape (n)
        The diagonal elements of the tridiagonal matrix.

    e : float, array-like, shape (max(1, n-1))
        The subdiagonal elements of the tridiagonal matrix.

    z : complex, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `compz` in ('V', 'I'): n; otherwise:
        1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compz` in ('V', 'I'): n; otherwise:
        1.

        If `compz` = 'V', `z` must contain the unitary matrix Q used in
        the reduction to tridiagonal form.

    Returns
    -------
    d : float, ndarray, shape (n)
        If no exception or warning is raised, the eigenvalues in
        ascending order.

    e : float, ndarray, shape (max(1, n-1))
        `e` is overwritten.

    z : complex, ndarray, shape (:, :)
        If `compz` = 'V', `z` contains the orthonormal eigenvectors of
        the original Hermitian matrix A, and if `compz` = 'I', `z`
        contains the orthonormal eigenvectors of the symmetric
        tridiagonal matrix T.

        If `compz` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `compz`.

            Constraint: `compz` = 'N', 'V' or 'I'.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` i > 0)
            The algorithm failed to compute an eigenvalue while working
            on the submatrix lying in rows and columns *<value>*/(n+1)
            through mod(*<value>*, n+1).

    Notes
    -----
    ``zstedc`` computes all the eigenvalues and, optionally, the
    eigenvectors of a real symmetric tridiagonal matrix T.
    That is, the function computes the spectral factorization of T given
    by

        T = Z Lambda Z^T,

    where Lambda is a diagonal matrix whose diagonal elements are the
    eigenvalues, lambda_i, of T and Z is an orthogonal matrix whose
    columns are the eigenvectors, z_i, of T.
    Thus

        Tz_i = lambda_i z_i, i = 1,2,...,n.

    The function may also be used to compute all the eigenvalues and
    eigenvectors of a complex full, or banded, Hermitian matrix A which
    has been reduced to real tridiagonal form T as

        A = QTQ^H,

    where Q is unitary.
    The spectral factorization of A is then given by

        A = (QZ)Lambda(QZ)^H.

    In this case Q must be formed explicitly and passed to ``zstedc`` in
    the array `z`, and the function called with `compz` = 'V'. Functions
    which may be called to form T and Q are

    [table omitted]

    When only eigenvalues are required then this function calls
    ``dsterf`` to compute the eigenvalues of the tridiagonal matrix T,
    but when eigenvectors of T are also required and the matrix is not
    too small, then a divide and conquer method is used, which can be
    much faster than ``zsteqr``, although more storage is required.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zstedc'

    _d = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'd'},
        d,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_d, {'fun_name': fun_name, 'entity': 'd'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _e = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'e'},
        e,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _z = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'z'},
        z,
        copy=True,
    )
    _b_lapackeig.zstedc(
        compz,
        _n,
        _d,
        _e,
        _z,
    )
    _d_py = _d
    _e_py = _e
    _z_py = _z
    _return_data = ZstedcReturnData(
        _d_py,
        _e_py,
        _z_py,
    )
    return _return_data

ZsteinReturnData = _collections.namedtuple(
    'ZsteinReturnData',
    ['z', 'ifailv'],
)

def zstein(d, e, m, w, iblock, isplit):
    u"""
    Computes selected eigenvectors of real symmetric tridiagonal matrix
    by inverse iteration, storing eigenvectors in complex array.

    ``zstein`` computes the eigenvectors of a real symmetric tridiagonal
    matrix corresponding to specified eigenvalues, by inverse iteration,
    storing the eigenvectors in a complex array.

    For full information please refer to the NAG Library document for
    f08jx

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08jxf.html

    Parameters
    ----------
    d : float, array-like, shape (n)
        The diagonal elements of the tridiagonal matrix T.

    e : float, array-like, shape (n-1)
        The off-diagonal elements of the tridiagonal matrix T.

    m : int
        m, the number of eigenvectors to be returned.

    w : float, array-like, shape (n)
        The eigenvalues of the tridiagonal matrix T stored in `w`[0] to
        `w`[m-1], as returned by ``dstebz`` with order = 'B'.
        Eigenvalues associated with the first sub-matrix must be
        supplied first, in nondecreasing order; then those associated
        with the second sub-matrix, again in nondecreasing order; and so
        on.

    iblock : int, array-like, shape (n)
        The first m elements must contain the sub-matrix indices
        associated with the specified eigenvalues, as returned by
        ``dstebz`` with order = 'B'. If the eigenvalues were not
        computed by ``dstebz`` with order = 'B', set `iblock`[i-1] to 1,
        for i = 1,2,...,m.

    isplit : int, array-like, shape (n)
        The points at which T breaks up into sub-matrices, as returned
        by ``dstebz`` with order = 'B'. If the eigenvalues were not
        computed by ``dstebz`` with order = 'B', set `isplit`[0] to n.

    Returns
    -------
    z : complex, ndarray, shape (n, `m`)
        The m eigenvectors, stored as columns of Z; the ith column
        corresponds to the ith specified eigenvalue, unless `errno` > 0
        (in which case see [Raises/Warns]).

    ifailv : int, ndarray, shape (`m`)
        If errno = i > 0, the first i elements of `ifailv` contain the
        indices of any eigenvectors which have failed to converge. The
        rest of the first `m` elements of `ifailv` are set to 0.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -4)
            On entry, error in parameter `m`.

            Constraint: 0 <= `m` <= n.

        (`errno` -5)
            On entry, error in parameter `w`.

            Constraint: `w`[i] <= `w`[i+1].

        (`errno` -6)
            On entry, error in parameter `iblock`.

            Constraint: `iblock`[i] <= `iblock`[i+1].

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            *<value>* eigenvectors (as indicated by argument `ifailv`)
            each failed to converge in five iterations. The current
            iterate after five iterations is stored in the corresponding
            column of `z`.

    Notes
    -----
    ``zstein`` computes the eigenvectors of a real symmetric tridiagonal
    matrix T corresponding to specified eigenvalues, by inverse
    iteration (see Jessup and Ipsen (1992)).
    It is designed to be used in particular after the specified
    eigenvalues have been computed by ``dstebz`` with order = 'B', but
    may also be used when the eigenvalues have been computed by other
    functions in submodule ``lapackeig`` or submodule ``eigen``.

    The eigenvectors of T are real, but are stored by this function in a
    complex array.
    If T has been formed by reduction of a full complex Hermitian matrix
    A to tridiagonal form, then eigenvectors of T may be transformed to
    (complex) eigenvectors of A by a call to ``zunmtr`` or ``zupmtr``.

    ``dstebz`` determines whether the matrix T splits into block
    diagonal form:

        T = [table omitted]

    and passes details of the block structure to this function in the
    arrays `iblock` and `isplit`.
    This function can then take advantage of the block structure by
    performing inverse iteration on each block T_i separately, which is
    more efficient than using the whole matrix.

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore

    Jessup, E and Ipsen, I C F, 1992, `Improving the accuracy of inverse
    iteration`, SIAM J. Sci. Statist. Comput. (13), 550--572
    """

    fun_name = 'naginterfaces.library.lapackeig.zstein'

    _d = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'd'},
        d,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_d, {'fun_name': fun_name, 'entity': 'd'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _e = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'e'},
        e,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _w = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'w'},
        w,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _iblock = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'iblock'},
        iblock,
        copy=False,
    )
    _isplit = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'isplit'},
        isplit,
        copy=False,
    )
    _m = _b_utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = _b_utils._EngineIntScalarType.to_py(_m)
    _z = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_n_py, _m_py),
        sorder=_b_utils._EngineData.default_sorder,
    )
    _ifailv = _b_utils._EngineIntArrayType.empty_ndarray(
        (_m_py,),
    )
    try:
        _b_lapackeig.zstein(
            _n,
            _d,
            _e,
            _m,
            _w,
            _iblock,
            _isplit,
            _z,
            _ifailv,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _z_py = _z
    _ifailv_py = _ifailv
    _return_data = ZsteinReturnData(
        _z_py,
        _ifailv_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

ZstegrReturnData = _collections.namedtuple(
    'ZstegrReturnData',
    ['d', 'e', 'm', 'w', 'z', 'isuppz'],
)

def zstegr(jobz, erange, d, e, vl, vu, il, iu):
    u"""
    Computes selected eigenvalues and, optionally, the corresponding
    eigenvectors of a real symmetric tridiagonal matrix or a complex
    Hermitian matrix reduced to this form (Relatively Robust
    Representations).

    ``zstegr`` computes selected eigenvalues and, optionally, the
    corresponding eigenvectors of a real n*n symmetric tridiagonal
    matrix.

    For full information please refer to the NAG Library document for
    f08jy

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08jyf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

            Only eigenvalues are computed.

        `jobz` = 'V'

            Eigenvalues and eigenvectors are computed.

    erange : str, length 1
        Indicates which eigenvalues should be returned.

        `erange` = 'A'

            All eigenvalues will be found.

        `erange` = 'V'

            All eigenvalues in the half-open interval (`vl`,`vu`] will
            be found.

        `erange` = 'I'

            The `il`th through `iu`th eigenvectors will be found.

    d : float, array-like, shape (n)
        The n diagonal elements of the tridiagonal matrix T.

    e : float, array-like, shape (n)
        `e`[0] to `e`[n-2] are the subdiagonal elements of the
        tridiagonal matrix T. `e`[n-1] need not be set.

    vl : float
        If `erange` = 'V', `vl` and `vu` contain the lower and upper
        bounds respectively of the interval to be searched for
        eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    vu : float
        If `erange` = 'V', `vl` and `vu` contain the lower and upper
        bounds respectively of the interval to be searched for
        eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    il : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    iu : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    Returns
    -------
    d : float, ndarray, shape (n)
        `d` is overwritten.

    e : float, ndarray, shape (n)
        `e` is overwritten.

    m : int
        The total number of eigenvalues found. 0 <= `m` <= n.

        If `erange` = 'A', `m` = n.

        If `erange` = 'I', `m` = `iu`-`il`+1.

    w : float, ndarray, shape (n)
        The eigenvalues in ascending order.

    z : complex, ndarray, shape (:, :)
        If `jobz` = 'V', then if no exception or warning is raised, the
        columns of `z` contain the orthonormal eigenvectors of the
        matrix T, with the ith column of Z holding the eigenvector
        associated with `w`[i-1].

        If `jobz` = 'N', `z` is not referenced.

        **Note:** you must ensure that at least max(1, `m`) columns are
        supplied in the array `z`; if `erange` = 'V', the exact value of
        `m` is not known in advance and an upper bound of at least n
        must be used.

    isuppz : int, ndarray, shape (:)
        The support of the eigenvectors in Z, i.e., the indices
        indicating the nonzero elements in Z. The ith eigenvector is
        nonzero only in elements `isuppz`[2*i-2] through
        `isuppz`[2*i-1].

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `erange`.

            Constraint: `erange` = 'A', 'V' or 'I'.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -7)
            On entry, error in parameter `vu`.

            Constraint: `vl` < `vu`.

        (`errno` -9)
            On entry, error in parameter `iu`.

            Constraint: 1 <= `il` <= `iu` <= n.

        (`errno` -9)
            On entry, error in parameter `iu`.

            Constraint: `il` = 1 and `iu` = 0.

        (`errno` (i > 0) and (i = 1))
            The dqds algorithm failed to converge.

        (`errno` (i > 0) and (i = 2))
            Inverse iteration failed to converge.

    Notes
    -----
    ``zstegr`` computes selected eigenvalues and, optionally, the
    corresponding eigenvectors, of a real symmetric tridiagonal matrix
    T.
    That is, the function computes the (partial) spectral factorization
    of T given by

        Z Lambda Z^T,

    where Lambda is a diagonal matrix whose diagonal elements are the
    selected eigenvalues, lambda_i, of T and Z is an orthogonal matrix
    whose columns are the corresponding eigenvectors, z_i, of T.
    Thus

        Tz_i = lambda_i z_i, i = 1,2,...,m

    where m is the number of selected eigenvectors computed.

    The function stores the real orthogonal matrix Z in a complex array,
    so that it may also be used to compute selected eigenvalues and the
    corresponding eigenvectors of a complex Hermitian matrix A which has
    been reduced to tridiagonal form T:

        (QZ)Lambda(QZ)^H, where Q is unitary.

    In this case, the matrix Q must be explicitly applied to the output
    matrix Z.
    The functions which must be called to perform the reduction to
    tridiagonal form and apply Q are:

    [table omitted]

    This function uses the dqds and the Relatively Robust Representation
    algorithms to compute the eigenvalues and eigenvectors respectively;
    see for example Parlett and Dhillon (2000) and Dhillon and Parlett
    (2004) for further details. ``zstegr`` can usually compute all the
    eigenvalues and eigenvectors in O(n^2) floating-point operations and
    so, for large matrices, is often considerably faster than the other
    symmetric tridiagonal functions in this module when all the
    eigenvectors are required, particularly so compared to those
    functions that are based on the QR algorithm.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Barlow, J and Demmel, J W, 1990, `Computing accurate eigensystems of
    scaled diagonally dominant matrices`, SIAM J. Numer. Anal. (27),
    762--791

    Dhillon, I S and Parlett, B N, 2004, `Orthogonal eigenvectors and
    relative gaps`, SIAM J. Appl. Math. (25), 858--899

    Parlett, B N and Dhillon, I S, 2000, `Relatively robust
    representations of symmetric tridiagonals`, Linear Algebra Appl.
    (309), 121--151
    """

    fun_name = 'naginterfaces.library.lapackeig.zstegr'

    _d = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'd'},
        d,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_d, {'fun_name': fun_name, 'entity': 'd'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _e = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'e'},
        e,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobz = _b_utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = _b_utils._EngineCharScalarType.to_py(_jobz)
    _erange = _b_utils._EngineCharScalarType(
        erange, {'fun_name': fun_name, 'entity': 'erange'}, exp_chrlen=1,
    ).to_ctypes()
    _erange_py = _b_utils._EngineCharScalarType.to_py(_erange)
    _iu = _b_utils._EngineIntScalarType(
        iu, {'fun_name': fun_name, 'entity': 'iu'},
    ).to_ctypes()
    _iu_py = _b_utils._EngineIntScalarType.to_py(_iu)
    _il = _b_utils._EngineIntScalarType(
        il, {'fun_name': fun_name, 'entity': 'il'},
    ).to_ctypes()
    _il_py = _b_utils._EngineIntScalarType.to_py(_il)
    if _jobz_py in ('V', 'v'):
        if _erange_py in ('I', 'i'):
            _vl_est_m_py = _iu_py-_il_py+1
        else:
            _vl_est_m_py = _n_py
    else:
        _vl_est_m_py = 0
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    exp_shape_z = (_z_dim1, _vl_est_m_py)
    _z = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_z,
        sorder=_b_utils._EngineData.default_sorder,
    )
    if _jobz_py in ('V', 'v'):
        _isuppz_dim1 = 2*max(1, _vl_est_m_py)
    else:
        _isuppz_dim1 = 0
    exp_shape_isuppz = (_isuppz_dim1,)
    _isuppz = _b_utils._EngineIntArrayType.empty_ndarray(
        exp_shape_isuppz,
    )
    _return_scalar = _b_lapackeig.zstegr(
        jobz,
        erange,
        _n,
        _d,
        _e,
        vl,
        vu,
        _il,
        _iu,
        _w,
        _z,
        _isuppz,
    )
    _d_py = _d
    _e_py = _e
    _m_py = _return_scalar
    _w_py = _w
    if _jobz_py in ('V', 'v'):
        _z_dim2 = _m_py
    else:
        _z_dim2 = 0
    _z_py = _z[:, :_z_dim2]
    if _jobz_py in ('V', 'v'):
        _isuppz_dim1 = 2*_m_py
    else:
        _isuppz_dim1 = 0
    _isuppz_py = _isuppz[:_isuppz_dim1]
    _return_data = ZstegrReturnData(
        _d_py,
        _e_py,
        _m_py,
        _w_py,
        _z_py,
        _isuppz_py,
    )
    return _return_data

DgelssReturnData = _collections.namedtuple(
    'DgelssReturnData',
    ['a', 'b', 's', 'rank'],
)

def dgelss(a, b, rcond):
    u"""
    Computes the minimum-norm solution to a real linear least squares
    problem using singular value decomposition.

    ``dgelss`` computes the minimum norm solution to a real linear least
    squares problem

        min_x||b-Ax||_2.

    For full information please refer to the NAG Library document for
    f08ka

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08kaf.html

    Parameters
    ----------
    a : float, array-like, shape (m, n)
        The m*n matrix A.

    b : float, array-like, shape (max(m, n), nrhs)
        The m*r right-hand side matrix B.

    rcond : float
        Used to determine the effective rank of A. Singular values
        `s`[i-1] <= `rcond`*`s`[0] are treated as zero. If `rcond` < 0,
        machine precision is used instead.

    Returns
    -------
    a : float, ndarray, shape (m, n)
        The first min(m, n) rows of A are overwritten with its right
        singular vectors, stored row-wise.

    b : float, ndarray, shape (max(m, n), nrhs)
        `b` is overwritten by the n*r solution matrix X. If m >= n and
        `rank` = n, the residual sum of squares for the solution in the
        ith column is given by the sum of squares of elements n+1,...,m
        in that column.

    s : float, ndarray, shape (min(m, n))
        The singular values of A in decreasing order.

    rank : int
        The effective rank of A, i.e., the number of singular values
        which are greater than `rcond`*`s`[0].

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -3)
            On entry, error in parameter nrhs.

            Constraint: nrhs >= 0.

        (`errno` i > 0)
            The algorithm for computing the SVD failed to converge;
            *<value>* off-diagonal elements of an intermediate
            bidiagonal form did not converge to zero.

    Notes
    -----
    ``dgelss`` uses the singular value decomposition (SVD) of A, where A
    is an m*n matrix which may be rank-deficient.

    Several right-hand side vectors b and solution vectors x can be
    handled in a single call; they are stored as the columns of the m*r
    right-hand side matrix B and the n*r solution matrix X.

    The effective rank of A is determined by treating as zero those
    singular values which are less than `rcond` times the largest
    singular value.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dgelss'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _nrhs_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[1]
    _nrhs = _b_utils._EngineIntScalarType(
        _nrhs_py, {'fun_name': fun_name, 'entity': '_nrhs'},
    ).to_ctypes()
    _s = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (min(_m_py, _n_py),),
    )
    _return_scalar = _b_lapackeig.dgelss(
        _m,
        _n,
        _nrhs,
        _a,
        _b,
        _s,
        rcond,
    )
    _a_py = _a
    _b_py = _b
    _s_py = _s
    _rank_py = _return_scalar
    _return_data = DgelssReturnData(
        _a_py,
        _b_py,
        _s_py,
        _rank_py,
    )
    return _return_data

DgesvdReturnData = _collections.namedtuple(
    'DgesvdReturnData',
    ['a', 's', 'u', 'vt', 'workb'],
)

def dgesvd(jobu, jobvt, a):
    u"""
    Computes the singular value decomposition of a real matrix,
    optionally computing the left and/or right singular vectors.

    ``dgesvd`` computes the singular value decomposition (SVD) of a real
    m*n matrix A, optionally computing the left and/or right singular
    vectors.

    For full information please refer to the NAG Library document for
    f08kb

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08kbf.html

    Parameters
    ----------
    jobu : str, length 1
        Specifies options for computing all or part of the matrix U.

        `jobu` = 'A'

            All m columns of U are returned in array `u`.

        `jobu` = 'S'

            The first min(m, n) columns of U (the left singular vectors)
            are returned in the array `u`.

        `jobu` = 'O'

            The first min(m, n) columns of U (the left singular vectors)
            are overwritten on the array `a`.

        `jobu` = 'N'

            No columns of U (no left singular vectors) are computed.

    jobvt : str, length 1
        Specifies options for computing all or part of the matrix V^T.

        `jobvt` = 'A'

            All n rows of V^T are returned in the array `vt`.

        `jobvt` = 'S'

            The first min(m, n) rows of V^T (the right singular vectors)
            are returned in the array `vt`.

        `jobvt` = 'O'

            The first min(m, n) rows of V^T (the right singular vectors)
            are overwritten on the array `a`.

        `jobvt` = 'N'

            No rows of V^T (no right singular vectors) are computed.

    a : float, array-like, shape (m, n)
        The m*n matrix A.

    Returns
    -------
    a : float, ndarray, shape (m, n)
        If `jobu` = 'O', `a` is overwritten with the first min(m, n)
        columns of U (the left singular vectors, stored column-wise).

        If `jobvt` = 'O', `a` is overwritten with the first min(m, n)
        rows of V^T (the right singular vectors, stored row-wise).

        If `jobu` != 'O' and `jobvt` != 'O', the contents of `a` are
        destroyed.

    s : float, ndarray, shape (min(m, n))
        The singular values of A, sorted so that `s`[i-1] >= `s`[i].

    u : float, ndarray, shape (:, :)
        If `jobu` = 'A', `u` contains the m*m orthogonal matrix U.

        If `jobu` = 'S', `u` contains the first min(m, n) columns of U
        (the left singular vectors, stored column-wise).

        If `jobu` = 'N' or 'O', `u` is not referenced.

    vt : float, ndarray, shape (:, :)
        If `jobvt` = 'A', `vt` contains the n*n orthogonal matrix V^T.

        If `jobvt` = 'S', `vt` contains the first min(m, n) rows of V^T
        (the right singular vectors, stored row-wise).

        If `jobvt` = 'N' or 'O', `vt` is not referenced.

    workb : float, ndarray, shape (min(m, n))
        If `errno` > 0, `workb`[1:min(m, n)] contains the unconverged
        superdiagonal elements of an upper bidiagonal matrix B whose
        diagonal is in `s` (not necessarily sorted). B satisfies A =
        UBV^T, so it has the same singular values as A, and singular
        vectors related by U and V^T.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobu`.

            Constraint: `jobu` = 'A', 'S', 'O' or 'N'.

        (`errno` -2)
            On entry, error in parameter `jobvt`.

            Constraint: `jobvt` = 'A', 'S', 'O' or 'N'.

        (`errno` -2)
            On entry, error in parameter `jobvt`.

            Constraint: `jobvt` != 'O' or `jobu` != 'O'.

        (`errno` -3)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` i > 0)
            If ``dgesvd`` did not converge, errno specifies how many
            superdiagonals of an intermediate bidiagonal form did not
            converge to zero.

    Notes
    -----
    The SVD is written as

        A = U Sigma V^T,

    where Sigma is an m*n matrix which is zero except for its min(m, n)
    diagonal elements, U is an m*m orthogonal matrix, and V is an n*n
    orthogonal matrix.
    The diagonal elements of Sigma are the singular values of A; they
    are real and non-negative, and are returned in descending order.
    The first min(m, n) columns of U and V are the left and right
    singular vectors of A.

    Note that the function returns V^T, not V.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dgesvd'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'),),
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _s = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (min(_m_py, _n_py),),
    )
    _jobu = _b_utils._EngineCharScalarType(
        jobu, {'fun_name': fun_name, 'entity': 'jobu'}, exp_chrlen=1,
    ).to_ctypes()
    _jobu_py = _b_utils._EngineCharScalarType.to_py(_jobu)
    if _jobu_py in ('A', 'a'):
        _u_dim1 = _m_py
    elif _jobu_py in ('S', 's'):
        _u_dim1 = _m_py
    else:
        _u_dim1 = 1
    if _jobu_py in ('A', 'a'):
        _u_dim2 = _m_py
    elif _jobu_py in ('S', 's'):
        _u_dim2 = min(_m_py, _n_py)
    else:
        _u_dim2 = 1
    exp_shape_u = (_u_dim1, _u_dim2)
    _u = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_u,
        sorder=_m_sorder,
    )
    _jobvt = _b_utils._EngineCharScalarType(
        jobvt, {'fun_name': fun_name, 'entity': 'jobvt'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvt_py = _b_utils._EngineCharScalarType.to_py(_jobvt)
    if _jobvt_py in ('A', 'a'):
        _vt_dim1 = _n_py
    elif _jobvt_py in ('S', 's'):
        _vt_dim1 = min(_m_py, _n_py)
    else:
        _vt_dim1 = 1
    if _jobvt_py in ('A', 'a'):
        _vt_dim2 = _n_py
    elif _jobvt_py in ('S', 's'):
        _vt_dim2 = _n_py
    else:
        _vt_dim2 = 1
    exp_shape_vt = (_vt_dim1, _vt_dim2)
    _vt = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_vt,
        sorder=_m_sorder,
    )
    _workb = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (min(_m_py, _n_py),),
    )
    _b_lapackeig.dgesvd(
        jobu,
        jobvt,
        _m,
        _n,
        _a,
        _s,
        _u,
        _vt,
        _workb,
    )
    _a_py = _a
    _s_py = _s
    _u_py = _u
    _vt_py = _vt
    _workb_py = _workb
    _return_data = DgesvdReturnData(
        _a_py,
        _s_py,
        _u_py,
        _vt_py,
        _workb_py,
    )
    return _return_data

DgelsdReturnData = _collections.namedtuple(
    'DgelsdReturnData',
    ['a', 'b', 's', 'rank'],
)

def dgelsd(a, b, rcond):
    u"""
    Computes the minimum-norm solution to a real linear least squares
    problem using singular value decomposition (divide-and-conquer).

    ``dgelsd`` computes the minimum norm solution to a real linear least
    squares problem

        min_x||b-Ax||_2.

    For full information please refer to the NAG Library document for
    f08kc

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08kcf.html

    Parameters
    ----------
    a : float, array-like, shape (m, n)
        The m*n coefficient matrix A.

    b : float, array-like, shape (max(m, n), nrhs)
        The m*r right-hand side matrix B.

    rcond : float
        Used to determine the effective rank of A. Singular values
        `s`[i-1] <= `rcond`*`s`[0] are treated as zero. If `rcond` < 0,
        machine precision is used instead.

    Returns
    -------
    a : float, ndarray, shape (m, n)
        The contents of `a` are destroyed.

    b : float, ndarray, shape (max(m, n), nrhs)
        `b` is overwritten by the n*r solution matrix X. If m >= n and
        `rank` = n, the residual sum of squares for the solution in the
        ith column is given by the sum of squares of elements n+1,...,m
        in that column.

    s : float, ndarray, shape (min(m, n))
        The singular values of A in decreasing order.

    rank : int
        The effective rank of A, i.e., the number of singular values
        which are greater than `rcond`*`s`[0].

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -3)
            On entry, error in parameter nrhs.

            Constraint: nrhs >= 0.

        (`errno` i > 0)
            The algorithm for computing the SVD failed to converge;
            *<value>* off-diagonal elements of an intermediate
            bidiagonal form did not converge to zero.

    Notes
    -----
    ``dgelsd`` uses the singular value decomposition (SVD) of A, where A
    is a real m*n matrix which may be rank-deficient.

    Several right-hand side vectors b and solution vectors x can be
    handled in a single call; they are stored as the columns of the m*r
    right-hand side matrix B and the n*r solution matrix X.

    The problem is solved in three steps:

    (1) reduce the coefficient matrix A to bidiagonal form with
        Householder transformations, reducing the original problem into
        a 'bidiagonal least squares problem' (BLS);

    (#) solve the BLS using a divide-and-conquer approach;

    (#) apply back all the Householder transformations to solve the
        original least squares problem.

    The effective rank of A is determined by treating as zero those
    singular values which are less than `rcond` times the largest
    singular value.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dgelsd'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _nrhs_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[1]
    _nrhs = _b_utils._EngineIntScalarType(
        _nrhs_py, {'fun_name': fun_name, 'entity': '_nrhs'},
    ).to_ctypes()
    _s = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (min(_m_py, _n_py),),
    )
    _return_scalar = _b_lapackeig.dgelsd(
        _m,
        _n,
        _nrhs,
        _a,
        _b,
        _s,
        rcond,
    )
    _a_py = _a
    _b_py = _b
    _s_py = _s
    _rank_py = _return_scalar
    _return_data = DgelsdReturnData(
        _a_py,
        _b_py,
        _s_py,
        _rank_py,
    )
    return _return_data

DgesddReturnData = _collections.namedtuple(
    'DgesddReturnData',
    ['a', 's', 'u', 'vt'],
)

def dgesdd(jobz, a):
    u"""
    Computes the singular value decomposition of a real matrix,
    optionally computing the left and/or right singular vectors
    (divide-and-conquer).

    ``dgesdd`` computes the singular value decomposition (SVD) of a real
    m*n matrix A, optionally computing the left and/or right singular
    vectors, by using a divide-and-conquer method.

    For full information please refer to the NAG Library document for
    f08kd

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08kdf.html

    Parameters
    ----------
    jobz : str, length 1
        Specifies options for computing all or part of the matrix U.

        `jobz` = 'A'

            All m columns of U and all n rows of V^T are returned in the
            arrays `u` and `vt`.

        `jobz` = 'S'

            The first min(m, n) columns of U and the first min(m, n)
            rows of V^T are returned in the arrays `u` and `vt`.

        `jobz` = 'O'

            If m >= n, the first n columns of U are overwritten on the
            array `a` and all rows of V^T are returned in the array
            `vt`. Otherwise, all columns of U are returned in the array
            `u` and the first m rows of V^T are overwritten in the array
            `vt`.

        `jobz` = 'N'

            No columns of U or rows of V^T are computed.

    a : float, array-like, shape (m, n)
        The m*n matrix A.

    Returns
    -------
    a : float, ndarray, shape (m, n)
        If `jobz` = 'O', `a` is overwritten with the first n columns of
        U (the left singular vectors, stored column-wise) if m >= n; `a`
        is overwritten with the first m rows of V^T (the right singular
        vectors, stored row-wise) otherwise.

        If `jobz` != 'O', the contents of `a` are destroyed.

    s : float, ndarray, shape (min(m, n))
        The singular values of A, sorted so that `s`[i-1] >= `s`[i].

    u : float, ndarray, shape (:, :)
        If `jobz` = 'A' or `jobz` = 'O' and m < n, `u` contains the m*m
        orthogonal matrix U.

        If `jobz` = 'S', `u` contains the first min(m, n) columns of U
        (the left singular vectors, stored column-wise).

        If `jobz` = 'O' and m >= n, or `jobz` = 'N', `u` is not
        referenced.

    vt : float, ndarray, shape (:, :)
        If `jobz` = 'A' or `jobz` = 'O' and m >= n, `vt` contains the
        n*n orthogonal matrix V^T.

        If `jobz` = 'S', `vt` contains the first min(m, n) rows of V^T
        (the right singular vectors, stored row-wise).

        If `jobz` = 'O' and m < n, or `jobz` = 'N', `vt` is not
        referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'A', 'S', 'O' or 'N'.

        (`errno` -2)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` i > 0)
            ``dgesdd`` did not converge, the updating process failed.

    Notes
    -----
    The SVD is written as

        A = U Sigma V^T,

    where Sigma is an m*n matrix which is zero except for its min(m, n)
    diagonal elements, U is an m*m orthogonal matrix, and V is an n*n
    orthogonal matrix.
    The diagonal elements of Sigma are the singular values of A; they
    are real and non-negative, and are returned in descending order.
    The first min(m, n) columns of U and V are the left and right
    singular vectors of A.

    Note that the function returns V^T, not V.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dgesdd'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'),),
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _s = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (min(_m_py, _n_py),),
    )
    _jobz = _b_utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = _b_utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py in ('A', 'a') or (_jobz_py in ('O', 'o') and _m_py < _n_py):
        _u_dim1 = _m_py
    elif _jobz_py in ('S', 's'):
        _u_dim1 = _m_py
    else:
        _u_dim1 = 1
    if _jobz_py in ('A', 'a') or (_jobz_py in ('O', 'o') and _m_py < _n_py):
        _u_dim2 = _m_py
    elif _jobz_py in ('S', 's'):
        _u_dim2 = min(_m_py, _n_py)
    else:
        _u_dim2 = 1
    exp_shape_u = (_u_dim1, _u_dim2)
    _u = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_u,
        sorder=_m_sorder,
    )
    if _jobz_py in ('A', 'a') or (_jobz_py in ('O', 'o') and _m_py >= _n_py):
        _vt_dim1 = _n_py
    elif _jobz_py in ('S', 's'):
        _vt_dim1 = min(_m_py, _n_py)
    else:
        _vt_dim1 = 1
    if _jobz_py in ('A', 'a') or (_jobz_py in ('O', 'o') and _m_py >= _n_py):
        _vt_dim2 = _n_py
    elif _jobz_py in ('S', 's'):
        _vt_dim2 = _n_py
    else:
        _vt_dim2 = 1
    exp_shape_vt = (_vt_dim1, _vt_dim2)
    _vt = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_vt,
        sorder=_m_sorder,
    )
    _b_lapackeig.dgesdd(
        jobz,
        _m,
        _n,
        _a,
        _s,
        _u,
        _vt,
    )
    _a_py = _a
    _s_py = _s
    _u_py = _u
    _vt_py = _vt
    _return_data = DgesddReturnData(
        _a_py,
        _s_py,
        _u_py,
        _vt_py,
    )
    return _return_data

DgebrdReturnData = _collections.namedtuple(
    'DgebrdReturnData',
    ['a', 'd', 'e', 'tauq', 'taup'],
)

def dgebrd(a):
    u"""
    Performs an orthogonal reduction of real general rectangular matrix
    to bidiagonal form.

    ``dgebrd`` reduces a real m*n matrix to bidiagonal form.

    For full information please refer to the NAG Library document for
    f08ke

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08kef.html

    Parameters
    ----------
    a : float, array-like, shape (m, n)
        The m*n matrix A.

    Returns
    -------
    a : float, ndarray, shape (m, n)
        If m >= n, the diagonal and first superdiagonal are overwritten
        by the upper bidiagonal matrix B, elements below the diagonal
        are overwritten by details of the orthogonal matrix Q and
        elements above the first superdiagonal are overwritten by
        details of the orthogonal matrix P.

        If m < n, the diagonal and first subdiagonal are overwritten by
        the lower bidiagonal matrix B, elements below the first
        subdiagonal are overwritten by details of the orthogonal matrix
        Q and elements above the diagonal are overwritten by details of
        the orthogonal matrix P.

    d : float, ndarray, shape (min(m, n))
        The diagonal elements of the bidiagonal matrix B.

    e : float, ndarray, shape (min(m, n)-1)
        The off-diagonal elements of the bidiagonal matrix B.

    tauq : float, ndarray, shape (min(m, n))
        Further details of the orthogonal matrix Q.

    taup : float, ndarray, shape (min(m, n))
        Further details of the orthogonal matrix P.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= 0.

    Notes
    -----
    ``dgebrd`` reduces a real m*n matrix A to bidiagonal form B by an
    orthogonal transformation: A = QBP^T, where Q and P^T are orthogonal
    matrices of order m and n respectively.

    If m >= n, the reduction is given by:

        A = Q(B_1;0)P^T = Q_1 B_1 P^T,

    where B_1 is an n*n upper bidiagonal matrix and Q_1 consists of the
    first n columns of Q.

    If m < n, the reduction is given by

        A = Q(B_1 0)P^T = QB_1 P_1^T,

    where B_1 is an m*m lower bidiagonal matrix and P_1^T consists of
    the first m rows of P^T.

    The orthogonal matrices Q and P are not formed explicitly but are
    represented as products of elementary reflectors (see [the F08
    Introduction] for details). Functions are provided to work with Q
    and P in this representation (see [Further Comments]).

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dgebrd'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _d = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (min(_m_py, _n_py),),
    )
    _e = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (min(_m_py, _n_py)-1,),
    )
    _tauq = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (min(_m_py, _n_py),),
    )
    _taup = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (min(_m_py, _n_py),),
    )
    _b_lapackeig.dgebrd(
        _m,
        _n,
        _a,
        _d,
        _e,
        _tauq,
        _taup,
    )
    _a_py = _a
    _d_py = _d
    _e_py = _e
    _tauq_py = _tauq
    _taup_py = _taup
    _return_data = DgebrdReturnData(
        _a_py,
        _d_py,
        _e_py,
        _tauq_py,
        _taup_py,
    )
    return _return_data

def dorgbr(vect, k, a, tau):
    u"""
    Generates an orthogonal transformation matrices from reduction to
    bidiagonal form determined by ``dgebrd``.

    ``dorgbr`` generates one of the real orthogonal matrices Q or P^T
    which were determined by ``dgebrd`` when reducing a real matrix to
    bidiagonal form.

    For full information please refer to the NAG Library document for
    f08kf

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08kff.html

    Parameters
    ----------
    vect : str, length 1
        Indicates whether the orthogonal matrix Q or P^T is generated.

        `vect` = 'Q'

            Q is generated.

        `vect` = 'P'

            P^T is generated.

    k : int
        If `vect` = 'Q', the number of columns in the original matrix A.

        If `vect` = 'P', the number of rows in the original matrix A.

    a : float, array-like, shape (m, n)
        Details of the vectors which define the elementary reflectors,
        as returned by ``dgebrd``.

    tau : float, array-like, shape (:)
        Note: the required length for this argument is determined as
        follows: if `vect`='Q': max(1, min(m, `k`)); if `vect`='P':
        max(1, min(n, `k`)); otherwise: 0.

        Further details of the elementary reflectors, as returned by
        ``dgebrd`` in its argument tauq if `vect` = 'Q', or in its
        argument taup if `vect` = 'P'.

    Returns
    -------
    a : float, ndarray, shape (m, n)
        The orthogonal matrix Q or P^T, or the leading rows or columns
        thereof, as specified by `vect`, `m` and `n`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `vect`.

            Constraint: `vect` = 'Q' or 'P'.

        (`errno` -2)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: m >= n >= `k`.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: m = n.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= m >= `k`.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n = m.

        (`errno` -4)
            On entry, error in parameter `k`.

            Constraint: `k` >= 0.

    Notes
    -----
    ``dorgbr`` is intended to be used after a call to ``dgebrd``, which
    reduces a real rectangular matrix A to bidiagonal form B by an
    orthogonal transformation: A = QBP^T. ``dgebrd`` represents the
    matrices Q and P^T as products of elementary reflectors.

    This function may be used to generate Q or P^T explicitly as square
    matrices, or in some cases just the leading columns of Q or the
    leading rows of P^T.

    The various possibilities are specified by the arguments `vect`, m,
    n and `k`.
    The appropriate values to cover the most likely cases are as follows
    (assuming that A was an m*n matrix):

    (1) To form the full m*m matrix Q: set `vect` = 'Q', n = m and `k` =
        n (note that the array `a` must have at least m columns).

    (#) If m > n, to form the n leading columns of Q: set `vect` = 'Q',
        and `k` = n

    (#) To form the full n*n matrix P^T: set `vect` = 'P', m = n and `k`
        = m (note that the array `a` must have at least n rows).

    (#) If m < n, to form the m leading rows of P^T: set `vect` = 'P'
        and `k` = m

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dorgbr'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _tau = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'tau'},
        tau,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b_lapackeig.dorgbr(
        vect,
        _m,
        _n,
        k,
        _a,
        _tau,
    )
    _a_py = _a
    return _a_py

def dormbr(vect, side, trans, k, a, tau, c):
    u"""
    Applies the orthogonal transformations from reduction to bidiagonal
    form determined by ``dgebrd``.

    ``dormbr`` multiplies an arbitrary real m*n matrix C by one of the
    real orthogonal matrices Q or P which were determined by ``dgebrd``
    when reducing a real matrix to bidiagonal form.

    For full information please refer to the NAG Library document for
    f08kg

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08kgf.html

    Parameters
    ----------
    vect : str, length 1
        Indicates whether Q or Q^T or P or P^T is to be applied to C.

        `vect` = 'Q'

            Q or Q^T is applied to C.

        `vect` = 'P'

            P or P^T is applied to C.

    side : str, length 1
        Indicates how Q or Q^T or P or P^T is to be applied to C.

        `side` = 'L'

            Q or Q^T or P or P^T is applied to C from the left.

        `side` = 'R'

            Q or Q^T or P or P^T is applied to C from the right.

    trans : str, length 1
        Indicates whether Q or P or Q^T or P^T is to be applied to C.

        `trans` = 'N'

            Q or P is applied to C.

        `trans` = 'T'

            Q^T or P^T is applied to C.

    k : int
        If `vect` = 'Q', the number of columns in the original matrix A.

        If `vect` = 'P', the number of rows in the original matrix A.

    a : float, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `vect`='Q': r; if `vect`='P': min(r,
        `k`); otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `vect`='Q': min(r, `k`); if
        `vect`='P': r; otherwise: 0.

        Details of the vectors which define the elementary reflectors,
        as returned by ``dgebrd``.

    tau : float, array-like, shape (min(r, `k`))
        Further details of the elementary reflectors, as returned by
        ``dgebrd`` in its argument tauq if `vect` = 'Q', or in its
        argument taup if `vect` = 'P'.

    c : float, array-like, shape (m, n)
        The matrix C.

    Returns
    -------
    c : float, ndarray, shape (m, n)
        `c` is overwritten by QC or Q^TC or CQ or C^TQ or PC or P^TC or
        CP or C^TP as specified by `vect`, `side` and `trans`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `vect`.

            Constraint: `vect` = 'Q' or 'P'.

        (`errno` -2)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -3)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'T'.

        (`errno` -4)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -5)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -6)
            On entry, error in parameter `k`.

            Constraint: `k` >= 0.

    Notes
    -----
    ``dormbr`` is intended to be used after a call to ``dgebrd``, which
    reduces a real rectangular matrix A to bidiagonal form B by an
    orthogonal transformation: A = QBP^T. ``dgebrd`` represents the
    matrices Q and P^T as products of elementary reflectors.

    This function may be used to form one of the matrix products

        QC,Q^TC,CQ,CQ^T,PC,P^TC,CP or CP^T,

    overwriting the result on C (which may be any real rectangular
    matrix).

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dormbr'

    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _side = _b_utils._EngineCharScalarType(
        side, {'fun_name': fun_name, 'entity': 'side'}, exp_chrlen=1,
    ).to_ctypes()
    _side_py = _b_utils._EngineCharScalarType.to_py(_side)
    if _side_py in ('L', 'l'):
        _vl_r_py = _m_py
    elif _side_py in ('R', 'r'):
        _vl_r_py = _n_py
    else:
        _vl_r_py = 0
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _tau = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'tau'},
        tau,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b_lapackeig.dormbr(
        vect,
        side,
        trans,
        _m,
        _n,
        k,
        _a,
        _tau,
        _c,
    )
    _c_py = _c
    return _c_py

DgejsvReturnData = _collections.namedtuple(
    'DgejsvReturnData',
    ['a', 'sva', 'u', 'v', 'workb', 'iworkb'],
)

def dgejsv(joba, jobu, jobv, jobr, jobt, jobp, a):
    u"""
    Computes the singular value decomposition of a real matrix,
    optionally computing the left and/or right singular vectors
    (preconditioned Jacobi).

    ``dgejsv`` computes the singular value decomposition (SVD) of a real
    m*n matrix A, where m >= n, and optionally computes the left and/or
    right singular vectors. ``dgejsv`` implements the preconditioned
    Jacobi SVD of Drmač and Veselić (2008a) and Drmač and Veselić
    (2008b).
    This is the expert driver function that calls ``dgesvj`` after
    certain preconditioning.
    In most cases ``dgesvd`` or ``dgesdd``, employing fast scaled
    rotations and de Rijk's pivoting strategy, is sufficient to obtain
    the SVD of a real matrix.
    These are much simpler to use and also handle the case m < n.

    For full information please refer to the NAG Library document for
    f08kh

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08khf.html

    Parameters
    ----------
    joba : str, length 1
        Specifies the form of pivoting for the QR factorization stage;
        whether an estimate of the condition number of the scaled matrix
        is required; and the form of rank reduction that is performed.

        `joba` = 'C'

            The initial QR factorization of the input matrix is
            performed with column pivoting; no estimate of condition
            number is computed; and, the rank is reduced by only the
            underflowed part of the triangular factor R. This option
            works well (high relative accuracy) if A can be written in
            the form A = BD, with well-conditioned B and arbitrary
            diagonal matrix D. The accuracy cannot be spoiled by column
            scaling. The accuracy of the computed output depends on the
            condition of B, and the procedure attempts to achieve the
            best theoretical accuracy.

        `joba` = 'E'

            Computation as with `joba` = 'C' with an additional estimate
            of the condition number of B. It provides a realistic error
            bound.

        `joba` = 'F'

            The initial QR factorization of the input matrix is
            performed with full row and column pivoting; no estimate of
            condition number is computed; and, the rank is reduced by
            only the underflowed part of the triangular factor R. If A =
            D_1*C*D_2 with ill-conditioned diagonal scalings D_1, D_2,
            and well-conditioned matrix C, this option gives higher
            accuracy than the `joba` = 'C' option. If the structure of
            the input matrix is not known, and relative accuracy is
            desirable, then this option is advisable.

        `joba` = 'G'

            Computation as with `joba` = 'F' with an additional estimate
            of the condition number of B, where A = DB (i.e., B =
            C*D_2). If A has heavily weighted rows, then using this
            condition number gives too pessimistic an error bound.

        `joba` = 'A'

            Computation as with `joba` = 'C' except in the treatment of
            rank reduction. In this case, small singular values are to
            be considered as noise and, if found, the matrix is treated
            as numerically rank deficient. The computed SVD, A = U Sigma
            V^T, is such that the relative residual norm (when comparing
            against A) is of the order O(m)*epsilon, where epsilon is
            machine precision. This gives the procedure licence to
            discard (set to zero) all singular values below
            n*epsilon*||A||.

        `joba` = 'R'

            Similar to `joba` = 'A'. The rank revealing property of the
            initial QR factorization is used to reveal (using the upper
            triangular factor) a gap, sigma_{r+1} < epsilonsigma_r, in
            which case the numerical rank is declared to be r. The SVD
            is computed with absolute error bounds, but more accurately
            than with `joba` = 'A'.

    jobu : str, length 1
        Specifies options for computing the left singular vectors U.

        `jobu` = 'U'

            The first n left singular vectors (columns of U) are
            computed and returned in the array `u`.

        `jobu` = 'F'

            All m left singular vectors are computed and returned in the
            array `u`.

        `jobu` = 'W'

            No left singular vectors are computed, but the array `u`
            (with ldu >= m and second dimension at least n) is available
            as workspace for computing right singular values. See the
            description of `u`.

        `jobu` = 'N'

            No left singular vectors are computed. `u` is not referenced
            when `jobv` = 'W' or 'N'.

    jobv : str, length 1
        Specifies options for computing the right singular vectors V.

        `jobv` = 'V'

            The n right singular vectors (columns of V) are computed and
            returned in the array `v`; Jacobi rotations are not
            explicitly accumulated.

        `jobv` = 'J'

            The n right singular vectors (columns of V) are computed and
            returned in the array `v`, but they are computed as the
            product of Jacobi rotations. This option is allowed only if
            `jobu` = 'U' or 'F', i.e., in computing the full SVD.

            This is equivalent to multiplying the input matrix, on the
            right, by the matrix V.

        `jobv` = 'W'

            No right singular values are computed, but the array `v`
            (with ldv >= n and second dimension at least n) is available
            as workspace for computing left singular values. See the
            description of `v`.

        `jobv` = 'N'

            No right singular vectors are computed. `v` is not
            referenced when `jobu` = 'W' or 'N' or `jobt` = 'N' or m !=
            n.

    jobr : str, length 1
        Specifies the conditions under which columns of A are to be set
        to zero. This effectively specifies a lower limit on the range
        of singular values; any singular values below this limit are
        (through column zeroing) set to zero. If A != 0 is scaled so
        that the largest column (in the Euclidean norm) of cA is equal
        to the square root of the overflow threshold, then `jobr` allows
        the function to kill columns of A whose norm in cA is less than
        sqrt(sfmin) (for `jobr` = 'R'), or less than sfmin/epsilon
        (otherwise). sfmin is the safe range parameter, as returned by
        function ``machine.real_safe``.

        `jobr` = 'N'

            Only set to zero those columns of A for which the norm of
            corresponding column of cA < sfmin/epsilon, that is, those
            columns that are effectively zero (to machine precision)
            anyway. If the condition number of A is greater than the
            overflow threshold lambda, where lambda is the value
            returned by ``machine.real_largest``, you are recommended to
            use function ``dgesvj``.

        `jobr` = 'R'

            Set to zero those columns of A for which the norm of the
            corresponding column of cA < sqrt(sfmin). This approximately
            represents a restricted range for sigma(cA) of
            [sqrt(sfmin),sqrt(lambda)].

        For computing the singular values in the full range from the
        safe minimum up to the overflow threshold use ``dgesvj``.

        Suggested value: `jobr` = 'R'.

    jobt : str, length 1
        Specifies, in the case n = m, whether the function is permitted
        to use the transpose of A for improved efficiency. If the matrix
        is square, then the procedure may use A^T if it seems to be
        better with respect to convergence. If the matrix is not square,
        `jobt` is ignored. The decision is based on two values of
        entropy over the adjoint orbit of A^TA. See the descriptions of
        work[5] and work[6].

        `jobt` = 'T'

            If n = m, perform an entropy test and, if the test indicates
            possibly faster convergence of the Jacobi process when using
            A^T, then form the transpose A^T. If A is replaced with A^T,
            then the row pivoting is included automatically.

        `jobt` = 'N'

            No entropy test and no transposition is performed.

        The option `jobt` = 'T' can be used to compute only the singular
        values, or the full SVD (U, Sigma and V).

        In the case where only one set of singular vectors (U or V) is
        required, the caller must still provide both `u` and `v`, as one
        of the matrices is used as workspace if the matrix A is
        transposed.

        See the descriptions of `u` and `v`.

    jobp : str, length 1
        Specifies whether the function should be allowed to introduce
        structured perturbations to drown denormalized numbers. For
        details see Drmač and Veselić (2008a) and Drmač and Veselić
        (2008b). For the sake of simplicity, these perturbations are
        included only when the full SVD or only the singular values are
        requested.

        `jobp` = 'P'

            Introduce perturbation if A is found to be very badly scaled
            (introducing denormalized numbers).

        `jobp` = 'N'

            Do not perturb.

    a : float, array-like, shape (m, n)
        The m*n matrix A.

    Returns
    -------
    a : float, ndarray, shape (m, n)
        The contents of `a` are overwritten.

    sva : float, ndarray, shape (n)
        The, possibly scaled, singular values of A.

        The singular values of A are sigma_i = alpha*`sva`[i-1], for i =
        1,2,...,n, where alpha = work[0]/work[1].

        Normally alpha = 1 and no scaling is required to obtain the
        singular values.

        However, if the largest singular value of A overflows or if
        small singular values have been saved from underflow by scaling
        the input matrix A, then alpha != 1.

        If `jobr` = 'R', then some of the singular values may be
        returned as exact zeros because they are below the numerical
        rank threshold or are denormalized numbers.

    u : float, ndarray, shape (:, :)
        If `jobu` = 'U', `u` contains the m*n matrix of left singular
        vectors.

        If `jobu` = 'F', `u` contains the m*m matrix of left singular
        vectors, including an orthonormal basis of the orthogonal
        complement of Range(A).

        `u` is not referenced when `jobu` = 'W' or 'N' and one of the
        following is satisfied:

            `jobv` = 'W' or 'N', or

            n = 1, or

            A is the zero matrix.

    v : float, ndarray, shape (:, :)
        If `jobv` = 'V' or 'J', `v` contains the n*n matrix of right
        singular vectors.

        `v` is not referenced when `jobv` = 'W' or 'N' and one of the
        following is satisfied:

            `jobu` = 'U' or 'F' and `jobt` = 'T', or

            n = 1, or

            A is the zero matrix.

    workb : float, ndarray, shape (7)
        Contains information about the completed job.

        `workb`[0]

            alpha = `workb`[0]/`workb`[1] is the scaling factor such
            that sigma_i = alpha*`sva`[i-1], for i = 1,2,...,n are the
            computed singular values of A. (See the description of
            `sva`.)

        `workb`[1]

            See the description of `workb`[0].

        `workb`[2]

            `sconda`, an estimate for the condition number of column
            equilibrated A (if `joba` = 'E' or 'G'). `sconda` is an
            estimate of sqrt((||(R^TR)^-1||_1)). It is computed using
            ``lapacklin.dpocon``. It satisfies n^(-1/4)*sconda <=
            ||R^-1||_2 <= n^(1/4)*sconda where R is the triangular
            factor from the QR factorization of A. However, if R is
            truncated and the numerical rank is determined to be
            strictly smaller than n, `sconda` is returned as -1, thus
            indicating that the smallest singular values might be lost.

        If full SVD is needed, and you are familiar with the details of
        the method, the following two condition numbers are useful for
        the analysis of the algorithm.

        `workb`[3]

            An estimate of the scaled condition number of the triangular
            factor in the first QR factorization.

        `workb`[4]

            An estimate of the scaled condition number of the triangular
            factor in the second QR factorization.

        The following two parameters are computed if `jobt` = 'T'.

        `workb`[5]

            The entropy of A^TA: this is the Shannon entropy of
            diag(A^TA)/trace(A^TA) taken as a point in the probability
            simplex.

        `workb`[6]

            The entropy of AA^T.

    iworkb : None or int, ndarray, shape (:)
        Contains information about the completed job.

        `iworkb`[0]

            The numerical rank of A determined after the initial QR
            factorization with pivoting. See the descriptions of `joba`
            and `jobr`.

        `iworkb`[1]

            The number of computed nonzero singular values.

        `iworkb`[2]

            If nonzero, a warning message. If `iworkb`[2] = 1, then some
            of the column norms of A were denormalized (tiny) numbers.
            The requested high accuracy is not warranted by the data.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `joba`.

            Constraint: `joba` = 'C', 'E', 'F', 'G', 'A' or 'R'.

        (`errno` -2)
            On entry, error in parameter `jobu`.

            Constraint: `jobu` = 'U', 'F', 'W' or 'N'.

        (`errno` -3)
            On entry, error in parameter `jobv`.

            Constraint: `jobv` = 'V', 'J', 'W' or 'N'.

        (`errno` -3)
            On entry, error in parameter `jobv`.

            Constraint: `jobv` != 'J'.

        (`errno` -4)
            On entry, error in parameter `jobr`.

            Constraint: `jobr` = 'N' or 'R'.

        (`errno` -5)
            On entry, error in parameter `jobt`.

            Constraint: `jobt` = 'T' or 'N'.

        (`errno` -6)
            On entry, error in parameter `jobp`.

            Constraint: `jobp` = 'P' or 'N'.

        (`errno` -7)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -8)
            On entry, error in parameter n.

            Constraint: m >= n >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            ``dgejsv`` did not converge in the allowed number of
            iterations (30). The computed values might be inaccurate.

    Notes
    -----
    The SVD is written as

        A = U Sigma V^T,

    where Sigma is an m*n matrix which is zero except for its n diagonal
    elements, U is an m*m orthogonal matrix, and V is an n*n orthogonal
    matrix.
    The diagonal elements of Sigma are the singular values of A in
    descending order of magnitude.
    The columns of U and V are the left and the right singular vectors
    of A, respectively.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Drmač, Z and Veselić, K, 2008, `New fast and accurate Jacobi SVD
    Algorithm I`, SIAM J. Matrix Anal. Appl. (29 4)

    Drmač, Z and Veselić, K, 2008, `New fast and accurate Jacobi SVD
    Algorithm II`, SIAM J. Matrix Anal. Appl. (29 4)

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dgejsv'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'),),
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _sva = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobu = _b_utils._EngineCharScalarType(
        jobu, {'fun_name': fun_name, 'entity': 'jobu'}, exp_chrlen=1,
    ).to_ctypes()
    _jobu_py = _b_utils._EngineCharScalarType.to_py(_jobu)
    if _jobu_py in ('F', 'f'):
        _u_dim1 = _m_py
    elif _jobu_py in ('U', 'u', 'W', 'w'):
        _u_dim1 = _m_py
    else:
        _u_dim1 = 1
    if _jobu_py in ('F', 'f'):
        _u_dim2 = _m_py
    elif _jobu_py in ('U', 'u', 'W', 'w'):
        _u_dim2 = _n_py
    else:
        _u_dim2 = 1
    exp_shape_u = (_u_dim1, _u_dim2)
    _u = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_u,
        sorder=_m_sorder,
    )
    _jobv = _b_utils._EngineCharScalarType(
        jobv, {'fun_name': fun_name, 'entity': 'jobv'}, exp_chrlen=1,
    ).to_ctypes()
    _jobv_py = _b_utils._EngineCharScalarType.to_py(_jobv)
    if _jobv_py in ('V', 'v', 'J', 'j', 'W', 'w'):
        _v_dim1 = _n_py
    else:
        _v_dim1 = 1
    if _jobv_py in ('V', 'v', 'J', 'j', 'W', 'w'):
        _v_dim2 = _n_py
    else:
        _v_dim2 = 1
    exp_shape_v = (_v_dim1, _v_dim2)
    _v = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_v,
        sorder=_m_sorder,
    )
    _workb = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (7,),
    )
    if _n_py > 0 and _m_py > 0:
        if _n_py > 0 and _m_py > 0:
            _iworkb_dim1 = 3
        else:
            _iworkb_dim1 = 0
        exp_shape_iworkb = (_iworkb_dim1,)
        _iworkb = _b_utils._EngineIntArrayType.empty_ndarray(
            exp_shape_iworkb,
        )
    else:
        _iworkb = None
    try:
        _b_lapackeig.dgejsv(
            joba,
            jobu,
            jobv,
            jobr,
            jobt,
            jobp,
            _m,
            _n,
            _a,
            _sva,
            _u,
            _v,
            _workb,
            _iworkb,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _a_py = _a
    _sva_py = _sva
    _u_py = _u
    _v_py = _v
    _workb_py = _workb
    if _n_py > 0 and _m_py > 0:
        _iworkb_py = _iworkb
    else:
        _iworkb_py = None
    _return_data = DgejsvReturnData(
        _a_py,
        _sva_py,
        _u_py,
        _v_py,
        _workb_py,
        _iworkb_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

DgesvjReturnData = _collections.namedtuple(
    'DgesvjReturnData',
    ['a', 'sva', 'v', 'work'],
)

def dgesvj(joba, jobu, jobv, a, mv, v, ctol):
    u"""
    Computes the singular value decomposition of a real matrix,
    optionally computing the left and/or right singular vectors (fast
    Jacobi).

    ``dgesvj`` computes the one-sided Jacobi singular value
    decomposition (SVD) of a real m*n matrix A, m >= n, with fast scaled
    rotations and de Rijk’s pivoting, optionally computing the left
    and/or right singular vectors.
    For m < n, the functions ``dgesvd`` or ``dgesdd`` may be used.

    For full information please refer to the NAG Library document for
    f08kj

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08kjf.html

    Parameters
    ----------
    joba : str, length 1
        Specifies the structure of matrix A.

        `joba` = 'L'

            The input matrix A is lower triangular.

        `joba` = 'U'

            The input matrix A is upper triangular.

        `joba` = 'G'

            The input matrix A is a general m*n matrix, m >= n.

    jobu : str, length 1
        Specifies whether to compute the left singular vectors and if so
        whether you want to control their numerical orthogonality
        threshold.

        `jobu` = 'U'

            The left singular vectors corresponding to the nonzero
            singular values are computed and returned in the leading
            columns of `a`. See more details in the description of `a`.
            The numerical orthogonality threshold is set to
            approximately tol = ctol*epsilon, where epsilon is the
            machine precision and ctol = sqrt(m).

        `jobu` = 'C'

            Analogous to `jobu` = 'U', except that you can control the
            level of numerical orthogonality of the computed left
            singular vectors. The orthogonality threshold is set to tol
            = `ctol`*epsilon. The option `jobu` = 'C' can be used if
            m*epsilon is a satisfactory orthogonality of the computed
            left singular vectors, so `ctol` = m could save a few sweeps
            of Jacobi rotations. See the descriptions of `a` and `ctol`.

        `jobu` = 'N'

            The matrix U is not computed. However, see the description
            of `a`.

    jobv : str, length 1
        Specifies whether and how to compute the right singular vectors.

        `jobv` = 'V'

            The matrix V is computed and returned in the array `v`.

        `jobv` = 'A'

            The Jacobi rotations are applied to the leading m_v*n part
            of the array `v`. In other words, the right singular vector
            matrix V is not computed explicitly, instead it is applied
            to an m_v*n matrix initially stored in the first `mv` rows
            of `v`.

        `jobv` = 'N'

            The matrix V is not computed and the array `v` is not
            referenced.

    a : float, array-like, shape (m, n)
        The m*n matrix A.

    mv : int
        If `jobv` = 'A', the product of Jacobi rotations is applied to
        the first m_v rows of `v`.

        If `jobv` != 'A', `mv` is ignored.

        See the description of `jobv`.

    v : float, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobv`='V': n; if `jobv`='A': `mv`;
        otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobv` in ('V', 'A'): n; otherwise: 1.

        If `jobv` = 'A', `v` must contain an m_v*n matrix to be
        premultiplied by the matrix V of right singular vectors.

    ctol : float
        If `jobu` = 'C', `ctol` = ctol, the threshold for convergence.
        The process stops if all columns of A are mutually orthogonal up
        to ctol*epsilon. It is required that ctol >= 1, i.e., it is not
        possible to force the function to obtain orthogonality below
        epsilon. ctol greater than 1/epsilon is meaningless, where
        epsilon is the machine precision.

    Returns
    -------
    a : float, ndarray, shape (m, n)
        The matrix U containing the left singular vectors of A.

        If `jobu` = 'U' or 'C'

            if errno = 0

                rank(A) orthonormal columns of U are returned in the
                leading rank(A) columns of the array `a`. Here rank(A)
                <= n is the number of computed singular values of A that
                are above the safe range parameter, as returned by
                ``machine.real_safe``. The singular vectors
                corresponding to underflowed or zero singular values are
                not computed. The value of rank(A) is returned by
                rounding `work`[1] to the nearest whole number. Also see
                the descriptions of `sva` and `work`. The computed
                columns of U are mutually numerically orthogonal up to
                approximately tol = sqrt(m)*epsilon; or tol =
                `ctol`*epsilon (`jobu` = 'C'), where epsilon is the
                machine precision, see the description of `jobu`.

            if errno > 0

                ``dgesvj`` did not converge in 30 iterations (sweeps).
                In this case, the computed columns of U may not be
                orthogonal up to tol. The output U (stored in `a`),
                Sigma (given by the computed singular values in `sva`)
                and V is still a decomposition of the input matrix A in
                the sense that the residual
                ||A-alpha*U*Sigma*V^T||_2/||A||_2 is small, where alpha
                is the value returned in `work`[0].

        If `jobu` = 'N'

            if errno = 0

                Note that the left singular vectors are 'for free' in
                the one-sided Jacobi SVD algorithm. However, if only the
                singular values are needed, the level of numerical
                orthogonality of U is not an issue and iterations are
                stopped when the columns of the iterated matrix are
                numerically orthogonal up to approximately m*epsilon.
                Thus, on exit, `a` contains the columns of U scaled with
                the corresponding singular values.

            if errno > 0

                ``dgesvj`` did not converge in 30 iterations (sweeps).

    sva : float, ndarray, shape (n)
        The, possibly scaled, singular values of A.

        If errno = 0

            The singular values of A are sigma_i = alpha`sva`[i-1], for
            i = 1,2,...,n, where alpha is the scale factor stored in
            `work`[0]. Normally alpha = 1, however, if some of the
            singular values of A might underflow or overflow, then alpha
            != 1 and the scale factor needs to be applied to obtain the
            singular values.

        If errno > 0

            ``dgesvj`` did not converge in 30 iterations and alpha*`sva`
            may not be accurate.

    v : float, ndarray, shape (:, :)
        The right singular vectors of A.

        If `jobv` = 'V', `v` contains the n*n matrix of the right
        singular vectors.

        If `jobv` = 'A', `v` contains the product of the computed right
        singular vector matrix and the initial matrix in the array `v`.

        If `jobv` = 'N', `v` is not referenced.

    work : float, ndarray, shape (6)
        Contains information about the completed job.

        `work`[0]

            The scaling factor, alpha, such that sigma_i =
            alpha`sva`[i-1], for i = 1,2,...,n are the computed singular
            values of A. (See the description of `sva`.)

        `work`[1]

            nint(`work`[1]) gives the number of the computed nonzero
            singular values.

        `work`[2]

            nint(`work`[2]) gives the number of the computed singular
            values that are larger than the underflow threshold.

        `work`[3]

            nint(`work`[3]) gives the number of iterations (sweeps of
            Jacobi rotations) needed for numerical convergence.

        `work`[4]

            max_{i != j}|cos(A(:,i), A(:,j))| in the last iteration
            (sweep). This is useful information in cases when ``dgesvj``
            did not converge, as it can be used to estimate whether the
            output is still useful and for subsequent analysis.

        `work`[5]

            The largest absolute value over all sines of the Jacobi
            rotation angles in the last sweep. It can be useful for
            subsequent analysis.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `joba`.

            Constraint: `joba` = 'L', 'U' or 'G'.

        (`errno` -2)
            On entry, error in parameter `jobu`.

            Constraint: `jobu` = 'U', 'C' or 'N'.

        (`errno` -3)
            On entry, error in parameter `jobv`.

            Constraint: `jobv` = 'V', 'A' or 'N'.

        (`errno` -4)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -5)
            On entry, error in parameter n.

            Constraint: m >= n >= 0.

        (`errno` -9)
            On entry, error in parameter `mv`.

            Constraint: `mv` >= 0.

        (`errno` -12)
            On entry, error in parameter `ctol`.

            Constraint: `ctol` >= 1.0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            ``dgesvj`` did not converge in the allowed number of
            iterations (30), but its output might still be useful.

    Notes
    -----
    The SVD is written as

        A = U Sigma V^T,

    where Sigma is an n*n diagonal matrix, U is an m*n orthonormal
    matrix, and V is an n*n orthogonal matrix.
    The diagonal elements of Sigma are the singular values of A in
    descending order of magnitude.
    The columns of U and V are the left and the right singular vectors
    of A.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Drmač, Z and Veselić, K, 2008, `New fast and accurate Jacobi SVD
    Algorithm I`, SIAM J. Matrix Anal. Appl. (29 4)

    Drmač, Z and Veselić, K, 2008, `New fast and accurate Jacobi SVD
    Algorithm II`, SIAM J. Matrix Anal. Appl. (29 4)

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dgesvj'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _sva = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _v = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'v'},
        v,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _work = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (6,),
    )
    try:
        _b_lapackeig.dgesvj(
            joba,
            jobu,
            jobv,
            _m,
            _n,
            _a,
            _sva,
            mv,
            _v,
            ctol,
            _work,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _a_py = _a
    _v_py = _v
    _sva_py = _sva
    _work_py = _work
    _return_data = DgesvjReturnData(
        _a_py,
        _sva_py,
        _v_py,
        _work_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

DgesvdxReturnData = _collections.namedtuple(
    'DgesvdxReturnData',
    ['a', 'ns', 's', 'u', 'vt', 'workb', 'jfail'],
)

def dgesvdx(jobu, jobvt, erange, a, vl, vu, il, iu):
    u"""
    Computes all or selected singular values of the singular value
    decomposition of a real general matrix, optionally computing the
    corresponding left and right singular vectors.

    ``dgesvdx`` computes the singular value decomposition (SVD) of a
    real m*n matrix A, optionally computing the left and/or right
    singular vectors.
    All singular values or a selected set of singular values may be
    computed.

    For full information please refer to the NAG Library document for
    f08km

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08kmf.html

    Parameters
    ----------
    jobu : str, length 1
        Specifies options for computing all or part of the matrix U.

        `jobu` = 'V'

            The `ns` columns of U, as specified by `erange`, are
            returned in array `u`.

        `jobu` = 'N'

            No columns of U (no left singular vectors) are computed.

    jobvt : str, length 1
        Specifies options for computing all or part of the matrix V^T.

        `jobvt` = 'V'

            The `ns` rows of V^T, as specified by `erange`, are returned
            in the array `vt`.

        `jobvt` = 'N'

            No rows of V^T (no right singular vectors) are computed.

    erange : str, length 1
        Indicates which singular values should be returned.

        `erange` = 'A'

            All singular values will be found.

        `erange` = 'V'

            All singular values in the half-open interval (`vl`,`vu`]
            will be found.

        `erange` = 'I'

            The `il`th through `iu`th singular values will be found.

    a : float, array-like, shape (m, n)
        The m*n matrix A.

    vl : float
        If `erange` = 'V', the lower bound of the interval to be
        searched for singular values.

        If `erange` = 'A' or 'I', `vl` is not referenced.

    vu : float
        If `erange` = 'V', the upper bound of the interval to be
        searched for singular values.

        If `erange` = 'A' or 'I', `vu` is not referenced.

    il : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest singular values to
        be returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    iu : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest singular values to
        be returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    Returns
    -------
    a : float, ndarray, shape (m, n)
        If `jobu` != 'N' and `jobvt` != 'N', the contents of `a` are
        destroyed.

    ns : int
        The total number of singular values found. 0 <= `ns` <= min(m,
        n).

        If `erange` = 'A', `ns` = min(m, n).

        If `erange` = 'I', `ns` = `iu`-`il`+1.

        If `erange` = 'V' then the value of `ns` is not known in advance
        and so an upper limit should be used when specifying the
        dimensions of array `u`, e.g., min(m, n).

    s : float, ndarray, shape (min(m, n))
        The singular values of A, sorted so that `s`[i-1] >= `s`[i].

    u : None or float, ndarray, shape (:, :)
        If `jobu` = 'V', `u` contains the first `ns` columns of U (the
        left singular vectors, stored column-wise).

        If `jobu` = 'N', `u` is not referenced.

    vt : None or float, ndarray, shape (:, :)
        If `jobvt` = 'V', `vt` contains the first `ns` rows of V^T (the
        right singular vectors, stored row-wise).

        If `jobvt` = 'N', `vt` is not referenced.

    workb : float, ndarray, shape (min(m, n))
        If `errno` > 0, `workb`[1:min(m, n)] contains the unconverged
        superdiagonal elements of an upper bidiagonal matrix B whose
        diagonal is in `s` (not necessarily sorted). B satisfies A =
        UBV^T, so it has the same singular values as A, and singular
        vectors related by U and V^T.

    jfail : int, ndarray, shape (2*min(m, n))
        If `errno` > 0, `jfail` contains, in its first k nonzero
        elements, the indices of the k eigenvectors (associated with a
        left or right singular vector, see ``dbdsvdx``) that failed to
        converge.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobu`.

            Constraint: `jobu` = 'V' or 'N'.

        (`errno` -2)
            On entry, error in parameter `jobvt`.

            Constraint: `jobvt` = 'V' or 'N'.

        (`errno` -3)
            On entry, error in parameter `erange`.

            Constraint: `erange` = 'A', 'V' or 'I'.

        (`errno` -4)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -5)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -8)
            On entry, error in parameter `vl`.

            Constraint: 0.0 <= `vl`.

        (`errno` -9)
            On entry, error in parameter `vu`.

            Constraint: `vl` < `vu`.

        (`errno` -10)
            On entry, error in parameter `il`.

        (`errno` -11)
            On entry, error in parameter `iu`.

        (`errno` i > 0)
            If ``dgesvdx`` did not converge, errno specifies how many
            superdiagonals of an intermediate bidiagonal form did not
            converge to zero.

    Notes
    -----
    The SVD is written as

        A = U Sigma V^T,

    where Sigma is an m*n matrix which is zero except for its min(m, n)
    diagonal elements, U is an m*m orthogonal matrix, and V is an n*n
    orthogonal matrix.
    The diagonal elements of Sigma are the singular values of A; they
    are real and non-negative, and are returned in descending order.
    The first min(m, n) columns of U and V are the left and right
    singular vectors of A, respectively.

    Note that the function returns V^T, not V.

    Alternative to computing all singular values of A, a selected set
    can be computed.
    The set is either those singular values lying in a given interval,
    sigma in (v_l,v_u], or those whose index (counting from largest to
    smallest in magnitude) lies in a given range 1 <= i_l,...,i_u <= n.
    In these cases, the corresponding left and right singular vectors
    can optionally be computed.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dgesvdx'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'),),
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _s = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (min(_m_py, _n_py),),
    )
    _erange = _b_utils._EngineCharScalarType(
        erange, {'fun_name': fun_name, 'entity': 'erange'}, exp_chrlen=1,
    ).to_ctypes()
    _erange_py = _b_utils._EngineCharScalarType.to_py(_erange)
    _iu = _b_utils._EngineIntScalarType(
        iu, {'fun_name': fun_name, 'entity': 'iu'},
    ).to_ctypes()
    _iu_py = _b_utils._EngineIntScalarType.to_py(_iu)
    _il = _b_utils._EngineIntScalarType(
        il, {'fun_name': fun_name, 'entity': 'il'},
    ).to_ctypes()
    _il_py = _b_utils._EngineIntScalarType.to_py(_il)
    if _erange_py in ('I', 'i'):
        _vl_nsmax_py = _iu_py-_il_py+1
    elif _erange_py in ('A', 'a', 'V', 'v'):
        _vl_nsmax_py = min(_m_py, _n_py)
    else:
        _vl_nsmax_py = 0
    _jobu = _b_utils._EngineCharScalarType(
        jobu, {'fun_name': fun_name, 'entity': 'jobu'}, exp_chrlen=1,
    ).to_ctypes()
    _jobu_py = _b_utils._EngineCharScalarType.to_py(_jobu)
    if _jobu_py in ('V', 'v'):
        if _jobu_py in ('V', 'v'):
            _u_dim1 = _m_py
        else:
            _u_dim1 = 1
        if _jobu_py in ('V', 'v'):
            _u_dim2 = _vl_nsmax_py
        else:
            _u_dim2 = 0
        exp_shape_u = (_u_dim1, _u_dim2)
        _u = _b_utils._EngineFloat64ArrayType.empty_ndarray(
            exp_shape_u,
            sorder=_m_sorder,
        )
    else:
        _u = None
    _jobvt = _b_utils._EngineCharScalarType(
        jobvt, {'fun_name': fun_name, 'entity': 'jobvt'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvt_py = _b_utils._EngineCharScalarType.to_py(_jobvt)
    if _jobvt_py in ('V', 'v'):
        if _jobvt_py in ('V', 'v'):
            _vt_dim1 = min(_m_py, _n_py)
        else:
            _vt_dim1 = 1
        if _jobvt_py in ('V', 'v'):
            _vt_dim2 = _n_py
        else:
            _vt_dim2 = 0
        exp_shape_vt = (_vt_dim1, _vt_dim2)
        _vt = _b_utils._EngineFloat64ArrayType.empty_ndarray(
            exp_shape_vt,
            sorder=_m_sorder,
        )
    else:
        _vt = None
    _workb = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (min(_m_py, _n_py),),
    )
    _jfail = _b_utils._EngineIntArrayType.empty_ndarray(
        (2*min(_m_py, _n_py),),
    )
    _return_scalar = _b_lapackeig.dgesvdx(
        jobu,
        jobvt,
        erange,
        _m,
        _n,
        _a,
        vl,
        vu,
        _il,
        _iu,
        _s,
        _u,
        _vt,
        _workb,
        _jfail,
    )
    _a_py = _a
    _ns_py = _return_scalar
    _s_py = _s
    if _jobu_py in ('V', 'v'):
        _u_py = _u
    else:
        _u_py = None
    if _jobvt_py in ('V', 'v'):
        _vt_py = _vt
    else:
        _vt_py = None
    _workb_py = _workb
    _jfail_py = _jfail
    _return_data = DgesvdxReturnData(
        _a_py,
        _ns_py,
        _s_py,
        _u_py,
        _vt_py,
        _workb_py,
        _jfail_py,
    )
    return _return_data

ZgelssReturnData = _collections.namedtuple(
    'ZgelssReturnData',
    ['a', 'b', 's', 'rank'],
)

def zgelss(a, b, rcond):
    u"""
    Computes the minimum-norm solution to a complex linear least squares
    problem using singular value decomposition.

    ``zgelss`` computes the minimum norm solution to a complex linear
    least squares problem

        min_x||b-Ax||_2.

    For full information please refer to the NAG Library document for
    f08kn

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08knf.html

    Parameters
    ----------
    a : complex, array-like, shape (m, n)
        The m*n matrix A.

    b : complex, array-like, shape (max(m, n), nrhs)
        The m*r right-hand side matrix B.

    rcond : float
        Used to determine the effective rank of A. Singular values
        `s`[i-1] <= `rcond`*`s`[0] are treated as zero. If `rcond` < 0,
        machine precision is used instead.

    Returns
    -------
    a : complex, ndarray, shape (m, n)
        The first min(m, n) rows of A are overwritten with its right
        singular vectors, stored row-wise.

    b : complex, ndarray, shape (max(m, n), nrhs)
        `b` is overwritten by the n*r solution matrix X. If m >= n and
        `rank` = n, the residual sum of squares for the solution in the
        ith column is given by the sum of squares of the modulus of
        elements n+1,...,m in that column.

    s : float, ndarray, shape (min(m, n))
        The singular values of A in decreasing order.

    rank : int
        The effective rank of A, i.e., the number of singular values
        which are greater than `rcond`*`s`[0].

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -3)
            On entry, error in parameter nrhs.

            Constraint: nrhs >= 0.

        (`errno` i > 0)
            The algorithm for computing the SVD failed to converge;
            *<value>* off-diagonal elements of an intermediate
            bidiagonal form did not converge to zero.

    Notes
    -----
    ``zgelss`` uses the singular value decomposition (SVD) of A, where A
    is an m*n matrix which may be rank-deficient.

    Several right-hand side vectors b and solution vectors x can be
    handled in a single call; they are stored as the columns of the m*r
    right-hand side matrix B and the n*r solution matrix X.

    The effective rank of A is determined by treating as zero those
    singular values which are less than `rcond` times the largest
    singular value.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zgelss'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
    )
    _nrhs_py = _b_utils._EngineComplex128ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[1]
    _nrhs = _b_utils._EngineIntScalarType(
        _nrhs_py, {'fun_name': fun_name, 'entity': '_nrhs'},
    ).to_ctypes()
    _s = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (min(_m_py, _n_py),),
    )
    _return_scalar = _b_lapackeig.zgelss(
        _m,
        _n,
        _nrhs,
        _a,
        _b,
        _s,
        rcond,
    )
    _a_py = _a
    _b_py = _b
    _s_py = _s
    _rank_py = _return_scalar
    _return_data = ZgelssReturnData(
        _a_py,
        _b_py,
        _s_py,
        _rank_py,
    )
    return _return_data

ZgesvdReturnData = _collections.namedtuple(
    'ZgesvdReturnData',
    ['a', 's', 'u', 'vt', 'rworkb'],
)

def zgesvd(jobu, jobvt, a):
    u"""
    Computes the singular value decomposition of a complex matrix,
    optionally computing the left and/or right singular vectors.

    ``zgesvd`` computes the singular value decomposition (SVD) of a
    complex m*n matrix A, optionally computing the left and/or right
    singular vectors.

    For full information please refer to the NAG Library document for
    f08kp

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08kpf.html

    Parameters
    ----------
    jobu : str, length 1
        Specifies options for computing all or part of the matrix U.

        `jobu` = 'A'

            All m columns of U are returned in array `u`.

        `jobu` = 'S'

            The first min(m, n) columns of U (the left singular vectors)
            are returned in the array `u`.

        `jobu` = 'O'

            The first min(m, n) columns of U (the left singular vectors)
            are overwritten on the array `a`.

        `jobu` = 'N'

            No columns of U (no left singular vectors) are computed.

    jobvt : str, length 1
        Specifies options for computing all or part of the matrix V^H.

        `jobvt` = 'A'

            All n rows of V^H are returned in the array `vt`.

        `jobvt` = 'S'

            The first min(m, n) rows of V^H (the right singular vectors)
            are returned in the array `vt`.

        `jobvt` = 'O'

            The first min(m, n) rows of V^H (the right singular vectors)
            are overwritten on the array `a`.

        `jobvt` = 'N'

            No rows of V^H (no right singular vectors) are computed.

    a : complex, array-like, shape (m, n)
        The m*n matrix A.

    Returns
    -------
    a : complex, ndarray, shape (m, n)
        If `jobu` = 'O', `a` is overwritten with the first min(m, n)
        columns of U (the left singular vectors, stored column-wise).

        If `jobvt` = 'O', `a` is overwritten with the first min(m, n)
        rows of V^H (the right singular vectors, stored row-wise).

        If `jobu` != 'O' and `jobvt` != 'O', the contents of `a` are
        destroyed.

    s : float, ndarray, shape (min(m, n))
        The singular values of A, sorted so that `s`[i-1] >= `s`[i].

    u : complex, ndarray, shape (:, :)
        If `jobu` = 'A', `u` contains the m*m unitary matrix U.

        If `jobu` = 'S', `u` contains the first min(m, n) columns of U
        (the left singular vectors, stored column-wise).

        If `jobu` = 'N' or 'O', `u` is not referenced.

    vt : complex, ndarray, shape (:, :)
        If `jobvt` = 'A', `vt` contains the n*n unitary matrix V^H.

        If `jobvt` = 'S', `vt` contains the first min(m, n) rows of V^H
        (the right singular vectors, stored row-wise).

        If `jobvt` = 'N' or 'O', `vt` is not referenced.

    rworkb : float, ndarray, shape (min(m, n))
        If `errno` > 0, `rworkb`[0:min(m, n)-1] contains the unconverged
        superdiagonal elements of an upper bidiagonal matrix B whose
        diagonal is in S (not necessarily sorted). B satisfies A =
        UBV^T, so it has the same singular values as A, and singular
        vectors related by U and V^T.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobu`.

            Constraint: `jobu` = 'A', 'S', 'O' or 'N'.

        (`errno` -2)
            On entry, error in parameter `jobvt`.

            Constraint: `jobvt` = 'A', 'S', 'O' or 'N'.

        (`errno` -2)
            On entry, error in parameter `jobvt`.

            Constraint: `jobvt` != 'O' or `jobu` != 'O'.

        (`errno` -3)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` i > 0)
            If ``zgesvd`` did not converge, errno specifies how many
            superdiagonals of an intermediate bidiagonal form did not
            converge to zero.

    Notes
    -----
    The SVD is written as

        A = U Sigma V^H,

    where Sigma is an m*n matrix which is zero except for its min(m, n)
    diagonal elements, U is an m*m unitary matrix, and V is an n*n
    unitary matrix.
    The diagonal elements of Sigma are the singular values of A; they
    are real and non-negative, and are returned in descending order.
    The first min(m, n) columns of U and V are the left and right
    singular vectors of A.

    Note that the function returns V^H, not V.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zgesvd'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'),),
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _s = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (min(_m_py, _n_py),),
    )
    _jobu = _b_utils._EngineCharScalarType(
        jobu, {'fun_name': fun_name, 'entity': 'jobu'}, exp_chrlen=1,
    ).to_ctypes()
    _jobu_py = _b_utils._EngineCharScalarType.to_py(_jobu)
    if _jobu_py in ('A', 'a'):
        _u_dim1 = _m_py
    elif _jobu_py in ('S', 's'):
        _u_dim1 = _m_py
    else:
        _u_dim1 = 1
    if _jobu_py in ('A', 'a'):
        _u_dim2 = _m_py
    elif _jobu_py in ('S', 's'):
        _u_dim2 = min(_m_py, _n_py)
    else:
        _u_dim2 = 1
    exp_shape_u = (_u_dim1, _u_dim2)
    _u = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_u,
        sorder=_m_sorder,
    )
    _jobvt = _b_utils._EngineCharScalarType(
        jobvt, {'fun_name': fun_name, 'entity': 'jobvt'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvt_py = _b_utils._EngineCharScalarType.to_py(_jobvt)
    if _jobvt_py in ('A', 'a'):
        _vt_dim1 = _n_py
    elif _jobvt_py in ('S', 's'):
        _vt_dim1 = min(_m_py, _n_py)
    else:
        _vt_dim1 = 1
    if _jobvt_py in ('A', 'a', 'S', 's'):
        _vt_dim2 = _n_py
    else:
        _vt_dim2 = 1
    exp_shape_vt = (_vt_dim1, _vt_dim2)
    _vt = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_vt,
        sorder=_m_sorder,
    )
    _rworkb = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (min(_m_py, _n_py),),
    )
    _b_lapackeig.zgesvd(
        jobu,
        jobvt,
        _m,
        _n,
        _a,
        _s,
        _u,
        _vt,
        _rworkb,
    )
    _a_py = _a
    _s_py = _s
    _u_py = _u
    _vt_py = _vt
    _rworkb_py = _rworkb
    _return_data = ZgesvdReturnData(
        _a_py,
        _s_py,
        _u_py,
        _vt_py,
        _rworkb_py,
    )
    return _return_data

ZgelsdReturnData = _collections.namedtuple(
    'ZgelsdReturnData',
    ['a', 'b', 's', 'rank'],
)

def zgelsd(a, b, rcond):
    u"""
    Computes the minimum-norm solution to a complex linear least squares
    problem using singular value decomposition (divide-and-conquer).

    ``zgelsd`` computes the minimum norm solution to a complex linear
    least squares problem

        min_x||b-Ax||_2.

    For full information please refer to the NAG Library document for
    f08kq

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08kqf.html

    Parameters
    ----------
    a : complex, array-like, shape (m, n)
        The m*n coefficient matrix A.

    b : complex, array-like, shape (max(m, n), nrhs)
        The m*r right-hand side matrix B.

    rcond : float
        Used to determine the effective rank of A. Singular values
        `s`[i-1] <= `rcond`*`s`[0] are treated as zero. If `rcond` < 0,
        machine precision is used instead.

    Returns
    -------
    a : complex, ndarray, shape (m, n)
        The contents of `a` are destroyed.

    b : complex, ndarray, shape (max(m, n), nrhs)
        `b` is overwritten by the n*r solution matrix X. If m >= n and
        `rank` = n, the residual sum of squares for the solution in the
        ith column is given by the sum of squares of the modulus of
        elements n+1,...,m in that column.

    s : float, ndarray, shape (min(m, n))
        The singular values of A in decreasing order.

    rank : int
        The effective rank of A, i.e., the number of singular values
        which are greater than `rcond`*`s`[0].

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -3)
            On entry, error in parameter nrhs.

            Constraint: nrhs >= 0.

        (`errno` i > 0)
            The algorithm for computing the SVD failed to converge;
            *<value>* off-diagonal elements of an intermediate
            bidiagonal form did not converge to zero.

    Notes
    -----
    ``zgelsd`` uses the singular value decomposition (SVD) of A, where A
    is a complex m*n matrix which may be rank-deficient.

    Several right-hand side vectors b and solution vectors x can be
    handled in a single call; they are stored as the columns of the m*r
    right-hand side matrix B and the n*r solution matrix X.

    The problem is solved in three steps:

    (1) reduce the coefficient matrix A to bidiagonal form with
        Householder transformations, reducing the original problem into
        a 'bidiagonal least squares problem' (BLS);

    (#) solve the BLS using a divide-and-conquer approach;

    (#) apply back all the Householder transformations to solve the
        original least squares problem.

    The effective rank of A is determined by treating as zero those
    singular values which are less than `rcond` times the largest
    singular value.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zgelsd'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
    )
    _nrhs_py = _b_utils._EngineComplex128ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[1]
    _nrhs = _b_utils._EngineIntScalarType(
        _nrhs_py, {'fun_name': fun_name, 'entity': '_nrhs'},
    ).to_ctypes()
    _s = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (min(_m_py, _n_py),),
    )
    _return_scalar = _b_lapackeig.zgelsd(
        _m,
        _n,
        _nrhs,
        _a,
        _b,
        _s,
        rcond,
    )
    _a_py = _a
    _b_py = _b
    _s_py = _s
    _rank_py = _return_scalar
    _return_data = ZgelsdReturnData(
        _a_py,
        _b_py,
        _s_py,
        _rank_py,
    )
    return _return_data

ZgesddReturnData = _collections.namedtuple(
    'ZgesddReturnData',
    ['a', 's', 'u', 'vt'],
)

def zgesdd(jobz, a):
    u"""
    Computes the singular value decomposition of a complex matrix,
    optionally computing the left and/or right singular vectors
    (divide-and-conquer).

    ``zgesdd`` computes the singular value decomposition (SVD) of a
    complex m*n matrix A, optionally computing the left and/or right
    singular vectors, by using a divide-and-conquer method.

    For full information please refer to the NAG Library document for
    f08kr

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08krf.html

    Parameters
    ----------
    jobz : str, length 1
        Specifies options for computing all or part of the matrix U.

        `jobz` = 'A'

            All m columns of U and all n rows of V^H are returned in the
            arrays `u` and `vt`.

        `jobz` = 'S'

            The first min(m, n) columns of U and the first min(m, n)
            rows of V^H are returned in the arrays `u` and `vt`.

        `jobz` = 'O'

            If m >= n, the first n columns of U are overwritten on the
            array `a` and all rows of V^H are returned in the array
            `vt`. Otherwise, all columns of U are returned in the array
            `u` and the first m rows of V^H are overwritten in the array
            `vt`.

        `jobz` = 'N'

            No columns of U or rows of V^H are computed.

    a : complex, array-like, shape (m, n)
        The m*n matrix A.

    Returns
    -------
    a : complex, ndarray, shape (m, n)
        If `jobz` = 'O', `a` is overwritten with the first n columns of
        U (the left singular vectors, stored column-wise) if m >= n; `a`
        is overwritten with the first m rows of V^H (the right singular
        vectors, stored row-wise) otherwise.

        If `jobz` != 'O', the contents of `a` are destroyed.

    s : float, ndarray, shape (min(m, n))
        The singular values of A, sorted so that `s`[i-1] >= `s`[i].

    u : complex, ndarray, shape (:, :)
        If `jobz` = 'A' or `jobz` = 'O' and m < n, `u` contains the m*m
        unitary matrix U.

        If `jobz` = 'S', `u` contains the first min(m, n) columns of U
        (the left singular vectors, stored column-wise).

        If `jobz` = 'O' and m >= n, or `jobz` = 'N', `u` is not
        referenced.

    vt : complex, ndarray, shape (:, :)
        If `jobz` = 'A' or `jobz` = 'O' and m >= n, `vt` contains the
        n*n unitary matrix V^H.

        If `jobz` = 'S', `vt` contains the first min(m, n) rows of V^H
        (the right singular vectors, stored row-wise).

        If `jobz` = 'O' and m < n, or `jobz` = 'N', `vt` is not
        referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'A', 'S', 'O' or 'N'.

        (`errno` -2)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` i > 0)
            ``zgesdd`` did not converge, the updating process failed.

    Notes
    -----
    The SVD is written as

        A = U Sigma V^H,

    where Sigma is an m*n matrix which is zero except for its min(m, n)
    diagonal elements, U is an m*m unitary matrix, and V is an n*n
    unitary matrix.
    The diagonal elements of Sigma are the singular values of A; they
    are real and non-negative, and are returned in descending order.
    The first min(m, n) columns of U and V are the left and right
    singular vectors of A.

    Note that the function returns V^H, not V.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zgesdd'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'),),
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _s = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (min(_m_py, _n_py),),
    )
    _jobz = _b_utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = _b_utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py in ('A', 'a') or (_jobz_py in ('O', 'o') and _m_py < _n_py):
        _u_dim1 = _m_py
    elif _jobz_py in ('S', 's'):
        _u_dim1 = _m_py
    else:
        _u_dim1 = 1
    if _jobz_py in ('A', 'a') or (_jobz_py in ('O', 'o') and _m_py < _n_py):
        _u_dim2 = _m_py
    elif _jobz_py in ('S', 's'):
        _u_dim2 = min(_m_py, _n_py)
    else:
        _u_dim2 = 1
    exp_shape_u = (_u_dim1, _u_dim2)
    _u = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_u,
        sorder=_m_sorder,
    )
    if _jobz_py in ('A', 'a') or (_jobz_py in ('O', 'o') and _m_py >= _n_py):
        _vt_dim1 = _n_py
    elif _jobz_py in ('S', 's'):
        _vt_dim1 = min(_m_py, _n_py)
    else:
        _vt_dim1 = 1
    if _jobz_py in ('A', 'a') or (_jobz_py in ('O', 'o') and _m_py >= _n_py):
        _vt_dim2 = _n_py
    elif _jobz_py in ('S', 's'):
        _vt_dim2 = _n_py
    else:
        _vt_dim2 = 1
    exp_shape_vt = (_vt_dim1, _vt_dim2)
    _vt = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_vt,
        sorder=_m_sorder,
    )
    _b_lapackeig.zgesdd(
        jobz,
        _m,
        _n,
        _a,
        _s,
        _u,
        _vt,
    )
    _a_py = _a
    _s_py = _s
    _u_py = _u
    _vt_py = _vt
    _return_data = ZgesddReturnData(
        _a_py,
        _s_py,
        _u_py,
        _vt_py,
    )
    return _return_data

ZgebrdReturnData = _collections.namedtuple(
    'ZgebrdReturnData',
    ['a', 'd', 'e', 'tauq', 'taup'],
)

def zgebrd(a):
    u"""
    Performs a unitary reduction of complex general rectangular matrix
    to bidiagonal form.

    ``zgebrd`` reduces a complex m*n matrix to bidiagonal form.

    For full information please refer to the NAG Library document for
    f08ks

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08ksf.html

    Parameters
    ----------
    a : complex, array-like, shape (m, n)
        The m*n matrix A.

    Returns
    -------
    a : complex, ndarray, shape (m, n)
        If m >= n, the diagonal and first superdiagonal are overwritten
        by the upper bidiagonal matrix B, elements below the diagonal
        are overwritten by details of the unitary matrix Q and elements
        above the first superdiagonal are overwritten by details of the
        unitary matrix P.

        If m < n, the diagonal and first subdiagonal are overwritten by
        the lower bidiagonal matrix B, elements below the first
        subdiagonal are overwritten by details of the unitary matrix Q
        and elements above the diagonal are overwritten by details of
        the unitary matrix P.

    d : float, ndarray, shape (min(m, n))
        The diagonal elements of the bidiagonal matrix B.

    e : float, ndarray, shape (min(m, n)-1)
        The off-diagonal elements of the bidiagonal matrix B.

    tauq : complex, ndarray, shape (min(m, n))
        Further details of the unitary matrix Q.

    taup : complex, ndarray, shape (min(m, n))
        Further details of the unitary matrix P.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= 0.

    Notes
    -----
    ``zgebrd`` reduces a complex m*n matrix A to real bidiagonal form B
    by a unitary transformation: A = QBP^H, where Q and P^H are unitary
    matrices of order m and n respectively.

    If m >= n, the reduction is given by:

        A = Q(B_1;0)P^H = Q_1 B_1 P^H,

    where B_1 is a real n*n upper bidiagonal matrix and Q_1 consists of
    the first n columns of Q.

    If m < n, the reduction is given by

        A = Q(B_1 0)P^H = QB_1 P_1^H,

    where B_1 is a real m*m lower bidiagonal matrix and P_1^H consists
    of the first m rows of P^H.

    The unitary matrices Q and P are not formed explicitly but are
    represented as products of elementary reflectors (see [the F08
    Introduction] for details). Functions are provided to work with Q
    and P in this representation (see [Further Comments]).

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zgebrd'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _d = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (min(_m_py, _n_py),),
    )
    _e = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (min(_m_py, _n_py)-1,),
    )
    _tauq = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (min(_m_py, _n_py),),
    )
    _taup = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (min(_m_py, _n_py),),
    )
    _b_lapackeig.zgebrd(
        _m,
        _n,
        _a,
        _d,
        _e,
        _tauq,
        _taup,
    )
    _a_py = _a
    _d_py = _d
    _e_py = _e
    _tauq_py = _tauq
    _taup_py = _taup
    _return_data = ZgebrdReturnData(
        _a_py,
        _d_py,
        _e_py,
        _tauq_py,
        _taup_py,
    )
    return _return_data

def zungbr(vect, k, a, tau):
    u"""
    Generates unitary transformation matrices from the reduction to
    bidiagonal form determined by ``zgebrd``.

    ``zungbr`` generates one of the complex unitary matrices Q or P^H
    which were determined by ``zgebrd`` when reducing a complex matrix
    to bidiagonal form.

    For full information please refer to the NAG Library document for
    f08kt

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08ktf.html

    Parameters
    ----------
    vect : str, length 1
        Indicates whether the unitary matrix Q or P^H is generated.

        `vect` = 'Q'

            Q is generated.

        `vect` = 'P'

            P^H is generated.

    k : int
        If `vect` = 'Q', the number of columns in the original matrix A.

        If `vect` = 'P', the number of rows in the original matrix A.

    a : complex, array-like, shape (m, n)
        Details of the vectors which define the elementary reflectors,
        as returned by ``zgebrd``.

    tau : complex, array-like, shape (:)
        Note: the required length for this argument is determined as
        follows: if `vect`='Q': min(m, `k`); if `vect`='P': min(n, `k`);
        otherwise: 0.

        Further details of the elementary reflectors, as returned by
        ``zgebrd`` in its argument tauq if `vect` = 'Q', or in its
        argument taup if `vect` = 'P'.

    Returns
    -------
    a : complex, ndarray, shape (m, n)
        The unitary matrix Q or P^H, or the leading rows or columns
        thereof, as specified by `vect`, `m` and `n`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `vect`.

            Constraint: `vect` = 'Q' or 'P'.

        (`errno` -2)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: m >= n >= `k`.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: m = n.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= m >= `k`.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n = m.

        (`errno` -4)
            On entry, error in parameter `k`.

            Constraint: `k` >= 0.

    Notes
    -----
    ``zungbr`` is intended to be used after a call to ``zgebrd``, which
    reduces a complex rectangular matrix A to real bidiagonal form B by
    a unitary transformation: A = QBP^H. ``zgebrd`` represents the
    matrices Q and P^H as products of elementary reflectors.

    This function may be used to generate Q or P^H explicitly as square
    matrices, or in some cases just the leading columns of Q or the
    leading rows of P^H.

    The various possibilities are specified by the arguments `vect`, m,
    n and `k`.
    The appropriate values to cover the most likely cases are as follows
    (assuming that A was an m*n matrix):

    (1) To form the full m*m matrix Q: set `vect` = 'Q', n = m and `k` =
        n (note that the array `a` must have at least m columns).

    (#) If m > n, to form the n leading columns of Q: set `vect` = 'Q'
        and `k` = n

    (#) To form the full n*n matrix P^H: set `vect` = 'P', m = n and `k`
        = m (note that the array `a` must have at least n rows).

    (#) If m < n, to form the m leading rows of P^H: set `vect` = 'P'
        and `k` = m

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zungbr'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _tau = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'tau'},
        tau,
        copy=False,
    )
    _b_lapackeig.zungbr(
        vect,
        _m,
        _n,
        k,
        _a,
        _tau,
    )
    _a_py = _a
    return _a_py

def zunmbr(vect, side, trans, k, a, tau, c):
    u"""
    Applies the unitary transformations from reduction to bidiagonal
    form determined by ``zgebrd``.

    ``zunmbr`` multiplies an arbitrary complex m*n matrix C by one of
    the complex unitary matrices Q or P which were determined by
    ``zgebrd`` when reducing a complex matrix to bidiagonal form.

    For full information please refer to the NAG Library document for
    f08ku

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08kuf.html

    Parameters
    ----------
    vect : str, length 1
        Indicates whether Q or Q^H or P or P^H is to be applied to C.

        `vect` = 'Q'

            Q or Q^H is applied to C.

        `vect` = 'P'

            P or P^H is applied to C.

    side : str, length 1
        Indicates how Q or Q^H or P or P^H is to be applied to C.

        `side` = 'L'

            Q or Q^H or P or P^H is applied to C from the left.

        `side` = 'R'

            Q or Q^H or P or P^H is applied to C from the right.

    trans : str, length 1
        Indicates whether Q or P or Q^H or P^H is to be applied to C.

        `trans` = 'N'

            Q or P is applied to C.

        `trans` = 'C'

            Q^H or P^H is applied to C.

    k : int
        If `vect` = 'Q', the number of columns in the original matrix A.

        If `vect` = 'P', the number of rows in the original matrix A.

    a : complex, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `vect`='Q': r; if `vect`='P': min(r,
        `k`); otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `vect`='Q': min(r, `k`); if
        `vect`='P': r; otherwise: 0.

        Details of the vectors which define the elementary reflectors,
        as returned by ``zgebrd``.

    tau : complex, array-like, shape (min(r, `k`))
        Further details of the elementary reflectors, as returned by
        ``zgebrd`` in its argument tauq if `vect` = 'Q', or in its
        argument taup if `vect` = 'P'.

    c : complex, array-like, shape (m, n)
        The matrix C.

    Returns
    -------
    c : complex, ndarray, shape (m, n)
        `c` is overwritten by QC or Q^HC or CQ or C^HQ or PC or P^HC or
        CP or C^HP as specified by `vect`, `side` and `trans`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `vect`.

            Constraint: `vect` = 'Q' or 'P'.

        (`errno` -2)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -3)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'C'.

        (`errno` -4)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -5)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -6)
            On entry, error in parameter `k`.

            Constraint: `k` >= 0.

    Notes
    -----
    ``zunmbr`` is intended to be used after a call to ``zgebrd``, which
    reduces a complex rectangular matrix A to real bidiagonal form B by
    a unitary transformation: A = QBP^H. ``zgebrd`` represents the
    matrices Q and P^H as products of elementary reflectors.

    This function may be used to form one of the matrix products

        QC,Q^HC,CQ,CQ^H,PC,P^HC,CP or CP^H,

    overwriting the result on C (which may be any complex rectangular
    matrix).

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zunmbr'

    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=True,
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _side = _b_utils._EngineCharScalarType(
        side, {'fun_name': fun_name, 'entity': 'side'}, exp_chrlen=1,
    ).to_ctypes()
    _side_py = _b_utils._EngineCharScalarType.to_py(_side)
    if _side_py in ('L', 'l'):
        _vl_r_py = _m_py
    elif _side_py in ('R', 'r'):
        _vl_r_py = _n_py
    else:
        _vl_r_py = 0
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _tau = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'tau'},
        tau,
        copy=False,
    )
    _b_lapackeig.zunmbr(
        vect,
        side,
        trans,
        _m,
        _n,
        k,
        _a,
        _tau,
        _c,
    )
    _c_py = _c
    return _c_py

ZgejsvReturnData = _collections.namedtuple(
    'ZgejsvReturnData',
    ['a', 'sva', 'u', 'v', 'rworkb', 'iworkb'],
)

def zgejsv(joba, jobu, jobv, jobr, jobt, jobp, a):
    u"""
    Computes the singular value decomposition of a complex matrix,
    optionally computing the left and/or right singular vectors
    (preconditioned Jacobi).

    ``zgejsv`` computes the singular value decomposition (SVD) of a real
    m*n matrix A, where m >= n, and optionally computes the left and/or
    right singular vectors. ``zgejsv`` implements the preconditioned
    Jacobi SVD of Drmač and Veselić (2008a) and Drmač and Veselić
    (2008b).
    This is an expert version of the Jacobi SVD function ``zgesvj`` that
    employs preprocessing and preconditioning to improve the accuracy of
    results for extremely ill-conditioned matrices.
    In most cases ``zgesvj``, employing fast scaled rotations and de
    Rijk's pivoting strategy, is sufficient and is simpler to use.
    Also consider using ``zgesvd`` or ``zgesdd`` which are much simpler
    to use and also handle the case m < n.

    For full information please refer to the NAG Library document for
    f08kv

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08kvf.html

    Parameters
    ----------
    joba : str, length 1
        Specifies the form of pivoting for the QR factorization stage;
        whether an estimate of the condition number of the scaled matrix
        is required; and the form of rank reduction that is performed.

        `joba` = 'C'

            The initial QR factorization of the input matrix is
            performed with column pivoting; no estimate of condition
            number is computed; and, the rank is reduced by only the
            underflowed part of the triangular factor R. This option
            works well (high relative accuracy) if A can be written in
            the form A = BD, with well-conditioned B and arbitrary
            diagonal matrix D. The accuracy cannot be spoiled by column
            scaling. The accuracy of the computed output depends on the
            condition of B, and the procedure attempts to achieve the
            best theoretical accuracy.

        `joba` = 'E'

            Computation as with `joba` = 'C' with an additional estimate
            of the condition number of B. It provides a realistic error
            bound.

        `joba` = 'F'

            The initial QR factorization of the input matrix is
            performed with full row and column pivoting; no estimate of
            condition number is computed; and, the rank is reduced by
            only the underflowed part of the triangular factor R. If A =
            D_1*C*D_2 with ill-conditioned diagonal scalings D_1, D_2,
            and well-conditioned matrix C, this option gives higher
            accuracy than the `joba` = 'C' option. If the structure of
            the input matrix is not known, and relative accuracy is
            desirable, then this option is advisable.

        `joba` = 'G'

            Computation as with `joba` = 'F' with an additional estimate
            of the condition number of B, where A = DB (i.e., B =
            C*D_2). If A has heavily weighted rows, then using this
            condition number gives too pessimistic an error bound.

        `joba` = 'A'

            Computation as with `joba` = 'C' except in the treatment of
            rank reduction. In this case, small singular values are to
            be considered as noise and, if found, the matrix is treated
            as numerically rank deficient. The computed SVD, A = U Sigma
            V^H, is such that the relative residual norm (when comparing
            against A) is of the order O(m)*epsilon, where epsilon is
            machine precision. This gives the procedure licence to
            discard (set to zero) all singular values below
            n*epsilon*||A||.

        `joba` = 'R'

            Similar to `joba` = 'A'. The rank revealing property of the
            initial QR factorization is used to reveal (using the upper
            triangular factor) a gap, sigma_{r+1} < epsilonsigma_r, in
            which case the numerical rank is declared to be r. The SVD
            is computed with absolute error bounds, but more accurately
            than with `joba` = 'A'.

    jobu : str, length 1
        Specifies options for computing the left singular vectors U.

        `jobu` = 'U'

            The first n left singular vectors (columns of U) are
            computed and returned in the array `u`.

        `jobu` = 'F'

            All m left singular vectors are computed and returned in the
            array `u`.

        `jobu` = 'W'

            No left singular vectors are computed, but the array `u`
            (with ldu >= m and second dimension at least n) is available
            as workspace for computing right singular values. See the
            description of `u`.

        `jobu` = 'N'

            No left singular vectors are computed. `u` is not referenced
            when `jobv` = 'W' or 'N'.

    jobv : str, length 1
        Specifies options for computing the right singular vectors V.

        `jobv` = 'V'

            The n right singular vectors (columns of V) are computed and
            returned in the array `v`; Jacobi rotations are not
            explicitly accumulated.

        `jobv` = 'J'

            The n right singular vectors (columns of V) are computed and
            returned in the array `v`, but they are computed as the
            product of Jacobi rotations. This option is allowed only if
            `jobu` = 'U' or 'F', i.e., in computing the full SVD.

            This is equivalent to multiplying the input matrix, on the
            right, by the matrix V.

        `jobv` = 'W'

            No right singular values are computed, but the array `v`
            (with ldv >= n and second dimension at least n) is available
            as workspace for computing left singular values. See the
            description of `v`.

        `jobv` = 'N'

            No right singular vectors are computed. `v` is not
            referenced when `jobu` = 'W' or 'N' or `jobt` = 'N' or m !=
            n.

    jobr : str, length 1
        Specifies the conditions under which columns of A are to be set
        to zero. This effectively specifies a lower limit on the range
        of singular values; any singular values below this limit are
        (through column zeroing) set to zero. If A != 0 is scaled so
        that the largest column (in the Euclidean norm) of cA is equal
        to the square root of the overflow threshold, then `jobr` allows
        the function to kill columns of A whose norm in cA is less than
        sqrt(sfmin) (for `jobr` = 'R'), or less than sfmin/epsilon
        (otherwise). sfmin is the safe range parameter, as returned by
        function ``machine.real_safe``.

        `jobr` = 'N'

            Only set to zero those columns of A for which the norm of
            corresponding column of cA < sfmin/epsilon, that is, those
            columns that are effectively zero (to machine precision)
            anyway. If the condition number of A is greater than the
            overflow threshold lambda, where lambda is the value
            returned by ``machine.real_largest``, you are recommended to
            use function ``zgesvj``.

        `jobr` = 'R'

            Set to zero those columns of A for which the norm of the
            corresponding column of cA < sqrt(sfmin). This approximately
            represents a restricted range for sigma(cA) of
            [sqrt(sfmin),sqrt(lambda)].

        For computing the singular values in the full range from the
        safe minimum up to the overflow threshold use ``zgesvj``.

        Suggested value: `jobr` = 'R'.

    jobt : str, length 1
        Specifies, in the case n = m, whether the function is permitted
        to use the conjugate transpose of A for improved efficiency. If
        the matrix is square, then the procedure may use A^H if it seems
        to be better with respect to convergence. If the matrix is not
        square, `jobt` is ignored. The decision is based on two values
        of entropy over the adjoint orbit of A^HA. See the descriptions
        of `rworkb`[5] and `rworkb`[6].

        `jobt` = 'T'

            If n = m, perform an entropy test and, if the test indicates
            possibly faster convergence of the Jacobi process when using
            A^H, then form the conjugate transpose A^H. If A is replaced
            with A^H, then the row pivoting is included automatically.

        `jobt` = 'N'

            No entropy test and no transposition is performed.

        The option `jobt` = 'T' can be used to compute only the singular
        values, or the full SVD (U, Sigma and V).

        In the case where only one set of singular vectors (U or V) is
        required, the caller must still provide both `u` and `v`, as one
        of the matrices is used as workspace if the matrix A is
        transposed.

        See the descriptions of `u` and `v`.

    jobp : str, length 1
        Specifies whether the function should be allowed to introduce
        structured perturbations to drown denormalized numbers. For
        details see Drmač and Veselić (2008a) and Drmač and Veselić
        (2008b). For the sake of simplicity, these perturbations are
        included only when the full SVD or only the singular values are
        requested.

        `jobp` = 'P'

            Introduce perturbation if A is found to be very badly scaled
            (introducing denormalized numbers).

        `jobp` = 'N'

            Do not perturb.

    a : complex, array-like, shape (m, n)
        The m*n matrix A.

    Returns
    -------
    a : complex, ndarray, shape (m, n)
        The contents of `a` are overwritten.

    sva : float, ndarray, shape (n)
        The, possibly scaled, singular values of A.

        The singular values of A are sigma_i = alpha*`sva`[i-1], for i =
        1,2,...,n, where alpha = `rworkb`[0]/`rworkb`[1].

        Normally alpha = 1 and no scaling is required to obtain the
        singular values.

        However, if the largest singular value of A overflows or if
        small singular values have been saved from underflow by scaling
        the input matrix A, then alpha != 1.

        If `jobr` = 'R', then some of the singular values may be
        returned as exact zeros because they are below the numerical
        rank threshold or are denormalized numbers.

    u : complex, ndarray, shape (:, :)
        If `jobu` = 'U', `u` contains the m*n matrix of left singular
        vectors.

        If `jobu` = 'F', `u` contains the m*m matrix of left singular
        vectors, including an orthonormal basis of the orthogonal
        complement of Range(A).

        `u` is not referenced when `jobu` = 'W' or 'N' and one of the
        following is satisfied:

            `jobv` = 'W' or 'N', or

            n = 1, or

            A is the zero matrix.

    v : complex, ndarray, shape (:, :)
        If `jobv` = 'V' or 'J', `v` contains the n*n matrix of right
        singular vectors.

        `v` is not referenced when `jobv` = 'W' or 'N' and one of the
        following is satisfied:

            `jobu` = 'U' or 'F' and `jobt` = 'T', or

            n = 1, or

            A is the zero matrix.

    rworkb : float, ndarray, shape (7)
        Contains information about the completed job.

        `rworkb`[0]

            alpha = `rworkb`[0]/`rworkb`[1] is the scaling factor such
            that sigma_i = alpha*`sva`[i-1], for i = 1,2,...,n, are the
            computed singular values of A. See the description of `sva`.

        `rworkb`[1]

            See the description of `rworkb`[0].

        `rworkb`[2]

            sconda, an estimate for the condition number of column
            equilibrated A (if `joba` = 'E' or 'G'). sconda is an
            estimate of sqrt((||(R^TR)^-1||_1)). It is computed using
            ``lapacklin.dpocon``. It satisfies n^(-1/4)*sconda <=
            ||R^-1||_2 <= n^(1/4)*sconda where R is the triangular
            factor from the QR factorization of A. However, if R is
            truncated and the numerical rank is determined to be
            strictly smaller than n, sconda is returned as -1, thus
            indicating that the smallest singular values might be lost.

        If the full SVD is needed, and you are familiar with the details
        of the method, the following two condition numbers are useful
        for the analysis of the algorithm.

        `rworkb`[3]

            An estimate of the scaled condition number of the triangular
            factor in the first QR factorization.

        `rworkb`[4]

            An estimate of the scaled condition number of the triangular
            factor in the second QR factorization.

        The following two parameters are computed if `jobt` = 'T'.

        `rworkb`[5]

            The entropy of A^TA: this is the Shannon entropy of
            diag(A^TA)/trace(A^TA) taken as a point in the probability
            simplex.

        `rworkb`[6]

            The entropy of AA^T.

    iworkb : None or int, ndarray, shape (:)
        Contains information about the completed job.

        `iworkb`[0]

            The numerical rank of A determined after the initial QR
            factorization with pivoting. See the descriptions of `joba`
            and `jobr`.

        `iworkb`[1]

            The number of computed nonzero singular values.

        `iworkb`[2]

            If nonzero, a warning message. If `iworkb`[2] = 1, then some
            of the column norms of A were denormalized (tiny) numbers.
            The requested high accuracy is not warranted by the data.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `joba`.

            Constraint: `joba` = 'C', 'E', 'F', 'G', 'A' or 'R'.

        (`errno` -2)
            On entry, error in parameter `jobu`.

            Constraint: `jobu` = 'U', 'F', 'W' or 'N'.

        (`errno` -3)
            On entry, error in parameter `jobv`.

            Constraint: `jobv` = 'V', 'J', 'W' or 'N'.

        (`errno` -3)
            On entry, error in parameter `jobv`.

            Constraint: `jobv` != 'J'.

        (`errno` -4)
            On entry, error in parameter `jobr`.

            Constraint: `jobr` = 'N' or 'R'.

        (`errno` -5)
            On entry, error in parameter `jobt`.

            Constraint: `jobt` = 'T' or 'N'.

        (`errno` -6)
            On entry, error in parameter `jobp`.

            Constraint: `jobp` = 'P' or 'N'.

        (`errno` -7)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -8)
            On entry, error in parameter n.

            Constraint: m >= n >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            ``zgejsv`` did not converge in the allowed number of
            iterations (30). The computed values might be inaccurate.

    Notes
    -----
    The SVD is written as

        A = U Sigma V^H,

    where Sigma is an m*n matrix which is zero except for its n diagonal
    elements, U is an m*m unitary matrix, and V is an n*n unitary
    matrix.
    The diagonal elements of Sigma are the singular values of A in
    descending order of magnitude.
    The columns of U and V are the left and the right singular vectors
    of A, respectively.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Drmač, Z and Veselić, K, 2008, `New fast and accurate Jacobi SVD
    Algorithm I`, SIAM J. Matrix Anal. Appl. (29 4)

    Drmač, Z and Veselić, K, 2008, `New fast and accurate Jacobi SVD
    Algorithm II`, SIAM J. Matrix Anal. Appl. (29 4)

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zgejsv'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'),),
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _sva = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobu = _b_utils._EngineCharScalarType(
        jobu, {'fun_name': fun_name, 'entity': 'jobu'}, exp_chrlen=1,
    ).to_ctypes()
    _jobu_py = _b_utils._EngineCharScalarType.to_py(_jobu)
    if _jobu_py in ('F', 'f'):
        _u_dim1 = _m_py
    elif _jobu_py in ('U', 'u', 'W', 'w'):
        _u_dim1 = _m_py
    else:
        _u_dim1 = 1
    if _jobu_py in ('F', 'f'):
        _u_dim2 = _m_py
    elif _jobu_py in ('U', 'u', 'W', 'w'):
        _u_dim2 = _n_py
    else:
        _u_dim2 = 1
    exp_shape_u = (_u_dim1, _u_dim2)
    _u = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_u,
        sorder=_m_sorder,
    )
    _jobv = _b_utils._EngineCharScalarType(
        jobv, {'fun_name': fun_name, 'entity': 'jobv'}, exp_chrlen=1,
    ).to_ctypes()
    _jobv_py = _b_utils._EngineCharScalarType.to_py(_jobv)
    if _jobv_py in ('V', 'v', 'J', 'j', 'W', 'w'):
        _v_dim1 = _n_py
    else:
        _v_dim1 = 1
    if _jobv_py in ('V', 'v', 'J', 'j', 'W', 'w'):
        _v_dim2 = _n_py
    else:
        _v_dim2 = 1
    exp_shape_v = (_v_dim1, _v_dim2)
    _v = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_v,
        sorder=_m_sorder,
    )
    _rworkb = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (7,),
    )
    if _n_py > 0 and _m_py > 0:
        if _n_py > 0 and _m_py > 0:
            _iworkb_dim1 = 3
        else:
            _iworkb_dim1 = 0
        exp_shape_iworkb = (_iworkb_dim1,)
        _iworkb = _b_utils._EngineIntArrayType.empty_ndarray(
            exp_shape_iworkb,
        )
    else:
        _iworkb = None
    try:
        _b_lapackeig.zgejsv(
            joba,
            jobu,
            jobv,
            jobr,
            jobt,
            jobp,
            _m,
            _n,
            _a,
            _sva,
            _u,
            _v,
            _rworkb,
            _iworkb,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _a_py = _a
    _sva_py = _sva
    _u_py = _u
    _v_py = _v
    _rworkb_py = _rworkb
    if _n_py > 0 and _m_py > 0:
        _iworkb_py = _iworkb
    else:
        _iworkb_py = None
    _return_data = ZgejsvReturnData(
        _a_py,
        _sva_py,
        _u_py,
        _v_py,
        _rworkb_py,
        _iworkb_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

ZgesvjReturnData = _collections.namedtuple(
    'ZgesvjReturnData',
    ['a', 'sva', 'v', 'rwork'],
)

def zgesvj(joba, jobu, jobv, a, mv, v, ctol):
    u"""
    Computes the singular value decomposition of a complex matrix,
    optionally computing the left and/or right singular vectors (fast
    Jacobi).

    ``zgesvj`` computes the one-sided Jacobi singular value
    decomposition (SVD) of a complex m*n, general or triangular, matrix
    A, m >= n, with fast scaled rotations and de Rijk’s pivoting,
    optionally computing the left and/or right singular vectors.
    For m < n, the functions ``zgesvd`` or ``zgesdd`` may be used.

    For full information please refer to the NAG Library document for
    f08kw

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08kwf.html

    Parameters
    ----------
    joba : str, length 1
        Specifies the structure of matrix A.

        `joba` = 'L'

            The input matrix A is lower triangular.

        `joba` = 'U'

            The input matrix A is upper triangular.

        `joba` = 'G'

            The input matrix A is a general m*n matrix, m >= n.

    jobu : str, length 1
        Specifies whether to compute the left singular vectors and if so
        whether you want to control their numerical orthogonality
        threshold.

        `jobu` = 'U'

            The left singular vectors corresponding to the nonzero
            singular values are computed and returned in the leading
            columns of `a`. See more details in the description of `a`.
            The numerical orthogonality threshold is set to
            approximately tol = sqrt(m)*epsilon, where epsilon is the
            machine precision.

        `jobu` = 'C'

            Analogous to `jobu` = 'U', except that you can control the
            level of numerical orthogonality of the computed left
            singular vectors. The orthogonality threshold is set to tol
            = `ctol`*epsilon. The option `jobu` = 'C' can be used if
            m*epsilon is a satisfactory orthogonality of the computed
            left singular vectors, so `ctol` = m could save a few sweeps
            of Jacobi rotations. See the descriptions of `a` and `ctol`.

        `jobu` = 'N'

            The matrix U is not computed. However, see the description
            of `a`.

    jobv : str, length 1
        Specifies whether and how to compute the right singular vectors.

        `jobv` = 'V'

            The matrix V is computed and returned in the array `v`.

        `jobv` = 'A'

            The Jacobi rotations are applied to the leading m_v*n part
            of the array `v`. In other words, the right singular vector
            matrix V is not computed explicitly, instead it is applied
            to an m_v*n matrix initially stored in the first `mv` rows
            of `v`.

        `jobv` = 'N'

            The matrix V is not computed and the array `v` is not
            referenced.

    a : complex, array-like, shape (m, n)
        The m*n matrix A.

    mv : int
        If `jobv` = 'A', the product of Jacobi rotations is applied to
        the first m_v rows of `v`.

        If `jobv` != 'A', `mv` is ignored.

        See the description of `jobv`.

    v : complex, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobv`='V': n; if `jobv`='A': `mv`;
        otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobv` in ('V', 'A'): n; otherwise: 1.

        If `jobv` = 'A', `v` must contain an m_v*n matrix to be
        premultiplied by the matrix V of right singular vectors.

    ctol : float
        If `jobu` = 'C', `ctol` = ctol, the threshold for convergence.
        The process stops if all columns of A are mutually orthogonal up
        to ctol*epsilon. It is required that ctol >= 1, i.e., it is not
        possible to force the function to obtain orthogonality below
        epsilon. ctol greater than 1/epsilon is meaningless, where
        epsilon is the machine precision.

    Returns
    -------
    a : complex, ndarray, shape (m, n)
        The matrix U containing the left singular vectors of A.

        If `jobu` = 'U' or 'C'

            if errno = 0

                rank(A) unitary columns of U are returned in the leading
                rank(A) columns of the array `a`. Here rank(A) <= n is
                the number of computed singular values of A that are
                above the safe range parameter, as returned by
                ``machine.real_safe``. The singular vectors
                corresponding to underflowed or zero singular values are
                not computed. The value of rank(A) is returned by
                rounding `rwork`[1] to the nearest whole number. Also
                see the descriptions of `sva` and `rwork`. The computed
                columns of U are mutually numerically unitary up to
                approximately tol = sqrt(m)*epsilon; or tol =
                `ctol`*epsilon (`jobu` = 'C'), where epsilon is the
                machine precision, see the description of `jobu`.

            if errno > 0

                ``zgesvj`` did not converge in 30 iterations (sweeps).
                In this case, the computed columns of U may not be
                unitary up to tol. The output U (stored in `a`), Sigma
                (given by the computed singular values in `sva`) and V
                is still a decomposition of the input matrix A in the
                sense that the residual
                ||A-alpha*U*Sigma*V^H||_2/||A||_2 is small, where alpha
                is the value returned in `rwork`[0].

        If `jobu` = 'N'

            if errno = 0

                Note that the left singular vectors are 'for free' in
                the one-sided Jacobi SVD algorithm. However, if only the
                singular values are needed, the level of numerical
                orthogonality of U is not an issue and iterations are
                stopped when the columns of the iterated matrix are
                numerically unitary up to approximately m*epsilon. Thus,
                on exit, `a` contains the columns of U scaled with the
                corresponding singular values.

            if errno > 0

                ``zgesvj`` did not converge in 30 iterations (sweeps).

    sva : float, ndarray, shape (n)
        The, possibly scaled, singular values of A.

        If errno = 0

            The singular values of A are sigma_i = alpha*`sva`[i-1], for
            i = 1,2,...,n, where alpha is the scale factor stored in
            `rwork`[0]. Normally alpha = 1, however, if some of the
            singular values of A might underflow or overflow, then alpha
            != 1 and the scale factor needs to be applied to obtain the
            singular values.

        If errno > 0

            ``zgesvj`` did not converge in 30 iterations and alpha*`sva`
            may not be accurate.

    v : complex, ndarray, shape (:, :)
        The right singular vectors of A.

        If `jobv` = 'V', `v` contains the n*n matrix of the right
        singular vectors.

        If `jobv` = 'A', `v` contains the product of the computed right
        singular vector matrix and the initial matrix in the array `v`.

        If `jobv` = 'N', `v` is not referenced.

    rwork : float, ndarray, shape (6)
        Contains information about the completed job.

        `rwork`[0]

            The scaling factor, alpha, such that sigma_i =
            alpha*`sva`[i-1], for i = 1,2,...,n, are the computed
            singular values of A. (See the description of `sva`.)

        `rwork`[1]

            nint(`rwork`[1]) gives the number of the computed nonzero
            singular values.

        `rwork`[2]

            nint(`rwork`[2]) gives the number of the computed singular
            values that are larger than the underflow threshold.

        `rwork`[3]

            nint(`rwork`[3]) gives the number of iterations (sweeps of
            Jacobi rotations) needed for numerical convergence.

        `rwork`[4]

            max_{i != j}|cos(A(:,i), A(:,j))| in the last iteration
            (sweep). This is useful information in cases when ``zgesvj``
            did not converge, as it can be used to estimate whether the
            output is still useful and for subsequent analysis.

        `rwork`[5]

            The largest absolute value over all sines of the Jacobi
            rotation angles in the last sweep. It can be useful for
            subsequent analysis.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `joba`.

            Constraint: `joba` = 'L', 'U' or 'G'.

        (`errno` -2)
            On entry, error in parameter `jobu`.

            Constraint: `jobu` = 'U', 'C' or 'N'.

        (`errno` -3)
            On entry, error in parameter `jobv`.

            Constraint: `jobv` = 'V', 'A' or 'N'.

        (`errno` -4)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -5)
            On entry, error in parameter n.

            Constraint: m >= n >= 0.

        (`errno` -9)
            On entry, error in parameter `mv`.

            Constraint: `mv` >= 0.

        (`errno` -14)
            On entry, error in parameter `ctol`.

            Constraint: `ctol` >= 1.0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            ``zgesvj`` did not converge in the allowed number of
            iterations (30), but its output might still be useful.

    Notes
    -----
    The SVD is written as

        A = U Sigma V^H,

    where Sigma is an n*n diagonal matrix, U is an m*n unitary matrix,
    and V is an n*n unitary matrix.
    The diagonal elements of Sigma are the singular values of A in
    descending order of magnitude.
    The columns of U and V are the left and the right singular vectors
    of A, respectively.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Drmač, Z and Veselić, K, 2008, `New fast and accurate Jacobi SVD
    Algorithm I`, SIAM J. Matrix Anal. Appl. (29 4)

    Drmač, Z and Veselić, K, 2008, `New fast and accurate Jacobi SVD
    Algorithm II`, SIAM J. Matrix Anal. Appl. (29 4)

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zgesvj'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _sva = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _v = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'v'},
        v,
        copy=True,
    )
    _rwork = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (6,),
    )
    try:
        _b_lapackeig.zgesvj(
            joba,
            jobu,
            jobv,
            _m,
            _n,
            _a,
            _sva,
            mv,
            _v,
            ctol,
            _rwork,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _a_py = _a
    _v_py = _v
    _sva_py = _sva
    _rwork_py = _rwork
    _return_data = ZgesvjReturnData(
        _a_py,
        _sva_py,
        _v_py,
        _rwork_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

ZgesvdxReturnData = _collections.namedtuple(
    'ZgesvdxReturnData',
    ['a', 'ns', 's', 'u', 'vt', 'rworkb', 'jfail'],
)

def zgesvdx(jobu, jobvt, erange, a, vl, vu, il, iu):
    u"""
    Computes all or selected singular values of the singular value
    decomposition of a complex general matrix, optionally computing the
    corresponding left and right singular vectors.

    ``zgesvdx`` computes the singular value decomposition (SVD) of a
    complex m*n matrix A, optionally computing the left and/or right
    singular vectors.
    All singular values or a selected set of singular values may be
    computed.

    For full information please refer to the NAG Library document for
    f08kz

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08kzf.html

    Parameters
    ----------
    jobu : str, length 1
        Specifies options for computing all or part of the matrix U.

        `jobu` = 'V'

            The `ns` columns of U, as specified by `erange`, are
            returned in array `u`.

        `jobu` = 'N'

            No columns of U (no left singular vectors) are computed.

    jobvt : str, length 1
        Specifies options for computing all or part of the matrix V^T.

        `jobvt` = 'V'

            The `ns` rows of V^T, as specified by `erange`, are returned
            in the array `vt`.

        `jobvt` = 'N'

            No rows of V^T (no right singular vectors) are computed.

    erange : str, length 1
        Indicates which singular values should be returned.

        `erange` = 'A'

            All singular values will be found.

        `erange` = 'V'

            All singular values in the half-open interval (`vl`,`vu`]
            will be found.

        `erange` = 'I'

            The `il`th through `iu`th singular values will be found.

    a : complex, array-like, shape (m, n)
        The m*n matrix A.

    vl : float
        If `erange` = 'V', the lower bound of the interval to be
        searched for singular values.

        If `erange` = 'A' or 'I', `vl` is not referenced.

    vu : float
        If `erange` = 'V', the upper bound of the interval to be
        searched for singular values.

        If `erange` = 'A' or 'I', `vu` is not referenced.

    il : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest singular values to
        be returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    iu : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest singular values to
        be returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    Returns
    -------
    a : complex, ndarray, shape (m, n)
        If `jobu` != 'N' and `jobvt` != 'N', the contents of `a` are
        destroyed.

    ns : int
        The total number of singular values found. 0 <= `ns` <= min(m,
        n).

        If `erange` = 'A', `ns` = min(m, n).

        If `erange` = 'I', `ns` = `iu`-`il`+1.

        If `erange` = 'V' then the value of `ns` is not known in advance
        and so an upper limit should be used when specifying the
        dimensions of array `u`, e.g., min(m, n).

    s : float, ndarray, shape (min(m, n))
        The singular values of A, sorted so that `s`[i-1] >= `s`[i].

    u : None or complex, ndarray, shape (:, :)
        If `jobu` = 'V', `u` contains the first `ns` columns of U (the
        left singular vectors, stored column-wise).

        If `jobu` = 'N', `u` is not referenced.

    vt : None or complex, ndarray, shape (:, :)
        If `jobvt` = 'V', `vt` contains the first `ns` rows of V^H (the
        right singular vectors, stored row-wise).

        If `jobvt` = 'N', `vt` is not referenced.

    rworkb : float, ndarray, shape (min(m, n))
        If `errno` > 0, `rworkb`[1:min(m, n)] contains the unconverged
        superdiagonal elements of an upper bidiagonal matrix B whose
        diagonal is in `s` (not necessarily sorted). B satisfies A =
        UBV^H, so it has the same singular values as A, and left and
        right singular vectors that are those of A pre-multiplied by U^H
        and V^H.

    jfail : int, ndarray, shape (2*min(m, n))
        If `errno` > 0, `jfail` contains, in its first k nonzero
        elements, the indices of the k eigenvectors (associated with a
        left or right singular vector, see ``dbdsvdx``) that failed to
        converge.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobu`.

            Constraint: `jobu` = 'V' or 'N'.

        (`errno` -2)
            On entry, error in parameter `jobvt`.

            Constraint: `jobvt` = 'V' or 'N'.

        (`errno` -3)
            On entry, error in parameter `erange`.

            Constraint: `erange` = 'A', 'V' or 'I'.

        (`errno` -4)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -5)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -8)
            On entry, error in parameter `vl`.

            Constraint: 0.0 <= `vl`.

        (`errno` -9)
            On entry, error in parameter `vu`.

            Constraint: `vl` < `vu`.

        (`errno` -10)
            On entry, error in parameter `il`.

        (`errno` -11)
            On entry, error in parameter `iu`.

        (`errno` i > 0)
            If ``zgesvdx`` did not converge, errno specifies how many
            superdiagonals of an intermediate bidiagonal form did not
            converge to zero.

    Notes
    -----
    The SVD is written as

        A = U Sigma V^H,

    where Sigma is an m*n matrix which is zero except for its min(m, n)
    diagonal elements, U is an m*m unitary matrix, and V is an n*n
    unitary matrix.
    The diagonal elements of Sigma are the singular values of A; they
    are complex and non-negative, and are returned in descending order.
    The first min(m, n) columns of U and V are the left and right
    singular vectors of A, respectively.

    Note that the function returns V^H, not V.

    Alternative to computing all singular values of A, a selected set
    can be computed.
    The set is either those singular values lying in a given interval,
    sigma in (v_l,v_u], or those whose index (counting from largest to
    smallest in magnitude) lies in a given range 1 <= i_l,...,i_u <= n.
    In these cases, the corresponding left and right singular vectors
    can optionally be computed.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zgesvdx'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'),),
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _s = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (min(_m_py, _n_py),),
    )
    _erange = _b_utils._EngineCharScalarType(
        erange, {'fun_name': fun_name, 'entity': 'erange'}, exp_chrlen=1,
    ).to_ctypes()
    _erange_py = _b_utils._EngineCharScalarType.to_py(_erange)
    _iu = _b_utils._EngineIntScalarType(
        iu, {'fun_name': fun_name, 'entity': 'iu'},
    ).to_ctypes()
    _iu_py = _b_utils._EngineIntScalarType.to_py(_iu)
    _il = _b_utils._EngineIntScalarType(
        il, {'fun_name': fun_name, 'entity': 'il'},
    ).to_ctypes()
    _il_py = _b_utils._EngineIntScalarType.to_py(_il)
    if _erange_py in ('I', 'i'):
        _vl_nsmax_py = _iu_py-_il_py+1
    elif _erange_py in ('A', 'a', 'V', 'v'):
        _vl_nsmax_py = min(_m_py, _n_py)
    else:
        _vl_nsmax_py = 0
    _jobu = _b_utils._EngineCharScalarType(
        jobu, {'fun_name': fun_name, 'entity': 'jobu'}, exp_chrlen=1,
    ).to_ctypes()
    _jobu_py = _b_utils._EngineCharScalarType.to_py(_jobu)
    if _jobu_py in ('V', 'v'):
        if _jobu_py in ('V', 'v'):
            _u_dim1 = _m_py
        else:
            _u_dim1 = 1
        if _jobu_py in ('V', 'v'):
            _u_dim2 = _vl_nsmax_py
        else:
            _u_dim2 = 0
        exp_shape_u = (_u_dim1, _u_dim2)
        _u = _b_utils._EngineComplex128ArrayType.empty_ndarray(
            exp_shape_u,
            sorder=_m_sorder,
        )
    else:
        _u = None
    _jobvt = _b_utils._EngineCharScalarType(
        jobvt, {'fun_name': fun_name, 'entity': 'jobvt'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvt_py = _b_utils._EngineCharScalarType.to_py(_jobvt)
    if _jobvt_py in ('V', 'v'):
        if _jobvt_py in ('V', 'v'):
            _vt_dim1 = min(_m_py, _n_py)
        else:
            _vt_dim1 = 1
        if _jobvt_py in ('V', 'v'):
            _vt_dim2 = _n_py
        else:
            _vt_dim2 = 0
        exp_shape_vt = (_vt_dim1, _vt_dim2)
        _vt = _b_utils._EngineComplex128ArrayType.empty_ndarray(
            exp_shape_vt,
            sorder=_m_sorder,
        )
    else:
        _vt = None
    _rworkb = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (min(_m_py, _n_py),),
    )
    _jfail = _b_utils._EngineIntArrayType.empty_ndarray(
        (2*min(_m_py, _n_py),),
    )
    _return_scalar = _b_lapackeig.zgesvdx(
        jobu,
        jobvt,
        erange,
        _m,
        _n,
        _a,
        vl,
        vu,
        _il,
        _iu,
        _s,
        _u,
        _vt,
        _rworkb,
        _jfail,
    )
    _a_py = _a
    _ns_py = _return_scalar
    _s_py = _s
    if _jobu_py in ('V', 'v'):
        _u_py = _u
    else:
        _u_py = None
    if _jobvt_py in ('V', 'v'):
        _vt_py = _vt
    else:
        _vt_py = None
    _rworkb_py = _rworkb
    _jfail_py = _jfail
    _return_data = ZgesvdxReturnData(
        _a_py,
        _ns_py,
        _s_py,
        _u_py,
        _vt_py,
        _rworkb_py,
        _jfail_py,
    )
    return _return_data

DgbbrdReturnData = _collections.namedtuple(
    'DgbbrdReturnData',
    ['ab', 'd', 'e', 'q', 'pt', 'c'],
)

def dgbbrd(vect, m, kl, ku, ab, c):
    u"""
    Performs a reduction of real rectangular band matrix to upper
    bidiagonal form.

    ``dgbbrd`` reduces a real m*n band matrix to upper bidiagonal form.

    For full information please refer to the NAG Library document for
    f08le

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08lef.html

    Parameters
    ----------
    vect : str, length 1
        Indicates whether the matrices Q and/or P^T are generated.

        `vect` = 'N'

            Neither Q nor P^T is generated.

        `vect` = 'Q'

            Q is generated.

        `vect` = 'P'

            P^T is generated.

        `vect` = 'B'

            Both Q and P^T are generated.

    m : int
        m, the number of rows of the matrix A.

    kl : int
        The number of subdiagonals, k_l, within the band of A.

    ku : int
        The number of superdiagonals, k_u, within the band of A.

    ab : float, array-like, shape (`kl`+`ku`+1, n)
        The original m*n band matrix A.

    c : float, array-like, shape (:, ncc)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if ncc>0: `m`; if ncc=0: 1; otherwise: 0.

        An m*n_C matrix C.

    Returns
    -------
    ab : float, ndarray, shape (`kl`+`ku`+1, n)
        `ab` is overwritten by values generated during the reduction.

    d : float, ndarray, shape (min(`m`, n))
        The diagonal elements of the bidiagonal matrix B.

    e : float, ndarray, shape (min(`m`, n)-1)
        The superdiagonal elements of the bidiagonal matrix B.

    q : float, ndarray, shape (:, :)
        If `vect` = 'Q' or 'B', contains the m*m orthogonal matrix Q.

        If `vect` = 'N' or 'P', `q` is not referenced.

    pt : float, ndarray, shape (:, :)
        The n*n orthogonal matrix P^T, if `vect` = 'P' or 'B'. If `vect`
        = 'N' or 'Q', `pt` is not referenced.

    c : float, ndarray, shape (:, ncc)
        `c` is overwritten by Q^TC. If ncc = 0, `c` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `vect`.

            Constraint: `vect` = 'N', 'Q', 'P' or 'B'.

        (`errno` -2)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -4)
            On entry, error in parameter ncc.

            Constraint: ncc >= 0.

        (`errno` -5)
            On entry, error in parameter `kl`.

            Constraint: `kl` >= 0.

        (`errno` -6)
            On entry, error in parameter `ku`.

            Constraint: `ku` >= 0.

    Notes
    -----
    ``dgbbrd`` reduces a real m*n band matrix to upper bidiagonal form B
    by an orthogonal transformation: A = QBP^T.
    The orthogonal matrices Q and P^T, of order m and n respectively,
    are determined as a product of Givens rotation matrices, and may be
    formed explicitly by the function if required.
    A matrix C may also be updated to give C~ = Q^TC.

    The function uses a vectorizable form of the reduction.
    """

    fun_name = 'naginterfaces.library.lapackeig.dgbbrd'

    _ab = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ab'},
        ab,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_ab, 'ab'), (_c, 'c')),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _ncc_py = _b_utils._EngineFloat64ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[1]
    _ncc = _b_utils._EngineIntScalarType(
        _ncc_py, {'fun_name': fun_name, 'entity': '_ncc'},
    ).to_ctypes()
    _m = _b_utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = _b_utils._EngineIntScalarType.to_py(_m)
    _d = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (min(_m_py, _n_py),),
    )
    _e = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (min(_m_py, _n_py)-1,),
    )
    _vect = _b_utils._EngineCharScalarType(
        vect, {'fun_name': fun_name, 'entity': 'vect'}, exp_chrlen=1,
    ).to_ctypes()
    _vect_py = _b_utils._EngineCharScalarType.to_py(_vect)
    if _vect_py in ('Q', 'q', 'B', 'b'):
        _q_dim1 = _m_py
    else:
        _q_dim1 = 1
    if _vect_py in ('Q', 'q', 'B', 'b'):
        _q_dim2 = _m_py
    else:
        _q_dim2 = 1
    exp_shape_q = (_q_dim1, _q_dim2)
    _q = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_q,
        sorder=_m_sorder,
    )
    if _vect_py in ('P', 'p', 'B', 'b'):
        _pt_dim1 = _n_py
    else:
        _pt_dim1 = 1
    if _vect_py in ('P', 'p', 'B', 'b'):
        _pt_dim2 = _n_py
    else:
        _pt_dim2 = 1
    exp_shape_pt = (_pt_dim1, _pt_dim2)
    _pt = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_pt,
        sorder=_m_sorder,
    )
    _b_lapackeig.dgbbrd(
        vect,
        _m,
        _n,
        _ncc,
        kl,
        ku,
        _ab,
        _d,
        _e,
        _q,
        _pt,
        _c,
    )
    _ab_py = _ab
    _c_py = _c
    _d_py = _d
    _e_py = _e
    _q_py = _q
    _pt_py = _pt
    _return_data = DgbbrdReturnData(
        _ab_py,
        _d_py,
        _e_py,
        _q_py,
        _pt_py,
        _c_py,
    )
    return _return_data

ZgbbrdReturnData = _collections.namedtuple(
    'ZgbbrdReturnData',
    ['ab', 'd', 'e', 'q', 'pt', 'c'],
)

def zgbbrd(vect, m, kl, ku, ab, c):
    u"""
    Reduction of complex rectangular band matrix to upper bidiagonal
    form.

    ``zgbbrd`` reduces a complex m*n band matrix to real upper
    bidiagonal form.

    For full information please refer to the NAG Library document for
    f08ls

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08lsf.html

    Parameters
    ----------
    vect : str, length 1
        Indicates whether the matrices Q and/or P^H are generated.

        `vect` = 'N'

            Neither Q nor P^H is generated.

        `vect` = 'Q'

            Q is generated.

        `vect` = 'P'

            P^H is generated.

        `vect` = 'B'

            Both Q and P^H are generated.

    m : int
        m, the number of rows of the matrix A.

    kl : int
        The number of subdiagonals, k_l, within the band of A.

    ku : int
        The number of superdiagonals, k_u, within the band of A.

    ab : complex, array-like, shape (`kl`+`ku`+1, n)
        The original m*n band matrix A.

    c : complex, array-like, shape (:, ncc)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if ncc>0: `m`; if ncc=0: 1; otherwise: 0.

        An m*n_C matrix C.

    Returns
    -------
    ab : complex, ndarray, shape (`kl`+`ku`+1, n)
        `ab` is overwritten by values generated during the reduction.

    d : float, ndarray, shape (min(`m`, n))
        The diagonal elements of the bidiagonal matrix B.

    e : float, ndarray, shape (min(`m`, n)-1)
        The superdiagonal elements of the bidiagonal matrix B.

    q : complex, ndarray, shape (:, :)
        If `vect` = 'Q' or 'B', contains the m*m unitary matrix Q.

        If `vect` = 'N' or 'P', `q` is not referenced.

    pt : complex, ndarray, shape (:, :)
        The n*n unitary matrix P^H, if `vect` = 'P' or 'B'. If `vect` =
        'N' or 'Q', `pt` is not referenced.

    c : complex, ndarray, shape (:, ncc)
        `c` is overwritten by Q^HC. If ncc = 0, `c` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `vect`.

            Constraint: `vect` = 'N', 'Q', 'P' or 'B'.

        (`errno` -2)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -4)
            On entry, error in parameter ncc.

            Constraint: ncc >= 0.

        (`errno` -5)
            On entry, error in parameter `kl`.

            Constraint: `kl` >= 0.

        (`errno` -6)
            On entry, error in parameter `ku`.

            Constraint: `ku` >= 0.

    Notes
    -----
    ``zgbbrd`` reduces a complex m*n band matrix to real upper
    bidiagonal form B by a unitary transformation: A = QBP^H.
    The unitary matrices Q and P^H, of order m and n respectively, are
    determined as a product of Givens rotation matrices, and may be
    formed explicitly by the function if required.
    A matrix C may also be updated to give C~ = Q^HC.

    The function uses a vectorizable form of the reduction.
    """

    fun_name = 'naginterfaces.library.lapackeig.zgbbrd'

    _ab = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ab'},
        ab,
        copy=True,
    )
    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_ab, 'ab'), (_c, 'c')),
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _ncc_py = _b_utils._EngineComplex128ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[1]
    _ncc = _b_utils._EngineIntScalarType(
        _ncc_py, {'fun_name': fun_name, 'entity': '_ncc'},
    ).to_ctypes()
    _m = _b_utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = _b_utils._EngineIntScalarType.to_py(_m)
    _d = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (min(_m_py, _n_py),),
    )
    _e = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (min(_m_py, _n_py)-1,),
    )
    _vect = _b_utils._EngineCharScalarType(
        vect, {'fun_name': fun_name, 'entity': 'vect'}, exp_chrlen=1,
    ).to_ctypes()
    _vect_py = _b_utils._EngineCharScalarType.to_py(_vect)
    if _vect_py in ('Q', 'q', 'B', 'b'):
        _q_dim1 = _m_py
    else:
        _q_dim1 = 1
    if _vect_py in ('Q', 'q', 'B', 'b'):
        _q_dim2 = _m_py
    else:
        _q_dim2 = 1
    exp_shape_q = (_q_dim1, _q_dim2)
    _q = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_q,
        sorder=_m_sorder,
    )
    if _vect_py in ('P', 'p', 'B', 'b'):
        _pt_dim1 = _n_py
    else:
        _pt_dim1 = 1
    if _vect_py in ('P', 'p', 'B', 'b'):
        _pt_dim2 = _n_py
    else:
        _pt_dim2 = 1
    exp_shape_pt = (_pt_dim1, _pt_dim2)
    _pt = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_pt,
        sorder=_m_sorder,
    )
    _b_lapackeig.zgbbrd(
        vect,
        _m,
        _n,
        _ncc,
        kl,
        ku,
        _ab,
        _d,
        _e,
        _q,
        _pt,
        _c,
    )
    _ab_py = _ab
    _c_py = _c
    _d_py = _d
    _e_py = _e
    _q_py = _q
    _pt_py = _pt
    _return_data = ZgbbrdReturnData(
        _ab_py,
        _d_py,
        _e_py,
        _q_py,
        _pt_py,
        _c_py,
    )
    return _return_data

DbdsvdxReturnData = _collections.namedtuple(
    'DbdsvdxReturnData',
    ['ns', 's', 'z'],
)

def dbdsvdx(uplo, jobz, erange, d, e, vl, vu, il, iu):
    u"""
    Computes all or selected singular values of the singular value
    decomposition of a real square bidiagonal matrix, optionally
    computing the corresponding left and right singular vectors.

    ``dbdsvdx`` computes all or selected singular values and,
    optionally, the corresponding left and right singular vectors of a
    real n*n (upper or lower) bidiagonal matrix B.

    For full information please refer to the NAG Library document for
    f08mb

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08mbf.html

    Parameters
    ----------
    uplo : str, length 1
        Indicates whether B is upper or lower bidiagonal.

        `uplo` = 'U'

            B is upper bidiagonal.

        `uplo` = 'L'

            B is lower bidiagonal.

    jobz : str, length 1
        Indicates whether singular vectors are computed.

        `jobz` = 'N'

            Only singular values are computed.

        `jobz` = 'V'

            Singular values and singular vectors are computed.

    erange : str, length 1
        Indicates which singular values should be returned.

        `erange` = 'A'

            All singular values will be found.

        `erange` = 'V'

            All singular values in the half-open interval (`vl`,`vu`]
            will be found.

        `erange` = 'I'

            The `il`th through `iu`th singular values will be found.

    d : float, array-like, shape (n)
        The diagonal elements d of the bidiagonal matrix B.

    e : float, array-like, shape (n-1)
        The (n-1) off-diagonal elements e of the bidiagonal matrix B.

    vl : float
        If `erange` = 'V', the lower bound of the interval to be
        searched for singular values.

        If `erange` = 'A' or 'I', `vl` is not referenced.

    vu : float
        If `erange` = 'V', the upper bound of the interval to be
        searched for singular values.

        If `erange` = 'A' or 'I', `vu` is not referenced.

    il : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest singular values to
        be returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    iu : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest singular values to
        be returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    Returns
    -------
    ns : int
        The total number of singular values found. 0 <= `ns` <= n.

        If `erange` = 'A', `ns` = n.

        If `erange` = 'I', `ns` = `iu`-`il`+1.

    s : float, ndarray, shape (n)
        The first `ns` elements contain the selected singular values in
        ascending order.

    z : float, ndarray, shape (:, :)
        If `jobz` = 'V', then if errno = 0 the first `ns` columns of `z`
        contain the singular vectors of the matrix B corresponding to
        the selected singular values, with U in rows 1 to n and V in
        rows n+1 to n*2.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `erange`.

            Constraint: `erange` = 'A', 'V' or 'I'.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -7)
            On entry, error in parameter `vl`.

            Constraint: 0.0 <= `vl`.

        (`errno` -8)
            On entry, error in parameter `vu`.

            Constraint: `vl` < `vu`.

        (`errno` -9)
            On entry, error in parameter `il`.

        (`errno` -10)
            On entry, error in parameter `iu`.

        (`errno` i > 0)
            The algorithm failed to converge; *<value>* eigenvectors of
            the associated eigenproblem did not converge. Their indices
            are stored in array iwork.

    Notes
    -----
    ``dbdsvdx`` computes the singular value decomposition (SVD) of a
    real n*n (upper or lower) bidiagonal matrix B as

        B = USV^T,

    where S is a diagonal matrix with non-negative diagonal elements
    (the singular values of B), and U and V^T are orthogonal matrices.
    The columns of U and V are the left and right singular vectors of B,
    respectively.

    Given an upper bidiagonal matrix B with diagonal d = (d_1 d_2 ...
    d_n) and superdiagonal e = (e_1 e_2 ... e_{N-1}), ``dbdsvdx``
    computes the singular value decomposition of B through the
    eigenvalues and eigenvectors of the (n*2)*(n*2) tridiagonal matrix

        TGK = [table omitted].

    If (s,u,v) is a singular triplet of B with ||u|| = ||v|| = 1, then
    (s,q_1) and (-s,q_2), ||q_1|| = ||q_2|| = 1, are eigenpairs of TGK,
    with q_1 = (v_1,u_1,v_2,u_2,...,v_n,u_n)/sqrt(2), and q_2 =
    (-v_1,u_1,-v_2,u_2,...,-v_n,u_n)/sqrt(2).

    Given a TGK matrix, you can either

    (i) compute -s,-v and change signs so that the singular values (and
        corresponding vectors) are already in descending order (as in
        ``dgesvd``) or

    (#) compute s,v and reorder the values (and corresponding vectors).

    ``dbdsvdx`` implements (i) by calling ``dstevx`` (bisection plus
    inverse iteration, to be replaced with a version of the Multiple
    Relative Robust Representation algorithm. (See Williams and Lang
    (2013).)

    Alternative to computing all singular values of B, a selected set
    can be computed.
    The set is either those singular values lying in a given interval,
    sigma in (v_l,v_u], or those whose index (counting from largest to
    smallest in magnitude) lies in a given range 1 <= i_l,...,i_u <= n.
    In these cases, the corresponding left and right singular vectors
    can optionally be computed.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Williams, P and Lang, B, 2013, `A framework for the MR^3 Algorithm:
    theory and implementation`, SIAM J. Sci. Comput. (35), 740--766
    """

    fun_name = 'naginterfaces.library.lapackeig.dbdsvdx'

    _d = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'd'},
        d,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_d, {'fun_name': fun_name, 'entity': 'd'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _e = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'e'},
        e,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _s = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobz = _b_utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = _b_utils._EngineCharScalarType.to_py(_jobz)
    _erange = _b_utils._EngineCharScalarType(
        erange, {'fun_name': fun_name, 'entity': 'erange'}, exp_chrlen=1,
    ).to_ctypes()
    _erange_py = _b_utils._EngineCharScalarType.to_py(_erange)
    _iu = _b_utils._EngineIntScalarType(
        iu, {'fun_name': fun_name, 'entity': 'iu'},
    ).to_ctypes()
    _iu_py = _b_utils._EngineIntScalarType.to_py(_iu)
    _il = _b_utils._EngineIntScalarType(
        il, {'fun_name': fun_name, 'entity': 'il'},
    ).to_ctypes()
    _il_py = _b_utils._EngineIntScalarType.to_py(_il)
    if _jobz_py in ('V', 'v'):
        _z_dim1 = max(2, _n_py*2)
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        if _erange_py in ('I', 'i'):
            _z_dim2 = _iu_py-_il_py+2
        else:
            _z_dim2 = _n_py+1
    else:
        _z_dim2 = 1
    exp_shape_z = (_z_dim1, _z_dim2)
    _z = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_z,
        sorder=_b_utils._EngineData.default_sorder,
    )
    _return_scalar = _b_lapackeig.dbdsvdx(
        uplo,
        jobz,
        erange,
        _n,
        _d,
        _e,
        vl,
        vu,
        _il,
        _iu,
        _s,
        _z,
    )
    _ns_py = _return_scalar
    _s_py = _s
    if _jobz_py in ('V', 'v'):
        _z_dim2 = min(_n_py, _ns_py)+1
    else:
        _z_dim2 = 1
    _z_py = _z[:, :_z_dim2]
    _return_data = DbdsvdxReturnData(
        _ns_py,
        _s_py,
        _z_py,
    )
    return _return_data

DbdsdcReturnData = _collections.namedtuple(
    'DbdsdcReturnData',
    ['d', 'u', 'vt', 'q', 'iq'],
)

def dbdsdc(uplo, compq, d, e):
    u"""
    Computes the singular value decomposition of a real bidiagonal
    matrix, optionally computing the singular vectors
    (divide-and-conquer).

    ``dbdsdc`` computes the singular values and, optionally, the left
    and right singular vectors of a real n*n (upper or lower) bidiagonal
    matrix B.

    For full information please refer to the NAG Library document for
    f08md

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08mdf.html

    Parameters
    ----------
    uplo : str, length 1
        Indicates whether B is upper or lower bidiagonal.

        `uplo` = 'U'

            B is upper bidiagonal.

        `uplo` = 'L'

            B is lower bidiagonal.

    compq : str, length 1
        Specifies whether singular vectors are to be computed.

        `compq` = 'N'

            Compute singular values only.

        `compq` = 'P'

            Compute singular values and compute singular vectors in
            compact form.

        `compq` = 'I'

            Compute singular values and singular vectors.

    d : float, array-like, shape (n)
        The n diagonal elements of the bidiagonal matrix B.

    e : float, array-like, shape (n-1)
        The (n-1) off-diagonal elements of the bidiagonal matrix B.

    Returns
    -------
    d : float, ndarray, shape (n)
        If no exception or warning is raised, the singular values of B.

    u : None or float, ndarray, shape (:, :)
        If `compq` = 'I', then if no exception or warning is raised, `u`
        contains the left singular vectors of the bidiagonal matrix B.

        If `compq` != 'I', `u` is not referenced.

    vt : None or float, ndarray, shape (:, :)
        If `compq` = 'I', then if no exception or warning is raised, the
        rows of `vt` contain the right singular vectors of the
        bidiagonal matrix B.

        If `compq` != 'I', `vt` is not referenced.

    q : None or float, ndarray, shape (:)
        If `compq` = 'P', then if no exception or warning is raised, `q`
        and `iq` contain the left and right singular vectors in a
        compact form, requiring O(n*log2(n)) space instead of 2*n^2. In
        particular, `q` contains all the real data in the first ldq =
        n*(11+2*smlsiz+8*int(log2(n/(smlsiz+1)))) elements of `q`, where
        smlsiz is equal to the maximum size of the subproblems at the
        bottom of the computation tree (usually about 25).

        If `compq` != 'P', `q` is not referenced.

    iq : None or int, ndarray, shape (:)
        If `compq` = 'P', then if no exception or warning is raised, `q`
        and `iq` contain the left and right singular vectors in a
        compact form, requiring O(n*log2(n)) space instead of 2*n^2. In
        particular, `iq` contains all integer data in the first ldiq =
        n*(3+3*int(log2(n/(smlsiz+1)))) elements of `iq`, where smlsiz
        is equal to the maximum size of the subproblems at the bottom of
        the computation tree (usually about 25).

        If `compq` != 'P', `iq` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter `compq`.

            Constraint: `compq` = 'N', 'P' or 'I'.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` i > 0)
            The algorithm failed to compute a singular value. The update
            process of divide-and-conquer failed.

    Notes
    -----
    ``dbdsdc`` computes the singular value decomposition (SVD) of the
    (upper or lower) bidiagonal matrix B as

        B = USV^T,

    where S is a diagonal matrix with non-negative diagonal elements
    s_{ii} = s_i, such that

        s_1 >= s_2 >= ... >= s_n >= 0,

    and U and V are orthogonal matrices.
    The diagonal elements of S are the singular values of B and the
    columns of U and V are respectively the corresponding left and right
    singular vectors of B.

    When only singular values are required the function uses the QR
    algorithm, but when singular vectors are required a divide and
    conquer method is used.
    The singular values can optionally be returned in compact form,
    although currently no function is available to apply U or V when
    stored in compact form.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dbdsdc'

    _d = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'd'},
        d,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_d, {'fun_name': fun_name, 'entity': 'd'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _e = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'e'},
        e,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _compq = _b_utils._EngineCharScalarType(
        compq, {'fun_name': fun_name, 'entity': 'compq'}, exp_chrlen=1,
    ).to_ctypes()
    _compq_py = _b_utils._EngineCharScalarType.to_py(_compq)
    if _compq_py in ('I', 'i'):
        if _compq_py in ('I', 'i'):
            _u_dim1 = _n_py
        else:
            _u_dim1 = 1
        if _compq_py in ('I', 'i'):
            _u_dim2 = _n_py
        else:
            _u_dim2 = 0
        exp_shape_u = (_u_dim1, _u_dim2)
        _u = _b_utils._EngineFloat64ArrayType.empty_ndarray(
            exp_shape_u,
            sorder=_b_utils._EngineData.default_sorder,
        )
    else:
        _u = None
    if _compq_py in ('I', 'i'):
        if _compq_py in ('I', 'i'):
            _vt_dim1 = _n_py
        else:
            _vt_dim1 = 1
        if _compq_py in ('I', 'i'):
            _vt_dim2 = _n_py
        else:
            _vt_dim2 = 0
        exp_shape_vt = (_vt_dim1, _vt_dim2)
        _vt = _b_utils._EngineFloat64ArrayType.empty_ndarray(
            exp_shape_vt,
            sorder=_b_utils._EngineData.default_sorder,
        )
    else:
        _vt = None
    _ldq, _ldiq = _b_utils._f08mdan(True, _n)
    _ldq_py = _b_utils._EngineIntScalarType.to_py(_ldq)
    _vl_ldq_py = _ldq_py
    if _compq_py in ('P', 'p'):
        if _compq_py in ('P', 'p'):
            _q_dim1 = _vl_ldq_py
        else:
            _q_dim1 = 0
        exp_shape_q = (_q_dim1,)
        _q = _b_utils._EngineFloat64ArrayType.empty_ndarray(
            exp_shape_q,
        )
    else:
        _q = None
    _ldiq_py = _b_utils._EngineIntScalarType.to_py(_ldiq)
    _vl_ldiq_py = _ldiq_py
    if _compq_py in ('P', 'p'):
        if _compq_py in ('P', 'p'):
            _iq_dim1 = max(_n_py, _vl_ldiq_py, _vl_ldq_py)
        else:
            _iq_dim1 = 0
        exp_shape_iq = (_iq_dim1,)
        _iq = _b_utils._EngineIntArrayType.empty_ndarray(
            exp_shape_iq,
        )
    else:
        _iq = None
    _b_lapackeig.dbdsdc(
        uplo,
        compq,
        _n,
        _d,
        _e,
        _u,
        _vt,
        _q,
        _iq,
    )
    _d_py = _d
    if _compq_py in ('I', 'i'):
        _u_py = _u
    else:
        _u_py = None
    if _compq_py in ('I', 'i'):
        _vt_py = _vt
    else:
        _vt_py = None
    if _compq_py in ('P', 'p'):
        _q_py = _q
    else:
        _q_py = None
    if _compq_py in ('P', 'p'):
        _iq_dim1 = max(_n_py, _vl_ldiq_py)
    else:
        _iq_dim1 = 0
    if _compq_py in ('P', 'p'):
        _iq_py = _iq[:_iq_dim1]
    else:
        _iq_py = None
    _return_data = DbdsdcReturnData(
        _d_py,
        _u_py,
        _vt_py,
        _q_py,
        _iq_py,
    )
    return _return_data

DbdsqrReturnData = _collections.namedtuple(
    'DbdsqrReturnData',
    ['d', 'e', 'vt', 'u', 'c'],
)

def dbdsqr(uplo, d, e, vt, u, c):
    u"""
    Performs an SVD of real bidiagonal matrix reduced from real general
    matrix.

    ``dbdsqr`` computes the singular value decomposition of a real upper
    or lower bidiagonal matrix, or of a real general matrix which has
    been reduced to bidiagonal form.

    For full information please refer to the NAG Library document for
    f08me

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08mef.html

    Parameters
    ----------
    uplo : str, length 1
        Indicates whether B is an upper or lower bidiagonal matrix.

        `uplo` = 'U'

            B is an upper bidiagonal matrix.

        `uplo` = 'L'

            B is a lower bidiagonal matrix.

    d : float, array-like, shape (n)
        The diagonal elements of the bidiagonal matrix B.

    e : float, array-like, shape (max(1, n-1))
        The off-diagonal elements of the bidiagonal matrix B.

    vt : float, array-like, shape (:, ncvt)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if ncvt>0: n; otherwise: 1.

        If ncvt > 0, `vt` must contain an n*ncvt matrix. If the right
        singular vectors of B are required, ncvt = n and `vt` must
        contain the unit matrix; if the right singular vectors of A are
        required, `vt` must contain the orthogonal matrix P^T returned
        by ``dorgbr`` with vect = 'P'.

    u : float, array-like, shape (nru, n)
        If nru > 0, `u` must contain an nru*n matrix. If the left
        singular vectors of B are required, nru = n and `u` must contain
        the unit matrix; if the left singular vectors of A are required,
        `u` must contain the orthogonal matrix Q returned by ``dorgbr``
        with vect = 'Q'.

    c : float, array-like, shape (:, ncc)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if ncc>0: n; otherwise: 1.

        The n*ncc matrix C if ncc > 0.

    Returns
    -------
    d : float, ndarray, shape (n)
        The singular values in decreasing order of magnitude, unless
        `errno` > 0 (in which case see [Raises/Warns]).

    e : float, ndarray, shape (max(1, n-1))
        `e` is overwritten, but if `errno` > 0 see [Raises/Warns].

    vt : float, ndarray, shape (:, ncvt)
        The n*ncvt matrix V^T or V^TP^T of right singular vectors,
        stored by rows.

        If ncvt = 0, `vt` is not referenced.

    u : float, ndarray, shape (nru, n)
        The nru*n matrix U or QU of left singular vectors, stored as
        columns of the matrix.

        If nru = 0, `u` is not referenced.

    c : float, ndarray, shape (:, ncc)
        `c` is overwritten by the matrix U^TC. If ncc = 0, `c` is not
        referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -3)
            On entry, error in parameter ncvt.

            Constraint: ncvt >= 0.

        (`errno` -4)
            On entry, error in parameter nru.

            Constraint: nru >= 0.

        (`errno` -5)
            On entry, error in parameter ncc.

            Constraint: ncc >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            *<value>* off-diagonals did not converge. The arrays `d` and
            `e` contain the diagonal and off-diagonal elements,
            respectively, of a bidiagonal matrix orthogonally equivalent
            to B.

    Notes
    -----
    ``dbdsqr`` computes the singular values and, optionally, the left or
    right singular vectors of a real upper or lower bidiagonal matrix B.
    In other words, it can compute the singular value decomposition
    (SVD) of B as

        B = U Sigma V^T.

    Here Sigma is a diagonal matrix with real diagonal elements sigma_i
    (the singular values of B), such that

        sigma_1 >= sigma_2 >= ... >= sigma_n >= 0;

    U is an orthogonal matrix whose columns are the left singular
    vectors u_i; V is an orthogonal matrix whose rows are the right
    singular vectors v_i.
    Thus

        Bu_i = sigma_i v_i and B^Tv_i = sigma_i u_i, i = 1,2,...,n.

    To compute U and/or V^T, the arrays `u` and/or `vt` must be
    initialized to the unit matrix before ``dbdsqr`` is called.

    The function may also be used to compute the SVD of a real general
    matrix A which has been reduced to bidiagonal form by an orthogonal
    transformation: A = QBP^T.
    If A is m*n with m >= n, then Q is m*n and P^T is n*n; if A is n*p
    with n < p, then Q is n*n and P^T is n*p.
    In this case, the matrices Q and/or P^T must be formed explicitly by
    ``dorgbr`` and passed to ``dbdsqr`` in the arrays `u` and/or `vt`
    respectively.

    ``dbdsqr`` also has the capability of forming U^TC, where C is an
    arbitrary real matrix; this is needed when using the SVD to solve
    linear least squares problems.

    ``dbdsqr`` uses two different algorithms.
    If any singular vectors are required (i.e., if ncvt > 0 or nru > 0
    or ncc > 0), the bidiagonal QR algorithm is used, switching between
    zero-shift and implicitly shifted forms to preserve the accuracy of
    small singular values, and switching between QR and QL variants in
    order to handle graded matrices effectively (see Demmel and Kahan
    (1990)).
    If only singular values are required (i.e., if ncvt = nru = ncc =
    0), they are computed by the differential qd algorithm (see Fernando
    and Parlett (1994)), which is faster and can achieve even greater
    accuracy.

    The singular vectors are normalized so that ||u_i|| = ||v_i|| = 1,
    but are determined only to within a factor +/-1.

    References
    ----------
    Demmel, J W and Kahan, W, 1990, `Accurate singular values of
    bidiagonal matrices`, SIAM J. Sci. Statist. Comput. (11), 873--912

    Fernando, K V and Parlett, B N, 1994, `Accurate singular values and
    differential qd algorithms`, Numer. Math. (67), 191--229

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dbdsqr'

    _d = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'd'},
        d,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_d, {'fun_name': fun_name, 'entity': 'd'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _vt = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'vt'},
        vt,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _ncvt_py = _b_utils._EngineFloat64ArrayType.get_shape(_vt, {'fun_name': fun_name, 'entity': 'vt'}, exp_rank=2)[1]
    _ncvt = _b_utils._EngineIntScalarType(
        _ncvt_py, {'fun_name': fun_name, 'entity': '_ncvt'},
    ).to_ctypes()
    _u = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'u'},
        u,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _nru_py = _b_utils._EngineFloat64ArrayType.get_shape(_u, {'fun_name': fun_name, 'entity': 'u'}, exp_rank=2)[0]
    _nru = _b_utils._EngineIntScalarType(
        _nru_py, {'fun_name': fun_name, 'entity': '_nru'},
    ).to_ctypes()
    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _ncc_py = _b_utils._EngineFloat64ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[1]
    _ncc = _b_utils._EngineIntScalarType(
        _ncc_py, {'fun_name': fun_name, 'entity': '_ncc'},
    ).to_ctypes()
    _e = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'e'},
        e,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    try:
        _b_lapackeig.dbdsqr(
            uplo,
            _n,
            _ncvt,
            _nru,
            _ncc,
            _d,
            _e,
            _vt,
            _u,
            _c,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _d_py = _d
    _e_py = _e
    _vt_py = _vt
    _u_py = _u
    _c_py = _c
    _return_data = DbdsqrReturnData(
        _d_py,
        _e_py,
        _vt_py,
        _u_py,
        _c_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

ZbdsqrReturnData = _collections.namedtuple(
    'ZbdsqrReturnData',
    ['d', 'e', 'vt', 'u', 'c'],
)

def zbdsqr(uplo, d, e, vt, u, c):
    u"""
    Performs an SVD of real bidiagonal matrix reduced from complex
    general matrix.

    ``zbdsqr`` computes the singular value decomposition of a complex
    general matrix which has been reduced to bidiagonal form.

    For full information please refer to the NAG Library document for
    f08ms

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08msf.html

    Parameters
    ----------
    uplo : str, length 1
        Indicates whether B is an upper or lower bidiagonal matrix.

        `uplo` = 'U'

            B is an upper bidiagonal matrix.

        `uplo` = 'L'

            B is a lower bidiagonal matrix.

    d : float, array-like, shape (n)
        The diagonal elements of the bidiagonal matrix B.

    e : float, array-like, shape (max(1, n-1))
        The off-diagonal elements of the bidiagonal matrix B.

    vt : complex, array-like, shape (:, ncvt)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if ncvt>0: n; otherwise: 1.

        If ncvt > 0, `vt` must contain an n*ncvt matrix. If the right
        singular vectors of B are required, ncvt = n and `vt` must
        contain the unit matrix; if the right singular vectors of A are
        required, `vt` must contain the unitary matrix P^H returned by
        ``zungbr`` with vect = 'P'.

    u : complex, array-like, shape (nru, n)
        If nru > 0, `u` must contain an nru*n matrix. If the left
        singular vectors of B are required, nru = n and `u` must contain
        the unit matrix; if the left singular vectors of A are required,
        `u` must contain the unitary matrix Q returned by ``zungbr``
        with vect = 'Q'.

    c : complex, array-like, shape (:, ncc)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if ncc>0: n; otherwise: 1.

        The n*ncc matrix C if ncc > 0.

    Returns
    -------
    d : float, ndarray, shape (n)
        The singular values in decreasing order of magnitude, unless
        `errno` > 0 (in which case see [Raises/Warns]).

    e : float, ndarray, shape (max(1, n-1))
        `e` is overwritten, but if `errno` > 0 see [Raises/Warns].

    vt : complex, ndarray, shape (:, ncvt)
        The n*ncvt matrix V^H or V^HP^H of right singular vectors,
        stored by rows.

        If ncvt = 0, `vt` is not referenced.

    u : complex, ndarray, shape (nru, n)
        The nru*n matrix U or QU of left singular vectors, stored as
        columns of the matrix.

        If nru = 0, `u` is not referenced.

    c : complex, ndarray, shape (:, ncc)
        `c` is overwritten by the matrix U^HC. If ncc = 0, `c` is not
        referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -3)
            On entry, error in parameter ncvt.

            Constraint: ncvt >= 0.

        (`errno` -4)
            On entry, error in parameter nru.

            Constraint: nru >= 0.

        (`errno` -5)
            On entry, error in parameter ncc.

            Constraint: ncc >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            *<value>* off-diagonals did not converge. The arrays `d` and
            `e` contain the diagonal and off-diagonal elements,
            respectively, of a bidiagonal matrix orthogonally equivalent
            to B.

    Notes
    -----
    ``zbdsqr`` computes the singular values and, optionally, the left or
    right singular vectors of a real upper or lower bidiagonal matrix B.
    In other words, it can compute the singular value decomposition
    (SVD) of B as

        B = U Sigma V^T.

    Here Sigma is a diagonal matrix with real diagonal elements sigma_i
    (the singular values of B), such that

        sigma_1 >= sigma_2 >= ... >= sigma_n >= 0;

    U is an orthogonal matrix whose columns are the left singular
    vectors u_i; V is an orthogonal matrix whose rows are the right
    singular vectors v_i.
    Thus

        Bu_i = sigma_i v_i and B^Tv_i = sigma_i u_i, i = 1,2,...,n.

    To compute U and/or V^T, the arrays `u` and/or `vt` must be
    initialized to the unit matrix before ``zbdsqr`` is called.

    The function stores the real orthogonal matrices U and V^T in
    complex arrays `u` and `vt`, so that it may also be used to compute
    the SVD of a complex general matrix A which has been reduced to
    bidiagonal form by a unitary transformation: A = QBP^H.
    If A is m*n with m >= n, then Q is m*n and P^H is n*n; if A is n*p
    with n < p, then Q is n*n and P^H is n*p.
    In this case, the matrices Q and/or P^H must be formed explicitly by
    ``zungbr`` and passed to ``zbdsqr`` in the arrays `u` and/or `vt`
    respectively.

    ``zbdsqr`` also has the capability of forming U^HC, where C is an
    arbitrary complex matrix; this is needed when using the SVD to solve
    linear least squares problems.

    ``zbdsqr`` uses two different algorithms.
    If any singular vectors are required (i.e., if ncvt > 0 or nru > 0
    or ncc > 0), the bidiagonal QR algorithm is used, switching between
    zero-shift and implicitly shifted forms to preserve the accuracy of
    small singular values, and switching between QR and QL variants in
    order to handle graded matrices effectively (see Demmel and Kahan
    (1990)).
    If only singular values are required (i.e., if ncvt = nru = ncc =
    0), they are computed by the differential qd algorithm (see Fernando
    and Parlett (1994)), which is faster and can achieve even greater
    accuracy.

    The singular vectors are normalized so that ||u_i|| = ||v_i|| = 1,
    but are determined only to within a complex factor of absolute value
    1.

    References
    ----------
    Demmel, J W and Kahan, W, 1990, `Accurate singular values of
    bidiagonal matrices`, SIAM J. Sci. Statist. Comput. (11), 873--912

    Fernando, K V and Parlett, B N, 1994, `Accurate singular values and
    differential qd algorithms`, Numer. Math. (67), 191--229

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zbdsqr'

    _d = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'd'},
        d,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_d, {'fun_name': fun_name, 'entity': 'd'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _vt = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'vt'},
        vt,
        copy=True,
    )
    _ncvt_py = _b_utils._EngineComplex128ArrayType.get_shape(_vt, {'fun_name': fun_name, 'entity': 'vt'}, exp_rank=2)[1]
    _ncvt = _b_utils._EngineIntScalarType(
        _ncvt_py, {'fun_name': fun_name, 'entity': '_ncvt'},
    ).to_ctypes()
    _u = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'u'},
        u,
        copy=True,
    )
    _nru_py = _b_utils._EngineComplex128ArrayType.get_shape(_u, {'fun_name': fun_name, 'entity': 'u'}, exp_rank=2)[0]
    _nru = _b_utils._EngineIntScalarType(
        _nru_py, {'fun_name': fun_name, 'entity': '_nru'},
    ).to_ctypes()
    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=True,
    )
    _ncc_py = _b_utils._EngineComplex128ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[1]
    _ncc = _b_utils._EngineIntScalarType(
        _ncc_py, {'fun_name': fun_name, 'entity': '_ncc'},
    ).to_ctypes()
    _e = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'e'},
        e,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    try:
        _b_lapackeig.zbdsqr(
            uplo,
            _n,
            _ncvt,
            _nru,
            _ncc,
            _d,
            _e,
            _vt,
            _u,
            _c,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _d_py = _d
    _e_py = _e
    _vt_py = _vt
    _u_py = _u
    _c_py = _c
    _return_data = ZbdsqrReturnData(
        _d_py,
        _e_py,
        _vt_py,
        _u_py,
        _c_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

DgeevReturnData = _collections.namedtuple(
    'DgeevReturnData',
    ['a', 'wr', 'wi', 'vl', 'vr'],
)

def dgeev(jobvl, jobvr, a):
    u"""
    Computes all eigenvalues and, optionally, left and/or right
    eigenvectors of a real nonsymmetric matrix.

    ``dgeev`` computes the eigenvalues and, optionally, the left and/or
    right eigenvectors for an n*n real nonsymmetric matrix A.

    For full information please refer to the NAG Library document for
    f08na

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08naf.html

    Parameters
    ----------
    jobvl : str, length 1
        If `jobvl` = 'N', the left eigenvectors of A are not computed.

        If `jobvl` = 'V', the left eigenvectors of A are computed.

    jobvr : str, length 1
        If `jobvr` = 'N', the right eigenvectors of A are not computed.

        If `jobvr` = 'V', the right eigenvectors of A are computed.

    a : float, array-like, shape (n, n)
        The n*n matrix A.

    Returns
    -------
    a : float, ndarray, shape (n, n)
        `a` has been overwritten.

    wr : float, ndarray, shape (n)
        `wr` and `wi` contain the real and imaginary parts,
        respectively, of the computed eigenvalues. Complex conjugate
        pairs of eigenvalues appear consecutively with the eigenvalue
        having the positive imaginary part first.

    wi : float, ndarray, shape (n)
        `wr` and `wi` contain the real and imaginary parts,
        respectively, of the computed eigenvalues. Complex conjugate
        pairs of eigenvalues appear consecutively with the eigenvalue
        having the positive imaginary part first.

    vl : float, ndarray, shape (:, :)
        If `jobvl` = 'V', the left eigenvectors u_j are stored one after
        another in the columns of `vl`, in the same order as their
        corresponding eigenvalues. If the jth eigenvalue is real, then
        u_j = `vl`[i-1,j-1], for i = 1,2,...,n. If the jth and (j+1)st
        eigenvalues form a complex conjugate pair, then u_j =
        `vl`[i-1,j-1]+i*`vl`[i-1,j] and u_{j+1} =
        `vl`[i-1,j-1]-i*`vl`[i-1,j], for i = 1,2,...,n.

        If `jobvl` = 'N', `vl` is not referenced.

    vr : float, ndarray, shape (:, :)
        If `jobvr` = 'V', the right eigenvectors v_j are stored one
        after another in the columns of `vr`, in the same order as their
        corresponding eigenvalues. If the jth eigenvalue is real, then
        v_j = `vr`[i-1,j-1], for i = 1,2,...,n. If the jth and (j+1)st
        eigenvalues form a complex conjugate pair, then v_j =
        `vr`[i-1,j-1]+i*`vr`[i-1,j] and v_{j+1} =
        `vr`[i-1,j-1]-i*`vr`[i-1,j], for i = 1,2,...,n.

        If `jobvr` = 'N', `vr` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobvl`.

            Constraint: `jobvl` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `jobvr`.

            Constraint: `jobvr` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            The QR algorithm failed to compute all the eigenvalues, and
            no eigenvectors have been computed; elements *<value>* to n
            of `wr` and `wi` contain eigenvalues which have converged.

    Notes
    -----
    The right eigenvector v_j of A satisfies

        Av_j = lambda_j v_j

    where lambda_j is the jth eigenvalue of A.
    The left eigenvector u_j of A satisfies

        u_j^HA = lambda_j u_j^H

    where u_j^H denotes the conjugate transpose of u_j.

    The matrix A is first reduced to upper Hessenberg form by means of
    orthogonal similarity transformations, and the QR algorithm is then
    used to further reduce the matrix to upper quasi-triangular Schur
    form, T, with 1*1 and 2*2 blocks on the main diagonal.
    The eigenvalues are computed from T, the 2*2 blocks corresponding to
    complex conjugate pairs and, optionally, the eigenvectors of T are
    computed and backtransformed to the eigenvectors of A.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dgeev'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'),),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _wr = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _wi = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobvl = _b_utils._EngineCharScalarType(
        jobvl, {'fun_name': fun_name, 'entity': 'jobvl'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvl_py = _b_utils._EngineCharScalarType.to_py(_jobvl)
    if _jobvl_py in ('V', 'v'):
        _vl_dim1 = _n_py
    else:
        _vl_dim1 = 1
    if _jobvl_py in ('V', 'v'):
        _vl_dim2 = _n_py
    else:
        _vl_dim2 = 1
    exp_shape_vl = (_vl_dim1, _vl_dim2)
    _vl = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_vl,
        sorder=_m_sorder,
    )
    _jobvr = _b_utils._EngineCharScalarType(
        jobvr, {'fun_name': fun_name, 'entity': 'jobvr'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvr_py = _b_utils._EngineCharScalarType.to_py(_jobvr)
    if _jobvr_py in ('V', 'v'):
        _vr_dim1 = _n_py
    else:
        _vr_dim1 = 1
    if _jobvr_py in ('V', 'v'):
        _vr_dim2 = _n_py
    else:
        _vr_dim2 = 1
    exp_shape_vr = (_vr_dim1, _vr_dim2)
    _vr = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_vr,
        sorder=_m_sorder,
    )
    try:
        _b_lapackeig.dgeev(
            jobvl,
            jobvr,
            _n,
            _a,
            _wr,
            _wi,
            _vl,
            _vr,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _a_py = _a
    _wr_py = _wr
    _wi_py = _wi
    _vl_py = _vl
    _vr_py = _vr
    _return_data = DgeevReturnData(
        _a_py,
        _wr_py,
        _wi_py,
        _vl_py,
        _vr_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

DgeevxReturnData = _collections.namedtuple(
    'DgeevxReturnData',
    ['a', 'wr', 'wi', 'vl', 'vr', 'ilo', 'ihi', 'scale', 'abnrm', 'rconde', 'rcondv'],
)

def dgeevx(balanc, jobvl, jobvr, sense, a):
    u"""
    Computes all eigenvalues and, optionally, left and/or right
    eigenvectors of a real nonsymmetric matrix; also, optionally, the
    balancing transformation, the reciprocal condition numbers for the
    eigenvalues and for the right eigenvectors.

    ``dgeevx`` computes the eigenvalues and, optionally, the left and/or
    right eigenvectors for an n*n real nonsymmetric matrix A.

    Optionally, it also computes a balancing transformation to improve
    the conditioning of the eigenvalues and eigenvectors, reciprocal
    condition numbers for the eigenvalues, and reciprocal condition
    numbers for the right eigenvectors.

    For full information please refer to the NAG Library document for
    f08nb

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08nbf.html

    Parameters
    ----------
    balanc : str, length 1
        Indicates how the input matrix should be diagonally scaled
        and/or permuted to improve the conditioning of its eigenvalues.

        `balanc` = 'N'

            Do not diagonally scale or permute.

        `balanc` = 'P'

            Perform permutations to make the matrix more nearly upper
            triangular. Do not diagonally scale.

        `balanc` = 'S'

            Diagonally scale the matrix, i.e., replace A*DAD^-1, where D
            is a diagonal matrix chosen to make the rows and columns of
            A more equal in norm. Do not permute.

        `balanc` = 'B'

            Both diagonally scale and permute A.

        Computed reciprocal condition numbers will be for the matrix
        after balancing and/or permuting.

        Permuting does not change condition numbers (in exact
        arithmetic), but balancing does.

    jobvl : str, length 1
        If `jobvl` = 'N', the left eigenvectors of A are not computed.

        If `jobvl` = 'V', the left eigenvectors of A are computed.

        If `sense` = 'E' or 'B', `jobvl` must be set to `jobvl` = 'V'.

    jobvr : str, length 1
        If `jobvr` = 'N', the right eigenvectors of A are not computed.

        If `jobvr` = 'V', the right eigenvectors of A are computed.

        If `sense` = 'E' or 'B', `jobvr` must be set to `jobvr` = 'V'.

    sense : str, length 1
        Determines which reciprocal condition numbers are computed.

        `sense` = 'N'

            None are computed.

        `sense` = 'E'

            Computed for eigenvalues only.

        `sense` = 'V'

            Computed for right eigenvectors only.

        `sense` = 'B'

            Computed for eigenvalues and right eigenvectors.

        If `sense` = 'E' or 'B', both left and right eigenvectors must
        also be computed (`jobvl` = 'V' and `jobvr` = 'V').

    a : float, array-like, shape (n, n)
        The n*n matrix A.

    Returns
    -------
    a : float, ndarray, shape (n, n)
        `a` has been overwritten. If `jobvl` = 'V' or `jobvr` = 'V', A
        contains the real Schur form of the balanced version of the
        input matrix A.

    wr : float, ndarray, shape (n)
        `wr` and `wi` contain the real and imaginary parts,
        respectively, of the computed eigenvalues. Complex conjugate
        pairs of eigenvalues appear consecutively with the eigenvalue
        having the positive imaginary part first.

    wi : float, ndarray, shape (n)
        `wr` and `wi` contain the real and imaginary parts,
        respectively, of the computed eigenvalues. Complex conjugate
        pairs of eigenvalues appear consecutively with the eigenvalue
        having the positive imaginary part first.

    vl : float, ndarray, shape (:, :)
        If `jobvl` = 'V', the left eigenvectors u_j are stored one after
        another in the columns of `vl`, in the same order as their
        corresponding eigenvalues. If the jth eigenvalue is real, then
        u_j = `vl`[i-1,j-1], for i = 1,2,...,n. If the jth and (j+1)st
        eigenvalues form a complex conjugate pair, then u_j =
        `vl`[i-1,j-1]+i*`vl`[i-1,j] and u_{j+1} =
        `vl`[i-1,j-1]-i*`vl`[i-1,j], for i = 1,2,...,n.

        If `jobvl` = 'N', `vl` is not referenced.

    vr : float, ndarray, shape (:, :)
        If `jobvr` = 'V', the right eigenvectors v_j are stored one
        after another in the columns of `vr`, in the same order as their
        corresponding eigenvalues. If the jth eigenvalue is real, then
        v_j = `vr`[i-1,j-1], for i = 1,2,...,n. If the jth and (j+1)st
        eigenvalues form a complex conjugate pair, then v_j =
        `vr`[i-1,j-1]+i*`vr`[i-1,j] and v_{j+1} =
        `vr`[i-1,j-1]-i*`vr`[i-1,j], for i = 1,2,...,n.

        If `jobvr` = 'N', `vr` is not referenced.

    ilo : int
        `ilo` and `ihi` are integer values determined when A was
        balanced. The balanced A has a_{ij} = 0 if i > j and j =
        1,2,...,`ilo`-1 or i = `ihi`+1,...,n.

    ihi : int
        `ilo` and `ihi` are integer values determined when A was
        balanced. The balanced A has a_{ij} = 0 if i > j and j =
        1,2,...,`ilo`-1 or i = `ihi`+1,...,n.

    scale : float, ndarray, shape (n)
        Details of the permutations and scaling factors applied when
        balancing A.

        If p_j is the index of the row and column interchanged with row
        and column j, and d_j is the scaling factor applied to row and
        column j, then

            `scale`[j-1] = p_j, for j = 1,2,...,`ilo`-1;

            `scale`[j-1] = d_j, for j = `ilo`,...,`ihi`;

            `scale`[j-1] = p_j, for j = `ihi`+1,...,n.

        The order in which the interchanges are made is n to `ihi`+1,
        then 1 to `ilo`-1.

    abnrm : float
        The 1-norm of the balanced matrix (the maximum of the sum of
        absolute values of elements of any column).

    rconde : float, ndarray, shape (n)
        `rconde`[j-1] is the reciprocal condition number of the jth
        eigenvalue.

    rcondv : float, ndarray, shape (n)
        `rcondv`[j-1] is the reciprocal condition number of the jth
        right eigenvector.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `balanc`.

            Constraint: `balanc` = 'N', 'P', 'S' or 'B'.

        (`errno` -2)
            On entry, error in parameter `jobvl`.

            Constraint: `jobvl` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `jobvr`.

            Constraint: `jobvr` = 'N' or 'V'.

        (`errno` -4)
            On entry, error in parameter `sense`.

            Constraint: `sense` = 'N', 'E', 'V' or 'B'.

        (`errno` -5)
            On entry, error in parameter n.

            Constraint: n >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            The QR algorithm failed to compute all the eigenvalues, and
            no eigenvectors or condition numbers have been computed;
            elements 1 to `ilo`-1 and *<value>* to n of `wr` and `wi`
            contain eigenvalues which have converged.

    Notes
    -----
    The right eigenvector v_j of A satisfies

        Av_j = lambda_j v_j

    where lambda_j is the jth eigenvalue of A.
    The left eigenvector u_j of A satisfies

        u_j^HA = lambda_j u_j^H

    where u_j^H denotes the conjugate transpose of u_j.

    Balancing a matrix means permuting the rows and columns to make it
    more nearly upper triangular, and applying a diagonal similarity
    transformation DAD^-1, where D is a diagonal matrix, with the aim of
    making its rows and columns closer in norm and the condition numbers
    of its eigenvalues and eigenvectors smaller.
    The computed reciprocal condition numbers correspond to the balanced
    matrix.
    Permuting rows and columns will not change the condition numbers (in
    exact arithmetic) but diagonal scaling will.
    For further explanation of balancing, see Section 4.8.1.2 of
    Anderson `et al.` (1999).

    Following the optional balancing, the matrix A is first reduced to
    upper Hessenberg form by means of unitary similarity
    transformations, and the QR algorithm is then used to further reduce
    the matrix to upper triangular Schur form, T, from which the
    eigenvalues are computed.
    Optionally, the eigenvectors of T are also computed and
    backtransformed to those of A.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dgeevx'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'),),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _wr = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _wi = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobvl = _b_utils._EngineCharScalarType(
        jobvl, {'fun_name': fun_name, 'entity': 'jobvl'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvl_py = _b_utils._EngineCharScalarType.to_py(_jobvl)
    if _jobvl_py in ('V', 'v'):
        _vl_dim1 = _n_py
    else:
        _vl_dim1 = 1
    if _jobvl_py in ('V', 'v'):
        _vl_dim2 = _n_py
    else:
        _vl_dim2 = 1
    exp_shape_vl = (_vl_dim1, _vl_dim2)
    _vl = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_vl,
        sorder=_m_sorder,
    )
    _jobvr = _b_utils._EngineCharScalarType(
        jobvr, {'fun_name': fun_name, 'entity': 'jobvr'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvr_py = _b_utils._EngineCharScalarType.to_py(_jobvr)
    if _jobvr_py in ('V', 'v'):
        _vr_dim1 = _n_py
    else:
        _vr_dim1 = 1
    if _jobvr_py in ('V', 'v'):
        _vr_dim2 = _n_py
    else:
        _vr_dim2 = 1
    exp_shape_vr = (_vr_dim1, _vr_dim2)
    _vr = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_vr,
        sorder=_m_sorder,
    )
    _scale = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _rconde = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _rcondv = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        _return_scalars = _b_lapackeig.dgeevx(
            balanc,
            jobvl,
            jobvr,
            sense,
            _n,
            _a,
            _wr,
            _wi,
            _vl,
            _vr,
            _scale,
            _rconde,
            _rcondv,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _a_py = _a
    _wr_py = _wr
    _wi_py = _wi
    _vl_py = _vl
    _vr_py = _vr
    _ilo_py = _return_scalars.ilo
    _ihi_py = _return_scalars.ihi
    _scale_py = _scale
    _abnrm_py = _return_scalars.abnrm
    _rconde_py = _rconde
    _rcondv_py = _rcondv
    _return_data = DgeevxReturnData(
        _a_py,
        _wr_py,
        _wi_py,
        _vl_py,
        _vr_py,
        _ilo_py,
        _ihi_py,
        _scale_py,
        _abnrm_py,
        _rconde_py,
        _rcondv_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

DgehrdReturnData = _collections.namedtuple(
    'DgehrdReturnData',
    ['a', 'tau'],
)

def dgehrd(ilo, ihi, a):
    u"""
    Performs an orthogonal reduction of real general matrix to upper
    Hessenberg form.

    ``dgehrd`` reduces a real general matrix to Hessenberg form.

    For full information please refer to the NAG Library document for
    f08ne

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08nef.html

    Parameters
    ----------
    ilo : int
        If A has been output by ``dgebal``, `ilo` and `ihi` **must**
        contain the values returned by that function. Otherwise, `ilo`
        must be set to 1 and `ihi` to n.

    ihi : int
        If A has been output by ``dgebal``, `ilo` and `ihi` **must**
        contain the values returned by that function. Otherwise, `ilo`
        must be set to 1 and `ihi` to n.

    a : float, array-like, shape (n, n)
        The n*n general matrix A.

    Returns
    -------
    a : float, ndarray, shape (n, n)
        `a` is overwritten by the upper Hessenberg matrix H and details
        of the orthogonal matrix Q.

    tau : float, ndarray, shape (max(1, n-1))
        Further details of the orthogonal matrix Q.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -2)
            On entry, error in parameter `ilo`.

        (`errno` -3)
            On entry, error in parameter `ihi`.

    Notes
    -----
    ``dgehrd`` reduces a real general matrix A to upper Hessenberg form
    H by an orthogonal similarity transformation: A = QHQ^T.

    The matrix Q is not formed explicitly, but is represented as a
    product of elementary reflectors (see [the F08 Introduction] for
    details). Functions are provided to work with Q in this
    representation (see [Further Comments]).

    The function can take advantage of a previous call to ``dgebal``,
    which may produce a matrix with the structure:

        [table omitted]

    where A_11 and A_33 are upper triangular.
    If so, only the central diagonal block A_22, in rows and columns
    i_lo to i_hi, needs to be reduced to Hessenberg form (the blocks
    A_12 and A_23 will also be affected by the reduction).
    Therefore, the values of i_lo and i_hi determined by ``dgebal`` can
    be supplied to the function directly.
    If ``dgebal`` has not previously been called however, then i_lo must
    be set to 1 and i_hi to n.

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dgehrd'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _tau = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (max(1, _n_py-1),),
    )
    _b_lapackeig.dgehrd(
        _n,
        ilo,
        ihi,
        _a,
        _tau,
    )
    _a_py = _a
    _tau_py = _tau
    _return_data = DgehrdReturnData(
        _a_py,
        _tau_py,
    )
    return _return_data

def dorghr(ilo, ihi, a, tau):
    u"""
    Generates an orthogonal transformation matrix from reduction to
    Hessenberg form determined by ``dgehrd``.

    ``dorghr`` generates the real orthogonal matrix Q which was
    determined by ``dgehrd`` when reducing a real general matrix A to
    Hessenberg form.

    For full information please refer to the NAG Library document for
    f08nf

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08nff.html

    Parameters
    ----------
    ilo : int
        These **must** be the same arguments `ilo` and `ihi`,
        respectively, as supplied to ``dgehrd``.

    ihi : int
        These **must** be the same arguments `ilo` and `ihi`,
        respectively, as supplied to ``dgehrd``.

    a : float, array-like, shape (n, n)
        Details of the vectors which define the elementary reflectors,
        as returned by ``dgehrd``.

    tau : float, array-like, shape (max(1, n-1))
        Further details of the elementary reflectors, as returned by
        ``dgehrd``.

    Returns
    -------
    a : float, ndarray, shape (n, n)
        The n*n orthogonal matrix Q.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -2)
            On entry, error in parameter `ilo`.

        (`errno` -3)
            On entry, error in parameter `ihi`.

    Notes
    -----
    ``dorghr`` is intended to be used following a call to ``dgehrd``,
    which reduces a real general matrix A to upper Hessenberg form H by
    an orthogonal similarity transformation: A = QHQ^T. ``dgehrd``
    represents the matrix Q as a product of i_hi-i_lo elementary
    reflectors.
    Here i_lo and i_hi are values determined by ``dgebal`` when
    balancing the matrix; if the matrix has not been balanced, i_lo = 1
    and i_hi = n.

    This function may be used to generate Q explicitly as a square
    matrix. Q has the structure:

        Q = [table omitted]

    where Q_22 occupies rows and columns i_lo to i_hi.

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dorghr'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _tau = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'tau'},
        tau,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b_lapackeig.dorghr(
        _n,
        ilo,
        ihi,
        _a,
        _tau,
    )
    _a_py = _a
    return _a_py

def dormhr(side, trans, ilo, ihi, a, tau, c):
    u"""
    Applies the orthogonal transformation matrix from reduction to
    Hessenberg form determined by ``dgehrd``.

    ``dormhr`` multiplies an arbitrary real matrix C by the real
    orthogonal matrix Q which was determined by ``dgehrd`` when reducing
    a real general matrix to Hessenberg form.

    For full information please refer to the NAG Library document for
    f08ng

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08ngf.html

    Parameters
    ----------
    side : str, length 1
        Indicates how Q or Q^T is to be applied to C.

        `side` = 'L'

            Q or Q^T is applied to C from the left.

        `side` = 'R'

            Q or Q^T is applied to C from the right.

    trans : str, length 1
        Indicates whether Q or Q^T is to be applied to C.

        `trans` = 'N'

            Q is applied to C.

        `trans` = 'T'

            Q^T is applied to C.

    ilo : int
        These **must** be the same arguments `ilo` and `ihi`,
        respectively, as supplied to ``dgehrd``.

    ihi : int
        These **must** be the same arguments `ilo` and `ihi`,
        respectively, as supplied to ``dgehrd``.

    a : float, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `side`='L': m; if `side`='R': n;
        otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `side`='L': m; if `side`='R': n;
        otherwise: 0.

        Details of the vectors which define the elementary reflectors,
        as returned by ``dgehrd``.

    tau : float, array-like, shape (:)
        Note: the required length for this argument is determined as
        follows: if `side`='L': m-1; if `side`='R': n-1; otherwise: 0.

        Further details of the elementary reflectors, as returned by
        ``dgehrd``.

    c : float, array-like, shape (m, n)
        The m*n matrix C.

    Returns
    -------
    c : float, ndarray, shape (m, n)
        `c` is overwritten by QC or Q^TC or CQ or CQ^T as specified by
        `side` and `trans`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'T'.

        (`errno` -3)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -5)
            On entry, error in parameter `ilo`.

        (`errno` -6)
            On entry, error in parameter `ihi`.

    Notes
    -----
    ``dormhr`` is intended to be used following a call to ``dgehrd``,
    which reduces a real general matrix A to upper Hessenberg form H by
    an orthogonal similarity transformation: A = QHQ^T. ``dgehrd``
    represents the matrix Q as a product of i_hi-i_lo elementary
    reflectors.
    Here i_lo and i_hi are values determined by ``dgebal`` when
    balancing the matrix; if the matrix has not been balanced, i_lo = 1
    and i_hi = n.

    This function may be used to form one of the matrix products

        QC,Q^TC,CQ or CQ^T,

    overwriting the result on C (which may be any real rectangular
    matrix).

    A common application of this function is to transform a matrix V of
    eigenvectors of H to the matrix QV of eigenvectors of A.

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dormhr'

    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _tau = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'tau'},
        tau,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b_lapackeig.dormhr(
        side,
        trans,
        _m,
        _n,
        ilo,
        ihi,
        _a,
        _tau,
        _c,
    )
    _c_py = _c
    return _c_py

DgebalReturnData = _collections.namedtuple(
    'DgebalReturnData',
    ['a', 'ilo', 'ihi', 'scale'],
)

def dgebal(job, n, a):
    u"""
    Balances a real general matrix.

    ``dgebal`` balances a real general matrix in order to improve the
    accuracy of computed eigenvalues and/or eigenvectors.

    For full information please refer to the NAG Library document for
    f08nh

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08nhf.html

    Parameters
    ----------
    job : str, length 1
        Indicates whether A is to be permuted and/or scaled (or
        neither).

        `job` = 'N'

            A is neither permuted nor scaled (but values are assigned to
            `ilo`, `ihi` and `scale`).

        `job` = 'P'

            A is permuted but not scaled.

        `job` = 'S'

            A is scaled but not permuted.

        `job` = 'B'

            A is both permuted and scaled.

    n : int
        n, the order of the matrix A.

    a : float, array-like, shape (`n`, `n`)
        The n*n matrix A.

    Returns
    -------
    a : float, ndarray, shape (`n`, `n`)
        `a` is overwritten by the balanced matrix. If `job` = 'N', `a`
        is not referenced.

    ilo : int
        The values i_lo and i_hi such that on exit `a`[i-1,j-1] is zero
        if i > j and 1 <= j < i_lo or i_hi < i <= n.

        If `job` = 'N' or 'S', i_lo = 1 and i_hi = n.

    ihi : int
        The values i_lo and i_hi such that on exit `a`[i-1,j-1] is zero
        if i > j and 1 <= j < i_lo or i_hi < i <= n.

        If `job` = 'N' or 'S', i_lo = 1 and i_hi = n.

    scale : float, ndarray, shape (`n`)
        Details of the permutations and scaling factors applied to A.
        More precisely, if p_j is the index of the row and column
        interchanged with row and column j and d_j is the scaling factor
        used to balance row and column j then

            `scale`[j-1] = {[table omitted]

        The order in which the interchanges are made is n to i_hi+1 then
        1 to i_lo-1.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'N', 'P', 'S' or 'B'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Notes
    -----
    ``dgebal`` balances a real general matrix A.
    The term 'balancing' covers two steps, each of which involves a
    similarity transformation of A.
    The function can perform either or both of these steps.

    (1) The function first attempts to permute A to block upper
        triangular form by a similarity transformation:

            PAP^T = A' = [table omitted]

        where P is a permutation matrix, and A'_11 and A'_33 are upper
        triangular. Then the diagonal elements of A'_11 and A'_33 are
        eigenvalues of A. The rest of the eigenvalues of A are the
        eigenvalues of the central diagonal block A'_22, in rows and
        columns i_lo to i_hi. Subsequent operations to compute the
        eigenvalues of A (or its Schur factorization) need only be
        applied to these rows and columns; this can save a significant
        amount of work if i_lo > 1 and i_hi < n. If no suitable
        permutation exists (as is often the case), the function sets
        i_lo = 1 and i_hi = n, and A'_22 is the whole of A.

    (#) The function applies a diagonal similarity transformation to A',
        to make the rows and columns of A'_22 as close in norm as
        possible:

            A'' = DA'D^-1 = [table omitted][table omitted][table
            omitted].

        This scaling can reduce the norm of the matrix (i.e.,
        ||A_22^''|| < ||A'_22||) and hence reduce the effect of rounding
        errors on the accuracy of computed eigenvalues and eigenvectors.

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dgebal'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _scale = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _return_scalars = _b_lapackeig.dgebal(
        job,
        _n,
        _a,
        _scale,
    )
    _a_py = _a
    _ilo_py = _return_scalars.ilo
    _ihi_py = _return_scalars.ihi
    _scale_py = _scale
    _return_data = DgebalReturnData(
        _a_py,
        _ilo_py,
        _ihi_py,
        _scale_py,
    )
    return _return_data

def dgebak(job, side, ilo, ihi, scale, v):
    u"""
    Transforms eigenvectors of real balanced matrix to those of original
    matrix supplied to ``dgebal``.

    ``dgebak`` transforms eigenvectors of a balanced matrix to those of
    the original real nonsymmetric matrix.

    For full information please refer to the NAG Library document for
    f08nj

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08njf.html

    Parameters
    ----------
    job : str, length 1
        This **must** be the same argument `job` as supplied to
        ``dgebal``.

    side : str, length 1
        Indicates whether left or right eigenvectors are to be
        transformed.

        `side` = 'L'

            The left eigenvectors are transformed.

        `side` = 'R'

            The right eigenvectors are transformed.

    ilo : int
        The values i_lo and i_hi, as returned by ``dgebal``.

    ihi : int
        The values i_lo and i_hi, as returned by ``dgebal``.

    scale : float, array-like, shape (n)
        Details of the permutations and/or the scaling factors used to
        balance the original real nonsymmetric matrix, as returned by
        ``dgebal``.

    v : float, array-like, shape (n, m)
        The matrix of left or right eigenvectors to be transformed.

    Returns
    -------
    v : float, ndarray, shape (n, m)
        The transformed eigenvectors.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'N', 'P', 'S' or 'B'.

        (`errno` -2)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -4)
            On entry, error in parameter `ilo`.

        (`errno` -5)
            On entry, error in parameter `ihi`.

        (`errno` -7)
            On entry, error in parameter m.

            Constraint: m >= 0.

    Notes
    -----
    ``dgebak`` is intended to be used after a real nonsymmetric matrix A
    has been balanced by ``dgebal``, and eigenvectors of the balanced
    matrix A_22^'' have subsequently been computed.

    For a description of balancing, see the document for ``dgebal``.
    The balanced matrix A'' is obtained as A'' = DPAP^TD^-1, where P is
    a permutation matrix and D is a diagonal scaling matrix.
    This function transforms left or right eigenvectors as follows:

        if x is a right eigenvector of A'', P^TD^-1x is a right
        eigenvector of A;

        if y is a left eigenvector of A'', P^TDy is a left eigenvector
        of A.
    """

    fun_name = 'naginterfaces.library.lapackeig.dgebak'

    _v = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'v'},
        v,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _scale = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'scale'},
        scale,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _b_lapackeig.dgebak(
        job,
        side,
        _n,
        ilo,
        ihi,
        _scale,
        _m,
        _v,
    )
    _v_py = _v
    return _v_py

ZgeevReturnData = _collections.namedtuple(
    'ZgeevReturnData',
    ['a', 'w', 'vl', 'vr'],
)

def zgeev(jobvl, jobvr, a):
    u"""
    Computes all eigenvalues and, optionally, left and/or right
    eigenvectors of a complex nonsymmetric matrix.

    ``zgeev`` computes the eigenvalues and, optionally, the left and/or
    right eigenvectors for an n*n complex nonsymmetric matrix A.

    For full information please refer to the NAG Library document for
    f08nn

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08nnf.html

    Parameters
    ----------
    jobvl : str, length 1
        If `jobvl` = 'N', the left eigenvectors of A are not computed.

        If `jobvl` = 'V', the left eigenvectors of A are computed.

    jobvr : str, length 1
        If `jobvr` = 'N', the right eigenvectors of A are not computed.

        If `jobvr` = 'V', the right eigenvectors of A are computed.

    a : complex, array-like, shape (n, n)
        The n*n matrix A.

    Returns
    -------
    a : complex, ndarray, shape (n, n)
        `a` has been overwritten.

    w : complex, ndarray, shape (n)
        Contains the computed eigenvalues.

    vl : complex, ndarray, shape (:, :)
        If `jobvl` = 'V', the left eigenvectors u_j are stored one after
        another in the columns of `vl`, in the same order as their
        corresponding eigenvalues; that is u_j = `vl`[i-1,j-1], for i =
        1,2,...,n.

        If `jobvl` = 'N', `vl` is not referenced.

    vr : complex, ndarray, shape (:, :)
        If `jobvr` = 'V', the right eigenvectors v_j are stored one
        after another in the columns of `vr`, in the same order as their
        corresponding eigenvalues; that is v_j = `vr`[i-1,j-1], for i =
        1,2,...,n.

        If `jobvr` = 'N', `vr` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobvl`.

            Constraint: `jobvl` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `jobvr`.

            Constraint: `jobvr` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            The QR algorithm failed to compute all the eigenvalues, and
            no eigenvectors have been computed; elements *<value>* to n
            of `w` contain eigenvalues which have converged.

    Notes
    -----
    The right eigenvector v_j of A satisfies

        Av_j = lambda_j v_j

    where lambda_j is the jth eigenvalue of A.
    The left eigenvector u_j of A satisfies

        u_j^HA = lambda_j u_j^H

    where u_j^H denotes the conjugate transpose of u_j.

    The matrix A is first reduced to upper Hessenberg form by means of
    unitary similarity transformations, and the QR algorithm is then
    used to further reduce the matrix to upper triangular Schur form, T,
    from which the eigenvalues are computed.
    Optionally, the eigenvectors of T are also computed and
    backtransformed to those of A.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zgeev'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'),),
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _w = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobvl = _b_utils._EngineCharScalarType(
        jobvl, {'fun_name': fun_name, 'entity': 'jobvl'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvl_py = _b_utils._EngineCharScalarType.to_py(_jobvl)
    if _jobvl_py in ('V', 'v'):
        _vl_dim1 = _n_py
    else:
        _vl_dim1 = 1
    if _jobvl_py in ('V', 'v'):
        _vl_dim2 = _n_py
    else:
        _vl_dim2 = 1
    exp_shape_vl = (_vl_dim1, _vl_dim2)
    _vl = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_vl,
        sorder=_m_sorder,
    )
    _jobvr = _b_utils._EngineCharScalarType(
        jobvr, {'fun_name': fun_name, 'entity': 'jobvr'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvr_py = _b_utils._EngineCharScalarType.to_py(_jobvr)
    if _jobvr_py in ('V', 'v'):
        _vr_dim1 = _n_py
    else:
        _vr_dim1 = 1
    if _jobvr_py in ('V', 'v'):
        _vr_dim2 = _n_py
    else:
        _vr_dim2 = 1
    exp_shape_vr = (_vr_dim1, _vr_dim2)
    _vr = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_vr,
        sorder=_m_sorder,
    )
    try:
        _b_lapackeig.zgeev(
            jobvl,
            jobvr,
            _n,
            _a,
            _w,
            _vl,
            _vr,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _a_py = _a
    _w_py = _w
    _vl_py = _vl
    _vr_py = _vr
    _return_data = ZgeevReturnData(
        _a_py,
        _w_py,
        _vl_py,
        _vr_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

ZgeevxReturnData = _collections.namedtuple(
    'ZgeevxReturnData',
    ['a', 'w', 'vl', 'vr', 'ilo', 'ihi', 'scale', 'abnrm', 'rconde', 'rcondv'],
)

def zgeevx(balanc, jobvl, jobvr, sense, a):
    u"""
    Computes all eigenvalues and, optionally, left and/or right
    eigenvectors of a complex nonsymmetric matrix; also, optionally, the
    balancing transformation, the reciprocal condition numbers for the
    eigenvalues and for the right eigenvectors.

    ``zgeevx`` computes the eigenvalues and, optionally, the left and/or
    right eigenvectors for an n*n complex nonsymmetric matrix A.

    Optionally, it also computes a balancing transformation to improve
    the conditioning of the eigenvalues and eigenvectors, reciprocal
    condition numbers for the eigenvalues, and reciprocal condition
    numbers for the right eigenvectors.

    For full information please refer to the NAG Library document for
    f08np

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08npf.html

    Parameters
    ----------
    balanc : str, length 1
        Indicates how the input matrix should be diagonally scaled
        and/or permuted to improve the conditioning of its eigenvalues.

        `balanc` = 'N'

            Do not diagonally scale or permute.

        `balanc` = 'P'

            Perform permutations to make the matrix more nearly upper
            triangular. Do not diagonally scale.

        `balanc` = 'S'

            Diagonally scale the matrix, i.e., replace A*DAD^-1, where D
            is a diagonal matrix chosen to make the rows and columns of
            A more equal in norm. Do not permute.

        `balanc` = 'B'

            Both diagonally scale and permute A.

        Computed reciprocal condition numbers will be for the matrix
        after balancing and/or permuting.

        Permuting does not change condition numbers (in exact
        arithmetic), but balancing does.

    jobvl : str, length 1
        If `jobvl` = 'N', the left eigenvectors of A are not computed.

        If `jobvl` = 'V', the left eigenvectors of A are computed.

        If `sense` = 'E' or 'B', `jobvl` must be set to `jobvl` = 'V'.

    jobvr : str, length 1
        If `jobvr` = 'N', the right eigenvectors of A are not computed.

        If `jobvr` = 'V', the right eigenvectors of A are computed.

        If `sense` = 'E' or 'B', `jobvr` must be set to `jobvr` = 'V'.

    sense : str, length 1
        Determines which reciprocal condition numbers are computed.

        `sense` = 'N'

            None are computed.

        `sense` = 'E'

            Computed for eigenvalues only.

        `sense` = 'V'

            Computed for right eigenvectors only.

        `sense` = 'B'

            Computed for eigenvalues and right eigenvectors.

        If `sense` = 'E' or 'B', both left and right eigenvectors must
        also be computed (`jobvl` = 'V' and `jobvr` = 'V').

    a : complex, array-like, shape (n, n)
        The n*n matrix A.

    Returns
    -------
    a : complex, ndarray, shape (n, n)
        `a` has been overwritten. If `jobvl` = 'V' or `jobvr` = 'V', A
        contains the Schur form of the balanced version of the matrix A.

    w : complex, ndarray, shape (n)
        Contains the computed eigenvalues.

    vl : complex, ndarray, shape (:, :)
        If `jobvl` = 'V', the left eigenvectors u_j are stored one after
        another in the columns of `vl`, in the same order as their
        corresponding eigenvalues; that is u_j = `vl`[i-1,j-1], for i =
        1,2,...,n.

        If `jobvl` = 'N', `vl` is not referenced.

    vr : complex, ndarray, shape (:, :)
        If `jobvr` = 'V', the right eigenvectors v_j are stored one
        after another in the columns of `vr`, in the same order as their
        corresponding eigenvalues; that is v_j = `vr`[i-1,j-1], for i =
        1,2,...,n.

        If `jobvr` = 'N', `vr` is not referenced.

    ilo : int
        `ilo` and `ihi` are integer values determined when A was
        balanced. The balanced A has a_{ij} = 0 if i > j and j =
        1,2,...,`ilo`-1 or i = `ihi`+1,...,n.

    ihi : int
        `ilo` and `ihi` are integer values determined when A was
        balanced. The balanced A has a_{ij} = 0 if i > j and j =
        1,2,...,`ilo`-1 or i = `ihi`+1,...,n.

    scale : float, ndarray, shape (n)
        Details of the permutations and scaling factors applied when
        balancing A.

        If p_j is the index of the row and column interchanged with row
        and column j, and d_j is the scaling factor applied to row and
        column j, then

            `scale`[j-1] = p_j, for j = 1,2,...,`ilo`-1;

            `scale`[j-1] = d_j, for j = `ilo`,...,`ihi`;

            `scale`[j-1] = p_j, for j = `ihi`+1,...,n.

        The order in which the interchanges are made is n to `ihi`+1,
        then 1 to `ilo`-1.

    abnrm : float
        The 1-norm of the balanced matrix (the maximum of the sum of
        absolute values of elements of any column).

    rconde : float, ndarray, shape (n)
        `rconde`[j-1] is the reciprocal condition number of the jth
        eigenvalue.

    rcondv : float, ndarray, shape (n)
        `rcondv`[j-1] is the reciprocal condition number of the jth
        right eigenvector.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `balanc`.

            Constraint: `balanc` = 'N', 'P', 'S' or 'B'.

        (`errno` -2)
            On entry, error in parameter `jobvl`.

            Constraint: `jobvl` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `jobvr`.

            Constraint: `jobvr` = 'N' or 'V'.

        (`errno` -4)
            On entry, error in parameter `sense`.

            Constraint: `sense` = 'N', 'E', 'V' or 'B'.

        (`errno` -5)
            On entry, error in parameter n.

            Constraint: n >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            The QR algorithm failed to compute all the eigenvalues, and
            no eigenvectors or condition numbers have been computed;
            elements 1 to `ilo`-1 and *<value>* to n of `w` contain
            eigenvalues which have converged.

    Notes
    -----
    The right eigenvector v_j of A satisfies

        Av_j = lambda_j v_j

    where lambda_j is the jth eigenvalue of A.
    The left eigenvector u_j of A satisfies

        u_j^HA = lambda_j u_j^H

    where u_j^H denotes the conjugate transpose of u_j.

    Balancing a matrix means permuting the rows and columns to make it
    more nearly upper triangular, and applying a diagonal similarity
    transformation DAD^-1, where D is a diagonal matrix, with the aim of
    making its rows and columns closer in norm and the condition numbers
    of its eigenvalues and eigenvectors smaller.
    The computed reciprocal condition numbers correspond to the balanced
    matrix.
    Permuting rows and columns will not change the condition numbers (in
    exact arithmetic) but diagonal scaling will.
    For further explanation of balancing, see Section 4.8.1.2 of
    Anderson `et al.` (1999).

    Following the optional balancing, the matrix A is first reduced to
    upper Hessenberg form by means of unitary similarity
    transformations, and the QR algorithm is then used to further reduce
    the matrix to upper triangular Schur form, T, from which the
    eigenvalues are computed.
    Optionally, the eigenvectors of T are also computed and
    backtransformed to those of A.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zgeevx'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'),),
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _w = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobvl = _b_utils._EngineCharScalarType(
        jobvl, {'fun_name': fun_name, 'entity': 'jobvl'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvl_py = _b_utils._EngineCharScalarType.to_py(_jobvl)
    if _jobvl_py in ('V', 'v'):
        _vl_dim1 = _n_py
    else:
        _vl_dim1 = 1
    if _jobvl_py in ('V', 'v'):
        _vl_dim2 = _n_py
    else:
        _vl_dim2 = 1
    exp_shape_vl = (_vl_dim1, _vl_dim2)
    _vl = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_vl,
        sorder=_m_sorder,
    )
    _jobvr = _b_utils._EngineCharScalarType(
        jobvr, {'fun_name': fun_name, 'entity': 'jobvr'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvr_py = _b_utils._EngineCharScalarType.to_py(_jobvr)
    if _jobvr_py in ('V', 'v'):
        _vr_dim1 = _n_py
    else:
        _vr_dim1 = 1
    if _jobvr_py in ('V', 'v'):
        _vr_dim2 = _n_py
    else:
        _vr_dim2 = 1
    exp_shape_vr = (_vr_dim1, _vr_dim2)
    _vr = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_vr,
        sorder=_m_sorder,
    )
    _scale = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _rconde = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _rcondv = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        _return_scalars = _b_lapackeig.zgeevx(
            balanc,
            jobvl,
            jobvr,
            sense,
            _n,
            _a,
            _w,
            _vl,
            _vr,
            _scale,
            _rconde,
            _rcondv,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _a_py = _a
    _w_py = _w
    _vl_py = _vl
    _vr_py = _vr
    _ilo_py = _return_scalars.ilo
    _ihi_py = _return_scalars.ihi
    _scale_py = _scale
    _abnrm_py = _return_scalars.abnrm
    _rconde_py = _rconde
    _rcondv_py = _rcondv
    _return_data = ZgeevxReturnData(
        _a_py,
        _w_py,
        _vl_py,
        _vr_py,
        _ilo_py,
        _ihi_py,
        _scale_py,
        _abnrm_py,
        _rconde_py,
        _rcondv_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

ZgehrdReturnData = _collections.namedtuple(
    'ZgehrdReturnData',
    ['a', 'tau'],
)

def zgehrd(ilo, ihi, a):
    u"""
    Performs a unitary reduction of complex general matrix to upper
    Hessenberg form.

    ``zgehrd`` reduces a complex general matrix to Hessenberg form.

    For full information please refer to the NAG Library document for
    f08ns

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08nsf.html

    Parameters
    ----------
    ilo : int
        If A has been output by ``zgebal``, `ilo` and `ihi` **must**
        contain the values returned by that function. Otherwise, `ilo`
        must be set to 1 and `ihi` to n.

    ihi : int
        If A has been output by ``zgebal``, `ilo` and `ihi` **must**
        contain the values returned by that function. Otherwise, `ilo`
        must be set to 1 and `ihi` to n.

    a : complex, array-like, shape (n, n)
        The n*n general matrix A.

    Returns
    -------
    a : complex, ndarray, shape (n, n)
        `a` is overwritten by the upper Hessenberg matrix H and details
        of the unitary matrix Q. The subdiagonal elements of H are real.

    tau : complex, ndarray, shape (max(1, n-1))
        Further details of the unitary matrix Q.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -2)
            On entry, error in parameter `ilo`.

        (`errno` -3)
            On entry, error in parameter `ihi`.

    Notes
    -----
    ``zgehrd`` reduces a complex general matrix A to upper Hessenberg
    form H by a unitary similarity transformation: A = QHQ^H. H has real
    subdiagonal elements.

    The matrix Q is not formed explicitly, but is represented as a
    product of elementary reflectors (see [the F08 Introduction] for
    details). Functions are provided to work with Q in this
    representation (see [Further Comments]).

    The function can take advantage of a previous call to ``zgebal``,
    which may produce a matrix with the structure:

        [table omitted]

    where A_11 and A_33 are upper triangular.
    If so, only the central diagonal block A_22, in rows and columns
    i_lo to i_hi, needs to be reduced to Hessenberg form (the blocks
    A_12 and A_23 will also be affected by the reduction).
    Therefore, the values of i_lo and i_hi determined by ``zgebal`` can
    be supplied to the function directly.
    If ``zgebal`` has not previously been called however, then i_lo must
    be set to 1 and i_hi to n.

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zgehrd'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _tau = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (max(1, _n_py-1),),
    )
    _b_lapackeig.zgehrd(
        _n,
        ilo,
        ihi,
        _a,
        _tau,
    )
    _a_py = _a
    _tau_py = _tau
    _return_data = ZgehrdReturnData(
        _a_py,
        _tau_py,
    )
    return _return_data

def zunghr(ilo, ihi, a, tau):
    u"""
    Generates a unitary transformation matrix from reduction to
    Hessenberg form determined by ``zgehrd``.

    ``zunghr`` generates the complex unitary matrix Q which was
    determined by ``zgehrd`` when reducing a complex general matrix A to
    Hessenberg form.

    For full information please refer to the NAG Library document for
    f08nt

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08ntf.html

    Parameters
    ----------
    ilo : int
        These **must** be the same arguments `ilo` and `ihi`,
        respectively, as supplied to ``zgehrd``.

    ihi : int
        These **must** be the same arguments `ilo` and `ihi`,
        respectively, as supplied to ``zgehrd``.

    a : complex, array-like, shape (n, n)
        Details of the vectors which define the elementary reflectors,
        as returned by ``zgehrd``.

    tau : complex, array-like, shape (max(1, n-1))
        Further details of the elementary reflectors, as returned by
        ``zgehrd``.

    Returns
    -------
    a : complex, ndarray, shape (n, n)
        The n*n unitary matrix Q.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -2)
            On entry, error in parameter `ilo`.

        (`errno` -3)
            On entry, error in parameter `ihi`.

    Notes
    -----
    ``zunghr`` is intended to be used following a call to ``zgehrd``,
    which reduces a complex general matrix A to upper Hessenberg form H
    by a unitary similarity transformation: A = QHQ^H. ``zgehrd``
    represents the matrix Q as a product of i_hi-i_lo elementary
    reflectors.
    Here i_lo and i_hi are values determined by ``zgebal`` when
    balancing the matrix; if the matrix has not been balanced, i_lo = 1
    and i_hi = n.

    This function may be used to generate Q explicitly as a square
    matrix. Q has the structure:

        Q = [table omitted]

    where Q_22 occupies rows and columns i_lo to i_hi.

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zunghr'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _tau = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'tau'},
        tau,
        copy=False,
    )
    _b_lapackeig.zunghr(
        _n,
        ilo,
        ihi,
        _a,
        _tau,
    )
    _a_py = _a
    return _a_py

def zunmhr(side, trans, ilo, ihi, a, tau, c):
    u"""
    Applies the unitary transformation matrix from reduction to
    Hessenberg form determined by ``zgehrd``.

    ``zunmhr`` multiplies an arbitrary complex matrix C by the complex
    unitary matrix Q which was determined by ``zgehrd`` when reducing a
    complex general matrix to Hessenberg form.

    For full information please refer to the NAG Library document for
    f08nu

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08nuf.html

    Parameters
    ----------
    side : str, length 1
        Indicates how Q or Q^H is to be applied to C.

        `side` = 'L'

            Q or Q^H is applied to C from the left.

        `side` = 'R'

            Q or Q^H is applied to C from the right.

    trans : str, length 1
        Indicates whether Q or Q^H is to be applied to C.

        `trans` = 'N'

            Q is applied to C.

        `trans` = 'C'

            Q^H is applied to C.

    ilo : int
        These **must** be the same arguments `ilo` and `ihi`,
        respectively, as supplied to ``zgehrd``.

    ihi : int
        These **must** be the same arguments `ilo` and `ihi`,
        respectively, as supplied to ``zgehrd``.

    a : complex, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `side`='L': m; if `side`='R': n;
        otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `side`='L': m; if `side`='R': n;
        otherwise: 0.

        Details of the vectors which define the elementary reflectors,
        as returned by ``zgehrd``.

    tau : complex, array-like, shape (:)
        Note: the required length for this argument is determined as
        follows: if `side`='L': m-1; if `side`='R': n-1; otherwise: 0.

        Further details of the elementary reflectors, as returned by
        ``zgehrd``.

    c : complex, array-like, shape (m, n)
        The m*n matrix C.

    Returns
    -------
    c : complex, ndarray, shape (m, n)
        `c` is overwritten by QC or Q^HC or CQ or CQ^H as specified by
        `side` and `trans`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'C'.

        (`errno` -3)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -5)
            On entry, error in parameter `ilo`.

        (`errno` -6)
            On entry, error in parameter `ihi`.

    Notes
    -----
    ``zunmhr`` is intended to be used following a call to ``zgehrd``,
    which reduces a complex general matrix A to upper Hessenberg form H
    by a unitary similarity transformation: A = QHQ^H. ``zgehrd``
    represents the matrix Q as a product of i_hi-i_lo elementary
    reflectors.
    Here i_lo and i_hi are values determined by ``zgebal`` when
    balancing the matrix; if the matrix has not been balanced, i_lo = 1
    and i_hi = n.

    This function may be used to form one of the matrix products

        QC,Q^HC,CQ or CQ^H,

    overwriting the result on C (which may be any complex rectangular
    matrix).

    A common application of this function is to transform a matrix V of
    eigenvectors of H to the matrix QV of eigenvectors of A.

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zunmhr'

    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=True,
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _tau = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'tau'},
        tau,
        copy=False,
    )
    _b_lapackeig.zunmhr(
        side,
        trans,
        _m,
        _n,
        ilo,
        ihi,
        _a,
        _tau,
        _c,
    )
    _c_py = _c
    return _c_py

ZgebalReturnData = _collections.namedtuple(
    'ZgebalReturnData',
    ['a', 'ilo', 'ihi', 'scale'],
)

def zgebal(job, n, a):
    u"""
    Balances a complex general matrix.

    ``zgebal`` balances a complex general matrix in order to improve the
    accuracy of computed eigenvalues and/or eigenvectors.

    For full information please refer to the NAG Library document for
    f08nv

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08nvf.html

    Parameters
    ----------
    job : str, length 1
        Indicates whether A is to be permuted and/or scaled (or
        neither).

        `job` = 'N'

            A is neither permuted nor scaled (but values are assigned to
            `ilo`, `ihi` and `scale`).

        `job` = 'P'

            A is permuted but not scaled.

        `job` = 'S'

            A is scaled but not permuted.

        `job` = 'B'

            A is both permuted and scaled.

    n : int
        n, the order of the matrix A.

    a : complex, array-like, shape (`n`, `n`)
        The n*n matrix A.

    Returns
    -------
    a : complex, ndarray, shape (`n`, `n`)
        `a` is overwritten by the balanced matrix. If `job` = 'N', `a`
        is not referenced.

    ilo : int
        The values i_lo and i_hi such that on exit `a`[i-1,j-1] is zero
        if i > j and 1 <= j < i_lo or i_hi < i <= n.

        If `job` = 'N' or 'S', i_lo = 1 and i_hi = n.

    ihi : int
        The values i_lo and i_hi such that on exit `a`[i-1,j-1] is zero
        if i > j and 1 <= j < i_lo or i_hi < i <= n.

        If `job` = 'N' or 'S', i_lo = 1 and i_hi = n.

    scale : float, ndarray, shape (`n`)
        Details of the permutations and scaling factors applied to A.
        More precisely, if p_j is the index of the row and column
        interchanged with row and column j and d_j is the scaling factor
        used to balance row and column j then

            `scale`[j-1] = {[table omitted]

        The order in which the interchanges are made is n to i_hi+1 then
        1 to i_lo-1.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'N', 'P', 'S' or 'B'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Notes
    -----
    ``zgebal`` balances a complex general matrix A.
    The term 'balancing' covers two steps, each of which involves a
    similarity transformation of A.
    The function can perform either or both of these steps.

    (1) The function first attempts to permute A to block upper
        triangular form by a similarity transformation:

            PAP^T = A' = [table omitted]

        where P is a permutation matrix, and A'_11 and A'_33 are upper
        triangular. Then the diagonal elements of A'_11 and A'_33 are
        eigenvalues of A. The rest of the eigenvalues of A are the
        eigenvalues of the central diagonal block A'_22, in rows and
        columns i_lo to i_hi. Subsequent operations to compute the
        eigenvalues of A (or its Schur factorization) need only be
        applied to these rows and columns; this can save a significant
        amount of work if i_lo > 1 and i_hi < n. If no suitable
        permutation exists (as is often the case), the function sets
        i_lo = 1 and i_hi = n, and A'_22 is the whole of A.

    (#) The function applies a diagonal similarity transformation to A',
        to make the rows and columns of A'_22 as close in norm as
        possible:

            A'' = DA'D^-1 = [table omitted][table omitted][table
            omitted].

        This scaling can reduce the norm of the matrix (i.e.,
        ||A_22^''|| < ||A'_22||) and hence reduce the effect of rounding
        errors on the accuracy of computed eigenvalues and eigenvectors.

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zgebal'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _scale = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _return_scalars = _b_lapackeig.zgebal(
        job,
        _n,
        _a,
        _scale,
    )
    _a_py = _a
    _ilo_py = _return_scalars.ilo
    _ihi_py = _return_scalars.ihi
    _scale_py = _scale
    _return_data = ZgebalReturnData(
        _a_py,
        _ilo_py,
        _ihi_py,
        _scale_py,
    )
    return _return_data

def zgebak(job, side, ilo, ihi, scale, v):
    u"""
    Transforms eigenvectors of complex balanced matrix to those of
    original matrix supplied to ``zgebal``.

    ``zgebak`` transforms eigenvectors of a balanced matrix to those of
    the original complex general matrix.

    For full information please refer to the NAG Library document for
    f08nw

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08nwf.html

    Parameters
    ----------
    job : str, length 1
        This **must** be the same argument `job` as supplied to
        ``zgebal``.

    side : str, length 1
        Indicates whether left or right eigenvectors are to be
        transformed.

        `side` = 'L'

            The left eigenvectors are transformed.

        `side` = 'R'

            The right eigenvectors are transformed.

    ilo : int
        The values i_lo and i_hi, as returned by ``zgebal``.

    ihi : int
        The values i_lo and i_hi, as returned by ``zgebal``.

    scale : float, array-like, shape (n)
        Details of the permutations and/or the scaling factors used to
        balance the original complex general matrix, as returned by
        ``zgebal``.

    v : complex, array-like, shape (n, m)
        The matrix of left or right eigenvectors to be transformed.

    Returns
    -------
    v : complex, ndarray, shape (n, m)
        The transformed eigenvectors.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'N', 'P', 'S' or 'B'.

        (`errno` -2)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -4)
            On entry, error in parameter `ilo`.

        (`errno` -5)
            On entry, error in parameter `ihi`.

        (`errno` -7)
            On entry, error in parameter m.

            Constraint: m >= 0.

    Notes
    -----
    ``zgebak`` is intended to be used after a complex general matrix A
    has been balanced by ``zgebal``, and eigenvectors of the balanced
    matrix A_22^'' have subsequently been computed.

    For a description of balancing, see the document for ``zgebal``.
    The balanced matrix A'' is obtained as A'' = DPAP^TD^-1, where P is
    a permutation matrix and D is a diagonal scaling matrix.
    This function transforms left or right eigenvectors as follows:

        if x is a right eigenvector of A'', P^TD^-1x is a right
        eigenvector of A;

        if y is a left eigenvector of A'', P^TDy is a left eigenvector
        of A.
    """

    fun_name = 'naginterfaces.library.lapackeig.zgebak'

    _v = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'v'},
        v,
        copy=True,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _scale = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'scale'},
        scale,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _b_lapackeig.zgebak(
        job,
        side,
        _n,
        ilo,
        ihi,
        _scale,
        _m,
        _v,
    )
    _v_py = _v
    return _v_py

DgeesReturnData = _collections.namedtuple(
    'DgeesReturnData',
    ['a', 'sdim', 'wr', 'wi', 'vs'],
)

def dgees(jobvs, sort, a, select=None, data=None):
    u"""
    Computes for real square nonsymmetric matrix, the eigenvalues, the
    real Schur form, and, optionally, the matrix of Schur vectors.

    ``dgees`` computes the eigenvalues, the real Schur form T, and,
    optionally, the matrix of Schur vectors Z for an n*n real
    nonsymmetric matrix A.

    For full information please refer to the NAG Library document for
    f08pa

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08paf.html

    Parameters
    ----------
    jobvs : str, length 1
        If `jobvs` = 'N', Schur vectors are not computed.

        If `jobvs` = 'V', Schur vectors are computed.

    sort : str, length 1
        Specifies whether or not to order the eigenvalues on the
        diagonal of the Schur form.

        `sort` = 'N'

            Eigenvalues are not ordered.

        `sort` = 'S'

            Eigenvalues are ordered (see `select`).

    a : float, array-like, shape (n, n)
        The n*n matrix A.

    select : None or callable retval = select(wr, wi, data=None),
        optional
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        If `sort` = 'S', `select` is used to select eigenvalues to sort
        to the top left of the Schur form.

        An eigenvalue `wr`[j-1]+sqrt(-1)*`wi`[j-1] is selected if
        `select`(`wr`[j-1],`wi`[j-1]) is True.

        If either one of a complex conjugate pair of eigenvalues is
        selected, then both are.

        Note that a selected complex eigenvalue may no longer satisfy
        `select`(`wr`[j-1],`wi`[j-1]) = True after ordering, since
        ordering may change the value of complex eigenvalues (especially
        if the eigenvalue is ill-conditioned); in this case errno is set
        to n+2.

        Parameters
        ~~~~~~~~~~
        wr : float
            The real and imaginary parts of the eigenvalue.

        wi : float
            The real and imaginary parts of the eigenvalue.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : bool
            Must be True if the eigenvalue is to be selected.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    a : float, ndarray, shape (n, n)
        `a` is overwritten by its real Schur form T.

    sdim : int
        If `sort` = 'N', `sdim` = 0.

        If `sort` = 'S', `sdim` = number of eigenvalues (after sorting)
        for which `select` is True. (Complex conjugate pairs for which
        `select` is True for either eigenvalue count as 2.)

    wr : float, ndarray, shape (n)
        See the description of `wi`.

    wi : float, ndarray, shape (n)
        `wr` and `wi` contain the real and imaginary parts,
        respectively, of the computed eigenvalues in the same order that
        they appear on the diagonal of the output Schur form T. Complex
        conjugate pairs of eigenvalues will appear consecutively with
        the eigenvalue having the positive imaginary part first.

    vs : float, ndarray, shape (:, :)
        If `jobvs` = 'V', `vs` contains the orthogonal matrix Z of Schur
        vectors.

        If `jobvs` = 'N', `vs` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobvs`.

            Constraint: `jobvs` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `sort`.

            Constraint: `sort` = 'N' or 'S'.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` (i > 0) and (i<=n))
            The QR algorithm failed to compute all the eigenvalues.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` n+1)
            The eigenvalues could not be reordered because some
            eigenvalues were too close to separate (the problem is very
            ill-conditioned).

        (`errno` n+2)
            After reordering, roundoff changed values of some complex
            eigenvalues so that leading eigenvalues in the Schur form no
            longer satisfy `select` = True. This could also be caused by
            underflow due to scaling.

    Notes
    -----
    The real Schur factorization of A is given by

        A = ZTZ^T,

    where Z, the matrix of Schur vectors, is orthogonal and T is the
    real Schur form.
    A matrix is in real Schur form if it is upper quasi-triangular with
    1*1 and 2*2 blocks. 2*2 blocks will be standardized in the form

        [[table omitted]]

    where bc < 0.
    The eigenvalues of such a block are a+/-sqrt(bc).

    Optionally, ``dgees`` also orders the eigenvalues on the diagonal of
    the real Schur form so that selected eigenvalues are at the top
    left.
    The leading columns of Z form an orthonormal basis for the invariant
    subspace corresponding to the selected eigenvalues.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    def cb_select(wr, wi, data=None):
        """Wrapping user's select. Called as base select."""
        if data is not None:
            _retval_select = select(
                wr,
                wi,
                data,
            )
        else:
            _retval_select = select(
                wr,
                wi,
            )
        return _retval_select

    fun_name = 'naginterfaces.library.lapackeig.dgees'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'),),
    )
    _b_utils._check_callable(select, {'fun_name': fun_name, 'entity': 'select'}, allow_none=True)
    _select = None if select is None else cb_select
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _wr = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _wi = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobvs = _b_utils._EngineCharScalarType(
        jobvs, {'fun_name': fun_name, 'entity': 'jobvs'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvs_py = _b_utils._EngineCharScalarType.to_py(_jobvs)
    if _jobvs_py in ('V', 'v'):
        _vs_dim1 = _n_py
    else:
        _vs_dim1 = 1
    if _jobvs_py in ('V', 'v'):
        _vs_dim2 = _n_py
    else:
        _vs_dim2 = 1
    exp_shape_vs = (_vs_dim1, _vs_dim2)
    _vs = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_vs,
        sorder=_m_sorder,
    )
    try:
        _return_scalar = _b_lapackeig.dgees(
            jobvs,
            sort,
            _select,
            _n,
            _a,
            _wr,
            _wi,
            _vs,
            data,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _a_py = _a
    _sdim_py = _return_scalar
    _wr_py = _wr
    _wi_py = _wi
    _vs_py = _vs
    _return_data = DgeesReturnData(
        _a_py,
        _sdim_py,
        _wr_py,
        _wi_py,
        _vs_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

DgeesxReturnData = _collections.namedtuple(
    'DgeesxReturnData',
    ['a', 'sdim', 'wr', 'wi', 'vs', 'rconde', 'rcondv'],
)

def dgeesx(jobvs, sort, sense, a, select=None, data=None):
    u"""
    Computes for real square nonsymmetric matrix, the eigenvalues, the
    real Schur form, and, optionally, the matrix of Schur vectors; also,
    optionally, computes reciprocal condition numbers for selected
    eigenvalues.

    ``dgeesx`` computes the eigenvalues, the real Schur form T, and,
    optionally, the matrix of Schur vectors Z for an n*n real
    nonsymmetric matrix A.

    For full information please refer to the NAG Library document for
    f08pb

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08pbf.html

    Parameters
    ----------
    jobvs : str, length 1
        If `jobvs` = 'N', Schur vectors are not computed.

        If `jobvs` = 'V', Schur vectors are computed.

    sort : str, length 1
        Specifies whether or not to order the eigenvalues on the
        diagonal of the Schur form.

        `sort` = 'N'

            Eigenvalues are not ordered.

        `sort` = 'S'

            Eigenvalues are ordered (see `select`).

    sense : str, length 1
        Determines which reciprocal condition numbers are computed.

        `sense` = 'N'

            None are computed.

        `sense` = 'E'

            Computed for average of selected eigenvalues only.

        `sense` = 'V'

            Computed for selected right invariant subspace only.

        `sense` = 'B'

            Computed for both.

        If `sense` = 'E', 'V' or 'B', `sort` = 'S'.

    a : float, array-like, shape (n, n)
        The n*n matrix A.

    select : None or callable retval = select(wr, wi, data=None),
        optional
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        If `sort` = 'S', `select` is used to select eigenvalues to sort
        to the top left of the Schur form.

        An eigenvalue `wr`[j-1]+sqrt(-1)*`wi`[j-1] is selected if
        `select`(`wr`[j-1],`wi`[j-1]) is True.

        If either one of a complex conjugate pair of eigenvalues is
        selected, then both are.

        Note that a selected complex eigenvalue may no longer satisfy
        `select`(`wr`[j-1],`wi`[j-1]) = True after ordering, since
        ordering may change the value of complex eigenvalues (especially
        if the eigenvalue is ill-conditioned); in this case errno is set
        to n+2.

        Parameters
        ~~~~~~~~~~
        wr : float
            The real and imaginary parts of the eigenvalue.

        wi : float
            The real and imaginary parts of the eigenvalue.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : bool
            Must be True if the eigenvalue is to be selected.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    a : float, ndarray, shape (n, n)
        `a` is overwritten by its real Schur form T.

    sdim : int
        If `sort` = 'N', `sdim` = 0.

        If `sort` = 'S', `sdim` = number of eigenvalues (after sorting)
        for which `select` is True. (Complex conjugate pairs for which
        `select` is True for either eigenvalue count as 2.)

    wr : float, ndarray, shape (n)
        See the description of `wi`.

    wi : float, ndarray, shape (n)
        `wr` and `wi` contain the real and imaginary parts,
        respectively, of the computed eigenvalues in the same order that
        they appear on the diagonal of the output Schur form T. Complex
        conjugate pairs of eigenvalues will appear consecutively with
        the eigenvalue having the positive imaginary part first.

    vs : float, ndarray, shape (:, :)
        If `jobvs` = 'V', `vs` contains the orthogonal matrix Z of Schur
        vectors.

        If `jobvs` = 'N', `vs` is not referenced.

    rconde : float
        If `sense` = 'E' or 'B', contains the reciprocal condition
        number for the average of the selected eigenvalues.

        If `sense` = 'N' or 'V', `rconde` is not referenced.

    rcondv : float
        If `sense` = 'V' or 'B', `rcondv` contains the reciprocal
        condition number for the selected right invariant subspace.

        If `sense` = 'N' or 'E', `rcondv` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobvs`.

            Constraint: `jobvs` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `sort`.

            Constraint: `sort` = 'N' or 'S'.

        (`errno` -4)
            On entry, error in parameter `sense`.

            Constraint: `sense` = 'N', 'E', 'V' or 'B'.

        (`errno` -5)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` (i > 0) and (i<=n))
            The QR algorithm failed to compute all the eigenvalues.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` n+1)
            The eigenvalues could not be reordered because some
            eigenvalues were too close to separate (the problem is very
            ill-conditioned).

        (`errno` n+2)
            After reordering, roundoff changed values of some complex
            eigenvalues so that leading eigenvalues in the Schur form no
            longer satisfy `select` = True. This could also be caused by
            underflow due to scaling.

    Notes
    -----
    The real Schur factorization of A is given by

        A = ZTZ^T,

    where Z, the matrix of Schur vectors, is orthogonal and T is the
    real Schur form.
    A matrix is in real Schur form if it is upper quasi-triangular with
    1*1 and 2*2 blocks. 2*2 blocks will be standardized in the form

        [[table omitted]]

    where bc < 0.
    The eigenvalues of such a block are a+/-sqrt(bc).

    Optionally, ``dgeesx`` also orders the eigenvalues on the diagonal
    of the real Schur form so that selected eigenvalues are at the top
    left; computes a reciprocal condition number for the average of the
    selected eigenvalues (`rconde`); and computes a reciprocal condition
    number for the right invariant subspace corresponding to the
    selected eigenvalues (`rcondv`).
    The leading columns of Z form an orthonormal basis for this
    invariant subspace.

    For further explanation of the reciprocal condition numbers `rconde`
    and `rcondv`, see Section 4.8 of Anderson `et al.` (1999) (where
    these quantities are called s and sep respectively).

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    def cb_select(wr, wi, data=None):
        """Wrapping user's select. Called as base select."""
        if data is not None:
            _retval_select = select(
                wr,
                wi,
                data,
            )
        else:
            _retval_select = select(
                wr,
                wi,
            )
        return _retval_select

    fun_name = 'naginterfaces.library.lapackeig.dgeesx'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'),),
    )
    _b_utils._check_callable(select, {'fun_name': fun_name, 'entity': 'select'}, allow_none=True)
    _select = None if select is None else cb_select
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _wr = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _wi = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobvs = _b_utils._EngineCharScalarType(
        jobvs, {'fun_name': fun_name, 'entity': 'jobvs'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvs_py = _b_utils._EngineCharScalarType.to_py(_jobvs)
    if _jobvs_py in ('V', 'v'):
        _vs_dim1 = _n_py
    else:
        _vs_dim1 = 1
    if _jobvs_py in ('V', 'v'):
        _vs_dim2 = _n_py
    else:
        _vs_dim2 = 1
    exp_shape_vs = (_vs_dim1, _vs_dim2)
    _vs = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_vs,
        sorder=_m_sorder,
    )
    try:
        _return_scalars = _b_lapackeig.dgeesx(
            jobvs,
            sort,
            _select,
            sense,
            _n,
            _a,
            _wr,
            _wi,
            _vs,
            data,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _a_py = _a
    _sdim_py = _return_scalars.sdim
    _wr_py = _wr
    _wi_py = _wi
    _vs_py = _vs
    _rconde_py = _return_scalars.rconde
    _rcondv_py = _return_scalars.rcondv
    _return_data = DgeesxReturnData(
        _a_py,
        _sdim_py,
        _wr_py,
        _wi_py,
        _vs_py,
        _rconde_py,
        _rcondv_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

DhseqrReturnData = _collections.namedtuple(
    'DhseqrReturnData',
    ['h', 'wr', 'wi', 'z'],
)

def dhseqr(job, compz, ilo, ihi, h, z):
    u"""
    Computes the eigenvalues and Schur factorization of real upper
    Hessenberg matrix reduced from real general matrix.

    ``dhseqr`` computes all the eigenvalues and, optionally, the Schur
    factorization of a real Hessenberg matrix or a real general matrix
    which has been reduced to Hessenberg form.

    For full information please refer to the NAG Library document for
    f08pe

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08pef.html

    Parameters
    ----------
    job : str, length 1
        Indicates whether eigenvalues only or the Schur form T is
        required.

        `job` = 'E'

            Eigenvalues only are required.

        `job` = 'S'

            The Schur form T is required.

    compz : str, length 1
        Indicates whether the Schur vectors are to be computed.

        `compz` = 'N'

            No Schur vectors are computed (and the array `z` is not
            referenced).

        `compz` = 'V'

            The Schur vectors of A are computed (and the array `z` must
            contain the matrix Q on entry).

        `compz` = 'I'

            The Schur vectors of H are computed (and the array `z` is
            initialized by the function).

    ilo : int
        If the matrix A has been balanced by ``dgebal``, `ilo` and `ihi`
        must contain the values returned by that function. Otherwise,
        `ilo` must be set to 1 and `ihi` to n.

    ihi : int
        If the matrix A has been balanced by ``dgebal``, `ilo` and `ihi`
        must contain the values returned by that function. Otherwise,
        `ilo` must be set to 1 and `ihi` to n.

    h : float, array-like, shape (n, n)
        The n*n upper Hessenberg matrix H, as returned by ``dgehrd``.

    z : float, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `compz` in ('V', 'I'): max(1, n); if
        `compz`='N': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compz` in ('V', 'I'): n; if
        `compz`='N': 1; otherwise: 0.

        If `compz` = 'V', `z` must contain the orthogonal matrix Q from
        the reduction to Hessenberg form.

        If `compz` = 'I', `z` need not be set.

    Returns
    -------
    h : float, ndarray, shape (n, n)
        If `job` = 'E', the array contains no useful information.

        If `job` = 'S', `h` is overwritten by the upper quasi-triangular
        matrix T from the Schur decomposition (the Schur form) unless
        `errno` > 0.

    wr : float, ndarray, shape (n)
        The real and imaginary parts, respectively, of the computed
        eigenvalues, unless `errno` > 0 (in which case see
        [Raises/Warns]). Complex conjugate pairs of eigenvalues appear
        consecutively with the eigenvalue having positive imaginary part
        first. The eigenvalues are stored in the same order as on the
        diagonal of the Schur form T (if computed); see [Further
        Comments] for details.

    wi : float, ndarray, shape (n)
        The real and imaginary parts, respectively, of the computed
        eigenvalues, unless `errno` > 0 (in which case see
        [Raises/Warns]). Complex conjugate pairs of eigenvalues appear
        consecutively with the eigenvalue having positive imaginary part
        first. The eigenvalues are stored in the same order as on the
        diagonal of the Schur form T (if computed); see [Further
        Comments] for details.

    z : float, ndarray, shape (:, :)
        If `compz` = 'V' or 'I', `z` contains the orthogonal matrix of
        the required Schur vectors, unless `errno` > 0.

        If `compz` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'E' or 'S'.

        (`errno` -2)
            On entry, error in parameter `compz`.

            Constraint: `compz` = 'N', 'V' or 'I'.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -4)
            On entry, error in parameter `ilo`.

        (`errno` -5)
            On entry, error in parameter `ihi`.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            The algorithm has failed to find all the eigenvalues after a
            total of 30*(`ihi`-`ilo`+1) iterations.

    Notes
    -----
    ``dhseqr`` computes all the eigenvalues and, optionally, the Schur
    factorization of a real upper Hessenberg matrix H:

        H = ZTZ^T,

    where T is an upper quasi-triangular matrix (the Schur form of H),
    and Z is the orthogonal matrix whose columns are the Schur vectors
    z_i.
    See [Further Comments] for details of the structure of T.

    The function may also be used to compute the Schur factorization of
    a real general matrix A which has been reduced to upper Hessenberg
    form H:

        [table omitted]

    In this case, after ``dgehrd`` has been called to reduce A to
    Hessenberg form, ``dorghr`` must be called to form Q explicitly; Q
    is then passed to ``dhseqr``, which must be called with `compz` =
    'V'.

    The function can also take advantage of a previous call to
    ``dgebal`` which may have balanced the original matrix before
    reducing it to Hessenberg form, so that the Hessenberg matrix H has
    the structure:

        [table omitted]

    where H_11 and H_33 are upper triangular.
    If so, only the central diagonal block H_22 (in rows and columns
    i_lo to i_hi) needs to be further reduced to Schur form (the blocks
    H_12 and H_23 are also affected).
    Therefore, the values of i_lo and i_hi can be supplied to ``dhseqr``
    directly.
    Also, ``dgebak`` must be called after this function to permute the
    Schur vectors of the balanced matrix to those of the original
    matrix.
    If ``dgebal`` has not been called however, then i_lo must be set to
    1 and i_hi to n.
    Note that if the Schur factorization of A is required, ``dgebal``
    must **not** be called with job = 'S' or 'B', because the balancing
    transformation is not orthogonal.

    ``dhseqr`` uses a multishift form of the upper Hessenberg QR
    algorithm, due to Bai and Demmel (1989).
    The Schur vectors are normalized so that ||z_i||_2 = 1, but are
    determined only to within a factor +/-1.

    References
    ----------
    Bai, Z and Demmel, J W, 1989, `On a block implementation of
    Hessenberg multishift QR iteration`, Internat. J. High Speed Comput.
    (1), 97--112

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dhseqr'

    _h = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'h'},
        h,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_h, {'fun_name': fun_name, 'entity': 'h'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _wr = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _wi = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _z = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'z'},
        z,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    try:
        _b_lapackeig.dhseqr(
            job,
            compz,
            _n,
            ilo,
            ihi,
            _h,
            _wr,
            _wi,
            _z,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _h_py = _h
    _z_py = _z
    _wr_py = _wr
    _wi_py = _wi
    _return_data = DhseqrReturnData(
        _h_py,
        _wr_py,
        _wi_py,
        _z_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

DhseinReturnData = _collections.namedtuple(
    'DhseinReturnData',
    ['select', 'wr', 'vl', 'vr', 'm', 'ifaill', 'ifailr'],
)

def dhsein(job, eigsrc, initv, select, h, wr, wi, vl, vr):
    u"""
    Computes selected right and/or left eigenvectors of real upper
    Hessenberg matrix by inverse iteration.

    ``dhsein`` computes selected left and/or right eigenvectors of a
    real upper Hessenberg matrix corresponding to specified eigenvalues,
    by inverse iteration.

    For full information please refer to the NAG Library document for
    f08pk

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08pkf.html

    Parameters
    ----------
    job : str, length 1
        Indicates whether left and/or right eigenvectors are to be
        computed.

        `job` = 'R'

            Only right eigenvectors are computed.

        `job` = 'L'

            Only left eigenvectors are computed.

        `job` = 'B'

            Both left and right eigenvectors are computed.

    eigsrc : str, length 1
        Indicates whether the eigenvalues of H (stored in `wr` and `wi`)
        were found using ``dhseqr``.

        `eigsrc` = 'Q'

            The eigenvalues of H were found using ``dhseqr``; thus if H
            has any zero subdiagonal elements (and so is block
            triangular), then the jth eigenvalue can be assumed to be an
            eigenvalue of the block containing the jth row/column. This
            property allows the function to perform inverse iteration on
            just one diagonal block.

        `eigsrc` = 'N'

            No such assumption is made and the function performs inverse
            iteration using the whole matrix.

    initv : str, length 1
        Indicates whether you are supplying initial estimates for the
        selected eigenvectors.

        `initv` = 'N'

            No initial estimates are supplied.

        `initv` = 'U'

            Initial estimates are supplied in `vl` and/or `vr`.

    select : bool, array-like, shape (n)
        Specifies which eigenvectors are to be computed. To obtain the
        real eigenvector corresponding to the real eigenvalue `wr`[j-1],
        `select`[j-1] must be set True. To select the complex
        eigenvector corresponding to the complex eigenvalue
        (`wr`[j-1],`wi`[j-1]) with complex conjugate (`wr`[j],`wi`[j]),
        `select`[j-1] and/or `select`[j] must be set True; the
        eigenvector corresponding to the **first** eigenvalue in the
        pair is computed.

    h : float, array-like, shape (n, n)
        The n*n upper Hessenberg matrix H. If a NaN is detected in `h`,
        the function will return with `errno` = -6.

    wr : float, array-like, shape (n)
        The real and imaginary parts, respectively, of the eigenvalues
        of the matrix H. Complex conjugate pairs of values must be
        stored in consecutive elements of the arrays. If `eigsrc` = 'Q',
        the arrays **must** be exactly as returned by ``dhseqr``.

    wi : float, array-like, shape (n)
        The real and imaginary parts, respectively, of the eigenvalues
        of the matrix H. Complex conjugate pairs of values must be
        stored in consecutive elements of the arrays. If `eigsrc` = 'Q',
        the arrays **must** be exactly as returned by ``dhseqr``.

    vl : float, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `job` in ('L', 'B'): n; if `job`='R':
        1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `job` in ('L', 'B'): mm; if `job`='R':
        1; otherwise: 0.

        If `initv` = 'U' and `job` = 'L' or 'B', `vl` must contain
        starting vectors for inverse iteration for the left
        eigenvectors. Each starting vector must be stored in the same
        column or columns as will be used to store the corresponding
        eigenvector.

        If `initv` = 'N', `vl` need not be set.

    vr : float, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `job` in ('R', 'B'): n; if `job`='L':
        1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `job` in ('R', 'B'): mm; if `job`='L':
        1; otherwise: 0.

        If `initv` = 'U' and `job` = 'R' or 'B', `vr` must contain
        starting vectors for inverse iteration for the right
        eigenvectors. Each starting vector must be stored in the same
        column or columns as will be used to store the corresponding
        eigenvector.

        If `initv` = 'N', `vr` need not be set.

    Returns
    -------
    select : bool, ndarray, shape (n)
        If a complex eigenvector was selected as specified above,
        `select`[j-1] is set to True and `select`[j] to False.

    wr : float, ndarray, shape (n)
        Some elements of `wr` may be modified, as close eigenvalues are
        perturbed slightly in searching for independent eigenvectors.

    vl : float, ndarray, shape (:, :)
        If `job` = 'L' or 'B', `vl` contains the computed left
        eigenvectors (as specified by `select`). The eigenvectors are
        stored consecutively in the columns of the array, in the same
        order as their eigenvalues. Corresponding to each selected real
        eigenvalue is a real eigenvector, occupying one column.
        Corresponding to each selected complex eigenvalue is a complex
        eigenvector, occupying two columns: the first column holds the
        real part and the second column holds the imaginary part.

        If `job` = 'R', `vl` is not referenced.

    vr : float, ndarray, shape (:, :)
        If `job` = 'R' or 'B', `vr` contains the computed right
        eigenvectors (as specified by `select`). The eigenvectors are
        stored consecutively in the columns of the array, in the same
        order as their eigenvalues. Corresponding to each selected real
        eigenvalue is a real eigenvector, occupying one column.
        Corresponding to each selected complex eigenvalue is a complex
        eigenvector, occupying two columns: the first column holds the
        real part and the second column holds the imaginary part.

        If `job` = 'L', `vr` is not referenced.

    m : int
        m, the number of columns of `vl` and/or `vr` required to store
        the selected eigenvectors.

    ifaill : int, ndarray, shape (:)
        If `job` = 'L' or 'B', then `ifaill`[i-1] = 0 if the selected
        left eigenvector converged and `ifaill`[i-1] = j > 0 if the
        eigenvector stored in the ith column of `vl` (corresponding to
        the jth eigenvalue as held in (`wr`[j-1],`wi`[j-1]) failed to
        converge. If the ith and (i+1)th columns of `vl` contain a
        selected complex eigenvector, then `ifaill`[i-1] and `ifaill`[i]
        are set to the same value.

        If `job` = 'R', `ifaill` is not referenced.

    ifailr : int, ndarray, shape (:)
        If `job` = 'R' or 'B', then `ifailr`[i-1] = 0 if the selected
        right eigenvector converged and `ifailr`[i-1] = j > 0 if the
        eigenvector stored in the ith row or column of `vr`
        (corresponding to the jth eigenvalue as held in
        (`wr`[j-1],`wi`[j-1])) failed to converge. If the ith and
        (i+1)th rows or columns of `vr` contain a selected complex
        eigenvector, then `ifailr`[i-1] and `ifailr`[i] are set to the
        same value.

        If `job` = 'L', `ifailr` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'R', 'L' or 'B'.

        (`errno` -2)
            On entry, error in parameter `eigsrc`.

            Constraint: `eigsrc` = 'Q' or 'N'.

        (`errno` -3)
            On entry, error in parameter `initv`.

            Constraint: `initv` = 'N' or 'U'.

        (`errno` -5)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -6)
            On entry, error in parameter `h`.

            Constraint: No element of `h` is equal to NaN.

        (`errno` -14)
            On entry, error in parameter mm.

            Constraint: mm >= m.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            *<value>* eigenvectors (as indicated by arguments `ifaill`
            and/or `ifailr`) failed to converge. The corresponding
            columns of `vl` and/or `vr` contain no useful information.

    Notes
    -----
    ``dhsein`` computes left and/or right eigenvectors of a real upper
    Hessenberg matrix H, corresponding to selected eigenvalues.

    The right eigenvector x, and the left eigenvector y, corresponding
    to an eigenvalue lambda, are defined by:

        Hx = lambda x and y^HH = lambda y^H(or H^Ty = lambda_y).

    Note that even though H is real, lambda, x and y may be complex.
    If x is an eigenvector corresponding to a complex eigenvalue lambda,
    then the complex conjugate vector x_ is the eigenvector
    corresponding to the complex conjugate eigenvalue lambda_.

    The eigenvectors are computed by inverse iteration.
    They are scaled so that, for a real eigenvector x, max(|x_i|) = 1,
    and for a complex eigenvector, max(|Re(x_i)|+|Im(x_i)|) = 1.

    If H has been formed by reduction of a real general matrix A to
    upper Hessenberg form, then the eigenvectors of H may be transformed
    to eigenvectors of A by a call to ``dormhr``.

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dhsein'

    _h = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'h'},
        h,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_h, {'fun_name': fun_name, 'entity': 'h'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _vl_k_py = 0
    _wi = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'wi'},
        wi,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _select = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'select'},
        select,
        copy=True,
    )
    _vl_m_py = 0
    while _vl_k_py < _n_py:
        _vl_k_py = _vl_k_py+1
        if _b_utils._EngineFloat64ArrayType.get_element(_wi, {'fun_name': fun_name, 'entity': 'wi'}, (_vl_k_py-1,)) == 0:
            if _b_utils._EngineLogicalArrayType.get_element(_select, {'fun_name': fun_name, 'entity': 'select'}, (_vl_k_py-1,)):
                _vl_m_py = _vl_m_py+1
        elif _vl_k_py < _n_py:
            if _b_utils._EngineLogicalArrayType.get_element(_select, {'fun_name': fun_name, 'entity': 'select'}, (_vl_k_py-1,)) or _b_utils._EngineLogicalArrayType.get_element(_select, {'fun_name': fun_name, 'entity': 'select'}, ((_vl_k_py+1)-1,)):
                _vl_m_py = _vl_m_py+2
            _vl_k_py = _vl_k_py+1
    _wr = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'wr'},
        wr,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _vl = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'vl'},
        vl,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _vr = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'vr'},
        vr,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _mm = _b_utils._EngineIntScalarType(
        _vl_m_py, {'fun_name': fun_name, 'entity': '_mm'},
    ).to_ctypes()
    _job = _b_utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _job_py = _b_utils._EngineCharScalarType.to_py(_job)
    _mm_py = _b_utils._EngineIntScalarType.to_py(_mm)
    if _job_py in ('L', 'l', 'B', 'b'):
        _ifaill_dim1 = _mm_py
    elif _job_py in ('R', 'r'):
        _ifaill_dim1 = 1
    else:
        _ifaill_dim1 = 0
    exp_shape_ifaill = (_ifaill_dim1,)
    _ifaill = _b_utils._EngineIntArrayType.empty_ndarray(
        exp_shape_ifaill,
    )
    if _job_py in ('R', 'r', 'B', 'b'):
        _ifailr_dim1 = _mm_py
    elif _job_py in ('L', 'l'):
        _ifailr_dim1 = 1
    else:
        _ifailr_dim1 = 0
    exp_shape_ifailr = (_ifailr_dim1,)
    _ifailr = _b_utils._EngineIntArrayType.empty_ndarray(
        exp_shape_ifailr,
    )
    try:
        _return_scalar = _b_lapackeig.dhsein(
            job,
            eigsrc,
            initv,
            _select,
            _n,
            _h,
            _wr,
            _wi,
            _vl,
            _vr,
            _mm,
            _ifaill,
            _ifailr,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _select_py = _select
    _wr_py = _wr
    _vl_py = _vl
    _vr_py = _vr
    _m_py = _return_scalar
    _ifaill_py = _ifaill
    _ifailr_py = _ifailr
    _return_data = DhseinReturnData(
        _select_py,
        _wr_py,
        _vl_py,
        _vr_py,
        _m_py,
        _ifaill_py,
        _ifailr_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

ZgeesReturnData = _collections.namedtuple(
    'ZgeesReturnData',
    ['a', 'sdim', 'w', 'vs'],
)

def zgees(jobvs, sort, a, select=None, data=None):
    u"""
    Computes for complex square nonsymmetric matrix, the eigenvalues,
    the Schur form, and, optionally, the matrix of Schur vectors.

    ``zgees`` computes the eigenvalues, the Schur form T, and,
    optionally, the matrix of Schur vectors Z for an n*n complex
    nonsymmetric matrix A.

    For full information please refer to the NAG Library document for
    f08pn

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08pnf.html

    Parameters
    ----------
    jobvs : str, length 1
        If `jobvs` = 'N', Schur vectors are not computed.

        If `jobvs` = 'V', Schur vectors are computed.

    sort : str, length 1
        Specifies whether or not to order the eigenvalues on the
        diagonal of the Schur form.

        `sort` = 'N'

            Eigenvalues are not ordered.

        `sort` = 'S'

            Eigenvalues are ordered (see `select`).

    a : complex, array-like, shape (n, n)
        The n*n matrix A.

    select : None or callable retval = select(w, data=None), optional
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        If `sort` = 'S', `select` is used to select eigenvalues to sort
        to the top left of the Schur form.

        An eigenvalue `w`[j-1] is selected if `select`(`w`[j-1]) is
        True.

        Parameters
        ~~~~~~~~~~
        w : complex
            The real and imaginary parts of the eigenvalue.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : bool
            Must be True if the eigenvalue is to be selected.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    a : complex, ndarray, shape (n, n)
        `a` is overwritten by its Schur form T.

    sdim : int
        If `sort` = 'N', `sdim` = 0.

        If `sort` = 'S', `sdim` = number of eigenvalues for which
        `select` is True.

    w : complex, ndarray, shape (n)
        Contains the computed eigenvalues, in the same order that they
        appear on the diagonal of the output Schur form T.

    vs : complex, ndarray, shape (:, :)
        If `jobvs` = 'V', `vs` contains the unitary matrix Z of Schur
        vectors.

        If `jobvs` = 'N', `vs` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobvs`.

            Constraint: `jobvs` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `sort`.

            Constraint: `sort` = 'N' or 'S'.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` (i > 0) and (i<=n))
            The QR algorithm failed to compute all the eigenvalues.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` n+1)
            The eigenvalues could not be reordered because some
            eigenvalues were too close to separate (the problem is very
            ill-conditioned).

        (`errno` n+2)
            After reordering, roundoff changed values of some complex
            eigenvalues so that leading eigenvalues in the Schur form no
            longer satisfy `select` = True. This could also be caused by
            underflow due to scaling.

    Notes
    -----
    The Schur factorization of A is given by

        A = ZTZ^H,

    where Z, the matrix of Schur vectors, is unitary and T is the Schur
    form.
    A complex matrix is in Schur form if it is upper triangular.

    Optionally, ``zgees`` also orders the eigenvalues on the diagonal of
    the Schur form so that selected eigenvalues are at the top left.
    The leading columns of Z form an orthonormal basis for the invariant
    subspace corresponding to the selected eigenvalues.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    def cb_select(w, data=None):
        """Wrapping user's select. Called as base select."""
        if data is not None:
            _retval_select = select(
                w,
                data,
            )
        else:
            _retval_select = select(
                w,
            )
        return _retval_select

    fun_name = 'naginterfaces.library.lapackeig.zgees'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'),),
    )
    _b_utils._check_callable(select, {'fun_name': fun_name, 'entity': 'select'}, allow_none=True)
    _select = None if select is None else cb_select
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _w = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobvs = _b_utils._EngineCharScalarType(
        jobvs, {'fun_name': fun_name, 'entity': 'jobvs'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvs_py = _b_utils._EngineCharScalarType.to_py(_jobvs)
    if _jobvs_py in ('V', 'v'):
        _vs_dim1 = _n_py
    else:
        _vs_dim1 = 1
    if _jobvs_py in ('V', 'v'):
        _vs_dim2 = _n_py
    else:
        _vs_dim2 = 1
    exp_shape_vs = (_vs_dim1, _vs_dim2)
    _vs = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_vs,
        sorder=_m_sorder,
    )
    try:
        _return_scalar = _b_lapackeig.zgees(
            jobvs,
            sort,
            _select,
            _n,
            _a,
            _w,
            _vs,
            data,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _a_py = _a
    _sdim_py = _return_scalar
    _w_py = _w
    _vs_py = _vs
    _return_data = ZgeesReturnData(
        _a_py,
        _sdim_py,
        _w_py,
        _vs_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

ZgeesxReturnData = _collections.namedtuple(
    'ZgeesxReturnData',
    ['a', 'sdim', 'w', 'vs', 'rconde', 'rcondv'],
)

def zgeesx(jobvs, sort, sense, a, select=None, data=None):
    u"""
    Computes for real square nonsymmetric matrix, the eigenvalues, the
    Schur form, and, optionally, the matrix of Schur vectors; also
    computes a reciprocal condition number for the average of the
    selected eigenvalues and for the right invariant subspace
    corresponding to these eigenvalues.

    ``zgeesx`` computes the eigenvalues, the Schur form T, and,
    optionally, the matrix of Schur vectors Z for an n*n complex
    nonsymmetric matrix A.

    For full information please refer to the NAG Library document for
    f08pp

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08ppf.html

    Parameters
    ----------
    jobvs : str, length 1
        If `jobvs` = 'N', Schur vectors are not computed.

        If `jobvs` = 'V', Schur vectors are computed.

    sort : str, length 1
        Specifies whether or not to order the eigenvalues on the
        diagonal of the Schur form.

        `sort` = 'N'

            Eigenvalues are not ordered.

        `sort` = 'S'

            Eigenvalues are ordered (see `select`).

    sense : str, length 1
        Determines which reciprocal condition numbers are computed.

        `sense` = 'N'

            None are computed.

        `sense` = 'E'

            Computed for average of selected eigenvalues only.

        `sense` = 'V'

            Computed for selected right invariant subspace only.

        `sense` = 'B'

            Computed for both.

        If `sense` = 'E', 'V' or 'B', `sort` = 'S'.

    a : complex, array-like, shape (n, n)
        The n*n matrix A.

    select : None or callable retval = select(w, data=None), optional
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        If `sort` = 'S', `select` is used to select eigenvalues to sort
        to the top left of the Schur form.

        An eigenvalue `w`[j-1] is selected if `select`(`w`[j-1]) is
        True.

        Parameters
        ~~~~~~~~~~
        w : complex
            The real and imaginary parts of the eigenvalue.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : bool
            Must be True if the eigenvalue is to be selected.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    a : complex, ndarray, shape (n, n)
        `a` is overwritten by its Schur form T.

    sdim : int
        If `sort` = 'N', `sdim` = 0.

        If `sort` = 'S', `sdim` = number of eigenvalues for which
        `select` is True.

    w : complex, ndarray, shape (n)
        Contains the computed eigenvalues, in the same order that they
        appear on the diagonal of the output Schur form T.

    vs : complex, ndarray, shape (:, :)
        If `jobvs` = 'V', `vs` contains the unitary matrix Z of Schur
        vectors.

        If `jobvs` = 'N', `vs` is not referenced.

    rconde : float
        If `sense` = 'E' or 'B', contains the reciprocal condition
        number for the average of the selected eigenvalues.

        If `sense` = 'N' or 'V', `rconde` is not referenced.

    rcondv : float
        If `sense` = 'V' or 'B', `rcondv` contains the reciprocal
        condition number for the selected right invariant subspace.

        If `sense` = 'N' or 'E', `rcondv` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobvs`.

            Constraint: `jobvs` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `sort`.

            Constraint: `sort` = 'N' or 'S'.

        (`errno` -4)
            On entry, error in parameter `sense`.

            Constraint: `sense` = 'N', 'E', 'V' or 'B'.

        (`errno` -5)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` (i > 0) and (i<=n))
            The QR algorithm failed to compute all the eigenvalues.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` n+1)
            The eigenvalues could not be reordered because some
            eigenvalues were too close to separate (the problem is very
            ill-conditioned).

        (`errno` n+2)
            After reordering, roundoff changed values of some complex
            eigenvalues so that leading eigenvalues in the Schur form no
            longer satisfy `select` = True. This could also be caused by
            underflow due to scaling.

    Notes
    -----
    The Schur factorization of A is given by

        A = ZTZ^H,

    where Z, the matrix of Schur vectors, is unitary and T is the Schur
    form.
    A complex matrix is in Schur form if it is upper triangular.

    Optionally, ``zgeesx`` also orders the eigenvalues on the diagonal
    of the Schur form so that selected eigenvalues are at the top left;
    computes a reciprocal condition number for the average of the
    selected eigenvalues (`rconde`); and computes a reciprocal condition
    number for the right invariant subspace corresponding to the
    selected eigenvalues (`rcondv`).
    The leading columns of Z form an orthonormal basis for this
    invariant subspace.

    For further explanation of the reciprocal condition numbers `rconde`
    and `rcondv`, see Section 4.8 of Anderson `et al.` (1999) (where
    these quantities are called s and sep respectively).

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    def cb_select(w, data=None):
        """Wrapping user's select. Called as base select."""
        if data is not None:
            _retval_select = select(
                w,
                data,
            )
        else:
            _retval_select = select(
                w,
            )
        return _retval_select

    fun_name = 'naginterfaces.library.lapackeig.zgeesx'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'),),
    )
    _b_utils._check_callable(select, {'fun_name': fun_name, 'entity': 'select'}, allow_none=True)
    _select = None if select is None else cb_select
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _w = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobvs = _b_utils._EngineCharScalarType(
        jobvs, {'fun_name': fun_name, 'entity': 'jobvs'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvs_py = _b_utils._EngineCharScalarType.to_py(_jobvs)
    if _jobvs_py in ('V', 'v'):
        _vs_dim1 = _n_py
    else:
        _vs_dim1 = 1
    if _jobvs_py in ('V', 'v'):
        _vs_dim2 = _n_py
    else:
        _vs_dim2 = 1
    exp_shape_vs = (_vs_dim1, _vs_dim2)
    _vs = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_vs,
        sorder=_m_sorder,
    )
    try:
        _return_scalars = _b_lapackeig.zgeesx(
            jobvs,
            sort,
            _select,
            sense,
            _n,
            _a,
            _w,
            _vs,
            data,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _a_py = _a
    _sdim_py = _return_scalars.sdim
    _w_py = _w
    _vs_py = _vs
    _rconde_py = _return_scalars.rconde
    _rcondv_py = _return_scalars.rcondv
    _return_data = ZgeesxReturnData(
        _a_py,
        _sdim_py,
        _w_py,
        _vs_py,
        _rconde_py,
        _rcondv_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

ZhseqrReturnData = _collections.namedtuple(
    'ZhseqrReturnData',
    ['h', 'w', 'z'],
)

def zhseqr(job, compz, ilo, ihi, h, z=None):
    u"""
    Computes the eigenvalues and Schur factorization of complex upper
    Hessenberg matrix reduced from complex general matrix.

    ``zhseqr`` computes all the eigenvalues and, optionally, the Schur
    factorization of a complex Hessenberg matrix or a complex general
    matrix which has been reduced to Hessenberg form.

    For full information please refer to the NAG Library document for
    f08ps

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08psf.html

    Parameters
    ----------
    job : str, length 1
        Indicates whether eigenvalues only or the Schur form T is
        required.

        `job` = 'E'

            Eigenvalues only are required.

        `job` = 'S'

            The Schur form T is required.

    compz : str, length 1
        Indicates whether the Schur vectors are to be computed.

        `compz` = 'N'

            No Schur vectors are computed (and the array `z` is not
            referenced).

        `compz` = 'V'

            The Schur vectors of A are computed (and the array `z` must
            contain the matrix Q on entry).

        `compz` = 'I'

            The Schur vectors of H are computed (and the array `z` is
            initialized by the function).

    ilo : int
        If the matrix A has been balanced by ``zgebal``, `ilo` and `ihi`
        must contain the values returned by that function. Otherwise,
        `ilo` must be set to 1 and `ihi` to n.

    ihi : int
        If the matrix A has been balanced by ``zgebal``, `ilo` and `ihi`
        must contain the values returned by that function. Otherwise,
        `ilo` must be set to 1 and `ihi` to n.

    h : complex, array-like, shape (n, n)
        The n*n upper Hessenberg matrix H, as returned by ``zgehrd``.

    z : None or complex, array-like, shape (:, :), optional
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `compz` in ('V', 'I'): max(1, n); if
        `compz`='N': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compz` in ('V', 'I'): n; if
        `compz`='N': 1; otherwise: 0.

        If `compz` = 'V', `z` must contain the unitary matrix Q from the
        reduction to Hessenberg form.

        If `compz` = 'I', `z` need not be set.

    Returns
    -------
    h : complex, ndarray, shape (n, n)
        If `job` = 'E', the array contains no useful information.

        If `job` = 'S', `h` is overwritten by the upper triangular
        matrix T from the Schur decomposition (the Schur form) unless
        `errno` > 0.

    w : complex, ndarray, shape (n)
        The computed eigenvalues, unless `errno` > 0 (in which case see
        [Raises/Warns]). The eigenvalues are stored in the same order as
        on the diagonal of the Schur form T (if computed).

    z : None or complex, ndarray, shape (:, :)
        If `compz` = 'V' or 'I', `z` contains the unitary matrix of the
        required Schur vectors, unless `errno` > 0.

        If `compz` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'E' or 'S'.

        (`errno` -2)
            On entry, error in parameter `compz`.

            Constraint: `compz` = 'N', 'V' or 'I'.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -4)
            On entry, error in parameter `ilo`.

        (`errno` -5)
            On entry, error in parameter `ihi`.

        (`errno` i > 0)
            The algorithm has failed to find all the eigenvalues after a
            total of 30*(`ihi`-`ilo`+1) iterations.

    Notes
    -----
    ``zhseqr`` computes all the eigenvalues and, optionally, the Schur
    factorization of a complex upper Hessenberg matrix H:

        H = ZTZ^H,

    where T is an upper triangular matrix (the Schur form of H), and Z
    is the unitary matrix whose columns are the Schur vectors z_i.
    The diagonal elements of T are the eigenvalues of H.

    The function may also be used to compute the Schur factorization of
    a complex general matrix A which has been reduced to upper
    Hessenberg form H:

        [table omitted]

    In this case, after ``zgehrd`` has been called to reduce A to
    Hessenberg form, ``zunghr`` must be called to form Q explicitly; Q
    is then passed to ``zhseqr``, which must be called with `compz` =
    'V'.

    The function can also take advantage of a previous call to
    ``zgebal`` which may have balanced the original matrix before
    reducing it to Hessenberg form, so that the Hessenberg matrix H has
    the structure:

        [table omitted]

    where H_11 and H_33 are upper triangular.
    If so, only the central diagonal block H_22 (in rows and columns
    i_lo to i_hi) needs to be further reduced to Schur form (the blocks
    H_12 and H_23 are also affected).
    Therefore, the values of i_lo and i_hi can be supplied to ``zhseqr``
    directly.
    Also, ``zgebak`` must be called after this function to permute the
    Schur vectors of the balanced matrix to those of the original
    matrix.
    If ``zgebal`` has not been called however, then i_lo must be set to
    1 and i_hi to n.
    Note that if the Schur factorization of A is required, ``zgebal``
    must **not** be called with job = 'S' or 'B', because the balancing
    transformation is not unitary.

    ``zhseqr`` uses a multishift form of the upper Hessenberg QR
    algorithm, due to Bai and Demmel (1989).
    The Schur vectors are normalized so that ||z_i||_2 = 1, but are
    determined only to within a complex factor of absolute value 1.

    References
    ----------
    Bai, Z and Demmel, J W, 1989, `On a block implementation of
    Hessenberg multishift QR iteration`, Internat. J. High Speed Comput.
    (1), 97--112

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zhseqr'

    _h = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'h'},
        h,
        copy=True,
    )
    _z = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'z'},
        z,
        copy=True,
        allow_none=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_h, 'h'), (_z, 'z')),
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_h, {'fun_name': fun_name, 'entity': 'h'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _w = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_n_py,),
    )
    _compz = _b_utils._EngineCharScalarType(
        compz, {'fun_name': fun_name, 'entity': 'compz'}, exp_chrlen=1,
    ).to_ctypes()
    _compz_py = _b_utils._EngineCharScalarType.to_py(_compz)
    if _compz_py in ('V', 'v'):
        if z is None:
            raise _b_utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'z'},
                'must not be None when compz in (\'V\', \'v\')',
            )
    if z is None:
        if _compz_py in ('V', 'v', 'I', 'i'):
            _z_dim1 = max(1, _n_py)
        elif _compz_py in ('N', 'n'):
            _z_dim1 = 1
        else:
            _z_dim1 = 0
        if _compz_py in ('V', 'v', 'I', 'i'):
            _z_dim2 = _n_py
        elif _compz_py in ('N', 'n'):
            _z_dim2 = 1
        else:
            _z_dim2 = 0
        exp_shape_z = (_z_dim1, _z_dim2)
        _z = _b_utils._EngineComplex128ArrayType.empty_ndarray(
            exp_shape_z,
            sorder=_m_sorder,
        )
    _b_lapackeig.zhseqr(
        job,
        compz,
        _n,
        ilo,
        ihi,
        _h,
        _w,
        _z,
    )
    _h_py = _h
    if _compz_py not in ('N', 'n'):
        _z_py = _z
    else:
        _z_py = None
    _w_py = _w
    _return_data = ZhseqrReturnData(
        _h_py,
        _w_py,
        _z_py,
    )
    return _return_data

ZhseinReturnData = _collections.namedtuple(
    'ZhseinReturnData',
    ['w', 'vl', 'vr', 'm', 'ifaill', 'ifailr'],
)

def zhsein(job, eigsrc, initv, select, h, w, vl=None, vr=None):
    u"""
    Computes selected right and/or left eigenvectors of complex upper
    Hessenberg matrix by inverse iteration.

    ``zhsein`` computes selected left and/or right eigenvectors of a
    complex upper Hessenberg matrix corresponding to specified
    eigenvalues, by inverse iteration.

    For full information please refer to the NAG Library document for
    f08px

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08pxf.html

    Parameters
    ----------
    job : str, length 1
        Indicates whether left and/or right eigenvectors are to be
        computed.

        `job` = 'R'

            Only right eigenvectors are computed.

        `job` = 'L'

            Only left eigenvectors are computed.

        `job` = 'B'

            Both left and right eigenvectors are computed.

    eigsrc : str, length 1
        Indicates whether the eigenvalues of H (stored in `w`) were
        found using ``zhseqr``.

        `eigsrc` = 'Q'

            The eigenvalues of H were found using ``zhseqr``; thus if H
            has any zero subdiagonal elements (and so is block
            triangular), then the jth eigenvalue can be assumed to be an
            eigenvalue of the block containing the jth row/column. This
            property allows the function to perform inverse iteration on
            just one diagonal block.

        `eigsrc` = 'N'

            No such assumption is made and the function performs inverse
            iteration using the whole matrix.

    initv : str, length 1
        Indicates whether you are supplying initial estimates for the
        selected eigenvectors.

        `initv` = 'N'

            No initial estimates are supplied.

        `initv` = 'U'

            Initial estimates are supplied in `vl` and/or `vr`.

    select : bool, array-like, shape (n)
        Specifies which eigenvectors are to be computed. To select the
        eigenvector corresponding to the eigenvalue `w`[j-1],
        `select`[j-1] must be set to True.

    h : complex, array-like, shape (n, n)
        The n*n upper Hessenberg matrix H. If a NaN is detected in `h`,
        the function will return with `errno` = -6.

    w : complex, array-like, shape (n)
        The eigenvalues of the matrix H. If `eigsrc` = 'Q', the array
        **must** be exactly as returned by ``zhseqr``.

    vl : None or complex, array-like, shape (:, :), optional
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `job` in ('L', 'B'): n; if `job`='R':
        1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `job` in ('L', 'B'): mm; if `job`='R':
        1; otherwise: 0.

        If `initv` = 'U' and `job` = 'L' or 'B', `vl` must contain
        starting vectors for inverse iteration for the left
        eigenvectors. Each starting vector must be stored in the same
        column as will be used to store the corresponding eigenvector.

        If `initv` = 'N', `vl` need not be set.

    vr : None or complex, array-like, shape (:, :), optional
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `job` in ('R', 'B'): n; if `job`='L':
        1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `job` in ('R', 'B'): mm; if `job`='L':
        1; otherwise: 0.

        If `initv` = 'U' and `job` = 'R' or 'B', `vr` must contain
        starting vectors for inverse iteration for the right
        eigenvectors. Each starting vector must be stored in the same
        column as will be used to store the corresponding eigenvector.

        If `initv` = 'N', `vr` need not be set.

    Returns
    -------
    w : complex, ndarray, shape (n)
        The real parts of some elements of `w` may be modified, as close
        eigenvalues are perturbed slightly in searching for independent
        eigenvectors.

    vl : None or complex, ndarray, shape (:, :)
        If `job` = 'L' or 'B', `vl` contains the computed left
        eigenvectors (as specified by `select`). The eigenvectors are
        stored consecutively in the columns of the array, in the same
        order as their eigenvalues.

        If `job` = 'R', `vl` is not referenced.

    vr : None or complex, ndarray, shape (:, :)
        If `job` = 'R' or 'B', `vr` contains the computed right
        eigenvectors (as specified by `select`). The eigenvectors are
        stored consecutively in the columns of the array, in the same
        order as their eigenvalues.

        If `job` = 'L', `vr` is not referenced.

    m : int
        m, the number of selected eigenvectors.

    ifaill : int, ndarray, shape (:)
        If `job` = 'L' or 'B', then `ifaill`[i-1] = 0 if the selected
        left eigenvector converged and `ifaill`[i-1] = j > 0 if the
        eigenvector stored in the ith row or column of `vl`
        (corresponding to the jth eigenvalue) failed to converge.

        If `job` = 'R', `ifaill` is not referenced.

    ifailr : int, ndarray, shape (:)
        If `job` = 'R' or 'B', then `ifailr`[i-1] = 0 if the selected
        right eigenvector converged and `ifailr`[i-1] = j > 0 if the
        eigenvector stored in the ith column of `vr` (corresponding to
        the jth eigenvalue) failed to converge.

        If `job` = 'L', `ifailr` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'R', 'L' or 'B'.

        (`errno` -2)
            On entry, error in parameter `eigsrc`.

            Constraint: `eigsrc` = 'Q' or 'N'.

        (`errno` -3)
            On entry, error in parameter `initv`.

            Constraint: `initv` = 'N' or 'U'.

        (`errno` -5)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -6)
            On entry, error in parameter `h`.

            Constraint: No element of `h` is equal to NaN.

        (`errno` -13)
            On entry, error in parameter mm.

            Constraint: mm >= m.

        (`errno` i > 0)
            *<value>* eigenvectors (as indicated by arguments `ifaill`
            and/or `ifailr`) failed to converge. The corresponding
            columns of `vl` and/or `vr` contain no useful information.

    Notes
    -----
    ``zhsein`` computes left and/or right eigenvectors of a complex
    upper Hessenberg matrix H, corresponding to selected eigenvalues.

    The right eigenvector x, and the left eigenvector y, corresponding
    to an eigenvalue lambda, are defined by:

        Hx = lambda x and y^HH = lambda y^H(or H^Hy = lambda_y).

    The eigenvectors are computed by inverse iteration.
    They are scaled so that max(|Re(x_i)|+|Im(x_i)|) = 1.

    If H has been formed by reduction of a complex general matrix A to
    upper Hessenberg form, then the eigenvectors of H may be transformed
    to eigenvectors of A by a call to ``zunmhr``.

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zhsein'

    _h = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'h'},
        h,
        copy=False,
    )
    _vl = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'vl'},
        vl,
        copy=True,
        allow_none=True,
    )
    _vr = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'vr'},
        vr,
        copy=True,
        allow_none=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_h, 'h'), (_vl, 'vl'), (_vr, 'vr')),
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_h, {'fun_name': fun_name, 'entity': 'h'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _select = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'select'},
        select,
        copy=False,
    )
    _vl_m_py = 0
    for _vl_j_py in range(1, _n_py+1):
        if _b_utils._EngineLogicalArrayType.get_element(_select, {'fun_name': fun_name, 'entity': 'select'}, (_vl_j_py-1,)):
            _vl_m_py = _vl_m_py + 1
    _w = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'w'},
        w,
        copy=True,
    )
    _initv = _b_utils._EngineCharScalarType(
        initv, {'fun_name': fun_name, 'entity': 'initv'}, exp_chrlen=1,
    ).to_ctypes()
    _initv_py = _b_utils._EngineCharScalarType.to_py(_initv)
    _job = _b_utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _job_py = _b_utils._EngineCharScalarType.to_py(_job)
    if (_initv_py in ('U', 'u')) and _job_py in ('L', 'l', 'B', 'b'):
        if vl is None:
            raise _b_utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'vl'},
                'must not be None when (initv in (\'U\', \'u\')) and job in (\'L\', \'l\', \'B\', \'b\')',
            )
    _mm = _b_utils._EngineIntScalarType(
        _vl_m_py, {'fun_name': fun_name, 'entity': '_mm'},
    ).to_ctypes()
    _mm_py = _b_utils._EngineIntScalarType.to_py(_mm)
    if vl is None:
        if _job_py in ('L', 'l', 'B', 'b'):
            _vl_dim1 = _n_py
        elif _job_py in ('R', 'r'):
            _vl_dim1 = 1
        else:
            _vl_dim1 = 0
        if _job_py in ('L', 'l', 'B', 'b'):
            _vl_dim2 = _mm_py
        elif _job_py in ('R', 'r'):
            _vl_dim2 = 1
        else:
            _vl_dim2 = 0
        exp_shape_vl = (_vl_dim1, _vl_dim2)
        _vl = _b_utils._EngineComplex128ArrayType.empty_ndarray(
            exp_shape_vl,
            sorder=_m_sorder,
        )
    if (_initv_py in ('U', 'u')) and _job_py in ('R', 'r', 'B', 'b'):
        if vr is None:
            raise _b_utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'vr'},
                'must not be None when (initv in (\'U\', \'u\')) and job in (\'R\', \'r\', \'B\', \'b\')',
            )
    if vr is None:
        if _job_py in ('R', 'r', 'B', 'b'):
            _vr_dim1 = _n_py
        elif _job_py in ('L', 'l'):
            _vr_dim1 = 1
        else:
            _vr_dim1 = 0
        if _job_py in ('R', 'r', 'B', 'b'):
            _vr_dim2 = _mm_py
        elif _job_py in ('L', 'l'):
            _vr_dim2 = 1
        else:
            _vr_dim2 = 0
        exp_shape_vr = (_vr_dim1, _vr_dim2)
        _vr = _b_utils._EngineComplex128ArrayType.empty_ndarray(
            exp_shape_vr,
            sorder=_m_sorder,
        )
    if _job_py in ('L', 'l', 'B', 'b'):
        _ifaill_dim1 = _mm_py
    elif _job_py in ('R', 'r'):
        _ifaill_dim1 = 1
    else:
        _ifaill_dim1 = 0
    exp_shape_ifaill = (_ifaill_dim1,)
    _ifaill = _b_utils._EngineIntArrayType.empty_ndarray(
        exp_shape_ifaill,
    )
    if _job_py in ('R', 'r', 'B', 'b'):
        _ifailr_dim1 = _mm_py
    elif _job_py in ('L', 'l'):
        _ifailr_dim1 = 1
    else:
        _ifailr_dim1 = 0
    exp_shape_ifailr = (_ifailr_dim1,)
    _ifailr = _b_utils._EngineIntArrayType.empty_ndarray(
        exp_shape_ifailr,
    )
    _return_scalar = _b_lapackeig.zhsein(
        job,
        eigsrc,
        initv,
        _select,
        _n,
        _h,
        _w,
        _vl,
        _vr,
        _mm,
        _ifaill,
        _ifailr,
    )
    _w_py = _w
    _m_py = _return_scalar
    if _job_py in ('L', 'l', 'B', 'b'):
        _vl_dim2 = max(1, _m_py)
    elif _job_py in ('R', 'r'):
        _vl_dim2 = 1
    else:
        _vl_dim2 = 0
    if _job_py in ('L', 'l', 'B', 'b'):
        _vl_py = _vl[:, :_vl_dim2]
    else:
        _vl_py = None
    if _job_py in ('R', 'r', 'B', 'b'):
        _vr_dim2 = max(1, _m_py)
    elif _job_py in ('L', 'l'):
        _vr_dim2 = 1
    else:
        _vr_dim2 = 0
    if _job_py in ('R', 'r', 'B', 'b'):
        _vr_py = _vr[:, :_vr_dim2]
    else:
        _vr_py = None
    _ifaill_py = _ifaill
    _ifailr_py = _ifailr
    _return_data = ZhseinReturnData(
        _w_py,
        _vl_py,
        _vr_py,
        _m_py,
        _ifaill_py,
        _ifailr_py,
    )
    return _return_data

DtrexcReturnData = _collections.namedtuple(
    'DtrexcReturnData',
    ['t', 'q', 'ifst', 'ilst'],
)

def dtrexc(compq, t, q, ifst, ilst):
    u"""
    Reorders a Schur factorization of real matrix using orthogonal
    similarity transformation.

    ``dtrexc`` reorders the Schur factorization of a real general
    matrix.

    For full information please refer to the NAG Library document for
    f08qf

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08qff.html

    Parameters
    ----------
    compq : str, length 1
        Indicates whether the matrix Q of Schur vectors is to be
        updated.

        `compq` = 'V'

            The matrix Q of Schur vectors is updated.

        `compq` = 'N'

            No Schur vectors are updated.

    t : float, array-like, shape (n, n)
        The n*n upper quasi-triangular matrix T in canonical Schur form,
        as returned by ``dhseqr``.

    q : float, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `compq`='V': n; if `compq`='N': 1;
        otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compq`='V': n; if `compq`='N': 1;
        otherwise: 0.

        If `compq` = 'V', `q` must contain the n*n orthogonal matrix Q
        of Schur vectors.

    ifst : int
        `ifst` and `ilst` must specify the reordering of the diagonal
        elements or blocks of T. The element or block with row index
        `ifst` is moved to row `ilst` by a sequence of exchanges between
        adjacent elements or blocks.

    ilst : int
        `ifst` and `ilst` must specify the reordering of the diagonal
        elements or blocks of T. The element or block with row index
        `ifst` is moved to row `ilst` by a sequence of exchanges between
        adjacent elements or blocks.

    Returns
    -------
    t : float, ndarray, shape (n, n)
        `t` is overwritten by the updated matrix T~. See also [Further
        Comments].

    q : float, ndarray, shape (:, :)
        If `compq` = 'V', `q` contains the updated matrix of Schur
        vectors.

        If `compq` = 'N', `q` is not referenced.

    ifst : int
        If `ifst` pointed to the second row of a 2*2 block on entry, it
        is changed to point to the first row. `ilst` always points to
        the first row of the block in its final position (which may
        differ from its input value by +/-1).

    ilst : int
        If `ifst` pointed to the second row of a 2*2 block on entry, it
        is changed to point to the first row. `ilst` always points to
        the first row of the block in its final position (which may
        differ from its input value by +/-1).

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `compq`.

            Constraint: `compq` = 'V' or 'N'.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -7)
            On entry, error in parameter `ifst`.

        (`errno` -8)
            On entry, error in parameter `ilst`.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            Two adjacent diagonal elements or blocks could not be
            successfully exchanged. This error can only occur if the
            exchange involves at least one 2*2 block; it implies that
            the problem is very ill-conditioned, and that the
            eigenvalues of the two blocks are very close. On exit, T may
            have been partially reordered, and `ilst` points to the
            first row of the current position of the block being moved;
            Q (if requested) is updated consistently with T.

    Notes
    -----
    ``dtrexc`` reorders the Schur factorization of a real general matrix
    A = QTQ^T, so that the diagonal element or block of T with row index
    `ifst` is moved to row `ilst`.

    The reordered Schur form T~ is computed by an orthogonal similarity
    transformation: T~ = Z^TTZ.
    Optionally the updated matrix Q~ of Schur vectors is computed as Q~
    = QZ, giving A = Q~T~Q~^T.

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dtrexc'

    _t = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 't'},
        t,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _q = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'q'},
        q,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    try:
        _return_scalars = _b_lapackeig.dtrexc(
            compq,
            _n,
            _t,
            _q,
            ifst,
            ilst,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _t_py = _t
    _q_py = _q
    _ifst_py = _return_scalars.ifst
    _ilst_py = _return_scalars.ilst
    _return_data = DtrexcReturnData(
        _t_py,
        _q_py,
        _ifst_py,
        _ilst_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

DtrsenReturnData = _collections.namedtuple(
    'DtrsenReturnData',
    ['t', 'q', 'wr', 'wi', 'm', 's', 'sep'],
)

def dtrsen(job, compq, select, t, q):
    u"""
    Reorders a Schur factorization of real matrix, form orthonormal
    basis of right invariant subspace for selected eigenvalues, with
    estimates of sensitivities.

    ``dtrsen`` reorders the Schur factorization of a real general matrix
    so that a selected cluster of eigenvalues appears in the leading
    elements or blocks on the diagonal of the Schur form.
    The function also optionally computes the reciprocal condition
    numbers of the cluster of eigenvalues and/or the invariant subspace.

    For full information please refer to the NAG Library document for
    f08qg

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08qgf.html

    Parameters
    ----------
    job : str, length 1
        Indicates whether condition numbers are required for the cluster
        of eigenvalues and/or the invariant subspace.

        `job` = 'N'

            No condition numbers are required.

        `job` = 'E'

            Only the condition number for the cluster of eigenvalues is
            computed.

        `job` = 'V'

            Only the condition number for the invariant subspace is
            computed.

        `job` = 'B'

            Condition numbers for both the cluster of eigenvalues and
            the invariant subspace are computed.

    compq : str, length 1
        Indicates whether the matrix Q of Schur vectors is to be
        updated.

        `compq` = 'V'

            The matrix Q of Schur vectors is updated.

        `compq` = 'N'

            No Schur vectors are updated.

    select : bool, array-like, shape (n)
        The eigenvalues in the selected cluster. To select a real
        eigenvalue lambda_j, `select`[j-1] must be set True. To select a
        complex conjugate pair of eigenvalues lambda_j and lambda_{j+1}
        (corresponding to a 2*2 diagonal block), `select`[j-1] and/or
        `select`[j] must be set to True. A complex conjugate pair of
        eigenvalues **must** be either both included in the cluster or
        both excluded. See also [Further Comments].

    t : float, array-like, shape (n, n)
        The n*n upper quasi-triangular matrix T in canonical Schur form,
        as returned by ``dhseqr``. See also [Further Comments].

    q : float, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `compq`='V': n; if `compq`='N': 1;
        otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compq`='V': n; if `compq`='N': 1;
        otherwise: 0.

        If `compq` = 'V', `q` must contain the n*n orthogonal matrix Q
        of Schur vectors, as returned by ``dhseqr``.

    Returns
    -------
    t : float, ndarray, shape (n, n)
        `t` is overwritten by the updated matrix T~.

    q : float, ndarray, shape (:, :)
        If `compq` = 'V', `q` contains the updated matrix of Schur
        vectors; the first m columns of Q form an orthonormal basis for
        the specified invariant subspace.

        If `compq` = 'N', `q` is not referenced.

    wr : float, ndarray, shape (n)
        The real and imaginary parts, respectively, of the reordered
        eigenvalues of T~. The eigenvalues are stored in the same order
        as on the diagonal of T~; see [Further Comments] for details.
        Note that if a complex eigenvalue is sufficiently
        ill-conditioned, then its value may differ significantly from
        its value before reordering.

    wi : float, ndarray, shape (n)
        The real and imaginary parts, respectively, of the reordered
        eigenvalues of T~. The eigenvalues are stored in the same order
        as on the diagonal of T~; see [Further Comments] for details.
        Note that if a complex eigenvalue is sufficiently
        ill-conditioned, then its value may differ significantly from
        its value before reordering.

    m : int
        m, the dimension of the specified invariant subspace. The value
        of m is obtained by counting 1 for each selected real eigenvalue
        and 2 for each selected complex conjugate pair of eigenvalues
        (see `select`); 0 <= m <= n.

    s : float
        If `job` = 'E' or 'B', `s` is a lower bound on the reciprocal
        condition number of the average of the selected cluster of
        eigenvalues. If `m` = 0 or n, `s` = 1; if `errno` = 1 (see
        [Raises/Warns]), `s` is set to zero.

        If `job` = 'N' or 'V', `s` is not referenced.

    sep : float
        If `job` = 'V' or 'B', `sep` is the estimated reciprocal
        condition number of the specified invariant subspace. If `m` = 0
        or n, `sep` = ||T||; if `errno` = 1 (see [Raises/Warns]), `sep`
        is set to zero.

        If `job` = 'N' or 'E', `sep` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'N', 'E', 'V' or 'B'.

        (`errno` -2)
            On entry, error in parameter `compq`.

            Constraint: `compq` = 'V' or 'N'.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            The reordering of T failed because a selected eigenvalue was
            too close to an unselected eigenvalue.

    Notes
    -----
    ``dtrsen`` reorders the Schur factorization of a real general matrix
    A = QTQ^T, so that a selected cluster of eigenvalues appears in the
    leading diagonal elements or blocks of the Schur form.

    The reordered Schur form T~ is computed by an orthogonal similarity
    transformation: T~ = Z^TTZ.
    Optionally the updated matrix Q~ of Schur vectors is computed as Q~
    = QZ, giving A = Q~T~Q~^T.

    Let T~ = [table omitted], where the selected eigenvalues are
    precisely the eigenvalues of the leading m*m sub-matrix T_11.
    Let Q~ be correspondingly partitioned as (Q_1 Q_2) where Q_1
    consists of the first m columns of Q.
    Then AQ_1 = Q_1 T_11, and so the m columns of Q_1 form an
    orthonormal basis for the invariant subspace corresponding to the
    selected cluster of eigenvalues.

    Optionally the function also computes estimates of the reciprocal
    condition numbers of the average of the cluster of eigenvalues and
    of the invariant subspace.

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dtrsen'

    _select = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'select'},
        select,
        copy=False,
    )
    _t = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 't'},
        t,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _q = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'q'},
        q,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _wr = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _wi = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        _return_scalars = _b_lapackeig.dtrsen(
            job,
            compq,
            _select,
            _n,
            _t,
            _q,
            _wr,
            _wi,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _t_py = _t
    _q_py = _q
    _wr_py = _wr
    _wi_py = _wi
    _m_py = _return_scalars.m
    _s_py = _return_scalars.s
    _sep_py = _return_scalars.sep
    _return_data = DtrsenReturnData(
        _t_py,
        _q_py,
        _wr_py,
        _wi_py,
        _m_py,
        _s_py,
        _sep_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

DtrsylReturnData = _collections.namedtuple(
    'DtrsylReturnData',
    ['c', 'scale'],
)

def dtrsyl(trana, tranb, isgn, a, b, c):
    u"""
    Solves the real Sylvester matrix equation AX+XB = C, A and B are
    upper quasi-triangular or transposes.

    ``dtrsyl`` solves the real quasi-triangular Sylvester matrix
    equation.

    For full information please refer to the NAG Library document for
    f08qh

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08qhf.html

    Parameters
    ----------
    trana : str, length 1
        Specifies the option op(A).

        `trana` = 'N'

            op(A) = A.

        `trana` = 'T' or 'C'

            op(A) = A^T.

    tranb : str, length 1
        Specifies the option op(B).

        `tranb` = 'N'

            op(B) = B.

        `tranb` = 'T' or 'C'

            op(B) = B^T.

    isgn : int
        Indicates the form of the Sylvester equation.

        `isgn` = +1

            The equation is of the form op(A)X+Xop(B) = alpha C.

        `isgn` = -1

            The equation is of the form op(A)X-Xop(B) = alpha C.

    a : float, array-like, shape (m, m)
        The m*m upper quasi-triangular matrix A in canonical Schur form,
        as returned by ``dhseqr``.

    b : float, array-like, shape (n, n)
        The n*n upper quasi-triangular matrix B in canonical Schur form,
        as returned by ``dhseqr``.

    c : float, array-like, shape (m, n)
        The m*n right-hand side matrix C.

    Returns
    -------
    c : float, ndarray, shape (m, n)
        `c` is overwritten by the solution matrix X.

    scale : float
        The value of the scale factor alpha.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `trana`.

            Constraint: `trana` = 'N', 'T' or 'C'.

        (`errno` -2)
            On entry, error in parameter `tranb`.

            Constraint: `tranb` = 'N', 'T' or 'C'.

        (`errno` -3)
            On entry, error in parameter `isgn`.

            Constraint: `isgn` = +1 or -1.

        (`errno` -4)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -5)
            On entry, error in parameter n.

            Constraint: n >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            A and B have common or close eigenvalues, perturbed values
            of which were used to solve the equation.

    Notes
    -----
    ``dtrsyl`` solves the real Sylvester matrix equation

        op(A)X+/-Xop(B) = alpha C,

    where op(A) = A or A^T, and the matrices A and B are upper
    quasi-triangular matrices in canonical Schur form (as returned by
    ``dhseqr``); alpha is a scale factor (<= 1) determined by the
    function to avoid overflow in X; A is m*m and B is n*n while the
    right-hand side matrix C and the solution matrix X are both m*n.
    The matrix X is obtained by a straightforward process of
    back-substitution (see Golub and Van Loan (1996)).

    Note that the equation has a unique solution if and only if
    alpha_i+/-beta_j != 0, where {alpha_i} and {beta_j} are the
    eigenvalues of A and B respectively and the sign (+ or -) is the
    same as that used in the equation to be solved.

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore

    Higham, N J, 1992, `Perturbation theory and backward error for AX-XB
    = C`, Numerical Analysis Report, University of Manchester
    """

    fun_name = 'naginterfaces.library.lapackeig.dtrsyl'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    try:
        _return_scalar = _b_lapackeig.dtrsyl(
            trana,
            tranb,
            isgn,
            _m,
            _n,
            _a,
            _b,
            _c,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _c_py = _c
    _scale_py = _return_scalar
    _return_data = DtrsylReturnData(
        _c_py,
        _scale_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

DtrevcReturnData = _collections.namedtuple(
    'DtrevcReturnData',
    ['select', 'vl', 'vr', 'm'],
)

def dtrevc(job, howmny, t, select=None, vl=None, vr=None):
    u"""
    Computes left and right eigenvectors of real upper quasi-triangular
    matrix.

    ``dtrevc`` computes selected left and/or right eigenvectors of a
    real upper quasi-triangular matrix.

    For full information please refer to the NAG Library document for
    f08qk

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08qkf.html

    Parameters
    ----------
    job : str, length 1
        Indicates whether left and/or right eigenvectors are to be
        computed.

        `job` = 'R'

            Only right eigenvectors are computed.

        `job` = 'L'

            Only left eigenvectors are computed.

        `job` = 'B'

            Both left and right eigenvectors are computed.

    howmny : str, length 1
        Indicates how many eigenvectors are to be computed.

        `howmny` = 'A'

            All eigenvectors (as specified by `job`) are computed.

        `howmny` = 'B'

            All eigenvectors (as specified by `job`) are computed and
            then pre-multiplied by the matrix Q (which is overwritten).

        `howmny` = 'S'

            Selected eigenvectors (as specified by `job` and `select`)
            are computed.

    t : float, array-like, shape (n, n)
        The n*n upper quasi-triangular matrix T in canonical Schur form,
        as returned by ``dhseqr``.

    select : None or bool, array-like, shape (:), optional
        Note: the required length for this argument is determined as
        follows: if `howmny`='S': n; otherwise: 1.

        Specifies which eigenvectors are to be computed if `howmny` =
        'S'. To obtain the real eigenvector corresponding to the real
        eigenvalue lambda_j, `select`[j-1] must be set True. To select
        the complex eigenvector corresponding to a complex conjugate
        pair of eigenvalues lambda_j and lambda_{j+1}, `select`[j-1]
        and/or `select`[j] must be set True; the eigenvector
        corresponding to the **first** eigenvalue in the pair is
        computed.

    vl : None or float, array-like, shape (:, :), optional
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `job` in ('L', 'B'): n; if `job`='R':
        1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `job` in ('L', 'B'): mm; otherwise: 0.

        If `howmny` = 'B' and `job` = 'L' or 'B', `vl` must contain an
        n*n matrix Q (usually the matrix of Schur vectors returned by
        ``dhseqr``).

        If `howmny` = 'A' or 'S', `vl` need not be set.

    vr : None or float, array-like, shape (:, :), optional
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `job` in ('R', 'B'): n; if `job`='L':
        1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `job` in ('R', 'B'): mm; otherwise: 0.

        If `howmny` = 'B' and `job` = 'R' or 'B', `vr` must contain an
        n*n matrix Q (usually the matrix of Schur vectors returned by
        ``dhseqr``).

        If `howmny` = 'A' or 'S', `vr` need not be set.

    Returns
    -------
    select : None or bool, ndarray, shape (:)
        If a complex eigenvector was selected as specified above,
        `select`[j-1] is set to True and `select`[j] to False.

        If `howmny` = 'A' or 'B', `select` is not referenced.

    vl : None or float, ndarray, shape (:, :)
        If `job` = 'L' or 'B', `vl` contains the computed left
        eigenvectors (as specified by `howmny` and `select`). The
        eigenvectors are stored consecutively in the columns of the
        array, in the same order as their eigenvalues. Corresponding to
        each real eigenvalue is a real eigenvector, occupying one
        column. Corresponding to each complex conjugate pair of
        eigenvalues, is a complex eigenvector occupying two columns; the
        first column holds the real part and the second column holds the
        imaginary part.

        If `job` = 'R', `vl` is not referenced.

    vr : None or float, ndarray, shape (:, :)
        If `job` = 'R' or 'B', `vr` contains the computed right
        eigenvectors (as specified by `howmny` and `select`). The
        eigenvectors are stored consecutively in the columns of the
        array, in the same order as their eigenvalues. Corresponding to
        each real eigenvalue is a real eigenvector, occupying one
        column. Corresponding to each complex conjugate pair of
        eigenvalues, is a complex eigenvector occupying two columns; the
        first column holds the real part and the second column holds the
        imaginary part.

        If `job` = 'L', `vr` is not referenced.

    m : int
        m, the number of columns of `vl` and/or `vr` actually used to
        store the computed eigenvectors. If `howmny` = 'A' or 'B', `m`
        is set to n.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'R', 'L' or 'B'.

        (`errno` -2)
            On entry, error in parameter `howmny`.

            Constraint: `howmny` = 'A', 'B', 'O' or 'S'.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -11)
            On entry, error in parameter mm.

            Constraint: mm >= n.

        (`errno` -11)
            On entry, error in parameter mm.

            Constraint: mm >= m.

    Notes
    -----
    ``dtrevc`` computes left and/or right eigenvectors of a real upper
    quasi-triangular matrix T in canonical Schur form.
    Such a matrix arises from the Schur factorization of a real general
    matrix, as computed by ``dhseqr``, for example.

    The right eigenvector x, and the left eigenvector y, corresponding
    to an eigenvalue lambda, are defined by:

        Tx = lambda x and y^HT = lambda y^H(or T^Ty = lambda_y).

    Note that even though T is real, lambda, x and y may be complex.
    If x is an eigenvector corresponding to a complex eigenvalue lambda,
    then the complex conjugate vector x_ is the eigenvector
    corresponding to the complex conjugate eigenvalue lambda_.

    The function can compute the eigenvectors corresponding to selected
    eigenvalues, or it can compute all the eigenvectors.
    In the latter case the eigenvectors may optionally be pre-multiplied
    by an input matrix Q.
    Normally Q is an orthogonal matrix from the Schur factorization of a
    matrix A as A = QTQ^T; if x is a (left or right) eigenvector of T,
    then Qx is an eigenvector of A.

    The eigenvectors are computed by forward or backward substitution.
    They are scaled so that, for a real eigenvector x, max(|x_i|) = 1,
    and for a complex eigenvector, max(|Re(x_i)|+|Im(x_i)|) = 1.

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dtrevc'

    _t = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 't'},
        t,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _vl = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'vl'},
        vl,
        copy=True,
        allow_none=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _vr = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'vr'},
        vr,
        copy=True,
        allow_none=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_t, 't'), (_vl, 'vl'), (_vr, 'vr')),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _vl_k_py = 0
    _howmny = _b_utils._EngineCharScalarType(
        howmny, {'fun_name': fun_name, 'entity': 'howmny'}, exp_chrlen=1,
    ).to_ctypes()
    _howmny_py = _b_utils._EngineCharScalarType.to_py(_howmny)
    _select = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'select'},
        select,
        copy=True,
        allow_none=True,
    )
    if (_howmny_py in ('S', 's')) and _b_utils._EngineFloat64ArrayType.get_shape(_t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=2)[0] >= _n_py:
        _vl_m_py = 0
        while _vl_k_py < _n_py:
            _vl_k_py = _vl_k_py+1
            if _vl_k_py < _n_py:
                if _b_utils._EngineFloat64ArrayType.get_element(_t, {'fun_name': fun_name, 'entity': 't'}, ((_vl_k_py+1)-1, _vl_k_py-1)) == 0:
                    if _b_utils._EngineLogicalArrayType.get_element(_select, {'fun_name': fun_name, 'entity': 'select'}, (_vl_k_py-1,)):
                        _vl_m_py = _vl_m_py+1
                else:
                    if _b_utils._EngineLogicalArrayType.get_element(_select, {'fun_name': fun_name, 'entity': 'select'}, (_vl_k_py-1,)) or _b_utils._EngineLogicalArrayType.get_element(_select, {'fun_name': fun_name, 'entity': 'select'}, ((_vl_k_py+1)-1,)):
                        _vl_m_py = _vl_m_py+2
                    _vl_k_py = _vl_k_py+1
            elif _b_utils._EngineLogicalArrayType.get_element(_select, {'fun_name': fun_name, 'entity': 'select'}, (_vl_k_py-1,)):
                _vl_m_py = _vl_m_py+1
    else:
        _vl_m_py = _n_py
    if _howmny_py in ('S', 's'):
        if select is None:
            raise _b_utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'select'},
                'must not be None when howmny in (\'S\', \'s\')',
            )
    if select is None:
        if _howmny_py in ('S', 's'):
            _select_dim1 = _n_py
        else:
            _select_dim1 = 1
        exp_shape_select = (_select_dim1,)
        _select = _b_utils._EngineLogicalArrayType.empty_ndarray(
            exp_shape_select,
        )
    _job = _b_utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _job_py = _b_utils._EngineCharScalarType.to_py(_job)
    if (_howmny_py in ('B', 'b')) and _job_py not in ('R', 'r'):
        if vl is None:
            raise _b_utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'vl'},
                'must not be None when (howmny in (\'B\', \'b\')) and job not in (\'R\', \'r\')',
            )
    if _howmny_py in ('S', 's'):
        _mm = _b_utils._EngineIntScalarType(
            _vl_m_py, {'fun_name': fun_name, 'entity': '_mm'},
        ).to_ctypes()
        _mm_py = _b_utils._EngineIntScalarType.to_py(_mm)
    else:
        _mm = _b_utils._EngineIntScalarType(
            _n_py, {'fun_name': fun_name, 'entity': '_mm'},
        ).to_ctypes()
        _mm_py = _b_utils._EngineIntScalarType.to_py(_mm)
    if vl is None:
        if _job_py in ('L', 'l', 'B', 'b'):
            _vl_dim1 = _n_py
        elif _job_py in ('R', 'r'):
            _vl_dim1 = 1
        else:
            _vl_dim1 = 0
        if _job_py in ('L', 'l', 'B', 'b'):
            _vl_dim2 = _mm_py
        else:
            _vl_dim2 = 0
        exp_shape_vl = (_vl_dim1, _vl_dim2)
        _vl = _b_utils._EngineFloat64ArrayType.empty_ndarray(
            exp_shape_vl,
            sorder=_m_sorder,
        )
    if (_howmny_py in ('B', 'b')) and _job_py not in ('L', 'l'):
        if vr is None:
            raise _b_utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'vr'},
                'must not be None when (howmny in (\'B\', \'b\')) and job not in (\'L\', \'l\')',
            )
    if vr is None:
        if _job_py in ('R', 'r', 'B', 'b'):
            _vr_dim1 = _n_py
        elif _job_py in ('L', 'l'):
            _vr_dim1 = 1
        else:
            _vr_dim1 = 0
        if _job_py in ('R', 'r', 'B', 'b'):
            _vr_dim2 = _mm_py
        else:
            _vr_dim2 = 0
        exp_shape_vr = (_vr_dim1, _vr_dim2)
        _vr = _b_utils._EngineFloat64ArrayType.empty_ndarray(
            exp_shape_vr,
            sorder=_m_sorder,
        )
    _return_scalar = _b_lapackeig.dtrevc(
        job,
        howmny,
        _select,
        _n,
        _t,
        _vl,
        _vr,
        _mm,
    )
    if _howmny_py in ('S', 's'):
        _select_py = _select
    else:
        _select_py = None
    if _job_py not in ('R', 'r'):
        _vl_py = _vl
    else:
        _vl_py = None
    if _job_py not in ('L', 'l'):
        _vr_py = _vr
    else:
        _vr_py = None
    _m_py = _return_scalar
    _return_data = DtrevcReturnData(
        _select_py,
        _vl_py,
        _vr_py,
        _m_py,
    )
    return _return_data

DtrsnaReturnData = _collections.namedtuple(
    'DtrsnaReturnData',
    ['s', 'sep', 'm'],
)

def dtrsna(job, howmny, t, select=None, vl=None, vr=None):
    u"""
    Computes estimates of sensitivities of selected eigenvalues and
    eigenvectors of real upper quasi-triangular matrix.

    ``dtrsna`` estimates condition numbers for specified eigenvalues
    and/or right eigenvectors of a real upper quasi-triangular matrix.

    For full information please refer to the NAG Library document for
    f08ql

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08qlf.html

    Parameters
    ----------
    job : str, length 1
        Indicates whether condition numbers are required for eigenvalues
        and/or eigenvectors.

        `job` = 'E'

            Condition numbers for eigenvalues only are computed.

        `job` = 'V'

            Condition numbers for eigenvectors only are computed.

        `job` = 'B'

            Condition numbers for both eigenvalues and eigenvectors are
            computed.

    howmny : str, length 1
        Indicates how many condition numbers are to be computed.

        `howmny` = 'A'

            Condition numbers for all eigenpairs are computed.

        `howmny` = 'S'

            Condition numbers for selected eigenpairs (as specified by
            `select`) are computed.

    t : float, array-like, shape (n, n)
        The n*n upper quasi-triangular matrix T in canonical Schur form,
        as returned by ``dhseqr``.

    select : None or bool, array-like, shape (:), optional
        Note: the required length for this argument is determined as
        follows: if `howmny`='S': n; otherwise: 1.

        Specifies the eigenpairs for which condition numbers are to be
        computed if `howmny` = 'S'. To select condition numbers for the
        eigenpair corresponding to the real eigenvalue lambda_j,
        `select`[j-1] must be set True. To select condition numbers
        corresponding to a complex conjugate pair of eigenvalues
        lambda_j and lambda_{j+1}, `select`[j-1] and/or `select`[j] must
        be set to True.

        If `howmny` = 'A', `select` is not referenced.

    vl : None or float, array-like, shape (:, :), optional
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `job` in ('E', 'B'): n; if `job`='V':
        1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `job` in ('E', 'B'): mm; otherwise: 0.

        If `job` = 'E' or 'B', `vl` must contain the left eigenvectors
        of T (or of any matrix QTQ^T with Q orthogonal) corresponding to
        the eigenpairs specified by `howmny` and `select`. The
        eigenvectors **must** be stored in consecutive columns of `vl`,
        as returned by ``dtrevc`` or ``dhsein``.

        If `job` = 'V', `vl` is not referenced.

    vr : None or float, array-like, shape (:, :), optional
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `job` in ('E', 'B'): n; if `job`='V':
        1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `job` in ('E', 'B'): mm; otherwise: 0.

        If `job` = 'E' or 'B', `vr` must contain the right eigenvectors
        of T (or of any matrix QTQ^T with Q orthogonal) corresponding to
        the eigenpairs specified by `howmny` and `select`. The
        eigenvectors **must** be stored in consecutive columns of `vr`,
        as returned by ``dtrevc`` or ``dhsein``.

        If `job` = 'V', `vr` is not referenced.

    Returns
    -------
    s : None or float, ndarray, shape (:)
        The reciprocal condition numbers of the selected eigenvalues if
        `job` = 'E' or 'B', stored in consecutive elements of the array.
        Thus `s`[j-1], `sep`[j-1] and the jth columns of `vl` and `vr`
        all correspond to the same eigenpair (but not in general the jth
        eigenpair unless all eigenpairs have been selected). For a
        complex conjugate pair of eigenvalues, two consecutive elements
        of `s` are set to the same value.

        If `job` = 'V', `s` is not referenced.

    sep : None or float, ndarray, shape (:)
        The estimated reciprocal condition numbers of the selected right
        eigenvectors if `job` = 'V' or 'B', stored in consecutive
        elements of the array. For a complex eigenvector, two
        consecutive elements of `sep` are set to the same value. If the
        eigenvalues cannot be reordered to compute `sep`[j], `sep`[j] is
        set to zero; this can only occur when the true value would be
        very small anyway.

        If `job` = 'E', `sep` is not referenced.

    m : int
        m, the number of elements of `s` and/or `sep` actually used to
        store the estimated condition numbers. If `howmny` = 'A', `m` is
        set to n.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'E', 'V' or 'B'.

        (`errno` -2)
            On entry, error in parameter `howmny`.

            Constraint: `howmny` = 'A' or 'S'.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -13)
            On entry, error in parameter mm.

    Notes
    -----
    ``dtrsna`` estimates condition numbers for specified eigenvalues
    and/or right eigenvectors of a real upper quasi-triangular matrix T
    in canonical Schur form.
    These are the same as the condition numbers of the eigenvalues and
    right eigenvectors of an original matrix A = ZTZ^T (with orthogonal
    Z), from which T may have been derived.

    ``dtrsna`` computes the reciprocal of the condition number of an
    eigenvalue lambda_i as

        s_i = |v^Hu|/||u||_E||v||_E,

    where u and v are the right and left eigenvectors of T,
    respectively, corresponding to lambda_i.
    This reciprocal condition number always lies between zero (i.e.,
    ill-conditioned) and one (i.e., well-conditioned).

    An approximate error estimate for a computed eigenvalue lambda_i is
    then given by

        epsilon||T||/(s_i),

    where epsilon is the machine precision.

    To estimate the reciprocal of the condition number of the right
    eigenvector corresponding to lambda_i, the function first calls
    ``dtrexc`` to reorder the eigenvalues so that lambda_i is in the
    leading position:

        T = Q[table omitted]Q^T.

    The reciprocal condition number of the eigenvector is then estimated
    as sep_i, the smallest singular value of the matrix (T_22-lambda_i
    I).
    This number ranges from zero (i.e., ill-conditioned) to very large
    (i.e., well-conditioned).

    An approximate error estimate for a computed right eigenvector u
    corresponding to lambda_i is then given by

        epsilon||T||/(sep_i).

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dtrsna'

    _t = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 't'},
        t,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _vl_k_py = 0
    _howmny = _b_utils._EngineCharScalarType(
        howmny, {'fun_name': fun_name, 'entity': 'howmny'}, exp_chrlen=1,
    ).to_ctypes()
    _howmny_py = _b_utils._EngineCharScalarType.to_py(_howmny)
    _select = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'select'},
        select,
        copy=False,
        allow_none=True,
    )
    if (_howmny_py in ('S', 's')) and _b_utils._EngineFloat64ArrayType.get_shape(_t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=2)[0] >= _n_py:
        _vl_m_py = 0
        while _vl_k_py < _n_py:
            _vl_k_py = _vl_k_py+1
            if _vl_k_py < _n_py:
                if _b_utils._EngineFloat64ArrayType.get_element(_t, {'fun_name': fun_name, 'entity': 't'}, ((_vl_k_py+1)-1, _vl_k_py-1)) == 0:
                    if _b_utils._EngineLogicalArrayType.get_element(_select, {'fun_name': fun_name, 'entity': 'select'}, (_vl_k_py-1,)):
                        _vl_m_py = _vl_m_py+1
                else:
                    if _b_utils._EngineLogicalArrayType.get_element(_select, {'fun_name': fun_name, 'entity': 'select'}, (_vl_k_py-1,)) or _b_utils._EngineLogicalArrayType.get_element(_select, {'fun_name': fun_name, 'entity': 'select'}, ((_vl_k_py+1)-1,)):
                        _vl_m_py = _vl_m_py+2
                    _vl_k_py = _vl_k_py+1
            elif _b_utils._EngineLogicalArrayType.get_element(_select, {'fun_name': fun_name, 'entity': 'select'}, (_vl_k_py-1,)):
                _vl_m_py = _vl_m_py+1
    else:
        _vl_m_py = _n_py
    _vl = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'vl'},
        vl,
        copy=False,
        allow_none=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _vr = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'vr'},
        vr,
        copy=False,
        allow_none=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    if _howmny_py in ('S', 's'):
        _mm = _b_utils._EngineIntScalarType(
            _vl_m_py, {'fun_name': fun_name, 'entity': '_mm'},
        ).to_ctypes()
        _mm_py = _b_utils._EngineIntScalarType.to_py(_mm)
    else:
        _mm = _b_utils._EngineIntScalarType(
            _n_py, {'fun_name': fun_name, 'entity': '_mm'},
        ).to_ctypes()
        _mm_py = _b_utils._EngineIntScalarType.to_py(_mm)
    _job = _b_utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _job_py = _b_utils._EngineCharScalarType.to_py(_job)
    if _job_py not in ('V', 'v'):
        if _job_py in ('E', 'e', 'B', 'b'):
            _s_dim1 = _mm_py
        else:
            _s_dim1 = 0
        exp_shape_s = (_s_dim1,)
        _s = _b_utils._EngineFloat64ArrayType.empty_ndarray(
            exp_shape_s,
        )
    else:
        _s = None
    if _job_py not in ('E', 'e'):
        if _job_py in ('V', 'v', 'B', 'b'):
            _sep_dim1 = _mm_py
        else:
            _sep_dim1 = 1
        exp_shape_sep = (_sep_dim1,)
        _sep = _b_utils._EngineFloat64ArrayType.empty_ndarray(
            exp_shape_sep,
        )
    else:
        _sep = None
    _return_scalar = _b_lapackeig.dtrsna(
        job,
        howmny,
        _select,
        _n,
        _t,
        _vl,
        _vr,
        _s,
        _sep,
        _mm,
    )
    if _job_py not in ('V', 'v'):
        _s_py = _s
    else:
        _s_py = None
    if _job_py not in ('E', 'e'):
        _sep_py = _sep
    else:
        _sep_py = None
    _m_py = _return_scalar
    _return_data = DtrsnaReturnData(
        _s_py,
        _sep_py,
        _m_py,
    )
    return _return_data

ZtrexcReturnData = _collections.namedtuple(
    'ZtrexcReturnData',
    ['t', 'q'],
)

def ztrexc(compq, t, q, ifst, ilst):
    u"""
    Reorders a Schur factorization of complex matrix using unitary
    similarity transformation.

    ``ztrexc`` reorders the Schur factorization of a complex general
    matrix.

    For full information please refer to the NAG Library document for
    f08qt

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08qtf.html

    Parameters
    ----------
    compq : str, length 1
        Indicates whether the matrix Q of Schur vectors is to be
        updated.

        `compq` = 'V'

            The matrix Q of Schur vectors is updated.

        `compq` = 'N'

            No Schur vectors are updated.

    t : complex, array-like, shape (n, n)
        The n*n upper triangular matrix T, as returned by ``zhseqr``.

    q : complex, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `compq`='V': n; if `compq`='N': 1;
        otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compq`='V': n; if `compq`='N': 1;
        otherwise: 0.

        If `compq` = 'V', `q` must contain the n*n unitary matrix Q of
        Schur vectors.

    ifst : int
        `ifst` and `ilst` must specify the reordering of the diagonal
        elements of T. The element with row index `ifst` is moved to row
        `ilst` by a sequence of exchanges between adjacent elements.

    ilst : int
        `ifst` and `ilst` must specify the reordering of the diagonal
        elements of T. The element with row index `ifst` is moved to row
        `ilst` by a sequence of exchanges between adjacent elements.

    Returns
    -------
    t : complex, ndarray, shape (n, n)
        `t` is overwritten by the updated matrix T~.

    q : complex, ndarray, shape (:, :)
        If `compq` = 'V', `q` contains the updated matrix of Schur
        vectors.

        If `compq` = 'N', `q` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `compq`.

            Constraint: `compq` = 'V' or 'N'.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -7)
            On entry, error in parameter `ifst`.

            Constraint: 1 <= `ifst` <= n.

        (`errno` -8)
            On entry, error in parameter `ilst`.

            Constraint: 1 <= `ilst` <= n.

    Notes
    -----
    ``ztrexc`` reorders the Schur factorization of a complex general
    matrix A = QTQ^H, so that the diagonal element of T with row index
    `ifst` is moved to row `ilst`.

    The reordered Schur form T~ is computed by a unitary similarity
    transformation: T~ = Z^HTZ.
    Optionally the updated matrix Q~ of Schur vectors is computed as Q~
    = QZ, giving A = Q~T~Q~^H.

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.ztrexc'

    _t = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 't'},
        t,
        copy=True,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _q = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'q'},
        q,
        copy=True,
    )
    _b_lapackeig.ztrexc(
        compq,
        _n,
        _t,
        _q,
        ifst,
        ilst,
    )
    _t_py = _t
    _q_py = _q
    _return_data = ZtrexcReturnData(
        _t_py,
        _q_py,
    )
    return _return_data

ZtrsenReturnData = _collections.namedtuple(
    'ZtrsenReturnData',
    ['t', 'q', 'w', 'm', 's', 'sep'],
)

def ztrsen(job, compq, select, t, q):
    u"""
    Reorders a Schur factorization of complex matrix, form orthonormal
    basis of right invariant subspace for selected eigenvalues, with
    estimates of sensitivities.

    ``ztrsen`` reorders the Schur factorization of a complex general
    matrix so that a selected cluster of eigenvalues appears in the
    leading elements on the diagonal of the Schur form.
    The function also optionally computes the reciprocal condition
    numbers of the cluster of eigenvalues and/or the invariant subspace.

    For full information please refer to the NAG Library document for
    f08qu

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08quf.html

    Parameters
    ----------
    job : str, length 1
        Indicates whether condition numbers are required for the cluster
        of eigenvalues and/or the invariant subspace.

        `job` = 'N'

            No condition numbers are required.

        `job` = 'E'

            Only the condition number for the cluster of eigenvalues is
            computed.

        `job` = 'V'

            Only the condition number for the invariant subspace is
            computed.

        `job` = 'B'

            Condition numbers for both the cluster of eigenvalues and
            the invariant subspace are computed.

    compq : str, length 1
        Indicates whether the matrix Q of Schur vectors is to be
        updated.

        `compq` = 'V'

            The matrix Q of Schur vectors is updated.

        `compq` = 'N'

            No Schur vectors are updated.

    select : bool, array-like, shape (n)
        Specifies the eigenvalues in the selected cluster. To select a
        complex eigenvalue lambda_j, `select`[j-1] must be set True.

    t : complex, array-like, shape (n, n)
        The n*n upper triangular matrix T, as returned by ``zhseqr``.

    q : complex, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `compq`='V': n; if `compq`='N': 1;
        otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compq`='V': n; if `compq`='N': 1;
        otherwise: 0.

        If `compq` = 'V', `q` must contain the n*n unitary matrix Q of
        Schur vectors, as returned by ``zhseqr``.

    Returns
    -------
    t : complex, ndarray, shape (n, n)
        `t` is overwritten by the updated matrix T~.

    q : complex, ndarray, shape (:, :)
        If `compq` = 'V', `q` contains the updated matrix of Schur
        vectors; the first m columns of Q form an orthonormal basis for
        the specified invariant subspace.

        If `compq` = 'N', `q` is not referenced.

    w : complex, ndarray, shape (n)
        The reordered eigenvalues of T~. The eigenvalues are stored in
        the same order as on the diagonal of T~.

    m : int
        m, the dimension of the specified invariant subspace, which is
        the same as the number of selected eigenvalues (see `select`); 0
        <= m <= n.

    s : float
        If `job` = 'E' or 'B', `s` is a lower bound on the reciprocal
        condition number of the average of the selected cluster of
        eigenvalues. If `m` = 0 or n, `s` = 1.

        If `job` = 'N' or 'V', `s` is not referenced.

    sep : float
        If `job` = 'V' or 'B', `sep` is the estimated reciprocal
        condition number of the specified invariant subspace. If `m` = 0
        or n, `sep` = ||T||.

        If `job` = 'N' or 'E', `sep` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'N', 'E', 'V' or 'B'.

        (`errno` -2)
            On entry, error in parameter `compq`.

            Constraint: `compq` = 'V' or 'N'.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n >= 0.

    Notes
    -----
    ``ztrsen`` reorders the Schur factorization of a complex general
    matrix A = QTQ^H, so that a selected cluster of eigenvalues appears
    in the leading diagonal elements of the Schur form.

    The reordered Schur form T~ is computed by a unitary similarity
    transformation: T~ = Z^HTZ.
    Optionally the updated matrix Q~ of Schur vectors is computed as Q~
    = QZ, giving A = Q~T~Q~^H.

    Let T~ = [table omitted], where the selected eigenvalues are
    precisely the eigenvalues of the leading m*m sub-matrix T_11.
    Let Q~ be correspondingly partitioned as (Q_1 Q_2) where Q_1
    consists of the first m columns of Q.
    Then AQ_1 = Q_1 T_11, and so the m columns of Q_1 form an
    orthonormal basis for the invariant subspace corresponding to the
    selected cluster of eigenvalues.

    Optionally the function also computes estimates of the reciprocal
    condition numbers of the average of the cluster of eigenvalues and
    of the invariant subspace.

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.ztrsen'

    _select = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'select'},
        select,
        copy=False,
    )
    _t = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 't'},
        t,
        copy=True,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _q = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'q'},
        q,
        copy=True,
    )
    _w = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_n_py,),
    )
    _return_scalars = _b_lapackeig.ztrsen(
        job,
        compq,
        _select,
        _n,
        _t,
        _q,
        _w,
    )
    _t_py = _t
    _q_py = _q
    _w_py = _w
    _m_py = _return_scalars.m
    _s_py = _return_scalars.s
    _sep_py = _return_scalars.sep
    _return_data = ZtrsenReturnData(
        _t_py,
        _q_py,
        _w_py,
        _m_py,
        _s_py,
        _sep_py,
    )
    return _return_data

ZtrsylReturnData = _collections.namedtuple(
    'ZtrsylReturnData',
    ['c', 'scal'],
)

def ztrsyl(trana, tranb, isgn, a, b, c):
    u"""
    Solves the complex Sylvester matrix equation AX+XB = C, A and B are
    upper triangular or conjugate-transposes.

    ``ztrsyl`` solves the complex triangular Sylvester matrix equation.

    For full information please refer to the NAG Library document for
    f08qv

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08qvf.html

    Parameters
    ----------
    trana : str, length 1
        Specifies the option op(A).

        `trana` = 'N'

            op(A) = A.

        `trana` = 'C'

            op(A) = A^H.

    tranb : str, length 1
        Specifies the option op(B).

        `tranb` = 'N'

            op(B) = B.

        `tranb` = 'C'

            op(B) = B^H.

    isgn : int
        Indicates the form of the Sylvester equation.

        `isgn` = +1

            The equation is of the form op(A)X+Xop(B) = alpha C.

        `isgn` = -1

            The equation is of the form op(A)X-Xop(B) = alpha C.

    a : complex, array-like, shape (m, m)
        The m*m upper triangular matrix A.

    b : complex, array-like, shape (n, n)
        The n*n upper triangular matrix B.

    c : complex, array-like, shape (m, n)
        The m*n right-hand side matrix C.

    Returns
    -------
    c : complex, ndarray, shape (m, n)
        `c` is overwritten by the solution matrix X.

    scal : float
        The value of the scale factor alpha.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `trana`.

            Constraint: `trana` = 'N' or 'C'.

        (`errno` -2)
            On entry, error in parameter `tranb`.

            Constraint: `tranb` = 'N' or 'C'.

        (`errno` -3)
            On entry, error in parameter `isgn`.

            Constraint: `isgn` = +1 or -1.

        (`errno` -4)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -5)
            On entry, error in parameter n.

            Constraint: n >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            A and B have common or close eigenvalues, perturbed values
            of which were used to solve the equation.

    Notes
    -----
    ``ztrsyl`` solves the complex Sylvester matrix equation

        op(A)X+/-Xop(B) = alpha C,

    where op(A) = A or A^H, and the matrices A and B are upper
    triangular; alpha is a scale factor (<= 1) determined by the
    function to avoid overflow in X; A is m*m and B is n*n while the
    right-hand side matrix C and the solution matrix X are both m*n.
    The matrix X is obtained by a straightforward process of
    back-substitution (see Golub and Van Loan (1996)).

    Note that the equation has a unique solution if and only if
    alpha_i+/-beta_j != 0, where {alpha_i} and {beta_j} are the
    eigenvalues of A and B respectively and the sign (+ or -) is the
    same as that used in the equation to be solved.

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore

    Higham, N J, 1992, `Perturbation theory and backward error for AX-XB
    = C`, Numerical Analysis Report, University of Manchester
    """

    fun_name = 'naginterfaces.library.lapackeig.ztrsyl'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=False,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=True,
    )
    try:
        _return_scalar = _b_lapackeig.ztrsyl(
            trana,
            tranb,
            isgn,
            _m,
            _n,
            _a,
            _b,
            _c,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _c_py = _c
    _scal_py = _return_scalar
    _return_data = ZtrsylReturnData(
        _c_py,
        _scal_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

ZtrevcReturnData = _collections.namedtuple(
    'ZtrevcReturnData',
    ['t', 'vl', 'vr', 'm'],
)

def ztrevc(job, howmny, t, select=None, vl=None, vr=None):
    u"""
    Computes left and right eigenvectors of complex upper triangular
    matrix.

    ``ztrevc`` computes selected left and/or right eigenvectors of a
    complex upper triangular matrix.

    For full information please refer to the NAG Library document for
    f08qx

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08qxf.html

    Parameters
    ----------
    job : str, length 1
        Indicates whether left and/or right eigenvectors are to be
        computed.

        `job` = 'R'

            Only right eigenvectors are computed.

        `job` = 'L'

            Only left eigenvectors are computed.

        `job` = 'B'

            Both left and right eigenvectors are computed.

    howmny : str, length 1
        Indicates how many eigenvectors are to be computed.

        `howmny` = 'A'

            All eigenvectors (as specified by `job`) are computed.

        `howmny` = 'B'

            All eigenvectors (as specified by `job`) are computed and
            then pre-multiplied by the matrix Q (which is overwritten).

        `howmny` = 'S'

            Selected eigenvectors (as specified by `job` and `select`)
            are computed.

    t : complex, array-like, shape (n, n)
        The n*n upper triangular matrix T, as returned by ``zhseqr``.

    select : None or bool, array-like, shape (:), optional
        Note: the required length for this argument is determined as
        follows: if `howmny`='S': n; otherwise: 1.

        Specifies which eigenvectors are to be computed if `howmny` =
        'S'. To obtain the eigenvector corresponding to the eigenvalue
        lambda_j, `select`[j-1] must be set True.

        If `howmny` = 'A' or 'B', `select` is not referenced.

    vl : None or complex, array-like, shape (:, :), optional
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `job` in ('L', 'B'): n; if `job`='R':
        1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `job` in ('L', 'B'): mm; otherwise: 0.

        If `howmny` = 'B' and `job` = 'L' or 'B', `vl` must contain an
        n*n matrix Q (usually the matrix of Schur vectors returned by
        ``zhseqr``).

        If `howmny` = 'A' or 'S', `vl` need not be set.

    vr : None or complex, array-like, shape (:, :), optional
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `job` in ('R', 'B'): n; if `job`='L':
        1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `job` in ('R', 'B'): mm; otherwise: 0.

        If `howmny` = 'B' and `job` = 'R' or 'B', `vr` must contain an
        n*n matrix Q (usually the matrix of Schur vectors returned by
        ``zhseqr``).

        If `howmny` = 'A' or 'S', `vr` need not be set.

    Returns
    -------
    t : complex, ndarray, shape (n, n)
        Is used as internal workspace prior to being restored and hence
        is unchanged.

    vl : None or complex, ndarray, shape (:, :)
        If `job` = 'L' or 'B', `vl` contains the computed left
        eigenvectors (as specified by `howmny` and `select`). The
        eigenvectors are stored consecutively in the columns of the
        array, in the same order as their eigenvalues.

        If `job` = 'R', `vl` is not referenced.

    vr : None or complex, ndarray, shape (:, :)
        If `job` = 'R' or 'B', `vr` contains the computed right
        eigenvectors (as specified by `howmny` and `select`). The
        eigenvectors are stored consecutively in the columns of the
        array, in the same order as their eigenvalues.

        If `job` = 'L', `vr` is not referenced.

    m : int
        m, the number of selected eigenvectors. If `howmny` = 'A' or
        'B', `m` is set to n.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'R', 'L' or 'B'.

        (`errno` -2)
            On entry, error in parameter `howmny`.

            Constraint: `howmny` = 'A', 'B', 'O' or 'S'.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -11)
            On entry, error in parameter mm.

            Constraint: mm >= n.

        (`errno` -11)
            On entry, error in parameter mm.

            Constraint: mm >= m.

    Notes
    -----
    ``ztrevc`` computes left and/or right eigenvectors of a complex
    upper triangular matrix T.
    Such a matrix arises from the Schur factorization of a complex
    general matrix, as computed by ``zhseqr``, for example.

    The right eigenvector x, and the left eigenvector y, corresponding
    to an eigenvalue lambda, are defined by:

        Tx = lambda x and y^HT = lambda y^H(or T^Hy = lambda_y).

    The function can compute the eigenvectors corresponding to selected
    eigenvalues, or it can compute all the eigenvectors.
    In the latter case the eigenvectors may optionally be pre-multiplied
    by an input matrix Q.
    Normally Q is a unitary matrix from the Schur factorization of a
    matrix A as A = QTQ^H; if x is a (left or right) eigenvector of T,
    then Qx is an eigenvector of A.

    The eigenvectors are computed by forward or backward substitution.
    They are scaled so that max(|Re(x_i)|+|Im(x_i)|) = 1.

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.ztrevc'

    _t = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 't'},
        t,
        copy=True,
    )
    _vl = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'vl'},
        vl,
        copy=True,
        allow_none=True,
    )
    _vr = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'vr'},
        vr,
        copy=True,
        allow_none=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_t, 't'), (_vl, 'vl'), (_vr, 'vr')),
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _howmny = _b_utils._EngineCharScalarType(
        howmny, {'fun_name': fun_name, 'entity': 'howmny'}, exp_chrlen=1,
    ).to_ctypes()
    _howmny_py = _b_utils._EngineCharScalarType.to_py(_howmny)
    _select = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'select'},
        select,
        copy=False,
        allow_none=True,
    )
    if _howmny_py in ('S', 's'):
        _vl_m_py = 0
        for _vl_j_py in range(1, _n_py+1):
            if _b_utils._EngineLogicalArrayType.get_element(_select, {'fun_name': fun_name, 'entity': 'select'}, (_vl_j_py-1,)):
                _vl_m_py = _vl_m_py + 1
    else:
        _vl_m_py = _n_py
    _job = _b_utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _job_py = _b_utils._EngineCharScalarType.to_py(_job)
    if (_howmny_py in ('B', 'b')) and _job_py not in ('R', 'r'):
        if vl is None:
            raise _b_utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'vl'},
                'must not be None when (howmny in (\'B\', \'b\')) and job not in (\'R\', \'r\')',
            )
    if _howmny_py in ('S', 's'):
        _mm = _b_utils._EngineIntScalarType(
            _vl_m_py, {'fun_name': fun_name, 'entity': '_mm'},
        ).to_ctypes()
        _mm_py = _b_utils._EngineIntScalarType.to_py(_mm)
    else:
        _mm = _b_utils._EngineIntScalarType(
            _n_py, {'fun_name': fun_name, 'entity': '_mm'},
        ).to_ctypes()
        _mm_py = _b_utils._EngineIntScalarType.to_py(_mm)
    if vl is None:
        if _job_py in ('L', 'l', 'B', 'b'):
            _vl_dim1 = _n_py
        elif _job_py in ('R', 'r'):
            _vl_dim1 = 1
        else:
            _vl_dim1 = 0
        if _job_py in ('L', 'l', 'B', 'b'):
            _vl_dim2 = _mm_py
        else:
            _vl_dim2 = 0
        exp_shape_vl = (_vl_dim1, _vl_dim2)
        _vl = _b_utils._EngineComplex128ArrayType.empty_ndarray(
            exp_shape_vl,
            sorder=_m_sorder,
        )
    if (_howmny_py in ('B', 'b')) and _job_py not in ('L', 'l'):
        if vr is None:
            raise _b_utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'vr'},
                'must not be None when (howmny in (\'B\', \'b\')) and job not in (\'L\', \'l\')',
            )
    if vr is None:
        if _job_py in ('R', 'r', 'B', 'b'):
            _vr_dim1 = _n_py
        elif _job_py in ('L', 'l'):
            _vr_dim1 = 1
        else:
            _vr_dim1 = 0
        if _job_py in ('R', 'r', 'B', 'b'):
            _vr_dim2 = _mm_py
        else:
            _vr_dim2 = 0
        exp_shape_vr = (_vr_dim1, _vr_dim2)
        _vr = _b_utils._EngineComplex128ArrayType.empty_ndarray(
            exp_shape_vr,
            sorder=_m_sorder,
        )
    _return_scalar = _b_lapackeig.ztrevc(
        job,
        howmny,
        _select,
        _n,
        _t,
        _vl,
        _vr,
        _mm,
    )
    _t_py = _t
    if _job_py not in ('R', 'r'):
        _vl_py = _vl
    else:
        _vl_py = None
    if _job_py not in ('L', 'l'):
        _vr_py = _vr
    else:
        _vr_py = None
    _m_py = _return_scalar
    _return_data = ZtrevcReturnData(
        _t_py,
        _vl_py,
        _vr_py,
        _m_py,
    )
    return _return_data

ZtrsnaReturnData = _collections.namedtuple(
    'ZtrsnaReturnData',
    ['s', 'sep', 'm'],
)

def ztrsna(job, howmny, t, select=None, vl=None, vr=None):
    u"""
    Computes estimates of sensitivities of selected eigenvalues and
    eigenvectors of complex upper triangular matrix.

    ``ztrsna`` estimates condition numbers for specified eigenvalues
    and/or right eigenvectors of a complex upper triangular matrix.

    For full information please refer to the NAG Library document for
    f08qy

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08qyf.html

    Parameters
    ----------
    job : str, length 1
        Indicates whether condition numbers are required for eigenvalues
        and/or eigenvectors.

        `job` = 'E'

            Condition numbers for eigenvalues only are computed.

        `job` = 'V'

            Condition numbers for eigenvectors only are computed.

        `job` = 'B'

            Condition numbers for both eigenvalues and eigenvectors are
            computed.

    howmny : str, length 1
        Indicates how many condition numbers are to be computed.

        `howmny` = 'A'

            Condition numbers for all eigenpairs are computed.

        `howmny` = 'S'

            Condition numbers for selected eigenpairs (as specified by
            `select`) are computed.

    t : complex, array-like, shape (n, n)
        The n*n upper triangular matrix T, as returned by ``zhseqr``.

    select : None or bool, array-like, shape (:), optional
        Note: the required length for this argument is determined as
        follows: if `howmny`='S': n; otherwise: 1.

        Specifies the eigenpairs for which condition numbers are to be
        computed if `howmny` = 'S'. To select condition numbers for the
        eigenpair corresponding to the eigenvalue lambda_j,
        `select`[j-1] must be set to True.

        If `howmny` = 'A', `select` is not referenced.

    vl : None or complex, array-like, shape (:, :), optional
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `job` in ('E', 'B'): n; if `job`='V':
        1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `job` in ('E', 'B'): mm; otherwise: 0.

        If `job` = 'E' or 'B', `vl` must contain the left eigenvectors
        of T (or of any matrix QTQ^H with Q unitary) corresponding to
        the eigenpairs specified by `howmny` and `select`. The
        eigenvectors **must** be stored in consecutive columns of `vl`,
        as returned by ``ztrevc`` or ``zhsein``.

        If `job` = 'V', `vl` is not referenced.

    vr : None or complex, array-like, shape (:, :), optional
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `job` in ('E', 'B'): n; if `job`='V':
        1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `job` in ('E', 'B'): mm; otherwise: 0.

        If `job` = 'E' or 'B', `vr` must contain the right eigenvectors
        of T (or of any matrix QTQ^H with Q unitary) corresponding to
        the eigenpairs specified by `howmny` and `select`. The
        eigenvectors **must** be stored in consecutive columns of `vr`,
        as returned by ``ztrevc`` or ``zhsein``.

        If `job` = 'V', `vr` is not referenced.

    Returns
    -------
    s : None or float, ndarray, shape (:)
        The reciprocal condition numbers of the selected eigenvalues if
        `job` = 'E' or 'B', stored in consecutive elements of the array.
        Thus `s`[j-1], `sep`[j-1] and the jth columns of `vl` and `vr`
        all correspond to the same eigenpair (but not in general the jth
        eigenpair unless all eigenpairs have been selected).

        If `job` = 'V', `s` is not referenced.

    sep : None or float, ndarray, shape (:)
        The estimated reciprocal condition numbers of the selected right
        eigenvectors if `job` = 'V' or 'B', stored in consecutive
        elements of the array.

        If `job` = 'E', `sep` is not referenced.

    m : int
        m, the number of selected eigenpairs. If `howmny` = 'A', `m` is
        set to n.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'E', 'V' or 'B'.

        (`errno` -2)
            On entry, error in parameter `howmny`.

            Constraint: `howmny` = 'A' or 'S'.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -13)
            On entry, error in parameter mm.

            Constraint: mm >= n.

        (`errno` -13)
            On entry, error in parameter mm.

            Constraint: mm >= m.

    Notes
    -----
    ``ztrsna`` estimates condition numbers for specified eigenvalues
    and/or right eigenvectors of a complex upper triangular matrix T.
    These are the same as the condition numbers of the eigenvalues and
    right eigenvectors of an original matrix A = ZTZ^H (with unitary Z),
    from which T may have been derived.

    ``ztrsna`` computes the reciprocal of the condition number of an
    eigenvalue lambda_i as

        s_i = |v^Hu|/||u||_E||v||_E,

    where u and v are the right and left eigenvectors of T,
    respectively, corresponding to lambda_i.
    This reciprocal condition number always lies between zero (i.e.,
    ill-conditioned) and one (i.e., well-conditioned).

    An approximate error estimate for a computed eigenvalue lambda_i is
    then given by

        epsilon||T||/(s_i),

    where epsilon is the machine precision.

    To estimate the reciprocal of the condition number of the right
    eigenvector corresponding to lambda_i, the function first calls
    ``ztrexc`` to reorder the eigenvalues so that lambda_i is in the
    leading position:

        T = Q[table omitted]Q^H.

    The reciprocal condition number of the eigenvector is then estimated
    as sep_i, the smallest singular value of the matrix (T_22-lambda_i
    I).
    This number ranges from zero (i.e., ill-conditioned) to very large
    (i.e., well-conditioned).

    An approximate error estimate for a computed right eigenvector u
    corresponding to lambda_i is then given by

        epsilon||T||/(sep_i).

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.ztrsna'

    _t = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 't'},
        t,
        copy=False,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _howmny = _b_utils._EngineCharScalarType(
        howmny, {'fun_name': fun_name, 'entity': 'howmny'}, exp_chrlen=1,
    ).to_ctypes()
    _howmny_py = _b_utils._EngineCharScalarType.to_py(_howmny)
    _select = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'select'},
        select,
        copy=False,
        allow_none=True,
    )
    if _howmny_py in ('S', 's'):
        _vl_m_py = 0
        for _vl_j_py in range(1, _n_py+1):
            if _b_utils._EngineLogicalArrayType.get_element(_select, {'fun_name': fun_name, 'entity': 'select'}, (_vl_j_py-1,)):
                _vl_m_py = _vl_m_py + 1
    else:
        _vl_m_py = _n_py
    _vl = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'vl'},
        vl,
        copy=False,
        allow_none=True,
    )
    _vr = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'vr'},
        vr,
        copy=False,
        allow_none=True,
    )
    if _howmny_py in ('S', 's'):
        _mm = _b_utils._EngineIntScalarType(
            _vl_m_py, {'fun_name': fun_name, 'entity': '_mm'},
        ).to_ctypes()
        _mm_py = _b_utils._EngineIntScalarType.to_py(_mm)
    else:
        _mm = _b_utils._EngineIntScalarType(
            _n_py, {'fun_name': fun_name, 'entity': '_mm'},
        ).to_ctypes()
        _mm_py = _b_utils._EngineIntScalarType.to_py(_mm)
    _job = _b_utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _job_py = _b_utils._EngineCharScalarType.to_py(_job)
    if _job_py not in ('V', 'v'):
        if _job_py in ('E', 'e', 'B', 'b'):
            _s_dim1 = _mm_py
        else:
            _s_dim1 = 0
        exp_shape_s = (_s_dim1,)
        _s = _b_utils._EngineFloat64ArrayType.empty_ndarray(
            exp_shape_s,
        )
    else:
        _s = None
    if _job_py not in ('E', 'e'):
        if _job_py in ('V', 'v', 'B', 'b'):
            _sep_dim1 = _mm_py
        else:
            _sep_dim1 = 1
        exp_shape_sep = (_sep_dim1,)
        _sep = _b_utils._EngineFloat64ArrayType.empty_ndarray(
            exp_shape_sep,
        )
    else:
        _sep = None
    _return_scalar = _b_lapackeig.ztrsna(
        job,
        howmny,
        _select,
        _n,
        _t,
        _vl,
        _vr,
        _s,
        _sep,
        _mm,
    )
    if _job_py not in ('V', 'v'):
        _s_py = _s
    else:
        _s_py = None
    if _job_py not in ('E', 'e'):
        _sep_py = _sep
    else:
        _sep_py = None
    _m_py = _return_scalar
    _return_data = ZtrsnaReturnData(
        _s_py,
        _sep_py,
        _m_py,
    )
    return _return_data

DorcsdReturnData = _collections.namedtuple(
    'DorcsdReturnData',
    ['x11', 'x12', 'x21', 'x22', 'theta', 'u1', 'u2', 'v1t', 'v2t'],
)

def dorcsd(m, p, q, x11, x12, x21, x22, jobu1='Y', jobu2='Y', jobv1t='Y', jobv2t='Y', trans='N', signs='D'):
    u"""
    Computes the CS decomposition of an orthogonal matrix partitioned
    into four real submatrices.

    ``dorcsd`` computes the CS decomposition of a real m*m orthogonal
    matrix X, partitioned into a 2*2 array of submatrices.

    For full information please refer to the NAG Library document for
    f08ra

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08raf.html

    Parameters
    ----------
    m : int
        m, the number of rows and columns in the orthogonal matrix X.

    p : int
        p, the number of rows in X_{11} and X_{12}.

    q : int
        q, the number of columns in X_{11} and X_{21}.

    x11 : float, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `trans`='T': `q`; otherwise: `p`.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `trans`='T': `p`; otherwise: `q`.

        The upper left partition of the orthogonal matrix X whose CSD is
        desired.

    x12 : float, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `trans`='T': `m`-`q`; otherwise: `p`.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `trans`='T': `p`; otherwise: `m`-`q`.

        The upper right partition of the orthogonal matrix X whose CSD
        is desired.

    x21 : float, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `trans`='T': `q`; otherwise: `m`-`p`.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `trans`='T': `m`-`p`; otherwise: `q`.

        The lower left partition of the orthogonal matrix X whose CSD is
        desired.

    x22 : float, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `trans`='T': `m`-`q`; otherwise:
        `m`-`p`.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `trans`='T': `m`-`p`; otherwise:
        `m`-`q`.

        The lower right partition of the orthogonal matrix X CSD is
        desired.

    jobu1 : str, length 1, optional
            If `jobu1` = 'Y', U_1 is computed;

            otherwise, U_1 is not computed.

    jobu2 : str, length 1, optional
            If `jobu2` = 'Y', U_2 is computed;

            otherwise, U_2 is not computed.

    jobv1t : str, length 1, optional
            If `jobv1t` = 'Y', V_1^T is computed;

            otherwise, V_1^T is not computed.

    jobv2t : str, length 1, optional
            If `jobv2t` = 'Y', V_2^T is computed;

            otherwise, V_2^T is not computed.

    trans : str, length 1, optional
            If `trans` = 'T', X, U_1, U_2, V_1^T and V_2^T are stored in
            row-major order;

            otherwise, X, U_1, U_2, V_1^T and V_2^T are stored in
            column-major order.

    signs : str, length 1, optional
            If `signs` = 'O', the lower-left block is made nonpositive
            (the other convention);

            otherwise, the upper-right block is made nonpositive (the
            default convention).

    Returns
    -------
    x11 : float, ndarray, shape (:, :)
        Contains details of the orthogonal matrix used in a simultaneous
        bidiagonalization process.

    x12 : float, ndarray, shape (:, :)
        Contains details of the orthogonal matrix used in a simultaneous
        bidiagonalization process.

    x21 : float, ndarray, shape (:, :)
        Contains details of the orthogonal matrix used in a simultaneous
        bidiagonalization process.

    x22 : float, ndarray, shape (:, :)
        Contains details of the orthogonal matrix used in a simultaneous
        bidiagonalization process.

    theta : float, ndarray, shape (min(`p`, `m`-`p`, `q`, `m`-`q`))
        The values theta_i for i = 1,2,...,r where r = min(p, m-p, q,
        m-q). The diagonal submatrices C and S of Sigma_p are
        constructed from these values as

            C = diag(cos(`theta`[0]), ..., cos(`theta`[r-1])) and

            S = diag(sin(`theta`[0]), ..., sin(`theta`[r-1])).

    u1 : None or float, ndarray, shape (:, :)
        If `jobu1` = 'Y', `u1` contains the p*p orthogonal matrix U_1.

    u2 : None or float, ndarray, shape (:, :)
        If `jobu2` = 'Y', `u2` contains the m-p*m-p orthogonal matrix
        U_2.

    v1t : None or float, ndarray, shape (:, :)
        If `jobv1t` = 'Y', `v1t` contains the q*q orthogonal matrix
        V_1^T.

    v2t : None or float, ndarray, shape (:, :)
        If `jobv2t` = 'Y', `v2t` contains the m-q*m-q orthogonal matrix
        V_2^T.

    Raises
    ------
    NagValueError
        (`errno` -7)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -8)
            On entry, error in parameter `p`.

            Constraint: 0 <= `p` <= `m`.

        (`errno` -9)
            On entry, error in parameter `q`.

            Constraint: 0 <= `q` <= `m`.

        (`errno` i > 0)
            The Jacobi-type procedure failed to converge during an
            internal reduction to bidiagonal-block form. The process
            requires convergence to min(`p`, `m`-`p`, `q`, `m`-`q`)
            values, the value of errno gives the number of converged
            values.

    Notes
    -----
    The m*m orthogonal matrix X is partitioned as

        X = [table omitted]

    where X_{11} is a p*q submatrix and the dimensions of the other
    submatrices X_{12}, X_{21} and X_{22} are such that X remains m*m.

    The CS decomposition of X is X = U Sigma_p V^T where U, V and
    Sigma_p are m*m matrices, such that

        U = [table omitted]

    is an orthogonal matrix containing the p*p orthogonal matrix U_1 and
    the (m-p)*(m-p) orthogonal matrix U_2;

        V = [table omitted]

    is an orthogonal matrix containing the q*q orthogonal matrix V_1 and
    the (m-q)*(m-q) orthogonal matrix V_2; and

        Sigma_p = [table omitted]

    contains the r*r non-negative diagonal submatrices C and S
    satisfying C^2+S^2 = I, where r = min(p, m-p, q, m-q) and the top
    left partition is p*q.

    The identity matrix I_{11} is of order min(p, q)-r and vanishes if
    min(p, q) = r.

    The identity matrix I_{12} is of order min(p, m-q)-r and vanishes if
    min(p, m-q) = r.

    The identity matrix I_{21} is of order min(m-p, q)-r and vanishes if
    min(m-p, q) = r.

    The identity matrix I_{22} is of order min(m-p, m-q)-r and vanishes
    if min(m-p, m-q) = r.

    In each of the four cases r = p,q,m-p,m-q at least two of the
    identity matrices vanish.

    The indicated zeros represent augmentations by additional rows or
    columns (but not both) to the square diagonal matrices formed by
    I_{ij} and C or S.

    Sigma_p does not need to be stored in full; it is sufficient to
    return only the values theta_i for i = 1,2,...,r where C_{ii} =
    cos(theta_i) and S_{ii} = sin(theta_i).

    The algorithm used to perform the complete CS decomposition is
    described fully in Sutton (2009) including discussions of the
    stability and accuracy of the algorithm.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 2012, `Matrix Computations`, (4th
    Edition), Johns Hopkins University Press, Baltimore

    Sutton, B D, 2009, `Computing the complete CS decomposition`,
    Numerical Algorithms (Volume 50) (1017--1398), Springer US, 33--65,
    https://dx.doi.org/10.1007/s11075-008-9215-6
    """

    fun_name = 'naginterfaces.library.lapackeig.dorcsd'

    _x11 = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x11'},
        x11,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _x12 = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x12'},
        x12,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _x21 = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x21'},
        x21,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _x22 = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x22'},
        x22,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_x11, 'x11'), (_x12, 'x12'), (_x21, 'x21'), (_x22, 'x22')),
    )
    _p = _b_utils._EngineIntScalarType(
        p, {'fun_name': fun_name, 'entity': 'p'},
    ).to_ctypes()
    _p_py = _b_utils._EngineIntScalarType.to_py(_p)
    _m = _b_utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = _b_utils._EngineIntScalarType.to_py(_m)
    _q = _b_utils._EngineIntScalarType(
        q, {'fun_name': fun_name, 'entity': 'q'},
    ).to_ctypes()
    _q_py = _b_utils._EngineIntScalarType.to_py(_q)
    _theta = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (min(_p_py, _m_py-_p_py, _q_py, _m_py-_q_py),),
    )
    _jobu1 = _b_utils._EngineCharScalarType(
        jobu1, {'fun_name': fun_name, 'entity': 'jobu1'}, exp_chrlen=1,
    ).to_ctypes()
    _jobu1_py = _b_utils._EngineCharScalarType.to_py(_jobu1)
    if _jobu1_py not in ('N', 'n'):
        if _jobu1_py in ('Y', 'y'):
            _u1_dim1 = _p_py
        else:
            _u1_dim1 = 0
        if _jobu1_py in ('Y', 'y'):
            _u1_dim2 = _p_py
        else:
            _u1_dim2 = 1
        exp_shape_u1 = (_u1_dim1, _u1_dim2)
        _u1 = _b_utils._EngineFloat64ArrayType.empty_ndarray(
            exp_shape_u1,
            sorder=_m_sorder,
        )
    else:
        _u1 = None
    _jobu2 = _b_utils._EngineCharScalarType(
        jobu2, {'fun_name': fun_name, 'entity': 'jobu2'}, exp_chrlen=1,
    ).to_ctypes()
    _jobu2_py = _b_utils._EngineCharScalarType.to_py(_jobu2)
    if _jobu2_py not in ('N', 'n'):
        if _jobu2_py in ('Y', 'y'):
            _u2_dim1 = _m_py-_p_py
        else:
            _u2_dim1 = 0
        if _jobu2_py in ('Y', 'y'):
            _u2_dim2 = _m_py-_p_py
        else:
            _u2_dim2 = 1
        exp_shape_u2 = (_u2_dim1, _u2_dim2)
        _u2 = _b_utils._EngineFloat64ArrayType.empty_ndarray(
            exp_shape_u2,
            sorder=_m_sorder,
        )
    else:
        _u2 = None
    _jobv1t = _b_utils._EngineCharScalarType(
        jobv1t, {'fun_name': fun_name, 'entity': 'jobv1t'}, exp_chrlen=1,
    ).to_ctypes()
    _jobv1t_py = _b_utils._EngineCharScalarType.to_py(_jobv1t)
    if _jobv1t_py not in ('N', 'n'):
        if _jobv1t_py in ('Y', 'y'):
            _v1t_dim1 = _q_py
        else:
            _v1t_dim1 = 0
        if _jobv1t_py in ('Y', 'y'):
            _v1t_dim2 = _q_py
        else:
            _v1t_dim2 = 1
        exp_shape_v1t = (_v1t_dim1, _v1t_dim2)
        _v1t = _b_utils._EngineFloat64ArrayType.empty_ndarray(
            exp_shape_v1t,
            sorder=_m_sorder,
        )
    else:
        _v1t = None
    _jobv2t = _b_utils._EngineCharScalarType(
        jobv2t, {'fun_name': fun_name, 'entity': 'jobv2t'}, exp_chrlen=1,
    ).to_ctypes()
    _jobv2t_py = _b_utils._EngineCharScalarType.to_py(_jobv2t)
    if _jobv2t_py not in ('N', 'n'):
        if _jobv2t_py in ('Y', 'y'):
            _v2t_dim1 = _m_py-_q_py
        else:
            _v2t_dim1 = 0
        if _jobv2t_py in ('Y', 'y'):
            _v2t_dim2 = _m_py-_q_py
        else:
            _v2t_dim2 = 1
        exp_shape_v2t = (_v2t_dim1, _v2t_dim2)
        _v2t = _b_utils._EngineFloat64ArrayType.empty_ndarray(
            exp_shape_v2t,
            sorder=_m_sorder,
        )
    else:
        _v2t = None
    _b_lapackeig.dorcsd(
        jobu1,
        jobu2,
        jobv1t,
        jobv2t,
        trans,
        signs,
        _m,
        _p,
        _q,
        _x11,
        _x12,
        _x21,
        _x22,
        _theta,
        _u1,
        _u2,
        _v1t,
        _v2t,
    )
    _x11_py = _x11
    _x12_py = _x12
    _x21_py = _x21
    _x22_py = _x22
    _theta_py = _theta
    if _jobu1_py not in ('N', 'n'):
        _u1_py = _u1
    else:
        _u1_py = None
    if _jobu2_py not in ('N', 'n'):
        _u2_py = _u2
    else:
        _u2_py = None
    if _jobv1t_py not in ('N', 'n'):
        _v1t_py = _v1t
    else:
        _v1t_py = None
    if _jobv2t_py not in ('N', 'n'):
        _v2t_py = _v2t
    else:
        _v2t_py = None
    _return_data = DorcsdReturnData(
        _x11_py,
        _x12_py,
        _x21_py,
        _x22_py,
        _theta_py,
        _u1_py,
        _u2_py,
        _v1t_py,
        _v2t_py,
    )
    return _return_data

ZuncsdReturnData = _collections.namedtuple(
    'ZuncsdReturnData',
    ['x11', 'x12', 'x21', 'x22', 'theta', 'u1', 'u2', 'v1t', 'v2t'],
)

def zuncsd(m, p, q, x11, x12, x21, x22, jobu1='Y', jobu2='Y', jobv1t='Y', jobv2t='Y', trans='N', signs='D'):
    u"""
    Computes the CS decomposition of a unitary matrix partitioned into
    four complex submatrices.

    ``zuncsd`` computes the CS decomposition of a complex m*m unitary
    matrix X, partitioned into a 2*2 array of submatrices.

    For full information please refer to the NAG Library document for
    f08rn

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08rnf.html

    Parameters
    ----------
    m : int
        m, the number of rows and columns in the unitary matrix X.

    p : int
        p, the number of rows in X_{11} and X_{12}.

    q : int
        q, the number of columns in X_{11} and X_{21}.

    x11 : complex, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `trans`='T': `q`; otherwise: `p`.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `trans`='T': `p`; otherwise: `q`.

        The upper left partition of the unitary matrix X whose CSD is
        desired.

    x12 : complex, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `trans`='T': `m`-`q`; otherwise: `p`.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `trans`='T': `p`; otherwise: `m`-`q`.

        The upper right partition of the unitary matrix X whose CSD is
        desired.

    x21 : complex, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `trans`='T': `q`; otherwise: `m`-`p`.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `trans`='T': `m`-`p`; otherwise: `q`.

        The lower left partition of the unitary matrix X whose CSD is
        desired.

    x22 : complex, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `trans`='T': `m`-`q`; otherwise:
        `m`-`p`.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `trans`='T': `m`-`p`; otherwise:
        `m`-`q`.

        The lower right partition of the unitary matrix X CSD is
        desired.

    jobu1 : str, length 1, optional
            If `jobu1` = 'Y', U_1 is computed;

            otherwise, U_1 is not computed.

    jobu2 : str, length 1, optional
            If `jobu2` = 'Y', U_2 is computed;

            otherwise, U_2 is not computed.

    jobv1t : str, length 1, optional
            If `jobv1t` = 'Y', V_1^T is computed;

            otherwise, V_1^T is not computed.

    jobv2t : str, length 1, optional
            If `jobv2t` = 'Y', V_2^T is computed;

            otherwise, V_2^T is not computed.

    trans : str, length 1, optional
            If `trans` = 'T', X, U_1, U_2, V_1^T and V_2^T are stored in
            row-major order;

            otherwise, X, U_1, U_2, V_1^T and V_2^T are stored in
            column-major order.

    signs : str, length 1, optional
            If `signs` = 'O', the lower-left block is made nonpositive
            (the other convention);

            otherwise, the upper-right block is made nonpositive (the
            default convention).

    Returns
    -------
    x11 : complex, ndarray, shape (:, :)
        Contains details of the unitary matrix used in a simultaneous
        bidiagonalization process.

    x12 : complex, ndarray, shape (:, :)
        Contains details of the unitary matrix used in a simultaneous
        bidiagonalization process.

    x21 : complex, ndarray, shape (:, :)
        Contains details of the unitary matrix used in a simultaneous
        bidiagonalization process.

    x22 : complex, ndarray, shape (:, :)
        Contains details of the unitary matrix used in a simultaneous
        bidiagonalization process.

    theta : float, ndarray, shape (min(`p`, `m`-`p`, `q`, `m`-`q`))
        The values theta_i for i = 1,2,...,r where r = min(p, m-p, q,
        m-q). The diagonal submatrices C and S of Sigma_p are
        constructed from these values as

            C = diag(cos(`theta`[0]), ..., cos(`theta`[r-1])) and

            S = diag(sin(`theta`[0]), ..., sin(`theta`[r-1])).

    u1 : None or complex, ndarray, shape (:, :)
        If `jobu1` = 'Y', `u1` contains the p*p unitary matrix U_1.

    u2 : None or complex, ndarray, shape (:, :)
        If `jobu2` = 'Y', `u2` contains the m-p*m-p unitary matrix U_2.

    v1t : None or complex, ndarray, shape (:, :)
        If `jobv1t` = 'Y', `v1t` contains the q*q unitary matrix V_1^H.

    v2t : None or complex, ndarray, shape (:, :)
        If `jobv2t` = 'Y', `v2t` contains the m-q*m-q unitary matrix
        V_2^H.

    Raises
    ------
    NagValueError
        (`errno` -7)
            On entry, error in parameter `m`.

            Constraint: `m` >= 0.

        (`errno` -8)
            On entry, error in parameter `p`.

            Constraint: 0 <= `p` <= `m`.

        (`errno` -9)
            On entry, error in parameter `q`.

            Constraint: 0 <= `q` <= `m`.

        (`errno` i > 0)
            The Jacobi-type procedure failed to converge during an
            internal reduction to bidiagonal-block form. The process
            requires convergence to min(`p`, `m`-`p`, `q`, `m`-`q`)
            values, the value of errno gives the number of converged
            values.

    Notes
    -----
    The m*m unitary matrix X is partitioned as

        X = [table omitted]

    where X_{11} is a p*q submatrix and the dimensions of the other
    submatrices X_{12}, X_{21} and X_{22} are such that X remains m*m.

    The CS decomposition of X is X = U Sigma_p V^T where U, V and
    Sigma_p are m*m matrices, such that

        U = [table omitted]

    is a unitary matrix containing the p*p unitary matrix U_1 and the
    (m-p)*(m-p) unitary matrix U_2;

        V = [table omitted]

    is a unitary matrix containing the q*q unitary matrix V_1 and the
    (m-q)*(m-q) unitary matrix V_2; and

        Sigma_p = [table omitted]

    contains the r*r non-negative diagonal submatrices C and S
    satisfying C^2+S^2 = I, where r = min(p, m-p, q, m-q) and the top
    left partition is p*q.

    The identity matrix I_{11} is of order min(p, q)-r and vanishes if
    min(p, q) = r.

    The identity matrix I_{12} is of order min(p, m-q)-r and vanishes if
    min(p, m-q) = r.

    The identity matrix I_{21} is of order min(m-p, q)-r and vanishes if
    min(m-p, q) = r.

    The identity matrix I_{22} is of order min(m-p, m-q)-r and vanishes
    if min(m-p, m-q) = r.

    In each of the four cases r = p,q,m-p,m-q at least two of the
    identity matrices vanish.

    The indicated zeros represent augmentations by additional rows or
    columns (but not both) to the square diagonal matrices formed by
    I_{ij} and C or S.

    Sigma_p does not need to be stored in full; it is sufficient to
    return only the values theta_i for i = 1,2,...,r where C_{ii} =
    cos(theta_i) and S_{ii} = sin(theta_i).

    The algorithm used to perform the complete CS decomposition is
    described fully in Sutton (2009) including discussions of the
    stability and accuracy of the algorithm.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 2012, `Matrix Computations`, (4th
    Edition), Johns Hopkins University Press, Baltimore

    Sutton, B D, 2009, `Computing the complete CS decomposition`,
    Numerical Algorithms (Volume 50) (1017--1398), Springer US, 33--65,
    https://dx.doi.org/10.1007/s11075-008-9215-6
    """

    fun_name = 'naginterfaces.library.lapackeig.zuncsd'

    _x11 = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x11'},
        x11,
        copy=True,
    )
    _x12 = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x12'},
        x12,
        copy=True,
    )
    _x21 = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x21'},
        x21,
        copy=True,
    )
    _x22 = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x22'},
        x22,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_x11, 'x11'), (_x12, 'x12'), (_x21, 'x21'), (_x22, 'x22')),
    )
    _p = _b_utils._EngineIntScalarType(
        p, {'fun_name': fun_name, 'entity': 'p'},
    ).to_ctypes()
    _p_py = _b_utils._EngineIntScalarType.to_py(_p)
    _m = _b_utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = _b_utils._EngineIntScalarType.to_py(_m)
    _q = _b_utils._EngineIntScalarType(
        q, {'fun_name': fun_name, 'entity': 'q'},
    ).to_ctypes()
    _q_py = _b_utils._EngineIntScalarType.to_py(_q)
    _theta = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (min(_p_py, _m_py-_p_py, _q_py, _m_py-_q_py),),
    )
    _jobu1 = _b_utils._EngineCharScalarType(
        jobu1, {'fun_name': fun_name, 'entity': 'jobu1'}, exp_chrlen=1,
    ).to_ctypes()
    _jobu1_py = _b_utils._EngineCharScalarType.to_py(_jobu1)
    if _jobu1_py not in ('N', 'n'):
        if _jobu1_py in ('Y', 'y'):
            _u1_dim1 = _p_py
        else:
            _u1_dim1 = 0
        if _jobu1_py in ('Y', 'y'):
            _u1_dim2 = _p_py
        else:
            _u1_dim2 = 1
        exp_shape_u1 = (_u1_dim1, _u1_dim2)
        _u1 = _b_utils._EngineComplex128ArrayType.empty_ndarray(
            exp_shape_u1,
            sorder=_m_sorder,
        )
    else:
        _u1 = None
    _jobu2 = _b_utils._EngineCharScalarType(
        jobu2, {'fun_name': fun_name, 'entity': 'jobu2'}, exp_chrlen=1,
    ).to_ctypes()
    _jobu2_py = _b_utils._EngineCharScalarType.to_py(_jobu2)
    if _jobu2_py not in ('N', 'n'):
        if _jobu2_py in ('Y', 'y'):
            _u2_dim1 = _m_py-_p_py
        else:
            _u2_dim1 = 0
        if _jobu2_py in ('Y', 'y'):
            _u2_dim2 = _m_py-_p_py
        else:
            _u2_dim2 = 1
        exp_shape_u2 = (_u2_dim1, _u2_dim2)
        _u2 = _b_utils._EngineComplex128ArrayType.empty_ndarray(
            exp_shape_u2,
            sorder=_m_sorder,
        )
    else:
        _u2 = None
    _jobv1t = _b_utils._EngineCharScalarType(
        jobv1t, {'fun_name': fun_name, 'entity': 'jobv1t'}, exp_chrlen=1,
    ).to_ctypes()
    _jobv1t_py = _b_utils._EngineCharScalarType.to_py(_jobv1t)
    if _jobv1t_py not in ('N', 'n'):
        if _jobv1t_py in ('Y', 'y'):
            _v1t_dim1 = _q_py
        else:
            _v1t_dim1 = 0
        if _jobv1t_py in ('Y', 'y'):
            _v1t_dim2 = _q_py
        else:
            _v1t_dim2 = 1
        exp_shape_v1t = (_v1t_dim1, _v1t_dim2)
        _v1t = _b_utils._EngineComplex128ArrayType.empty_ndarray(
            exp_shape_v1t,
            sorder=_m_sorder,
        )
    else:
        _v1t = None
    _jobv2t = _b_utils._EngineCharScalarType(
        jobv2t, {'fun_name': fun_name, 'entity': 'jobv2t'}, exp_chrlen=1,
    ).to_ctypes()
    _jobv2t_py = _b_utils._EngineCharScalarType.to_py(_jobv2t)
    if _jobv2t_py not in ('N', 'n'):
        if _jobv2t_py in ('Y', 'y'):
            _v2t_dim1 = _m_py-_q_py
        else:
            _v2t_dim1 = 0
        if _jobv2t_py in ('Y', 'y'):
            _v2t_dim2 = _m_py-_q_py
        else:
            _v2t_dim2 = 1
        exp_shape_v2t = (_v2t_dim1, _v2t_dim2)
        _v2t = _b_utils._EngineComplex128ArrayType.empty_ndarray(
            exp_shape_v2t,
            sorder=_m_sorder,
        )
    else:
        _v2t = None
    _b_lapackeig.zuncsd(
        jobu1,
        jobu2,
        jobv1t,
        jobv2t,
        trans,
        signs,
        _m,
        _p,
        _q,
        _x11,
        _x12,
        _x21,
        _x22,
        _theta,
        _u1,
        _u2,
        _v1t,
        _v2t,
    )
    _x11_py = _x11
    _x12_py = _x12
    _x21_py = _x21
    _x22_py = _x22
    _theta_py = _theta
    if _jobu1_py not in ('N', 'n'):
        _u1_py = _u1
    else:
        _u1_py = None
    if _jobu2_py not in ('N', 'n'):
        _u2_py = _u2
    else:
        _u2_py = None
    if _jobv1t_py not in ('N', 'n'):
        _v1t_py = _v1t
    else:
        _v1t_py = None
    if _jobv2t_py not in ('N', 'n'):
        _v2t_py = _v2t
    else:
        _v2t_py = None
    _return_data = ZuncsdReturnData(
        _x11_py,
        _x12_py,
        _x21_py,
        _x22_py,
        _theta_py,
        _u1_py,
        _u2_py,
        _v1t_py,
        _v2t_py,
    )
    return _return_data

DsygvReturnData = _collections.namedtuple(
    'DsygvReturnData',
    ['a', 'b', 'w'],
)

def dsygv(itype, jobz, uplo, a, b):
    u"""
    Computes all the eigenvalues, and optionally, the eigenvectors of a
    real generalized symmetric-definite eigenproblem.

    ``dsygv`` computes all the eigenvalues and, optionally, the
    eigenvectors of a real generalized symmetric-definite eigenproblem,
    of the form

        Az = lambda Bz, ABz = lambda z or BAz = lambda z,

    where A and B are symmetric and B is also positive definite.

    For full information please refer to the NAG Library document for
    f08sa

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08saf.html

    Parameters
    ----------
    itype : int
        Specifies the problem type to be solved.

        `itype` = 1

            Az = lambda Bz.

        `itype` = 2

            ABz = lambda z.

        `itype` = 3

            BAz = lambda z.

    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

            Only eigenvalues are computed.

        `jobz` = 'V'

            Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangles of A and B are stored.

        If `uplo` = 'L', the lower triangles of A and B are stored.

    a : float, array-like, shape (n, n)
        The n*n symmetric matrix A.

    b : float, array-like, shape (n, n)
        The n*n symmetric positive definite matrix B.

    Returns
    -------
    a : float, ndarray, shape (n, n)
        If `jobz` = 'V', `a` contains the matrix Z of eigenvectors. The
        eigenvectors are normalized as follows:

            if `itype` = 1 or 2, Z^TBZ = I;

            if `itype` = 3, Z^TB^-1Z = I.

        If `jobz` = 'N', the upper triangle (if `uplo` = 'U') or the
        lower triangle (if `uplo` = 'L') of `a`, including the diagonal,
        is overwritten.

    b : float, ndarray, shape (n, n)
        If the function exits successfully or `errno` = 1 ... `n`, the
        part of `b` containing the matrix is overwritten by the
        triangular factor U or L from the Cholesky factorization B =
        U^TU or B = LL^T.

    w : float, ndarray, shape (n)
        The eigenvalues in ascending order.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `itype`.

            Constraint: `itype` = 1, 2 or 3.

        (`errno` -2)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` i<=n)
            The algorithm failed to converge; *<value>* off-diagonal
            elements of an intermediate tridiagonal form did not
            converge to zero.

        (`errno` i > n)
            If errno = n+*<value>*, for 1 <= *<value>* <= n, then the
            leading minor of order *<value>* of B is not positive
            definite. The factorization of B could not be completed and
            no eigenvalues or eigenvectors were computed.

    Notes
    -----
    ``dsygv`` first performs a Cholesky factorization of the matrix B as
    B = U^TU, when `uplo` = 'U' or B = LL^T, when `uplo` = 'L'.
    The generalized problem is then reduced to a standard symmetric
    eigenvalue problem

        Cx = lambda x,

    which is solved for the eigenvalues and, optionally, the
    eigenvectors; the eigenvectors are then backtransformed to give the
    eigenvectors of the original problem.

    For the problem Az = lambda Bz, the eigenvectors are normalized so
    that the matrix of eigenvectors, z, satisfies

        Z^TAZ = Lambda and Z^TBZ = I,

    where Lambda is the diagonal matrix whose diagonal elements are the
    eigenvalues.
    For the problem ABz = lambda z we correspondingly have

        Z^-1AZ^-T = Lambda and Z^TBZ = I,

    and for BAz = lambda z we have

        Z^TAZ = Lambda and Z^TB^-1Z = I.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dsygv'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _b_lapackeig.dsygv(
        itype,
        jobz,
        uplo,
        _n,
        _a,
        _b,
        _w,
    )
    _a_py = _a
    _b_py = _b
    _w_py = _w
    _return_data = DsygvReturnData(
        _a_py,
        _b_py,
        _w_py,
    )
    return _return_data

DsygvxReturnData = _collections.namedtuple(
    'DsygvxReturnData',
    ['a', 'b', 'm', 'w', 'z', 'jfail'],
)

def dsygvx(itype, jobz, erange, uplo, a, b, vl, vu, il, iu, abstol):
    u"""
    Computes selected eigenvalues, and optionally, the eigenvectors of a
    real generalized symmetric-definite eigenproblem.

    ``dsygvx`` computes selected eigenvalues and, optionally,
    eigenvectors of a real generalized symmetric-definite eigenproblem,
    of the form

        Az = lambda Bz, ABz = lambda z or BAz = lambda z,

    where A and B are symmetric and B is also positive definite.
    Eigenvalues and eigenvectors can be selected by specifying either a
    range of values or a range of indices for the desired eigenvalues.

    For full information please refer to the NAG Library document for
    f08sb

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08sbf.html

    Parameters
    ----------
    itype : int
        Specifies the problem type to be solved.

        `itype` = 1

            Az = lambda Bz.

        `itype` = 2

            ABz = lambda z.

        `itype` = 3

            BAz = lambda z.

    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

            Only eigenvalues are computed.

        `jobz` = 'V'

            Eigenvalues and eigenvectors are computed.

    erange : str, length 1
        If `erange` = 'A', all eigenvalues will be found.

        If `erange` = 'V', all eigenvalues in the half-open interval
        (`vl`,`vu`] will be found.

        If `erange` = 'I', the `il`th to `iu`th eigenvalues will be
        found.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangles of A and B are stored.

        If `uplo` = 'L', the lower triangles of A and B are stored.

    a : float, array-like, shape (n, n)
        The n*n symmetric matrix A.

    b : float, array-like, shape (n, n)
        The n*n symmetric matrix B.

    vl : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    vu : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    il : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    iu : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    abstol : float
        The absolute error tolerance for the eigenvalues. An approximate
        eigenvalue is accepted as converged when it is determined to lie
        in an interval [a,b] of width less than or equal to

            `abstol`+epsilon max(|a|, |b|),

        where epsilon is the machine precision. If `abstol` is less than
        or equal to zero, then epsilon||T||_1 will be used in its place,
        where T is the tridiagonal matrix obtained by reducing C to
        tridiagonal form. Eigenvalues will be computed most accurately
        when `abstol` is set to twice the underflow threshold
        2*machine.real_safe(), not zero. If this function returns with
        `errno` = 1 ... `n`, indicating that some eigenvectors did not
        converge, try setting `abstol` to 2*machine.real_safe(). See
        Demmel and Kahan (1990).

    Returns
    -------
    a : float, ndarray, shape (n, n)
        The lower triangle (if `uplo` = 'L') or the upper triangle (if
        `uplo` = 'U') of `a`, including the diagonal, is overwritten.

    b : float, ndarray, shape (n, n)
        The triangular factor U or L from the Cholesky factorization B =
        U^TU or B = LL^T.

    m : int
        The total number of eigenvalues found. 0 <= `m` <= n.

        If `erange` = 'A', `m` = n.

        If `erange` = 'I', `m` = `iu`-`il`+1.

    w : float, ndarray, shape (n)
        The first `m` elements contain the selected eigenvalues in
        ascending order.

    z : float, ndarray, shape (:, :)
        If `jobz` = 'V', then

            if no exception or warning is raised, the first `m` columns
            of Z contain the orthonormal eigenvectors of the matrix A
            corresponding to the selected eigenvalues, with the ith
            column of Z holding the eigenvector associated with
            `w`[i-1]. The eigenvectors are normalized as follows:

                if `itype` = 1 or 2, Z^TBZ = I;

                if `itype` = 3, Z^TB^-1Z = I;

            if an eigenvector fails to converge (`errno` = 1 ... `n`),
            then that column of Z contains the latest approximation to
            the eigenvector, and the index of the eigenvector is
            returned in `jfail`.

        If `jobz` = 'N', `z` is not referenced.

        **Note:** you must ensure that at least max(1, `m`) columns are
        supplied in the array `z`; if `erange` = 'V', the exact value of
        `m` is not known in advance and an upper bound of at least n
        must be used.

    jfail : int, ndarray, shape (n)
        If `jobz` = 'V', then

            if no exception or warning is raised, the first `m` elements
            of `jfail` are zero;

            if `errno` = 1 ... `n`, the first errno elements of `jfail`
            contains the indices of the eigenvectors that failed to
            converge.

        If `jobz` = 'N', `jfail` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `itype`.

            Constraint: `itype` = 1, 2 or 3.

        (`errno` -2)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `erange`.

            Constraint: `erange` = 'A', 'V' or 'I'.

        (`errno` -4)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -5)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -11)
            On entry, error in parameter `vu`.

            Constraint: `vl` < `vu`.

        (`errno` -12)
            On entry, error in parameter `il`.

        (`errno` -13)
            On entry, error in parameter `iu`.

        (`errno` i<=n)
            The algorithm failed to converge; *<value>* eigenvectors
            failed to converge.

        (`errno` i > n)
            If errno = n+*<value>*, for 1 <= *<value>* <= n, then the
            leading minor of order *<value>* of B is not positive
            definite. The factorization of B could not be completed and
            no eigenvalues or eigenvectors were computed.

    Notes
    -----
    ``dsygvx`` first performs a Cholesky factorization of the matrix B
    as B = U^TU, when `uplo` = 'U' or B = LL^T, when `uplo` = 'L'.
    The generalized problem is then reduced to a standard symmetric
    eigenvalue problem

        Cx = lambda x,

    which is solved for the desired eigenvalues and eigenvectors; the
    eigenvectors are then backtransformed to give the eigenvectors of
    the original problem.

    For the problem Az = lambda Bz, the eigenvectors are normalized so
    that the matrix of eigenvectors, Z, satisfies

        Z^TAZ = Lambda and Z^TBZ = I,

    where Lambda is the diagonal matrix whose diagonal elements are the
    eigenvalues.
    For the problem ABz = lambda z we correspondingly have

        Z^-1AZ^-T = Lambda and Z^TBZ = I,

    and for BAz = lambda z we have

        Z^TAZ = Lambda and Z^TB^-1Z = I.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Demmel, J W and Kahan, W, 1990, `Accurate singular values of
    bidiagonal matrices`, SIAM J. Sci. Statist. Comput. (11), 873--912

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dsygvx'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'), (_b, 'b')),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobz = _b_utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = _b_utils._EngineCharScalarType.to_py(_jobz)
    _erange = _b_utils._EngineCharScalarType(
        erange, {'fun_name': fun_name, 'entity': 'erange'}, exp_chrlen=1,
    ).to_ctypes()
    _erange_py = _b_utils._EngineCharScalarType.to_py(_erange)
    _iu = _b_utils._EngineIntScalarType(
        iu, {'fun_name': fun_name, 'entity': 'iu'},
    ).to_ctypes()
    _iu_py = _b_utils._EngineIntScalarType.to_py(_iu)
    _il = _b_utils._EngineIntScalarType(
        il, {'fun_name': fun_name, 'entity': 'il'},
    ).to_ctypes()
    _il_py = _b_utils._EngineIntScalarType.to_py(_il)
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        if _erange_py in ('I', 'i'):
            _z_dim2 = _iu_py-_il_py+1
        else:
            _z_dim2 = _n_py
    else:
        _z_dim2 = 0
    exp_shape_z = (_z_dim1, _z_dim2)
    _z = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_z,
        sorder=_m_sorder,
    )
    _jfail = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    _return_scalar = _b_lapackeig.dsygvx(
        itype,
        jobz,
        erange,
        uplo,
        _n,
        _a,
        _b,
        vl,
        vu,
        _il,
        _iu,
        abstol,
        _w,
        _z,
        _jfail,
    )
    _a_py = _a
    _b_py = _b
    _m_py = _return_scalar
    _w_py = _w
    if _jobz_py in ('V', 'v'):
        _z_dim2 = _m_py
    else:
        _z_dim2 = 0
    _z_py = _z[:, :_z_dim2]
    _jfail_py = _jfail
    _return_data = DsygvxReturnData(
        _a_py,
        _b_py,
        _m_py,
        _w_py,
        _z_py,
        _jfail_py,
    )
    return _return_data

DsygvdReturnData = _collections.namedtuple(
    'DsygvdReturnData',
    ['a', 'b', 'w'],
)

def dsygvd(itype, jobz, uplo, a, b):
    u"""
    Computes all the eigenvalues, and optionally, the eigenvectors of a
    real generalized symmetric-definite eigenproblem
    (divide-and-conquer).

    ``dsygvd`` computes all the eigenvalues and, optionally, the
    eigenvectors of a real generalized symmetric-definite eigenproblem,
    of the form

        Az = lambda Bz, ABz = lambda z or BAz = lambda z,

    where A and B are symmetric and B is also positive definite.
    If eigenvectors are desired, it uses a divide-and-conquer algorithm.

    For full information please refer to the NAG Library document for
    f08sc

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08scf.html

    Parameters
    ----------
    itype : int
        Specifies the problem type to be solved.

        `itype` = 1

            Az = lambda Bz.

        `itype` = 2

            ABz = lambda z.

        `itype` = 3

            BAz = lambda z.

    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

            Only eigenvalues are computed.

        `jobz` = 'V'

            Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangles of A and B are stored.

        If `uplo` = 'L', the lower triangles of A and B are stored.

    a : float, array-like, shape (n, n)
        The n*n symmetric matrix A.

    b : float, array-like, shape (n, n)
        The n*n symmetric matrix B.

    Returns
    -------
    a : float, ndarray, shape (n, n)
        If `jobz` = 'V', `a` contains the matrix Z of eigenvectors. The
        eigenvectors are normalized as follows:

            if `itype` = 1 or 2, Z^TBZ = I;

            if `itype` = 3, Z^TB^-1Z = I.

        If `jobz` = 'N', the upper triangle (if `uplo` = 'U') or the
        lower triangle (if `uplo` = 'L') of `a`, including the diagonal,
        is overwritten.

    b : float, ndarray, shape (n, n)
        The triangular factor U or L from the Cholesky factorization B =
        U^TU or B = LL^T.

    w : float, ndarray, shape (n)
        The eigenvalues in ascending order.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `itype`.

            Constraint: `itype` = 1, 2 or 3.

        (`errno` -2)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` i<=n)
            The algorithm failed to converge; *<value>* off-diagonal
            elements of an intermediate tridiagonal form did not
            converge to zero.

        (`errno` i > n)
            If errno = n+*<value>*, for 1 <= *<value>* <= n, then the
            leading minor of order *<value>* of B is not positive
            definite. The factorization of B could not be completed and
            no eigenvalues or eigenvectors were computed.

    Notes
    -----
    ``dsygvd`` first performs a Cholesky factorization of the matrix B
    as B = U^TU, when `uplo` = 'U' or B = LL^T, when `uplo` = 'L'.
    The generalized problem is then reduced to a standard symmetric
    eigenvalue problem

        Cx = lambda x,

    which is solved for the eigenvalues and, optionally, the
    eigenvectors; the eigenvectors are then backtransformed to give the
    eigenvectors of the original problem.

    For the problem Az = lambda Bz, the eigenvectors are normalized so
    that the matrix of eigenvectors, z, satisfies

        Z^TAZ = Lambda and Z^TBZ = I,

    where Lambda is the diagonal matrix whose diagonal elements are the
    eigenvalues.
    For the problem ABz = lambda z we correspondingly have

        Z^-1AZ^-T = Lambda and Z^TBZ = I,

    and for BAz = lambda z we have

        Z^TAZ = Lambda and Z^TB^-1Z = I.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dsygvd'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _b_lapackeig.dsygvd(
        itype,
        jobz,
        uplo,
        _n,
        _a,
        _b,
        _w,
    )
    _a_py = _a
    _b_py = _b
    _w_py = _w
    _return_data = DsygvdReturnData(
        _a_py,
        _b_py,
        _w_py,
    )
    return _return_data

def dsygst(itype, uplo, a, b):
    u"""
    Performs a reduction to standard form of real symmetric-definite
    generalized eigenproblem Ax = lambda Bx, ABx = lambda x or BAx =
    lambda x, B factorized by ``lapacklin.dpotrf``.

    ``dsygst`` reduces a real symmetric-definite generalized
    eigenproblem Az = lambda Bz, ABz = lambda z or BAz = lambda z to the
    standard form Cy = lambda y, where A is a real symmetric matrix and
    B has been factorized by ``lapacklin.dpotrf``.

    For full information please refer to the NAG Library document for
    f08se

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08sef.html

    Parameters
    ----------
    itype : int
        Indicates how the standard form is computed.

        `itype` = 1

                if `uplo` = 'U', C = U^-TAU^-1;

                if `uplo` = 'L', C = L^-1AL^-T.

        `itype` = 2 or 3

                if `uplo` = 'U', C = UAU^T;

                if `uplo` = 'L', C = L^TAL.

    uplo : str, length 1
        Indicates whether the upper or lower triangular part of A is
        stored and how B has been factorized.

        `uplo` = 'U'

            The upper triangular part of A is stored and B = U^TU.

        `uplo` = 'L'

            The lower triangular part of A is stored and B = LL^T.

    a : float, array-like, shape (n, n)
        The n*n symmetric matrix A.

    b : float, array-like, shape (n, n)
        The Cholesky factor of B as specified by uplo and returned by
        ``lapacklin.dpotrf``.

    Returns
    -------
    a : float, ndarray, shape (n, n)
        The upper or lower triangle of `a` is overwritten by the
        corresponding upper or lower triangle of C as specified by
        `itype` and `uplo`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `itype`.

            Constraint: `itype` = 1, 2 or 3.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

    Notes
    -----
    To reduce the real symmetric-definite generalized eigenproblem Az =
    lambda Bz, ABz = lambda z or BAz = lambda z to the standard form Cy
    = lambda y, ``dsygst`` must be preceded by a call to
    ``lapacklin.dpotrf`` which computes the Cholesky factorization of B;
    B must be positive definite.

    The different problem types are specified by the argument `itype`,
    as indicated in the table below.
    The table shows how C is computed by the function, and also how the
    eigenvectors z of the original problem can be recovered from the
    eigenvectors of the standard form.

    [table omitted]

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dsygst'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b_lapackeig.dsygst(
        itype,
        uplo,
        _n,
        _a,
        _b,
    )
    _a_py = _a
    return _a_py

ZhegvReturnData = _collections.namedtuple(
    'ZhegvReturnData',
    ['a', 'b', 'w'],
)

def zhegv(itype, jobz, uplo, a, b):
    u"""
    Computes all the eigenvalues, and optionally, the eigenvectors of a
    complex generalized Hermitian-definite eigenproblem.

    ``zhegv`` computes all the eigenvalues and, optionally, the
    eigenvectors of a complex generalized Hermitian-definite
    eigenproblem, of the form

        Az = lambda Bz, ABz = lambda z or BAz = lambda z,

    where A and B are Hermitian and B is also positive definite.

    For full information please refer to the NAG Library document for
    f08sn

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08snf.html

    Parameters
    ----------
    itype : int
        Specifies the problem type to be solved.

        `itype` = 1

            Az = lambda Bz.

        `itype` = 2

            ABz = lambda z.

        `itype` = 3

            BAz = lambda z.

    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

            Only eigenvalues are computed.

        `jobz` = 'V'

            Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangles of A and B are stored.

        If `uplo` = 'L', the lower triangles of A and B are stored.

    a : complex, array-like, shape (n, n)
        The n*n Hermitian matrix A.

    b : complex, array-like, shape (n, n)
        The n*n Hermitian positive definite matrix B.

    Returns
    -------
    a : complex, ndarray, shape (n, n)
        If `jobz` = 'V', `a` contains the matrix Z of eigenvectors. The
        eigenvectors are normalized as follows:

            if `itype` = 1 or 2, Z^HBZ = I;

            if `itype` = 3, Z^HB^-1Z = I.

        If `jobz` = 'N', the upper triangle (if `uplo` = 'U') or the
        lower triangle (if `uplo` = 'L') of `a`, including the diagonal,
        is overwritten.

    b : complex, ndarray, shape (n, n)
        If the function exits successfully or `errno` = 1 ... `n`, the
        part of `b` containing the matrix is overwritten by the
        triangular factor U or L from the Cholesky factorization B =
        U^HU or B = LL^H.

    w : float, ndarray, shape (n)
        The eigenvalues in ascending order.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `itype`.

            Constraint: `itype` = 1, 2 or 3.

        (`errno` -2)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` i<=n)
            The algorithm failed to converge; *<value>* off-diagonal
            elements of an intermediate tridiagonal form did not
            converge to zero.

        (`errno` i > n)
            If errno = n+*<value>*, for 1 <= *<value>* <= n, then the
            leading minor of order *<value>* of B is not positive
            definite. The factorization of B could not be completed and
            no eigenvalues or eigenvectors were computed.

    Notes
    -----
    ``zhegv`` first performs a Cholesky factorization of the matrix B as
    B = U^HU, when `uplo` = 'U' or B = LL^H, when `uplo` = 'L'.
    The generalized problem is then reduced to a standard symmetric
    eigenvalue problem

        Cx = lambda x,

    which is solved for the eigenvalues and, optionally, the
    eigenvectors; the eigenvectors are then backtransformed to give the
    eigenvectors of the original problem.

    For the problem Az = lambda Bz, the eigenvectors are normalized so
    that the matrix of eigenvectors, z, satisfies

        Z^HAZ = Lambda and Z^HBZ = I,

    where Lambda is the diagonal matrix whose diagonal elements are the
    eigenvalues.
    For the problem ABz = lambda z we correspondingly have

        Z^-1AZ^-H = Lambda and Z^HBZ = I,

    and for BAz = lambda z we have

        Z^HAZ = Lambda and Z^HB^-1Z = I.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zhegv'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
    )
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _b_lapackeig.zhegv(
        itype,
        jobz,
        uplo,
        _n,
        _a,
        _b,
        _w,
    )
    _a_py = _a
    _b_py = _b
    _w_py = _w
    _return_data = ZhegvReturnData(
        _a_py,
        _b_py,
        _w_py,
    )
    return _return_data

ZhegvxReturnData = _collections.namedtuple(
    'ZhegvxReturnData',
    ['a', 'b', 'm', 'w', 'z', 'jfail'],
)

def zhegvx(itype, jobz, erange, uplo, a, b, vl, vu, il, iu, abstol):
    u"""
    Computes selected eigenvalues, and optionally, the eigenvectors of a
    complex generalized Hermitian-definite eigenproblem.

    ``zhegvx`` computes selected eigenvalues and, optionally,
    eigenvectors of a complex generalized Hermitian-definite
    eigenproblem, of the form

        Az = lambda Bz, ABz = lambda z or BAz = lambda z,

    where A and B are Hermitian and B is also positive definite.
    Eigenvalues and eigenvectors can be selected by specifying either a
    range of values or a range of indices for the desired eigenvalues.

    For full information please refer to the NAG Library document for
    f08sp

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08spf.html

    Parameters
    ----------
    itype : int
        Specifies the problem type to be solved.

        `itype` = 1

            Az = lambda Bz.

        `itype` = 2

            ABz = lambda z.

        `itype` = 3

            BAz = lambda z.

    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

            Only eigenvalues are computed.

        `jobz` = 'V'

            Eigenvalues and eigenvectors are computed.

    erange : str, length 1
        If `erange` = 'A', all eigenvalues will be found.

        If `erange` = 'V', all eigenvalues in the half-open interval
        (`vl`,`vu`] will be found.

        If `erange` = 'I', the `il`th to `iu`th eigenvalues will be
        found.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangles of A and B are stored.

        If `uplo` = 'L', the lower triangles of A and B are stored.

    a : complex, array-like, shape (n, n)
        The n*n Hermitian matrix A.

    b : complex, array-like, shape (n, n)
        The n*n Hermitian matrix B.

    vl : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    vu : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    il : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    iu : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    abstol : float
        The absolute error tolerance for the eigenvalues. An approximate
        eigenvalue is accepted as converged when it is determined to lie
        in an interval [a,b] of width less than or equal to

            `abstol`+epsilon max(|a|, |b|),

        where epsilon is the machine precision. If `abstol` is less than
        or equal to zero, then epsilon||T||_1 will be used in its place,
        where T is the tridiagonal matrix obtained by reducing C to
        tridiagonal form. Eigenvalues will be computed most accurately
        when `abstol` is set to twice the underflow threshold
        2*machine.real_safe(), not zero. If this function returns with
        `errno` = 1 ... `n`, indicating that some eigenvectors did not
        converge, try setting `abstol` to 2*machine.real_safe(). See
        Demmel and Kahan (1990).

    Returns
    -------
    a : complex, ndarray, shape (n, n)
        The lower triangle (if `uplo` = 'L') or the upper triangle (if
        `uplo` = 'U') of `a`, including the diagonal, is overwritten.

    b : complex, ndarray, shape (n, n)
        The triangular factor U or L from the Cholesky factorization B =
        U^HU or B = LL^H.

    m : int
        The total number of eigenvalues found. 0 <= `m` <= n.

        If `erange` = 'A', `m` = n.

        If `erange` = 'I', `m` = `iu`-`il`+1.

    w : float, ndarray, shape (n)
        The first `m` elements contain the selected eigenvalues in
        ascending order.

    z : complex, ndarray, shape (:, :)
        If `jobz` = 'V', then

            if no exception or warning is raised, the first `m` columns
            of Z contain the orthonormal eigenvectors of the matrix A
            corresponding to the selected eigenvalues, with the ith
            column of Z holding the eigenvector associated with
            `w`[i-1]. The eigenvectors are normalized as follows:

                if `itype` = 1 or 2, Z^HBZ = I;

                if `itype` = 3, Z^HB^-1Z = I;

            if an eigenvector fails to converge (`errno` = 1 ... `n`),
            then that column of Z contains the latest approximation to
            the eigenvector, and the index of the eigenvector is
            returned in `jfail`.

        If `jobz` = 'N', `z` is not referenced.

        **Note:** you must ensure that at least max(1, `m`) columns are
        supplied in the array `z`; if `erange` = 'V', the exact value of
        `m` is not known in advance and an upper bound of at least n
        must be used.

    jfail : int, ndarray, shape (n)
        If `jobz` = 'V', then

            if no exception or warning is raised, the first `m` elements
            of `jfail` are zero;

            if `errno` = 1 ... `n`, the first errno elements of `jfail`
            contains the indices of the eigenvectors that failed to
            converge.

        If `jobz` = 'N', `jfail` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `itype`.

            Constraint: `itype` = 1, 2 or 3.

        (`errno` -2)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `erange`.

            Constraint: `erange` = 'A', 'V' or 'I'.

        (`errno` -4)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -5)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -11)
            On entry, error in parameter `vu`.

            Constraint: `vl` < `vu`.

        (`errno` -12)
            On entry, error in parameter `il`.

        (`errno` -13)
            On entry, error in parameter `iu`.

        (`errno` i > n)
            If errno = n+*<value>*, for 1 <= *<value>* <= n, then the
            leading minor of order *<value>* of B is not positive
            definite. The factorization of B could not be completed and
            no eigenvalues or eigenvectors were computed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i<=n)
            The algorithm failed to converge; *<value>* eigenvectors
            failed to converge.

    Notes
    -----
    ``zhegvx`` first performs a Cholesky factorization of the matrix B
    as B = U^HU, when `uplo` = 'U' or B = LL^H, when `uplo` = 'L'.
    The generalized problem is then reduced to a standard symmetric
    eigenvalue problem

        Cx = lambda x,

    which is solved for the desired eigenvalues and eigenvectors; the
    eigenvectors are then backtransformed to give the eigenvectors of
    the original problem.

    For the problem Az = lambda Bz, the eigenvectors are normalized so
    that the matrix of eigenvectors, Z, satisfies

        Z^HAZ = Lambda and Z^HBZ = I,

    where Lambda is the diagonal matrix whose diagonal elements are the
    eigenvalues.
    For the problem ABz = lambda z we correspondingly have

        Z^-1AZ^-H = Lambda and Z^HBZ = I,

    and for BAz = lambda z we have

        Z^HAZ = Lambda and Z^HB^-1Z = I.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Demmel, J W and Kahan, W, 1990, `Accurate singular values of
    bidiagonal matrices`, SIAM J. Sci. Statist. Comput. (11), 873--912

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zhegvx'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'), (_b, 'b')),
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobz = _b_utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = _b_utils._EngineCharScalarType.to_py(_jobz)
    _erange = _b_utils._EngineCharScalarType(
        erange, {'fun_name': fun_name, 'entity': 'erange'}, exp_chrlen=1,
    ).to_ctypes()
    _erange_py = _b_utils._EngineCharScalarType.to_py(_erange)
    _iu = _b_utils._EngineIntScalarType(
        iu, {'fun_name': fun_name, 'entity': 'iu'},
    ).to_ctypes()
    _iu_py = _b_utils._EngineIntScalarType.to_py(_iu)
    _il = _b_utils._EngineIntScalarType(
        il, {'fun_name': fun_name, 'entity': 'il'},
    ).to_ctypes()
    _il_py = _b_utils._EngineIntScalarType.to_py(_il)
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        if _erange_py in ('I', 'i'):
            _z_dim2 = _iu_py-_il_py+1
        else:
            _z_dim2 = _n_py
    else:
        _z_dim2 = 0
    exp_shape_z = (_z_dim1, _z_dim2)
    _z = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_z,
        sorder=_m_sorder,
    )
    _jfail = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        _return_scalar = _b_lapackeig.zhegvx(
            itype,
            jobz,
            erange,
            uplo,
            _n,
            _a,
            _b,
            vl,
            vu,
            _il,
            _iu,
            abstol,
            _w,
            _z,
            _jfail,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _a_py = _a
    _b_py = _b
    _m_py = _return_scalar
    _w_py = _w
    if _jobz_py in ('V', 'v'):
        _z_dim2 = _m_py
    else:
        _z_dim2 = 0
    _z_py = _z[:, :_z_dim2]
    _jfail_py = _jfail
    _return_data = ZhegvxReturnData(
        _a_py,
        _b_py,
        _m_py,
        _w_py,
        _z_py,
        _jfail_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

ZhegvdReturnData = _collections.namedtuple(
    'ZhegvdReturnData',
    ['a', 'b', 'w'],
)

def zhegvd(itype, jobz, uplo, a, b):
    u"""
    Computes all the eigenvalues, and optionally, the eigenvectors of a
    complex generalized Hermitian-definite eigenproblem
    (divide-and-conquer).

    ``zhegvd`` computes all the eigenvalues and, optionally, the
    eigenvectors of a complex generalized Hermitian-definite
    eigenproblem, of the form

        Az = lambda Bz, ABz = lambda z or BAz = lambda z,

    where A and B are Hermitian and B is also positive definite.
    If eigenvectors are desired, it uses a divide-and-conquer algorithm.

    For full information please refer to the NAG Library document for
    f08sq

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08sqf.html

    Parameters
    ----------
    itype : int
        Specifies the problem type to be solved.

        `itype` = 1

            Az = lambda Bz.

        `itype` = 2

            ABz = lambda z.

        `itype` = 3

            BAz = lambda z.

    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

            Only eigenvalues are computed.

        `jobz` = 'V'

            Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangles of A and B are stored.

        If `uplo` = 'L', the lower triangles of A and B are stored.

    a : complex, array-like, shape (n, n)
        The n*n Hermitian matrix A.

    b : complex, array-like, shape (n, n)
        The n*n Hermitian matrix B.

    Returns
    -------
    a : complex, ndarray, shape (n, n)
        If `jobz` = 'V', `a` contains the matrix Z of eigenvectors. The
        eigenvectors are normalized as follows:

            if `itype` = 1 or 2, Z^HBZ = I;

            if `itype` = 3, Z^HB^-1Z = I.

        If `jobz` = 'N', the upper triangle (if `uplo` = 'U') or the
        lower triangle (if `uplo` = 'L') of `a`, including the diagonal,
        is overwritten.

    b : complex, ndarray, shape (n, n)
        The triangular factor U or L from the Cholesky factorization B =
        U^HU or B = LL^H.

    w : float, ndarray, shape (n)
        The eigenvalues in ascending order.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `itype`.

            Constraint: `itype` = 1, 2 or 3.

        (`errno` -2)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` i<=n)
            The algorithm failed to converge; *<value>* off-diagonal
            elements of an intermediate tridiagonal form did not
            converge to zero.

        (`errno` i > n)
            If errno = n+*<value>*, for 1 <= *<value>* <= n, then the
            leading minor of order *<value>* of B is not positive
            definite. The factorization of B could not be completed and
            no eigenvalues or eigenvectors were computed.

    Notes
    -----
    ``zhegvd`` first performs a Cholesky factorization of the matrix B
    as B = U^HU, when `uplo` = 'U' or B = LL^H, when `uplo` = 'L'.
    The generalized problem is then reduced to a standard symmetric
    eigenvalue problem

        Cx = lambda x,

    which is solved for the eigenvalues and, optionally, the
    eigenvectors; the eigenvectors are then backtransformed to give the
    eigenvectors of the original problem.

    For the problem Az = lambda Bz, the eigenvectors are normalized so
    that the matrix of eigenvectors, z, satisfies

        Z^HAZ = Lambda and Z^HBZ = I,

    where Lambda is the diagonal matrix whose diagonal elements are the
    eigenvalues.
    For the problem ABz = lambda z we correspondingly have

        Z^-1AZ^-H = Lambda and Z^HBZ = I,

    and for BAz = lambda z we have

        Z^HAZ = Lambda and Z^HB^-1Z = I.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zhegvd'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
    )
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _b_lapackeig.zhegvd(
        itype,
        jobz,
        uplo,
        _n,
        _a,
        _b,
        _w,
    )
    _a_py = _a
    _b_py = _b
    _w_py = _w
    _return_data = ZhegvdReturnData(
        _a_py,
        _b_py,
        _w_py,
    )
    return _return_data

def zhegst(itype, uplo, a, b):
    u"""
    Performs a reduction to standard form of complex Hermitian-definite
    generalized eigenproblem Ax = lambda Bx, ABx = lambda x or BAx =
    lambda x, B factorized by ``lapacklin.zpotrf``.

    ``zhegst`` reduces a complex Hermitian-definite generalized
    eigenproblem Az = lambda Bz, ABz = lambda z or BAz = lambda z to the
    standard form Cy = lambda y, where A is a complex Hermitian matrix
    and B has been factorized by ``lapacklin.zpotrf``.

    For full information please refer to the NAG Library document for
    f08ss

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08ssf.html

    Parameters
    ----------
    itype : int
        Indicates how the standard form is computed.

        `itype` = 1

                if `uplo` = 'U', C = U^-HAU^-1;

                if `uplo` = 'L', C = L^-1AL^-H.

        `itype` = 2 or 3

                if `uplo` = 'U', C = UAU^H;

                if `uplo` = 'L', C = L^HAL.

    uplo : str, length 1
        Indicates whether the upper or lower triangular part of A is
        stored and how B has been factorized.

        `uplo` = 'U'

            The upper triangular part of A is stored and B = U^HU.

        `uplo` = 'L'

            The lower triangular part of A is stored and B = LL^H.

    a : complex, array-like, shape (n, n)
        The n*n Hermitian matrix A.

    b : complex, array-like, shape (n, n)
        The Cholesky factor of B as specified by uplo and returned by
        ``lapacklin.zpotrf``.

    Returns
    -------
    a : complex, ndarray, shape (n, n)
        The upper or lower triangle of `a` is overwritten by the
        corresponding upper or lower triangle of C as specified by
        `itype` and `uplo`.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `itype`.

            Constraint: `itype` = 1, 2 or 3.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

    Notes
    -----
    To reduce the complex Hermitian-definite generalized eigenproblem Az
    = lambda Bz, ABz = lambda z or BAz = lambda z to the standard form
    Cy = lambda y, ``zhegst`` must be preceded by a call to
    ``lapacklin.zpotrf`` which computes the Cholesky factorization of B;
    B must be positive definite.

    The different problem types are specified by the argument `itype`,
    as indicated in the table below.
    The table shows how C is computed by the function, and also how the
    eigenvectors z of the original problem can be recovered from the
    eigenvectors of the standard form.

    [table omitted]

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zhegst'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
    )
    _b_lapackeig.zhegst(
        itype,
        uplo,
        _n,
        _a,
        _b,
    )
    _a_py = _a
    return _a_py

DspgvReturnData = _collections.namedtuple(
    'DspgvReturnData',
    ['ap', 'bp', 'w', 'z'],
)

def dspgv(itype, jobz, uplo, n, ap, bp):
    u"""
    Computes all the eigenvalues, and optionally, the eigenvectors of a
    real generalized symmetric-definite eigenproblem, packed storage.

    ``dspgv`` computes all the eigenvalues and, optionally, all the
    eigenvectors of a real generalized symmetric-definite eigenproblem,
    of the form

        Az = lambda Bz, ABz = lambda z or BAz = lambda z,

    where A and B are symmetric, stored in packed format, and B is also
    positive definite.

    For full information please refer to the NAG Library document for
    f08ta

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08taf.html

    Parameters
    ----------
    itype : int
        Specifies the problem type to be solved.

        `itype` = 1

            Az = lambda Bz.

        `itype` = 2

            ABz = lambda z.

        `itype` = 3

            BAz = lambda z.

    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

            Only eigenvalues are computed.

        `jobz` = 'V'

            Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangles of A and B are stored.

        If `uplo` = 'L', the lower triangles of A and B are stored.

    n : int
        n, the order of the matrices A and B.

    ap : float, array-like, shape (`n`*(`n`+1)/2)
        The upper or lower triangle of the n*n symmetric matrix A,
        packed by columns.

    bp : float, array-like, shape (`n`*(`n`+1)/2)
        The upper or lower triangle of the n*n symmetric matrix B,
        packed by columns.

    Returns
    -------
    ap : float, ndarray, shape (`n`*(`n`+1)/2)
        The contents of `ap` are destroyed.

    bp : float, ndarray, shape (`n`*(`n`+1)/2)
        The triangular factor U or L from the Cholesky factorization B =
        U^TU or B = LL^T, in the same storage format as B.

    w : float, ndarray, shape (`n`)
        The eigenvalues in ascending order.

    z : float, ndarray, shape (:, :)
        If `jobz` = 'V', `z` contains the matrix Z of eigenvectors. The
        eigenvectors are normalized as follows:

            if `itype` = 1 or 2, Z^TBZ = I;

            if `itype` = 3, Z^TB^-1Z = I.

        If `jobz` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `itype`.

            Constraint: `itype` = 1, 2 or 3.

        (`errno` -2)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` i<=`n`)
            The algorithm failed to converge; *<value>* off-diagonal
            elements of an intermediate tridiagonal form did not
            converge to zero.

        (`errno` i > `n`)
            If errno = `n`+*<value>*, for 1 <= *<value>* <= `n`, then
            the leading minor of order *<value>* of B is not positive
            definite. The factorization of B could not be completed and
            no eigenvalues or eigenvectors were computed.

    Notes
    -----
    ``dspgv`` first performs a Cholesky factorization of the matrix B as
    B = U^TU, when `uplo` = 'U' or B = LL^T, when `uplo` = 'L'.
    The generalized problem is then reduced to a standard symmetric
    eigenvalue problem

        Cx = lambda x,

    which is solved for the eigenvalues and, optionally, the
    eigenvectors; the eigenvectors are then backtransformed to give the
    eigenvectors of the original problem.

    For the problem Az = lambda Bz, the eigenvectors are normalized so
    that the matrix of eigenvectors, Z, satisfies

        Z^TAZ = Lambda and Z^TBZ = I,

    where Lambda is the diagonal matrix whose diagonal elements are the
    eigenvalues.
    For the problem ABz = lambda z we correspondingly have

        Z^-1AZ^-T = Lambda and Z^TBZ = I,

    and for BAz = lambda z we have

        Z^TAZ = Lambda and Z^TB^-1Z = I.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dspgv'

    _ap = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ap'},
        ap,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _bp = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'bp'},
        bp,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobz = _b_utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = _b_utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        _z_dim2 = _n_py
    else:
        _z_dim2 = 1
    exp_shape_z = (_z_dim1, _z_dim2)
    _z = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_z,
        sorder=_b_utils._EngineData.default_sorder,
    )
    _b_lapackeig.dspgv(
        itype,
        jobz,
        uplo,
        _n,
        _ap,
        _bp,
        _w,
        _z,
    )
    _ap_py = _ap
    _bp_py = _bp
    _w_py = _w
    _z_py = _z
    _return_data = DspgvReturnData(
        _ap_py,
        _bp_py,
        _w_py,
        _z_py,
    )
    return _return_data

DspgvxReturnData = _collections.namedtuple(
    'DspgvxReturnData',
    ['ap', 'bp', 'm', 'w', 'z', 'jfail'],
)

def dspgvx(itype, jobz, erange, uplo, n, ap, bp, vl, vu, il, iu, abstol):
    u"""
    Computes selected eigenvalues, and optionally, the eigenvectors of a
    real generalized symmetric-definite eigenproblem, packed storage.

    ``dspgvx`` computes selected eigenvalues and, optionally,
    eigenvectors of a real generalized symmetric-definite eigenproblem,
    of the form

        Az = lambda Bz, ABz = lambda z or BAz = lambda z,

    where A and B are symmetric, stored in packed storage, and B is also
    positive definite.
    Eigenvalues and eigenvectors can be selected by specifying either a
    range of values or a range of indices for the desired eigenvalues.

    For full information please refer to the NAG Library document for
    f08tb

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08tbf.html

    Parameters
    ----------
    itype : int
        Specifies the problem type to be solved.

        `itype` = 1

            Az = lambda Bz.

        `itype` = 2

            ABz = lambda z.

        `itype` = 3

            BAz = lambda z.

    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

            Only eigenvalues are computed.

        `jobz` = 'V'

            Eigenvalues and eigenvectors are computed.

    erange : str, length 1
        If `erange` = 'A', all eigenvalues will be found.

        If `erange` = 'V', all eigenvalues in the half-open interval
        (`vl`,`vu`] will be found.

        If `erange` = 'I', the `il`th to `iu`th eigenvalues will be
        found.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangles of A and B are stored.

        If `uplo` = 'L', the lower triangles of A and B are stored.

    n : int
        n, the order of the matrices A and B.

    ap : float, array-like, shape (`n`*(`n`+1)/2)
        The upper or lower triangle of the n*n symmetric matrix A,
        packed by columns.

    bp : float, array-like, shape (`n`*(`n`+1)/2)
        The upper or lower triangle of the n*n symmetric matrix B,
        packed by columns.

    vl : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    vu : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    il : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    iu : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    abstol : float
        The absolute error tolerance for the eigenvalues. An approximate
        eigenvalue is accepted as converged when it is determined to lie
        in an interval [a,b] of width less than or equal to

            `abstol`+epsilon max(|a|, |b|),

        where epsilon is the machine precision. If `abstol` is less than
        or equal to zero, then epsilon||T||_1 will be used in its place,
        where T is the tridiagonal matrix obtained by reducing C to
        tridiagonal form. Eigenvalues will be computed most accurately
        when `abstol` is set to twice the underflow threshold
        2*machine.real_safe(), not zero. If this function returns with
        `errno` = 1 ... `n`, indicating that some eigenvectors did not
        converge, try setting `abstol` to 2*machine.real_safe(). See
        Demmel and Kahan (1990).

    Returns
    -------
    ap : float, ndarray, shape (`n`*(`n`+1)/2)
        The contents of `ap` are destroyed.

    bp : float, ndarray, shape (`n`*(`n`+1)/2)
        The triangular factor U or L from the Cholesky factorization B =
        U^TU or B = LL^T, in the same storage format as B.

    m : int
        The total number of eigenvalues found. 0 <= `m` <= `n`.

        If `erange` = 'A', `m` = `n`.

        If `erange` = 'I', `m` = `iu`-`il`+1.

    w : float, ndarray, shape (`n`)
        The first `m` elements contain the selected eigenvalues in
        ascending order.

    z : float, ndarray, shape (:, :)
        If `jobz` = 'V', then

            if no exception or warning is raised, the first `m` columns
            of Z contain the orthonormal eigenvectors of the matrix A
            corresponding to the selected eigenvalues, with the ith
            column of Z holding the eigenvector associated with
            `w`[i-1]. The eigenvectors are normalized as follows:

                if `itype` = 1 or 2, Z^TBZ = I;

                if `itype` = 3, Z^TB^-1Z = I;

            if an eigenvector fails to converge (`errno` = 1 ... `n`),
            then that column of Z contains the latest approximation to
            the eigenvector, and the index of the eigenvector is
            returned in `jfail`.

        If `jobz` = 'N', `z` is not referenced.

        **Note:** you must ensure that at least max(1, `m`) columns are
        supplied in the array `z`; if `erange` = 'V', the exact value of
        `m` is not known in advance and an upper bound of at least `n`
        must be used.

    jfail : int, ndarray, shape (`n`)
        If `jobz` = 'V', then

            if no exception or warning is raised, the first `m` elements
            of `jfail` are zero;

            if `errno` = 1 ... `n`, the first errno elements of `jfail`
            contains the indices of the eigenvectors that failed to
            converge.

        If `jobz` = 'N', `jfail` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `itype`.

            Constraint: `itype` = 1, 2 or 3.

        (`errno` -2)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `erange`.

            Constraint: `erange` = 'A', 'V' or 'I'.

        (`errno` -4)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -5)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -9)
            On entry, error in parameter `vu`.

            Constraint: `vl` < `vu`.

        (`errno` -10)
            On entry, error in parameter `il`.

        (`errno` -11)
            On entry, error in parameter `iu`.

        (`errno` i > `n`)
            If errno = `n`+*<value>*, for 1 <= *<value>* <= `n`, then
            the leading minor of order *<value>* of B is not positive
            definite. The factorization of B could not be completed and
            no eigenvalues or eigenvectors were computed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i<=`n`))
            The algorithm failed to converge; *<value>* eigenvectors did
            not converge. Their indices are stored in array `jfail`.

    Notes
    -----
    ``dspgvx`` first performs a Cholesky factorization of the matrix B
    as B = U^TU, when `uplo` = 'U' or B = LL^T, when `uplo` = 'L'.
    The generalized problem is then reduced to a standard symmetric
    eigenvalue problem

        Cx = lambda x,

    which is solved for the desired eigenvalues and eigenvectors; the
    eigenvectors are then backtransformed to give the eigenvectors of
    the original problem.

    For the problem Az = lambda Bz, the eigenvectors are normalized so
    that the matrix of eigenvectors, Z, satisfies

        Z^TAZ = Lambda and Z^TBZ = I,

    where Lambda is the diagonal matrix whose diagonal elements are the
    eigenvalues.
    For the problem ABz = lambda z we correspondingly have

        Z^-1AZ^-T = Lambda and Z^TBZ = I,

    and for BAz = lambda z we have

        Z^TAZ = Lambda and Z^TB^-1Z = I.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Demmel, J W and Kahan, W, 1990, `Accurate singular values of
    bidiagonal matrices`, SIAM J. Sci. Statist. Comput. (11), 873--912

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dspgvx'

    _ap = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ap'},
        ap,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _bp = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'bp'},
        bp,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobz = _b_utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = _b_utils._EngineCharScalarType.to_py(_jobz)
    _erange = _b_utils._EngineCharScalarType(
        erange, {'fun_name': fun_name, 'entity': 'erange'}, exp_chrlen=1,
    ).to_ctypes()
    _erange_py = _b_utils._EngineCharScalarType.to_py(_erange)
    _iu = _b_utils._EngineIntScalarType(
        iu, {'fun_name': fun_name, 'entity': 'iu'},
    ).to_ctypes()
    _iu_py = _b_utils._EngineIntScalarType.to_py(_iu)
    _il = _b_utils._EngineIntScalarType(
        il, {'fun_name': fun_name, 'entity': 'il'},
    ).to_ctypes()
    _il_py = _b_utils._EngineIntScalarType.to_py(_il)
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        if _erange_py in ('I', 'i'):
            _z_dim2 = _iu_py-_il_py+1
        else:
            _z_dim2 = _n_py
    else:
        _z_dim2 = 0
    exp_shape_z = (_z_dim1, _z_dim2)
    _z = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_z,
        sorder=_b_utils._EngineData.default_sorder,
    )
    _jfail = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        _return_scalar = _b_lapackeig.dspgvx(
            itype,
            jobz,
            erange,
            uplo,
            _n,
            _ap,
            _bp,
            vl,
            vu,
            _il,
            _iu,
            abstol,
            _w,
            _z,
            _jfail,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _ap_py = _ap
    _bp_py = _bp
    _m_py = _return_scalar
    _w_py = _w
    if _jobz_py in ('V', 'v'):
        _z_dim2 = _m_py
    else:
        _z_dim2 = 0
    _z_py = _z[:, :_z_dim2]
    _jfail_py = _jfail
    _return_data = DspgvxReturnData(
        _ap_py,
        _bp_py,
        _m_py,
        _w_py,
        _z_py,
        _jfail_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

DspgvdReturnData = _collections.namedtuple(
    'DspgvdReturnData',
    ['ap', 'bp', 'w', 'z'],
)

def dspgvd(itype, jobz, uplo, n, ap, bp):
    u"""
    Computes all the eigenvalues, and optionally, the eigenvectors of a
    real generalized symmetric-definite eigenproblem, packed storage
    (divide-and-conquer).

    ``dspgvd`` computes all the eigenvalues and, optionally, the
    eigenvectors of a real generalized symmetric-definite eigenproblem,
    of the form

        Az = lambda Bz, ABz = lambda z or BAz = lambda z,

    where A and B are symmetric, stored in packed format, and B is also
    positive definite.
    If eigenvectors are desired, it uses a divide-and-conquer algorithm.

    For full information please refer to the NAG Library document for
    f08tc

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08tcf.html

    Parameters
    ----------
    itype : int
        Specifies the problem type to be solved.

        `itype` = 1

            Az = lambda Bz.

        `itype` = 2

            ABz = lambda z.

        `itype` = 3

            BAz = lambda z.

    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

            Only eigenvalues are computed.

        `jobz` = 'V'

            Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangles of A and B are stored.

        If `uplo` = 'L', the lower triangles of A and B are stored.

    n : int
        n, the order of the matrices A and B.

    ap : float, array-like, shape (`n`*(`n`+1)/2)
        The upper or lower triangle of the n*n symmetric matrix A,
        packed by columns.

    bp : float, array-like, shape (`n`*(`n`+1)/2)
        The upper or lower triangle of the n*n symmetric matrix B,
        packed by columns.

    Returns
    -------
    ap : float, ndarray, shape (`n`*(`n`+1)/2)
        The contents of `ap` are destroyed.

    bp : float, ndarray, shape (`n`*(`n`+1)/2)
        The triangular factor U or L from the Cholesky factorization B =
        U^TU or B = LL^T, in the same storage format as B.

    w : float, ndarray, shape (`n`)
        The eigenvalues in ascending order.

    z : float, ndarray, shape (:, :)
        If `jobz` = 'V', `z` contains the matrix Z of eigenvectors. The
        eigenvectors are normalized as follows:

            if `itype` = 1 or 2, Z^TBZ = I;

            if `itype` = 3, Z^TB^-1Z = I.

        If `jobz` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `itype`.

            Constraint: `itype` = 1, 2 or 3.

        (`errno` -2)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` i<=`n`)
            The algorithm failed to converge; *<value>* off-diagonal
            elements of an intermediate tridiagonal form did not
            converge to zero.

        (`errno` i > `n`)
            If errno = `n`+*<value>*, for 1 <= *<value>* <= `n`, then
            the leading minor of order *<value>* of B is not positive
            definite. The factorization of B could not be completed and
            no eigenvalues or eigenvectors were computed.

    Notes
    -----
    ``dspgvd`` first performs a Cholesky factorization of the matrix B
    as B = U^TU, when `uplo` = 'U' or B = LL^T, when `uplo` = 'L'.
    The generalized problem is then reduced to a standard symmetric
    eigenvalue problem

        Cx = lambda x,

    which is solved for the eigenvalues and, optionally, the
    eigenvectors; the eigenvectors are then backtransformed to give the
    eigenvectors of the original problem.

    For the problem Az = lambda Bz, the eigenvectors are normalized so
    that the matrix of eigenvectors, z, satisfies

        Z^TAZ = Lambda and Z^TBZ = I,

    where Lambda is the diagonal matrix whose diagonal elements are the
    eigenvalues.
    For the problem ABz = lambda z we correspondingly have

        Z^-1AZ^-T = Lambda and Z^TBZ = I,

    and for BAz = lambda z we have

        Z^TAZ = Lambda and Z^TB^-1Z = I.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dspgvd'

    _ap = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ap'},
        ap,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _bp = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'bp'},
        bp,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobz = _b_utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = _b_utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        _z_dim2 = _n_py
    else:
        _z_dim2 = 1
    exp_shape_z = (_z_dim1, _z_dim2)
    _z = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_z,
        sorder=_b_utils._EngineData.default_sorder,
    )
    _b_lapackeig.dspgvd(
        itype,
        jobz,
        uplo,
        _n,
        _ap,
        _bp,
        _w,
        _z,
    )
    _ap_py = _ap
    _bp_py = _bp
    _w_py = _w
    _z_py = _z
    _return_data = DspgvdReturnData(
        _ap_py,
        _bp_py,
        _w_py,
        _z_py,
    )
    return _return_data

def dspgst(itype, uplo, n, ap, bp):
    u"""
    Performs a reduction to standard form of real symmetric-definite
    generalized eigenproblem Ax = lambda Bx, ABx = lambda x or BAx =
    lambda x, packed storage, B factorized by ``lapacklin.dpptrf``.

    ``dspgst`` reduces a real symmetric-definite generalized
    eigenproblem Az = lambda Bz, ABz = lambda z or BAz = lambda z to the
    standard form Cy = lambda y, where A is a real symmetric matrix and
    B has been factorized by ``lapacklin.dpptrf``, using packed storage.

    For full information please refer to the NAG Library document for
    f08te

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08tef.html

    Parameters
    ----------
    itype : int
        Indicates how the standard form is computed.

        `itype` = 1

                if `uplo` = 'U', C = U^-TAU^-1;

                if `uplo` = 'L', C = L^-1AL^-T.

        `itype` = 2 or 3

                if `uplo` = 'U', C = UAU^T;

                if `uplo` = 'L', C = L^TAL.

    uplo : str, length 1
        Indicates whether the upper or lower triangular part of A is
        stored and how B has been factorized.

        `uplo` = 'U'

            The upper triangular part of A is stored and B = U^TU.

        `uplo` = 'L'

            The lower triangular part of A is stored and B = LL^T.

    n : int
        n, the order of the matrices A and B.

    ap : float, array-like, shape (`n`*(`n`+1)/2)
        The upper or lower triangle of the n*n symmetric matrix A,
        packed by columns.

    bp : float, array-like, shape (`n`*(`n`+1)/2)
        The Cholesky factor of B as specified by `uplo` and returned by
        ``lapacklin.dpptrf``.

    Returns
    -------
    ap : float, ndarray, shape (`n`*(`n`+1)/2)
        The upper or lower triangle of `ap` is overwritten by the
        corresponding upper or lower triangle of C as specified by
        `itype` and `uplo`, using the same packed storage format as
        described above.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `itype`.

            Constraint: `itype` = 1, 2 or 3.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Notes
    -----
    To reduce the real symmetric-definite generalized eigenproblem Az =
    lambda Bz, ABz = lambda z or BAz = lambda z to the standard form Cy
    = lambda y using packed storage, ``dspgst`` must be preceded by a
    call to ``lapacklin.dpptrf`` which computes the Cholesky
    factorization of B; B must be positive definite.

    The different problem types are specified by the argument `itype`,
    as indicated in the table below.
    The table shows how C is computed by the function, and also how the
    eigenvectors z of the original problem can be recovered from the
    eigenvectors of the standard form.

    [table omitted]

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dspgst'

    _ap = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ap'},
        ap,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _bp = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'bp'},
        bp,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b_lapackeig.dspgst(
        itype,
        uplo,
        n,
        _ap,
        _bp,
    )
    _ap_py = _ap
    return _ap_py

ZhpgvReturnData = _collections.namedtuple(
    'ZhpgvReturnData',
    ['ap', 'bp', 'w', 'z'],
)

def zhpgv(itype, jobz, uplo, n, ap, bp):
    u"""
    Computes all the eigenvalues, and optionally, the eigenvectors of a
    complex generalized Hermitian-definite eigenproblem, packed storage.

    ``zhpgv`` computes all the eigenvalues and, optionally, all the
    eigenvectors of a complex generalized Hermitian-definite
    eigenproblem, of the form

        Az = lambda Bz, ABz = lambda z or BAz = lambda z,

    where A and B are Hermitian, stored in packed format, and B is also
    positive definite.

    For full information please refer to the NAG Library document for
    f08tn

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08tnf.html

    Parameters
    ----------
    itype : int
        Specifies the problem type to be solved.

        `itype` = 1

            Az = lambda Bz.

        `itype` = 2

            ABz = lambda z.

        `itype` = 3

            BAz = lambda z.

    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

            Only eigenvalues are computed.

        `jobz` = 'V'

            Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangles of A and B are stored.

        If `uplo` = 'L', the lower triangles of A and B are stored.

    n : int
        n, the order of the matrices A and B.

    ap : complex, array-like, shape (`n`*(`n`+1)/2)
        The upper or lower triangle of the n*n Hermitian matrix A,
        packed by columns.

    bp : complex, array-like, shape (`n`*(`n`+1)/2)
        The upper or lower triangle of the n*n Hermitian matrix B,
        packed by columns.

    Returns
    -------
    ap : complex, ndarray, shape (`n`*(`n`+1)/2)
        The contents of `ap` are destroyed.

    bp : complex, ndarray, shape (`n`*(`n`+1)/2)
        The triangular factor U or L from the Cholesky factorization B =
        U^HU or B = LL^H, in the same storage format as B.

    w : float, ndarray, shape (`n`)
        The eigenvalues in ascending order.

    z : complex, ndarray, shape (:, :)
        If `jobz` = 'V', `z` contains the matrix Z of eigenvectors. The
        eigenvectors are normalized as follows:

            if `itype` = 1 or 2, Z^HBZ = I;

            if `itype` = 3, Z^HB^-1Z = I.

        If `jobz` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `itype`.

            Constraint: `itype` = 1, 2 or 3.

        (`errno` -2)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` i<=`n`)
            The algorithm failed to converge; *<value>* off-diagonal
            elements of an intermediate tridiagonal form did not
            converge to zero.

        (`errno` i > `n`)
            If errno = `n`+*<value>*, for 1 <= *<value>* <= `n`, then
            the leading minor of order *<value>* of B is not positive
            definite. The factorization of B could not be completed and
            no eigenvalues or eigenvectors were computed.

    Notes
    -----
    ``zhpgv`` first performs a Cholesky factorization of the matrix B as
    B = U^HU, when `uplo` = 'U' or B = LL^H, when `uplo` = 'L'.
    The generalized problem is then reduced to a standard symmetric
    eigenvalue problem

        Cx = lambda x,

    which is solved for the eigenvalues and, optionally, the
    eigenvectors; the eigenvectors are then backtransformed to give the
    eigenvectors of the original problem.

    For the problem Az = lambda Bz, the eigenvectors are normalized so
    that the matrix of eigenvectors, Z, satisfies

        Z^HAZ = Lambda and Z^HBZ = I,

    where Lambda is the diagonal matrix whose diagonal elements are the
    eigenvalues.
    For the problem ABz = lambda z we correspondingly have

        Z^-1AZ^-H = Lambda and Z^HBZ = I,

    and for BAz = lambda z we have

        Z^HAZ = Lambda and Z^HB^-1Z = I.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zhpgv'

    _ap = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ap'},
        ap,
        copy=True,
    )
    _bp = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'bp'},
        bp,
        copy=True,
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobz = _b_utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = _b_utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        _z_dim2 = _n_py
    else:
        _z_dim2 = 1
    exp_shape_z = (_z_dim1, _z_dim2)
    _z = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_z,
        sorder=_b_utils._EngineData.default_sorder,
    )
    _b_lapackeig.zhpgv(
        itype,
        jobz,
        uplo,
        _n,
        _ap,
        _bp,
        _w,
        _z,
    )
    _ap_py = _ap
    _bp_py = _bp
    _w_py = _w
    _z_py = _z
    _return_data = ZhpgvReturnData(
        _ap_py,
        _bp_py,
        _w_py,
        _z_py,
    )
    return _return_data

ZhpgvxReturnData = _collections.namedtuple(
    'ZhpgvxReturnData',
    ['ap', 'bp', 'm', 'w', 'z', 'jfail'],
)

def zhpgvx(itype, jobz, erange, uplo, n, ap, bp, vl, vu, il, iu, abstol):
    u"""
    Computes selected eigenvalues, and optionally, the eigenvectors of a
    complex generalized Hermitian-definite eigenproblem, packed storage.

    ``zhpgvx`` computes selected eigenvalues and, optionally,
    eigenvectors of a complex generalized Hermitian-definite
    eigenproblem, of the form

        Az = lambda Bz, ABz = lambda z or BAz = lambda z,

    where A and B are Hermitian, stored in packed format, and B is also
    positive definite.
    Eigenvalues and eigenvectors can be selected by specifying either a
    range of values or a range of indices for the desired eigenvalues.

    For full information please refer to the NAG Library document for
    f08tp

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08tpf.html

    Parameters
    ----------
    itype : int
        Specifies the problem type to be solved.

        `itype` = 1

            Az = lambda Bz.

        `itype` = 2

            ABz = lambda z.

        `itype` = 3

            BAz = lambda z.

    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

            Only eigenvalues are computed.

        `jobz` = 'V'

            Eigenvalues and eigenvectors are computed.

    erange : str, length 1
        If `erange` = 'A', all eigenvalues will be found.

        If `erange` = 'V', all eigenvalues in the half-open interval
        (`vl`,`vu`] will be found.

        If `erange` = 'I', the `il`th to `iu`th eigenvalues will be
        found.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangles of A and B are stored.

        If `uplo` = 'L', the lower triangles of A and B are stored.

    n : int
        n, the order of the matrices A and B.

    ap : complex, array-like, shape (`n`*(`n`+1)/2)
        The upper or lower triangle of the n*n Hermitian matrix A,
        packed by columns.

    bp : complex, array-like, shape (`n`*(`n`+1)/2)
        The upper or lower triangle of the n*n Hermitian matrix B,
        packed by columns.

    vl : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    vu : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    il : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    iu : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    abstol : float
        The absolute error tolerance for the eigenvalues. An approximate
        eigenvalue is accepted as converged when it is determined to lie
        in an interval [a,b] of width less than or equal to

            `abstol`+epsilon max(|a|, |b|),

        where epsilon is the machine precision. If `abstol` is less than
        or equal to zero, then epsilon||T||_1 will be used in its place,
        where T is the tridiagonal matrix obtained by reducing C to
        tridiagonal form. Eigenvalues will be computed most accurately
        when `abstol` is set to twice the underflow threshold
        2*machine.real_safe(), not zero. If this function returns with
        `errno` = 1 ... `n`, indicating that some eigenvectors did not
        converge, try setting `abstol` to 2*machine.real_safe(). See
        Demmel and Kahan (1990).

    Returns
    -------
    ap : complex, ndarray, shape (`n`*(`n`+1)/2)
        The contents of `ap` are destroyed.

    bp : complex, ndarray, shape (`n`*(`n`+1)/2)
        The triangular factor U or L from the Cholesky factorization B =
        U^HU or B = LL^H, in the same storage format as B.

    m : int
        The total number of eigenvalues found. 0 <= `m` <= `n`.

        If `erange` = 'A', `m` = `n`.

        If `erange` = 'I', `m` = `iu`-`il`+1.

    w : float, ndarray, shape (`n`)
        The first `m` elements contain the selected eigenvalues in
        ascending order.

    z : complex, ndarray, shape (:, :)
        If `jobz` = 'V', then

            if no exception or warning is raised, the first `m` columns
            of Z contain the orthonormal eigenvectors of the matrix A
            corresponding to the selected eigenvalues, with the ith
            column of Z holding the eigenvector associated with
            `w`[i-1]. The eigenvectors are normalized as follows:

                if `itype` = 1 or 2, Z^HBZ = I;

                if `itype` = 3, Z^HB^-1Z = I;

            if an eigenvector fails to converge (`errno` = 1 ... `n`),
            then that column of Z contains the latest approximation to
            the eigenvector, and the index of the eigenvector is
            returned in `jfail`.

        If `jobz` = 'N', `z` is not referenced.

        **Note:** you must ensure that at least max(1, `m`) columns are
        supplied in the array `z`; if `erange` = 'V', the exact value of
        `m` is not known in advance and an upper bound of at least `n`
        must be used.

    jfail : int, ndarray, shape (`n`)
        If `jobz` = 'V', then

            if no exception or warning is raised, the first `m` elements
            of `jfail` are zero;

            if `errno` = 1 ... `n`, the first errno elements of `jfail`
            contains the indices of the eigenvectors that failed to
            converge.

        If `jobz` = 'N', `jfail` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `itype`.

            Constraint: `itype` = 1, 2 or 3.

        (`errno` -2)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `erange`.

            Constraint: `erange` = 'A', 'V' or 'I'.

        (`errno` -4)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -5)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -9)
            On entry, error in parameter `vu`.

            Constraint: `vl` < `vu`.

        (`errno` -10)
            On entry, error in parameter `il`.

        (`errno` -11)
            On entry, error in parameter `iu`.

        (`errno` i > `n`)
            If errno = `n`+*<value>*, for 1 <= *<value>* <= `n`, then
            the leading minor of order *<value>* of B is not positive
            definite. The factorization of B could not be completed and
            no eigenvalues or eigenvectors were computed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i<=`n`)
            The algorithm failed to converge; *<value>* eigenvectors
            failed to converge.

    Notes
    -----
    ``zhpgvx`` first performs a Cholesky factorization of the matrix B
    as B = U^HU, when `uplo` = 'U' or B = LL^H, when `uplo` = 'L'.
    The generalized problem is then reduced to a standard symmetric
    eigenvalue problem

        Cx = lambda x,

    which is solved for the desired eigenvalues and eigenvectors; the
    eigenvectors are then backtransformed to give the eigenvectors of
    the original problem.

    For the problem Az = lambda Bz, the eigenvectors are normalized so
    that the matrix of eigenvectors, Z, satisfies

        Z^HAZ = Lambda and Z^HBZ = I,

    where Lambda is the diagonal matrix whose diagonal elements are the
    eigenvalues.
    For the problem ABz = lambda z we correspondingly have

        Z^-1AZ^-H = Lambda and Z^HBZ = I,

    and for BAz = lambda z we have

        Z^HAZ = Lambda and Z^HB^-1Z = I.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Demmel, J W and Kahan, W, 1990, `Accurate singular values of
    bidiagonal matrices`, SIAM J. Sci. Statist. Comput. (11), 873--912

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zhpgvx'

    _ap = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ap'},
        ap,
        copy=True,
    )
    _bp = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'bp'},
        bp,
        copy=True,
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobz = _b_utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = _b_utils._EngineCharScalarType.to_py(_jobz)
    _erange = _b_utils._EngineCharScalarType(
        erange, {'fun_name': fun_name, 'entity': 'erange'}, exp_chrlen=1,
    ).to_ctypes()
    _erange_py = _b_utils._EngineCharScalarType.to_py(_erange)
    _iu = _b_utils._EngineIntScalarType(
        iu, {'fun_name': fun_name, 'entity': 'iu'},
    ).to_ctypes()
    _iu_py = _b_utils._EngineIntScalarType.to_py(_iu)
    _il = _b_utils._EngineIntScalarType(
        il, {'fun_name': fun_name, 'entity': 'il'},
    ).to_ctypes()
    _il_py = _b_utils._EngineIntScalarType.to_py(_il)
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        if _erange_py in ('I', 'i'):
            _z_dim2 = _iu_py-_il_py+1
        else:
            _z_dim2 = _n_py
    else:
        _z_dim2 = 0
    exp_shape_z = (_z_dim1, _z_dim2)
    _z = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_z,
        sorder=_b_utils._EngineData.default_sorder,
    )
    _jfail = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        _return_scalar = _b_lapackeig.zhpgvx(
            itype,
            jobz,
            erange,
            uplo,
            _n,
            _ap,
            _bp,
            vl,
            vu,
            _il,
            _iu,
            abstol,
            _w,
            _z,
            _jfail,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _ap_py = _ap
    _bp_py = _bp
    _m_py = _return_scalar
    _w_py = _w
    if _jobz_py in ('V', 'v'):
        _z_dim2 = _m_py
    else:
        _z_dim2 = 0
    _z_py = _z[:, :_z_dim2]
    _jfail_py = _jfail
    _return_data = ZhpgvxReturnData(
        _ap_py,
        _bp_py,
        _m_py,
        _w_py,
        _z_py,
        _jfail_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

ZhpgvdReturnData = _collections.namedtuple(
    'ZhpgvdReturnData',
    ['ap', 'bp', 'w', 'z'],
)

def zhpgvd(itype, jobz, uplo, n, ap, bp):
    u"""
    Computes selected eigenvalues, and optionally, the eigenvectors of a
    complex generalized Hermitian-definite eigenproblem, packed storage
    (divide-and-conquer).

    ``zhpgvd`` computes all the eigenvalues and, optionally, the
    eigenvectors of a complex generalized Hermitian-definite
    eigenproblem, of the form

        Az = lambda Bz, ABz = lambda z or BAz = lambda z,

    where A and B are Hermitian, stored in packed format, and B is also
    positive definite.
    If eigenvectors are desired, it uses a divide-and-conquer algorithm.

    For full information please refer to the NAG Library document for
    f08tq

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08tqf.html

    Parameters
    ----------
    itype : int
        Specifies the problem type to be solved.

        `itype` = 1

            Az = lambda Bz.

        `itype` = 2

            ABz = lambda z.

        `itype` = 3

            BAz = lambda z.

    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

            Only eigenvalues are computed.

        `jobz` = 'V'

            Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangles of A and B are stored.

        If `uplo` = 'L', the lower triangles of A and B are stored.

    n : int
        n, the order of the matrices A and B.

    ap : complex, array-like, shape (`n`*(`n`+1)/2)
        The upper or lower triangle of the n*n Hermitian matrix A,
        packed by columns.

    bp : complex, array-like, shape (`n`*(`n`+1)/2)
        The upper or lower triangle of the n*n Hermitian matrix B,
        packed by columns.

    Returns
    -------
    ap : complex, ndarray, shape (`n`*(`n`+1)/2)
        The contents of `ap` are destroyed.

    bp : complex, ndarray, shape (`n`*(`n`+1)/2)
        The triangular factor U or L from the Cholesky factorization B =
        U^HU or B = LL^H, in the same storage format as B.

    w : float, ndarray, shape (`n`)
        The eigenvalues in ascending order.

    z : complex, ndarray, shape (:, :)
        If `jobz` = 'V', `z` contains the matrix Z of eigenvectors. The
        eigenvectors are normalized as follows:

            if `itype` = 1 or 2, Z^HBZ = I;

            if `itype` = 3, Z^HB^-1Z = I.

        If `jobz` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `itype`.

            Constraint: `itype` = 1, 2 or 3.

        (`errno` -2)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` i<=`n`)
            The algorithm failed to converge; *<value>* off-diagonal
            elements of an intermediate tridiagonal form did not
            converge to zero.

        (`errno` i > `n`)
            If errno = `n`+*<value>*, for 1 <= *<value>* <= `n`, then
            the leading minor of order *<value>* of B is not positive
            definite. The factorization of B could not be completed and
            no eigenvalues or eigenvectors were computed.

    Notes
    -----
    ``zhpgvd`` first performs a Cholesky factorization of the matrix B
    as B = U^HU, when `uplo` = 'U' or B = LL^H, when `uplo` = 'L'.
    The generalized problem is then reduced to a standard symmetric
    eigenvalue problem

        Cx = lambda x,

    which is solved for the eigenvalues and, optionally, the
    eigenvectors; the eigenvectors are then backtransformed to give the
    eigenvectors of the original problem.

    For the problem Az = lambda Bz, the eigenvectors are normalized so
    that the matrix of eigenvectors, z, satisfies

        Z^HAZ = Lambda and Z^HBZ = I,

    where Lambda is the diagonal matrix whose diagonal elements are the
    eigenvalues.
    For the problem ABz = lambda z we correspondingly have

        Z^-1AZ^-H = Lambda and Z^HBZ = I,

    and for BAz = lambda z we have

        Z^HAZ = Lambda and Z^HB^-1Z = I.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zhpgvd'

    _ap = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ap'},
        ap,
        copy=True,
    )
    _bp = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'bp'},
        bp,
        copy=True,
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobz = _b_utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = _b_utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        _z_dim2 = _n_py
    else:
        _z_dim2 = 1
    exp_shape_z = (_z_dim1, _z_dim2)
    _z = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_z,
        sorder=_b_utils._EngineData.default_sorder,
    )
    _b_lapackeig.zhpgvd(
        itype,
        jobz,
        uplo,
        _n,
        _ap,
        _bp,
        _w,
        _z,
    )
    _ap_py = _ap
    _bp_py = _bp
    _w_py = _w
    _z_py = _z
    _return_data = ZhpgvdReturnData(
        _ap_py,
        _bp_py,
        _w_py,
        _z_py,
    )
    return _return_data

def zhpgst(itype, uplo, n, ap, bp):
    u"""
    Performs a reduction to standard form of complex Hermitian-definite
    generalized eigenproblem Ax = lambda Bx, ABx = lambda x or BAx =
    lambda x, packed storage, B factorized by ``lapacklin.zpptrf``.

    ``zhpgst`` reduces a complex Hermitian-definite generalized
    eigenproblem Az = lambda Bz, ABz = lambda z or BAz = lambda z to the
    standard form Cy = lambda y, where A is a complex Hermitian matrix
    and B has been factorized by ``lapacklin.zpptrf``, using packed
    storage.

    For full information please refer to the NAG Library document for
    f08ts

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08tsf.html

    Parameters
    ----------
    itype : int
        Indicates how the standard form is computed.

        `itype` = 1

                if `uplo` = 'U', C = U^-HAU^-1;

                if `uplo` = 'L', C = L^-1AL^-H.

        `itype` = 2 or 3

                if `uplo` = 'U', C = UAU^H;

                if `uplo` = 'L', C = L^HAL.

    uplo : str, length 1
        Indicates whether the upper or lower triangular part of A is
        stored and how B has been factorized.

        `uplo` = 'U'

            The upper triangular part of A is stored and B = U^HU.

        `uplo` = 'L'

            The lower triangular part of A is stored and B = LL^H.

    n : int
        n, the order of the matrices A and B.

    ap : complex, array-like, shape (`n`*(`n`+1)/2)
        The upper or lower triangle of the n*n Hermitian matrix A,
        packed by columns.

    bp : complex, array-like, shape (`n`*(`n`+1)/2)
        The Cholesky factor of B as specified by `uplo` and returned by
        ``lapacklin.zpptrf``.

    Returns
    -------
    ap : complex, ndarray, shape (`n`*(`n`+1)/2)
        The upper or lower triangle of `ap` is overwritten by the
        corresponding upper or lower triangle of C as specified by
        `itype` and `uplo`, using the same packed storage format as
        described above.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `itype`.

            Constraint: `itype` = 1, 2 or 3.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Notes
    -----
    To reduce the complex Hermitian-definite generalized eigenproblem Az
    = lambda Bz, ABz = lambda z or BAz = lambda z to the standard form
    Cy = lambda y using packed storage, ``zhpgst`` must be preceded by a
    call to ``lapacklin.zpptrf`` which computes the Cholesky
    factorization of B; B must be positive definite.

    The different problem types are specified by the argument `itype`,
    as indicated in the table below.
    The table shows how C is computed by the function, and also how the
    eigenvectors z of the original problem can be recovered from the
    eigenvectors of the standard form.

    [table omitted]

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zhpgst'

    _ap = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ap'},
        ap,
        copy=True,
    )
    _bp = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'bp'},
        bp,
        copy=False,
    )
    _b_lapackeig.zhpgst(
        itype,
        uplo,
        n,
        _ap,
        _bp,
    )
    _ap_py = _ap
    return _ap_py

DsbgvReturnData = _collections.namedtuple(
    'DsbgvReturnData',
    ['ab', 'bb', 'w', 'z'],
)

def dsbgv(jobz, uplo, ka, kb, ab, bb):
    u"""
    Computes all the eigenvalues, and optionally, the eigenvectors of a
    real banded generalized symmetric-definite eigenproblem.

    ``dsbgv`` computes all the eigenvalues and, optionally, the
    eigenvectors of a real generalized symmetric-definite banded
    eigenproblem, of the form

        Az = lambda Bz,

    where A and B are symmetric and banded, and B is also positive
    definite.

    For full information please refer to the NAG Library document for
    f08ua

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08uaf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

            Only eigenvalues are computed.

        `jobz` = 'V'

            Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangles of A and B are stored.

        If `uplo` = 'L', the lower triangles of A and B are stored.

    ka : int
        If `uplo` = 'U', the number of superdiagonals, k_a, of the
        matrix A.

        If `uplo` = 'L', the number of subdiagonals, k_a, of the matrix
        A.

    kb : int
        If `uplo` = 'U', the number of superdiagonals, k_b, of the
        matrix B.

        If `uplo` = 'L', the number of subdiagonals, k_b, of the matrix
        B.

    ab : float, array-like, shape (`ka`+1, n)
        The upper or lower triangle of the n*n symmetric band matrix A.

    bb : float, array-like, shape (`kb`+1, n)
        The upper or lower triangle of the n*n symmetric band matrix B.

    Returns
    -------
    ab : float, ndarray, shape (`ka`+1, n)
        The contents of `ab` are overwritten.

    bb : float, ndarray, shape (`kb`+1, n)
        The factor S from the split Cholesky factorization B = S^TS, as
        returned by ``dpbstf``.

    w : float, ndarray, shape (n)
        The eigenvalues in ascending order.

    z : float, ndarray, shape (:, :)
        If `jobz` = 'V', `z` contains the matrix Z of eigenvectors, with
        the ith column of Z holding the eigenvector associated with
        `w`[i-1]. The eigenvectors are normalized so that Z^TBZ = I.

        If `jobz` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -4)
            On entry, error in parameter `ka`.

            Constraint: `ka` >= 0.

        (`errno` -5)
            On entry, error in parameter `kb`.

            Constraint: `ka` >= `kb` >= 0.

        (`errno` i<=n)
            The algorithm failed to converge; *<value>* off-diagonal
            elements of an intermediate tridiagonal form did not
            converge to zero.

        (`errno` i > n)
            If errno = n+*<value>*, for 1 <= *<value>* <= n, ``dpbstf``
            returned errno = *<value>*: B is not positive definite. The
            factorization of B could not be completed and no eigenvalues
            or eigenvectors were computed.

    Notes
    -----
    The generalized symmetric-definite band problem

        Az = lambda Bz

    is first reduced to a standard band symmetric problem

        Cx = lambda x,

    where C is a symmetric band matrix, using Wilkinson's modification
    to Crawford's algorithm (see Crawford (1973) and Wilkinson (1977)).
    The symmetric eigenvalue problem is then solved for the eigenvalues
    and the eigenvectors, if required, which are then backtransformed to
    the eigenvectors of the original problem.

    The eigenvectors are normalized so that the matrix of eigenvectors,
    Z, satisfies

        Z^TAZ = Lambda and Z^TBZ = I,

    where Lambda is the diagonal matrix whose diagonal elements are the
    eigenvalues.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Crawford, C R, 1973, `Reduction of a band-symmetric generalized
    eigenvalue problem`, Comm. ACM (16), 41--44

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore

    Wilkinson, J H, 1977, `Some recent advances in numerical linear
    algebra`, The State of the Art in Numerical Analysis, (ed D A H
    Jacobs), Academic Press
    """

    fun_name = 'naginterfaces.library.lapackeig.dsbgv'

    _ab = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ab'},
        ab,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _bb = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'bb'},
        bb,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_ab, 'ab'), (_bb, 'bb')),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobz = _b_utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = _b_utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        _z_dim2 = _n_py
    else:
        _z_dim2 = 1
    exp_shape_z = (_z_dim1, _z_dim2)
    _z = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_z,
        sorder=_m_sorder,
    )
    _b_lapackeig.dsbgv(
        jobz,
        uplo,
        _n,
        ka,
        kb,
        _ab,
        _bb,
        _w,
        _z,
    )
    _ab_py = _ab
    _bb_py = _bb
    _w_py = _w
    _z_py = _z
    _return_data = DsbgvReturnData(
        _ab_py,
        _bb_py,
        _w_py,
        _z_py,
    )
    return _return_data

DsbgvxReturnData = _collections.namedtuple(
    'DsbgvxReturnData',
    ['ab', 'bb', 'q', 'm', 'w', 'z', 'jfail'],
)

def dsbgvx(jobz, erange, uplo, ka, kb, ab, bb, vl, vu, il, iu, abstol):
    u"""
    Computes selected eigenvalues, and optionally, the eigenvectors of a
    real banded generalized symmetric-definite eigenproblem.

    ``dsbgvx`` computes selected eigenvalues and, optionally,
    eigenvectors of a real generalized symmetric-definite banded
    eigenproblem, of the form

        Az = lambda Bz,

    where A and B are symmetric and banded, and B is also positive
    definite.
    Eigenvalues and eigenvectors can be selected by specifying either
    all eigenvalues, a range of values or a range of indices for the
    desired eigenvalues.

    For full information please refer to the NAG Library document for
    f08ub

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08ubf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

            Only eigenvalues are computed.

        `jobz` = 'V'

            Eigenvalues and eigenvectors are computed.

    erange : str, length 1
        If `erange` = 'A', all eigenvalues will be found.

        If `erange` = 'V', all eigenvalues in the half-open interval
        (`vl`,`vu`] will be found.

        If `erange` = 'I', the `il`th to `iu`th eigenvalues will be
        found.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangles of A and B are stored.

        If `uplo` = 'L', the lower triangles of A and B are stored.

    ka : int
        If `uplo` = 'U', the number of superdiagonals, k_a, of the
        matrix A.

        If `uplo` = 'L', the number of subdiagonals, k_a, of the matrix
        A.

    kb : int
        If `uplo` = 'U', the number of superdiagonals, k_b, of the
        matrix B.

        If `uplo` = 'L', the number of subdiagonals, k_b, of the matrix
        B.

    ab : float, array-like, shape (`ka`+1, n)
        The upper or lower triangle of the n*n symmetric band matrix A.

    bb : float, array-like, shape (`kb`+1, n)
        The upper or lower triangle of the n*n symmetric positive
        definite band matrix B.

    vl : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    vu : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    il : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    iu : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    abstol : float
        The absolute error tolerance for the eigenvalues. An approximate
        eigenvalue is accepted as converged when it is determined to lie
        in an interval [a,b] of width less than or equal to

            `abstol`+epsilon max(|a|, |b|),

        where epsilon is the machine precision. If `abstol` is less than
        or equal to zero, then epsilon||T||_1 will be used in its place,
        where T is the tridiagonal matrix obtained by reducing C to
        tridiagonal form. Eigenvalues will be computed most accurately
        when `abstol` is set to twice the underflow threshold
        2*machine.real_safe(), not zero. If this function returns with
        `errno` = 1 ... `n`, indicating that some eigenvectors did not
        converge, try setting `abstol` to 2*machine.real_safe(). See
        Demmel and Kahan (1990).

    Returns
    -------
    ab : float, ndarray, shape (`ka`+1, n)
        The contents of `ab` are overwritten.

    bb : float, ndarray, shape (`kb`+1, n)
        The factor S from the split Cholesky factorization B = S^TS, as
        returned by ``dpbstf``.

    q : float, ndarray, shape (:, :)
        If `jobz` = 'V', the n*n matrix, Q used in the reduction of the
        standard form, i.e., Cx = lambda x, from symmetric banded to
        tridiagonal form.

        If `jobz` = 'N', `q` is not referenced.

    m : int
        The total number of eigenvalues found. 0 <= `m` <= n.

        If `erange` = 'A', `m` = n.

        If `erange` = 'I', `m` = `iu`-`il`+1.

    w : float, ndarray, shape (n)
        The eigenvalues in ascending order.

    z : float, ndarray, shape (:, :)
        If `jobz` = 'V', then

            if no exception or warning is raised, the first `m` columns
            of Z contain the eigenvectors corresponding to the selected
            eigenvalues, with the ith column of Z holding the
            eigenvector associated with `w`[i-1]. The eigenvectors are
            normalized so that Z^TBZ = I;

            if an eigenvector fails to converge (`errno` = 1 ... `n`),
            then that column of Z contains the latest approximation to
            the eigenvector, and the index of the eigenvector is
            returned in `jfail`.

        If `jobz` = 'N', `z` is not referenced.

        **Note:** you must ensure that at least max(1, `m`) columns are
        supplied in the array `z`; if `erange` = 'V', the exact value of
        `m` is not known in advance and an upper bound of at least n
        must be used.

    jfail : int, ndarray, shape (n)
        If `jobz` = 'V', then

            if no exception or warning is raised, the first `m` elements
            of `jfail` are zero;

            if `errno` = 1 ... `n`, the first errno elements of `jfail`
            contains the indices of the eigenvectors that failed to
            converge.

        If `jobz` = 'N', `jfail` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `erange`.

            Constraint: `erange` = 'A', 'V' or 'I'.

        (`errno` -3)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -5)
            On entry, error in parameter `ka`.

            Constraint: `ka` >= 0.

        (`errno` -6)
            On entry, error in parameter `kb`.

            Constraint: `ka` >= `kb` >= 0.

        (`errno` -14)
            On entry, error in parameter `vu`.

            Constraint: `vl` < `vu`.

        (`errno` -15)
            On entry, error in parameter `il`.

        (`errno` -16)
            On entry, error in parameter `iu`.

        (`errno` i > n)
            If errno = n+*<value>*, for 1 <= *<value>* <= n, ``dpbstf``
            returned errno = *<value>*: B is not positive definite. The
            factorization of B could not be completed and no eigenvalues
            or eigenvectors were computed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i<=n))
            The algorithm failed to converge; *<value>* eigenvectors did
            not converge. Their indices are stored in array `jfail`.

    Notes
    -----
    The generalized symmetric-definite band problem

        Az = lambda Bz

    is first reduced to a standard band symmetric problem

        Cx = lambda x,

    where C is a symmetric band matrix, using Wilkinson's modification
    to Crawford's algorithm (see Crawford (1973) and Wilkinson (1977)).
    The symmetric eigenvalue problem is then solved for the required
    eigenvalues and eigenvectors, and the eigenvectors are then
    backtransformed to the eigenvectors of the original problem.

    The eigenvectors are normalized so that

        z^TAz = lambda and z^TBz = 1.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Crawford, C R, 1973, `Reduction of a band-symmetric generalized
    eigenvalue problem`, Comm. ACM (16), 41--44

    Demmel, J W and Kahan, W, 1990, `Accurate singular values of
    bidiagonal matrices`, SIAM J. Sci. Statist. Comput. (11), 873--912

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore

    Wilkinson, J H, 1977, `Some recent advances in numerical linear
    algebra`, The State of the Art in Numerical Analysis, (ed D A H
    Jacobs), Academic Press
    """

    fun_name = 'naginterfaces.library.lapackeig.dsbgvx'

    _ab = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ab'},
        ab,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _bb = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'bb'},
        bb,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_ab, 'ab'), (_bb, 'bb')),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _jobz = _b_utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = _b_utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py in ('V', 'v'):
        _q_dim1 = _n_py
    else:
        _q_dim1 = 1
    if _jobz_py in ('V', 'v'):
        _q_dim2 = _n_py
    else:
        _q_dim2 = 1
    exp_shape_q = (_q_dim1, _q_dim2)
    _q = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_q,
        sorder=_m_sorder,
    )
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _erange = _b_utils._EngineCharScalarType(
        erange, {'fun_name': fun_name, 'entity': 'erange'}, exp_chrlen=1,
    ).to_ctypes()
    _erange_py = _b_utils._EngineCharScalarType.to_py(_erange)
    _iu = _b_utils._EngineIntScalarType(
        iu, {'fun_name': fun_name, 'entity': 'iu'},
    ).to_ctypes()
    _iu_py = _b_utils._EngineIntScalarType.to_py(_iu)
    _il = _b_utils._EngineIntScalarType(
        il, {'fun_name': fun_name, 'entity': 'il'},
    ).to_ctypes()
    _il_py = _b_utils._EngineIntScalarType.to_py(_il)
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        if _erange_py in ('I', 'i'):
            _z_dim2 = _iu_py-_il_py+1
        else:
            _z_dim2 = _n_py
    else:
        _z_dim2 = 0
    exp_shape_z = (_z_dim1, _z_dim2)
    _z = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_z,
        sorder=_m_sorder,
    )
    _jfail = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        _return_scalar = _b_lapackeig.dsbgvx(
            jobz,
            erange,
            uplo,
            _n,
            ka,
            kb,
            _ab,
            _bb,
            _q,
            vl,
            vu,
            _il,
            _iu,
            abstol,
            _w,
            _z,
            _jfail,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _ab_py = _ab
    _bb_py = _bb
    _q_py = _q
    _m_py = _return_scalar
    _w_py = _w
    if _jobz_py in ('V', 'v'):
        _z_dim2 = _m_py
    else:
        _z_dim2 = 0
    _z_py = _z[:, :_z_dim2]
    _jfail_py = _jfail
    _return_data = DsbgvxReturnData(
        _ab_py,
        _bb_py,
        _q_py,
        _m_py,
        _w_py,
        _z_py,
        _jfail_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

DsbgvdReturnData = _collections.namedtuple(
    'DsbgvdReturnData',
    ['ab', 'bb', 'w', 'z'],
)

def dsbgvd(jobz, uplo, ka, kb, ab, bb):
    u"""
    Computes all the eigenvalues, and optionally, the eigenvectors of a
    real banded generalized symmetric-definite eigenproblem
    (divide-and-conquer).

    ``dsbgvd`` computes all the eigenvalues and, optionally, the
    eigenvectors of a real generalized symmetric-definite banded
    eigenproblem, of the form

        Az = lambda Bz,

    where A and B are symmetric and banded, and B is also positive
    definite.
    If eigenvectors are desired, it uses a divide-and-conquer algorithm.

    For full information please refer to the NAG Library document for
    f08uc

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08ucf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

            Only eigenvalues are computed.

        `jobz` = 'V'

            Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangles of A and B are stored.

        If `uplo` = 'L', the lower triangles of A and B are stored.

    ka : int
        If `uplo` = 'U', the number of superdiagonals, k_a, of the
        matrix A.

        If `uplo` = 'L', the number of subdiagonals, k_a, of the matrix
        A.

    kb : int
        If `uplo` = 'U', the number of superdiagonals, k_b, of the
        matrix B.

        If `uplo` = 'L', the number of subdiagonals, k_b, of the matrix
        B.

    ab : float, array-like, shape (`ka`+1, n)
        The upper or lower triangle of the n*n symmetric band matrix A.

    bb : float, array-like, shape (`kb`+1, n)
        The upper or lower triangle of the n*n symmetric band matrix B.

    Returns
    -------
    ab : float, ndarray, shape (`ka`+1, n)
        The contents of `ab` are overwritten.

    bb : float, ndarray, shape (`kb`+1, n)
        The factor S from the split Cholesky factorization B = S^TS, as
        returned by ``dpbstf``.

    w : float, ndarray, shape (n)
        The eigenvalues in ascending order.

    z : float, ndarray, shape (:, :)
        If `jobz` = 'V', `z` contains the matrix Z of eigenvectors, with
        the ith column of Z holding the eigenvector associated with
        `w`[i-1]. The eigenvectors are normalized so that Z^TBZ = I.

        If `jobz` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -4)
            On entry, error in parameter `ka`.

            Constraint: `ka` >= 0.

        (`errno` -5)
            On entry, error in parameter `kb`.

            Constraint: `ka` >= `kb` >= 0.

        (`errno` i<=n)
            The algorithm failed to converge; *<value>* off-diagonal
            elements of an intermediate tridiagonal form did not
            converge to zero.

        (`errno` i > n)
            If errno = n+*<value>*, for 1 <= *<value>* <= n, ``dpbstf``
            returned errno = *<value>*: B is not positive definite. The
            factorization of B could not be completed and no eigenvalues
            or eigenvectors were computed.

    Notes
    -----
    The generalized symmetric-definite band problem

        Az = lambda Bz

    is first reduced to a standard band symmetric problem

        Cx = lambda x,

    where C is a symmetric band matrix, using Wilkinson's modification
    to Crawford's algorithm (see Crawford (1973) and Wilkinson (1977)).
    The symmetric eigenvalue problem is then solved for the eigenvalues
    and the eigenvectors, if required, which are then backtransformed to
    the eigenvectors of the original problem.

    The eigenvectors are normalized so that the matrix of eigenvectors,
    Z, satisfies

        Z^TAZ = Lambda and Z^TBZ = I,

    where Lambda is the diagonal matrix whose diagonal elements are the
    eigenvalues.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Crawford, C R, 1973, `Reduction of a band-symmetric generalized
    eigenvalue problem`, Comm. ACM (16), 41--44

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore

    Wilkinson, J H, 1977, `Some recent advances in numerical linear
    algebra`, The State of the Art in Numerical Analysis, (ed D A H
    Jacobs), Academic Press
    """

    fun_name = 'naginterfaces.library.lapackeig.dsbgvd'

    _ab = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ab'},
        ab,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _bb = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'bb'},
        bb,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_ab, 'ab'), (_bb, 'bb')),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobz = _b_utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = _b_utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        _z_dim2 = _n_py
    else:
        _z_dim2 = 1
    exp_shape_z = (_z_dim1, _z_dim2)
    _z = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_z,
        sorder=_m_sorder,
    )
    _b_lapackeig.dsbgvd(
        jobz,
        uplo,
        _n,
        ka,
        kb,
        _ab,
        _bb,
        _w,
        _z,
    )
    _ab_py = _ab
    _bb_py = _bb
    _w_py = _w
    _z_py = _z
    _return_data = DsbgvdReturnData(
        _ab_py,
        _bb_py,
        _w_py,
        _z_py,
    )
    return _return_data

DsbgstReturnData = _collections.namedtuple(
    'DsbgstReturnData',
    ['ab', 'x'],
)

def dsbgst(vect, uplo, ka, kb, ab, bb):
    u"""
    Performs a reduction of real symmetric-definite banded generalized
    eigenproblem Ax = lambda Bx to standard form Cy = lambda y, such
    that C has the same bandwidth as A.

    ``dsbgst`` reduces a real symmetric-definite generalized
    eigenproblem Az = lambda Bz to the standard form Cy = lambda y,
    where A and B are band matrices, A is a real symmetric matrix, and B
    has been factorized by ``dpbstf``.

    For full information please refer to the NAG Library document for
    f08ue

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08uef.html

    Parameters
    ----------
    vect : str, length 1
        Indicates whether X is to be returned.

        `vect` = 'N'

            X is not returned.

        `vect` = 'V'

            X is returned.

    uplo : str, length 1
        Indicates whether the upper or lower triangular part of A is
        stored.

        `uplo` = 'U'

            The upper triangular part of A is stored.

        `uplo` = 'L'

            The lower triangular part of A is stored.

    ka : int
        If `uplo` = 'U', the number of superdiagonals, k_a, of the
        matrix A.

        If `uplo` = 'L', the number of subdiagonals, k_a, of the matrix
        A.

    kb : int
        If `uplo` = 'U', the number of superdiagonals, k_b, of the
        matrix B.

        If `uplo` = 'L', the number of subdiagonals, k_b, of the matrix
        B.

    ab : float, array-like, shape (`ka`+1, n)
        The upper or lower triangle of the n*n symmetric band matrix A.

    bb : float, array-like, shape (`kb`+1, n)
        The banded split Cholesky factor of B as specified by `uplo`,
        `n` and `kb` and returned by ``dpbstf``.

    Returns
    -------
    ab : float, ndarray, shape (`ka`+1, n)
        The upper or lower triangle of `ab` is overwritten by the
        corresponding upper or lower triangle of C as specified by
        `uplo`.

    x : float, ndarray, shape (:, :)
        The n*n matrix X = S^-1Q, if `vect` = 'V'.

        If `vect` = 'N', `x` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `vect`.

            Constraint: `vect` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -4)
            On entry, error in parameter `ka`.

            Constraint: `ka` >= 0.

        (`errno` -5)
            On entry, error in parameter `kb`.

            Constraint: `ka` >= `kb` >= 0.

    Notes
    -----
    To reduce the real symmetric-definite generalized eigenproblem Az =
    lambda Bz to the standard form Cy = lambda y, where A, B and C are
    banded, ``dsbgst`` must be preceded by a call to ``dpbstf`` which
    computes the split Cholesky factorization of the positive definite
    matrix B: B = S^TS.
    The split Cholesky factorization, compared with the ordinary
    Cholesky factorization, allows the work to be approximately halved.

    This function overwrites A with C = X^TAX, where X = S^-1Q and Q is
    a orthogonal matrix chosen (implicitly) to preserve the bandwidth of
    A.
    The function also has an option to allow the accumulation of X, and
    then, if z is an eigenvector of C, Xz is an eigenvector of the
    original system.

    References
    ----------
    Crawford, C R, 1973, `Reduction of a band-symmetric generalized
    eigenvalue problem`, Comm. ACM (16), 41--44

    Kaufman, L, 1984, `Banded eigenvalue solvers on vector machines`,
    ACM Trans. Math. Software (10), 73--86
    """

    fun_name = 'naginterfaces.library.lapackeig.dsbgst'

    _ab = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ab'},
        ab,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _bb = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'bb'},
        bb,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_ab, 'ab'), (_bb, 'bb')),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_ab, {'fun_name': fun_name, 'entity': 'ab'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _vect = _b_utils._EngineCharScalarType(
        vect, {'fun_name': fun_name, 'entity': 'vect'}, exp_chrlen=1,
    ).to_ctypes()
    _vect_py = _b_utils._EngineCharScalarType.to_py(_vect)
    if _vect_py in ('V', 'v'):
        _x_dim1 = _n_py
    elif _vect_py in ('N', 'n'):
        _x_dim1 = 1
    else:
        _x_dim1 = 0
    if _vect_py in ('V', 'v'):
        _x_dim2 = _n_py
    elif _vect_py in ('N', 'n'):
        _x_dim2 = 1
    else:
        _x_dim2 = 0
    exp_shape_x = (_x_dim1, _x_dim2)
    _x = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_x,
        sorder=_m_sorder,
    )
    _b_lapackeig.dsbgst(
        vect,
        uplo,
        _n,
        ka,
        kb,
        _ab,
        _bb,
        _x,
    )
    _ab_py = _ab
    _x_py = _x
    _return_data = DsbgstReturnData(
        _ab_py,
        _x_py,
    )
    return _return_data

def dpbstf(uplo, kb, bb):
    u"""
    Computes a split Cholesky factorization of real symmetric positive
    definite band matrix A.

    ``dpbstf`` computes a split Cholesky factorization of a real
    symmetric positive definite band matrix.

    For full information please refer to the NAG Library document for
    f08uf

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08uff.html

    Parameters
    ----------
    uplo : str, length 1
        Indicates whether the upper or lower triangular part of B is
        stored.

        `uplo` = 'U'

            The upper triangular part of B is stored.

        `uplo` = 'L'

            The lower triangular part of B is stored.

    kb : int
        If `uplo` = 'U', the number of superdiagonals, k_b, of the
        matrix B.

        If `uplo` = 'L', the number of subdiagonals, k_b, of the matrix
        B.

    bb : float, array-like, shape (`kb`+1, n)
        The n*n symmetric positive definite band matrix B.

    Returns
    -------
    bb : float, ndarray, shape (`kb`+1, n)
        B is overwritten by the elements of its split Cholesky factor S.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -3)
            On entry, error in parameter `kb`.

            Constraint: `kb` >= 0.

        (`errno` i > 0)
            The factorization could not be completed, because the
            updated element b(*<value>*,*<value>*) would be the square
            root of a negative number. Hence B is not positive definite.
            This may indicate an error in forming the matrix B.

    Notes
    -----
    ``dpbstf`` computes a split Cholesky factorization of a real
    symmetric positive definite band matrix B.
    It is designed to be used in conjunction with ``dsbgst``.

    The factorization has the form B = S^TS, where S is a band matrix of
    the same bandwidth as B and the following structure: S is upper
    triangular in the first (n+k)/2 rows, and transposed --- hence,
    lower triangular --- in the remaining rows.
    For example, if n = 9 and k = 2, then

        S = [table omitted].
    """

    fun_name = 'naginterfaces.library.lapackeig.dpbstf'

    _bb = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'bb'},
        bb,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_bb, {'fun_name': fun_name, 'entity': 'bb'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _b_lapackeig.dpbstf(
        uplo,
        _n,
        kb,
        _bb,
    )
    _bb_py = _bb
    return _bb_py

ZhbgvReturnData = _collections.namedtuple(
    'ZhbgvReturnData',
    ['ab', 'bb', 'w', 'z'],
)

def zhbgv(jobz, uplo, n, ka, kb, ab, bb):
    u"""
    Computes all the eigenvalues, and optionally, the eigenvectors of a
    complex banded generalized Hermitian-definite eigenproblem.

    ``zhbgv`` computes all the eigenvalues and, optionally, the
    eigenvectors of a complex generalized Hermitian-definite banded
    eigenproblem, of the form

        Az = lambda Bz,

    where A and B are Hermitian and banded, and B is also positive
    definite.

    For full information please refer to the NAG Library document for
    f08un

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08unf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

            Only eigenvalues are computed.

        `jobz` = 'V'

            Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangles of A and B are stored.

        If `uplo` = 'L', the lower triangles of A and B are stored.

    n : int
        n, the order of the matrices A and B.

    ka : int
        If `uplo` = 'U', the number of superdiagonals, k_a, of the
        matrix A.

        If `uplo` = 'L', the number of subdiagonals, k_a, of the matrix
        A.

    kb : int
        If `uplo` = 'U', the number of superdiagonals, k_b, of the
        matrix B.

        If `uplo` = 'L', the number of subdiagonals, k_b, of the matrix
        B.

    ab : complex, array-like, shape (`ka`+1, `n`)
        The upper or lower triangle of the n*n Hermitian band matrix A.

    bb : complex, array-like, shape (`kb`+1, `n`)
        The upper or lower triangle of the n*n Hermitian band matrix B.

    Returns
    -------
    ab : complex, ndarray, shape (`ka`+1, `n`)
        The contents of `ab` are overwritten.

    bb : complex, ndarray, shape (`kb`+1, `n`)
        The factor S from the split Cholesky factorization B = S^HS, as
        returned by ``zpbstf``.

    w : float, ndarray, shape (`n`)
        The eigenvalues in ascending order.

    z : complex, ndarray, shape (:, :)
        If `jobz` = 'V', `z` contains the matrix Z of eigenvectors, with
        the ith column of Z holding the eigenvector associated with
        `w`[i-1]. The eigenvectors are normalized so that Z^HBZ = I.

        If `jobz` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `ka`.

            Constraint: `ka` >= 0.

        (`errno` -5)
            On entry, error in parameter `kb`.

            Constraint: `ka` >= `kb` >= 0.

        (`errno` i<=`n`)
            The algorithm failed to converge; *<value>* off-diagonal
            elements of an intermediate tridiagonal form did not
            converge to zero.

        (`errno` i > `n`)
            If errno = `n`+*<value>*, for 1 <= *<value>* <= `n`,
            ``zpbstf`` returned errno = *<value>*: B is not positive
            definite. The factorization of B could not be completed and
            no eigenvalues or eigenvectors were computed.

    Notes
    -----
    The generalized Hermitian-definite band problem

        Az = lambda Bz

    is first reduced to a standard band Hermitian problem

        Cx = lambda x,

    where C is a Hermitian band matrix, using Wilkinson's modification
    to Crawford's algorithm (see Crawford (1973) and Wilkinson (1977)).
    The Hermitian eigenvalue problem is then solved for the eigenvalues
    and the eigenvectors, if required, which are then backtransformed to
    the eigenvectors of the original problem.

    The eigenvectors are normalized so that the matrix of eigenvectors,
    Z, satisfies

        Z^HAZ = Lambda and Z^HBZ = I,

    where Lambda is the diagonal matrix whose diagonal elements are the
    eigenvalues.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Crawford, C R, 1973, `Reduction of a band-symmetric generalized
    eigenvalue problem`, Comm. ACM (16), 41--44

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore

    Wilkinson, J H, 1977, `Some recent advances in numerical linear
    algebra`, The State of the Art in Numerical Analysis, (ed D A H
    Jacobs), Academic Press
    """

    fun_name = 'naginterfaces.library.lapackeig.zhbgv'

    _ab = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ab'},
        ab,
        copy=True,
    )
    _bb = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'bb'},
        bb,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_ab, 'ab'), (_bb, 'bb')),
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobz = _b_utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = _b_utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        _z_dim2 = _n_py
    else:
        _z_dim2 = 1
    exp_shape_z = (_z_dim1, _z_dim2)
    _z = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_z,
        sorder=_m_sorder,
    )
    _b_lapackeig.zhbgv(
        jobz,
        uplo,
        _n,
        ka,
        kb,
        _ab,
        _bb,
        _w,
        _z,
    )
    _ab_py = _ab
    _bb_py = _bb
    _w_py = _w
    _z_py = _z
    _return_data = ZhbgvReturnData(
        _ab_py,
        _bb_py,
        _w_py,
        _z_py,
    )
    return _return_data

ZhbgvxReturnData = _collections.namedtuple(
    'ZhbgvxReturnData',
    ['ab', 'bb', 'q', 'm', 'w', 'z', 'jfail'],
)

def zhbgvx(jobz, erange, uplo, n, ka, kb, ab, bb, vl, vu, il, iu, abstol):
    u"""
    Computes selected eigenvalues, and optionally, the eigenvectors of a
    complex banded generalized Hermitian-definite eigenproblem.

    ``zhbgvx`` computes selected the eigenvalues and, optionally, the
    eigenvectors of a complex generalized Hermitian-definite banded
    eigenproblem, of the form

        Az = lambda Bz,

    where A and B are Hermitian and banded, and B is also positive
    definite.
    Eigenvalues and eigenvectors can be selected by specifying either
    all eigenvalues, a range of values or a range of indices for the
    desired eigenvalues.

    For full information please refer to the NAG Library document for
    f08up

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08upf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

            Only eigenvalues are computed.

        `jobz` = 'V'

            Eigenvalues and eigenvectors are computed.

    erange : str, length 1
        If `erange` = 'A', all eigenvalues will be found.

        If `erange` = 'V', all eigenvalues in the half-open interval
        (`vl`,`vu`] will be found.

        If `erange` = 'I', the `il`th to `iu`th eigenvalues will be
        found.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangles of A and B are stored.

        If `uplo` = 'L', the lower triangles of A and B are stored.

    n : int
        n, the order of the matrices A and B.

    ka : int
        If `uplo` = 'U', the number of superdiagonals, k_a, of the
        matrix A.

        If `uplo` = 'L', the number of subdiagonals, k_a, of the matrix
        A.

    kb : int
        If `uplo` = 'U', the number of superdiagonals, k_b, of the
        matrix B.

        If `uplo` = 'L', the number of subdiagonals, k_b, of the matrix
        B.

    ab : complex, array-like, shape (`ka`+1, `n`)
        The upper or lower triangle of the n*n Hermitian band matrix A.

    bb : complex, array-like, shape (`kb`+1, `n`)
        The upper or lower triangle of the n*n Hermitian positive
        definite band matrix B.

    vl : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    vu : float
        If `erange` = 'V', the lower and upper bounds of the interval to
        be searched for eigenvalues.

        If `erange` = 'A' or 'I', `vl` and `vu` are not referenced.

    il : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    iu : int
        If `erange` = 'I', `il` and `iu` specify the indices (in
        ascending order) of the smallest and largest eigenvalues to be
        returned, respectively.

        If `erange` = 'A' or 'V', `il` and `iu` are not referenced.

    abstol : float
        The absolute error tolerance for the eigenvalues. An approximate
        eigenvalue is accepted as converged when it is determined to lie
        in an interval [a,b] of width less than or equal to

            `abstol`+epsilon max(|a|, |b|),

        where epsilon is the machine precision. If `abstol` is less than
        or equal to zero, then epsilon||T||_1 will be used in its place,
        where T is the tridiagonal matrix obtained by reducing C to
        tridiagonal form. Eigenvalues will be computed most accurately
        when `abstol` is set to twice the underflow threshold
        2*machine.real_safe(), not zero. If this function returns with
        `errno` = 1 ... `n`, indicating that some eigenvectors did not
        converge, try setting `abstol` to 2*machine.real_safe(). See
        Demmel and Kahan (1990).

    Returns
    -------
    ab : complex, ndarray, shape (`ka`+1, `n`)
        The contents of `ab` are overwritten.

    bb : complex, ndarray, shape (`kb`+1, `n`)
        The factor S from the split Cholesky factorization B = S^HS, as
        returned by ``zpbstf``.

    q : complex, ndarray, shape (:, :)
        If `jobz` = 'V', the n*n matrix, Q used in the reduction of the
        standard form, i.e., Cx = lambda x, from symmetric banded to
        tridiagonal form.

        If `jobz` = 'N', `q` is not referenced.

    m : int
        The total number of eigenvalues found. 0 <= `m` <= `n`.

        If `erange` = 'A', `m` = `n`.

        If `erange` = 'I', `m` = `iu`-`il`+1.

    w : float, ndarray, shape (`n`)
        The eigenvalues in ascending order.

    z : complex, ndarray, shape (:, :)
        If `jobz` = 'V', then

            if no exception or warning is raised, the first `m` columns
            of Z contain the eigenvectors corresponding to the selected
            eigenvalues, with the ith column of Z holding the
            eigenvector associated with `w`[i-1]. The eigenvectors are
            normalized so that Z^HBZ = I;

            if an eigenvector fails to converge (`errno` = 1 ... `n`),
            then that column of Z contains the latest approximation to
            the eigenvector, and the index of the eigenvector is
            returned in `jfail`.

        If `jobz` = 'N', `z` is not referenced.

        **Note:** you must ensure that at least max(1, `m`) columns are
        supplied in the array `z`; if `erange` = 'V', the exact value of
        `m` is not known in advance and an upper bound of at least `n`
        must be used.

    jfail : int, ndarray, shape (`n`)
        If `jobz` = 'V', then

            if no exception or warning is raised, the first `m` elements
            of `jfail` are zero;

            if `errno` = 1 ... `n`, the first errno elements of `jfail`
            contains the indices of the eigenvectors that failed to
            converge.

        If `jobz` = 'N', `jfail` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `erange`.

            Constraint: `erange` = 'A', 'V' or 'I'.

        (`errno` -3)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -4)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -5)
            On entry, error in parameter `ka`.

            Constraint: `ka` >= 0.

        (`errno` -6)
            On entry, error in parameter `kb`.

            Constraint: `ka` >= `kb` >= 0.

        (`errno` -14)
            On entry, error in parameter `vu`.

            Constraint: `vl` < `vu`.

        (`errno` -15)
            On entry, error in parameter `il`.

        (`errno` -16)
            On entry, error in parameter `iu`.

        (`errno` i > `n`)
            If errno = `n`+*<value>*, for 1 <= *<value>* <= `n`,
            ``zpbstf`` returned errno = *<value>*: B is not positive
            definite. The factorization of B could not be completed and
            no eigenvalues or eigenvectors were computed.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i<=`n`))
            The algorithm failed to converge; *<value>* eigenvectors did
            not converge. Their indices are stored in array `jfail`.

    Notes
    -----
    The generalized Hermitian-definite band problem

        Az = lambda Bz

    is first reduced to a standard band Hermitian problem

        Cx = lambda x,

    where C is a Hermitian band matrix, using Wilkinson's modification
    to Crawford's algorithm (see Crawford (1973) and Wilkinson (1977)).
    The Hermitian eigenvalue problem is then solved for the required
    eigenvalues and eigenvectors, and the eigenvectors are then
    backtransformed to the eigenvectors of the original problem.

    The eigenvectors are normalized so that

        z^HAz = lambda and z^HBz = 1.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Crawford, C R, 1973, `Reduction of a band-symmetric generalized
    eigenvalue problem`, Comm. ACM (16), 41--44

    Demmel, J W and Kahan, W, 1990, `Accurate singular values of
    bidiagonal matrices`, SIAM J. Sci. Statist. Comput. (11), 873--912

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore

    Wilkinson, J H, 1977, `Some recent advances in numerical linear
    algebra`, The State of the Art in Numerical Analysis, (ed D A H
    Jacobs), Academic Press
    """

    fun_name = 'naginterfaces.library.lapackeig.zhbgvx'

    _ab = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ab'},
        ab,
        copy=True,
    )
    _bb = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'bb'},
        bb,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_ab, 'ab'), (_bb, 'bb')),
    )
    _jobz = _b_utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = _b_utils._EngineCharScalarType.to_py(_jobz)
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    if _jobz_py in ('V', 'v'):
        _q_dim1 = _n_py
    else:
        _q_dim1 = 1
    if _jobz_py in ('V', 'v'):
        _q_dim2 = _n_py
    else:
        _q_dim2 = 1
    exp_shape_q = (_q_dim1, _q_dim2)
    _q = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_q,
        sorder=_m_sorder,
    )
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _erange = _b_utils._EngineCharScalarType(
        erange, {'fun_name': fun_name, 'entity': 'erange'}, exp_chrlen=1,
    ).to_ctypes()
    _erange_py = _b_utils._EngineCharScalarType.to_py(_erange)
    _iu = _b_utils._EngineIntScalarType(
        iu, {'fun_name': fun_name, 'entity': 'iu'},
    ).to_ctypes()
    _iu_py = _b_utils._EngineIntScalarType.to_py(_iu)
    _il = _b_utils._EngineIntScalarType(
        il, {'fun_name': fun_name, 'entity': 'il'},
    ).to_ctypes()
    _il_py = _b_utils._EngineIntScalarType.to_py(_il)
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        if _erange_py in ('I', 'i'):
            _z_dim2 = _iu_py-_il_py+1
        else:
            _z_dim2 = _n_py
    else:
        _z_dim2 = 0
    exp_shape_z = (_z_dim1, _z_dim2)
    _z = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_z,
        sorder=_m_sorder,
    )
    _jfail = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        _return_scalar = _b_lapackeig.zhbgvx(
            jobz,
            erange,
            uplo,
            _n,
            ka,
            kb,
            _ab,
            _bb,
            _q,
            vl,
            vu,
            _il,
            _iu,
            abstol,
            _w,
            _z,
            _jfail,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _ab_py = _ab
    _bb_py = _bb
    _q_py = _q
    _m_py = _return_scalar
    _w_py = _w
    if _jobz_py in ('V', 'v'):
        _z_dim2 = _m_py
    else:
        _z_dim2 = 0
    _z_py = _z[:, :_z_dim2]
    _jfail_py = _jfail
    _return_data = ZhbgvxReturnData(
        _ab_py,
        _bb_py,
        _q_py,
        _m_py,
        _w_py,
        _z_py,
        _jfail_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

ZhbgvdReturnData = _collections.namedtuple(
    'ZhbgvdReturnData',
    ['ab', 'bb', 'w', 'z'],
)

def zhbgvd(jobz, uplo, n, ka, kb, ab, bb):
    u"""
    Computes all the eigenvalues, and optionally, the eigenvectors of a
    complex banded generalized Hermitian-definite eigenproblem
    (divide-and-conquer).

    ``zhbgvd`` computes all the eigenvalues and, optionally, the
    eigenvectors of a complex generalized Hermitian-definite banded
    eigenproblem, of the form

        Az = lambda Bz,

    where A and B are Hermitian and banded, and B is also positive
    definite.
    If eigenvectors are desired, it uses a divide-and-conquer algorithm.

    For full information please refer to the NAG Library document for
    f08uq

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08uqf.html

    Parameters
    ----------
    jobz : str, length 1
        Indicates whether eigenvectors are computed.

        `jobz` = 'N'

            Only eigenvalues are computed.

        `jobz` = 'V'

            Eigenvalues and eigenvectors are computed.

    uplo : str, length 1
        If `uplo` = 'U', the upper triangles of A and B are stored.

        If `uplo` = 'L', the lower triangles of A and B are stored.

    n : int
        n, the order of the matrices A and B.

    ka : int
        If `uplo` = 'U', the number of superdiagonals, k_a, of the
        matrix A.

        If `uplo` = 'L', the number of subdiagonals, k_a, of the matrix
        A.

    kb : int
        If `uplo` = 'U', the number of superdiagonals, k_b, of the
        matrix B.

        If `uplo` = 'L', the number of subdiagonals, k_b, of the matrix
        B.

    ab : complex, array-like, shape (`ka`+1, `n`)
        The upper or lower triangle of the n*n Hermitian band matrix A.

    bb : complex, array-like, shape (`kb`+1, `n`)
        The upper or lower triangle of the n*n Hermitian band matrix B.

    Returns
    -------
    ab : complex, ndarray, shape (`ka`+1, `n`)
        The contents of `ab` are overwritten.

    bb : complex, ndarray, shape (`kb`+1, `n`)
        The factor S from the split Cholesky factorization B = S^HS, as
        returned by ``zpbstf``.

    w : float, ndarray, shape (`n`)
        The eigenvalues in ascending order.

    z : complex, ndarray, shape (:, :)
        If `jobz` = 'V', `z` contains the matrix Z of eigenvectors, with
        the ith column of Z holding the eigenvector associated with
        `w`[i-1]. The eigenvectors are normalized so that Z^HBZ = I.

        If `jobz` = 'N', `z` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobz`.

            Constraint: `jobz` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `ka`.

            Constraint: `ka` >= 0.

        (`errno` -5)
            On entry, error in parameter `kb`.

            Constraint: `ka` >= `kb` >= 0.

        (`errno` i<=`n`)
            The algorithm failed to converge; *<value>* off-diagonal
            elements of an intermediate tridiagonal form did not
            converge to zero.

        (`errno` i > `n`)
            If errno = `n`+*<value>*, for 1 <= *<value>* <= `n`,
            ``zpbstf`` returned errno = *<value>*: B is not positive
            definite. The factorization of B could not be completed and
            no eigenvalues or eigenvectors were computed.

    Notes
    -----
    The generalized Hermitian-definite band problem

        Az = lambda Bz

    is first reduced to a standard band Hermitian problem

        Cx = lambda x,

    where C is a Hermitian band matrix, using Wilkinson's modification
    to Crawford's algorithm (see Crawford (1973) and Wilkinson (1977)).
    The Hermitian eigenvalue problem is then solved for the eigenvalues
    and the eigenvectors, if required, which are then backtransformed to
    the eigenvectors of the original problem.

    The eigenvectors are normalized so that the matrix of eigenvectors,
    Z, satisfies

        Z^HAZ = Lambda and Z^HBZ = I,

    where Lambda is the diagonal matrix whose diagonal elements are the
    eigenvalues.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Crawford, C R, 1973, `Reduction of a band-symmetric generalized
    eigenvalue problem`, Comm. ACM (16), 41--44

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore

    Wilkinson, J H, 1977, `Some recent advances in numerical linear
    algebra`, The State of the Art in Numerical Analysis, (ed D A H
    Jacobs), Academic Press
    """

    fun_name = 'naginterfaces.library.lapackeig.zhbgvd'

    _ab = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ab'},
        ab,
        copy=True,
    )
    _bb = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'bb'},
        bb,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_ab, 'ab'), (_bb, 'bb')),
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _w = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobz = _b_utils._EngineCharScalarType(
        jobz, {'fun_name': fun_name, 'entity': 'jobz'}, exp_chrlen=1,
    ).to_ctypes()
    _jobz_py = _b_utils._EngineCharScalarType.to_py(_jobz)
    if _jobz_py in ('V', 'v'):
        _z_dim1 = _n_py
    else:
        _z_dim1 = 1
    if _jobz_py in ('V', 'v'):
        _z_dim2 = _n_py
    else:
        _z_dim2 = 1
    exp_shape_z = (_z_dim1, _z_dim2)
    _z = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_z,
        sorder=_m_sorder,
    )
    _b_lapackeig.zhbgvd(
        jobz,
        uplo,
        _n,
        ka,
        kb,
        _ab,
        _bb,
        _w,
        _z,
    )
    _ab_py = _ab
    _bb_py = _bb
    _w_py = _w
    _z_py = _z
    _return_data = ZhbgvdReturnData(
        _ab_py,
        _bb_py,
        _w_py,
        _z_py,
    )
    return _return_data

ZhbgstReturnData = _collections.namedtuple(
    'ZhbgstReturnData',
    ['ab', 'x'],
)

def zhbgst(vect, uplo, n, ka, kb, ab, bb):
    u"""
    Performs a reduction of complex Hermitian-definite banded
    generalized eigenproblem Ax = lambda Bx to standard form Cy = lambda
    y, such that C has the same bandwidth as A.

    ``zhbgst`` reduces a complex Hermitian-definite generalized
    eigenproblem Az = lambda Bz to the standard form Cy = lambda y,
    where A and B are band matrices, A is a complex Hermitian matrix,
    and B has been factorized by ``zpbstf``.

    For full information please refer to the NAG Library document for
    f08us

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08usf.html

    Parameters
    ----------
    vect : str, length 1
        Indicates whether X is to be returned.

        `vect` = 'N'

            X is not returned.

        `vect` = 'V'

            X is returned.

    uplo : str, length 1
        Indicates whether the upper or lower triangular part of A is
        stored.

        `uplo` = 'U'

            The upper triangular part of A is stored.

        `uplo` = 'L'

            The lower triangular part of A is stored.

    n : int
        n, the order of the matrices A and B.

    ka : int
        If `uplo` = 'U', the number of superdiagonals, k_a, of the
        matrix A.

        If `uplo` = 'L', the number of subdiagonals, k_a, of the matrix
        A.

    kb : int
        If `uplo` = 'U', the number of superdiagonals, k_b, of the
        matrix B.

        If `uplo` = 'L', the number of subdiagonals, k_b, of the matrix
        B.

    ab : complex, array-like, shape (`ka`+1, `n`)
        The upper or lower triangle of the n*n Hermitian band matrix A.

    bb : complex, array-like, shape (`kb`+1, `n`)
        The banded split Cholesky factor of B as specified by `uplo`,
        `n` and `kb` and returned by ``zpbstf``.

    Returns
    -------
    ab : complex, ndarray, shape (`ka`+1, `n`)
        The upper or lower triangle of `ab` is overwritten by the
        corresponding upper or lower triangle of C as specified by
        `uplo`.

    x : complex, ndarray, shape (:, :)
        The n*n matrix X = S^-1Q, if `vect` = 'V'.

        If `vect` = 'N', `x` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `vect`.

            Constraint: `vect` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -3)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -4)
            On entry, error in parameter `ka`.

            Constraint: `ka` >= 0.

        (`errno` -5)
            On entry, error in parameter `kb`.

            Constraint: `ka` >= `kb` >= 0.

    Notes
    -----
    To reduce the complex Hermitian-definite generalized eigenproblem Az
    = lambda Bz to the standard form Cy = lambda y, where A, B and C are
    banded, ``zhbgst`` must be preceded by a call to ``zpbstf`` which
    computes the split Cholesky factorization of the positive definite
    matrix B: B = S^HS.
    The split Cholesky factorization, compared with the ordinary
    Cholesky factorization, allows the work to be approximately halved.

    This function overwrites A with C = X^HAX, where X = S^-1Q and Q is
    a unitary matrix chosen (implicitly) to preserve the bandwidth of A.
    The function also has an option to allow the accumulation of X, and
    then, if z is an eigenvector of C, Xz is an eigenvector of the
    original system.

    References
    ----------
    Crawford, C R, 1973, `Reduction of a band-symmetric generalized
    eigenvalue problem`, Comm. ACM (16), 41--44

    Kaufman, L, 1984, `Banded eigenvalue solvers on vector machines`,
    ACM Trans. Math. Software (10), 73--86
    """

    fun_name = 'naginterfaces.library.lapackeig.zhbgst'

    _ab = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ab'},
        ab,
        copy=True,
    )
    _bb = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'bb'},
        bb,
        copy=False,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_ab, 'ab'), (_bb, 'bb')),
    )
    _vect = _b_utils._EngineCharScalarType(
        vect, {'fun_name': fun_name, 'entity': 'vect'}, exp_chrlen=1,
    ).to_ctypes()
    _vect_py = _b_utils._EngineCharScalarType.to_py(_vect)
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    if _vect_py in ('V', 'v'):
        _x_dim1 = _n_py
    elif _vect_py in ('N', 'n'):
        _x_dim1 = 1
    else:
        _x_dim1 = 0
    if _vect_py in ('V', 'v'):
        _x_dim2 = _n_py
    elif _vect_py in ('N', 'n'):
        _x_dim2 = 1
    else:
        _x_dim2 = 0
    exp_shape_x = (_x_dim1, _x_dim2)
    _x = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_x,
        sorder=_m_sorder,
    )
    _b_lapackeig.zhbgst(
        vect,
        uplo,
        _n,
        ka,
        kb,
        _ab,
        _bb,
        _x,
    )
    _ab_py = _ab
    _x_py = _x
    _return_data = ZhbgstReturnData(
        _ab_py,
        _x_py,
    )
    return _return_data

def zpbstf(uplo, kb, bb):
    u"""
    Computes a split Cholesky factorization of complex Hermitian
    positive definite band matrix A.

    ``zpbstf`` computes a split Cholesky factorization of a complex
    Hermitian positive definite band matrix.

    For full information please refer to the NAG Library document for
    f08ut

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08utf.html

    Parameters
    ----------
    uplo : str, length 1
        Indicates whether the upper or lower triangular part of B is
        stored.

        `uplo` = 'U'

            The upper triangular part of B is stored.

        `uplo` = 'L'

            The lower triangular part of B is stored.

    kb : int
        If `uplo` = 'U', the number of superdiagonals, k_b, of the
        matrix B.

        If `uplo` = 'L', the number of subdiagonals, k_b, of the matrix
        B.

    bb : complex, array-like, shape (`kb`+1, n)
        The n*n Hermitian positive definite band matrix B.

    Returns
    -------
    bb : complex, ndarray, shape (`kb`+1, n)
        B is overwritten by the elements of its split Cholesky factor S.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `uplo`.

            Constraint: `uplo` = 'U' or 'L'.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -3)
            On entry, error in parameter `kb`.

            Constraint: `kb` >= 0.

        (`errno` i > 0)
            The factorization could not be completed, because the
            updated element b(*<value>*,*<value>*) would be the square
            root of a negative number. Hence B is not positive definite.
            This may indicate an error in forming the matrix B.

    Notes
    -----
    ``zpbstf`` computes a split Cholesky factorization of a complex
    Hermitian positive definite band matrix B.
    It is designed to be used in conjunction with ``zhbgst``.

    The factorization has the form B = S^HS, where S is a band matrix of
    the same bandwidth as B and the following structure: S is upper
    triangular in the first (n+k)/2 rows, and transposed --- hence,
    lower triangular --- in the remaining rows.
    For example, if n = 9 and k = 2, then

        S = [table omitted].
    """

    fun_name = 'naginterfaces.library.lapackeig.zpbstf'

    _bb = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'bb'},
        bb,
        copy=True,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_bb, {'fun_name': fun_name, 'entity': 'bb'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _b_lapackeig.zpbstf(
        uplo,
        _n,
        kb,
        _bb,
    )
    _bb_py = _bb
    return _bb_py

DggsvdReturnData = _collections.namedtuple(
    'DggsvdReturnData',
    ['k', 'l', 'a', 'b', 'alpha', 'beta', 'u', 'v', 'q', 'iwork'],
)

@_b_utils._withdrawal(
    'naginterfaces.library.lapackeig.dggsvd',
    mkwdrn=None,
    advice=[
        'Please use dggsvd3 instead.',
        'See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html',
    ]
)
def dggsvd(jobu, jobv, jobq, a, b):
    u"""
    Computes the generalized singular value decomposition of a real
    matrix pair.

    ``dggsvd`` computes the generalized singular value decomposition
    (GSVD) of an m*n real matrix A and a p*n real matrix B. ``dggsvd``
    is marked as `deprecated` by LAPACK; the replacement routine is
    ``dggsvd3`` which makes better use of Level 3 BLAS.

    Note: ``dggsvd`` is deprecated.
    Please use ``dggsvd3`` instead.
    See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html

    For full information please refer to the NAG Library document for
    f08va

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08vaf.html

    Parameters
    ----------
    jobu : str, length 1
        If `jobu` = 'U', the orthogonal matrix U is computed.

        If `jobu` = 'N', U is not computed.

    jobv : str, length 1
        If `jobv` = 'V', the orthogonal matrix V is computed.

        If `jobv` = 'N', V is not computed.

    jobq : str, length 1
        If `jobq` = 'Q', the orthogonal matrix Q is computed.

        If `jobq` = 'N', Q is not computed.

    a : float, array-like, shape (m, n)
        The m*n matrix A.

    b : float, array-like, shape (p, n)
        The p*n matrix B.

    Returns
    -------
    k : int
        `k` and `l` specify the dimension of the subblocks k and l as
        described in [Notes]; (k+l) is the effective numerical rank of
        (A;B).

    l : int
        `k` and `l` specify the dimension of the subblocks k and l as
        described in [Notes]; (k+l) is the effective numerical rank of
        (A;B).

    a : float, ndarray, shape (m, n)
        Contains the triangular matrix R, or part of R. See [Notes] for
        details.

    b : float, ndarray, shape (p, n)
        Contains the triangular matrix R if m-k-l < 0. See [Notes] for
        details.

    alpha : float, ndarray, shape (n)
        See the description of `beta`.

    beta : float, ndarray, shape (n)
        `alpha` and `beta` contain the generalized singular value pairs
        of A and B, alpha_i and beta_i;

            `alpha`[0:`k`] = 1,

            `beta`[0:`k`] = 0,

        and if m-k-l >= 0,

            `alpha`[`k`:`k`+`l`] = C,

            `beta`[`k`:`k`+`l`] = S,

        or if m-k-l < 0,

            `alpha`[`k`:m] = C,

            `alpha`[m:`k`+`l`] = 0,

            `beta`[`k`:m] = S,

            `beta`[m:`k`+`l`] = 1, and

            `alpha`[`k`+`l`:n] = 0,

            `beta`[`k`+`l`:n] = 0.

        The notation `alpha`[`k`-1:n] above refers to consecutive
        elements `alpha`[i-1], for i = `k`,...,n.

    u : float, ndarray, shape (:, :)
        If `jobu` = 'U', `u` contains the m*m orthogonal matrix U.

        If `jobu` = 'N', `u` is not referenced.

    v : float, ndarray, shape (:, :)
        If `jobv` = 'V', `v` contains the p*p orthogonal matrix V.

        If `jobv` = 'N', `v` is not referenced.

    q : float, ndarray, shape (:, :)
        If `jobq` = 'Q', `q` contains the n*n orthogonal matrix Q.

        If `jobq` = 'N', `q` is not referenced.

    iwork : int, ndarray, shape (n)
        Stores the sorting information. More precisely, the following
        loop will sort `alpha`

        ::

          for i in range(k+1, min(m,k+l)+1):
            swap alpha[i-1] and alpha[iwork[i-1]-1]

        such that `alpha`[0] >= `alpha`[1] >= ... >= `alpha`[n-1].

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobu`.

            Constraint: `jobu` = 'U' or 'N'.

        (`errno` -2)
            On entry, error in parameter `jobv`.

            Constraint: `jobv` = 'V' or 'N'.

        (`errno` -3)
            On entry, error in parameter `jobq`.

            Constraint: `jobq` = 'Q' or 'N'.

        (`errno` -4)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -5)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -6)
            On entry, error in parameter p.

            Constraint: p >= 0.

        (`errno` 1)
            The Jacobi-type procedure failed to converge.

    Notes
    -----
    The generalized singular value decomposition is given by

        U^TAQ = D_1(0 R), V^TBQ = D_2(0 R),

    where U, V and Q are orthogonal matrices.
    Let (k+l) be the effective numerical rank of the matrix (A;B), then
    R is a (k+l)*(k+l) nonsingular upper triangular matrix, D_1 and D_2
    are m*(k+l) and p*(k+l) 'diagonal' matrices structured as follows:

    if m-k-l >= 0,

        D_1 = [table omitted]

        D_2 = [table omitted]

        (0 R) = [table omitted]

    where

        C = diag(alpha_{k+1},...,alpha_{k+l}),

        S = diag(beta_{k+1},...,beta_{k+l}),

    and

        C^2+S^2 = I.

    R is stored as a submatrix of A with elements R_{ij} stored as
    A_{i,n-k-l+j} on exit.

    If m-k-l < 0,

        D_1 = [table omitted]

        D_2 = [table omitted]

        (0 R) = [table omitted]

    where

        C = diag(alpha_{k+1},...,alpha_m),

        S = diag(beta_{k+1},...,beta_m),

    and

        C^2+S^2 = I.

    [table omitted] is stored as a submatrix of A with R_{ij} stored as
    A_{i,n-k-l+j}, and R_33 is stored as a submatrix of B with
    (R_33)_{ij} stored as B_{m-k+i,n+m-k-l+j}.

    The function computes C, S, R and, optionally, the orthogonal
    transformation matrices U, V and Q.

    In particular, if B is an n*n nonsingular matrix, then the GSVD of A
    and B implicitly gives the SVD of AB^-1:

        AB^-1 = U(D_1 D_2^-1)V^T.

    If (A;B) has orthonormal columns, then the GSVD of A and B is also
    equal to the CS decomposition of A and B.
    Furthermore, the GSVD can be used to derive the solution of the
    eigenvalue problem:

        A^TAx = lambda B^TBx.

    In some literature, the GSVD of A and B is presented in the form

        U^TAX = (0 D_1), V^TBX = (0 D_2),

    where U and V are orthogonal and X is nonsingular, and D_1 and D_2
    are 'diagonal'.
    The former GSVD form can be converted to the latter form by taking
    the nonsingular matrix X as

        X = Q[table omitted].

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dggsvd'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'), (_b, 'b')),
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _p_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[0]
    _p = _b_utils._EngineIntScalarType(
        _p_py, {'fun_name': fun_name, 'entity': '_p'},
    ).to_ctypes()
    _alpha = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _beta = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobu = _b_utils._EngineCharScalarType(
        jobu, {'fun_name': fun_name, 'entity': 'jobu'}, exp_chrlen=1,
    ).to_ctypes()
    _jobu_py = _b_utils._EngineCharScalarType.to_py(_jobu)
    if _jobu_py in ('U', 'u'):
        _u_dim1 = _m_py
    else:
        _u_dim1 = 1
    if _jobu_py in ('U', 'u'):
        _u_dim2 = _m_py
    else:
        _u_dim2 = 0
    exp_shape_u = (_u_dim1, _u_dim2)
    _u = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_u,
        sorder=_m_sorder,
    )
    _jobv = _b_utils._EngineCharScalarType(
        jobv, {'fun_name': fun_name, 'entity': 'jobv'}, exp_chrlen=1,
    ).to_ctypes()
    _jobv_py = _b_utils._EngineCharScalarType.to_py(_jobv)
    if _jobv_py in ('V', 'v'):
        _v_dim1 = _p_py
    else:
        _v_dim1 = 1
    if _jobv_py in ('V', 'v'):
        _v_dim2 = _p_py
    else:
        _v_dim2 = 0
    exp_shape_v = (_v_dim1, _v_dim2)
    _v = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_v,
        sorder=_m_sorder,
    )
    _jobq = _b_utils._EngineCharScalarType(
        jobq, {'fun_name': fun_name, 'entity': 'jobq'}, exp_chrlen=1,
    ).to_ctypes()
    _jobq_py = _b_utils._EngineCharScalarType.to_py(_jobq)
    if _jobq_py in ('Q', 'q'):
        _q_dim1 = _n_py
    else:
        _q_dim1 = 1
    if _jobq_py in ('Q', 'q'):
        _q_dim2 = _n_py
    else:
        _q_dim2 = 0
    exp_shape_q = (_q_dim1, _q_dim2)
    _q = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_q,
        sorder=_m_sorder,
    )
    _iwork = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    _return_scalars = _b_lapackeig.dggsvd(
        jobu,
        jobv,
        jobq,
        _m,
        _n,
        _p,
        _a,
        _b,
        _alpha,
        _beta,
        _u,
        _v,
        _q,
        _iwork,
    )
    _a_py = _a
    _b_py = _b
    _k_py = _return_scalars.k
    _l_py = _return_scalars.l
    _alpha_py = _alpha
    _beta_py = _beta
    _u_py = _u
    _v_py = _v
    _q_py = _q
    _iwork_py = _iwork
    _return_data = DggsvdReturnData(
        _k_py,
        _l_py,
        _a_py,
        _b_py,
        _alpha_py,
        _beta_py,
        _u_py,
        _v_py,
        _q_py,
        _iwork_py,
    )
    return _return_data

Dggsvd3ReturnData = _collections.namedtuple(
    'Dggsvd3ReturnData',
    ['k', 'l', 'a', 'b', 'alpha', 'beta', 'u', 'v', 'q', 'iwork'],
)

def dggsvd3(jobu, jobv, jobq, a, b):
    u"""
    Computes, using BLAS-3, the generalized singular value decomposition
    of a real matrix pair.

    ``dggsvd3`` computes the generalized singular value decomposition
    (GSVD) of an m*n real matrix A and a p*n real matrix B.

    For full information please refer to the NAG Library document for
    f08vc

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08vcf.html

    Parameters
    ----------
    jobu : str, length 1
        If `jobu` = 'U', the orthogonal matrix U is computed.

        If `jobu` = 'N', U is not computed.

    jobv : str, length 1
        If `jobv` = 'V', the orthogonal matrix V is computed.

        If `jobv` = 'N', V is not computed.

    jobq : str, length 1
        If `jobq` = 'Q', the orthogonal matrix Q is computed.

        If `jobq` = 'N', Q is not computed.

    a : float, array-like, shape (m, n)
        The m*n matrix A.

    b : float, array-like, shape (p, n)
        The p*n matrix B.

    Returns
    -------
    k : int
        `k` and `l` specify the dimension of the subblocks k and l as
        described in [Notes]; (k+l) is the effective numerical rank of
        (A;B).

    l : int
        `k` and `l` specify the dimension of the subblocks k and l as
        described in [Notes]; (k+l) is the effective numerical rank of
        (A;B).

    a : float, ndarray, shape (m, n)
        Contains the triangular matrix R, or part of R. See [Notes] for
        details.

    b : float, ndarray, shape (p, n)
        Contains the triangular matrix R if m-k-l < 0. See [Notes] for
        details.

    alpha : float, ndarray, shape (n)
        See the description of `beta`.

    beta : float, ndarray, shape (n)
        `alpha` and `beta` contain the generalized singular value pairs
        of A and B, alpha_i and beta_i;

            `alpha`[0:`k`] = 1,

            `beta`[0:`k`] = 0,

        and if m-k-l >= 0,

            `alpha`[`k`:`k`+`l`] = C,

            `beta`[`k`:`k`+`l`] = S,

        or if m-k-l < 0,

            `alpha`[`k`:m] = C,

            `alpha`[m:`k`+`l`] = 0,

            `beta`[`k`:m] = S,

            `beta`[m:`k`+`l`] = 1, and

            `alpha`[`k`+`l`:n] = 0,

            `beta`[`k`+`l`:n] = 0.

        The notation `alpha`[`k`-1:n] above refers to consecutive
        elements `alpha`[i-1], for i = `k`,...,n.

    u : float, ndarray, shape (:, :)
        If `jobu` = 'U', `u` contains the m*m orthogonal matrix U.

        If `jobu` = 'N', `u` is not referenced.

    v : float, ndarray, shape (:, :)
        If `jobv` = 'V', `v` contains the p*p orthogonal matrix V.

        If `jobv` = 'N', `v` is not referenced.

    q : float, ndarray, shape (:, :)
        If `jobq` = 'Q', `q` contains the n*n orthogonal matrix Q.

        If `jobq` = 'N', `q` is not referenced.

    iwork : int, ndarray, shape (n)
        Stores the sorting information. More precisely, if I is the
        ordered set of indices of `alpha` containing C (denote as
        `alpha`[I], see `beta`), then the corresponding elements
        `iwork`[I]-1 contain the swap pivots, J, that sorts I such that
        `alpha`[I] is in descending numerical order.

        The following pseudocode sorts the set I:

        for i in I; j = J_i; swap I_i and I_j;end

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobu`.

            Constraint: `jobu` = 'U' or 'N'.

        (`errno` -2)
            On entry, error in parameter `jobv`.

            Constraint: `jobv` = 'V' or 'N'.

        (`errno` -3)
            On entry, error in parameter `jobq`.

            Constraint: `jobq` = 'Q' or 'N'.

        (`errno` -4)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -5)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -6)
            On entry, error in parameter p.

            Constraint: p >= 0.

        (`errno` 1)
            The Jacobi-type procedure failed to converge.

    Notes
    -----
    Given an m*n real matrix A and a p*n real matrix B, the generalized
    singular value decomposition is given by

        U^TAQ = D_1(0 R), V^TBQ = D_2(0 R),

    where U, V and Q are orthogonal matrices.
    Let l be the effective numerical rank of B and (k+l) be the
    effective numerical rank of the matrix (A;B), then the first k
    generalized singular values are infinite and the remaining l are
    finite. R is a (k+l)*(k+l) nonsingular upper triangular matrix, D_1
    and D_2 are m*(k+l) and p*(k+l) 'diagonal' matrices structured as
    follows:

    if m-k-l >= 0,

        D_1 = [table omitted]

        D_2 = [table omitted]

        (0 R) = [table omitted]

    where

        C = diag(alpha_{k+1},...,alpha_{k+l}),

        S = diag(beta_{k+1},...,beta_{k+l}),

    and

        C^2+S^2 = I.

    R is stored as a submatrix of A with elements R_{ij} stored as
    A_{i,n-k-l+j} on exit.

    If m-k-l < 0,

        D_1 = [table omitted]

        D_2 = [table omitted]

        (0 R) = [table omitted]

    where

        C = diag(alpha_{k+1},...,alpha_m),

        S = diag(beta_{k+1},...,beta_m),

    and

        C^2+S^2 = I.

    [table omitted] is stored as a submatrix of A with R_{ij} stored as
    A_{i,n-k-l+j}, and R_33 is stored as a submatrix of B with
    (R_33)_{ij} stored as B_{m-k+i,n+m-k-l+j}.

    The function computes C, S, R and, optionally, the orthogonal
    transformation matrices U, V and Q.

    In particular, if B is an n*n nonsingular matrix, then the GSVD of A
    and B implicitly gives the SVD of AB^-1:

        AB^-1 = U(D_1 D_2^-1)V^T.

    If (A;B) has orthonormal columns, then the GSVD of A and B is also
    equal to the CS decomposition of A and B.
    Furthermore, the GSVD can be used to derive the solution of the
    eigenvalue problem:

        A^TAx = lambda B^TBx.

    In some literature, the GSVD of A and B is presented in the form

        U^TAX = (0 D_1), V^TBX = (0 D_2),

    where U and V are orthogonal and X is nonsingular, and D_1 and D_2
    are 'diagonal'.
    The former GSVD form can be converted to the latter form by setting

        X = Q[table omitted].

    A two stage process is used to compute the GSVD of the matrix pair
    (A,B).
    The pair is first reduced to upper triangular form by orthogonal
    transformations using ``dggsvp3``.
    The GSVD of the resulting upper triangular matrix pair is then
    performed by ``dtgsja`` which uses a variant of the Kogbetliantz
    algorithm (a cyclic Jacobi method).

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 2012, `Matrix Computations`, (4th
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dggsvd3'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'), (_b, 'b')),
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _p_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[0]
    _p = _b_utils._EngineIntScalarType(
        _p_py, {'fun_name': fun_name, 'entity': '_p'},
    ).to_ctypes()
    _alpha = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _beta = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobu = _b_utils._EngineCharScalarType(
        jobu, {'fun_name': fun_name, 'entity': 'jobu'}, exp_chrlen=1,
    ).to_ctypes()
    _jobu_py = _b_utils._EngineCharScalarType.to_py(_jobu)
    if _jobu_py in ('U', 'u'):
        _u_dim1 = _m_py
    else:
        _u_dim1 = 1
    if _jobu_py in ('U', 'u'):
        _u_dim2 = _m_py
    else:
        _u_dim2 = 1
    exp_shape_u = (_u_dim1, _u_dim2)
    _u = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_u,
        sorder=_m_sorder,
    )
    _jobv = _b_utils._EngineCharScalarType(
        jobv, {'fun_name': fun_name, 'entity': 'jobv'}, exp_chrlen=1,
    ).to_ctypes()
    _jobv_py = _b_utils._EngineCharScalarType.to_py(_jobv)
    if _jobv_py in ('V', 'v'):
        _v_dim1 = _p_py
    else:
        _v_dim1 = 1
    if _jobv_py in ('V', 'v'):
        _v_dim2 = _p_py
    else:
        _v_dim2 = 1
    exp_shape_v = (_v_dim1, _v_dim2)
    _v = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_v,
        sorder=_m_sorder,
    )
    _jobq = _b_utils._EngineCharScalarType(
        jobq, {'fun_name': fun_name, 'entity': 'jobq'}, exp_chrlen=1,
    ).to_ctypes()
    _jobq_py = _b_utils._EngineCharScalarType.to_py(_jobq)
    if _jobq_py in ('Q', 'q'):
        _q_dim1 = _n_py
    else:
        _q_dim1 = 1
    if _jobq_py in ('Q', 'q'):
        _q_dim2 = _n_py
    else:
        _q_dim2 = 1
    exp_shape_q = (_q_dim1, _q_dim2)
    _q = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_q,
        sorder=_m_sorder,
    )
    _iwork = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    _return_scalars = _b_lapackeig.dggsvd3(
        jobu,
        jobv,
        jobq,
        _m,
        _n,
        _p,
        _a,
        _b,
        _alpha,
        _beta,
        _u,
        _v,
        _q,
        _iwork,
    )
    _a_py = _a
    _b_py = _b
    _k_py = _return_scalars.k
    _l_py = _return_scalars.l
    _alpha_py = _alpha
    _beta_py = _beta
    _u_py = _u
    _v_py = _v
    _q_py = _q
    _iwork_py = _iwork
    _return_data = Dggsvd3ReturnData(
        _k_py,
        _l_py,
        _a_py,
        _b_py,
        _alpha_py,
        _beta_py,
        _u_py,
        _v_py,
        _q_py,
        _iwork_py,
    )
    return _return_data

DggsvpReturnData = _collections.namedtuple(
    'DggsvpReturnData',
    ['a', 'b', 'k', 'l', 'u', 'v', 'q'],
)

@_b_utils._withdrawal(
    'naginterfaces.library.lapackeig.dggsvp',
    mkwdrn=None,
    advice=[
        'Please use dggsvp3 instead.',
        'See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html',
    ]
)
def dggsvp(jobu, jobv, jobq, a, b, tola, tolb):
    u"""
    Produces orthogonal matrices that simultaneously reduce the m*n
    matrix A and the p*n matrix B to upper triangular form.

    ``dggsvp`` uses orthogonal transformations to simultaneously reduce
    the m*n matrix A and the p*n matrix B to upper triangular form.
    This factorization is usually used as a preprocessing step for
    computing the generalized singular value decomposition (GSVD).
    ``dggsvp`` is marked as `deprecated` by LAPACK; the replacement
    routine is ``dggsvp3`` which makes better use of Level 3 BLAS.

    Note: ``dggsvp`` is deprecated.
    Please use ``dggsvp3`` instead.
    See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html

    For full information please refer to the NAG Library document for
    f08ve

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08vef.html

    Parameters
    ----------
    jobu : str, length 1
        If `jobu` = 'U', the orthogonal matrix U is computed.

        If `jobu` = 'N', U is not computed.

    jobv : str, length 1
        If `jobv` = 'V', the orthogonal matrix V is computed.

        If `jobv` = 'N', V is not computed.

    jobq : str, length 1
        If `jobq` = 'Q', the orthogonal matrix Q is computed.

        If `jobq` = 'N', Q is not computed.

    a : float, array-like, shape (m, n)
        The m*n matrix A.

    b : float, array-like, shape (p, n)
        The p*n matrix B.

    tola : float
        `tola` and `tolb` are the thresholds to determine the effective
        numerical rank of matrix B and a subblock of A. Generally, they
        are set to

            `tola` = max(m, n)||A||epsilon,;`tolb` = max(p,
            n)||B||epsilon,

        where epsilon is the machine precision.

        The size of `tola` and `tolb` may affect the size of backward
        errors of the decomposition.

    tolb : float
        `tola` and `tolb` are the thresholds to determine the effective
        numerical rank of matrix B and a subblock of A. Generally, they
        are set to

            `tola` = max(m, n)||A||epsilon,;`tolb` = max(p,
            n)||B||epsilon,

        where epsilon is the machine precision.

        The size of `tola` and `tolb` may affect the size of backward
        errors of the decomposition.

    Returns
    -------
    a : float, ndarray, shape (m, n)
        Contains the triangular (or trapezoidal) matrix described in
        [Notes].

    b : float, ndarray, shape (p, n)
        Contains the triangular matrix described in [Notes].

    k : int
        `k` and `l` specify the dimension of the subblocks k and l as
        described in [Notes]; (k+l) is the effective numerical rank of
        (`a`^T `b`^T)^T.

    l : int
        `k` and `l` specify the dimension of the subblocks k and l as
        described in [Notes]; (k+l) is the effective numerical rank of
        (`a`^T `b`^T)^T.

    u : float, ndarray, shape (:, :)
        If `jobu` = 'U', `u` contains the orthogonal matrix U.

        If `jobu` = 'N', `u` is not referenced.

    v : float, ndarray, shape (:, :)
        If `jobv` = 'V', `v` contains the orthogonal matrix V.

        If `jobv` = 'N', `v` is not referenced.

    q : float, ndarray, shape (:, :)
        If `jobq` = 'Q', `q` contains the orthogonal matrix Q.

        If `jobq` = 'N', `q` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobu`.

            Constraint: `jobu` = 'U' or 'N'.

        (`errno` -2)
            On entry, error in parameter `jobv`.

            Constraint: `jobv` = 'V' or 'N'.

        (`errno` -3)
            On entry, error in parameter `jobq`.

            Constraint: `jobq` = 'Q' or 'N'.

        (`errno` -4)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -5)
            On entry, error in parameter p.

            Constraint: p >= 0.

        (`errno` -6)
            On entry, error in parameter n.

            Constraint: n >= 0.

    Notes
    -----
    ``dggsvp`` computes orthogonal matrices U, V and Q such that

        U^TAQ = { [table omitted], if m-k-l >= 0; ; [table omitted], if
        m-k-l < 0; ; V^TBQ = [table omitted]

    where the k*k matrix A_12 and l*l matrix B_13 are nonsingular upper
    triangular; A_23 is l*l upper triangular if m-k-l >= 0 and is
    (m-k)*l upper trapezoidal otherwise. (k+l) is the effective
    numerical rank of the (m+p)*n matrix (A^T B^T)^T.

    This decomposition is usually used as the preprocessing step for
    computing the Generalized Singular Value Decomposition (GSVD), see
    function ``dggsvd``.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dggsvp'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'), (_b, 'b')),
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _p_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[0]
    _p = _b_utils._EngineIntScalarType(
        _p_py, {'fun_name': fun_name, 'entity': '_p'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _jobu = _b_utils._EngineCharScalarType(
        jobu, {'fun_name': fun_name, 'entity': 'jobu'}, exp_chrlen=1,
    ).to_ctypes()
    _jobu_py = _b_utils._EngineCharScalarType.to_py(_jobu)
    if _jobu_py in ('U', 'u'):
        _u_dim1 = _m_py
    else:
        _u_dim1 = 1
    if _jobu_py in ('U', 'u'):
        _u_dim2 = _m_py
    else:
        _u_dim2 = 1
    exp_shape_u = (_u_dim1, _u_dim2)
    _u = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_u,
        sorder=_m_sorder,
    )
    _jobv = _b_utils._EngineCharScalarType(
        jobv, {'fun_name': fun_name, 'entity': 'jobv'}, exp_chrlen=1,
    ).to_ctypes()
    _jobv_py = _b_utils._EngineCharScalarType.to_py(_jobv)
    if _jobv_py in ('V', 'v'):
        _v_dim1 = _p_py
    else:
        _v_dim1 = 1
    if _jobv_py in ('V', 'v'):
        _v_dim2 = _p_py
    else:
        _v_dim2 = 1
    exp_shape_v = (_v_dim1, _v_dim2)
    _v = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_v,
        sorder=_m_sorder,
    )
    _jobq = _b_utils._EngineCharScalarType(
        jobq, {'fun_name': fun_name, 'entity': 'jobq'}, exp_chrlen=1,
    ).to_ctypes()
    _jobq_py = _b_utils._EngineCharScalarType.to_py(_jobq)
    if _jobq_py in ('Q', 'q'):
        _q_dim1 = _n_py
    else:
        _q_dim1 = 1
    if _jobq_py in ('Q', 'q'):
        _q_dim2 = _n_py
    else:
        _q_dim2 = 1
    exp_shape_q = (_q_dim1, _q_dim2)
    _q = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_q,
        sorder=_m_sorder,
    )
    _return_scalars = _b_lapackeig.dggsvp(
        jobu,
        jobv,
        jobq,
        _m,
        _p,
        _n,
        _a,
        _b,
        tola,
        tolb,
        _u,
        _v,
        _q,
    )
    _a_py = _a
    _b_py = _b
    _k_py = _return_scalars.k
    _l_py = _return_scalars.l
    _u_py = _u
    _v_py = _v
    _q_py = _q
    _return_data = DggsvpReturnData(
        _a_py,
        _b_py,
        _k_py,
        _l_py,
        _u_py,
        _v_py,
        _q_py,
    )
    return _return_data

Dggsvp3ReturnData = _collections.namedtuple(
    'Dggsvp3ReturnData',
    ['a', 'b', 'k', 'l', 'u', 'v', 'q'],
)

def dggsvp3(jobu, jobv, jobq, a, b, tola, tolb):
    u"""
    Produces orthogonal matrices, using BLAS-3, that simultaneously
    reduce the m*n matrix A and the p*n matrix B to upper triangular
    form.

    ``dggsvp3`` uses orthogonal transformations to simultaneously reduce
    the m*n matrix A and the p*n matrix B to upper triangular form.
    This factorization is usually used as a preprocessing step for
    computing the generalized singular value decomposition (GSVD).
    For sufficiently large problems, a blocked algorithm is used to make
    best use of Level 3 BLAS.

    For full information please refer to the NAG Library document for
    f08vg

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08vgf.html

    Parameters
    ----------
    jobu : str, length 1
        If `jobu` = 'U', the orthogonal matrix U is computed.

        If `jobu` = 'N', U is not computed.

    jobv : str, length 1
        If `jobv` = 'V', the orthogonal matrix V is computed.

        If `jobv` = 'N', V is not computed.

    jobq : str, length 1
        If `jobq` = 'Q', the orthogonal matrix Q is computed.

        If `jobq` = 'N', Q is not computed.

    a : float, array-like, shape (m, n)
        The m*n matrix A.

    b : float, array-like, shape (p, n)
        The p*n matrix B.

    tola : float
        `tola` and `tolb` are the thresholds to determine the effective
        numerical rank of matrix B and a subblock of A. Generally, they
        are set to

            `tola` = max(m, n)||A||epsilon,;`tolb` = max(p,
            n)||B||epsilon,

        where epsilon is the machine precision.

        The size of `tola` and `tolb` may affect the size of backward
        errors of the decomposition.

    tolb : float
        `tola` and `tolb` are the thresholds to determine the effective
        numerical rank of matrix B and a subblock of A. Generally, they
        are set to

            `tola` = max(m, n)||A||epsilon,;`tolb` = max(p,
            n)||B||epsilon,

        where epsilon is the machine precision.

        The size of `tola` and `tolb` may affect the size of backward
        errors of the decomposition.

    Returns
    -------
    a : float, ndarray, shape (m, n)
        Contains the triangular (or trapezoidal) matrix described in
        [Notes].

    b : float, ndarray, shape (p, n)
        Contains the triangular matrix described in [Notes].

    k : int
        `k` and `l` specify the dimension of the subblocks k and l as
        described in [Notes]; (k+l) is the effective numerical rank of
        (`a`^T `b`^T)^T.

    l : int
        `k` and `l` specify the dimension of the subblocks k and l as
        described in [Notes]; (k+l) is the effective numerical rank of
        (`a`^T `b`^T)^T.

    u : float, ndarray, shape (:, :)
        If `jobu` = 'U', `u` contains the orthogonal matrix U.

        If `jobu` = 'N', `u` is not referenced.

    v : float, ndarray, shape (:, :)
        If `jobv` = 'V', `v` contains the orthogonal matrix V.

        If `jobv` = 'N', `v` is not referenced.

    q : float, ndarray, shape (:, :)
        If `jobq` = 'Q', `q` contains the orthogonal matrix Q.

        If `jobq` = 'N', `q` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobu`.

            Constraint: `jobu` = 'U' or 'N'.

        (`errno` -2)
            On entry, error in parameter `jobv`.

            Constraint: `jobv` = 'V' or 'N'.

        (`errno` -3)
            On entry, error in parameter `jobq`.

            Constraint: `jobq` = 'Q' or 'N'.

        (`errno` -4)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -5)
            On entry, error in parameter p.

            Constraint: p >= 0.

        (`errno` -6)
            On entry, error in parameter n.

            Constraint: n >= 0.

    Notes
    -----
    ``dggsvp3`` computes orthogonal matrices U, V and Q such that

        U^TAQ = { [table omitted], if m-k-l >= 0; ; [table omitted], if
        m-k-l < 0; ; V^TBQ = [table omitted]

    where the k*k matrix A_12 and l*l matrix B_13 are nonsingular upper
    triangular; A_23 is l*l upper triangular if m-k-l >= 0 and is
    (m-k)*l upper trapezoidal otherwise. (k+l) is the effective
    numerical rank of the (m+p)*n matrix (A^T B^T)^T.

    This decomposition is usually used as the preprocessing step for
    computing the Generalized Singular Value Decomposition (GSVD), see
    function ``dtgsja``; the two steps are combined in ``dggsvd3``.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 2012, `Matrix Computations`, (4th
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dggsvp3'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'), (_b, 'b')),
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _p_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[0]
    _p = _b_utils._EngineIntScalarType(
        _p_py, {'fun_name': fun_name, 'entity': '_p'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _jobu = _b_utils._EngineCharScalarType(
        jobu, {'fun_name': fun_name, 'entity': 'jobu'}, exp_chrlen=1,
    ).to_ctypes()
    _jobu_py = _b_utils._EngineCharScalarType.to_py(_jobu)
    if _jobu_py in ('U', 'u'):
        _u_dim1 = _m_py
    else:
        _u_dim1 = 1
    if _jobu_py in ('U', 'u'):
        _u_dim2 = _m_py
    else:
        _u_dim2 = 1
    exp_shape_u = (_u_dim1, _u_dim2)
    _u = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_u,
        sorder=_m_sorder,
    )
    _jobv = _b_utils._EngineCharScalarType(
        jobv, {'fun_name': fun_name, 'entity': 'jobv'}, exp_chrlen=1,
    ).to_ctypes()
    _jobv_py = _b_utils._EngineCharScalarType.to_py(_jobv)
    if _jobv_py in ('V', 'v'):
        _v_dim1 = _p_py
    else:
        _v_dim1 = 1
    if _jobv_py in ('V', 'v'):
        _v_dim2 = _p_py
    else:
        _v_dim2 = 1
    exp_shape_v = (_v_dim1, _v_dim2)
    _v = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_v,
        sorder=_m_sorder,
    )
    _jobq = _b_utils._EngineCharScalarType(
        jobq, {'fun_name': fun_name, 'entity': 'jobq'}, exp_chrlen=1,
    ).to_ctypes()
    _jobq_py = _b_utils._EngineCharScalarType.to_py(_jobq)
    if _jobq_py in ('Q', 'q'):
        _q_dim1 = _n_py
    else:
        _q_dim1 = 1
    if _jobq_py in ('Q', 'q'):
        _q_dim2 = _n_py
    else:
        _q_dim2 = 1
    exp_shape_q = (_q_dim1, _q_dim2)
    _q = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_q,
        sorder=_m_sorder,
    )
    _return_scalars = _b_lapackeig.dggsvp3(
        jobu,
        jobv,
        jobq,
        _m,
        _p,
        _n,
        _a,
        _b,
        tola,
        tolb,
        _u,
        _v,
        _q,
    )
    _a_py = _a
    _b_py = _b
    _k_py = _return_scalars.k
    _l_py = _return_scalars.l
    _u_py = _u
    _v_py = _v
    _q_py = _q
    _return_data = Dggsvp3ReturnData(
        _a_py,
        _b_py,
        _k_py,
        _l_py,
        _u_py,
        _v_py,
        _q_py,
    )
    return _return_data

ZggsvdReturnData = _collections.namedtuple(
    'ZggsvdReturnData',
    ['k', 'l', 'a', 'b', 'alpha', 'beta', 'u', 'v', 'q', 'iwork'],
)

@_b_utils._withdrawal(
    'naginterfaces.library.lapackeig.zggsvd',
    mkwdrn=None,
    advice=[
        'Please use zggsvd3 instead.',
        'See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html',
    ]
)
def zggsvd(jobu, jobv, jobq, a, b):
    u"""
    Computes the generalized singular value decomposition of a complex
    matrix pair.

    ``zggsvd`` computes the generalized singular value decomposition
    (GSVD) of an m*n complex matrix A and a p*n complex matrix B.
    ``zggsvd`` is marked as `deprecated` by LAPACK; the replacement
    routine is ``zggsvd3`` which makes better use of Level 3 BLAS.

    Note: ``zggsvd`` is deprecated.
    Please use ``zggsvd3`` instead.
    See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html

    For full information please refer to the NAG Library document for
    f08vn

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08vnf.html

    Parameters
    ----------
    jobu : str, length 1
        If `jobu` = 'U', the unitary matrix U is computed.

        If `jobu` = 'N', U is not computed.

    jobv : str, length 1
        If `jobv` = 'V', the unitary matrix V is computed.

        If `jobv` = 'N', V is not computed.

    jobq : str, length 1
        If `jobq` = 'Q', the unitary matrix Q is computed.

        If `jobq` = 'N', Q is not computed.

    a : complex, array-like, shape (m, n)
        The m*n matrix A.

    b : complex, array-like, shape (p, n)
        The p*n matrix B.

    Returns
    -------
    k : int
        `k` and `l` specify the dimension of the subblocks k and l as
        described in [Notes]; (k+l) is the effective numerical rank of
        (A;B).

    l : int
        `k` and `l` specify the dimension of the subblocks k and l as
        described in [Notes]; (k+l) is the effective numerical rank of
        (A;B).

    a : complex, ndarray, shape (m, n)
        Contains the triangular matrix R, or part of R. See [Notes] for
        details.

    b : complex, ndarray, shape (p, n)
        Contains the triangular matrix R if m-k-l < 0. See [Notes] for
        details.

    alpha : float, ndarray, shape (n)
        See the description of `beta`.

    beta : float, ndarray, shape (n)
        `alpha` and `beta` contain the generalized singular value pairs
        of A and B, alpha_i and beta_i;

            `alpha`[0:`k`] = 1,

            `beta`[0:`k`] = 0,

        and if m-k-l >= 0,

            `alpha`[`k`:`k`+`l`] = C,

            `beta`[`k`:`k`+`l`] = S,

        or if m-k-l < 0,

            `alpha`[`k`:m] = C,

            `alpha`[m:`k`+`l`] = 0,

            `beta`[`k`:m] = S,

            `beta`[m:`k`+`l`] = 1, and

            `alpha`[`k`+`l`:n] = 0,

            `beta`[`k`+`l`:n] = 0.

        The notation `alpha`[`k`-1:n] above refers to consecutive
        elements `alpha`[i-1], for i = `k`,...,n.

    u : complex, ndarray, shape (:, :)
        If `jobu` = 'U', `u` contains the m*m unitary matrix U.

        If `jobu` = 'N', `u` is not referenced.

    v : complex, ndarray, shape (:, :)
        If `jobv` = 'V', `v` contains the p*p unitary matrix V.

        If `jobv` = 'N', `v` is not referenced.

    q : complex, ndarray, shape (:, :)
        If `jobq` = 'Q', `q` contains the n*n unitary matrix Q.

        If `jobq` = 'N', `q` is not referenced.

    iwork : int, ndarray, shape (n)
        Stores the sorting information. More precisely, the following
        loop will sort `alpha`

        ::

          for i in range(k+1, min(m,k+l)+1):
            swap alpha[i-1] and alpha[iwork[i-1]-1]

        such that `alpha`[0] >= `alpha`[1] >= ... >= `alpha`[n-1].

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobu`.

            Constraint: `jobu` = 'U' or 'N'.

        (`errno` -2)
            On entry, error in parameter `jobv`.

            Constraint: `jobv` = 'V' or 'N'.

        (`errno` -3)
            On entry, error in parameter `jobq`.

            Constraint: `jobq` = 'Q' or 'N'.

        (`errno` -4)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -5)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -6)
            On entry, error in parameter p.

            Constraint: p >= 0.

        (`errno` 1)
            The Jacobi-type procedure failed to converge.

    Notes
    -----
    The generalized singular value decomposition is given by

        U^HAQ = D_1(0 R), V^HBQ = D_2(0 R),

    where U, V and Q are unitary matrices.
    Let (k+l) be the effective numerical rank of the matrix (A;B), then
    R is a (k+l)*(k+l) nonsingular upper triangular matrix, D_1 and D_2
    are m*(k+l) and p*(k+l) 'diagonal' matrices structured as follows:

    if m-k-l >= 0,

        D_1 = [table omitted]

        D_2 = [table omitted]

        (0 R) = [table omitted]

    where

        C = diag(alpha_{k+1},...,alpha_{k+l}),

        S = diag(beta_{k+1},...,beta_{k+l}),

    and

        C^2+S^2 = I.

    R is stored as a submatrix of A with elements R_{ij} stored as
    A_{i,n-k-l+j} on exit.

    If m-k-l < 0,

        D_1 = [table omitted]

        D_2 = [table omitted]

        (0 R) = [table omitted]

    where

        C = diag(alpha_{k+1},...,alpha_m),

        S = diag(beta_{k+1},...,beta_m),

    and

        C^2+S^2 = I.

    [table omitted] is stored as a submatrix of A with R_{ij} stored as
    A_{i,n-k-l+j}, and R_33 is stored as a submatrix of B with
    (R_33)_{ij} stored as B_{m-k+i,n+m-k-l+j}.

    The function computes C, S, R and, optionally, the unitary
    transformation matrices U, V and Q.

    In particular, if B is an n*n nonsingular matrix, then the GSVD of A
    and B implicitly gives the SVD of A*B^-1:

        AB^-1 = U(D_1 D_2^-1)V^H.

    If (A;B) has orthonormal columns, then the GSVD of A and B is also
    equal to the CS decomposition of A and B.
    Furthermore, the GSVD can be used to derive the solution of the
    eigenvalue problem:

        A^HAx = lambda B^HBx.

    In some literature, the GSVD of A and B is presented in the form

        U^HAX = (0 D_1), V^HBX = (0 D_2),

    where U and V are orthogonal and X is nonsingular, and D_1 and D_2
    are 'diagonal'.
    The former GSVD form can be converted to the latter form by taking
    the nonsingular matrix X as

        X = Q[table omitted].

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zggsvd'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'), (_b, 'b')),
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _p_py = _b_utils._EngineComplex128ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[0]
    _p = _b_utils._EngineIntScalarType(
        _p_py, {'fun_name': fun_name, 'entity': '_p'},
    ).to_ctypes()
    _alpha = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _beta = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobu = _b_utils._EngineCharScalarType(
        jobu, {'fun_name': fun_name, 'entity': 'jobu'}, exp_chrlen=1,
    ).to_ctypes()
    _jobu_py = _b_utils._EngineCharScalarType.to_py(_jobu)
    if _jobu_py in ('U', 'u'):
        _u_dim1 = _m_py
    else:
        _u_dim1 = 1
    if _jobu_py in ('U', 'u'):
        _u_dim2 = _m_py
    else:
        _u_dim2 = 0
    exp_shape_u = (_u_dim1, _u_dim2)
    _u = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_u,
        sorder=_m_sorder,
    )
    _jobv = _b_utils._EngineCharScalarType(
        jobv, {'fun_name': fun_name, 'entity': 'jobv'}, exp_chrlen=1,
    ).to_ctypes()
    _jobv_py = _b_utils._EngineCharScalarType.to_py(_jobv)
    if _jobv_py in ('V', 'v'):
        _v_dim1 = _p_py
    else:
        _v_dim1 = 1
    if _jobv_py in ('V', 'v'):
        _v_dim2 = _p_py
    else:
        _v_dim2 = 0
    exp_shape_v = (_v_dim1, _v_dim2)
    _v = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_v,
        sorder=_m_sorder,
    )
    _jobq = _b_utils._EngineCharScalarType(
        jobq, {'fun_name': fun_name, 'entity': 'jobq'}, exp_chrlen=1,
    ).to_ctypes()
    _jobq_py = _b_utils._EngineCharScalarType.to_py(_jobq)
    if _jobq_py in ('Q', 'q'):
        _q_dim1 = _n_py
    else:
        _q_dim1 = 1
    if _jobq_py in ('Q', 'q'):
        _q_dim2 = _n_py
    else:
        _q_dim2 = 0
    exp_shape_q = (_q_dim1, _q_dim2)
    _q = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_q,
        sorder=_m_sorder,
    )
    _iwork = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    _return_scalars = _b_lapackeig.zggsvd(
        jobu,
        jobv,
        jobq,
        _m,
        _n,
        _p,
        _a,
        _b,
        _alpha,
        _beta,
        _u,
        _v,
        _q,
        _iwork,
    )
    _a_py = _a
    _b_py = _b
    _k_py = _return_scalars.k
    _l_py = _return_scalars.l
    _alpha_py = _alpha
    _beta_py = _beta
    _u_py = _u
    _v_py = _v
    _q_py = _q
    _iwork_py = _iwork
    _return_data = ZggsvdReturnData(
        _k_py,
        _l_py,
        _a_py,
        _b_py,
        _alpha_py,
        _beta_py,
        _u_py,
        _v_py,
        _q_py,
        _iwork_py,
    )
    return _return_data

Zggsvd3ReturnData = _collections.namedtuple(
    'Zggsvd3ReturnData',
    ['k', 'l', 'a', 'b', 'alpha', 'beta', 'u', 'v', 'q', 'iwork'],
)

def zggsvd3(jobu, jobv, jobq, a, b):
    u"""
    Computes, using BLAS-3, the generalized singular value decomposition
    of a complex matrix pair.

    ``zggsvd3`` computes the generalized singular value decomposition
    (GSVD) of an m*n complex matrix A and a p*n complex matrix B.

    For full information please refer to the NAG Library document for
    f08vq

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08vqf.html

    Parameters
    ----------
    jobu : str, length 1
        If `jobu` = 'U', the unitary matrix U is computed.

        If `jobu` = 'N', U is not computed.

    jobv : str, length 1
        If `jobv` = 'V', the unitary matrix V is computed.

        If `jobv` = 'N', V is not computed.

    jobq : str, length 1
        If `jobq` = 'Q', the unitary matrix Q is computed.

        If `jobq` = 'N', Q is not computed.

    a : complex, array-like, shape (m, n)
        The m*n matrix A.

    b : complex, array-like, shape (p, n)
        The p*n matrix B.

    Returns
    -------
    k : int
        `k` and `l` specify the dimension of the subblocks k and l as
        described in [Notes]; (k+l) is the effective numerical rank of
        (A;B).

    l : int
        `k` and `l` specify the dimension of the subblocks k and l as
        described in [Notes]; (k+l) is the effective numerical rank of
        (A;B).

    a : complex, ndarray, shape (m, n)
        Contains the triangular matrix R, or part of R. See [Notes] for
        details.

    b : complex, ndarray, shape (p, n)
        Contains the triangular matrix R if m-k-l < 0. See [Notes] for
        details.

    alpha : float, ndarray, shape (n)
        See the description of `beta`.

    beta : float, ndarray, shape (n)
        `alpha` and `beta` contain the generalized singular value pairs
        of A and B, alpha_i and beta_i;

            `alpha`[0:`k`] = 1,

            `beta`[0:`k`] = 0,

        and if m-k-l >= 0,

            `alpha`[`k`:`k`+`l`] = C,

            `beta`[`k`:`k`+`l`] = S,

        or if m-k-l < 0,

            `alpha`[`k`:m] = C,

            `alpha`[m:`k`+`l`] = 0,

            `beta`[`k`:m] = S,

            `beta`[m:`k`+`l`] = 1, and

            `alpha`[`k`+`l`:n] = 0,

            `beta`[`k`+`l`:n] = 0.

        The notation `alpha`[`k`-1:n] above refers to consecutive
        elements `alpha`[i-1], for i = `k`,...,n.

    u : complex, ndarray, shape (:, :)
        If `jobu` = 'U', `u` contains the m*m unitary matrix U.

        If `jobu` = 'N', `u` is not referenced.

    v : complex, ndarray, shape (:, :)
        If `jobv` = 'V', `v` contains the p*p unitary matrix V.

        If `jobv` = 'N', `v` is not referenced.

    q : complex, ndarray, shape (:, :)
        If `jobq` = 'Q', `q` contains the n*n unitary matrix Q.

        If `jobq` = 'N', `q` is not referenced.

    iwork : int, ndarray, shape (n)
        Stores the sorting information. More precisely, if I is the
        ordered set of indices of `alpha` containing C (denote as
        `alpha`[I], see `beta`), then the corresponding elements
        `iwork`[I]-1 contain the swap pivots, J, that sorts I such that
        `alpha`[I] is in descending numerical order.

        The following pseudocode sorts the set I:

        for i in I; j = J_i; swap I_i and I_j;end

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobu`.

            Constraint: `jobu` = 'U' or 'N'.

        (`errno` -2)
            On entry, error in parameter `jobv`.

            Constraint: `jobv` = 'V' or 'N'.

        (`errno` -3)
            On entry, error in parameter `jobq`.

            Constraint: `jobq` = 'Q' or 'N'.

        (`errno` -4)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -5)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -6)
            On entry, error in parameter p.

            Constraint: p >= 0.

        (`errno` 1)
            The Jacobi-type procedure failed to converge.

    Notes
    -----
    Given an m*n complex matrix A and a p*n complex matrix B, the
    generalized singular value decomposition is given by

        U^HAQ = D_1(0 R), V^HBQ = D_2(0 R),

    where U, V and Q are unitary matrices.
    Let l be the effective numerical rank of B and (k+l) be the
    effective numerical rank of the matrix (A;B), then the first k
    generalized singular values are infinite and the remaining l are
    finite. R is a (k+l)*(k+l) nonsingular upper triangular matrix, D_1
    and D_2 are m*(k+l) and p*(k+l) 'diagonal' matrices structured as
    follows:

    if m-k-l >= 0,

        D_1 = [table omitted]

        D_2 = [table omitted]

        (0 R) = [table omitted]

    where

        C = diag(alpha_{k+1},...,alpha_{k+l}),

        S = diag(beta_{k+1},...,beta_{k+l}),

    and

        C^2+S^2 = I.

    R is stored as a submatrix of A with elements R_{ij} stored as
    A_{i,n-k-l+j} on exit.

    If m-k-l < 0,

        D_1 = [table omitted]

        D_2 = [table omitted]

        (0 R) = [table omitted]

    where

        C = diag(alpha_{k+1},...,alpha_m),

        S = diag(beta_{k+1},...,beta_m),

    and

        C^2+S^2 = I.

    [table omitted] is stored as a submatrix of A with R_{ij} stored as
    A_{i,n-k-l+j}, and R_33 is stored as a submatrix of B with
    (R_33)_{ij} stored as B_{m-k+i,n+m-k-l+j}.

    The function computes C, S, R and, optionally, the unitary
    transformation matrices U, V and Q.

    In particular, if B is an n*n nonsingular matrix, then the GSVD of A
    and B implicitly gives the SVD of A*B^-1:

        AB^-1 = U(D_1 D_2^-1)V^H.

    If (A;B) has orthonormal columns, then the GSVD of A and B is also
    equal to the CS decomposition of A and B.
    Furthermore, the GSVD can be used to derive the solution of the
    eigenvalue problem:

        A^HAx = lambda B^HBx.

    In some literature, the GSVD of A and B is presented in the form

        U^HAX = (0 D_1), V^HBX = (0 D_2),

    where U and V are orthogonal and X is nonsingular, and D_1 and D_2
    are 'diagonal'.
    The former GSVD form can be converted to the latter form by setting

        X = Q[table omitted].

    A two stage process is used to compute the GSVD of the matrix pair
    (A,B).
    The pair is first reduced to upper triangular form by unitary
    transformations using ``zggsvp3``.
    The GSVD of the resulting upper triangular matrix pair is then
    performed by ``ztgsja`` which uses a variant of the Kogbetliantz
    algorithm (a cyclic Jacobi method).

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 2012, `Matrix Computations`, (4th
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zggsvd3'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'), (_b, 'b')),
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _p_py = _b_utils._EngineComplex128ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[0]
    _p = _b_utils._EngineIntScalarType(
        _p_py, {'fun_name': fun_name, 'entity': '_p'},
    ).to_ctypes()
    _alpha = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _beta = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobu = _b_utils._EngineCharScalarType(
        jobu, {'fun_name': fun_name, 'entity': 'jobu'}, exp_chrlen=1,
    ).to_ctypes()
    _jobu_py = _b_utils._EngineCharScalarType.to_py(_jobu)
    if _jobu_py in ('U', 'u'):
        _u_dim1 = _m_py
    else:
        _u_dim1 = 1
    if _jobu_py in ('U', 'u'):
        _u_dim2 = _m_py
    else:
        _u_dim2 = 1
    exp_shape_u = (_u_dim1, _u_dim2)
    _u = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_u,
        sorder=_m_sorder,
    )
    _jobv = _b_utils._EngineCharScalarType(
        jobv, {'fun_name': fun_name, 'entity': 'jobv'}, exp_chrlen=1,
    ).to_ctypes()
    _jobv_py = _b_utils._EngineCharScalarType.to_py(_jobv)
    if _jobv_py in ('V', 'v'):
        _v_dim1 = _p_py
    else:
        _v_dim1 = 1
    if _jobv_py in ('V', 'v'):
        _v_dim2 = _p_py
    else:
        _v_dim2 = 1
    exp_shape_v = (_v_dim1, _v_dim2)
    _v = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_v,
        sorder=_m_sorder,
    )
    _jobq = _b_utils._EngineCharScalarType(
        jobq, {'fun_name': fun_name, 'entity': 'jobq'}, exp_chrlen=1,
    ).to_ctypes()
    _jobq_py = _b_utils._EngineCharScalarType.to_py(_jobq)
    if _jobq_py in ('Q', 'q'):
        _q_dim1 = _n_py
    else:
        _q_dim1 = 1
    if _jobq_py in ('Q', 'q'):
        _q_dim2 = _n_py
    else:
        _q_dim2 = 1
    exp_shape_q = (_q_dim1, _q_dim2)
    _q = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_q,
        sorder=_m_sorder,
    )
    _iwork = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n_py,),
    )
    _return_scalars = _b_lapackeig.zggsvd3(
        jobu,
        jobv,
        jobq,
        _m,
        _n,
        _p,
        _a,
        _b,
        _alpha,
        _beta,
        _u,
        _v,
        _q,
        _iwork,
    )
    _a_py = _a
    _b_py = _b
    _k_py = _return_scalars.k
    _l_py = _return_scalars.l
    _alpha_py = _alpha
    _beta_py = _beta
    _u_py = _u
    _v_py = _v
    _q_py = _q
    _iwork_py = _iwork
    _return_data = Zggsvd3ReturnData(
        _k_py,
        _l_py,
        _a_py,
        _b_py,
        _alpha_py,
        _beta_py,
        _u_py,
        _v_py,
        _q_py,
        _iwork_py,
    )
    return _return_data

ZggsvpReturnData = _collections.namedtuple(
    'ZggsvpReturnData',
    ['a', 'b', 'k', 'l', 'u', 'v', 'q'],
)

@_b_utils._withdrawal(
    'naginterfaces.library.lapackeig.zggsvp',
    mkwdrn=None,
    advice=[
        'Please use zggsvp3 instead.',
        'See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html',
    ]
)
def zggsvp(jobu, jobv, jobq, a, b, tola, tolb):
    u"""
    Produces unitary matrices that simultaneously reduce the complex,
    m*n, matrix A and the complex, p*n, matrix B to upper triangular
    form.

    ``zggsvp`` uses unitary transformations to simultaneously reduce the
    m*n matrix A and the p*n matrix B to upper triangular form.
    This factorization is usually used as a preprocessing step for
    computing the generalized singular value decomposition (GSVD).
    ``zggsvp`` is marked as `deprecated` by LAPACK; the replacement
    routine is ``zggsvp3`` which makes better use of Level 3 BLAS.

    Note: ``zggsvp`` is deprecated.
    Please use ``zggsvp3`` instead.
    See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html

    For full information please refer to the NAG Library document for
    f08vs

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08vsf.html

    Parameters
    ----------
    jobu : str, length 1
        If `jobu` = 'U', the unitary matrix U is computed.

        If `jobu` = 'N', U is not computed.

    jobv : str, length 1
        If `jobv` = 'V', the unitary matrix V is computed.

        If `jobv` = 'N', V is not computed.

    jobq : str, length 1
        If `jobq` = 'Q', the unitary matrix Q is computed.

        If `jobq` = 'N', Q is not computed.

    a : complex, array-like, shape (m, n)
        The m*n matrix A.

    b : complex, array-like, shape (p, n)
        The p*n matrix B.

    tola : float
        `tola` and `tolb` are the thresholds to determine the effective
        numerical rank of matrix B and a subblock of A. Generally, they
        are set to

            `tola` = max(m, n)||A||epsilon,;`tolb` = max(p,
            n)||B||epsilon,

        where epsilon is the machine precision.

        The size of `tola` and `tolb` may affect the size of backward
        errors of the decomposition.

    tolb : float
        `tola` and `tolb` are the thresholds to determine the effective
        numerical rank of matrix B and a subblock of A. Generally, they
        are set to

            `tola` = max(m, n)||A||epsilon,;`tolb` = max(p,
            n)||B||epsilon,

        where epsilon is the machine precision.

        The size of `tola` and `tolb` may affect the size of backward
        errors of the decomposition.

    Returns
    -------
    a : complex, ndarray, shape (m, n)
        Contains the triangular (or trapezoidal) matrix described in
        [Notes].

    b : complex, ndarray, shape (p, n)
        Contains the triangular matrix described in [Notes].

    k : int
        `k` and `l` specify the dimension of the subblocks k and l as
        described in [Notes]; (k+l) is the effective numerical rank of
        (`a`^T `b`^T)^T.

    l : int
        `k` and `l` specify the dimension of the subblocks k and l as
        described in [Notes]; (k+l) is the effective numerical rank of
        (`a`^T `b`^T)^T.

    u : complex, ndarray, shape (:, :)
        If `jobu` = 'U', `u` contains the unitary matrix U.

        If `jobu` = 'N', `u` is not referenced.

    v : complex, ndarray, shape (:, :)
        If `jobv` = 'V', `v` contains the unitary matrix V.

        If `jobv` = 'N', `v` is not referenced.

    q : complex, ndarray, shape (:, :)
        If `jobq` = 'Q', `q` contains the unitary matrix Q.

        If `jobq` = 'N', `q` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobu`.

            Constraint: `jobu` = 'U' or 'N'.

        (`errno` -2)
            On entry, error in parameter `jobv`.

            Constraint: `jobv` = 'V' or 'N'.

        (`errno` -3)
            On entry, error in parameter `jobq`.

            Constraint: `jobq` = 'Q' or 'N'.

        (`errno` -4)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -5)
            On entry, error in parameter p.

            Constraint: p >= 0.

        (`errno` -6)
            On entry, error in parameter n.

            Constraint: n >= 0.

    Notes
    -----
    ``zggsvp`` computes unitary matrices U, V and Q such that

        U^HAQ = { [table omitted], if m-k-l >= 0; ; [table omitted], if
        m-k-l < 0; ; V^HBQ = [table omitted]

    where the k*k matrix A_12 and l*l matrix B_13 are nonsingular upper
    triangular; A_23 is l*l upper triangular if m-k-l >= 0 and is
    (m-k)*l upper trapezoidal otherwise. (k+l) is the effective
    numerical rank of the (m+p)*n matrix (A^H B^H)^H.

    This decomposition is usually used as the preprocessing step for
    computing the Generalized Singular Value Decomposition (GSVD), see
    function ``zggsvd``.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zggsvp'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'), (_b, 'b')),
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _p_py = _b_utils._EngineComplex128ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[0]
    _p = _b_utils._EngineIntScalarType(
        _p_py, {'fun_name': fun_name, 'entity': '_p'},
    ).to_ctypes()
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _jobu = _b_utils._EngineCharScalarType(
        jobu, {'fun_name': fun_name, 'entity': 'jobu'}, exp_chrlen=1,
    ).to_ctypes()
    _jobu_py = _b_utils._EngineCharScalarType.to_py(_jobu)
    if _jobu_py in ('U', 'u'):
        _u_dim1 = _m_py
    else:
        _u_dim1 = 1
    if _jobu_py in ('U', 'u'):
        _u_dim2 = _m_py
    else:
        _u_dim2 = 1
    exp_shape_u = (_u_dim1, _u_dim2)
    _u = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_u,
        sorder=_m_sorder,
    )
    _jobv = _b_utils._EngineCharScalarType(
        jobv, {'fun_name': fun_name, 'entity': 'jobv'}, exp_chrlen=1,
    ).to_ctypes()
    _jobv_py = _b_utils._EngineCharScalarType.to_py(_jobv)
    if _jobv_py in ('V', 'v'):
        _v_dim1 = _p_py
    else:
        _v_dim1 = 1
    if _jobv_py in ('V', 'v'):
        _v_dim2 = _p_py
    else:
        _v_dim2 = 1
    exp_shape_v = (_v_dim1, _v_dim2)
    _v = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_v,
        sorder=_m_sorder,
    )
    _jobq = _b_utils._EngineCharScalarType(
        jobq, {'fun_name': fun_name, 'entity': 'jobq'}, exp_chrlen=1,
    ).to_ctypes()
    _jobq_py = _b_utils._EngineCharScalarType.to_py(_jobq)
    if _jobq_py in ('Q', 'q'):
        _q_dim1 = _n_py
    else:
        _q_dim1 = 1
    if _jobq_py in ('Q', 'q'):
        _q_dim2 = _n_py
    else:
        _q_dim2 = 1
    exp_shape_q = (_q_dim1, _q_dim2)
    _q = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_q,
        sorder=_m_sorder,
    )
    _return_scalars = _b_lapackeig.zggsvp(
        jobu,
        jobv,
        jobq,
        _m,
        _p,
        _n,
        _a,
        _b,
        tola,
        tolb,
        _u,
        _v,
        _q,
    )
    _a_py = _a
    _b_py = _b
    _k_py = _return_scalars.k
    _l_py = _return_scalars.l
    _u_py = _u
    _v_py = _v
    _q_py = _q
    _return_data = ZggsvpReturnData(
        _a_py,
        _b_py,
        _k_py,
        _l_py,
        _u_py,
        _v_py,
        _q_py,
    )
    return _return_data

Zggsvp3ReturnData = _collections.namedtuple(
    'Zggsvp3ReturnData',
    ['a', 'b', 'k', 'l', 'u', 'v', 'q'],
)

def zggsvp3(jobu, jobv, jobq, a, b, tola, tolb):
    u"""
    Produces unitary matrices, using BLAS-3, that simultaneously reduce
    the complex, m*n, matrix A and the complex, p*n, matrix B to upper
    triangular form.

    ``zggsvp3`` uses unitary transformations to simultaneously reduce
    the m*n matrix A and the p*n matrix B to upper triangular form.
    This factorization is usually used as a preprocessing step for
    computing the generalized singular value decomposition (GSVD).
    For sufficiently large problems, a blocked algorithm is used to make
    best use of Level 3 BLAS.

    For full information please refer to the NAG Library document for
    f08vu

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08vuf.html

    Parameters
    ----------
    jobu : str, length 1
        If `jobu` = 'U', the unitary matrix U is computed.

        If `jobu` = 'N', U is not computed.

    jobv : str, length 1
        If `jobv` = 'V', the unitary matrix V is computed.

        If `jobv` = 'N', V is not computed.

    jobq : str, length 1
        If `jobq` = 'Q', the unitary matrix Q is computed.

        If `jobq` = 'N', Q is not computed.

    a : complex, array-like, shape (m, n)
        The m*n matrix A.

    b : complex, array-like, shape (p, n)
        The p*n matrix B.

    tola : float
        `tola` and `tolb` are the thresholds to determine the effective
        numerical rank of matrix B and a subblock of A. Generally, they
        are set to

            `tola` = max(m, n)||A||epsilon,;`tolb` = max(p,
            n)||B||epsilon,

        where epsilon is the machine precision.

        The size of `tola` and `tolb` may affect the size of backward
        errors of the decomposition.

    tolb : float
        `tola` and `tolb` are the thresholds to determine the effective
        numerical rank of matrix B and a subblock of A. Generally, they
        are set to

            `tola` = max(m, n)||A||epsilon,;`tolb` = max(p,
            n)||B||epsilon,

        where epsilon is the machine precision.

        The size of `tola` and `tolb` may affect the size of backward
        errors of the decomposition.

    Returns
    -------
    a : complex, ndarray, shape (m, n)
        Contains the triangular (or trapezoidal) matrix described in
        [Notes].

    b : complex, ndarray, shape (p, n)
        Contains the triangular matrix described in [Notes].

    k : int
        `k` and `l` specify the dimension of the subblocks k and l as
        described in [Notes]; (k+l) is the effective numerical rank of
        (`a`^T `b`^T)^T.

    l : int
        `k` and `l` specify the dimension of the subblocks k and l as
        described in [Notes]; (k+l) is the effective numerical rank of
        (`a`^T `b`^T)^T.

    u : complex, ndarray, shape (:, :)
        If `jobu` = 'U', `u` contains the unitary matrix U.

        If `jobu` = 'N', `u` is not referenced.

    v : complex, ndarray, shape (:, :)
        If `jobv` = 'V', `v` contains the unitary matrix V.

        If `jobv` = 'N', `v` is not referenced.

    q : complex, ndarray, shape (:, :)
        If `jobq` = 'Q', `q` contains the unitary matrix Q.

        If `jobq` = 'N', `q` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobu`.

            Constraint: `jobu` = 'U' or 'N'.

        (`errno` -2)
            On entry, error in parameter `jobv`.

            Constraint: `jobv` = 'V' or 'N'.

        (`errno` -3)
            On entry, error in parameter `jobq`.

            Constraint: `jobq` = 'Q' or 'N'.

        (`errno` -4)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -5)
            On entry, error in parameter p.

            Constraint: p >= 0.

        (`errno` -6)
            On entry, error in parameter n.

            Constraint: n >= 0.

    Notes
    -----
    ``zggsvp3`` computes unitary matrices U, V and Q such that

        U^HAQ = { [table omitted], if m-k-l >= 0; ; [table omitted], if
        m-k-l < 0; ; V^HBQ = [table omitted]

    where the k*k matrix A_12 and l*l matrix B_13 are nonsingular upper
    triangular; A_23 is l*l upper triangular if m-k-l >= 0 and is
    (m-k)*l upper trapezoidal otherwise. (k+l) is the effective
    numerical rank of the (m+p)*n matrix (A^H B^H)^H.

    This decomposition is usually used as the preprocessing step for
    computing the Generalized Singular Value Decomposition (GSVD), see
    function ``ztgsja``; the two steps are combined in ``zggsvd3``.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 2012, `Matrix Computations`, (4th
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.zggsvp3'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'), (_b, 'b')),
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _p_py = _b_utils._EngineComplex128ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[0]
    _p = _b_utils._EngineIntScalarType(
        _p_py, {'fun_name': fun_name, 'entity': '_p'},
    ).to_ctypes()
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _jobu = _b_utils._EngineCharScalarType(
        jobu, {'fun_name': fun_name, 'entity': 'jobu'}, exp_chrlen=1,
    ).to_ctypes()
    _jobu_py = _b_utils._EngineCharScalarType.to_py(_jobu)
    if _jobu_py in ('U', 'u'):
        _u_dim1 = _m_py
    else:
        _u_dim1 = 1
    if _jobu_py in ('U', 'u'):
        _u_dim2 = _m_py
    else:
        _u_dim2 = 1
    exp_shape_u = (_u_dim1, _u_dim2)
    _u = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_u,
        sorder=_m_sorder,
    )
    _jobv = _b_utils._EngineCharScalarType(
        jobv, {'fun_name': fun_name, 'entity': 'jobv'}, exp_chrlen=1,
    ).to_ctypes()
    _jobv_py = _b_utils._EngineCharScalarType.to_py(_jobv)
    if _jobv_py in ('V', 'v'):
        _v_dim1 = _p_py
    else:
        _v_dim1 = 1
    if _jobv_py in ('V', 'v'):
        _v_dim2 = _p_py
    else:
        _v_dim2 = 1
    exp_shape_v = (_v_dim1, _v_dim2)
    _v = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_v,
        sorder=_m_sorder,
    )
    _jobq = _b_utils._EngineCharScalarType(
        jobq, {'fun_name': fun_name, 'entity': 'jobq'}, exp_chrlen=1,
    ).to_ctypes()
    _jobq_py = _b_utils._EngineCharScalarType.to_py(_jobq)
    if _jobq_py in ('Q', 'q'):
        _q_dim1 = _n_py
    else:
        _q_dim1 = 1
    if _jobq_py in ('Q', 'q'):
        _q_dim2 = _n_py
    else:
        _q_dim2 = 1
    exp_shape_q = (_q_dim1, _q_dim2)
    _q = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_q,
        sorder=_m_sorder,
    )
    _return_scalars = _b_lapackeig.zggsvp3(
        jobu,
        jobv,
        jobq,
        _m,
        _p,
        _n,
        _a,
        _b,
        tola,
        tolb,
        _u,
        _v,
        _q,
    )
    _a_py = _a
    _b_py = _b
    _k_py = _return_scalars.k
    _l_py = _return_scalars.l
    _u_py = _u
    _v_py = _v
    _q_py = _q
    _return_data = Zggsvp3ReturnData(
        _a_py,
        _b_py,
        _k_py,
        _l_py,
        _u_py,
        _v_py,
        _q_py,
    )
    return _return_data

DggevReturnData = _collections.namedtuple(
    'DggevReturnData',
    ['a', 'b', 'alphar', 'alphai', 'beta', 'vl', 'vr'],
)

@_b_utils._withdrawal(
    'naginterfaces.library.lapackeig.dggev',
    mkwdrn=None,
    advice=[
        'Please use dggev3 instead.',
        'See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html',
    ]
)
def dggev(jobvl, jobvr, a, b):
    u"""
    Computes, for a real nonsymmetric matrix pair, the generalized
    eigenvalues, and optionally, the left and/or right generalized
    eigenvectors.

    ``dggev`` computes for a pair of n*n real nonsymmetric matrices
    (A,B) the generalized eigenvalues and, optionally, the left and/or
    right generalized eigenvectors using the QZ algorithm. ``dggev`` is
    marked as `deprecated` by LAPACK; the replacement routine is
    ``dggev3`` which makes better use of Level 3 BLAS.

    Note: ``dggev`` is deprecated.
    Please use ``dggev3`` instead.
    See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html

    For full information please refer to the NAG Library document for
    f08wa

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08waf.html

    Parameters
    ----------
    jobvl : str, length 1
        If `jobvl` = 'N', do not compute the left generalized
        eigenvectors.

        If `jobvl` = 'V', compute the left generalized eigenvectors.

    jobvr : str, length 1
        If `jobvr` = 'N', do not compute the right generalized
        eigenvectors.

        If `jobvr` = 'V', compute the right generalized eigenvectors.

    a : float, array-like, shape (n, n)
        The matrix A in the pair (A,B).

    b : float, array-like, shape (n, n)
        The matrix B in the pair (A,B).

    Returns
    -------
    a : float, ndarray, shape (n, n)
        `a` has been overwritten.

    b : float, ndarray, shape (n, n)
        `b` has been overwritten.

    alphar : float, ndarray, shape (n)
        The element `alphar`[j-1] contains the real part of alpha_j.

    alphai : float, ndarray, shape (n)
        The element `alphai`[j-1] contains the imaginary part of
        alpha_j.

    beta : float, ndarray, shape (n)
        (`alphar`[j-1]+`alphai`[j-1]*i)/`beta`[j-1], for j = 1,2,...,n,
        will be the generalized eigenvalues.

        If `alphai`[j-1] is zero, then the jth eigenvalue is real; if
        positive, then the jth and (j+1)st eigenvalues are a complex
        conjugate pair, with `alphai`[j] negative.

        **Note:** the quotients `alphar`[j-1]/`beta`[j-1] and
        `alphai`[j-1]/`beta`[j-1] may easily overflow or underflow, and
        `beta`[j-1] may even be zero.

        Thus, you should avoid naively computing the ratio
        alpha_j/beta_j.

        However, max(|alpha_j|) will always be less than and usually
        comparable with ||A||_2 in magnitude, and max(|beta_j|) will
        always be less than and usually comparable with ||B||_2.

    vl : float, ndarray, shape (:, :)
        If `jobvl` = 'V', the left eigenvectors u_j are stored one after
        another in the columns of `vl`, in the same order as the
        corresponding eigenvalues.

        If the jth eigenvalue is real, then u_j = `vl`[:,j], the jth
        column of `vl`.

        If the jth and (j+1)th eigenvalues form a complex conjugate
        pair, then u_j = `vl`[:,j]+i*`vl`[:,j+1] and u(j+1) =
        `vl`[:,j]-i*`vl`[:,j+1].

        Each eigenvector will be scaled so the largest component has
        |real part|+|imag. part| = 1.

        If `jobvl` = 'N', `vl` is not referenced.

    vr : float, ndarray, shape (:, :)
        If `jobvr` = 'V', the right eigenvectors v_j are stored one
        after another in the columns of `vr`, in the same order as the
        corresponding eigenvalues.

        If the jth eigenvalue is real, then v_j = `vr`[:,j], the jth
        column of VR.

        If the jth and (j+1)th eigenvalues form a complex conjugate
        pair, then v_j = `vr`[:,j]+i*`vr`[:,j+1] and v_{j+1} =
        `vr`[:,j]-i*`vr`[:,j+1].

        Each eigenvector will be scaled so the largest component has
        |real part|+|imag. part| = 1.

        If `jobvr` = 'N', `vr` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobvl`.

            Constraint: `jobvl` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `jobvr`.

            Constraint: `jobvr` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` n+1)
            The QZ iteration failed with an unexpected error, please
            contact NAG.

        (`errno` n+2)
            A failure occurred in ``dtgevc`` while computing generalized
            eigenvectors.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i<=n))
            The QZ iteration failed. No eigenvectors have been
            calculated but `alphar`[j], `alphai`[j] and `beta`[j] should
            be correct from element *<value>*.

    Notes
    -----
    A generalized eigenvalue for a pair of matrices (A,B) is a scalar
    lambda or a ratio alpha/beta = lambda, such that A-lambda B is
    singular.
    It is usually represented as the pair (alpha,beta), as there is a
    reasonable interpretation for beta = 0, and even for both being
    zero.

    The right eigenvector v_j corresponding to the eigenvalue lambda_j
    of (A,B) satisfies

        Av_j = lambda_j Bv_j.

    The left eigenvector u_j corresponding to the eigenvalue lambda_j of
    (A,B) satisfies

        u_j^HA = lambda_j u_j^HB,

    where u_j^H is the conjugate-transpose of u_j.

    All the eigenvalues and, if required, all the eigenvectors of the
    generalized eigenproblem Ax = lambda Bx, where A and B are real,
    square matrices, are determined using the QZ algorithm.
    The QZ algorithm consists of four stages:

    (1) A is reduced to upper Hessenberg form and at the same time B is
        reduced to upper triangular form.

    (#) A is further reduced to quasi-triangular form while the
        triangular form of B is maintained. This is the real generalized
        Schur form of the pair (A,B).

    (#) The quasi-triangular form of A is reduced to triangular form and
        the eigenvalues extracted. This function does not actually
        produce the eigenvalues lambda_j, but instead returns alpha_j
        and beta_j such that

            lambda_j = alpha_j/beta_j, j = 1,2,...,n.

        The division by beta_j becomes your responsibility, since beta_j
        may be zero, indicating an infinite eigenvalue. Pairs of complex
        eigenvalues occur with alpha_j/beta_j and alpha_{j+1}/beta_{j+1}
        complex conjugates, even though alpha_j and alpha_{j+1} are not
        conjugate.

    (#) If the eigenvectors are required they are obtained from the
        triangular matrices and then transformed back into the original
        coordinate system.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 2012, `Matrix Computations`, (4th
    Edition), Johns Hopkins University Press, Baltimore

    Wilkinson, J H, 1979, `Kronecker's canonical form and the QZ
    algorithm`, Linear Algebra Appl. (28), 285--303
    """

    fun_name = 'naginterfaces.library.lapackeig.dggev'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'), (_b, 'b')),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _alphar = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _alphai = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _beta = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobvl = _b_utils._EngineCharScalarType(
        jobvl, {'fun_name': fun_name, 'entity': 'jobvl'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvl_py = _b_utils._EngineCharScalarType.to_py(_jobvl)
    if _jobvl_py in ('V', 'v'):
        _vl_dim1 = _n_py
    else:
        _vl_dim1 = 1
    if _jobvl_py in ('V', 'v'):
        _vl_dim2 = _n_py
    else:
        _vl_dim2 = 1
    exp_shape_vl = (_vl_dim1, _vl_dim2)
    _vl = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_vl,
        sorder=_m_sorder,
    )
    _jobvr = _b_utils._EngineCharScalarType(
        jobvr, {'fun_name': fun_name, 'entity': 'jobvr'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvr_py = _b_utils._EngineCharScalarType.to_py(_jobvr)
    if _jobvr_py in ('V', 'v'):
        _vr_dim1 = _n_py
    else:
        _vr_dim1 = 1
    if _jobvr_py in ('V', 'v'):
        _vr_dim2 = _n_py
    else:
        _vr_dim2 = 1
    exp_shape_vr = (_vr_dim1, _vr_dim2)
    _vr = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_vr,
        sorder=_m_sorder,
    )
    try:
        _b_lapackeig.dggev(
            jobvl,
            jobvr,
            _n,
            _a,
            _b,
            _alphar,
            _alphai,
            _beta,
            _vl,
            _vr,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _a_py = _a
    _b_py = _b
    _alphar_py = _alphar
    _alphai_py = _alphai
    _beta_py = _beta
    _vl_py = _vl
    _vr_py = _vr
    _return_data = DggevReturnData(
        _a_py,
        _b_py,
        _alphar_py,
        _alphai_py,
        _beta_py,
        _vl_py,
        _vr_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

DggevxReturnData = _collections.namedtuple(
    'DggevxReturnData',
    ['a', 'b', 'alphar', 'alphai', 'beta', 'vl', 'vr', 'ilo', 'ihi', 'lscale', 'rscale', 'abnrm', 'bbnrm', 'rconde', 'rcondv'],
)

def dggevx(balanc, jobvl, jobvr, sense, a, b):
    u"""
    Computes, for a real nonsymmetric matrix pair, the generalized
    eigenvalues, and optionally, the left and/or right generalized
    eigenvectors; also, optionally, the balancing transformation, the
    reciprocal condition numbers for the eigenvalues and for the right
    eigenvectors.

    ``dggevx`` computes for a pair of n*n real nonsymmetric matrices
    (A,B) the generalized eigenvalues and, optionally, the left and/or
    right generalized eigenvectors using the QZ algorithm.

    Optionally it also computes a balancing transformation to improve
    the conditioning of the eigenvalues and eigenvectors, reciprocal
    condition numbers for the eigenvalues, and reciprocal condition
    numbers for the right eigenvectors.

    For full information please refer to the NAG Library document for
    f08wb

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08wbf.html

    Parameters
    ----------
    balanc : str, length 1
        Specifies the balance option to be performed.

        `balanc` = 'N'

            Do not diagonally scale or permute.

        `balanc` = 'P'

            Permute only.

        `balanc` = 'S'

            Scale only.

        `balanc` = 'B'

            Both permute and scale.

        Computed reciprocal condition numbers will be for the matrices
        after permuting and/or balancing.

        Permuting does not change condition numbers (in exact
        arithmetic), but balancing does.

        In the absence of other information, `balanc` = 'B' is
        recommended.

    jobvl : str, length 1
        If `jobvl` = 'N', do not compute the left generalized
        eigenvectors.

        If `jobvl` = 'V', compute the left generalized eigenvectors.

    jobvr : str, length 1
        If `jobvr` = 'N', do not compute the right generalized
        eigenvectors.

        If `jobvr` = 'V', compute the right generalized eigenvectors.

    sense : str, length 1
        Determines which reciprocal condition numbers are computed.

        `sense` = 'N'

            None are computed.

        `sense` = 'E'

            Computed for eigenvalues only.

        `sense` = 'V'

            Computed for eigenvectors only.

        `sense` = 'B'

            Computed for eigenvalues and eigenvectors.

    a : float, array-like, shape (n, n)
        The matrix A in the pair (A,B).

    b : float, array-like, shape (n, n)
        The matrix B in the pair (A,B).

    Returns
    -------
    a : float, ndarray, shape (n, n)
        `a` has been overwritten. If `jobvl` = 'V' or `jobvr` = 'V' or
        both, then A contains the first part of the real Schur form of
        the 'balanced' versions of the input A and B.

    b : float, ndarray, shape (n, n)
        `b` has been overwritten.

    alphar : float, ndarray, shape (n)
        The element `alphar`[j-1] contains the real part of alpha_j.

    alphai : float, ndarray, shape (n)
        The element `alphai`[j-1] contains the imaginary part of
        alpha_j.

    beta : float, ndarray, shape (n)
        (`alphar`[j-1]+`alphai`[j-1]*i)/`beta`[j-1], for j = 1,2,...,n,
        will be the generalized eigenvalues.

        If `alphai`[j-1] is zero, then the jth eigenvalue is real; if
        positive, then the jth and (j+1)st eigenvalues are a complex
        conjugate pair, with `alphai`[j] negative.

        **Note:** the quotients `alphar`[j-1]/`beta`[j-1] and
        `alphai`[j-1]/`beta`[j-1] may easily overflow or underflow, and
        `beta`[j-1] may even be zero.

        Thus, you should avoid naively computing the ratio
        alpha_j/beta_j.

        However, max(|alpha_j|) will always be less than and usually
        comparable with ||A||_2 in magnitude, and max(|beta_j|) will
        always be less than and usually comparable with ||B||_2.

    vl : float, ndarray, shape (:, :)
        If `jobvl` = 'V', the left generalized eigenvectors u_j are
        stored one after another in the columns of `vl`, in the same
        order as the corresponding eigenvalues. Each eigenvector will be
        scaled so the largest component will have |real part|+|imag.
        part| = 1.

        If `jobvl` = 'N', `vl` is not referenced.

    vr : float, ndarray, shape (:, :)
        If `jobvr` = 'V', the right generalized eigenvectors v_j are
        stored one after another in the columns of `vr`, in the same
        order as the corresponding eigenvalues. Each eigenvector will be
        scaled so the largest component will have |real part|+|imag.
        part| = 1.

        If `jobvr` = 'N', `vr` is not referenced.

    ilo : int
        `ilo` and `ihi` are integer values such that `a`[i-1,j-1] = 0
        and `b`[i-1,j-1] = 0 if i > j and j = 1,2,...,`ilo`-1 or i =
        `ihi`+1,...,n.

        If `balanc` = 'N' or 'S', `ilo` = 1 and `ihi` = n.

    ihi : int
        `ilo` and `ihi` are integer values such that `a`[i-1,j-1] = 0
        and `b`[i-1,j-1] = 0 if i > j and j = 1,2,...,`ilo`-1 or i =
        `ihi`+1,...,n.

        If `balanc` = 'N' or 'S', `ilo` = 1 and `ihi` = n.

    lscale : float, ndarray, shape (n)
        Details of the permutations and scaling factors applied to the
        left side of A and B.

        If pl_j is the index of the row interchanged with row j, and
        dl_j is the scaling factor applied to row j, then:

            `lscale`[j-1] = pl_j, for j = 1,2,...,`ilo`-1;

            `lscale` = dl_j, for j = `ilo`,...,`ihi`;

            `lscale` = pl_j, for j = `ihi`+1,...,n.

        The order in which the interchanges are made is n to `ihi`+1,
        then 1 to `ilo`-1.

    rscale : float, ndarray, shape (n)
        Details of the permutations and scaling factors applied to the
        right side of A and B.

        If pr_j is the index of the column interchanged with column j,
        and dr_j is the scaling factor applied to column j, then:

            `rscale`[j-1] = pr_j, for j = 1,2,...,`ilo`-1;

            if `rscale` = dr_j, for j = `ilo`,...,`ihi`;

            if `rscale` = pr_j, for j = `ihi`+1,...,n.

        The order in which the interchanges are made is n to `ihi`+1,
        then 1 to `ilo`-1.

    abnrm : float
        The 1-norm of the balanced matrix A.

    bbnrm : float
        The 1-norm of the balanced matrix B.

    rconde : float, ndarray, shape (n)
        If `sense` = 'E' or 'B', the reciprocal condition numbers of the
        eigenvalues, stored in consecutive elements of the array. For a
        complex conjugate pair of eigenvalues two consecutive elements
        of `rconde` are set to the same value. Thus `rconde`[j-1],
        `rcondv`[j-1], and the jth columns of `vl` and `vr` all
        correspond to the jth eigenpair.

        If `sense` = 'V', `rconde` is not referenced.

    rcondv : float, ndarray, shape (n)
        If `sense` = 'V' or 'B', the estimated reciprocal condition
        numbers of the eigenvectors, stored in consecutive elements of
        the array. For a complex eigenvector two consecutive elements of
        `rcondv` are set to the same value.

        If `sense` = 'E', `rcondv` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `balanc`.

            Constraint: `balanc` = 'N', 'P', 'S' or 'B'.

        (`errno` -2)
            On entry, error in parameter `jobvl`.

            Constraint: `jobvl` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `jobvr`.

            Constraint: `jobvr` = 'N' or 'V'.

        (`errno` -4)
            On entry, error in parameter `sense`.

            Constraint: `sense` = 'N', 'E', 'V' or 'B'.

        (`errno` -5)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` n+1)
            The QZ iteration failed with an unexpected error, please
            contact NAG.

        (`errno` n+2)
            A failure occurred in ``dtgevc`` while computing generalized
            eigenvectors.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i<=n))
            The QZ iteration failed. No eigenvectors have been
            calculated but `alphar`[j], `alphai`[j] and `beta`[j] should
            be correct from element *<value>*.

    Notes
    -----
    A generalized eigenvalue for a pair of matrices (A,B) is a scalar
    lambda or a ratio alpha/beta = lambda, such that A-lambda B is
    singular.
    It is usually represented as the pair (alpha,beta), as there is a
    reasonable interpretation for beta = 0, and even for both being
    zero.

    The right eigenvector v_j corresponding to the eigenvalue lambda_j
    of (A,B) satisfies

        Av_j = lambda_j Bv_j.

    The left eigenvector u_j corresponding to the eigenvalue lambda_j of
    (A,B) satisfies

        u_j^HA = lambda_j u_j^HB,

    where u_j^H is the conjugate-transpose of u_j.

    All the eigenvalues and, if required, all the eigenvectors of the
    generalized eigenproblem Ax = lambda Bx, where A and B are real,
    square matrices, are determined using the QZ algorithm.
    The QZ algorithm consists of four stages:

    (1) A is reduced to upper Hessenberg form and at the same time B is
        reduced to upper triangular form.

    (#) A is further reduced to quasi-triangular form while the
        triangular form of B is maintained. This is the real generalized
        Schur form of the pair (A,B).

    (#) The quasi-triangular form of A is reduced to triangular form and
        the eigenvalues extracted. This function does not actually
        produce the eigenvalues lambda_j, but instead returns alpha_j
        and beta_j such that

            lambda_j = alpha_j/beta_j, j = 1,2,...,n.

        The division by beta_j becomes your responsibility, since beta_j
        may be zero, indicating an infinite eigenvalue. Pairs of complex
        eigenvalues occur with alpha_j/beta_j and alpha_{j+1}/beta_{j+1}
        complex conjugates, even though alpha_j and alpha_{j+1} are not
        conjugate.

    (#) If the eigenvectors are required they are obtained from the
        triangular matrices and then transformed back into the original
        coordinate system.

    For details of the balancing option, see [Notes for dggbal].

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 2012, `Matrix Computations`, (4th
    Edition), Johns Hopkins University Press, Baltimore

    Wilkinson, J H, 1979, `Kronecker's canonical form and the QZ
    algorithm`, Linear Algebra Appl. (28), 285--303
    """

    fun_name = 'naginterfaces.library.lapackeig.dggevx'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'), (_b, 'b')),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _alphar = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _alphai = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _beta = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobvl = _b_utils._EngineCharScalarType(
        jobvl, {'fun_name': fun_name, 'entity': 'jobvl'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvl_py = _b_utils._EngineCharScalarType.to_py(_jobvl)
    if _jobvl_py in ('V', 'v'):
        _vl_dim1 = _n_py
    else:
        _vl_dim1 = 1
    if _jobvl_py in ('V', 'v'):
        _vl_dim2 = _n_py
    else:
        _vl_dim2 = 1
    exp_shape_vl = (_vl_dim1, _vl_dim2)
    _vl = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_vl,
        sorder=_m_sorder,
    )
    _jobvr = _b_utils._EngineCharScalarType(
        jobvr, {'fun_name': fun_name, 'entity': 'jobvr'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvr_py = _b_utils._EngineCharScalarType.to_py(_jobvr)
    if _jobvr_py in ('V', 'v'):
        _vr_dim1 = _n_py
    else:
        _vr_dim1 = 1
    if _jobvr_py in ('V', 'v'):
        _vr_dim2 = _n_py
    else:
        _vr_dim2 = 1
    exp_shape_vr = (_vr_dim1, _vr_dim2)
    _vr = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_vr,
        sorder=_m_sorder,
    )
    _lscale = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _rscale = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _rconde = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _rcondv = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        _return_scalars = _b_lapackeig.dggevx(
            balanc,
            jobvl,
            jobvr,
            sense,
            _n,
            _a,
            _b,
            _alphar,
            _alphai,
            _beta,
            _vl,
            _vr,
            _lscale,
            _rscale,
            _rconde,
            _rcondv,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _a_py = _a
    _b_py = _b
    _alphar_py = _alphar
    _alphai_py = _alphai
    _beta_py = _beta
    _vl_py = _vl
    _vr_py = _vr
    _ilo_py = _return_scalars.ilo
    _ihi_py = _return_scalars.ihi
    _lscale_py = _lscale
    _rscale_py = _rscale
    _abnrm_py = _return_scalars.abnrm
    _bbnrm_py = _return_scalars.bbnrm
    _rconde_py = _rconde
    _rcondv_py = _rcondv
    _return_data = DggevxReturnData(
        _a_py,
        _b_py,
        _alphar_py,
        _alphai_py,
        _beta_py,
        _vl_py,
        _vr_py,
        _ilo_py,
        _ihi_py,
        _lscale_py,
        _rscale_py,
        _abnrm_py,
        _bbnrm_py,
        _rconde_py,
        _rcondv_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

Dggev3ReturnData = _collections.namedtuple(
    'Dggev3ReturnData',
    ['a', 'b', 'alphar', 'alphai', 'beta', 'vl', 'vr'],
)

def dggev3(jobvl, jobvr, a, b):
    u"""
    Computes, for a real nonsymmetric matrix pair, using BLAS-3, the
    generalized eigenvalues, and optionally, the left and/or right
    generalized eigenvectors.

    ``dggev3`` computes for a pair of n*n real nonsymmetric matrices
    (A,B) the generalized eigenvalues and, optionally, the left and/or
    right generalized eigenvectors using the QZ algorithm.

    For full information please refer to the NAG Library document for
    f08wc

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08wcf.html

    Parameters
    ----------
    jobvl : str, length 1
        If `jobvl` = 'N', do not compute the left generalized
        eigenvectors.

        If `jobvl` = 'V', compute the left generalized eigenvectors.

    jobvr : str, length 1
        If `jobvr` = 'N', do not compute the right generalized
        eigenvectors.

        If `jobvr` = 'V', compute the right generalized eigenvectors.

    a : float, array-like, shape (n, n)
        The matrix A in the pair (A,B).

    b : float, array-like, shape (n, n)
        The matrix B in the pair (A,B).

    Returns
    -------
    a : float, ndarray, shape (n, n)
        `a` has been overwritten.

    b : float, ndarray, shape (n, n)
        `b` has been overwritten.

    alphar : float, ndarray, shape (n)
        The element `alphar`[j-1] contains the real part of alpha_j.

    alphai : float, ndarray, shape (n)
        The element `alphai`[j-1] contains the imaginary part of
        alpha_j.

    beta : float, ndarray, shape (n)
        (`alphar`[j-1]+`alphai`[j-1]*i)/`beta`[j-1], for j = 1,2,...,n,
        will be the generalized eigenvalues.

        If `alphai`[j-1] is zero, then the jth eigenvalue is real; if
        positive, then the jth and (j+1)st eigenvalues are a complex
        conjugate pair, with `alphai`[j] negative.

        **Note:** the quotients `alphar`[j-1]/`beta`[j-1] and
        `alphai`[j-1]/`beta`[j-1] may easily overflow or underflow, and
        `beta`[j-1] may even be zero.

        Thus, you should avoid naively computing the ratio
        alpha_j/beta_j.

        However, max(|alpha_j|) will always be less than and usually
        comparable with ||A||_2 in magnitude, and max(|beta_j|) will
        always be less than and usually comparable with ||B||_2.

    vl : float, ndarray, shape (:, :)
        If `jobvl` = 'V', the left eigenvectors u_j are stored one after
        another in the columns of `vl`, in the same order as the
        corresponding eigenvalues.

        If the jth eigenvalue is real, then u_j = `vl`[:,j], the jth
        column of `vl`.

        If the jth and (j+1)th eigenvalues form a complex conjugate
        pair, then u_j = `vl`[:,j]+i*`vl`[:,j+1] and u(j+1) =
        `vl`[:,j]-i*`vl`[:,j+1].

        Each eigenvector will be scaled so the largest component has
        |real part|+|imag. part| = 1.

        If `jobvl` = 'N', `vl` is not referenced.

    vr : float, ndarray, shape (:, :)
        If `jobvr` = 'V', the right eigenvectors v_j are stored one
        after another in the columns of `vr`, in the same order as the
        corresponding eigenvalues.

        If the jth eigenvalue is real, then v_j = `vr`[:,j], the jth
        column of VR.

        If the jth and (j+1)th eigenvalues form a complex conjugate
        pair, then v_j = `vr`[:,j]+i*`vr`[:,j+1] and v_{j+1} =
        `vr`[:,j]-i*`vr`[:,j+1].

        Each eigenvector will be scaled so the largest component has
        |real part|+|imag. part| = 1.

        If `jobvr` = 'N', `vr` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobvl`.

            Constraint: `jobvl` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `jobvr`.

            Constraint: `jobvr` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` n+1)
            The QZ iteration failed with an unexpected error, please
            contact NAG.

        (`errno` n+2)
            A failure occurred in ``dtgevc`` while computing generalized
            eigenvectors.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i<=n))
            The QZ iteration failed. No eigenvectors have been
            calculated but `alphar`[j], `alphai`[j] and `beta`[j] should
            be correct from element *<value>*.

    Notes
    -----
    A generalized eigenvalue for a pair of matrices (A,B) is a scalar
    lambda or a ratio alpha/beta = lambda, such that A-lambda B is
    singular.
    It is usually represented as the pair (alpha,beta), as there is a
    reasonable interpretation for beta = 0, and even for both being
    zero.

    The right eigenvector v_j corresponding to the eigenvalue lambda_j
    of (A,B) satisfies

        Av_j = lambda_j Bv_j.

    The left eigenvector u_j corresponding to the eigenvalue lambda_j of
    (A,B) satisfies

        u_j^HA = lambda_j u_j^HB,

    where u_j^H is the conjugate-transpose of u_j.

    All the eigenvalues and, if required, all the eigenvectors of the
    generalized eigenproblem Ax = lambda Bx, where A and B are real,
    square matrices, are determined using the QZ algorithm.
    The QZ algorithm consists of four stages:

    (1) A is reduced to upper Hessenberg form and at the same time B is
        reduced to upper triangular form.

    (#) A is further reduced to quasi-triangular form while the
        triangular form of B is maintained. This is the real generalized
        Schur form of the pair (A,B).

    (#) The quasi-triangular form of A is reduced to triangular form and
        the eigenvalues extracted. This function does not actually
        produce the eigenvalues lambda_j, but instead returns alpha_j
        and beta_j such that

            lambda_j = alpha_j/beta_j, j = 1,2,...,n.

        The division by beta_j becomes your responsibility, since beta_j
        may be zero, indicating an infinite eigenvalue. Pairs of complex
        eigenvalues occur with alpha_j/beta_j and alpha_{j+1}/beta_{j+1}
        complex conjugates, even though alpha_j and alpha_{j+1} are not
        conjugate.

    (#) If the eigenvectors are required they are obtained from the
        triangular matrices and then transformed back into the original
        coordinate system.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 2012, `Matrix Computations`, (4th
    Edition), Johns Hopkins University Press, Baltimore

    Wilkinson, J H, 1979, `Kronecker's canonical form and the QZ
    algorithm`, Linear Algebra Appl. (28), 285--303
    """

    fun_name = 'naginterfaces.library.lapackeig.dggev3'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'), (_b, 'b')),
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _alphar = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _alphai = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _beta = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobvl = _b_utils._EngineCharScalarType(
        jobvl, {'fun_name': fun_name, 'entity': 'jobvl'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvl_py = _b_utils._EngineCharScalarType.to_py(_jobvl)
    if _jobvl_py in ('V', 'v'):
        _vl_dim1 = _n_py
    else:
        _vl_dim1 = 1
    if _jobvl_py in ('V', 'v'):
        _vl_dim2 = _n_py
    else:
        _vl_dim2 = 1
    exp_shape_vl = (_vl_dim1, _vl_dim2)
    _vl = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_vl,
        sorder=_m_sorder,
    )
    _jobvr = _b_utils._EngineCharScalarType(
        jobvr, {'fun_name': fun_name, 'entity': 'jobvr'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvr_py = _b_utils._EngineCharScalarType.to_py(_jobvr)
    if _jobvr_py in ('V', 'v'):
        _vr_dim1 = _n_py
    else:
        _vr_dim1 = 1
    if _jobvr_py in ('V', 'v'):
        _vr_dim2 = _n_py
    else:
        _vr_dim2 = 1
    exp_shape_vr = (_vr_dim1, _vr_dim2)
    _vr = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_vr,
        sorder=_m_sorder,
    )
    try:
        _b_lapackeig.dggev3(
            jobvl,
            jobvr,
            _n,
            _a,
            _b,
            _alphar,
            _alphai,
            _beta,
            _vl,
            _vr,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _a_py = _a
    _b_py = _b
    _alphar_py = _alphar
    _alphai_py = _alphai
    _beta_py = _beta
    _vl_py = _vl
    _vr_py = _vr
    _return_data = Dggev3ReturnData(
        _a_py,
        _b_py,
        _alphar_py,
        _alphai_py,
        _beta_py,
        _vl_py,
        _vr_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

DgghrdReturnData = _collections.namedtuple(
    'DgghrdReturnData',
    ['a', 'b', 'q', 'z'],
)

@_b_utils._withdrawal(
    'naginterfaces.library.lapackeig.dgghrd',
    mkwdrn=None,
    advice=[
        'Please use dgghd3 instead.',
        'See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html',
    ]
)
def dgghrd(compq, compz, ilo, ihi, a, b, q, z):
    u"""
    Performs an orthogonal reduction of a pair of real general matrices
    to generalized upper Hessenberg form.

    ``dgghrd`` reduces a pair of real matrices (A,B), where B is upper
    triangular, to the generalized upper Hessenberg form using
    orthogonal transformations. ``dgghrd`` is marked as `deprecated` by
    LAPACK; the replacement routine is ``dgghd3`` which makes better use
    of Level 3 BLAS.

    Note: ``dgghrd`` is deprecated.
    Please use ``dgghd3`` instead.
    See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html

    For full information please refer to the NAG Library document for
    f08we

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08wef.html

    Parameters
    ----------
    compq : str, length 1
        Specifies the form of the computed orthogonal matrix Q.

        `compq` = 'N'

            Do not compute Q.

        `compq` = 'I'

            The orthogonal matrix Q is returned.

        `compq` = 'V'

            `q` must contain an orthogonal matrix Q_1, and the product
            Q_1 Q is returned.

    compz : str, length 1
        Specifies the form of the computed orthogonal matrix Z.

        `compz` = 'N'

            Do not compute Z.

        `compz` = 'I'

            The orthogonal matrix Z is returned.

        `compz` = 'V'

            `z` must contain an orthogonal matrix Z_1, and the product
            Z_1 Z is returned.

    ilo : int
        i_lo and i_hi as determined by a previous call to ``dggbal``.
        Otherwise, they should be set to 1 and n, respectively.

    ihi : int
        i_lo and i_hi as determined by a previous call to ``dggbal``.
        Otherwise, they should be set to 1 and n, respectively.

    a : float, array-like, shape (n, n)
        The matrix A of the matrix pair (A,B). Usually, this is the
        matrix A returned by ``dormqr``.

    b : float, array-like, shape (n, n)
        The upper triangular matrix B of the matrix pair (A,B). Usually,
        this is the matrix B returned by the QR factorization function
        ``dgeqrf``.

    q : float, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `compq` in ('I', 'V'): n; if
        `compq`='N': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compq` in ('I', 'V'): n; if
        `compq`='N': 1; otherwise: 0.

        If `compq` = 'V', `q` must contain an orthogonal matrix Q_1.

        If `compq` = 'N', `q` is not referenced.

    z : float, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `compz` in ('V', 'I'): max(1, n); if
        `compz`='N': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compz` in ('V', 'I'): n; if
        `compz`='N': 1; otherwise: 0.

        If `compz` = 'V', `z` must contain an orthogonal matrix Z_1.

        If `compz` = 'N', `z` is not referenced.

    Returns
    -------
    a : float, ndarray, shape (n, n)
        `a` is overwritten by the upper Hessenberg matrix H.

    b : float, ndarray, shape (n, n)
        `b` is overwritten by the upper triangular matrix T.

    q : float, ndarray, shape (:, :)
        If `compq` = 'I', `q` contains the orthogonal matrix Q.

        If `compq` = 'V', `q` is overwritten by Q_1 Q.

    z : float, ndarray, shape (:, :)
        If `compz` = 'I', `z` contains the orthogonal matrix Z.

        If `compz` = 'V', `z` is overwritten by Z_1 Z.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `compq`.

            Constraint: `compq` = 'N', 'I' or 'V'.

        (`errno` -2)
            On entry, error in parameter `compz`.

            Constraint: `compz` = 'N', 'V' or 'I'.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -4)
            On entry, error in parameter `ilo`.

            Constraint: `ilo` >= 1.

        (`errno` -5)
            On entry, error in parameter `ihi`.

            Constraint: 1 <= `ilo` <= `ihi` <= n.

        (`errno` -5)
            On entry, error in parameter `ihi`.

            Constraint: `ilo` = 1 and `ihi` = 0.

    Notes
    -----
    ``dgghrd`` is the third step in the solution of the real generalized
    eigenvalue problem

        Ax = lambda Bx.

    The (optional) first step balances the two matrices using
    ``dggbal``.
    In the second step, matrix B is reduced to upper triangular form
    using the QR factorization function ``dgeqrf`` and this orthogonal
    transformation Q is applied to matrix A by calling ``dormqr``.

    ``dgghrd`` reduces a pair of real matrices (A,B), where B is upper
    triangular, to the generalized upper Hessenberg form using
    orthogonal transformations.
    This two-sided transformation is of the form

        Q^TAZ = H;Q^TBZ = T

    where H is an upper Hessenberg matrix, T is an upper triangular
    matrix and Q and Z are orthogonal matrices determined as products of
    Givens rotations.
    They may either be formed explicitly, or they may be postmultiplied
    into input matrices Q_1 and Z_1, so that

        Q_1 AZ_1^T = (Q_1 Q)H(Z_1 Z)^T,;Q_1 BZ_1^T = (Q_1 Q)T(Z_1 Z)^T.

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore

    Moler, C B and Stewart, G W, 1973, `An algorithm for generalized
    matrix eigenproblems`, SIAM J. Numer. Anal. (10), 241--256
    """

    fun_name = 'naginterfaces.library.lapackeig.dgghrd'

    _z = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'z'},
        z,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _q = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'q'},
        q,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b_lapackeig.dgghrd(
        compq,
        compz,
        _n,
        ilo,
        ihi,
        _a,
        _b,
        _q,
        _z,
    )
    _a_py = _a
    _b_py = _b
    _q_py = _q
    _z_py = _z
    _return_data = DgghrdReturnData(
        _a_py,
        _b_py,
        _q_py,
        _z_py,
    )
    return _return_data

Dgghd3ReturnData = _collections.namedtuple(
    'Dgghd3ReturnData',
    ['a', 'b', 'q', 'z'],
)

def dgghd3(compq, compz, ilo, ihi, a, b, q, z):
    u"""
    Performs, using BLAS-3, an orthogonal reduction of a pair of real
    general matrices to generalized upper Hessenberg form.

    ``dgghd3`` reduces a pair of real matrices (A,B), where B is upper
    triangular, to the generalized upper Hessenberg form using
    orthogonal transformations.

    For full information please refer to the NAG Library document for
    f08wf

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08wff.html

    Parameters
    ----------
    compq : str, length 1
        Specifies the form of the computed orthogonal matrix Q.

        `compq` = 'N'

            Do not compute Q.

        `compq` = 'I'

            The orthogonal matrix Q is returned.

        `compq` = 'V'

            `q` must contain an orthogonal matrix Q_1, and the product
            Q_1 Q is returned.

    compz : str, length 1
        Specifies the form of the computed orthogonal matrix Z.

        `compz` = 'N'

            Do not compute Z.

        `compz` = 'I'

            The orthogonal matrix Z is returned.

        `compz` = 'V'

            `z` must contain an orthogonal matrix Z_1, and the product
            Z_1 Z is returned.

    ilo : int
        i_lo and i_hi as determined by a previous call to ``dggbal``.
        Otherwise, they should be set to 1 and n, respectively.

    ihi : int
        i_lo and i_hi as determined by a previous call to ``dggbal``.
        Otherwise, they should be set to 1 and n, respectively.

    a : float, array-like, shape (n, n)
        The matrix A of the matrix pair (A,B). Usually, this is the
        matrix A returned by ``dormqr``.

    b : float, array-like, shape (n, n)
        The upper triangular matrix B of the matrix pair (A,B). Usually,
        this is the matrix B returned by the QR factorization function
        ``dgeqrf``.

    q : float, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `compq` in ('I', 'V'): n; if
        `compq`='N': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compq` in ('I', 'V'): n; if
        `compq`='N': 1; otherwise: 0.

        If `compq` = 'V', `q` must contain an orthogonal matrix Q_1.

        If `compq` = 'N', `q` is not referenced.

    z : float, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `compz` in ('V', 'I'): max(1, n); if
        `compz`='N': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compz` in ('V', 'I'): n; if
        `compz`='N': 1; otherwise: 0.

        If `compz` = 'V', `z` must contain an orthogonal matrix Z_1.

        If `compz` = 'N', `z` is not referenced.

    Returns
    -------
    a : float, ndarray, shape (n, n)
        `a` is overwritten by the upper Hessenberg matrix H.

    b : float, ndarray, shape (n, n)
        `b` is overwritten by the upper triangular matrix T.

    q : float, ndarray, shape (:, :)
        If `compq` = 'I', `q` contains the orthogonal matrix Q.

        If `compq` = 'V', `q` is overwritten by Q_1 Q.

    z : float, ndarray, shape (:, :)
        If `compz` = 'I', `z` contains the orthogonal matrix Z.

        If `compz` = 'V', `z` is overwritten by Z_1 Z.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `compq`.

            Constraint: `compq` = 'N', 'I' or 'V'.

        (`errno` -2)
            On entry, error in parameter `compz`.

            Constraint: `compz` = 'N', 'V' or 'I'.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -4)
            On entry, error in parameter `ilo`.

            Constraint: `ilo` >= 1.

        (`errno` -5)
            On entry, error in parameter `ihi`.

            Constraint: 1 <= `ilo` <= `ihi` <= n.

        (`errno` -5)
            On entry, error in parameter `ihi`.

            Constraint: `ilo` = 1 and `ihi` = 0.

    Notes
    -----
    ``dgghd3`` is the third step in the solution of the real generalized
    eigenvalue problem

        Ax = lambda Bx.

    The (optional) first step balances the two matrices using
    ``dggbal``.
    In the second step, matrix B is reduced to upper triangular form
    using the QR factorization function ``dgeqrf`` and this orthogonal
    transformation Q is applied to matrix A by calling ``dormqr``.
    The driver, ``dggev3``, solves the real generalized eigenvalue
    problem by combining all the required steps including those just
    listed.

    ``dgghd3`` reduces a pair of real matrices (A,B), where B is upper
    triangular, to the generalized upper Hessenberg form using
    orthogonal transformations.
    This two-sided transformation is of the form

        Q^TAZ = H,;Q^TBZ = T

    where H is an upper Hessenberg matrix, T is an upper triangular
    matrix and Q and Z are orthogonal matrices determined as products of
    Givens rotations.
    They may either be formed explicitly, or they may be postmultiplied
    into input matrices Q_1 and Z_1, so that

        Q_1 AZ_1^T = (Q_1 Q)H(Z_1 Z)^T,;Q_1 BZ_1^T = (Q_1 Q)T(Z_1 Z)^T.

    References
    ----------
    Golub, G H and Van Loan, C F, 2012, `Matrix Computations`, (4th
    Edition), Johns Hopkins University Press, Baltimore

    Moler, C B and Stewart, G W, 1973, `An algorithm for generalized
    matrix eigenproblems`, SIAM J. Numer. Anal. (10), 241--256
    """

    fun_name = 'naginterfaces.library.lapackeig.dgghd3'

    _z = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'z'},
        z,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_z, {'fun_name': fun_name, 'entity': 'z'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _q = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'q'},
        q,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b_lapackeig.dgghd3(
        compq,
        compz,
        _n,
        ilo,
        ihi,
        _a,
        _b,
        _q,
        _z,
    )
    _a_py = _a
    _b_py = _b
    _q_py = _q
    _z_py = _z
    _return_data = Dgghd3ReturnData(
        _a_py,
        _b_py,
        _q_py,
        _z_py,
    )
    return _return_data

DggbalReturnData = _collections.namedtuple(
    'DggbalReturnData',
    ['a', 'b', 'ilo', 'ihi', 'lscale', 'rscale'],
)

def dggbal(job, n, a, b):
    u"""
    Balances a pair of real, square, matrices.

    ``dggbal`` balances a pair of real square matrices (A,B) of order n.
    Balancing usually improves the accuracy of computed generalized
    eigenvalues and eigenvectors.

    For full information please refer to the NAG Library document for
    f08wh

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08whf.html

    Parameters
    ----------
    job : str, length 1
        Specifies the operations to be performed on matrices A and B.

        `job` = 'N'

            No balancing is done. Initialize `ilo` = 1, `ihi` = `n`,
            `lscale`[i-1] = 1.0 and `rscale`[i-1] = 1.0, for i =
            1,2,...,n.

        `job` = 'P'

            Only permutations are used in balancing.

        `job` = 'S'

            Only scalings are are used in balancing.

        `job` = 'B'

            Both permutations and scalings are used in balancing.

    n : int
        n, the order of the matrices A and B.

    a : float, array-like, shape (`n`, `n`)
        The n*n matrix A.

    b : float, array-like, shape (`n`, `n`)
        The n*n matrix B.

    Returns
    -------
    a : float, ndarray, shape (`n`, `n`)
        `a` is overwritten by the balanced matrix. If `job` = 'N', `a`
        is not referenced.

    b : float, ndarray, shape (`n`, `n`)
        `b` is overwritten by the balanced matrix. If `job` = 'N', `b`
        is not referenced.

    ilo : int
        i_lo and i_hi are set such that `a`[i-1,j-1] = 0 and
        `b`[i-1,j-1] = 0 if i > j and 1 <= j < i_lo or i_hi < i <= n.

        If `job` = 'N' or 'S', i_lo = 1 and i_hi = n.

    ihi : int
        i_lo and i_hi are set such that `a`[i-1,j-1] = 0 and
        `b`[i-1,j-1] = 0 if i > j and 1 <= j < i_lo or i_hi < i <= n.

        If `job` = 'N' or 'S', i_lo = 1 and i_hi = n.

    lscale : float, ndarray, shape (`n`)
        Details of the permutations and scaling factors applied to the
        left side of the matrices A and B. If P_i is the index of the
        row interchanged with row i and d_i is the scaling factor
        applied to row i, then

            `lscale`[i-1] = P_i, for i = 1,2,...,i_lo-1;

            `lscale`[i-1] = d_i, for i = i_lo,...,i_hi;

            `lscale`[i-1] = P_i, for i = i_hi+1,...,n.

        The order in which the interchanges are made is n to i_hi+1,
        then 1 to i_lo-1.

    rscale : float, ndarray, shape (`n`)
        Details of the permutations and scaling factors applied to the
        right side of the matrices A and B.

        If P_j is the index of the column interchanged with column j and
        d^_j is the scaling factor applied to column j, then

            `rscale`[j-1] = P_j, for j = 1,2,...,i_lo-1;

            `rscale`[j-1] = d^_j, for j = i_lo,...,i_hi;

            `rscale`[j-1] = P_j, for j = i_hi+1,...,n.

        The order in which the interchanges are made is n to i_hi+1,
        then 1 to i_lo-1.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'N', 'P', 'S' or 'B'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Notes
    -----
    Balancing may reduce the 1-norms of the matrices and improve the
    accuracy of the computed eigenvalues and eigenvectors in the real
    generalized eigenvalue problem

        Ax = lambda Bx.

    ``dggbal`` is usually the first step in the solution of the above
    generalized eigenvalue problem.
    Balancing is optional but it is highly recommended.

    The term 'balancing' covers two steps, each of which involves
    similarity transformations on A and B.
    The function can perform either or both of these steps.
    Both steps are optional.

    (1) The function first attempts to permute A and B to block upper
        triangular form by a similarity transformation:

            PAP^T = F = [table omitted]

            PBP^T = G = [table omitted]

        where P is a permutation matrix, F_11, F_33, G_11 and G_33 are
        upper triangular. Then the diagonal elements of the matrix pairs
        (F_11,G_11) and (F_33,G_33) are generalized eigenvalues of
        (A,B). The rest of the generalized eigenvalues are given by the
        matrix pair (F_22,G_22) which are in rows and columns i_lo to
        i_hi. Subsequent operations to compute the generalized
        eigenvalues of (A,B) need only be applied to the matrix pair
        (F_22,G_22); this can save a significant amount of work if i_lo
        > 1 and i_hi < n. If no suitable permutation exists (as is often
        the case), the function sets i_lo = 1 and i_hi = n.

    (#) The function applies a diagonal similarity transformation to
        (F,G), to make the rows and columns of (F_22,G_22) as close in
        norm as possible:

            DFD^= [table omitted][table omitted][table omitted]

            DGD^= [table omitted][table omitted][table omitted]

        This transformation usually improves the accuracy of computed
        generalized eigenvalues and eigenvectors.

    References
    ----------
    Ward, R C, 1981, `Balancing the generalized eigenvalue problem`,
    SIAM J. Sci. Stat. Comp. (2), 141--152
    """

    fun_name = 'naginterfaces.library.lapackeig.dggbal'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _lscale = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _rscale = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _return_scalars = _b_lapackeig.dggbal(
        job,
        _n,
        _a,
        _b,
        _lscale,
        _rscale,
    )
    _a_py = _a
    _b_py = _b
    _ilo_py = _return_scalars.ilo
    _ihi_py = _return_scalars.ihi
    _lscale_py = _lscale
    _rscale_py = _rscale
    _return_data = DggbalReturnData(
        _a_py,
        _b_py,
        _ilo_py,
        _ihi_py,
        _lscale_py,
        _rscale_py,
    )
    return _return_data

def dggbak(job, side, ilo, ihi, lscale, rscale, v):
    u"""
    Transforms eigenvectors of a pair of real balanced matrices to those
    of original matrix pair supplied to ``dggbal``.

    ``dggbak`` forms the right or left eigenvectors of the real
    generalized eigenvalue problem Ax = lambda Bx, by backward
    transformation on the computed eigenvectors given by ``dtgevc``.
    It is necessary to call this function only if the optional balancing
    function ``dggbal`` was previously called to balance the matrix pair
    (A,B).

    For full information please refer to the NAG Library document for
    f08wj

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08wjf.html

    Parameters
    ----------
    job : str, length 1
        Specifies the backward transformation step required.

        `job` = 'N'

            No transformations are done.

        `job` = 'P'

            Only do backward transformations based on permutations.

        `job` = 'S'

            Only do backward transformations based on scaling.

        `job` = 'B'

            Do backward transformations for both permutations and
            scaling.

        **Note:** this must be the same argument `job` as supplied to
        ``dggbal``.

    side : str, length 1
        Indicates whether left or right eigenvectors are to be
        transformed.

        `side` = 'L'

            The left eigenvectors are transformed.

        `side` = 'R'

            The right eigenvectors are transformed.

    ilo : int
        i_lo and i_hi as determined by a previous call to ``dggbal``.

    ihi : int
        i_lo and i_hi as determined by a previous call to ``dggbal``.

    lscale : float, array-like, shape (n)
        Details of the permutations and scaling factors applied to the
        left side of the matrices A and B, as returned by a previous
        call to ``dggbal``.

    rscale : float, array-like, shape (n)
        Details of the permutations and scaling factors applied to the
        right side of the matrices A and B, as returned by a previous
        call to ``dggbal``.

    v : float, array-like, shape (n, m)
        The matrix of right or left eigenvectors, as returned by
        ``dggbal``.

    Returns
    -------
    v : float, ndarray, shape (n, m)
        The transformed right or left eigenvectors.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'N', 'P', 'S' or 'B'.

        (`errno` -2)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -4)
            On entry, error in parameter `ilo`.

        (`errno` -5)
            On entry, error in parameter `ihi`.

        (`errno` -8)
            On entry, error in parameter m.

            Constraint: 0 <= m <= n.

    Notes
    -----
    If the matrix pair has been previously balanced using the function
    ``dggbal`` then ``dggbak`` backtransforms the eigenvector solution
    given by ``dtgevc``.
    This is usually the sixth and last step in the solution of the
    generalized eigenvalue problem.

    For a description of balancing, see the document for ``dggbal``.

    References
    ----------
    Ward, R C, 1981, `Balancing the generalized eigenvalue problem`,
    SIAM J. Sci. Stat. Comp. (2), 141--152
    """

    fun_name = 'naginterfaces.library.lapackeig.dggbak'

    _v = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'v'},
        v,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _lscale = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'lscale'},
        lscale,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _rscale = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'rscale'},
        rscale,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _b_lapackeig.dggbak(
        job,
        side,
        _n,
        ilo,
        ihi,
        _lscale,
        _rscale,
        _m,
        _v,
    )
    _v_py = _v
    return _v_py

ZggevReturnData = _collections.namedtuple(
    'ZggevReturnData',
    ['a', 'b', 'alpha', 'beta', 'vl', 'vr'],
)

@_b_utils._withdrawal(
    'naginterfaces.library.lapackeig.zggev',
    mkwdrn=None,
    advice=[
        'Please use zggev3 instead.',
        'See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html',
    ]
)
def zggev(jobvl, jobvr, a, b):
    u"""
    Computes, for a complex nonsymmetric matrix pair, the generalized
    eigenvalues, and optionally, the left and/or right generalized
    eigenvectors.

    ``zggev`` computes for a pair of n*n complex nonsymmetric matrices
    (A,B) the generalized eigenvalues and, optionally, the left and/or
    right generalized eigenvectors using the QZ algorithm. ``zggev`` is
    marked as `deprecated` by LAPACK; the replacement routine is
    ``zggev3`` which makes better use of Level 3 BLAS.

    Note: ``zggev`` is deprecated.
    Please use ``zggev3`` instead.
    See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html

    For full information please refer to the NAG Library document for
    f08wn

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08wnf.html

    Parameters
    ----------
    jobvl : str, length 1
        If `jobvl` = 'N', do not compute the left generalized
        eigenvectors.

        If `jobvl` = 'V', compute the left generalized eigenvectors.

    jobvr : str, length 1
        If `jobvr` = 'N', do not compute the right generalized
        eigenvectors.

        If `jobvr` = 'V', compute the right generalized eigenvectors.

    a : complex, array-like, shape (n, n)
        The matrix A in the pair (A,B).

    b : complex, array-like, shape (n, n)
        The matrix B in the pair (A,B).

    Returns
    -------
    a : complex, ndarray, shape (n, n)
        `a` has been overwritten.

    b : complex, ndarray, shape (n, n)
        `b` has been overwritten.

    alpha : complex, ndarray, shape (n)
        See the description of `beta`.

    beta : complex, ndarray, shape (n)
        `alpha`[j-1]/`beta`[j-1], for j = 1,2,...,n, will be the
        generalized eigenvalues.

        **Note:** the quotients `alpha`[j-1]/`beta`[j-1] may easily
        overflow or underflow, and `beta`[j-1] may even be zero.

        Thus, you should avoid naively computing the ratio
        alpha_j/beta_j.

        However, max(|alpha_j|) will always be less than and usually
        comparable with ||A||_2 in magnitude, and max(|beta_j|) will
        always be less than and usually comparable with ||B||_2.

    vl : complex, ndarray, shape (:, :)
        If `jobvl` = 'V', the left generalized eigenvectors u_j are
        stored one after another in the columns of `vl`, in the same
        order as the corresponding eigenvalues. Each eigenvector will be
        scaled so the largest component will have |real part|+|imag.
        part| = 1.

        If `jobvl` = 'N', `vl` is not referenced.

    vr : complex, ndarray, shape (:, :)
        If `jobvr` = 'V', the right generalized eigenvectors v_j are
        stored one after another in the columns of `vr`, in the same
        order as the corresponding eigenvalues. Each eigenvector will be
        scaled so the largest component will have |real part|+|imag.
        part| = 1.

        If `jobvr` = 'N', `vr` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobvl`.

            Constraint: `jobvl` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `jobvr`.

            Constraint: `jobvr` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` n+1)
            The QZ iteration failed with an unexpected error, please
            contact NAG.

        (`errno` n+2)
            A failure occurred in ``ztgevc`` while computing generalized
            eigenvectors.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i<=n))
            The QZ iteration failed. No eigenvectors have been
            calculated but `alpha` and `beta` should be correct from
            element *<value>*.

    Notes
    -----
    A generalized eigenvalue for a pair of matrices (A,B) is a scalar
    lambda or a ratio alpha/beta = lambda, such that A-lambda B is
    singular.
    It is usually represented as the pair (alpha,beta), as there is a
    reasonable interpretation for beta = 0, and even for both being
    zero.

    The right generalized eigenvector v_j corresponding to the
    generalized eigenvalue lambda_j of (A,B) satisfies

        Av_j = lambda_j Bv_j.

    The left generalized eigenvector u_j corresponding to the
    generalized eigenvalue lambda_j of (A,B) satisfies

        u_j^HA = lambda_j u_j^HB,

    where u_j^H is the conjugate-transpose of u_j.

    All the eigenvalues and, if required, all the eigenvectors of the
    complex generalized eigenproblem Ax = lambda Bx, where A and B are
    complex, square matrices, are determined using the QZ algorithm.
    The complex QZ algorithm consists of three stages:

    (1) A is reduced to upper Hessenberg form (with real, non-negative
        subdiagonal elements) and at the same time B is reduced to upper
        triangular form.

    (#) A is further reduced to triangular form while the triangular
        form of B is maintained and the diagonal elements of B are made
        real and non-negative. This is the generalized Schur form of the
        pair (A,B).

        This function does not actually produce the eigenvalues
        lambda_j, but instead returns alpha_j and beta_j such that

            lambda_j = alpha_j/beta_j, j = 1,2,...,n.

        The division by beta_j becomes your responsibility, since beta_j
        may be zero, indicating an infinite eigenvalue.

    (#) If the eigenvectors are required they are obtained from the
        triangular matrices and then transformed back into the original
        coordinate system.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 2012, `Matrix Computations`, (4th
    Edition), Johns Hopkins University Press, Baltimore

    Wilkinson, J H, 1979, `Kronecker's canonical form and the QZ
    algorithm`, Linear Algebra Appl. (28), 285--303
    """

    fun_name = 'naginterfaces.library.lapackeig.zggev'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'), (_b, 'b')),
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _alpha = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_n_py,),
    )
    _beta = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobvl = _b_utils._EngineCharScalarType(
        jobvl, {'fun_name': fun_name, 'entity': 'jobvl'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvl_py = _b_utils._EngineCharScalarType.to_py(_jobvl)
    if _jobvl_py in ('V', 'v'):
        _vl_dim1 = _n_py
    else:
        _vl_dim1 = 1
    if _jobvl_py in ('V', 'v'):
        _vl_dim2 = _n_py
    else:
        _vl_dim2 = 1
    exp_shape_vl = (_vl_dim1, _vl_dim2)
    _vl = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_vl,
        sorder=_m_sorder,
    )
    _jobvr = _b_utils._EngineCharScalarType(
        jobvr, {'fun_name': fun_name, 'entity': 'jobvr'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvr_py = _b_utils._EngineCharScalarType.to_py(_jobvr)
    if _jobvr_py in ('V', 'v'):
        _vr_dim1 = _n_py
    else:
        _vr_dim1 = 1
    if _jobvr_py in ('V', 'v'):
        _vr_dim2 = _n_py
    else:
        _vr_dim2 = 1
    exp_shape_vr = (_vr_dim1, _vr_dim2)
    _vr = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_vr,
        sorder=_m_sorder,
    )
    try:
        _b_lapackeig.zggev(
            jobvl,
            jobvr,
            _n,
            _a,
            _b,
            _alpha,
            _beta,
            _vl,
            _vr,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _a_py = _a
    _b_py = _b
    _alpha_py = _alpha
    _beta_py = _beta
    _vl_py = _vl
    _vr_py = _vr
    _return_data = ZggevReturnData(
        _a_py,
        _b_py,
        _alpha_py,
        _beta_py,
        _vl_py,
        _vr_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

ZggevxReturnData = _collections.namedtuple(
    'ZggevxReturnData',
    ['a', 'b', 'alpha', 'beta', 'vl', 'vr', 'ilo', 'ihi', 'lscale', 'rscale', 'abnrm', 'bbnrm', 'rconde', 'rcondv'],
)

def zggevx(balanc, jobvl, jobvr, sense, a, b):
    u"""
    Computes, for a complex nonsymmetric matrix pair, the generalized
    eigenvalues, and optionally, the left and/or right generalized
    eigenvectors; also, optionally, the balancing transformation, the
    reciprocal condition numbers for the eigenvalues and for the right
    eigenvectors.

    ``zggevx`` computes for a pair of n*n complex nonsymmetric matrices
    (A,B) the generalized eigenvalues and, optionally, the left and/or
    right generalized eigenvectors using the QZ algorithm.

    Optionally it also computes a balancing transformation to improve
    the conditioning of the eigenvalues and eigenvectors, reciprocal
    condition numbers for the eigenvalues, and reciprocal condition
    numbers for the right eigenvectors.

    For full information please refer to the NAG Library document for
    f08wp

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08wpf.html

    Parameters
    ----------
    balanc : str, length 1
        Specifies the balance option to be performed.

        `balanc` = 'N'

            Do not diagonally scale or permute.

        `balanc` = 'P'

            Permute only.

        `balanc` = 'S'

            Scale only.

        `balanc` = 'B'

            Both permute and scale.

        Computed reciprocal condition numbers will be for the matrices
        after permuting and/or balancing.

        Permuting does not change condition numbers (in exact
        arithmetic), but balancing does.

        In the absence of other information, `balanc` = 'B' is
        recommended.

    jobvl : str, length 1
        If `jobvl` = 'N', do not compute the left generalized
        eigenvectors.

        If `jobvl` = 'V', compute the left generalized eigenvectors.

    jobvr : str, length 1
        If `jobvr` = 'N', do not compute the right generalized
        eigenvectors.

        If `jobvr` = 'V', compute the right generalized eigenvectors.

    sense : str, length 1
        Determines which reciprocal condition numbers are computed.

        `sense` = 'N'

            None are computed.

        `sense` = 'E'

            Computed for eigenvalues only.

        `sense` = 'V'

            Computed for eigenvectors only.

        `sense` = 'B'

            Computed for eigenvalues and eigenvectors.

    a : complex, array-like, shape (n, n)
        The matrix A in the pair (A,B).

    b : complex, array-like, shape (n, n)
        The matrix B in the pair (A,B).

    Returns
    -------
    a : complex, ndarray, shape (n, n)
        `a` has been overwritten. If `jobvl` = 'V' or `jobvr` = 'V' or
        both, then A contains the first part of the Schur form of the
        'balanced' versions of the input A and B.

    b : complex, ndarray, shape (n, n)
        `b` has been overwritten.

    alpha : complex, ndarray, shape (n)
        See the description of `beta`.

    beta : complex, ndarray, shape (n)
        `alpha`[j-1]/`beta`[j-1], for j = 1,2,...,n, will be the
        generalized eigenvalues.

        **Note:** the quotients `alpha`[j-1]/`beta`[j-1] may easily
        overflow or underflow, and `beta`[j-1] may even be zero.

        Thus, you should avoid naively computing the ratio
        alpha_j/beta_j.

        However, max(|alpha_j|) will always be less than and usually
        comparable with ||A||_2 in magnitude, and max(|beta_j|) will
        always be less than and usually comparable with ||B||_2.

    vl : complex, ndarray, shape (:, :)
        If `jobvl` = 'V', the left generalized eigenvectors u_j are
        stored one after another in the columns of `vl`, in the same
        order as the corresponding eigenvalues. Each eigenvector will be
        scaled so the largest component will have |real part|+|imag.
        part| = 1.

        If `jobvl` = 'N', `vl` is not referenced.

    vr : complex, ndarray, shape (:, :)
        If `jobvr` = 'V', the right generalized eigenvectors v_j are
        stored one after another in the columns of `vr`, in the same
        order as the corresponding eigenvalues. Each eigenvector will be
        scaled so the largest component will have |real part|+|imag.
        part| = 1.

        If `jobvr` = 'N', `vr` is not referenced.

    ilo : int
        `ilo` and `ihi` are integer values such that `a`[i-1,j-1] = 0
        and `b`[i-1,j-1] = 0 if i > j and j = 1,2,...,`ilo`-1 or i =
        `ihi`+1,...,n.

        If `balanc` = 'N' or 'S', `ilo` = 1 and `ihi` = n.

    ihi : int
        `ilo` and `ihi` are integer values such that `a`[i-1,j-1] = 0
        and `b`[i-1,j-1] = 0 if i > j and j = 1,2,...,`ilo`-1 or i =
        `ihi`+1,...,n.

        If `balanc` = 'N' or 'S', `ilo` = 1 and `ihi` = n.

    lscale : float, ndarray, shape (n)
        Details of the permutations and scaling factors applied to the
        left side of A and B.

        If pl_j is the index of the row interchanged with row j, and
        dl_j is the scaling factor applied to row j, then:

            `lscale`[j-1] = pl_j, for j = 1,2,...,`ilo`-1;

            `lscale` = dl_j, for j = `ilo`,...,`ihi`;

            `lscale` = pl_j, for j = `ihi`+1,...,n.

        The order in which the interchanges are made is n to `ihi`+1,
        then 1 to `ilo`-1.

    rscale : float, ndarray, shape (n)
        Details of the permutations and scaling factors applied to the
        right side of A and B.

        If pr_j is the index of the column interchanged with column j,
        and dr_j is the scaling factor applied to column j, then:

            `rscale`[j-1] = pr_j, for j = 1,2,...,`ilo`-1;

            if `rscale` = dr_j, for j = `ilo`,...,`ihi`;

            if `rscale` = pr_j, for j = `ihi`+1,...,n.

        The order in which the interchanges are made is n to `ihi`+1,
        then 1 to `ilo`-1.

    abnrm : float
        The 1-norm of the balanced matrix A.

    bbnrm : float
        The 1-norm of the balanced matrix B.

    rconde : float, ndarray, shape (n)
        If `sense` = 'E' or 'B', the reciprocal condition numbers of the
        eigenvalues, stored in consecutive elements of the array.

        If `sense` = 'N' or 'V', `rconde` is not referenced.

    rcondv : float, ndarray, shape (n)
        If `sense` = 'V' or 'B', the estimated reciprocal condition
        numbers of the selected eigenvectors, stored in consecutive
        elements of the array.

        If `sense` = 'N' or 'E', `rcondv` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `balanc`.

            Constraint: `balanc` = 'N', 'P', 'S' or 'B'.

        (`errno` -2)
            On entry, error in parameter `jobvl`.

            Constraint: `jobvl` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `jobvr`.

            Constraint: `jobvr` = 'N' or 'V'.

        (`errno` -4)
            On entry, error in parameter `sense`.

            Constraint: `sense` = 'N', 'E', 'V' or 'B'.

        (`errno` -5)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` n+1)
            The QZ iteration failed with an unexpected error, please
            contact NAG.

        (`errno` n+2)
            A failure occurred in ``ztgevc`` while computing generalized
            eigenvectors.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i<=n))
            The QZ iteration failed. No eigenvectors have been
            calculated but `alpha` and `beta` should be correct from
            element *<value>*.

    Notes
    -----
    A generalized eigenvalue for a pair of matrices (A,B) is a scalar
    lambda or a ratio alpha/beta = lambda, such that A-lambda B is
    singular.
    It is usually represented as the pair (alpha,beta), as there is a
    reasonable interpretation for beta = 0, and even for both being
    zero.

    The right generalized eigenvector v_j corresponding to the
    generalized eigenvalue lambda_j of (A,B) satisfies

        Av_j = lambda_j Bv_j.

    The left generalized eigenvector u_j corresponding to the
    generalized eigenvalue lambda_j of (A,B) satisfies

        u_j^HA = lambda_j u_j^HB,

    where u_j^H is the conjugate-transpose of u_j.

    All the eigenvalues and, if required, all the eigenvectors of the
    complex generalized eigenproblem Ax = lambda Bx, where A and B are
    complex, square matrices, are determined using the QZ algorithm.
    The complex QZ algorithm consists of three stages:

    (1) A is reduced to upper Hessenberg form (with real, non-negative
        subdiagonal elements) and at the same time B is reduced to upper
        triangular form.

    (#) A is further reduced to triangular form while the triangular
        form of B is maintained and the diagonal elements of B are made
        real and non-negative. This is the generalized Schur form of the
        pair (A,B).

        This function does not actually produce the eigenvalues
        lambda_j, but instead returns alpha_j and beta_j such that

            lambda_j = alpha_j/beta_j, j = 1,2,...,n.

        The division by beta_j becomes your responsibility, since beta_j
        may be zero, indicating an infinite eigenvalue.

    (#) If the eigenvectors are required they are obtained from the
        triangular matrices and then transformed back into the original
        coordinate system.

    For details of the balancing option, see [Notes for zggbal].

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 2012, `Matrix Computations`, (4th
    Edition), Johns Hopkins University Press, Baltimore

    Wilkinson, J H, 1979, `Kronecker's canonical form and the QZ
    algorithm`, Linear Algebra Appl. (28), 285--303
    """

    fun_name = 'naginterfaces.library.lapackeig.zggevx'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'), (_b, 'b')),
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _alpha = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_n_py,),
    )
    _beta = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobvl = _b_utils._EngineCharScalarType(
        jobvl, {'fun_name': fun_name, 'entity': 'jobvl'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvl_py = _b_utils._EngineCharScalarType.to_py(_jobvl)
    if _jobvl_py in ('V', 'v'):
        _vl_dim1 = _n_py
    else:
        _vl_dim1 = 1
    if _jobvl_py in ('V', 'v'):
        _vl_dim2 = _n_py
    else:
        _vl_dim2 = 1
    exp_shape_vl = (_vl_dim1, _vl_dim2)
    _vl = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_vl,
        sorder=_m_sorder,
    )
    _jobvr = _b_utils._EngineCharScalarType(
        jobvr, {'fun_name': fun_name, 'entity': 'jobvr'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvr_py = _b_utils._EngineCharScalarType.to_py(_jobvr)
    if _jobvr_py in ('V', 'v'):
        _vr_dim1 = _n_py
    else:
        _vr_dim1 = 1
    if _jobvr_py in ('V', 'v'):
        _vr_dim2 = _n_py
    else:
        _vr_dim2 = 1
    exp_shape_vr = (_vr_dim1, _vr_dim2)
    _vr = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_vr,
        sorder=_m_sorder,
    )
    _lscale = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _rscale = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _rconde = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _rcondv = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        _return_scalars = _b_lapackeig.zggevx(
            balanc,
            jobvl,
            jobvr,
            sense,
            _n,
            _a,
            _b,
            _alpha,
            _beta,
            _vl,
            _vr,
            _lscale,
            _rscale,
            _rconde,
            _rcondv,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _a_py = _a
    _b_py = _b
    _alpha_py = _alpha
    _beta_py = _beta
    _vl_py = _vl
    _vr_py = _vr
    _ilo_py = _return_scalars.ilo
    _ihi_py = _return_scalars.ihi
    _lscale_py = _lscale
    _rscale_py = _rscale
    _abnrm_py = _return_scalars.abnrm
    _bbnrm_py = _return_scalars.bbnrm
    _rconde_py = _rconde
    _rcondv_py = _rcondv
    _return_data = ZggevxReturnData(
        _a_py,
        _b_py,
        _alpha_py,
        _beta_py,
        _vl_py,
        _vr_py,
        _ilo_py,
        _ihi_py,
        _lscale_py,
        _rscale_py,
        _abnrm_py,
        _bbnrm_py,
        _rconde_py,
        _rcondv_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

Zggev3ReturnData = _collections.namedtuple(
    'Zggev3ReturnData',
    ['a', 'b', 'alpha', 'beta', 'vl', 'vr'],
)

def zggev3(jobvl, jobvr, a, b):
    u"""
    Computes, for a complex nonsymmetric matrix pair, using BLAS-3, the
    generalized eigenvalues, and optionally, the left and/or right
    generalized eigenvectors.

    ``zggev3`` computes for a pair of n*n complex nonsymmetric matrices
    (A,B) the generalized eigenvalues and, optionally, the left and/or
    right generalized eigenvectors using the QZ algorithm.

    For full information please refer to the NAG Library document for
    f08wq

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08wqf.html

    Parameters
    ----------
    jobvl : str, length 1
        If `jobvl` = 'N', do not compute the left generalized
        eigenvectors.

        If `jobvl` = 'V', compute the left generalized eigenvectors.

    jobvr : str, length 1
        If `jobvr` = 'N', do not compute the right generalized
        eigenvectors.

        If `jobvr` = 'V', compute the right generalized eigenvectors.

    a : complex, array-like, shape (n, n)
        The matrix A in the pair (A,B).

    b : complex, array-like, shape (n, n)
        The matrix B in the pair (A,B).

    Returns
    -------
    a : complex, ndarray, shape (n, n)
        `a` has been overwritten.

    b : complex, ndarray, shape (n, n)
        `b` has been overwritten.

    alpha : complex, ndarray, shape (n)
        See the description of `beta`.

    beta : complex, ndarray, shape (n)
        `alpha`[j-1]/`beta`[j-1], for j = 1,2,...,n, will be the
        generalized eigenvalues.

        **Note:** the quotients `alpha`[j-1]/`beta`[j-1] may easily
        overflow or underflow, and `beta`[j-1] may even be zero.

        Thus, you should avoid naively computing the ratio
        alpha_j/beta_j.

        However, max(|alpha_j|) will always be less than and usually
        comparable with ||A||_2 in magnitude, and max(|beta_j|) will
        always be less than and usually comparable with ||B||_2.

    vl : complex, ndarray, shape (:, :)
        If `jobvl` = 'V', the left generalized eigenvectors u_j are
        stored one after another in the columns of `vl`, in the same
        order as the corresponding eigenvalues. Each eigenvector will be
        scaled so the largest component will have |real part|+|imag.
        part| = 1.

        If `jobvl` = 'N', `vl` is not referenced.

    vr : complex, ndarray, shape (:, :)
        If `jobvr` = 'V', the right generalized eigenvectors v_j are
        stored one after another in the columns of `vr`, in the same
        order as the corresponding eigenvalues. Each eigenvector will be
        scaled so the largest component will have |real part|+|imag.
        part| = 1.

        If `jobvr` = 'N', `vr` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobvl`.

            Constraint: `jobvl` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `jobvr`.

            Constraint: `jobvr` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` n+1)
            The QZ iteration failed with an unexpected error, please
            contact NAG.

        (`errno` n+2)
            A failure occurred in ``ztgevc`` while computing generalized
            eigenvectors.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i<=n))
            The QZ iteration failed. No eigenvectors have been
            calculated but `alpha` and `beta` should be correct from
            element *<value>*.

    Notes
    -----
    A generalized eigenvalue for a pair of matrices (A,B) is a scalar
    lambda or a ratio alpha/beta = lambda, such that A-lambda B is
    singular.
    It is usually represented as the pair (alpha,beta), as there is a
    reasonable interpretation for beta = 0, and even for both being
    zero.

    The right generalized eigenvector v_j corresponding to the
    generalized eigenvalue lambda_j of (A,B) satisfies

        Av_j = lambda_j Bv_j.

    The left generalized eigenvector u_j corresponding to the
    generalized eigenvalue lambda_j of (A,B) satisfies

        u_j^HA = lambda_j u_j^HB,

    where u_j^H is the conjugate-transpose of u_j.

    All the eigenvalues and, if required, all the eigenvectors of the
    complex generalized eigenproblem Ax = lambda Bx, where A and B are
    complex, square matrices, are determined using the QZ algorithm.
    The complex QZ algorithm consists of three stages:

    (1) A is reduced to upper Hessenberg form (with real, non-negative
        subdiagonal elements) and at the same time B is reduced to upper
        triangular form.

    (#) A is further reduced to triangular form while the triangular
        form of B is maintained and the diagonal elements of B are made
        real and non-negative. This is the generalized Schur form of the
        pair (A,B).

        This function does not actually produce the eigenvalues
        lambda_j, but instead returns alpha_j and beta_j such that

            lambda_j = alpha_j/beta_j, j = 1,2,...,n.

        The division by beta_j becomes your responsibility, since beta_j
        may be zero, indicating an infinite eigenvalue.

    (#) If the eigenvectors are required they are obtained from the
        triangular matrices and then transformed back into the original
        coordinate system.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 2012, `Matrix Computations`, (4th
    Edition), Johns Hopkins University Press, Baltimore

    Wilkinson, J H, 1979, `Kronecker's canonical form and the QZ
    algorithm`, Linear Algebra Appl. (28), 285--303
    """

    fun_name = 'naginterfaces.library.lapackeig.zggev3'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'), (_b, 'b')),
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _alpha = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_n_py,),
    )
    _beta = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobvl = _b_utils._EngineCharScalarType(
        jobvl, {'fun_name': fun_name, 'entity': 'jobvl'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvl_py = _b_utils._EngineCharScalarType.to_py(_jobvl)
    if _jobvl_py in ('V', 'v'):
        _vl_dim1 = _n_py
    else:
        _vl_dim1 = 1
    if _jobvl_py in ('V', 'v'):
        _vl_dim2 = _n_py
    else:
        _vl_dim2 = 1
    exp_shape_vl = (_vl_dim1, _vl_dim2)
    _vl = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_vl,
        sorder=_m_sorder,
    )
    _jobvr = _b_utils._EngineCharScalarType(
        jobvr, {'fun_name': fun_name, 'entity': 'jobvr'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvr_py = _b_utils._EngineCharScalarType.to_py(_jobvr)
    if _jobvr_py in ('V', 'v'):
        _vr_dim1 = _n_py
    else:
        _vr_dim1 = 1
    if _jobvr_py in ('V', 'v'):
        _vr_dim2 = _n_py
    else:
        _vr_dim2 = 1
    exp_shape_vr = (_vr_dim1, _vr_dim2)
    _vr = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_vr,
        sorder=_m_sorder,
    )
    try:
        _b_lapackeig.zggev3(
            jobvl,
            jobvr,
            _n,
            _a,
            _b,
            _alpha,
            _beta,
            _vl,
            _vr,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _a_py = _a
    _b_py = _b
    _alpha_py = _alpha
    _beta_py = _beta
    _vl_py = _vl
    _vr_py = _vr
    _return_data = Zggev3ReturnData(
        _a_py,
        _b_py,
        _alpha_py,
        _beta_py,
        _vl_py,
        _vr_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

ZgghrdReturnData = _collections.namedtuple(
    'ZgghrdReturnData',
    ['a', 'b', 'q', 'z'],
)

@_b_utils._withdrawal(
    'naginterfaces.library.lapackeig.zgghrd',
    mkwdrn=None,
    advice=[
        'Please use zgghd3 instead.',
        'See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html',
    ]
)
def zgghrd(compq, compz, ilo, ihi, a, b, q, z):
    u"""
    Performs a unitary reduction of a pair of complex general matrices
    to generalized upper Hessenberg form.

    ``zgghrd`` reduces a pair of complex matrices (A,B), where B is
    upper triangular, to the generalized upper Hessenberg form using
    unitary transformations.

    Note: ``zgghrd`` is deprecated.
    Please use ``zgghd3`` instead.
    See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html

    For full information please refer to the NAG Library document for
    f08ws

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08wsf.html

    Parameters
    ----------
    compq : str, length 1
        Specifies the form of the computed unitary matrix Q.

        `compq` = 'N'

            Do not compute Q.

        `compq` = 'I'

            The unitary matrix Q is returned.

        `compq` = 'V'

            `q` must contain a unitary matrix Q_1, and the product Q_1 Q
            is returned.

    compz : str, length 1
        Specifies the form of the computed unitary matrix Z.

        `compz` = 'N'

            Do not compute Z.

        `compz` = 'V'

            `z` must contain a unitary matrix Z_1, and the product Z_1 Z
            is returned.

        `compz` = 'I'

            The unitary matrix Z is returned.

    ilo : int
        i_lo and i_hi as determined by a previous call to ``zggbal``.
        Otherwise, they should be set to 1 and n, respectively.

    ihi : int
        i_lo and i_hi as determined by a previous call to ``zggbal``.
        Otherwise, they should be set to 1 and n, respectively.

    a : complex, array-like, shape (n, n)
        The matrix A of the matrix pair (A,B). Usually, this is the
        matrix A returned by ``zunmqr``.

    b : complex, array-like, shape (n, n)
        The upper triangular matrix B of the matrix pair (A,B). Usually,
        this is the matrix B returned by the QR factorization function
        ``zgeqrf``.

    q : complex, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `compq` in ('I', 'V'): n; if
        `compq`='N': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compq` in ('I', 'V'): n; if
        `compq`='N': 1; otherwise: 0.

        If `compq` = 'V', `q` must contain a unitary matrix Q_1.

        If `compq` = 'N', `q` is not referenced.

    z : complex, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `compz` in ('V', 'I'): max(1, n); if
        `compz`='N': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compz` in ('V', 'I'): n; if
        `compz`='N': 1; otherwise: 0.

        If `compz` = 'V', `z` must contain a unitary matrix Z_1.

        If `compz` = 'N', `z` is not referenced.

    Returns
    -------
    a : complex, ndarray, shape (n, n)
        `a` is overwritten by the upper Hessenberg matrix H.

    b : complex, ndarray, shape (n, n)
        `b` is overwritten by the upper triangular matrix T.

    q : complex, ndarray, shape (:, :)
        If `compq` = 'I', `q` contains the unitary matrix Q.

        Iif `compq` = 'V', `q` is overwritten by Q_1 Q.

    z : complex, ndarray, shape (:, :)
        If `compz` = 'I', `z` contains the unitary matrix Z.

        If `compz` = 'V', `z` is overwritten by Z_1 Z.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `compq`.

            Constraint: `compq` = 'N', 'I' or 'V'.

        (`errno` -2)
            On entry, error in parameter `compz`.

            Constraint: `compz` = 'N', 'V' or 'I'.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -4)
            On entry, error in parameter `ilo`.

            Constraint: `ilo` >= 1.

        (`errno` -5)
            On entry, error in parameter `ihi`.

            Constraint: 1 <= `ilo` <= `ihi` <= n.

        (`errno` -5)
            On entry, error in parameter `ihi`.

            Constraint: `ilo` = 1 and `ihi` = 0.

    Notes
    -----
    ``zgghrd`` is usually the third step in the solution of the complex
    generalized eigenvalue problem

        Ax = lambda Bx.

    The (optional) first step balances the two matrices using
    ``zggbal``.
    In the second step, matrix B is reduced to upper triangular form
    using the QR factorization function ``zgeqrf`` and this unitary
    transformation Q is applied to matrix A by calling ``zunmqr``.

    ``zgghrd`` reduces a pair of complex matrices (A,B), where B is
    triangular, to the generalized upper Hessenberg form using unitary
    transformations.
    This two-sided transformation is of the form

        Q^HAZ = H;Q^HBZ = T

    where H is an upper Hessenberg matrix, T is an upper triangular
    matrix and Q and Z are unitary matrices determined as products of
    Givens rotations.
    They may either be formed explicitly, or they may be postmultiplied
    into input matrices Q_1 and Z_1, so that

        Q_1 AZ_1^H = (Q_1 Q)H(Z_1 Z)^H,;Q_1 BZ_1^H = (Q_1 Q)T(Z_1 Z)^H.

    References
    ----------
    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore

    Moler, C B and Stewart, G W, 1973, `An algorithm for generalized
    matrix eigenproblems`, SIAM J. Numer. Anal. (10), 241--256
    """

    fun_name = 'naginterfaces.library.lapackeig.zgghrd'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
    )
    _q = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'q'},
        q,
        copy=True,
    )
    _z = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'z'},
        z,
        copy=True,
    )
    _b_lapackeig.zgghrd(
        compq,
        compz,
        _n,
        ilo,
        ihi,
        _a,
        _b,
        _q,
        _z,
    )
    _a_py = _a
    _b_py = _b
    _q_py = _q
    _z_py = _z
    _return_data = ZgghrdReturnData(
        _a_py,
        _b_py,
        _q_py,
        _z_py,
    )
    return _return_data

Zgghd3ReturnData = _collections.namedtuple(
    'Zgghd3ReturnData',
    ['a', 'b', 'q', 'z'],
)

def zgghd3(compq, compz, ilo, ihi, a, b, q, z):
    u"""
    Performs, using BLAS-3, a unitary reduction of a pair of complex
    general matrices to generalized upper Hessenberg form.

    ``zgghd3`` reduces a pair of complex matrices (A,B), where B is
    upper triangular, to the generalized upper Hessenberg form using
    unitary transformations.

    For full information please refer to the NAG Library document for
    f08wt

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08wtf.html

    Parameters
    ----------
    compq : str, length 1
        Specifies the form of the computed unitary matrix Q.

        `compq` = 'N'

            Do not compute Q.

        `compq` = 'I'

            The unitary matrix Q is returned.

        `compq` = 'V'

            `q` must contain a unitary matrix Q_1, and the product Q_1 Q
            is returned.

    compz : str, length 1
        Specifies the form of the computed unitary matrix Z.

        `compz` = 'N'

            Do not compute Z.

        `compz` = 'V'

            `z` must contain a unitary matrix Z_1, and the product Z_1 Z
            is returned.

        `compz` = 'I'

            The unitary matrix Z is returned.

    ilo : int
        i_lo and i_hi as determined by a previous call to ``zggbal``.
        Otherwise, they should be set to 1 and n, respectively.

    ihi : int
        i_lo and i_hi as determined by a previous call to ``zggbal``.
        Otherwise, they should be set to 1 and n, respectively.

    a : complex, array-like, shape (n, n)
        The matrix A of the matrix pair (A,B). Usually, this is the
        matrix A returned by ``zunmqr``.

    b : complex, array-like, shape (n, n)
        The upper triangular matrix B of the matrix pair (A,B). Usually,
        this is the matrix B returned by the QR factorization function
        ``zgeqrf``.

    q : complex, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `compq` in ('I', 'V'): n; if
        `compq`='N': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compq` in ('I', 'V'): n; if
        `compq`='N': 1; otherwise: 0.

        If `compq` = 'V', `q` must contain a unitary matrix Q_1.

        If `compq` = 'N', `q` is not referenced.

    z : complex, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `compz` in ('V', 'I'): max(1, n); if
        `compz`='N': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compz` in ('V', 'I'): n; if
        `compz`='N': 1; otherwise: 0.

        If `compz` = 'V', `z` must contain a unitary matrix Z_1.

        If `compz` = 'N', `z` is not referenced.

    Returns
    -------
    a : complex, ndarray, shape (n, n)
        `a` is overwritten by the upper Hessenberg matrix H.

    b : complex, ndarray, shape (n, n)
        `b` is overwritten by the upper triangular matrix T.

    q : complex, ndarray, shape (:, :)
        If `compq` = 'I', `q` contains the unitary matrix Q.

        Iif `compq` = 'V', `q` is overwritten by Q_1 Q.

    z : complex, ndarray, shape (:, :)
        If `compz` = 'I', `z` contains the unitary matrix Z.

        If `compz` = 'V', `z` is overwritten by Z_1 Z.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `compq`.

            Constraint: `compq` = 'N', 'I' or 'V'.

        (`errno` -2)
            On entry, error in parameter `compz`.

            Constraint: `compz` = 'N', 'V' or 'I'.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -4)
            On entry, error in parameter `ilo`.

            Constraint: `ilo` >= 1.

        (`errno` -5)
            On entry, error in parameter `ihi`.

            Constraint: 1 <= `ilo` <= `ihi` <= n.

        (`errno` -5)
            On entry, error in parameter `ihi`.

            Constraint: `ilo` = 1 and `ihi` = 0.

    Notes
    -----
    ``zgghd3`` is usually the third step in the solution of the complex
    generalized eigenvalue problem

        Ax = lambda Bx.

    The (optional) first step balances the two matrices using
    ``zggbal``.
    In the second step, matrix B is reduced to upper triangular form
    using the QR factorization function ``zgeqrf`` and this unitary
    transformation Q is applied to matrix A by calling ``zunmqr``.
    The driver, ``zggev3``, solves the complex generalized eigenvalue
    problem by combining all the required steps including those just
    listed.

    ``zgghd3`` reduces a pair of complex matrices (A,B), where B is
    triangular, to the generalized upper Hessenberg form using unitary
    transformations.
    This two-sided transformation is of the form

        Q^HAZ = H,;Q^HBZ = T

    where H is an upper Hessenberg matrix, T is an upper triangular
    matrix and Q and Z are unitary matrices determined as products of
    Givens rotations.
    They may either be formed explicitly, or they may be postmultiplied
    into input matrices Q_1 and Z_1, so that

        Q_1 AZ_1^H = (Q_1 Q)H(Z_1 Z)^H,;Q_1 BZ_1^H = (Q_1 Q)T(Z_1 Z)^H.

    References
    ----------
    Golub, G H and Van Loan, C F, 2012, `Matrix Computations`, (4th
    Edition), Johns Hopkins University Press, Baltimore

    Moler, C B and Stewart, G W, 1973, `An algorithm for generalized
    matrix eigenproblems`, SIAM J. Numer. Anal. (10), 241--256
    """

    fun_name = 'naginterfaces.library.lapackeig.zgghd3'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
    )
    _q = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'q'},
        q,
        copy=True,
    )
    _z = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'z'},
        z,
        copy=True,
    )
    _b_lapackeig.zgghd3(
        compq,
        compz,
        _n,
        ilo,
        ihi,
        _a,
        _b,
        _q,
        _z,
    )
    _a_py = _a
    _b_py = _b
    _q_py = _q
    _z_py = _z
    _return_data = Zgghd3ReturnData(
        _a_py,
        _b_py,
        _q_py,
        _z_py,
    )
    return _return_data

ZggbalReturnData = _collections.namedtuple(
    'ZggbalReturnData',
    ['a', 'b', 'ilo', 'ihi', 'lscale', 'rscale'],
)

def zggbal(job, n, a, b):
    u"""
    Balances a pair of complex, square, matrices.

    ``zggbal`` balances a pair of complex square matrices (A,B) of order
    n.
    Balancing usually improves the accuracy of computed generalized
    eigenvalues and eigenvectors.

    For full information please refer to the NAG Library document for
    f08wv

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08wvf.html

    Parameters
    ----------
    job : str, length 1
        Specifies the operations to be performed on matrices A and B.

        `job` = 'N'

            No balancing is done. Initialize `ilo` = 1, `ihi` = `n`,
            `lscale`[i-1] = 1.0 and `rscale`[i-1] = 1.0, for i =
            1,2,...,n.

        `job` = 'P'

            Only permutations are used in balancing.

        `job` = 'S'

            Only scalings are are used in balancing.

        `job` = 'B'

            Both permutations and scalings are used in balancing.

    n : int
        n, the order of the matrices A and B.

    a : complex, array-like, shape (`n`, `n`)
        The n*n matrix A.

    b : complex, array-like, shape (`n`, `n`)
        The n*n matrix B.

    Returns
    -------
    a : complex, ndarray, shape (`n`, `n`)
        `a` is overwritten by the balanced matrix. If `job` = 'N', `a`
        is not referenced.

    b : complex, ndarray, shape (`n`, `n`)
        `b` is overwritten by the balanced matrix. If `job` = 'N', `b`
        is not referenced.

    ilo : int
        i_lo and i_hi are set such that `a`[i-1,j-1] = 0 and
        `b`[i-1,j-1] = 0 if i > j and 1 <= j < i_lo or i_hi < i <= n.

        If `job` = 'N' or 'S', i_lo = 1 and i_hi = n.

    ihi : int
        i_lo and i_hi are set such that `a`[i-1,j-1] = 0 and
        `b`[i-1,j-1] = 0 if i > j and 1 <= j < i_lo or i_hi < i <= n.

        If `job` = 'N' or 'S', i_lo = 1 and i_hi = n.

    lscale : float, ndarray, shape (`n`)
        Details of the permutations and scaling factors applied to the
        left side of the matrices A and B. If P_i is the index of the
        row interchanged with row i and d_i is the scaling factor
        applied to row i, then

            `lscale`[i-1] = P_i, for i = 1,2,...,i_lo-1;

            `lscale`[i-1] = d_i, for i = i_lo,...,i_hi;

            `lscale`[i-1] = P_i, for i = i_hi+1,...,n.

        The order in which the interchanges are made is n to i_hi+1,
        then 1 to i_lo-1.

    rscale : float, ndarray, shape (`n`)
        Details of the permutations and scaling factors applied to the
        right side of the matrices A and B.

        If P_j is the index of the column interchanged with column j and
        d^_j is the scaling factor applied to column j, then

            `rscale`[j-1] = P_j, for j = 1,2,...,i_lo-1;

            `rscale`[j-1] = d^_j, for j = i_lo,...,i_hi;

            `rscale`[j-1] = P_j, for j = i_hi+1,...,n.

        The order in which the interchanges are made is n to i_hi+1,
        then 1 to i_lo-1.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'N', 'P', 'S' or 'B'.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

    Notes
    -----
    Balancing may reduce the 1-norm of the matrices and improve the
    accuracy of the computed eigenvalues and eigenvectors in the complex
    generalized eigenvalue problem

        Ax = lambda Bx.

    ``zggbal`` is usually the first step in the solution of the above
    generalized eigenvalue problem.
    Balancing is optional but it is highly recommended.

    The term 'balancing' covers two steps, each of which involves
    similarity transformations on A and B.
    The function can perform either or both of these steps.
    Both steps are optional.

    (1) The function first attempts to permute A and B to block upper
        triangular form by a similarity transformation:

            PAP^T = F = [table omitted]

            PBP^T = G = [table omitted]

        where P is a permutation matrix, F_11, F_33, G_11 and G_33 are
        upper triangular. Then the diagonal elements of the matrix pairs
        (F_11,G_11) and (F_33,G_33) are generalized eigenvalues of
        (A,B). The rest of the generalized eigenvalues are given by the
        matrix pair (F_22,G_22) which are in rows and columns i_lo to
        i_hi. Subsequent operations to compute the generalized
        eigenvalues of (A,B) need only be applied to the matrix pair
        (F_22,G_22); this can save a significant amount of work if i_lo
        > 1 and i_hi < n. If no suitable permutation exists (as is often
        the case), the function sets i_lo = 1 and i_hi = n.

    (#) The function applies a diagonal similarity transformation to
        (F,G), to make the rows and columns of (F_22,G_22) as close in
        norm as possible:

            DFD^= [table omitted][table omitted][table omitted]

            DGD^-1 = [table omitted][table omitted][table omitted]

        This transformation usually improves the accuracy of computed
        generalized eigenvalues and eigenvectors.

    References
    ----------
    Ward, R C, 1981, `Balancing the generalized eigenvalue problem`,
    SIAM J. Sci. Stat. Comp. (2), 141--152
    """

    fun_name = 'naginterfaces.library.lapackeig.zggbal'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _lscale = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _rscale = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _return_scalars = _b_lapackeig.zggbal(
        job,
        _n,
        _a,
        _b,
        _lscale,
        _rscale,
    )
    _a_py = _a
    _b_py = _b
    _ilo_py = _return_scalars.ilo
    _ihi_py = _return_scalars.ihi
    _lscale_py = _lscale
    _rscale_py = _rscale
    _return_data = ZggbalReturnData(
        _a_py,
        _b_py,
        _ilo_py,
        _ihi_py,
        _lscale_py,
        _rscale_py,
    )
    return _return_data

def zggbak(job, side, ilo, ihi, lscale, rscale, v):
    u"""
    Transforms eigenvectors of a pair of complex balanced matrices to
    those of original matrix pair supplied to ``zggbal``.

    ``zggbak`` forms the right or left eigenvectors of the real
    generalized eigenvalue problem Ax = lambda Bx, by backward
    transformation on the computed eigenvectors given by ``ztgevc``.
    It is necessary to call this function only if the optional balancing
    function ``zggbal`` was previously called to balance the matrix pair
    (A,B).

    For full information please refer to the NAG Library document for
    f08ww

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08wwf.html

    Parameters
    ----------
    job : str, length 1
        Specifies the backtransformation step required.

        `job` = 'N'

            No transformations are done.

        `job` = 'P'

            Only do backward transformations based on permutations.

        `job` = 'S'

            Only do backward transformations based on scaling.

        `job` = 'B'

            Do backward transformations for both permutations and
            scaling.

        **Note:** this must be identical to the argument `job` as
        supplied to ``zggbal``.

    side : str, length 1
        Indicates whether left or right eigenvectors are to be
        transformed.

        `side` = 'L'

            The left eigenvectors are transformed.

        `side` = 'R'

            The right eigenvectors are transformed.

    ilo : int
        i_lo and i_hi as determined by a previous call to ``zggbal``.

    ihi : int
        i_lo and i_hi as determined by a previous call to ``zggbal``.

    lscale : float, array-like, shape (n)
        Details of the permutations and scaling factors applied to the
        left side of the matrices A and B, as returned by a previous
        call to ``zggbal``.

    rscale : float, array-like, shape (n)
        Details of the permutations and scaling factors applied to the
        right side of the matrices A and B, as returned by a previous
        call to ``zggbal``.

    v : complex, array-like, shape (n, m)
        The matrix of right or left eigenvectors, as returned by
        ``zggbal``.

    Returns
    -------
    v : complex, ndarray, shape (n, m)
        The transformed right or left eigenvectors.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'N', 'P', 'S' or 'B'.

        (`errno` -2)
            On entry, error in parameter `side`.

            Constraint: `side` = 'L' or 'R'.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -4)
            On entry, error in parameter `ilo`.

        (`errno` -5)
            On entry, error in parameter `ihi`.

        (`errno` -8)
            On entry, error in parameter m.

            Constraint: 0 <= m <= n.

    Notes
    -----
    If the matrix pair has been previously balanced using the function
    ``zggbal`` then ``zggbak`` backtransforms the eigenvector solution
    given by ``ztgevc``.
    This is usually the sixth and last step in the solution of the
    generalized eigenvalue problem.

    For a description of balancing, see the document for ``zggbal``.

    References
    ----------
    Ward, R C, 1981, `Balancing the generalized eigenvalue problem`,
    SIAM J. Sci. Stat. Comp. (2), 141--152
    """

    fun_name = 'naginterfaces.library.lapackeig.zggbak'

    _v = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'v'},
        v,
        copy=True,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _lscale = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'lscale'},
        lscale,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _rscale = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'rscale'},
        rscale,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_v, {'fun_name': fun_name, 'entity': 'v'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _b_lapackeig.zggbak(
        job,
        side,
        _n,
        ilo,
        ihi,
        _lscale,
        _rscale,
        _m,
        _v,
    )
    _v_py = _v
    return _v_py

DggesReturnData = _collections.namedtuple(
    'DggesReturnData',
    ['a', 'b', 'sdim', 'alphar', 'alphai', 'beta', 'vsl', 'vsr'],
)

@_b_utils._withdrawal(
    'naginterfaces.library.lapackeig.dgges',
    mkwdrn=None,
    advice=[
        'Please use dgges3 instead.',
        'See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html',
    ]
)
def dgges(jobvsl, jobvsr, sort, a, b, selctg=None, data=None):
    u"""
    Computes, for a real nonsymmetric matrix pair, the generalized
    eigenvalues, the generalized real Schur form and, optionally, the
    left and/or right matrices of Schur vectors.

    ``dgges`` computes the generalized eigenvalues, the generalized real
    Schur form (S,T) and, optionally, the left and/or right generalized
    Schur vectors for a pair of n*n real nonsymmetric matrices (A,B).
    ``dgges`` is marked as `deprecated` by LAPACK; the replacement
    routine is ``dgges3`` which makes better use of Level 3 BLAS.

    Note: ``dgges`` is deprecated.
    Please use ``dgges3`` instead.
    See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html

    For full information please refer to the NAG Library document for
    f08xa

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08xaf.html

    Parameters
    ----------
    jobvsl : str, length 1
        If `jobvsl` = 'N', do not compute the left Schur vectors.

        If `jobvsl` = 'V', compute the left Schur vectors.

    jobvsr : str, length 1
        If `jobvsr` = 'N', do not compute the right Schur vectors.

        If `jobvsr` = 'V', compute the right Schur vectors.

    sort : str, length 1
        Specifies whether or not to order the eigenvalues on the
        diagonal of the generalized Schur form.

        `sort` = 'N'

            Eigenvalues are not ordered.

        `sort` = 'S'

            Eigenvalues are ordered (see `selctg`).

    a : float, array-like, shape (n, n)
        The first of the pair of matrices, A.

    b : float, array-like, shape (n, n)
        The second of the pair of matrices, B.

    selctg : None or callable retval = selctg(ar, ai, b, data=None),
        optional
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        If `sort` = 'S', `selctg` is used to select generalized
        eigenvalues to be moved to the top left of the generalized Schur
        form.

        Parameters
        ~~~~~~~~~~
        ar : float
            An eigenvalue (`ar`[j-1]+sqrt(-1)*`ai`[j-1])/`b`[j-1] is
            selected if `selctg`(`ar`[j-1],`ai`[j-1],`b`[j-1]) = True.
            If either one of a complex conjugate pair is selected, then
            both complex generalized eigenvalues are selected.

            Note that in the ill-conditioned case, a selected complex
            generalized eigenvalue may no longer satisfy
            `selctg`(`ar`[j-1],`ai`[j-1],`b`[j-1]) = True after
            ordering. `errno` = `n` + 2 in this case.

        ai : float
            An eigenvalue (`ar`[j-1]+sqrt(-1)*`ai`[j-1])/`b`[j-1] is
            selected if `selctg`(`ar`[j-1],`ai`[j-1],`b`[j-1]) = True.
            If either one of a complex conjugate pair is selected, then
            both complex generalized eigenvalues are selected.

            Note that in the ill-conditioned case, a selected complex
            generalized eigenvalue may no longer satisfy
            `selctg`(`ar`[j-1],`ai`[j-1],`b`[j-1]) = True after
            ordering. `errno` = `n` + 2 in this case.

        b : float
            An eigenvalue (`ar`[j-1]+sqrt(-1)*`ai`[j-1])/`b`[j-1] is
            selected if `selctg`(`ar`[j-1],`ai`[j-1],`b`[j-1]) = True.
            If either one of a complex conjugate pair is selected, then
            both complex generalized eigenvalues are selected.

            Note that in the ill-conditioned case, a selected complex
            generalized eigenvalue may no longer satisfy
            `selctg`(`ar`[j-1],`ai`[j-1],`b`[j-1]) = True after
            ordering. `errno` = `n` + 2 in this case.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : bool
            Must be True if the eigenvalue is to be selected.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    a : float, ndarray, shape (n, n)
        `a` has been overwritten by its generalized Schur form S.

    b : float, ndarray, shape (n, n)
        `b` has been overwritten by its generalized Schur form T.

    sdim : int
        If `sort` = 'N', `sdim` = 0.

        If `sort` = 'S', `sdim` = number of eigenvalues (after sorting)
        for which `selctg` is True. (Complex conjugate pairs for which
        `selctg` is True for either eigenvalue count as 2.)

    alphar : float, ndarray, shape (n)
        See the description of `beta`.

    alphai : float, ndarray, shape (n)
        See the description of `beta`.

    beta : float, ndarray, shape (n)
        (`alphar`[j-1]+`alphai`[j-1]*i)/`beta`[j-1], for j = 1,2,...,n,
        will be the generalized eigenvalues.
        `alphar`[j-1]+`alphai`[j-1]*i, and `beta`[j-1], for j =
        1,2,...,n, are the diagonals of the complex Schur form (S,T)
        that would result if the 2*2 diagonal blocks of the real Schur
        form of (A,B) were further reduced to triangular form using 2*2
        complex unitary transformations.

        If `alphai`[j-1] is zero, then the jth eigenvalue is real; if
        positive, then the jth and (j+1)st eigenvalues are a complex
        conjugate pair, with `alphai`[j] negative.

        **Note:** the quotients `alphar`[j-1]/`beta`[j-1] and
        `alphai`[j-1]/`beta`[j-1] may easily overflow or underflow, and
        `beta`[j-1] may even be zero.

        Thus, you should avoid naively computing the ratio alpha/beta.

        However, `alphar` and `alphai` will always be less than and
        usually comparable with ||A||_2 in magnitude, and `beta` will
        always be less than and usually comparable with ||B||_2.

    vsl : float, ndarray, shape (:, :)
        If `jobvsl` = 'V', `vsl` will contain the left Schur vectors, Q.

        If `jobvsl` = 'N', `vsl` is not referenced.

    vsr : float, ndarray, shape (:, :)
        If `jobvsr` = 'V', `vsr` will contain the right Schur vectors,
        Z.

        If `jobvsr` = 'N', `vsr` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobvsl`.

            Constraint: `jobvsl` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `jobvsr`.

            Constraint: `jobvsr` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `sort`.

            Constraint: `sort` = 'N' or 'S'.

        (`errno` -5)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` n+1)
            The QZ iteration failed with an unexpected error, please
            contact NAG.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i<=n))
            The QZ iteration failed. No eigenvectors have been
            calculated but `alphar`[j], `alphai`[j] and `beta`[j] should
            be correct from element *<value>*.

        (`errno` n+2)
            After reordering, roundoff changed values of some complex
            eigenvalues so that leading eigenvalues in the generalized
            Schur form no longer satisfy `selctg` = True. This could
            also be caused by underflow due to scaling.

        (`errno` n+3)
            The eigenvalues could not be reordered because some
            eigenvalues were too close to separate (the problem is very
            ill-conditioned).

    Notes
    -----
    The generalized Schur factorization for a pair of real matrices
    (A,B) is given by

        A = QSZ^T, B = QTZ^T,

    where Q and Z are orthogonal, T is upper triangular and S is upper
    quasi-triangular with 1*1 and 2*2 diagonal blocks.
    The generalized eigenvalues, lambda, of (A,B) are computed from the
    diagonals of S and T and satisfy

        Az = lambda Bz,

    where z is the corresponding generalized eigenvector. lambda is
    actually returned as the pair (alpha,beta) such that

        lambda = alpha/beta

    since beta, or even both alpha and beta can be zero.
    The columns of Q and Z are the left and right generalized Schur
    vectors of (A,B).

    Optionally, ``dgges`` can order the generalized eigenvalues on the
    diagonals of (S,T) so that selected eigenvalues are at the top left.
    The leading columns of Q and Z then form an orthonormal basis for
    the corresponding eigenspaces, the deflating subspaces.

    ``dgges`` computes T to have non-negative diagonal elements, and the
    2*2 blocks of S correspond to complex conjugate pairs of generalized
    eigenvalues.
    The generalized Schur factorization, before reordering, is computed
    by the QZ algorithm.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    def cb_selctg(ar, ai, b, data=None):
        """Wrapping user's selctg. Called as base selctg."""
        if data is not None:
            _retval_selctg = selctg(
                ar,
                ai,
                b,
                data,
            )
        else:
            _retval_selctg = selctg(
                ar,
                ai,
                b,
            )
        return _retval_selctg

    fun_name = 'naginterfaces.library.lapackeig.dgges'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'), (_b, 'b')),
    )
    _b_utils._check_callable(selctg, {'fun_name': fun_name, 'entity': 'selctg'}, allow_none=True)
    _selctg = None if selctg is None else cb_selctg
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _alphar = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _alphai = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _beta = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobvsl = _b_utils._EngineCharScalarType(
        jobvsl, {'fun_name': fun_name, 'entity': 'jobvsl'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvsl_py = _b_utils._EngineCharScalarType.to_py(_jobvsl)
    if _jobvsl_py in ('V', 'v'):
        _vsl_dim1 = _n_py
    else:
        _vsl_dim1 = 1
    if _jobvsl_py in ('V', 'v'):
        _vsl_dim2 = _n_py
    else:
        _vsl_dim2 = 1
    exp_shape_vsl = (_vsl_dim1, _vsl_dim2)
    _vsl = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_vsl,
        sorder=_m_sorder,
    )
    _jobvsr = _b_utils._EngineCharScalarType(
        jobvsr, {'fun_name': fun_name, 'entity': 'jobvsr'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvsr_py = _b_utils._EngineCharScalarType.to_py(_jobvsr)
    if _jobvsr_py in ('V', 'v'):
        _vsr_dim1 = _n_py
    else:
        _vsr_dim1 = 1
    if _jobvsr_py in ('V', 'v'):
        _vsr_dim2 = _n_py
    else:
        _vsr_dim2 = 1
    exp_shape_vsr = (_vsr_dim1, _vsr_dim2)
    _vsr = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_vsr,
        sorder=_m_sorder,
    )
    try:
        _return_scalar = _b_lapackeig.dgges(
            jobvsl,
            jobvsr,
            sort,
            _selctg,
            _n,
            _a,
            _b,
            _alphar,
            _alphai,
            _beta,
            _vsl,
            _vsr,
            data,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _a_py = _a
    _b_py = _b
    _sdim_py = _return_scalar
    _alphar_py = _alphar
    _alphai_py = _alphai
    _beta_py = _beta
    _vsl_py = _vsl
    _vsr_py = _vsr
    _return_data = DggesReturnData(
        _a_py,
        _b_py,
        _sdim_py,
        _alphar_py,
        _alphai_py,
        _beta_py,
        _vsl_py,
        _vsr_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

DggesxReturnData = _collections.namedtuple(
    'DggesxReturnData',
    ['a', 'b', 'sdim', 'alphar', 'alphai', 'beta', 'vsl', 'vsr', 'rconde', 'rcondv'],
)

def dggesx(jobvsl, jobvsr, sort, sense, a, b, selctg=None, data=None):
    u"""
    Computes, for a real nonsymmetric matrix pair, the generalized
    eigenvalues, the generalized real Schur form and, optionally, the
    left and/or right matrices of Schur vectors; also, optionally,
    computes reciprocal condition numbers for selected eigenvalues.

    ``dggesx`` computes the generalized eigenvalues, the generalized
    real Schur form (S,T) and, optionally, the left and/or right
    generalized Schur vectors for a pair of n*n real nonsymmetric
    matrices (A,B).

    Estimates of condition numbers for selected generalized eigenvalue
    clusters and Schur vectors are also computed.

    For full information please refer to the NAG Library document for
    f08xb

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08xbf.html

    Parameters
    ----------
    jobvsl : str, length 1
        If `jobvsl` = 'N', do not compute the left Schur vectors.

        If `jobvsl` = 'V', compute the left Schur vectors.

    jobvsr : str, length 1
        If `jobvsr` = 'N', do not compute the right Schur vectors.

        If `jobvsr` = 'V', compute the right Schur vectors.

    sort : str, length 1
        Specifies whether or not to order the eigenvalues on the
        diagonal of the generalized Schur form.

        `sort` = 'N'

            Eigenvalues are not ordered.

        `sort` = 'S'

            Eigenvalues are ordered (see `selctg`).

    sense : str, length 1
        Determines which reciprocal condition numbers are computed.

        `sense` = 'N'

            None are computed.

        `sense` = 'E'

            Computed for average of selected eigenvalues only.

        `sense` = 'V'

            Computed for selected deflating subspaces only.

        `sense` = 'B'

            Computed for both.

        If `sense` = 'E', 'V' or 'B', `sort` = 'S'.

    a : float, array-like, shape (n, n)
        The first of the pair of matrices, A.

    b : float, array-like, shape (n, n)
        The second of the pair of matrices, B.

    selctg : None or callable retval = selctg(ar, ai, b, data=None),
        optional
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        If `sort` = 'S', `selctg` is used to select generalized
        eigenvalues to be moved to the top left of the generalized Schur
        form.

        Parameters
        ~~~~~~~~~~
        ar : float
            An eigenvalue (`ar`[j-1]+sqrt(-1)*`ai`[j-1])/`b`[j-1] is
            selected if `selctg`(`ar`[j-1],`ai`[j-1],`b`[j-1]) is True.
            If either one of a complex conjugate pair is selected, then
            both complex generalized eigenvalues are selected.

            Note that in the ill-conditioned case, a selected complex
            generalized eigenvalue may no longer satisfy
            `selctg`(`ar`[j-1],`ai`[j-1],`b`[j-1]) = True after
            ordering. `errno` = `n` + 2 in this case.

        ai : float
            An eigenvalue (`ar`[j-1]+sqrt(-1)*`ai`[j-1])/`b`[j-1] is
            selected if `selctg`(`ar`[j-1],`ai`[j-1],`b`[j-1]) is True.
            If either one of a complex conjugate pair is selected, then
            both complex generalized eigenvalues are selected.

            Note that in the ill-conditioned case, a selected complex
            generalized eigenvalue may no longer satisfy
            `selctg`(`ar`[j-1],`ai`[j-1],`b`[j-1]) = True after
            ordering. `errno` = `n` + 2 in this case.

        b : float
            An eigenvalue (`ar`[j-1]+sqrt(-1)*`ai`[j-1])/`b`[j-1] is
            selected if `selctg`(`ar`[j-1],`ai`[j-1],`b`[j-1]) is True.
            If either one of a complex conjugate pair is selected, then
            both complex generalized eigenvalues are selected.

            Note that in the ill-conditioned case, a selected complex
            generalized eigenvalue may no longer satisfy
            `selctg`(`ar`[j-1],`ai`[j-1],`b`[j-1]) = True after
            ordering. `errno` = `n` + 2 in this case.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : bool
            Must be True if the eigenvalue is to be selected.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    a : float, ndarray, shape (n, n)
        `a` has been overwritten by its generalized Schur form S.

    b : float, ndarray, shape (n, n)
        `b` has been overwritten by its generalized Schur form T.

    sdim : int
        If `sort` = 'N', `sdim` = 0.

        If `sort` = 'S', `sdim` = number of eigenvalues (after sorting)
        for which `selctg` is True. (Complex conjugate pairs for which
        `selctg` is True for either eigenvalue count as 2.)

    alphar : float, ndarray, shape (n)
        See the description of `beta`.

    alphai : float, ndarray, shape (n)
        See the description of `beta`.

    beta : float, ndarray, shape (n)
        (`alphar`[j-1]+`alphai`[j-1]*i)/`beta`[j-1], for j = 1,2,...,n,
        will be the generalized eigenvalues.
        `alphar`[j-1]+`alphai`[j-1]*i, and `beta`[j-1], for j =
        1,2,...,n, are the diagonals of the complex Schur form (S,T)
        that would result if the 2*2 diagonal blocks of the real Schur
        form of (A,B) were further reduced to triangular form using 2*2
        complex unitary transformations.

        If `alphai`[j-1] is zero, then the jth eigenvalue is real; if
        positive, then the jth and (j+1)st eigenvalues are a complex
        conjugate pair, with `alphai`[j] negative.

        **Note:** the quotients `alphar`[j-1]/`beta`[j-1] and
        `alphai`[j-1]/`beta`[j-1] may easily overflow or underflow, and
        `beta`[j-1] may even be zero.

        Thus, you should avoid naively computing the ratio alpha/beta.

        However, `alphar` and `alphai` will always be less than and
        usually comparable with ||A||_2 in magnitude, and `beta` will
        always be less than and usually comparable with ||B||_2.

    vsl : float, ndarray, shape (:, :)
        If `jobvsl` = 'V', `vsl` will contain the left Schur vectors, Q.

        If `jobvsl` = 'N', `vsl` is not referenced.

    vsr : float, ndarray, shape (:, :)
        If `jobvsr` = 'V', `vsr` will contain the right Schur vectors,
        Z.

        If `jobvsr` = 'N', `vsr` is not referenced.

    rconde : float, ndarray, shape (2)
        If `sense` = 'E' or 'B', `rconde`[0] and `rconde`[1] contain the
        reciprocal condition numbers for the average of the selected
        eigenvalues.

        If `sense` = 'N' or 'V', `rconde` is not referenced.

    rcondv : float, ndarray, shape (2)
        If `sense` = 'V' or 'B', `rcondv`[0] and `rcondv`[1] contain the
        reciprocal condition numbers for the selected deflating
        subspaces.

        if `sense` = 'N' or 'E', `rcondv` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobvsl`.

            Constraint: `jobvsl` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `jobvsr`.

            Constraint: `jobvsr` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `sort`.

            Constraint: `sort` = 'N' or 'S'.

        (`errno` -5)
            On entry, error in parameter `sense`.

            Constraint: `sense` = 'N', 'E', 'V' or 'B'.

        (`errno` -6)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` n+1)
            The QZ iteration failed with an unexpected error, please
            contact NAG.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i<=n))
            The QZ iteration failed. No eigenvectors have been
            calculated but `alphar`[j], `alphai`[j] and `beta`[j] should
            be correct from element *<value>*.

        (`errno` n+2)
            After reordering, roundoff changed values of some complex
            eigenvalues so that leading eigenvalues in the generalized
            Schur form no longer satisfy `selctg` = True. This could
            also be caused by underflow due to scaling.

        (`errno` n+3)
            The eigenvalues could not be reordered because some
            eigenvalues were too close to separate (the problem is very
            ill-conditioned).

    Notes
    -----
    The generalized real Schur factorization of (A,B) is given by

        A = QSZ^T, B = QTZ^T,

    where Q and Z are orthogonal, T is upper triangular and S is upper
    quasi-triangular with 1*1 and 2*2 diagonal blocks.
    The generalized eigenvalues, lambda, of (A,B) are computed from the
    diagonals of T and S and satisfy

        Az = lambda Bz,

    where z is the corresponding generalized eigenvector. lambda is
    actually returned as the pair (alpha,beta) such that

        lambda = alpha/beta

    since beta, or even both alpha and beta can be zero.
    The columns of Q and Z are the left and right generalized Schur
    vectors of (A,B).

    Optionally, ``dggesx`` can order the generalized eigenvalues on the
    diagonals of (S,T) so that selected eigenvalues are at the top left.
    The leading columns of Q and Z then form an orthonormal basis for
    the corresponding eigenspaces, the deflating subspaces.

    ``dggesx`` computes T to have non-negative diagonal elements, and
    the 2*2 blocks of S correspond to complex conjugate pairs of
    generalized eigenvalues.
    The generalized Schur factorization, before reordering, is computed
    by the QZ algorithm.

    The reciprocals of the condition estimates, the reciprocal values of
    the left and right projection norms, are returned in `rconde`[0] and
    `rconde`[1] respectively, for the selected generalized eigenvalues,
    together with reciprocal condition estimates for the corresponding
    left and right deflating subspaces, in `rcondv`[0] and `rcondv`[1].
    See Section 4.11 of Anderson `et al.` (1999) for further
    information.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    def cb_selctg(ar, ai, b, data=None):
        """Wrapping user's selctg. Called as base selctg."""
        if data is not None:
            _retval_selctg = selctg(
                ar,
                ai,
                b,
                data,
            )
        else:
            _retval_selctg = selctg(
                ar,
                ai,
                b,
            )
        return _retval_selctg

    fun_name = 'naginterfaces.library.lapackeig.dggesx'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'), (_b, 'b')),
    )
    _b_utils._check_callable(selctg, {'fun_name': fun_name, 'entity': 'selctg'}, allow_none=True)
    _selctg = None if selctg is None else cb_selctg
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _alphar = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _alphai = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _beta = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobvsl = _b_utils._EngineCharScalarType(
        jobvsl, {'fun_name': fun_name, 'entity': 'jobvsl'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvsl_py = _b_utils._EngineCharScalarType.to_py(_jobvsl)
    if _jobvsl_py in ('V', 'v'):
        _vsl_dim1 = _n_py
    else:
        _vsl_dim1 = 1
    if _jobvsl_py in ('V', 'v'):
        _vsl_dim2 = _n_py
    else:
        _vsl_dim2 = 0
    exp_shape_vsl = (_vsl_dim1, _vsl_dim2)
    _vsl = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_vsl,
        sorder=_m_sorder,
    )
    _jobvsr = _b_utils._EngineCharScalarType(
        jobvsr, {'fun_name': fun_name, 'entity': 'jobvsr'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvsr_py = _b_utils._EngineCharScalarType.to_py(_jobvsr)
    if _jobvsr_py in ('V', 'v'):
        _vsr_dim1 = _n_py
    else:
        _vsr_dim1 = 1
    if _jobvsr_py in ('V', 'v'):
        _vsr_dim2 = _n_py
    else:
        _vsr_dim2 = 0
    exp_shape_vsr = (_vsr_dim1, _vsr_dim2)
    _vsr = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_vsr,
        sorder=_m_sorder,
    )
    _rconde = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (2,),
    )
    _rcondv = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (2,),
    )
    try:
        _return_scalar = _b_lapackeig.dggesx(
            jobvsl,
            jobvsr,
            sort,
            _selctg,
            sense,
            _n,
            _a,
            _b,
            _alphar,
            _alphai,
            _beta,
            _vsl,
            _vsr,
            _rconde,
            _rcondv,
            data,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _a_py = _a
    _b_py = _b
    _sdim_py = _return_scalar
    _alphar_py = _alphar
    _alphai_py = _alphai
    _beta_py = _beta
    _vsl_py = _vsl
    _vsr_py = _vsr
    _rconde_py = _rconde
    _rcondv_py = _rcondv
    _return_data = DggesxReturnData(
        _a_py,
        _b_py,
        _sdim_py,
        _alphar_py,
        _alphai_py,
        _beta_py,
        _vsl_py,
        _vsr_py,
        _rconde_py,
        _rcondv_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

Dgges3ReturnData = _collections.namedtuple(
    'Dgges3ReturnData',
    ['a', 'b', 'sdim', 'alphar', 'alphai', 'beta', 'vsl', 'vsr'],
)

def dgges3(jobvsl, jobvsr, sort, a, b, selctg=None, data=None):
    u"""
    Computes, for a real nonsymmetric matrix pair, using BLAS-3, the
    generalized eigenvalues, the generalized real Schur form and,
    optionally, the left and/or right matrices of Schur vectors.

    ``dgges3`` computes the generalized eigenvalues, the generalized
    real Schur form (S,T) and, optionally, the left and/or right
    generalized Schur vectors for a pair of n*n real nonsymmetric
    matrices (A,B).

    For full information please refer to the NAG Library document for
    f08xc

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08xcf.html

    Parameters
    ----------
    jobvsl : str, length 1
        If `jobvsl` = 'N', do not compute the left Schur vectors.

        If `jobvsl` = 'V', compute the left Schur vectors.

    jobvsr : str, length 1
        If `jobvsr` = 'N', do not compute the right Schur vectors.

        If `jobvsr` = 'V', compute the right Schur vectors.

    sort : str, length 1
        Specifies whether or not to order the eigenvalues on the
        diagonal of the generalized Schur form.

        `sort` = 'N'

            Eigenvalues are not ordered.

        `sort` = 'S'

            Eigenvalues are ordered (see `selctg`).

    a : float, array-like, shape (n, n)
        The first of the pair of matrices, A.

    b : float, array-like, shape (n, n)
        The second of the pair of matrices, B.

    selctg : None or callable retval = selctg(ar, ai, b, data=None),
        optional
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        If `sort` = 'S', `selctg` is used to select generalized
        eigenvalues to be moved to the top left of the generalized Schur
        form.

        Parameters
        ~~~~~~~~~~
        ar : float
            An eigenvalue (`ar`[j-1]+sqrt(-1)*`ai`[j-1])/`b`[j-1] is
            selected if `selctg`(`ar`[j-1],`ai`[j-1],`b`[j-1]) = True.
            If either one of a complex conjugate pair is selected, then
            both complex generalized eigenvalues are selected.

            Note that in the ill-conditioned case, a selected complex
            generalized eigenvalue may no longer satisfy
            `selctg`(`ar`[j-1],`ai`[j-1],`b`[j-1]) = True after
            ordering. `errno` = `n` + 2 in this case.

        ai : float
            An eigenvalue (`ar`[j-1]+sqrt(-1)*`ai`[j-1])/`b`[j-1] is
            selected if `selctg`(`ar`[j-1],`ai`[j-1],`b`[j-1]) = True.
            If either one of a complex conjugate pair is selected, then
            both complex generalized eigenvalues are selected.

            Note that in the ill-conditioned case, a selected complex
            generalized eigenvalue may no longer satisfy
            `selctg`(`ar`[j-1],`ai`[j-1],`b`[j-1]) = True after
            ordering. `errno` = `n` + 2 in this case.

        b : float
            An eigenvalue (`ar`[j-1]+sqrt(-1)*`ai`[j-1])/`b`[j-1] is
            selected if `selctg`(`ar`[j-1],`ai`[j-1],`b`[j-1]) = True.
            If either one of a complex conjugate pair is selected, then
            both complex generalized eigenvalues are selected.

            Note that in the ill-conditioned case, a selected complex
            generalized eigenvalue may no longer satisfy
            `selctg`(`ar`[j-1],`ai`[j-1],`b`[j-1]) = True after
            ordering. `errno` = `n` + 2 in this case.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : bool
            Must be True if the eigenvalue is to be selected.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    a : float, ndarray, shape (n, n)
        `a` has been overwritten by its generalized Schur form S.

    b : float, ndarray, shape (n, n)
        `b` has been overwritten by its generalized Schur form T.

    sdim : int
        If `sort` = 'N', `sdim` = 0.

        If `sort` = 'S', `sdim` = number of eigenvalues (after sorting)
        for which `selctg` is True. (Complex conjugate pairs for which
        `selctg` is True for either eigenvalue count as 2.)

    alphar : float, ndarray, shape (n)
        See the description of `beta`.

    alphai : float, ndarray, shape (n)
        See the description of `beta`.

    beta : float, ndarray, shape (n)
        (`alphar`[j-1]+`alphai`[j-1]*i)/`beta`[j-1], for j = 1,2,...,n,
        will be the generalized eigenvalues.
        `alphar`[j-1]+`alphai`[j-1]*i, and `beta`[j-1], for j =
        1,2,...,n, are the diagonals of the complex Schur form (S,T)
        that would result if the 2*2 diagonal blocks of the real Schur
        form of (A,B) were further reduced to triangular form using 2*2
        complex unitary transformations.

        If `alphai`[j-1] is zero, then the jth eigenvalue is real; if
        positive, then the jth and (j+1)st eigenvalues are a complex
        conjugate pair, with `alphai`[j] negative.

        **Note:** the quotients `alphar`[j-1]/`beta`[j-1] and
        `alphai`[j-1]/`beta`[j-1] may easily overflow or underflow, and
        `beta`[j-1] may even be zero.

        Thus, you should avoid naively computing the ratio alpha/beta.

        However, `alphar` and `alphai` will always be less than and
        usually comparable with ||A||_2 in magnitude, and `beta` will
        always be less than and usually comparable with ||B||_2.

    vsl : float, ndarray, shape (:, :)
        If `jobvsl` = 'V', `vsl` will contain the left Schur vectors, Q.

        If `jobvsl` = 'N', `vsl` is not referenced.

    vsr : float, ndarray, shape (:, :)
        If `jobvsr` = 'V', `vsr` will contain the right Schur vectors,
        Z.

        If `jobvsr` = 'N', `vsr` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobvsl`.

            Constraint: `jobvsl` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `jobvsr`.

            Constraint: `jobvsr` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `sort`.

            Constraint: `sort` = 'N' or 'S'.

        (`errno` -5)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` n+1)
            The QZ iteration failed with an unexpected error, please
            contact NAG.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i<=n))
            The QZ iteration failed. No eigenvectors have been
            calculated but `alphar`[j], `alphai`[j] and `beta`[j] should
            be correct from element *<value>*.

        (`errno` n+2)
            After reordering, roundoff changed values of some complex
            eigenvalues so that leading eigenvalues in the generalized
            Schur form no longer satisfy `selctg` = True. This could
            also be caused by underflow due to scaling.

        (`errno` n+3)
            The eigenvalues could not be reordered because some
            eigenvalues were too close to separate (the problem is very
            ill-conditioned).

    Notes
    -----
    The generalized Schur factorization for a pair of real matrices
    (A,B) is given by

        A = QSZ^T, B = QTZ^T,

    where Q and Z are orthogonal, T is upper triangular and S is upper
    quasi-triangular with 1*1 and 2*2 diagonal blocks.
    The generalized eigenvalues, lambda, of (A,B) are computed from the
    diagonals of S and T and satisfy

        Az = lambda Bz,

    where z is the corresponding generalized eigenvector. lambda is
    actually returned as the pair (alpha,beta) such that

        lambda = alpha/beta

    since beta, or even both alpha and beta can be zero.
    The columns of Q and Z are the left and right generalized Schur
    vectors of (A,B).

    Optionally, ``dgges3`` can order the generalized eigenvalues on the
    diagonals of (S,T) so that selected eigenvalues are at the top left.
    The leading columns of Q and Z then form an orthonormal basis for
    the corresponding eigenspaces, the deflating subspaces.

    ``dgges3`` computes T to have non-negative diagonal elements, and
    the 2*2 blocks of S correspond to complex conjugate pairs of
    generalized eigenvalues.
    The generalized Schur factorization, before reordering, is computed
    by the QZ algorithm.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 2012, `Matrix Computations`, (4th
    Edition), Johns Hopkins University Press, Baltimore
    """

    def cb_selctg(ar, ai, b, data=None):
        """Wrapping user's selctg. Called as base selctg."""
        if data is not None:
            _retval_selctg = selctg(
                ar,
                ai,
                b,
                data,
            )
        else:
            _retval_selctg = selctg(
                ar,
                ai,
                b,
            )
        return _retval_selctg

    fun_name = 'naginterfaces.library.lapackeig.dgges3'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'), (_b, 'b')),
    )
    _b_utils._check_callable(selctg, {'fun_name': fun_name, 'entity': 'selctg'}, allow_none=True)
    _selctg = None if selctg is None else cb_selctg
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _alphar = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _alphai = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _beta = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobvsl = _b_utils._EngineCharScalarType(
        jobvsl, {'fun_name': fun_name, 'entity': 'jobvsl'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvsl_py = _b_utils._EngineCharScalarType.to_py(_jobvsl)
    if _jobvsl_py in ('V', 'v'):
        _vsl_dim1 = _n_py
    else:
        _vsl_dim1 = 1
    if _jobvsl_py in ('V', 'v'):
        _vsl_dim2 = _n_py
    else:
        _vsl_dim2 = 1
    exp_shape_vsl = (_vsl_dim1, _vsl_dim2)
    _vsl = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_vsl,
        sorder=_m_sorder,
    )
    _jobvsr = _b_utils._EngineCharScalarType(
        jobvsr, {'fun_name': fun_name, 'entity': 'jobvsr'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvsr_py = _b_utils._EngineCharScalarType.to_py(_jobvsr)
    if _jobvsr_py in ('V', 'v'):
        _vsr_dim1 = _n_py
    else:
        _vsr_dim1 = 1
    if _jobvsr_py in ('V', 'v'):
        _vsr_dim2 = _n_py
    else:
        _vsr_dim2 = 1
    exp_shape_vsr = (_vsr_dim1, _vsr_dim2)
    _vsr = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_vsr,
        sorder=_m_sorder,
    )
    try:
        _return_scalar = _b_lapackeig.dgges3(
            jobvsl,
            jobvsr,
            sort,
            _selctg,
            _n,
            _a,
            _b,
            _alphar,
            _alphai,
            _beta,
            _vsl,
            _vsr,
            data,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _a_py = _a
    _b_py = _b
    _sdim_py = _return_scalar
    _alphar_py = _alphar
    _alphai_py = _alphai
    _beta_py = _beta
    _vsl_py = _vsl
    _vsr_py = _vsr
    _return_data = Dgges3ReturnData(
        _a_py,
        _b_py,
        _sdim_py,
        _alphar_py,
        _alphai_py,
        _beta_py,
        _vsl_py,
        _vsr_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

DhgeqzReturnData = _collections.namedtuple(
    'DhgeqzReturnData',
    ['a', 'b', 'alphar', 'alphai', 'beta', 'q', 'z'],
)

def dhgeqz(job, compq, compz, ilo, ihi, a, b, q, z):
    u"""
    Computes eigenvalues and generalized Schur factorization of real
    generalized upper Hessenberg form reduced from a pair of real
    general matrices.

    ``dhgeqz`` implements the QZ method for finding generalized
    eigenvalues of the real matrix pair (A,B) of order n, which is in
    the generalized upper Hessenberg form.

    For full information please refer to the NAG Library document for
    f08xe

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08xef.html

    Parameters
    ----------
    job : str, length 1
        Specifies the operations to be performed on (A,B).

        `job` = 'E'

            The matrix pair (A,B) on exit might not be in the
            generalized Schur form.

        `job` = 'S'

            The matrix pair (A,B) on exit will be in the generalized
            Schur form.

    compq : str, length 1
        Specifies the operations to be performed on Q:

        `compq` = 'N'

            The array `q` is unchanged.

        `compq` = 'V'

            The left transformation Q is accumulated on the array `q`.

        `compq` = 'I'

            The array `q` is initialized to the identity matrix before
            the left transformation Q is accumulated in `q`.

    compz : str, length 1
        Specifies the operations to be performed on Z.

        `compz` = 'N'

            The array `z` is unchanged.

        `compz` = 'V'

            The right transformation Z is accumulated on the array `z`.

        `compz` = 'I'

            The array `z` is initialized to the identity matrix before
            the right transformation Z is accumulated in `z`.

    ilo : int
        The indices i_lo and i_hi, respectively which define the upper
        triangular parts of A. The submatrices A(1:i_lo-1,1:i_lo-1) and
        A(i_hi+1:n,i_hi+1:n) are then upper triangular. These arguments
        are provided by ``dggbal`` if the matrix pair was previously
        balanced; otherwise, `ilo` = 1 and `ihi` = n.

    ihi : int
        The indices i_lo and i_hi, respectively which define the upper
        triangular parts of A. The submatrices A(1:i_lo-1,1:i_lo-1) and
        A(i_hi+1:n,i_hi+1:n) are then upper triangular. These arguments
        are provided by ``dggbal`` if the matrix pair was previously
        balanced; otherwise, `ilo` = 1 and `ihi` = n.

    a : float, array-like, shape (n, n)
        The n*n upper Hessenberg matrix A. The elements below the first
        subdiagonal must be set to zero.

    b : float, array-like, shape (n, n)
        The n*n upper triangular matrix B. The elements below the
        diagonal must be zero.

    q : float, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `compq` in ('V', 'I'): n; if
        `compq`='N': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compq` in ('V', 'I'): n; if
        `compq`='N': 1; otherwise: 0.

        If `compq` = 'V', the matrix Q_0. The matrix Q_0 is usually the
        matrix Q returned by ``dgghd3``.

        If `compq` = 'N', `q` is not referenced.

    z : float, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `compz` in ('V', 'I'): n; if
        `compz`='N': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compz` in ('V', 'I'): n; if
        `compz`='N': 1; otherwise: 0.

        If `compz` = 'V', the matrix Z_0. The matrix Z_0 is usually the
        matrix Z returned by ``dgghd3``.

        If `compz` = 'N', `z` is not referenced.

    Returns
    -------
    a : float, ndarray, shape (n, n)
        If `job` = 'S', the matrix pair (A,B) will be simultaneously
        reduced to generalized Schur form.

        If `job` = 'E', the 1*1 and 2*2 diagonal blocks of the matrix
        pair (A,B) will give generalized eigenvalues but the remaining
        elements will be irrelevant.

    b : float, ndarray, shape (n, n)
        If `job` = 'S', the matrix pair (A,B) will be simultaneously
        reduced to generalized Schur form.

        If `job` = 'E', the 1*1 and 2*2 diagonal blocks of the matrix
        pair (A,B) will give generalized eigenvalues but the remaining
        elements will be irrelevant.

    alphar : float, ndarray, shape (n)
        The real parts of alpha_j, for j = 1,2,...,n.

    alphai : float, ndarray, shape (n)
        The imaginary parts of alpha_j, for j = 1,2,...,n.

    beta : float, ndarray, shape (n)
        beta_j, for j = 1,2,...,n.

    q : float, ndarray, shape (:, :)
        If `compq` = 'V', `q` contains the matrix product QQ_0.

        If `compq` = 'I', `q` contains the transformation matrix Q.

    z : float, ndarray, shape (:, :)
        If `compz` = 'V', `z` contains the matrix product ZZ_0.

        If `compz` = 'I', `z` contains the transformation matrix Z.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'E' or 'S'.

        (`errno` -2)
            On entry, error in parameter `compq`.

            Constraint: `compq` = 'N', 'V' or 'I'.

        (`errno` -3)
            On entry, error in parameter `compz`.

            Constraint: `compz` = 'N', 'V' or 'I'.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -6)
            On entry, error in parameter `ihi`.

            Constraint: 1 <= `ilo` <= `ihi` <= n.

        (`errno` -6)
            On entry, error in parameter `ihi`.

            Constraint: `ilo` = 1 and `ihi` = 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i < n)
            The QZ iteration did not converge and the matrix pair (A,B)
            is not in the generalized Schur form. The computed alpha_i
            and beta_i should be correct for i =
            *<value>*,...,*<value>*.

        (`errno` i>n and i<=2*n)
            The computation of shifts failed and the matrix pair (A,B)
            is not in the generalized Schur form. The computed alpha_i
            and beta_i should be correct for i =
            *<value>*,...,*<value>*.

        (`errno` i > 2*n)
            An unexpected Library error has occurred.

    Notes
    -----
    ``dhgeqz`` implements a single-double-shift version of the QZ method
    for finding the generalized eigenvalues of the real matrix pair
    (A,B) which is in the generalized upper Hessenberg form.
    If the matrix pair (A,B) is not in the generalized upper Hessenberg
    form, then the function ``dgghd3`` should be called before invoking
    ``dhgeqz``.

    This problem is mathematically equivalent to solving the equation

        det(A-lambda B) = 0.

    Note that, to avoid underflow, overflow and other arithmetic
    problems, the generalized eigenvalues lambda_j are never computed
    explicitly by this function but defined as ratios between two
    computed values, alpha_j and beta_j:

        lambda_j = alpha_j/beta_j.

    The arguments alpha_j, in general, are finite complex values and
    beta_j are finite real non-negative values.

    If desired, the matrix pair (A,B) may be reduced to generalized
    Schur form.
    That is, the transformed matrix B is upper triangular and the
    transformed matrix A is block upper triangular, where the diagonal
    blocks are either 1*1 or 2*2.
    The 1*1 blocks provide generalized eigenvalues which are real and
    the 2*2 blocks give complex generalized eigenvalues.

    The argument `job` specifies two options.
    If `job` = 'S' then the matrix pair (A,B) is simultaneously reduced
    to Schur form by applying one orthogonal transformation (usually
    called Q) on the left and another (usually called Z) on the right.
    That is,

        A←Q^TAZ;B←Q^TBZ

    The 2*2 upper-triangular diagonal blocks of B corresponding to 2*2
    blocks of `a` will be reduced to non-negative diagonal matrices.
    That is, if `a`[j,j-1] is nonzero, then `b`[j,j-1] = `b`[j-1,j] = 0
    and `b`[j-1,j-1] and `b`[j,j] will be non-negative.

    If `job` = 'E', then at each iteration the same transformations are
    computed but they are only applied to those parts of A and B which
    are needed to compute alpha and beta.
    This option could be used if generalized eigenvalues are required
    but not generalized eigenvectors.

    If `job` = 'S' and `compq` = 'V' or 'I', and `compz` = 'V' or 'I',
    then the orthogonal transformations used to reduce the pair (A,B)
    are accumulated into the input arrays `q` and `z`.
    If generalized eigenvectors are required then `job` must be set to
    `job` = 'S' and if left (right) generalized eigenvectors are to be
    computed then `compq` (`compz`) must be set to `compq` = 'V' or 'I'
    and not `compq` != 'N'.

    If `compq` = 'I', then eigenvectors are accumulated on the identity
    matrix and on exit the array `q` contains the left eigenvector
    matrix Q.
    However, if `compq` = 'V' then the transformations are accumulated
    on the user-supplied matrix Q_0 in array `q` on entry and thus on
    exit `q` contains the matrix product QQ_0.
    A similar convention is used for `compz`.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore

    Moler, C B and Stewart, G W, 1973, `An algorithm for generalized
    matrix eigenproblems`, SIAM J. Numer. Anal. (10), 241--256

    Stewart, G W and Sun, J-G, 1990, `Matrix Perturbation Theory`,
    Academic Press, London
    """

    fun_name = 'naginterfaces.library.lapackeig.dhgeqz'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _alphar = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _alphai = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _beta = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _q = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'q'},
        q,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _z = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'z'},
        z,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    try:
        _b_lapackeig.dhgeqz(
            job,
            compq,
            compz,
            _n,
            ilo,
            ihi,
            _a,
            _b,
            _alphar,
            _alphai,
            _beta,
            _q,
            _z,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
    _a_py = _a
    _b_py = _b
    _q_py = _q
    _z_py = _z
    _alphar_py = _alphar
    _alphai_py = _alphai
    _beta_py = _beta
    _return_data = DhgeqzReturnData(
        _a_py,
        _b_py,
        _alphar_py,
        _alphai_py,
        _beta_py,
        _q_py,
        _z_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

ZggesReturnData = _collections.namedtuple(
    'ZggesReturnData',
    ['a', 'b', 'sdim', 'alpha', 'beta', 'vsl', 'vsr'],
)

@_b_utils._withdrawal(
    'naginterfaces.library.lapackeig.zgges',
    mkwdrn=None,
    advice=[
        'Please use zgges3 instead.',
        'See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html',
    ]
)
def zgges(jobvsl, jobvsr, sort, n, a, b, selctg=None, data=None):
    u"""
    Computes, for a complex nonsymmetric matrix pair, the generalized
    eigenvalues, the generalized complex Schur form and, optionally, the
    left and/or right matrices of Schur vectors.

    ``zgges`` computes the generalized eigenvalues, the generalized
    Schur form (S,T) and, optionally, the left and/or right generalized
    Schur vectors for a pair of n*n complex nonsymmetric matrices (A,B).
    ``zgges`` is marked as `deprecated` by LAPACK; the replacement
    routine is ``zgges3`` which makes better use of Level 3 BLAS.

    Note: ``zgges`` is deprecated.
    Please use ``zgges3`` instead.
    See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html

    For full information please refer to the NAG Library document for
    f08xn

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08xnf.html

    Parameters
    ----------
    jobvsl : str, length 1
        If `jobvsl` = 'N', do not compute the left Schur vectors.

        If `jobvsl` = 'V', compute the left Schur vectors.

    jobvsr : str, length 1
        If `jobvsr` = 'N', do not compute the right Schur vectors.

        If `jobvsr` = 'V', compute the right Schur vectors.

    sort : str, length 1
        Specifies whether or not to order the eigenvalues on the
        diagonal of the generalized Schur form.

        `sort` = 'N'

            Eigenvalues are not ordered.

        `sort` = 'S'

            Eigenvalues are ordered (see `selctg`).

    n : int
        n, the order of the matrices A and B.

    a : complex, array-like, shape (`n`, `n`)
        The first of the pair of matrices, A.

    b : complex, array-like, shape (`n`, `n`)
        The second of the pair of matrices, B.

    selctg : None or callable retval = selctg(a, b, data=None), optional
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        If `sort` = 'S', `selctg` is used to select generalized
        eigenvalues to be moved to the top left of the generalized Schur
        form.

        Parameters
        ~~~~~~~~~~
        a : complex
            An eigenvalue `a`[j-1]/`b`[j-1] is selected if
            `selctg`(`a`[j-1],`b`[j-1]) is True.

            Note that in the ill-conditioned case, a selected
            generalized eigenvalue may no longer satisfy
            `selctg`(`a`[j-1],`b`[j-1]) = True after ordering. `errno` =
            `n` + 2 in this case.

        b : complex
            An eigenvalue `a`[j-1]/`b`[j-1] is selected if
            `selctg`(`a`[j-1],`b`[j-1]) is True.

            Note that in the ill-conditioned case, a selected
            generalized eigenvalue may no longer satisfy
            `selctg`(`a`[j-1],`b`[j-1]) = True after ordering. `errno` =
            `n` + 2 in this case.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : bool
            Must be True if the eigenvalue is to be selected.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    a : complex, ndarray, shape (`n`, `n`)
        `a` has been overwritten by its generalized Schur form S.

    b : complex, ndarray, shape (`n`, `n`)
        `b` has been overwritten by its generalized Schur form T.

    sdim : int
        If `sort` = 'N', `sdim` = 0.

        If `sort` = 'S', `sdim` = number of eigenvalues (after sorting)
        for which `selctg` is True.

    alpha : complex, ndarray, shape (`n`)
        See the description of `beta`.

    beta : complex, ndarray, shape (`n`)
        `alpha`[j-1]/`beta`[j-1], for j = 1,2,...,`n`, will be the
        generalized eigenvalues. `alpha`[j-1], for j = 1,2,...,`n` and
        `beta`[j-1], for j = 1,2,...,`n`, are the diagonals of the
        complex Schur form (A,B) output by ``zgges``. The `beta`[j-1]
        will be non-negative real.

        **Note:** the quotients `alpha`[j-1]/`beta`[j-1] may easily
        overflow or underflow, and `beta`[j-1] may even be zero.

        Thus, you should avoid naively computing the ratio alpha/beta.

        However, `alpha` will always be less than and usually comparable
        with ||A||_2 in magnitude, and `beta` will always be less than
        and usually comparable with ||B||_2.

    vsl : complex, ndarray, shape (:, :)
        If `jobvsl` = 'V', `vsl` will contain the left Schur vectors, Q.

        If `jobvsl` = 'N', `vsl` is not referenced.

    vsr : complex, ndarray, shape (:, :)
        If `jobvsr` = 'V', `vsr` will contain the right Schur vectors,
        Z.

        If `jobvsr` = 'N', `vsr` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobvsl`.

            Constraint: `jobvsl` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `jobvsr`.

            Constraint: `jobvsr` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `sort`.

            Constraint: `sort` = 'N' or 'S'.

        (`errno` -5)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` `n`+1)
            The QZ iteration failed with an unexpected error, please
            contact NAG.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` `n`+2)
            After reordering, roundoff changed values of some complex
            eigenvalues so that leading eigenvalues in the generalized
            Schur form no longer satisfy `selctg` = True. This could
            also be caused by underflow due to scaling.

        (`errno` `n`+3)
            The eigenvalues could not be reordered because some
            eigenvalues were too close to separate (the problem is very
            ill-conditioned).

        (`errno` i < `n`)
            The QZ iteration did not converge and the matrix pair (A,B)
            is not in the generalized Schur form. The computed alpha_i
            and beta_i should be correct for i =
            *<value>*,...,*<value>*.

    Notes
    -----
    The generalized Schur factorization for a pair of complex matrices
    (A,B) is given by

        A = QSZ^H, B = QTZ^H,

    where Q and Z are unitary, T and S are upper triangular.
    The generalized eigenvalues, lambda, of (A,B) are computed from the
    diagonals of T and S and satisfy

        Az = lambda Bz,

    where z is the corresponding generalized eigenvector. lambda is
    actually returned as the pair (alpha,beta) such that

        lambda = alpha/beta

    since beta, or even both alpha and beta can be zero.
    The columns of Q and Z are the left and right generalized Schur
    vectors of (A,B).

    Optionally, ``zgges`` can order the generalized eigenvalues on the
    diagonals of (S,T) so that selected eigenvalues are at the top left.
    The leading columns of Q and Z then form an orthonormal basis for
    the corresponding eigenspaces, the deflating subspaces.

    ``zgges`` computes T to have real non-negative diagonal entries.
    The generalized Schur factorization, before reordering, is computed
    by the QZ algorithm.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    def cb_selctg(a, b, data=None):
        """Wrapping user's selctg. Called as base selctg."""
        if data is not None:
            _retval_selctg = selctg(
                a,
                b,
                data,
            )
        else:
            _retval_selctg = selctg(
                a,
                b,
            )
        return _retval_selctg

    fun_name = 'naginterfaces.library.lapackeig.zgges'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'), (_b, 'b')),
    )
    _b_utils._check_callable(selctg, {'fun_name': fun_name, 'entity': 'selctg'}, allow_none=True)
    _selctg = None if selctg is None else cb_selctg
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _alpha = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_n_py,),
    )
    _beta = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobvsl = _b_utils._EngineCharScalarType(
        jobvsl, {'fun_name': fun_name, 'entity': 'jobvsl'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvsl_py = _b_utils._EngineCharScalarType.to_py(_jobvsl)
    if _jobvsl_py in ('V', 'v'):
        _vsl_dim1 = _n_py
    else:
        _vsl_dim1 = 1
    if _jobvsl_py in ('V', 'v'):
        _vsl_dim2 = _n_py
    else:
        _vsl_dim2 = 1
    exp_shape_vsl = (_vsl_dim1, _vsl_dim2)
    _vsl = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_vsl,
        sorder=_m_sorder,
    )
    _jobvsr = _b_utils._EngineCharScalarType(
        jobvsr, {'fun_name': fun_name, 'entity': 'jobvsr'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvsr_py = _b_utils._EngineCharScalarType.to_py(_jobvsr)
    if _jobvsr_py in ('V', 'v'):
        _vsr_dim1 = _n_py
    else:
        _vsr_dim1 = 1
    if _jobvsr_py in ('V', 'v'):
        _vsr_dim2 = _n_py
    else:
        _vsr_dim2 = 1
    exp_shape_vsr = (_vsr_dim1, _vsr_dim2)
    _vsr = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_vsr,
        sorder=_m_sorder,
    )
    try:
        _return_scalar = _b_lapackeig.zgges(
            jobvsl,
            jobvsr,
            sort,
            _selctg,
            _n,
            _a,
            _b,
            _alpha,
            _beta,
            _vsl,
            _vsr,
            data,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _a_py = _a
    _b_py = _b
    _sdim_py = _return_scalar
    _alpha_py = _alpha
    _beta_py = _beta
    _vsl_py = _vsl
    _vsr_py = _vsr
    _return_data = ZggesReturnData(
        _a_py,
        _b_py,
        _sdim_py,
        _alpha_py,
        _beta_py,
        _vsl_py,
        _vsr_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

ZggesxReturnData = _collections.namedtuple(
    'ZggesxReturnData',
    ['a', 'b', 'sdim', 'alpha', 'beta', 'vsl', 'vsr', 'rconde', 'rcondv'],
)

def zggesx(jobvsl, jobvsr, sort, sense, a, b, selctg=None, data=None):
    u"""
    Computes, for a complex nonsymmetric matrix pair, the generalized
    eigenvalues, the generalized complex Schur form and, optionally, the
    left and/or right matrices of Schur vectors; also, optionally,
    computes reciprocal condition numbers for selected eigenvalues.

    ``zggesx`` computes the generalized eigenvalues, the generalized
    Schur form (S,T) and, optionally, the left and/or right generalized
    Schur vectors for a pair of n*n complex nonsymmetric matrices (A,B).

    Estimates of condition numbers for selected generalized eigenvalue
    clusters and Schur vectors are also computed.

    For full information please refer to the NAG Library document for
    f08xp

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08xpf.html

    Parameters
    ----------
    jobvsl : str, length 1
        If `jobvsl` = 'N', do not compute the left Schur vectors.

        If `jobvsl` = 'V', compute the left Schur vectors.

    jobvsr : str, length 1
        If `jobvsr` = 'N', do not compute the right Schur vectors.

        If `jobvsr` = 'V', compute the right Schur vectors.

    sort : str, length 1
        Specifies whether or not to order the eigenvalues on the
        diagonal of the generalized Schur form.

        `sort` = 'N'

            Eigenvalues are not ordered.

        `sort` = 'S'

            Eigenvalues are ordered (see `selctg`).

    sense : str, length 1
        Determines which reciprocal condition numbers are computed.

        `sense` = 'N'

            None are computed.

        `sense` = 'E'

            Computed for average of selected eigenvalues only.

        `sense` = 'V'

            Computed for selected deflating subspaces only.

        `sense` = 'B'

            Computed for both.

        If `sense` = 'E', 'V' or 'B', `sort` = 'S'.

    a : complex, array-like, shape (n, n)
        The first of the pair of matrices, A.

    b : complex, array-like, shape (n, n)
        The second of the pair of matrices, B.

    selctg : None or callable retval = selctg(a, b, data=None), optional
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        If `sort` = 'S', `selctg` is used to select generalized
        eigenvalues to be moved to the top left of the generalized Schur
        form.

        Parameters
        ~~~~~~~~~~
        a : complex
            An eigenvalue `a`[j-1]/`b`[j-1] is selected if
            `selctg`(`a`[j-1],`b`[j-1]) is True.

            Note that in the ill-conditioned case, a selected
            generalized eigenvalue may no longer satisfy
            `selctg`(`a`[j-1],`b`[j-1]) = True after ordering. `errno` =
            `n` + 2 in this case.

        b : complex
            An eigenvalue `a`[j-1]/`b`[j-1] is selected if
            `selctg`(`a`[j-1],`b`[j-1]) is True.

            Note that in the ill-conditioned case, a selected
            generalized eigenvalue may no longer satisfy
            `selctg`(`a`[j-1],`b`[j-1]) = True after ordering. `errno` =
            `n` + 2 in this case.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : bool
            Must be True if the eigenvalue is to be selected.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    a : complex, ndarray, shape (n, n)
        `a` has been overwritten by its generalized Schur form S.

    b : complex, ndarray, shape (n, n)
        `b` has been overwritten by its generalized Schur form T.

    sdim : int
        If `sort` = 'N', `sdim` = 0.

        If `sort` = 'S', `sdim` = number of eigenvalues (after sorting)
        for which `selctg` is True.

    alpha : complex, ndarray, shape (n)
        See the description of `beta`.

    beta : complex, ndarray, shape (n)
        `alpha`[j-1]/`beta`[j-1], for j = 1,2,...,n, will be the
        generalized eigenvalues. `alpha`[j-1] and `beta`[j-1],j =
        1,2,...,n are the diagonals of the complex Schur form (S,T).
        `beta`[j-1] will be non-negative real.

        **Note:** the quotients `alpha`[j-1]/`beta`[j-1] may easily
        overflow or underflow, and `beta`[j-1] may even be zero.

        Thus, you should avoid naively computing the ratio alpha/beta.

        However, `alpha` will always be less than and usually comparable
        with ||`a`|| in magnitude, and `beta` will always be less than
        and usually comparable with ||`b`||.

    vsl : complex, ndarray, shape (:, :)
        If `jobvsl` = 'V', `vsl` will contain the left Schur vectors, Q.

        If `jobvsl` = 'N', `vsl` is not referenced.

    vsr : complex, ndarray, shape (:, :)
        If `jobvsr` = 'V', `vsr` will contain the right Schur vectors,
        Z.

        If `jobvsr` = 'N', `vsr` is not referenced.

    rconde : float, ndarray, shape (2)
        If `sense` = 'E' or 'B', `rconde`[0] and `rconde`[1] contain the
        reciprocal condition numbers for the average of the selected
        eigenvalues.

        If `sense` = 'N' or 'V', `rconde` is not referenced.

    rcondv : float, ndarray, shape (2)
        If `sense` = 'V' or 'B', `rcondv`[0] and `rcondv`[1] contain the
        reciprocal condition numbers for the selected deflating
        subspaces.

        if `sense` = 'N' or 'E', `rcondv` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobvsl`.

            Constraint: `jobvsl` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `jobvsr`.

            Constraint: `jobvsr` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `sort`.

            Constraint: `sort` = 'N' or 'S'.

        (`errno` -5)
            On entry, error in parameter `sense`.

            Constraint: `sense` = 'N', 'E', 'V' or 'B'.

        (`errno` -6)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` n+1)
            The QZ iteration failed with an unexpected error, please
            contact NAG.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` (i > 0) and (i<=n))
            The QZ iteration failed. (A,B) are not in Schur form, but
            `alpha`[j] and `beta`[j] should be correct from element
            *<value>*.

        (`errno` n+2)
            After reordering, roundoff changed values of some complex
            eigenvalues so that leading eigenvalues in the generalized
            Schur form no longer satisfy `selctg` = True. This could
            also be caused by underflow due to scaling.

        (`errno` n+3)
            The eigenvalues could not be reordered because some
            eigenvalues were too close to separate (the problem is very
            ill-conditioned).

    Notes
    -----
    The generalized Schur factorization for a pair of complex matrices
    (A,B) is given by

        A = QSZ^H, B = QTZ^H,

    where Q and Z are unitary, T and S are upper triangular.
    The generalized eigenvalues, lambda, of (A,B) are computed from the
    diagonals of T and S and satisfy

        Az = lambda Bz,

    where z is the corresponding generalized eigenvector. lambda is
    actually returned as the pair (alpha,beta) such that

        lambda = alpha/beta

    since beta, or even both alpha and beta can be zero.
    The columns of Q and Z are the left and right generalized Schur
    vectors of (A,B).

    Optionally, ``zggesx`` can order the generalized eigenvalues on the
    diagonals of (S,T) so that selected eigenvalues are at the top left.
    The leading columns of Q and Z then form an orthonormal basis for
    the corresponding eigenspaces, the deflating subspaces.

    ``zggesx`` computes T to have real non-negative diagonal entries.
    The generalized Schur factorization, before reordering, is computed
    by the QZ algorithm.

    The reciprocals of the condition estimates, the reciprocal values of
    the left and right projection norms, are returned in `rconde`[0] and
    `rconde`[1] respectively, for the selected generalized eigenvalues,
    together with reciprocal condition estimates for the corresponding
    left and right deflating subspaces, in `rcondv`[0] and `rcondv`[1].
    See Section 4.11 of Anderson `et al.` (1999) for further
    information.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    def cb_selctg(a, b, data=None):
        """Wrapping user's selctg. Called as base selctg."""
        if data is not None:
            _retval_selctg = selctg(
                a,
                b,
                data,
            )
        else:
            _retval_selctg = selctg(
                a,
                b,
            )
        return _retval_selctg

    fun_name = 'naginterfaces.library.lapackeig.zggesx'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'), (_b, 'b')),
    )
    _b_utils._check_callable(selctg, {'fun_name': fun_name, 'entity': 'selctg'}, allow_none=True)
    _selctg = None if selctg is None else cb_selctg
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _alpha = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_n_py,),
    )
    _beta = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobvsl = _b_utils._EngineCharScalarType(
        jobvsl, {'fun_name': fun_name, 'entity': 'jobvsl'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvsl_py = _b_utils._EngineCharScalarType.to_py(_jobvsl)
    if _jobvsl_py in ('V', 'v'):
        _vsl_dim1 = _n_py
    else:
        _vsl_dim1 = 1
    if _jobvsl_py in ('V', 'v'):
        _vsl_dim2 = _n_py
    else:
        _vsl_dim2 = 1
    exp_shape_vsl = (_vsl_dim1, _vsl_dim2)
    _vsl = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_vsl,
        sorder=_m_sorder,
    )
    _jobvsr = _b_utils._EngineCharScalarType(
        jobvsr, {'fun_name': fun_name, 'entity': 'jobvsr'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvsr_py = _b_utils._EngineCharScalarType.to_py(_jobvsr)
    if _jobvsr_py in ('V', 'v'):
        _vsr_dim1 = _n_py
    else:
        _vsr_dim1 = 1
    if _jobvsr_py in ('V', 'v'):
        _vsr_dim2 = _n_py
    else:
        _vsr_dim2 = 1
    exp_shape_vsr = (_vsr_dim1, _vsr_dim2)
    _vsr = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_vsr,
        sorder=_m_sorder,
    )
    _rconde = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (2,),
    )
    _rcondv = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (2,),
    )
    try:
        _return_scalar = _b_lapackeig.zggesx(
            jobvsl,
            jobvsr,
            sort,
            _selctg,
            sense,
            _n,
            _a,
            _b,
            _alpha,
            _beta,
            _vsl,
            _vsr,
            _rconde,
            _rcondv,
            data,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _a_py = _a
    _b_py = _b
    _sdim_py = _return_scalar
    _alpha_py = _alpha
    _beta_py = _beta
    _vsl_py = _vsl
    _vsr_py = _vsr
    _rconde_py = _rconde
    _rcondv_py = _rcondv
    _return_data = ZggesxReturnData(
        _a_py,
        _b_py,
        _sdim_py,
        _alpha_py,
        _beta_py,
        _vsl_py,
        _vsr_py,
        _rconde_py,
        _rcondv_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

Zgges3ReturnData = _collections.namedtuple(
    'Zgges3ReturnData',
    ['a', 'b', 'sdim', 'alpha', 'beta', 'vsl', 'vsr'],
)

def zgges3(jobvsl, jobvsr, sort, n, a, b, selctg=None, data=None):
    u"""
    Computes, for a complex nonsymmetric matrix pair, using BLAS-3, the
    generalized eigenvalues, the generalized complex Schur form and,
    optionally, the left and/or right matrices of Schur vectors.

    ``zgges3`` computes the generalized eigenvalues, the generalized
    Schur form (S,T) and, optionally, the left and/or right generalized
    Schur vectors for a pair of n*n complex nonsymmetric matrices (A,B).

    For full information please refer to the NAG Library document for
    f08xq

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08xqf.html

    Parameters
    ----------
    jobvsl : str, length 1
        If `jobvsl` = 'N', do not compute the left Schur vectors.

        If `jobvsl` = 'V', compute the left Schur vectors.

    jobvsr : str, length 1
        If `jobvsr` = 'N', do not compute the right Schur vectors.

        If `jobvsr` = 'V', compute the right Schur vectors.

    sort : str, length 1
        Specifies whether or not to order the eigenvalues on the
        diagonal of the generalized Schur form.

        `sort` = 'N'

            Eigenvalues are not ordered.

        `sort` = 'S'

            Eigenvalues are ordered (see `selctg`).

    n : int
        n, the order of the matrices A and B.

    a : complex, array-like, shape (`n`, `n`)
        The first of the pair of matrices, A.

    b : complex, array-like, shape (`n`, `n`)
        The second of the pair of matrices, B.

    selctg : None or callable retval = selctg(a, b, data=None), optional
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        If `sort` = 'S', `selctg` is used to select generalized
        eigenvalues to be moved to the top left of the generalized Schur
        form.

        Parameters
        ~~~~~~~~~~
        a : complex
            An eigenvalue `a`[j-1]/`b`[j-1] is selected if
            `selctg`(`a`[j-1],`b`[j-1]) is True.

            Note that in the ill-conditioned case, a selected
            generalized eigenvalue may no longer satisfy
            `selctg`(`a`[j-1],`b`[j-1]) = True after ordering. `errno` =
            `n` + 2 in this case.

        b : complex
            An eigenvalue `a`[j-1]/`b`[j-1] is selected if
            `selctg`(`a`[j-1],`b`[j-1]) is True.

            Note that in the ill-conditioned case, a selected
            generalized eigenvalue may no longer satisfy
            `selctg`(`a`[j-1],`b`[j-1]) = True after ordering. `errno` =
            `n` + 2 in this case.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : bool
            Must be True if the eigenvalue is to be selected.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    a : complex, ndarray, shape (`n`, `n`)
        `a` has been overwritten by its generalized Schur form S.

    b : complex, ndarray, shape (`n`, `n`)
        `b` has been overwritten by its generalized Schur form T.

    sdim : int
        If `sort` = 'N', `sdim` = 0.

        If `sort` = 'S', `sdim` = number of eigenvalues (after sorting)
        for which `selctg` is True.

    alpha : complex, ndarray, shape (`n`)
        See the description of `beta`.

    beta : complex, ndarray, shape (`n`)
        `alpha`[j-1]/`beta`[j-1], for j = 1,2,...,`n`, will be the
        generalized eigenvalues. `alpha`[j-1], for j = 1,2,...,`n` and
        `beta`[j-1], for j = 1,2,...,`n`, are the diagonals of the
        complex Schur form (A,B) output by ``zgges3``. The `beta`[j-1]
        will be non-negative real.

        **Note:** the quotients `alpha`[j-1]/`beta`[j-1] may easily
        overflow or underflow, and `beta`[j-1] may even be zero.

        Thus, you should avoid naively computing the ratio alpha/beta.

        However, `alpha` will always be less than and usually comparable
        with ||A||_2 in magnitude, and `beta` will always be less than
        and usually comparable with ||B||_2.

    vsl : complex, ndarray, shape (:, :)
        If `jobvsl` = 'V', `vsl` will contain the left Schur vectors, Q.

        If `jobvsl` = 'N', `vsl` is not referenced.

    vsr : complex, ndarray, shape (:, :)
        If `jobvsr` = 'V', `vsr` will contain the right Schur vectors,
        Z.

        If `jobvsr` = 'N', `vsr` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobvsl`.

            Constraint: `jobvsl` = 'N' or 'V'.

        (`errno` -2)
            On entry, error in parameter `jobvsr`.

            Constraint: `jobvsr` = 'N' or 'V'.

        (`errno` -3)
            On entry, error in parameter `sort`.

            Constraint: `sort` = 'N' or 'S'.

        (`errno` -5)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` `n`+1)
            The QZ iteration failed with an unexpected error, please
            contact NAG.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` `n`+2)
            After reordering, roundoff changed values of some complex
            eigenvalues so that leading eigenvalues in the generalized
            Schur form no longer satisfy `selctg` = True. This could
            also be caused by underflow due to scaling.

        (`errno` `n`+3)
            The eigenvalues could not be reordered because some
            eigenvalues were too close to separate (the problem is very
            ill-conditioned).

        (`errno` i < `n`)
            The QZ iteration did not converge and the matrix pair (A,B)
            is not in the generalized Schur form. The computed alpha_i
            and beta_i should be correct for i =
            *<value>*,...,*<value>*.

    Notes
    -----
    The generalized Schur factorization for a pair of complex matrices
    (A,B) is given by

        A = QSZ^H, B = QTZ^H,

    where Q and Z are unitary, T and S are upper triangular.
    The generalized eigenvalues, lambda, of (A,B) are computed from the
    diagonals of T and S and satisfy

        Az = lambda Bz,

    where z is the corresponding generalized eigenvector. lambda is
    actually returned as the pair (alpha,beta) such that

        lambda = alpha/beta

    since beta, or even both alpha and beta can be zero.
    The columns of Q and Z are the left and right generalized Schur
    vectors of (A,B).

    Optionally, ``zgges3`` can order the generalized eigenvalues on the
    diagonals of (S,T) so that selected eigenvalues are at the top left.
    The leading columns of Q and Z then form an orthonormal basis for
    the corresponding eigenspaces, the deflating subspaces.

    ``zgges3`` computes T to have real non-negative diagonal entries.
    The generalized Schur factorization, before reordering, is computed
    by the QZ algorithm.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 2012, `Matrix Computations`, (4th
    Edition), Johns Hopkins University Press, Baltimore
    """

    def cb_selctg(a, b, data=None):
        """Wrapping user's selctg. Called as base selctg."""
        if data is not None:
            _retval_selctg = selctg(
                a,
                b,
                data,
            )
        else:
            _retval_selctg = selctg(
                a,
                b,
            )
        return _retval_selctg

    fun_name = 'naginterfaces.library.lapackeig.zgges3'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'), (_b, 'b')),
    )
    _b_utils._check_callable(selctg, {'fun_name': fun_name, 'entity': 'selctg'}, allow_none=True)
    _selctg = None if selctg is None else cb_selctg
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _alpha = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_n_py,),
    )
    _beta = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_n_py,),
    )
    _jobvsl = _b_utils._EngineCharScalarType(
        jobvsl, {'fun_name': fun_name, 'entity': 'jobvsl'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvsl_py = _b_utils._EngineCharScalarType.to_py(_jobvsl)
    if _jobvsl_py in ('V', 'v'):
        _vsl_dim1 = _n_py
    else:
        _vsl_dim1 = 1
    if _jobvsl_py in ('V', 'v'):
        _vsl_dim2 = _n_py
    else:
        _vsl_dim2 = 1
    exp_shape_vsl = (_vsl_dim1, _vsl_dim2)
    _vsl = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_vsl,
        sorder=_m_sorder,
    )
    _jobvsr = _b_utils._EngineCharScalarType(
        jobvsr, {'fun_name': fun_name, 'entity': 'jobvsr'}, exp_chrlen=1,
    ).to_ctypes()
    _jobvsr_py = _b_utils._EngineCharScalarType.to_py(_jobvsr)
    if _jobvsr_py in ('V', 'v'):
        _vsr_dim1 = _n_py
    else:
        _vsr_dim1 = 1
    if _jobvsr_py in ('V', 'v'):
        _vsr_dim2 = _n_py
    else:
        _vsr_dim2 = 1
    exp_shape_vsr = (_vsr_dim1, _vsr_dim2)
    _vsr = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        exp_shape_vsr,
        sorder=_m_sorder,
    )
    try:
        _return_scalar = _b_lapackeig.zgges3(
            jobvsl,
            jobvsr,
            sort,
            _selctg,
            _n,
            _a,
            _b,
            _alpha,
            _beta,
            _vsl,
            _vsr,
            data,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _a_py = _a
    _b_py = _b
    _sdim_py = _return_scalar
    _alpha_py = _alpha
    _beta_py = _beta
    _vsl_py = _vsl
    _vsr_py = _vsr
    _return_data = Zgges3ReturnData(
        _a_py,
        _b_py,
        _sdim_py,
        _alpha_py,
        _beta_py,
        _vsl_py,
        _vsr_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

ZhgeqzReturnData = _collections.namedtuple(
    'ZhgeqzReturnData',
    ['a', 'b', 'alpha', 'beta', 'q', 'z'],
)

def zhgeqz(job, compq, compz, ilo, ihi, a, b, q, z):
    u"""
    Eigenvalues and generalized Schur factorization of complex
    generalized upper Hessenberg form reduced from a pair of complex,
    square, matrices.

    ``zhgeqz`` implements the QZ method for finding generalized
    eigenvalues of the complex matrix pair (A,B) of order n, which is in
    the generalized upper Hessenberg form.

    For full information please refer to the NAG Library document for
    f08xs

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08xsf.html

    Parameters
    ----------
    job : str, length 1
        Specifies the operations to be performed on (A,B).

        `job` = 'E'

            The matrix pair (A,B) on exit might not be in the
            generalized Schur form.

        `job` = 'S'

            The matrix pair (A,B) on exit will be in the generalized
            Schur form.

    compq : str, length 1
        Specifies the operations to be performed on Q:

        `compq` = 'N'

            The array `q` is unchanged.

        `compq` = 'V'

            The left transformation Q is accumulated on the array `q`.

        `compq` = 'I'

            The array `q` is initialized to the identity matrix before
            the left transformation Q is accumulated in `q`.

    compz : str, length 1
        Specifies the operations to be performed on Z.

        `compz` = 'N'

            The array `z` is unchanged.

        `compz` = 'V'

            The right transformation Z is accumulated on the array `z`.

        `compz` = 'I'

            The array `z` is initialized to the identity matrix before
            the right transformation Z is accumulated in `z`.

    ilo : int
        The indices i_lo and i_hi, respectively which define the upper
        triangular parts of A. The submatrices A(1:i_lo-1,1:i_lo-1) and
        A(i_hi+1:n,i_hi+1:n) are then upper triangular. These arguments
        are provided by ``zggbal`` if the matrix pair was previously
        balanced; otherwise, `ilo` = 1 and `ihi` = n.

    ihi : int
        The indices i_lo and i_hi, respectively which define the upper
        triangular parts of A. The submatrices A(1:i_lo-1,1:i_lo-1) and
        A(i_hi+1:n,i_hi+1:n) are then upper triangular. These arguments
        are provided by ``zggbal`` if the matrix pair was previously
        balanced; otherwise, `ilo` = 1 and `ihi` = n.

    a : complex, array-like, shape (n, n)
        The n*n upper Hessenberg matrix A. The elements below the first
        subdiagonal must be set to zero.

    b : complex, array-like, shape (n, n)
        The n*n upper triangular matrix B. The elements below the
        diagonal must be zero.

    q : complex, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `compq` in ('V', 'I'): n; if
        `compq`='N': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compq` in ('V', 'I'): n; if
        `compq`='N': 1; otherwise: 0.

        If `compq` = 'V', the matrix Q_0. The matrix Q_0 is usually the
        matrix Q returned by ``zgghd3``.

        If `compq` = 'N', `q` is not referenced.

    z : complex, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `compz` in ('V', 'I'): n; if
        `compz`='N': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `compz` in ('V', 'I'): n; if
        `compz`='N': 1; otherwise: 0.

        If `compz` = 'V', the matrix Z_0. The matrix Z_0 is usually the
        matrix Z returned by ``zgghd3``.

        If `compz` = 'N', `z` is not referenced.

    Returns
    -------
    a : complex, ndarray, shape (n, n)
        If `job` = 'S', the matrix pair (A,B) will be simultaneously
        reduced to generalized Schur form.

        If `job` = 'E', the 1*1 and 2*2 diagonal blocks of the matrix
        pair (A,B) will give generalized eigenvalues but the remaining
        elements will be irrelevant.

    b : complex, ndarray, shape (n, n)
        If `job` = 'S', the matrix pair (A,B) will be simultaneously
        reduced to generalized Schur form.

        If `job` = 'E', the 1*1 and 2*2 diagonal blocks of the matrix
        pair (A,B) will give generalized eigenvalues but the remaining
        elements will be irrelevant.

    alpha : complex, ndarray, shape (n)
        alpha_j, for j = 1,2,...,n.

    beta : complex, ndarray, shape (n)
        beta_j, for j = 1,2,...,n.

    q : complex, ndarray, shape (:, :)
        If `compq` = 'V', `q` contains the matrix product QQ_0.

        If `compq` = 'I', `q` contains the transformation matrix Q.

    z : complex, ndarray, shape (:, :)
        If `compz` = 'V', `z` contains the matrix product ZZ_0.

        If `compz` = 'I', `z` contains the transformation matrix Z.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'E' or 'S'.

        (`errno` -2)
            On entry, error in parameter `compq`.

            Constraint: `compq` = 'N', 'V' or 'I'.

        (`errno` -3)
            On entry, error in parameter `compz`.

            Constraint: `compz` = 'N', 'V' or 'I'.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -6)
            On entry, error in parameter `ihi`.

            Constraint: 1 <= `ilo` <= `ihi` <= n.

        (`errno` -6)
            On entry, error in parameter `ihi`.

            Constraint: `ilo` = 1 and `ihi` = 0.

        (`errno` i < n)
            The QZ iteration did not converge and the matrix pair (A,B)
            is not in the generalized Schur form. The computed alpha_i
            and beta_i should be correct for i =
            *<value>*,...,*<value>*.

        (`errno` i>n and i<=2*n)
            The computation of shifts failed and the matrix pair (A,B)
            is not in the generalized Schur form. The computed alpha_i
            and beta_i should be correct for i =
            *<value>*,...,*<value>*.

        (`errno` i > 2*n)
            An unexpected Library error has occurred.

    Notes
    -----
    ``zhgeqz`` implements a single-shift version of the QZ method for
    finding the generalized eigenvalues of the complex matrix pair (A,B)
    which is in the generalized upper Hessenberg form.
    If the matrix pair (A,B) is not in the generalized upper Hessenberg
    form, then the function ``zgghd3`` should be called before invoking
    ``zhgeqz``.

    This problem is mathematically equivalent to solving the matrix
    equation

        det(A-lambda B) = 0.

    Note that, to avoid underflow, overflow and other arithmetic
    problems, the generalized eigenvalues lambda_j are never computed
    explicitly by this function but defined as ratios between two
    computed values, alpha_j and beta_j:

        lambda_j = alpha_j/beta_j.

    The arguments alpha_j, in general, are finite complex values and
    beta_j are finite real non-negative values.

    If desired, the matrix pair (A,B) may be reduced to generalized
    Schur form.
    That is, the transformed matrices A and B are upper triangular and
    the diagonal values of A and B provide alpha and beta.

    The argument `job` specifies two options.
    If `job` = 'S' then the matrix pair (A,B) is simultaneously reduced
    to Schur form by applying one unitary transformation (usually called
    Q) on the left and another (usually called Z) on the right.
    That is,

        A←Q^HAZ;B←Q^HBZ

    If `job` = 'E', then at each iteration the same transformations are
    computed but they are only applied to those parts of A and B which
    are needed to compute alpha and beta.
    This option could be used if generalized eigenvalues are required
    but not generalized eigenvectors.

    If `job` = 'S' and `compq` = 'V' or 'I', and `compz` = 'V' or 'I',
    then the unitary transformations used to reduce the pair (A,B) are
    accumulated into the input arrays `q` and `z`.
    If generalized eigenvectors are required then `job` must be set to
    `job` = 'S' and if left (right) generalized eigenvectors are to be
    computed then `compq` (`compz`) must be set to `compq` = 'V' or 'I'
    rather than `compq` = 'N'.

    If `compq` = 'I', then eigenvectors are accumulated on the identity
    matrix and on exit the array `q` contains the left eigenvector
    matrix Q.
    However, if `compq` = 'V' then the transformations are accumulated
    in the user-supplied matrix Q_0 in array `q` on entry and thus on
    exit `q` contains the matrix product QQ_0.
    A similar convention is used for `compz`.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore

    Moler, C B and Stewart, G W, 1973, `An algorithm for generalized
    matrix eigenproblems`, SIAM J. Numer. Anal. (10), 241--256

    Stewart, G W and Sun, J-G, 1990, `Matrix Perturbation Theory`,
    Academic Press, London
    """

    fun_name = 'naginterfaces.library.lapackeig.zhgeqz'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
    )
    _alpha = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_n_py,),
    )
    _beta = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_n_py,),
    )
    _q = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'q'},
        q,
        copy=True,
    )
    _z = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'z'},
        z,
        copy=True,
    )
    _b_lapackeig.zhgeqz(
        job,
        compq,
        compz,
        _n,
        ilo,
        ihi,
        _a,
        _b,
        _alpha,
        _beta,
        _q,
        _z,
    )
    _a_py = _a
    _b_py = _b
    _q_py = _q
    _z_py = _z
    _alpha_py = _alpha
    _beta_py = _beta
    _return_data = ZhgeqzReturnData(
        _a_py,
        _b_py,
        _alpha_py,
        _beta_py,
        _q_py,
        _z_py,
    )
    return _return_data

DtgsjaReturnData = _collections.namedtuple(
    'DtgsjaReturnData',
    ['a', 'b', 'alpha', 'beta', 'u', 'v', 'q', 'ncycle'],
)

def dtgsja(jobu, jobv, jobq, k, l, a, b, tola, tolb, u, v, q):
    u"""
    Computes the generalized singular value decomposition of a real
    upper triangular (or trapezoidal) matrix pair.

    ``dtgsja`` computes the generalized singular value decomposition
    (GSVD) of two real upper trapezoidal matrices A and B, where A is an
    m*n matrix and B is a p*n matrix.

    A and B are assumed to be in the form returned by ``dggsvp3``.

    For full information please refer to the NAG Library document for
    f08ye

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08yef.html

    Parameters
    ----------
    jobu : str, length 1
        If `jobu` = 'U', `u` must contain an orthogonal matrix U_1 on
        entry, and the product U_1 U is returned.

        If `jobu` = 'I', `u` is initialized to the unit matrix, and the
        orthogonal matrix U is returned.

        If `jobu` = 'N', U is not computed.

    jobv : str, length 1
        If `jobv` = 'V', `v` must contain an orthogonal matrix V_1 on
        entry, and the product V_1 V is returned.

        If `jobv` = 'I', `v` is initialized to the unit matrix, and the
        orthogonal matrix V is returned.

        If `jobv` = 'N', V is not computed.

    jobq : str, length 1
        If `jobq` = 'Q', `q` must contain an orthogonal matrix Q_1 on
        entry, and the product Q_1 Q is returned.

        If `jobq` = 'I', `q` is initialized to the unit matrix, and the
        orthogonal matrix Q is returned.

        If `jobq` = 'N', Q is not computed.

    k : int
        `k` and `l` specify the sizes, k and l, of the subblocks of A
        and B, whose GSVD is to be computed by ``dtgsja``.

    l : int
        `k` and `l` specify the sizes, k and l, of the subblocks of A
        and B, whose GSVD is to be computed by ``dtgsja``.

    a : float, array-like, shape (m, n)
        The m*n matrix A.

    b : float, array-like, shape (p, n)
        The p*n matrix B.

    tola : float
        `tola` and `tolb` are the convergence criteria for the
        Jacobi--Kogbetliantz iteration procedure. Generally, they should
        be the same as used in the preprocessing step performed by
        ``dggsvp3``, say

            `tola` = max(m, n)||A||epsilon,;`tolb` = max(p,
            n)||B||epsilon,

        where epsilon is the machine precision.

    tolb : float
        `tola` and `tolb` are the convergence criteria for the
        Jacobi--Kogbetliantz iteration procedure. Generally, they should
        be the same as used in the preprocessing step performed by
        ``dggsvp3``, say

            `tola` = max(m, n)||A||epsilon,;`tolb` = max(p,
            n)||B||epsilon,

        where epsilon is the machine precision.

    u : float, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobu` in ('U', 'I'): m; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobu` in ('U', 'I'): m; otherwise: 1.

        If `jobu` = 'U', `u` must contain an m*m matrix U_1 (usually the
        orthogonal matrix returned by ``dggsvp3``).

    v : float, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobv` in ('V', 'I'): p; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobv` in ('V', 'I'): p; otherwise: 1.

        If `jobv` = 'V', `v` must contain an p*p matrix V_1 (usually the
        orthogonal matrix returned by ``dggsvp3``).

    q : float, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobq` in ('Q', 'I'): n; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobq` in ('Q', 'I'): n; otherwise: 1.

        If `jobq` = 'Q', `q` must contain an n*n matrix Q_1 (usually the
        orthogonal matrix returned by ``dggsvp3``).

    Returns
    -------
    a : float, ndarray, shape (m, n)
        If m-k-l >= 0, `a`[0:k+l,n-k-l:n] contains the (k+l)*(k+l) upper
        triangular matrix R.

        If m-k-l < 0, `a`[0:m,n-k-l:n] contains the first m rows of the
        (k+l)*(k+l) upper triangular matrix R, and the submatrix R_33 is
        returned in `b`[m-k:l,n+m-k-l:n].

    b : float, ndarray, shape (p, n)
        If m-k-l < 0, `b`[m-k:l,n+m-k-l:n] contains the submatrix R_33
        of R.

    alpha : float, ndarray, shape (n)
        See the description of `beta`.

    beta : float, ndarray, shape (n)
        `alpha` and `beta` contain the generalized singular value pairs
        of A and B;

            `alpha`[i] = 1, `beta`[i] = 0, for i = 0,...,k-1, and

            if m-k-l >= 0, `alpha`[i] = alpha_i, `beta`[i] = beta_i, for
            i = k,...,k+l-1, or

            if m-k-l < 0, `alpha`[i] = alpha_i, `beta`[i] = beta_i, for
            i = k,...,m-1 and `alpha`[i] = 0, `beta`[i] = 1, for i =
            m,...,k+l-1.

        Furthermore, if k+l < n, `alpha`[i] = `beta`[i] = 0, for i =
        k+l,...,n-1.

    u : float, ndarray, shape (:, :)
        If `jobu` = 'U', `u` contains the product U_1 U.

        If `jobu` = 'I', `u` contains the orthogonal matrix U.

        If `jobu` = 'N', `u` is not referenced.

    v : float, ndarray, shape (:, :)
        If `jobv` = 'I', `v` contains the orthogonal matrix V.

        If `jobv` = 'V', `v` contains the product V_1 V.

        If `jobv` = 'N', `v` is not referenced.

    q : float, ndarray, shape (:, :)
        If `jobq` = 'I', `q` contains the orthogonal matrix Q.

        If `jobq` = 'Q', `q` contains the product Q_1 Q.

        If `jobq` = 'N', `q` is not referenced.

    ncycle : int
        The number of cycles required for convergence.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobu`.

            Constraint: `jobu` = 'U', 'I' or 'N'.

        (`errno` -2)
            On entry, error in parameter `jobv`.

            Constraint: `jobv` = 'V', 'I' or 'N'.

        (`errno` -3)
            On entry, error in parameter `jobq`.

            Constraint: `jobq` = 'Q', 'I' or 'N'.

        (`errno` -4)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -5)
            On entry, error in parameter p.

            Constraint: p >= 0.

        (`errno` -6)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` 1)
            The procedure does not converge after 40 cycles.

    Notes
    -----
    ``dtgsja`` computes the GSVD of the matrices A and B which are
    assumed to have the form as returned by ``dggsvp3``

        [table omitted]

    where the k*k matrix A_12 and the l*l matrix B_13 are nonsingular
    upper triangular, A_23 is l*l upper triangular if m-k-l >= 0 and is
    (m-k)*l upper trapezoidal otherwise.

    ``dtgsja`` computes orthogonal matrices Q, U and V, diagonal
    matrices D_1 and D_2, and an upper triangular matrix R such that

        U^TAQ = D_1(0 R), V^TBQ = D_2(0 R).

    Optionally Q, U and V may or may not be computed, or they may be
    premultiplied by matrices Q_1, U_1 and V_1 respectively.

    If (m-k-l) >= 0 then D_1, D_2 and R have the form

        D_1 = [table omitted],

        D_2 = [table omitted],

        R = [table omitted],

    where C = diag(alpha_{k+1},,, ...,,, alpha_{k+l}), S =
    diag(beta_{k+1},,, ...,,, beta_{k+l}).

    If (m-k-l) < 0 then D_1, D_2 and R have the form

        D_1 = [table omitted],

        D_2 = [table omitted],

        R = [table omitted],

    where C = diag(alpha_{k+1},,, ...,,, alpha_m), S =
    diag(beta_{k+1},,, ...,,, beta_m).

    In both cases the diagonal matrix C has non-negative diagonal
    elements, the diagonal matrix S has positive diagonal elements, so
    that S is nonsingular, and C^2+S^2 = 1.
    See Section 2.3.5.3 of Anderson `et al.` (1999) for further
    information.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.dtgsja'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _p_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[0]
    _p = _b_utils._EngineIntScalarType(
        _p_py, {'fun_name': fun_name, 'entity': '_p'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _alpha = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _beta = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _u = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'u'},
        u,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _v = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'v'},
        v,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _q = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'q'},
        q,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _return_scalar = _b_lapackeig.dtgsja(
        jobu,
        jobv,
        jobq,
        _m,
        _p,
        _n,
        k,
        l,
        _a,
        _b,
        tola,
        tolb,
        _alpha,
        _beta,
        _u,
        _v,
        _q,
    )
    _a_py = _a
    _b_py = _b
    _u_py = _u
    _v_py = _v
    _q_py = _q
    _alpha_py = _alpha
    _beta_py = _beta
    _ncycle_py = _return_scalar
    _return_data = DtgsjaReturnData(
        _a_py,
        _b_py,
        _alpha_py,
        _beta_py,
        _u_py,
        _v_py,
        _q_py,
        _ncycle_py,
    )
    return _return_data

DtgexcReturnData = _collections.namedtuple(
    'DtgexcReturnData',
    ['a', 'b', 'q', 'z', 'ifst', 'ilst'],
)

def dtgexc(wantq, wantz, a, b, q, z, ifst, ilst):
    u"""
    Reorders the generalized real Schur decomposition of a real matrix
    pair using an orthogonal equivalence transformation.

    ``dtgexc`` reorders the generalized Schur factorization of a matrix
    pair in real generalized Schur form.

    For full information please refer to the NAG Library document for
    f08yf

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08yff.html

    Parameters
    ----------
    wantq : bool
        If `wantq` = True, update the left transformation matrix Q.

        If `wantq` = False, do not update Q.

    wantz : bool
        If `wantz` = True, update the right transformation matrix Z.

        If `wantz` = False, do not update Z.

    a : float, array-like, shape (n, n)
        The matrix S in the pair (S,T).

    b : float, array-like, shape (n, n)
        The matrix T, in the pair (S,T).

    q : float, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `wantq`= True: n; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `wantq`= True: n; otherwise: 1.

        If `wantq` = True, the orthogonal matrix Q.

    z : float, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `wantz`= True: n; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `wantz`= True: n; otherwise: 1.

        If `wantz` = True, the orthogonal matrix Z.

    ifst : int
        The indices i_1 and i_2 that specify the reordering of the
        diagonal blocks of (S,T). The block with row index `ifst` is
        moved to row `ilst`, by a sequence of swapping between adjacent
        blocks.

    ilst : int
        The indices i_1 and i_2 that specify the reordering of the
        diagonal blocks of (S,T). The block with row index `ifst` is
        moved to row `ilst`, by a sequence of swapping between adjacent
        blocks.

    Returns
    -------
    a : float, ndarray, shape (n, n)
        The updated matrix S^.

    b : float, ndarray, shape (n, n)
        The updated matrix T^

    q : float, ndarray, shape (:, :)
        If `wantq` = True, the updated matrix QQ^.

        If `wantq` = False, `q` is not referenced.

    z : float, ndarray, shape (:, :)
        If `wantz` = True, the updated matrix ZZ^.

        If `wantz` = False, `z` is not referenced.

    ifst : int
        If `ifst` pointed on entry to the second row of a 2*2 block, it
        is changed to point to the first row; `ilst` always points to
        the first row of the block in its final position (which may
        differ from its input value by +1 or -1).

    ilst : int
        If `ifst` pointed on entry to the second row of a 2*2 block, it
        is changed to point to the first row; `ilst` always points to
        the first row of the block in its final position (which may
        differ from its input value by +1 or -1).

    Raises
    ------
    NagValueError
        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -13)
            On entry, error in parameter `ilst`.

            Constraint: 1 <= `ifst` <= n and 1 <= `ilst` <= n.

        (`errno` 1)
            The transformed matrix pair would be too far from
            generalized Schur form; the problem is ill-conditioned.
            (S,T) may have been partially reordered, and `ilst` points
            to the first row of the current position of the block being
            moved.

    Notes
    -----
    ``dtgexc`` reorders the generalized real n*n matrix pair (S,T) in
    real generalized Schur form, so that the diagonal element or block
    of (S,T) with row index i_1 is moved to row i_2, using an orthogonal
    equivalence transformation.
    That is, S and T are factorized as

        S = Q^S^Z^^T, T = Q^T^Z^^T,

    where (S^,T^) are also in real generalized Schur form.

    The pair (S,T) are in real generalized Schur form if S is block
    upper triangular with 1*1 and 2*2 diagonal blocks and T is upper
    triangular as returned, for example, by ``dgges3``, or ``dhgeqz``
    with job = 'S'.

    If S and T are the result of a generalized Schur factorization of a
    matrix pair (A,B)

        A = QSZ^T, B = QTZ^T

    then, optionally, the matrices Q and Z can be updated as QQ^ and
    ZZ^.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug
    """

    fun_name = 'naginterfaces.library.lapackeig.dtgexc'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _q = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'q'},
        q,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _z = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'z'},
        z,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _return_scalars = _b_lapackeig.dtgexc(
        wantq,
        wantz,
        _n,
        _a,
        _b,
        _q,
        _z,
        ifst,
        ilst,
    )
    _a_py = _a
    _b_py = _b
    _q_py = _q
    _z_py = _z
    _ifst_py = _return_scalars.ifst
    _ilst_py = _return_scalars.ilst
    _return_data = DtgexcReturnData(
        _a_py,
        _b_py,
        _q_py,
        _z_py,
        _ifst_py,
        _ilst_py,
    )
    return _return_data

DtgsenReturnData = _collections.namedtuple(
    'DtgsenReturnData',
    ['a', 'b', 'alphar', 'alphai', 'beta', 'q', 'z', 'm', 'pl', 'pr', 'dif'],
)

def dtgsen(ijob, wantq, wantz, select, a, b, q, z):
    u"""
    Reorders the generalized real Schur decomposition of a real matrix
    pair using an orthogonal equivalence transformation, computes the
    generalized eigenvalues of the reordered pair and, optionally,
    computes the estimates of reciprocal condition numbers for
    eigenvalues and eigenspaces.

    ``dtgsen`` reorders the generalized Schur factorization of a matrix
    pair in real generalized Schur form, so that a selected cluster of
    eigenvalues appears in the leading elements, or blocks on the
    diagonal of the generalized Schur form.
    The function also, optionally, computes the reciprocal condition
    numbers of the cluster of eigenvalues and/or corresponding deflating
    subspaces.

    For full information please refer to the NAG Library document for
    f08yg

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08ygf.html

    Parameters
    ----------
    ijob : int
        Specifies whether condition numbers are required for the cluster
        of eigenvalues (p and q) or the deflating subspaces (Dif_u and
        Dif_l).

        `ijob` = 0

            Only reorder with respect to `select`. No extras.

        `ijob` = 1

            Reciprocal of norms of 'projections' onto left and right
            eigenspaces with respect to the selected cluster (p and q).

        `ijob` = 2

            The upper bounds on Dif_u and Dif_l. F-norm-based estimate
            (`dif`[0:2]).

        `ijob` = 3

            Estimate of Dif_u and Dif_l. 1-norm-based estimate
            (`dif`[0:2]). About five times as expensive as `ijob` = 2.

        `ijob` = 4

            Compute `pl`, `pr` and `dif` as in `ijob` = 0, 1 or 2.
            Economic version to get it all.

        `ijob` = 5

            Compute `pl`, `pr` and `dif` as in `ijob` = 0, 1 or 3.

    wantq : bool
        If `wantq` = True, update the left transformation matrix Q.

        If `wantq` = False, do not update Q.

    wantz : bool
        If `wantz` = True, update the right transformation matrix Z.

        If `wantz` = False, do not update Z.

    select : bool, array-like, shape (n)
        Specifies the eigenvalues in the selected cluster. To select a
        real eigenvalue lambda_j, `select`[j-1] must be set to True.

        To select a complex conjugate pair of eigenvalues lambda_j and
        lambda_{j+1}, corresponding to a 2*2 diagonal block, either
        `select`[j-1] or `select`[j] or both must be set to True; a
        complex conjugate pair of eigenvalues must be either both
        included in the cluster or both excluded.

    a : float, array-like, shape (n, n)
        The matrix S in the pair (S,T).

    b : float, array-like, shape (n, n)
        The matrix T, in the pair (S,T).

    q : float, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `wantq`= True: n; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `wantq`= True: n; otherwise: 1.

        If `wantq` = True, the n*n matrix Q.

    z : float, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `wantz`= True: n; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `wantz`= True: n; otherwise: 1.

        If `wantz` = True, the n*n matrix Z.

    Returns
    -------
    a : float, ndarray, shape (n, n)
        The updated matrix S^.

    b : float, ndarray, shape (n, n)
        The updated matrix T^

    alphar : float, ndarray, shape (n)
        See the description of `beta`.

    alphai : float, ndarray, shape (n)
        See the description of `beta`.

    beta : float, ndarray, shape (n)
        `alphar`[j-1]/`beta`[j-1] and `alphai`[j-1]/`beta`[j-1] are the
        real and imaginary parts respectively of the jth eigenvalue, for
        j = 1,2,...,n.

        If `alphai`[j-1] is zero, then the jth eigenvalue is real; if
        positive then `alphai`[j] is negative, and the jth and (j+1)st
        eigenvalues are a complex conjugate pair.

        Conjugate pairs of eigenvalues correspond to the 2*2 diagonal
        blocks of S^.

        These 2*2 blocks can be reduced by applying complex unitary
        transformations to (S^,T^) to obtain the complex Schur form
        (S~,T~), where S~ is triangular (and complex).

        In this form `alphar`+i`alphai` and `beta` are the diagonals of
        S~ and T~ respectively.

    q : float, ndarray, shape (:, :)
        If `wantq` = True, the updated matrix QQ^.

        If `wantq` = False, `q` is not referenced.

    z : float, ndarray, shape (:, :)
        If `wantz` = True, the updated matrix ZZ^.

        If `wantz` = False, `z` is not referenced.

    m : int
        The dimension of the specified pair of left and right
        eigenspaces (deflating subspaces).

    pl : float
        If `ijob` = 1, 4 or 5, `pl` and `pr` are lower bounds on the
        reciprocal of the norm of 'projections' p and q onto left and
        right eigenspaces with respect to the selected cluster. 0 <
        `pl`, `pr` <= 1.

        If `m` = 0 or `m` = n, `pl` = `pr` = 1.

        If `ijob` = 0, 2 or 3, `pl` and `pr` are not referenced.

    pr : float
        If `ijob` = 1, 4 or 5, `pl` and `pr` are lower bounds on the
        reciprocal of the norm of 'projections' p and q onto left and
        right eigenspaces with respect to the selected cluster. 0 <
        `pl`, `pr` <= 1.

        If `m` = 0 or `m` = n, `pl` = `pr` = 1.

        If `ijob` = 0, 2 or 3, `pl` and `pr` are not referenced.

    dif : float, ndarray, shape (2)
        If `ijob` >= 2, `dif`[0:2] store the estimates of Dif_u and
        Dif_l.

        If `ijob` = 2 or 4, `dif`[0:2] are F-norm-based upper bounds on
        Dif_u and Dif_l.

        If `ijob` = 3 or 5, `dif`[0:2] are 1-norm-based estimates of
        Dif_u and Dif_l.

        If `m` = 0 or n, `dif`[0:2] = ||(A,B)||_F.

        If `ijob` = 0 or 1, `dif` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `ijob`.

            Constraint: 0 <= `ijob` <= 5.

        (`errno` -5)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` 1)
            Reordering of (S,T) failed because the transformed matrix
            pair would be too far from generalized Schur form; the
            problem is very ill-conditioned. (S,T) may have been
            partially reordered. If requested, 0 is returned in `dif`[0]
            and `dif`[1], `pl` and `pr`.

    Notes
    -----
    ``dtgsen`` factorizes the generalized real n*n matrix pair (S,T) in
    real generalized Schur form, using an orthogonal equivalence
    transformation as

        S = Q^S^Z^^T, T = Q^T^Z^^T,

    where (S^,T^) are also in real generalized Schur form and have the
    selected eigenvalues as the leading diagonal elements, or diagonal
    blocks.
    The leading columns of Q and Z are the generalized Schur vectors
    corresponding to the selected eigenvalues and form orthonormal
    subspaces for the left and right eigenspaces (deflating subspaces)
    of the pair (S,T).

    The pair (S,T) are in real generalized Schur form if S is block
    upper triangular with 1*1 and 2*2 diagonal blocks and T is upper
    triangular as returned, for example, by ``dgges3``, or ``dhgeqz``
    with job = 'S'.
    The diagonal elements, or blocks, define the generalized eigenvalues
    (alpha_i,beta_i), for i = 1,2,...,n, of the pair (S,T).
    The eigenvalues are given by

        lambda_i = alpha_i/beta_i,

    but are returned as the pair (alpha_i,beta_i) in order to avoid
    possible overflow in computing lambda_i.
    Optionally, the function returns reciprocals of condition number
    estimates for the selected eigenvalue cluster, p and q, the right
    and left projection norms, and of deflating subspaces, Dif_u and
    Dif_l.
    For more information see Sections 2.4.8 and 4.11 of Anderson `et
    al.` (1999).

    If S and T are the result of a generalized Schur factorization of a
    matrix pair (A,B)

        A = QSZ^T, B = QTZ^T

    then, optionally, the matrices Q and Z can be updated as QQ^ and
    ZZ^.
    Note that the condition numbers of the pair (S,T) are the same as
    those of the pair (A,B).

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug
    """

    fun_name = 'naginterfaces.library.lapackeig.dtgsen'

    _select = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'select'},
        select,
        copy=False,
    )
    _n_py = _b_utils._EngineLogicalArrayType.get_shape(_select, {'fun_name': fun_name, 'entity': 'select'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _alphar = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _alphai = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _beta = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _q = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'q'},
        q,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _z = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'z'},
        z,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _dif = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (2,),
    )
    _return_scalars = _b_lapackeig.dtgsen(
        ijob,
        wantq,
        wantz,
        _select,
        _n,
        _a,
        _b,
        _alphar,
        _alphai,
        _beta,
        _q,
        _z,
        _dif,
    )
    _a_py = _a
    _b_py = _b
    _q_py = _q
    _z_py = _z
    _alphar_py = _alphar
    _alphai_py = _alphai
    _beta_py = _beta
    _m_py = _return_scalars.m
    _pl_py = _return_scalars.pl
    _pr_py = _return_scalars.pr
    _dif_py = _dif
    _return_data = DtgsenReturnData(
        _a_py,
        _b_py,
        _alphar_py,
        _alphai_py,
        _beta_py,
        _q_py,
        _z_py,
        _m_py,
        _pl_py,
        _pr_py,
        _dif_py,
    )
    return _return_data

DtgsylReturnData = _collections.namedtuple(
    'DtgsylReturnData',
    ['c', 'f', 'scale', 'dif'],
)

def dtgsyl(trans, ijob, a, b, c, d, e, f):
    u"""
    Solves the real-valued, generalized, quasi-trangular, Sylvester
    equation.

    ``dtgsyl`` solves the generalized real quasi-triangular Sylvester
    equations.

    For full information please refer to the NAG Library document for
    f08yh

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08yhf.html

    Parameters
    ----------
    trans : str, length 1
        If `trans` = 'N', solve the generalized Sylvester equation (1).

        If `trans` = 'T', solve the 'transposed' system (2).

    ijob : int
        Specifies what kind of functionality is to be performed when
        `trans` = 'N'.

        `ijob` = 0

            Solve (1) only.

        `ijob` = 1

            The functionality of `ijob` = 0 or 3.

        `ijob` = 2

            The functionality of `ijob` = 0 or 4.

        `ijob` = 3

            Only an estimate of Dif[(A,D),(B,E)] is computed based on
            the Frobenius norm.

        `ijob` = 4

            Only an estimate of Dif[(A,D),(B,E)] is computed based on
            the 1-norm.

        If `trans` = 'T', `ijob` is not referenced.

    a : float, array-like, shape (m, m)
        The upper quasi-triangular matrix A.

    b : float, array-like, shape (n, n)
        The upper quasi-triangular matrix B.

    c : float, array-like, shape (m, n)
        Contains the right-hand-side matrix C.

    d : float, array-like, shape (m, m)
        The upper triangular matrix D.

    e : float, array-like, shape (n, n)
        The upper triangular matrix E.

    f : float, array-like, shape (m, n)
        Contains the right-hand side matrix F.

    Returns
    -------
    c : float, ndarray, shape (m, n)
        If `ijob` = 0, 1 or 2, `c` is overwritten by the solution matrix
        R.

        If `trans` = 'N' and `ijob` = 3 or 4, `c` holds R, the solution
        achieved during the computation of the Dif estimate.

    f : float, ndarray, shape (m, n)
        If `ijob` = 0, 1 or 2, `f` is overwritten by the solution matrix
        L.

        If `trans` = 'N' and `ijob` = 3 or 4, `f` holds L, the solution
        achieved during the computation of the Dif estimate.

    scale : float
        alpha, the scaling factor in (1) and (2).

        If 0 < `scale` < 1, `c` and `f` hold the solutions R and L,
        respectively, to a slightly perturbed system but the input
        arrays `a`, `b`, `d` and `e` have not been changed.

        If `scale` = 0, `c` and `f` hold the solutions R and L,
        respectively, to the homogeneous system with C = F = 0.

        In this case `dif` is not referenced.

        Normally, `scale` = 1.

    dif : float
        The estimate of Dif. If `ijob` = 0, `dif` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'T'.

        (`errno` -2)
            On entry, error in parameter `ijob`.

            Constraint: 0 <= `ijob` <= 4.

        (`errno` -3)
            On entry, error in parameter m.

            Constraint: m > 0.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n > 0.

        (`errno` i > 0)
            (A,D) and (B,E) have common or close eigenvalues and so no
            solution could be computed.

    Notes
    -----
    ``dtgsyl`` solves either the generalized real Sylvester equations

        [table omitted]

    or the equations

        [table omitted]

    where the pair (A,D) are given m*m matrices in real generalized
    Schur form, (B,E) are given n*n matrices in real generalized Schur
    form and (C,F) are given m*n matrices.
    The pair (R,L) are the m*n solution matrices, and alpha is an output
    scaling factor determined by the function to avoid overflow in
    computing (R,L).

    Equations (1) are equivalent to equations of the form

        Zx = alpha b,

    where

        Z = (I⊗A-B^T⊗I;I⊗D-E^T⊗I)

    and ⊗ is the Kronecker product.
    Equations (2) are then equivalent to

        Z^Ty = alpha b.

    The pair (S,T) are in real generalized Schur form if S is block
    upper triangular with 1*1 and 2*2 diagonal blocks on the diagonal
    and T is upper triangular as returned, for example, by ``dgges3``,
    or ``dhgeqz`` with job = 'S'.

    Optionally, the function estimates Dif[(A,D),(B,E)], the separation
    between the matrix pairs (A,D) and (B,E), which is the smallest
    singular value of Z.
    The estimate can be based on either the Frobenius norm, or the
    1-norm.
    The 1-norm estimate can be three to ten times more expensive than
    the Frobenius norm estimate, but makes the condition estimation
    uniform with the nonsymmetric eigenproblem.
    The Frobenius norm estimate provides a low cost, but equally
    reliable estimate.
    For more information see Sections 2.4.8.3 and 4.11.1.3 of Anderson
    `et al.` (1999) and Kågström and Poromaa (1996).

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Kågström, B, 1994, `A perturbation analysis of the generalized
    Sylvester equation (AR-LB,DR-LE) = (c,F)`, SIAM J. Matrix Anal.
    Appl. (15), 1045--1060

    Kågström, B and Poromaa, P, 1996, `LAPACK-style algorithms and
    software for solving the generalized Sylvester equation and
    estimating the separation between regular matrix pairs`, ACM Trans.
    Math. Software (22), 78--103
    """

    fun_name = 'naginterfaces.library.lapackeig.dtgsyl'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _d = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'd'},
        d,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _e = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'e'},
        e,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _f = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'f'},
        f,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _return_scalars = _b_lapackeig.dtgsyl(
        trans,
        ijob,
        _m,
        _n,
        _a,
        _b,
        _c,
        _d,
        _e,
        _f,
    )
    _c_py = _c
    _f_py = _f
    _scale_py = _return_scalars.scale
    _dif_py = _return_scalars.dif
    _return_data = DtgsylReturnData(
        _c_py,
        _f_py,
        _scale_py,
        _dif_py,
    )
    return _return_data

DtgevcReturnData = _collections.namedtuple(
    'DtgevcReturnData',
    ['vl', 'vr', 'm'],
)

def dtgevc(side, howmny, a, b, select=None, vl=None, vr=None):
    u"""
    Computes right and left generalized eigenvectors of the matrix pair
    (A,B) which is assumed to be in generalized upper Schur form.

    ``dtgevc`` computes some or all of the right and/or left generalized
    eigenvectors of a pair of real matrices (A,B) which are in
    generalized real Schur form.

    For full information please refer to the NAG Library document for
    f08yk

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08ykf.html

    Parameters
    ----------
    side : str, length 1
        Specifies the required sets of generalized eigenvectors.

        `side` = 'R'

            Only right eigenvectors are computed.

        `side` = 'L'

            Only left eigenvectors are computed.

        `side` = 'B'

            Both left and right eigenvectors are computed.

    howmny : str, length 1
        Specifies further details of the required generalized
        eigenvectors.

        `howmny` = 'A'

            All right and/or left eigenvectors are computed.

        `howmny` = 'B'

            All right and/or left eigenvectors are computed; they are
            backtransformed using the input matrices supplied in arrays
            `vr` and/or `vl`.

        `howmny` = 'S'

            Selected right and/or left eigenvectors, defined by the
            array `select`, are computed.

    a : float, array-like, shape (n, n)
        The matrix pair (A,B) must be in the generalized Schur form.
        Usually, this is the matrix A returned by ``dhgeqz``.

    b : float, array-like, shape (n, n)
        The matrix pair (A,B) must be in the generalized Schur form. If
        A has a 2*2 diagonal block then the corresponding 2*2 block of B
        must be diagonal with positive elements. Usually, this is the
        matrix B returned by ``dhgeqz``.

    select : None or bool, array-like, shape (:), optional
        Note: the required length for this argument is determined as
        follows: if `howmny`='S': n; otherwise: 1.

        Specifies the eigenvectors to be computed if `howmny` = 'S'. To
        select the generalized eigenvector corresponding to the jth
        generalized eigenvalue, the jth element of `select` should be
        set to True; if the eigenvalue corresponds to a complex
        conjugate pair, then real and imaginary parts of eigenvectors
        corresponding to the complex conjugate eigenvalue pair will be
        computed.

        If `howmny` = 'A' or 'B', `select` is not referenced.

    vl : None or float, array-like, shape (:, :), optional
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `side` in ('L', 'B'): n; if
        `side`='R': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `side` in ('L', 'B'): mm; if
        `side`='R': 1; otherwise: 0.

        If `howmny` = 'B' and `side` = 'L' or 'B', `vl` must be
        initialized to an n*n matrix Q. Usually, this is the orthogonal
        matrix Q of left Schur vectors returned by ``dhgeqz``.

    vr : None or float, array-like, shape (:, :), optional
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `side` in ('R', 'B'): n; if
        `side`='L': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `side` in ('R', 'B'): mm; if
        `side`='L': 1; otherwise: 0.

        If `howmny` = 'B' and `side` = 'R' or 'B', `vr` must be
        initialized to an n*n matrix Z. Usually, this is the orthogonal
        matrix Z of right Schur vectors returned by ``dhgeqz``.

    Returns
    -------
    vl : None or float, ndarray, shape (:, :)
        If `side` = 'L' or 'B', `vl` contains:

            if `howmny` = 'A', the matrix Y of left eigenvectors of
            (A,B);

            if `howmny` = 'B', the matrix QY;

            if `howmny` = 'S', the left eigenvectors of (A,B) specified
            by `select`, stored consecutively in the columns of the
            array `vl`, in the same order as their corresponding
            eigenvalues.

        A complex eigenvector corresponding to a complex eigenvalue is
        stored in two consecutive columns, the first holding the real
        part, and the second the imaginary part.

        If `side` = 'R', `vl` is not referenced.

    vr : None or float, ndarray, shape (:, :)
        If `side` = 'R' or 'B', `vr` contains:

            if `howmny` = 'A', the matrix X of right eigenvectors of
            (A,B);

            if `howmny` = 'B', the matrix ZX;

            if `howmny` = 'S', the right eigenvectors of (A,B) specified
            by `select`, stored consecutively in the columns of the
            array `vr`, in the same order as their corresponding
            eigenvalues.

        A complex eigenvector corresponding to a complex eigenvalue is
        stored in two consecutive columns, the first holding the real
        part, and the second the imaginary part.

        If `side` = 'L', `vr` is not referenced.

    m : int
        The number of columns in the arrays `vl` and/or `vr` actually
        used to store the eigenvectors. If `howmny` = 'A' or 'B', `m` is
        set to n. Each selected real eigenvector occupies one column and
        each selected complex eigenvector occupies two columns.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `side`.

            Constraint: `side` = 'B', 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `howmny`.

            Constraint: `howmny` = 'A', 'B' or 'S'.

        (`errno` -3)
            On entry, error in parameter `select`.

            Constraint: `select`[j] = True or False, for j = 0,...,n-1.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -5)
            On entry, error in parameter `a`.

        (`errno` -7)
            On entry, error in parameter `b`.

        (`errno` -13)
            On entry, error in parameter mm.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` i > 0)
            The 2*2 block (*<value>*:*<value>*+1) does not have complex
            eigenvalues.

    Notes
    -----
    ``dtgevc`` computes some or all of the right and/or left generalized
    eigenvectors of the matrix pair (A,B) which is assumed to be in
    generalized upper Schur form.
    If the matrix pair (A,B) is not in the generalized upper Schur form,
    then ``dhgeqz`` should be called before invoking ``dtgevc``.

    The right generalized eigenvector x and the left generalized
    eigenvector y of (A,B) corresponding to a generalized eigenvalue
    lambda are defined by

        (A-lambda B)x = 0

    and

        y^H(A-lambda B) = 0.

    If a generalized eigenvalue is determined as 0/0, which is due to
    zero diagonal elements at the same locations in both A and B, a unit
    vector is returned as the corresponding eigenvector.

    Note that the generalized eigenvalues are computed using ``dhgeqz``
    but ``dtgevc`` does not explicitly require the generalized
    eigenvalues to compute eigenvectors.
    The ordering of the eigenvectors is based on the ordering of the
    eigenvalues as computed by ``dtgevc``.

    If all eigenvectors are requested, the function may either return
    the matrices X and/or Y of right or left eigenvectors of (A,B), or
    the products ZX and/or QY, where Z and Q are two matrices supplied
    by you.
    Usually, Q and Z are chosen as the orthogonal matrices returned by
    ``dhgeqz``.
    Equivalently, Q and Z are the left and right Schur vectors of the
    matrix pair supplied to ``dhgeqz``.
    In that case, QY and ZX are the left and right generalized
    eigenvectors, respectively, of the matrix pair supplied to
    ``dhgeqz``.

    A must be block upper triangular; with 1*1 and 2*2 diagonal blocks.
    Corresponding to each 2*2 diagonal block is a complex conjugate pair
    of eigenvalues and eigenvectors; only one eigenvector of the pair is
    computed, namely the one corresponding to the eigenvalue with
    positive imaginary part.
    Each 1*1 block gives a real generalized eigenvalue and a
    corresponding eigenvector.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore

    Moler, C B and Stewart, G W, 1973, `An algorithm for generalized
    matrix eigenproblems`, SIAM J. Numer. Anal. (10), 241--256

    Stewart, G W and Sun, J-G, 1990, `Matrix Perturbation Theory`,
    Academic Press, London
    """

    fun_name = 'naginterfaces.library.lapackeig.dtgevc'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _vl = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'vl'},
        vl,
        copy=True,
        allow_none=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _vr = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'vr'},
        vr,
        copy=True,
        allow_none=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'), (_b, 'b'), (_vl, 'vl'), (_vr, 'vr')),
    )
    _vl_ilabad_py = False
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    if _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0] >= _n_py and _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[0] >= _n_py:
        _vl_ilbbad_py = False
        for _vl_j_py in range(1, _n_py-1+1):
            if _b_utils._EngineFloat64ArrayType.get_element(_a, {'fun_name': fun_name, 'entity': 'a'}, ((_vl_j_py+1)-1, _vl_j_py-1)) != 0:
                if _vl_j_py < _n_py-1:
                    if _b_utils._EngineFloat64ArrayType.get_element(_a, {'fun_name': fun_name, 'entity': 'a'}, ((_vl_j_py+2)-1, (_vl_j_py+1)-1)) != 0:
                        _vl_ilabad_py = True
                        break
                if _b_utils._EngineFloat64ArrayType.get_element(_b, {'fun_name': fun_name, 'entity': 'b'}, (_vl_j_py-1, _vl_j_py-1)) == 0 or _b_utils._EngineFloat64ArrayType.get_element(_b, {'fun_name': fun_name, 'entity': 'b'}, ((_vl_j_py+1)-1, (_vl_j_py+1)-1)) == 0 or _b_utils._EngineFloat64ArrayType.get_element(_b, {'fun_name': fun_name, 'entity': 'b'}, (_vl_j_py-1, (_vl_j_py+1)-1)) != 0:
                    _vl_ilbbad_py = True
                    break
        _vl_ilbbad = _b_utils._EngineLogicalScalarType(
            _vl_ilbbad_py, {'fun_name': fun_name, 'entity': '_vl_ilbbad_py'},
        ).to_ctypes()
    else:
        _vl_ilbbad_py = False
    _vl_k_py = 0
    _howmny = _b_utils._EngineCharScalarType(
        howmny, {'fun_name': fun_name, 'entity': 'howmny'}, exp_chrlen=1,
    ).to_ctypes()
    _howmny_py = _b_utils._EngineCharScalarType.to_py(_howmny)
    _select = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'select'},
        select,
        copy=False,
        allow_none=True,
    )
    if (_howmny_py in ('S', 's')) and _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0] >= _n_py:
        _vl_m_py = 0
        while _vl_k_py < _n_py:
            _vl_k_py = _vl_k_py+1
            if _vl_k_py < _n_py:
                if _b_utils._EngineFloat64ArrayType.get_element(_a, {'fun_name': fun_name, 'entity': 'a'}, ((_vl_k_py+1)-1, _vl_k_py-1)) == 0:
                    if _b_utils._EngineLogicalArrayType.get_element(_select, {'fun_name': fun_name, 'entity': 'select'}, (_vl_k_py-1,)):
                        _vl_m_py = _vl_m_py+1
                else:
                    if _b_utils._EngineLogicalArrayType.get_element(_select, {'fun_name': fun_name, 'entity': 'select'}, (_vl_k_py-1,)) or _b_utils._EngineLogicalArrayType.get_element(_select, {'fun_name': fun_name, 'entity': 'select'}, ((_vl_k_py+1)-1,)):
                        _vl_m_py = _vl_m_py+2
                    _vl_k_py = _vl_k_py+1
            elif _b_utils._EngineLogicalArrayType.get_element(_select, {'fun_name': fun_name, 'entity': 'select'}, (_vl_k_py-1,)):
                _vl_m_py = _vl_m_py+1
    else:
        _vl_m_py = _n_py
    _side = _b_utils._EngineCharScalarType(
        side, {'fun_name': fun_name, 'entity': 'side'}, exp_chrlen=1,
    ).to_ctypes()
    _side_py = _b_utils._EngineCharScalarType.to_py(_side)
    if (_howmny_py in ('B', 'b')) and _side_py in ('L', 'l', 'B', 'b'):
        if vl is None:
            raise _b_utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'vl'},
                'must not be None when (howmny in (\'B\', \'b\')) and side in (\'L\', \'l\', \'B\', \'b\')',
            )
    if _howmny_py in ('S', 's'):
        _mm = _b_utils._EngineIntScalarType(
            _vl_m_py, {'fun_name': fun_name, 'entity': '_mm'},
        ).to_ctypes()
        _mm_py = _b_utils._EngineIntScalarType.to_py(_mm)
    else:
        _mm = _b_utils._EngineIntScalarType(
            _n_py, {'fun_name': fun_name, 'entity': '_mm'},
        ).to_ctypes()
        _mm_py = _b_utils._EngineIntScalarType.to_py(_mm)
    if vl is None:
        if _side_py in ('L', 'l', 'B', 'b'):
            _vl_dim1 = _n_py
        elif _side_py in ('R', 'r'):
            _vl_dim1 = 1
        else:
            _vl_dim1 = 0
        if _side_py in ('L', 'l', 'B', 'b'):
            _vl_dim2 = _mm_py
        elif _side_py in ('R', 'r'):
            _vl_dim2 = 1
        else:
            _vl_dim2 = 0
        exp_shape_vl = (_vl_dim1, _vl_dim2)
        _vl = _b_utils._EngineFloat64ArrayType.empty_ndarray(
            exp_shape_vl,
            sorder=_m_sorder,
        )
    if (_howmny_py in ('B', 'b')) and _side_py in ('R', 'r', 'B', 'b'):
        if vr is None:
            raise _b_utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'vr'},
                'must not be None when (howmny in (\'B\', \'b\')) and side in (\'R\', \'r\', \'B\', \'b\')',
            )
    if vr is None:
        if _side_py in ('R', 'r', 'B', 'b'):
            _vr_dim1 = _n_py
        elif _side_py in ('L', 'l'):
            _vr_dim1 = 1
        else:
            _vr_dim1 = 0
        if _side_py in ('R', 'r', 'B', 'b'):
            _vr_dim2 = _mm_py
        elif _side_py in ('L', 'l'):
            _vr_dim2 = 1
        else:
            _vr_dim2 = 0
        exp_shape_vr = (_vr_dim1, _vr_dim2)
        _vr = _b_utils._EngineFloat64ArrayType.empty_ndarray(
            exp_shape_vr,
            sorder=_m_sorder,
        )
    try:
        _return_scalar = _b_lapackeig.dtgevc(
            side,
            howmny,
            _select,
            _n,
            _a,
            _b,
            _vl,
            _vr,
            _mm,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    if _side_py in ('L', 'l', 'B', 'b'):
        _vl_py = _vl
    else:
        _vl_py = None
    if _side_py in ('R', 'r', 'B', 'b'):
        _vr_py = _vr
    else:
        _vr_py = None
    _m_py = _return_scalar
    _return_data = DtgevcReturnData(
        _vl_py,
        _vr_py,
        _m_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

DtgsnaReturnData = _collections.namedtuple(
    'DtgsnaReturnData',
    ['s', 'dif', 'm'],
)

def dtgsna(job, howmny, a, b, mm, select=None, vl=None, vr=None):
    u"""
    Estimates reciprocal condition numbers for specified eigenvalues
    and/or eigenvectors of a real matrix pair in generalized real Schur
    canonical form.

    ``dtgsna`` estimates condition numbers for specified eigenvalues
    and/or eigenvectors of a matrix pair in generalized real Schur form.

    For full information please refer to the NAG Library document for
    f08yl

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08ylf.html

    Parameters
    ----------
    job : str, length 1
        Indicates whether condition numbers are required for eigenvalues
        and/or eigenvectors.

        `job` = 'E'

            Condition numbers for eigenvalues only are computed.

        `job` = 'V'

            Condition numbers for eigenvectors only are computed.

        `job` = 'B'

            Condition numbers for both eigenvalues and eigenvectors are
            computed.

    howmny : str, length 1
        Indicates how many condition numbers are to be computed.

        `howmny` = 'A'

            Condition numbers for all eigenpairs are computed.

        `howmny` = 'S'

            Condition numbers for selected eigenpairs (as specified by
            `select`) are computed.

    a : float, array-like, shape (n, n)
        The upper quasi-triangular matrix S.

    b : float, array-like, shape (n, n)
        The upper triangular matrix T.

    mm : int
        The number of elements in the arrays `s` and `dif`.

    select : None or bool, array-like, shape (:), optional
        Note: the required length for this argument is determined as
        follows: if `howmny`='S': n; otherwise: 1.

        Specifies the eigenpairs for which condition numbers are to be
        computed if `howmny` = 'S'. To select condition numbers for the
        eigenpair corresponding to the real eigenvalue lambda_j,
        `select`[j-1] must be set True. To select condition numbers
        corresponding to a complex conjugate pair of eigenvalues
        lambda_j and lambda_{j+1}, `select`[j-1] and/or `select`[j] must
        be set to True.

        If `howmny` = 'A', `select` is not referenced.

    vl : None or float, array-like, shape (:, :), optional
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `job` in ('E', 'B'): n; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `job` in ('E', 'B'): `mm`; otherwise:
        0.

        If `job` = 'E' or 'B', `vl` must contain left eigenvectors of
        (S,T), corresponding to the eigenpairs specified by `howmny` and
        `select`. The eigenvectors must be stored in consecutive columns
        of `vl`, as returned by ``dggev3`` or ``dtgevc``.

        If `job` = 'V', `vl` is not referenced.

    vr : None or float, array-like, shape (:, :), optional
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `job` in ('E', 'B'): n; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `job` in ('E', 'B'): `mm`; otherwise:
        0.

        If `job` = 'E' or 'B', `vr` must contain right eigenvectors of
        (S,T), corresponding to the eigenpairs specified by `howmny` and
        `select`. The eigenvectors must be stored in consecutive columns
        of `vr`, as returned by ``dggev3`` or ``dtgevc``.

        If `job` = 'V', `vr` is not referenced.

    Returns
    -------
    s : float, ndarray, shape (:)
        If `job` = 'E' or 'B', the reciprocal condition numbers of the
        selected eigenvalues, stored in consecutive elements of the
        array. For a complex conjugate pair of eigenvalues two
        consecutive elements of `s` are set to the same value. Thus
        `s`[j-1], `dif`[j-1], and the jth columns of VL and VR all
        correspond to the same eigenpair (but not in general the jth
        eigenpair, unless all eigenpairs are selected).

        If `job` = 'V', `s` is not referenced.

    dif : float, ndarray, shape (:)
        If `job` = 'V' or 'B', the estimated reciprocal condition
        numbers of the selected eigenvectors, stored in consecutive
        elements of the array. For a complex eigenvector two consecutive
        elements of `dif` are set to the same value. If the eigenvalues
        cannot be reordered to compute `dif`[j-1], `dif`[j-1] is set to
        0; this can only occur when the true value would be very small
        anyway.

        If `job` = 'E', `dif` is not referenced.

    m : int
        M, the number of elements of the arrays `s` and `dif` used to
        store the specified condition numbers; for each selected real
        eigenvalue one element is used, and for each selected complex
        conjugate pair of eigenvalues, two elements are used. If
        `howmny` = 'A', `m` is set to n.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'E', 'V' or 'B'.

        (`errno` -2)
            On entry, error in parameter `howmny`.

            Constraint: `howmny` = 'A' or 'S'.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -15)
            On entry, error in parameter `mm`.

            Constraint: `mm` >= n.

        (`errno` -15)
            On entry, error in parameter `mm`.

            Constraint: `mm` >= m.

    Notes
    -----
    ``dtgsna`` estimates condition numbers for specified eigenvalues
    and/or right eigenvectors of an n*n matrix pair (S,T) in real
    generalized Schur form.
    The function actually returns estimates of the reciprocals of the
    condition numbers in order to avoid possible overflow.

    The pair (S,T) are in real generalized Schur form if S is block
    upper triangular with 1*1 and 2*2 diagonal blocks and T is upper
    triangular as returned, for example, by ``dggesx`` or ``dgges3``, or
    ``dhgeqz`` with job = 'S'.
    The diagonal elements, or blocks, define the generalized eigenvalues
    (alpha_i,beta_i), for i = 1,2,...,n, of the pair (S,T) and the
    eigenvalues are given by

        lambda_i = alpha_i/beta_i,

    so that

        beta_i Sx_i = alpha_i Tx_i or Sx_i = lambda_i Tx_i,

    where x_i is the corresponding (right) eigenvector.

    If S and T are the result of a generalized Schur factorization of a
    matrix pair (A,B)

        A = QSZ^T, B = QTZ^T

    then the eigenvalues and condition numbers of the pair (S,T) are the
    same as those of the pair (A,B).

    Let (alpha,beta) != (0,0) be a simple generalized eigenvalue of
    (A,B).
    Then the reciprocal of the condition number of the eigenvalue lambda
    = alpha/beta is defined as

        s(lambda) = ((|y^TAx|^2+|y^TBx|^2)^(1/2))/((||x||_2||y||_2)),

    where x and y are the right and left eigenvectors of (A,B)
    corresponding to lambda.
    If both alpha and beta are zero, then (A,B) is singular and
    s(lambda) = -1 is returned.

    The definition of the reciprocal of the estimated condition number
    of the right eigenvector x and the left eigenvector y corresponding
    to the simple eigenvalue lambda depends upon whether lambda is a
    real eigenvalue, or one of a complex conjugate pair.

    If the eigenvalue lambda is real and U and V are orthogonal
    transformations such that

        U^T(A,B)V = (S,T) = [table omitted][table omitted],

    where S_22 and T_22 are (n-1)*(n-1) matrices, then the reciprocal
    condition number is given by

        Dif(x) = Dif(y) = Dif((alpha,beta),(S_22,T_22)) = sigma_min(Z),

    where sigma_min(Z) denotes the smallest singular value of the
    2*(n-1)*2*(n-1) matrix

        Z = [table omitted]

    and ⊗ is the Kronecker product.

    If lambda is part of a complex conjugate pair and U and V are
    orthogonal transformations such that

        U^T(A,B)V = (S,T) = [table omitted][table omitted],

    where S_11 and T_11 are two by two matrices, S_22 and T_22 are
    (n-2)*(n-2) matrices, and (S_11,T_11) corresponds to the complex
    conjugate eigenvalue pair lambda, lambda_, then there exist unitary
    matrices U_1 and V_1 such that

        U_1^HS_11 V_1 = [table omitted] and U_1^HT_11 V_1 = [table
        omitted].

    The eigenvalues are given by lambda = s_11/t_11 and lambda_=
    s_22/t_22.
    Then the Frobenius norm-based, estimated reciprocal condition number
    is bounded by

        Dif(x) = Dif(y) <= min(d_1, max(1, |Re(s_11)/Re(s_22)|), d_2)

    where Re(z) denotes the real part of z, d_1 =
    Dif((s_11,t_11),(s_22,t_22)) = sigma_min(Z_1), Z_1 is the complex
    two by two matrix

        Z_1 = [table omitted],

    and d_2 is an upper bound on Dif((S_11,T_11),(S_22,T_22)); i.e., an
    upper bound on sigma_min(Z_2), where Z_2 is the (2n-2)*(2n-2) matrix

        Z_2 = [table omitted].

    See Sections 2.4.8 and 4.11 of Anderson `et al.` (1999) and Kågström
    and Poromaa (1996) for further details and information.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Kågström, B and Poromaa, P, 1996, `LAPACK-style algorithms and
    software for solving the generalized Sylvester equation and
    estimating the separation between regular matrix pairs`, ACM Trans.
    Math. Software (22), 78--103
    """

    fun_name = 'naginterfaces.library.lapackeig.dtgsna'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _vl_k_py = 0
    _howmny = _b_utils._EngineCharScalarType(
        howmny, {'fun_name': fun_name, 'entity': 'howmny'}, exp_chrlen=1,
    ).to_ctypes()
    _howmny_py = _b_utils._EngineCharScalarType.to_py(_howmny)
    _select = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'select'},
        select,
        copy=False,
        allow_none=True,
    )
    if (_howmny_py in ('S', 's')) and _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0] >= _n_py:
        _vl_m_py = 0
        while _vl_k_py < _n_py:
            _vl_k_py = _vl_k_py+1
            if _vl_k_py < _n_py:
                if _b_utils._EngineFloat64ArrayType.get_element(_a, {'fun_name': fun_name, 'entity': 'a'}, ((_vl_k_py+1)-1, _vl_k_py-1)) == 0:
                    if _b_utils._EngineLogicalArrayType.get_element(_select, {'fun_name': fun_name, 'entity': 'select'}, (_vl_k_py-1,)):
                        _vl_m_py = _vl_m_py+1
                else:
                    if _b_utils._EngineLogicalArrayType.get_element(_select, {'fun_name': fun_name, 'entity': 'select'}, (_vl_k_py-1,)) or _b_utils._EngineLogicalArrayType.get_element(_select, {'fun_name': fun_name, 'entity': 'select'}, ((_vl_k_py+1)-1,)):
                        _vl_m_py = _vl_m_py+2
                    _vl_k_py = _vl_k_py+1
            elif _b_utils._EngineLogicalArrayType.get_element(_select, {'fun_name': fun_name, 'entity': 'select'}, (_vl_k_py-1,)):
                _vl_m_py = _vl_m_py+1
    else:
        _vl_m_py = _n_py
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _vl = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'vl'},
        vl,
        copy=False,
        allow_none=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _vr = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'vr'},
        vr,
        copy=False,
        allow_none=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _job = _b_utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _job_py = _b_utils._EngineCharScalarType.to_py(_job)
    _mm = _b_utils._EngineIntScalarType(
        mm, {'fun_name': fun_name, 'entity': 'mm'},
    ).to_ctypes()
    _mm_py = _b_utils._EngineIntScalarType.to_py(_mm)
    if _job_py in ('E', 'e', 'B', 'b'):
        _s_dim1 = _mm_py
    else:
        _s_dim1 = 0
    exp_shape_s = (_s_dim1,)
    _s = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_s,
    )
    if _job_py in ('V', 'v', 'B', 'b'):
        _dif_dim1 = _mm_py
    else:
        _dif_dim1 = 0
    exp_shape_dif = (_dif_dim1,)
    _dif = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_dif,
    )
    _return_scalar = _b_lapackeig.dtgsna(
        job,
        howmny,
        _select,
        _n,
        _a,
        _b,
        _vl,
        _vr,
        _s,
        _dif,
        _mm,
    )
    _s_py = _s
    _dif_py = _dif
    _m_py = _return_scalar
    _return_data = DtgsnaReturnData(
        _s_py,
        _dif_py,
        _m_py,
    )
    return _return_data

ZtgsjaReturnData = _collections.namedtuple(
    'ZtgsjaReturnData',
    ['a', 'b', 'alpha', 'beta', 'u', 'v', 'q', 'ncycle'],
)

def ztgsja(jobu, jobv, jobq, k, l, a, b, tola, tolb, u, v, q):
    u"""
    Computes the generalized singular value decomposition of a complex
    upper triangular (or trapezoidal) matrix pair.

    ``ztgsja`` computes the generalized singular value decomposition
    (GSVD) of two complex upper trapezoidal matrices A and B, where A is
    an m*n matrix and B is a p*n matrix.

    A and B are assumed to be in the form returned by ``zggsvp3``.

    For full information please refer to the NAG Library document for
    f08ys

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08ysf.html

    Parameters
    ----------
    jobu : str, length 1
        If `jobu` = 'U', `u` must contain a unitary matrix U_1 on entry,
        and the product U_1 U is returned.

        If `jobu` = 'I', `u` is initialized to the unit matrix, and the
        unitary matrix U is returned.

        If `jobu` = 'N', U is not computed.

    jobv : str, length 1
        If `jobv` = 'V', `v` must contain a unitary matrix V_1 on entry,
        and the product V_1 V is returned.

        If `jobv` = 'I', `v` is initialized to the unit matrix, and the
        unitary matrix V is returned.

        If `jobv` = 'N', V is not computed.

    jobq : str, length 1
        If `jobq` = 'Q', `q` must contain a unitary matrix Q_1 on entry,
        and the product Q_1 Q is returned.

        If `jobq` = 'I', `q` is initialized to the unit matrix, and the
        unitary matrix Q is returned.

        If `jobq` = 'N', Q is not computed.

    k : int
        `k` and `l` specify the sizes, k and l, of the subblocks of A
        and B, whose GSVD is to be computed by ``ztgsja``.

    l : int
        `k` and `l` specify the sizes, k and l, of the subblocks of A
        and B, whose GSVD is to be computed by ``ztgsja``.

    a : complex, array-like, shape (m, n)
        The m*n matrix A.

    b : complex, array-like, shape (p, n)
        The p*n matrix B.

    tola : float
        `tola` and `tolb` are the convergence criteria for the
        Jacobi--Kogbetliantz iteration procedure. Generally, they should
        be the same as used in the preprocessing step performed by
        ``zggsvp3``, say

            `tola` = max(m, n)||A||epsilon,;`tolb` = max(p,
            n)||B||epsilon,

        where epsilon is the machine precision.

    tolb : float
        `tola` and `tolb` are the convergence criteria for the
        Jacobi--Kogbetliantz iteration procedure. Generally, they should
        be the same as used in the preprocessing step performed by
        ``zggsvp3``, say

            `tola` = max(m, n)||A||epsilon,;`tolb` = max(p,
            n)||B||epsilon,

        where epsilon is the machine precision.

    u : complex, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobu` in ('U', 'I'): m; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobu` in ('U', 'I'): m; otherwise: 1.

        If `jobu` = 'U', `u` must contain an m*m matrix U_1 (usually the
        unitary matrix returned by ``zggsvp3``).

    v : complex, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobv` in ('V', 'I'): p; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobv` in ('V', 'I'): p; otherwise: 1.

        If `jobv` = 'V', `v` must contain an p*p matrix V_1 (usually the
        unitary matrix returned by ``zggsvp3``).

    q : complex, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `jobq` in ('Q', 'I'): n; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `jobq` in ('Q', 'I'): n; otherwise: 1.

        If `jobq` = 'Q', `q` must contain an n*n matrix Q_1 (usually the
        unitary matrix returned by ``zggsvp3``).

    Returns
    -------
    a : complex, ndarray, shape (m, n)
        If m-k-l >= 0, `a`[0:k+l,n-k-l:n] contains the (k+l)*(k+l) upper
        triangular matrix R.

        If m-k-l < 0, `a`[0:m,n-k-l:n] contains the first m rows of the
        (k+l)*(k+l) upper triangular matrix R, and the submatrix R_33 is
        returned in `b`[m-k:l,n+m-k-l:n].

    b : complex, ndarray, shape (p, n)
        If m-k-l < 0, `b`[m-k:l,n+m-k-l:n] contains the submatrix R_33
        of R.

    alpha : float, ndarray, shape (n)
        See the description of `beta`.

    beta : float, ndarray, shape (n)
        `alpha` and `beta` contain the generalized singular value pairs
        of A and B;

            `alpha`[i] = 1, `beta`[i] = 0, for i = 0,...,k-1, and

            if m-k-l >= 0, `alpha`[i] = alpha_i, `beta`[i] = beta_i, for
            i = k,...,k+l-1, or

            if m-k-l < 0, `alpha`[i] = alpha_i, `beta`[i] = beta_i, for
            i = k,...,m-1 and `alpha`[i] = 0, `beta`[i] = 1, for i =
            m,...,k+l-1.

        Furthermore, if k+l < n, `alpha`[i] = `beta`[i] = 0, for i =
        k+l,...,n-1.

    u : complex, ndarray, shape (:, :)
        If `jobu` = 'U', `u` contains the product U_1 U.

        If `jobu` = 'I', `u` contains the unitary matrix U.

        If `jobu` = 'N', `u` is not referenced.

    v : complex, ndarray, shape (:, :)
        If `jobv` = 'I', `v` contains the unitary matrix V.

        If `jobv` = 'V', `v` contains the product V_1 V.

        If `jobv` = 'N', `v` is not referenced.

    q : complex, ndarray, shape (:, :)
        If `jobq` = 'I', `q` contains the unitary matrix Q.

        If `jobq` = 'Q', `q` contains the product Q_1 Q.

        If `jobq` = 'N', `q` is not referenced.

    ncycle : int
        The number of cycles required for convergence.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `jobu`.

            Constraint: `jobu` = 'U', 'I' or 'N'.

        (`errno` -2)
            On entry, error in parameter `jobv`.

            Constraint: `jobv` = 'V', 'I' or 'N'.

        (`errno` -3)
            On entry, error in parameter `jobq`.

            Constraint: `jobq` = 'Q', 'I' or 'N'.

        (`errno` -4)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -5)
            On entry, error in parameter p.

            Constraint: p >= 0.

        (`errno` -6)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` 1)
            The procedure does not converge after 40 cycles.

    Notes
    -----
    ``ztgsja`` computes the GSVD of the matrices A and B which are
    assumed to have the form as returned by ``zggsvp3``

        [table omitted]

    where the k*k matrix A_12 and the l*l matrix B_13 are nonsingular
    upper triangular, A_23 is l*l upper triangular if m-k-l >= 0 and is
    (m-k)*l upper trapezoidal otherwise.

    ``ztgsja`` computes unitary matrices Q, U and V, diagonal matrices
    D_1 and D_2, and an upper triangular matrix R such that

        U^HAQ = D_1(0 R), V^HBQ = D_2(0 R).

    Optionally Q, U and V may or may not be computed, or they may be
    premultiplied by matrices Q_1, U_1 and V_1 respectively.

    If (m-k-l) >= 0 then D_1, D_2 and R have the form

        D_1 = [table omitted],

        D_2 = [table omitted],

        R = [table omitted],

    where C = diag(alpha_{k+1},,, ...,,, alpha_{k+l}), S =
    diag(beta_{k+1},,, ...,,, beta_{k+l}).

    If (m-k-l) < 0 then D_1, D_2 and R have the form

        D_1 = [table omitted],

        D_2 = [table omitted],

        R = [table omitted],

    where C = diag(alpha_{k+1},,, ...,,, alpha_m), S =
    diag(beta_{k+1},,, ...,,, beta_m).

    In both cases the diagonal matrix C has real non-negative diagonal
    elements, the diagonal matrix S has real positive diagonal elements,
    so that S is nonsingular, and C^2+S^2 = 1.
    See Section 2.3.5.3 of Anderson `et al.` (1999) for further
    information.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore
    """

    fun_name = 'naginterfaces.library.lapackeig.ztgsja'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
    )
    _p_py = _b_utils._EngineComplex128ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[0]
    _p = _b_utils._EngineIntScalarType(
        _p_py, {'fun_name': fun_name, 'entity': '_p'},
    ).to_ctypes()
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _alpha = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _beta = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _u = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'u'},
        u,
        copy=True,
    )
    _v = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'v'},
        v,
        copy=True,
    )
    _q = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'q'},
        q,
        copy=True,
    )
    _return_scalar = _b_lapackeig.ztgsja(
        jobu,
        jobv,
        jobq,
        _m,
        _p,
        _n,
        k,
        l,
        _a,
        _b,
        tola,
        tolb,
        _alpha,
        _beta,
        _u,
        _v,
        _q,
    )
    _a_py = _a
    _b_py = _b
    _u_py = _u
    _v_py = _v
    _q_py = _q
    _alpha_py = _alpha
    _beta_py = _beta
    _ncycle_py = _return_scalar
    _return_data = ZtgsjaReturnData(
        _a_py,
        _b_py,
        _alpha_py,
        _beta_py,
        _u_py,
        _v_py,
        _q_py,
        _ncycle_py,
    )
    return _return_data

ZtgexcReturnData = _collections.namedtuple(
    'ZtgexcReturnData',
    ['a', 'b', 'q', 'z', 'ilst'],
)

def ztgexc(wantq, wantz, a, b, q, z, ifst, ilst):
    u"""
    Reorders the generalized Schur decomposition of a complex matrix
    pair using an unitary equivalence transformation.

    ``ztgexc`` reorders the generalized Schur factorization of a complex
    matrix pair in generalized Schur form.

    For full information please refer to the NAG Library document for
    f08yt

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08ytf.html

    Parameters
    ----------
    wantq : bool
        If `wantq` = True, update the left transformation matrix Q.

        If `wantq` = False, do not update Q.

    wantz : bool
        If `wantz` = True, update the right transformation matrix Z.

        If `wantz` = False, do not update Z.

    a : complex, array-like, shape (n, n)
        The matrix S in the pair (S,T).

    b : complex, array-like, shape (n, n)
        The matrix T, in the pair (S,T).

    q : complex, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `wantq`= True: n; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `wantq`= True: n; otherwise: 1.

        If `wantq` = True, the unitary matrix Q.

    z : complex, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `wantz`= True: n; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `wantz`= True: n; otherwise: 1.

        If `wantz` = True, the unitary matrix Z.

    ifst : int
        The indices i_1 and i_2 that specify the reordering of the
        diagonal elements of (S,T). The element with row index `ifst` is
        moved to row `ilst`, by a sequence of swapping between adjacent
        diagonal elements.

    ilst : int
        The indices i_1 and i_2 that specify the reordering of the
        diagonal elements of (S,T). The element with row index `ifst` is
        moved to row `ilst`, by a sequence of swapping between adjacent
        diagonal elements.

    Returns
    -------
    a : complex, ndarray, shape (n, n)
        The updated matrix S^.

    b : complex, ndarray, shape (n, n)
        The updated matrix T^

    q : complex, ndarray, shape (:, :)
        If `wantq` = True, the updated matrix QQ^.

        If `wantq` = False, `q` is not referenced.

    z : complex, ndarray, shape (:, :)
        If `wantz` = True, the updated matrix ZZ^.

        If `wantz` = False, `z` is not referenced.

    ilst : int
        `ilst` points to the row in its final position.

    Raises
    ------
    NagValueError
        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -13)
            On entry, error in parameter `ilst`.

            Constraint: 1 <= `ifst` <= n and 1 <= `ilst` <= n.

        (`errno` 1)
            The transformed matrix pair would be too far from
            generalized Schur form; the problem is ill-conditioned.
            (S,T) may have been partially reordered, and `ilst` points
            to the first row of the current position of the block being
            moved.

    Notes
    -----
    ``ztgexc`` reorders the generalized complex n*n matrix pair (S,T) in
    generalized Schur form, so that the diagonal element of (S,T) with
    row index i_1 is moved to row i_2, using a unitary equivalence
    transformation.
    That is, S and T are factorized as

        S = Q^S^Z^^H, T = Q^T^Z^^H,

    where (S^,T^) are also in generalized Schur form.

    The pair (S,T) are in generalized Schur form if S and T are upper
    triangular as returned, for example, by ``zgges3``, or ``zhgeqz``
    with job = 'S'.

    If S and T are the result of a generalized Schur factorization of a
    matrix pair (A,B)

        A = QSZ^H, B = QTZ^H

    then, optionally, the matrices Q and Z can be updated as QQ^ and
    ZZ^.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug
    """

    fun_name = 'naginterfaces.library.lapackeig.ztgexc'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
    )
    _q = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'q'},
        q,
        copy=True,
    )
    _z = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'z'},
        z,
        copy=True,
    )
    _return_scalar = _b_lapackeig.ztgexc(
        wantq,
        wantz,
        _n,
        _a,
        _b,
        _q,
        _z,
        ifst,
        ilst,
    )
    _a_py = _a
    _b_py = _b
    _q_py = _q
    _z_py = _z
    _ilst_py = _return_scalar
    _return_data = ZtgexcReturnData(
        _a_py,
        _b_py,
        _q_py,
        _z_py,
        _ilst_py,
    )
    return _return_data

ZtgsenReturnData = _collections.namedtuple(
    'ZtgsenReturnData',
    ['a', 'b', 'alpha', 'beta', 'q', 'z', 'm', 'pl', 'pr', 'dif'],
)

def ztgsen(ijob, wantq, wantz, select, a, b, q, z):
    u"""
    Reorders the generalized Schur decomposition of a complex matrix
    pair using an unitary equivalence transformation, computes the
    generalized eigenvalues of the reordered pair and, optionally,
    computes the estimates of reciprocal condition numbers for
    eigenvalues and eigenspaces.

    ``ztgsen`` reorders the generalized Schur factorization of a complex
    matrix pair in generalized Schur form, so that a selected cluster of
    eigenvalues appears in the leading elements on the diagonal of the
    generalized Schur form.
    The function also, optionally, computes the reciprocal condition
    numbers of the cluster of eigenvalues and/or corresponding deflating
    subspaces.

    For full information please refer to the NAG Library document for
    f08yu

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08yuf.html

    Parameters
    ----------
    ijob : int
        Specifies whether condition numbers are required for the cluster
        of eigenvalues (p and q) or the deflating subspaces (Dif_u and
        Dif_l).

        `ijob` = 0

            Only reorder with respect to `select`. No extras.

        `ijob` = 1

            Reciprocal of norms of 'projections' onto left and right
            eigenspaces with respect to the selected cluster (p and q).

        `ijob` = 2

            The upper bounds on Dif_u and Dif_l. F-norm-based estimate
            (`dif`[0:2]).

        `ijob` = 3

            Estimate of Dif_u and Dif_l. 1-norm-based estimate
            (`dif`[0:2]). About five times as expensive as `ijob` = 2.

        `ijob` = 4

            Compute `pl`, `pr` and `dif` as in `ijob` = 0, 1 or 2.
            Economic version to get it all.

        `ijob` = 5

            Compute `pl`, `pr` and `dif` as in `ijob` = 0, 1 or 3.

    wantq : bool
        If `wantq` = True, update the left transformation matrix Q.

        If `wantq` = False, do not update Q.

    wantz : bool
        If `wantz` = True, update the right transformation matrix Z.

        If `wantz` = False, do not update Z.

    select : bool, array-like, shape (n)
        Specifies the eigenvalues in the selected cluster. To select an
        eigenvalue lambda_j, `select`[j-1] must be set to True.

    a : complex, array-like, shape (n, n)
        The matrix S in the pair (S,T).

    b : complex, array-like, shape (n, n)
        The matrix T, in the pair (S,T).

    q : complex, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `wantq`= True: n; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `wantq`= True: n; otherwise: 1.

        If `wantq` = True, the n*n matrix Q.

    z : complex, array-like, shape (:, :)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `wantz`= True: n; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `wantz`= True: n; otherwise: 1.

        If `wantz` = True, the n*n matrix Z.

    Returns
    -------
    a : complex, ndarray, shape (n, n)
        The updated matrix S^.

    b : complex, ndarray, shape (n, n)
        The updated matrix T^

    alpha : complex, ndarray, shape (n)
        `alpha` and `beta` contain diagonal elements of S^ and T^,
        respectively, when the pair (S,T) has been reduced to
        generalized Schur form. `alpha`[i-1]/`beta`[i-1], for i =
        1,2,...,n, are the eigenvalues.

    beta : complex, ndarray, shape (n)
        `alpha` and `beta` contain diagonal elements of S^ and T^,
        respectively, when the pair (S,T) has been reduced to
        generalized Schur form. `alpha`[i-1]/`beta`[i-1], for i =
        1,2,...,n, are the eigenvalues.

    q : complex, ndarray, shape (:, :)
        If `wantq` = True, the updated matrix QQ^.

        If `wantq` = False, `q` is not referenced.

    z : complex, ndarray, shape (:, :)
        If `wantz` = True, the updated matrix ZZ^.

        If `wantz` = False, `z` is not referenced.

    m : int
        The dimension of the specified pair of left and right
        eigenspaces (deflating subspaces), where `m` will be in the
        range 0 <= `m` <= n.

    pl : float
        If `ijob` = 1, 4 or 5, `pl` and `pr` are lower bounds on the
        reciprocal of the norm of 'projections' p and q onto left and
        right eigenspace with respect to the selected cluster. 0 < `pl`,
        `pr` <= 1.

        If `m` = 0 or `m` = n, `pl` = `pr` = 1.

        If `ijob` = 0, 2 or 3, `pl` and `pr` are not referenced.

    pr : float
        If `ijob` = 1, 4 or 5, `pl` and `pr` are lower bounds on the
        reciprocal of the norm of 'projections' p and q onto left and
        right eigenspace with respect to the selected cluster. 0 < `pl`,
        `pr` <= 1.

        If `m` = 0 or `m` = n, `pl` = `pr` = 1.

        If `ijob` = 0, 2 or 3, `pl` and `pr` are not referenced.

    dif : float, ndarray, shape (2)
        If `ijob` >= 2, `dif`[0:2] store the estimates of Dif_u and
        Dif_l.

        If `ijob` = 2 or 4, `dif`[0:2] are F-norm-based upper bounds on
        Dif_u and Dif_l.

        If `ijob` = 3 or 5, `dif`[0:2] are 1-norm-based estimates of
        Dif_u and Dif_l.

        If `m` = 0 or n, `dif`[0:2] = ||(A,B)||_F.

        If `ijob` = 0 or 1, `dif` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `ijob`.

            Constraint: 0 <= `ijob` <= 5.

        (`errno` -5)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` 1)
            Reordering of (S,T) failed because the transformed matrix
            pair would be too far from generalized Schur form; the
            problem is very ill-conditioned. (S,T) may have been
            partially reordered. If requested, 0 is returned in `dif`[0]
            and `dif`[1], `pl` and `pr`.

    Notes
    -----
    ``ztgsen`` factorizes the generalized complex n*n matrix pair (S,T)
    in generalized Schur form, using a unitary equivalence
    transformation as

        S = Q^S^Z^^H, T = Q^T^Z^^H,

    where (S^,T^) are also in generalized Schur form and have the
    selected eigenvalues as the leading diagonal elements.
    The leading columns of Q and Z are the generalized Schur vectors
    corresponding to the selected eigenvalues and form orthonormal
    subspaces for the left and right eigenspaces (deflating subspaces)
    of the pair (S,T).

    The pair (S,T) are in generalized Schur form if S and T are upper
    triangular as returned, for example, by ``zgges3``, or ``zhgeqz``
    with job = 'S'.
    The diagonal elements define the generalized eigenvalues
    (alpha_i,beta_i), for i = 1,2,...,n, of the pair (S,T).
    The eigenvalues are given by

        lambda_i = alpha_i/beta_i,

    but are returned as the pair (alpha_i,beta_i) in order to avoid
    possible overflow in computing lambda_i.
    Optionally, the function returns reciprocals of condition number
    estimates for the selected eigenvalue cluster, p and q, the right
    and left projection norms, and of deflating subspaces, Dif_u and
    Dif_l.
    For more information see Sections 2.4.8 and 4.11 of Anderson `et
    al.` (1999).

    If S and T are the result of a generalized Schur factorization of a
    matrix pair (A,B)

        A = QSZ^H, B = QTZ^H

    then, optionally, the matrices Q and Z can be updated as QQ^ and
    ZZ^.
    Note that the condition numbers of the pair (S,T) are the same as
    those of the pair (A,B).

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug
    """

    fun_name = 'naginterfaces.library.lapackeig.ztgsen'

    _select = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'select'},
        select,
        copy=False,
    )
    _n_py = _b_utils._EngineLogicalArrayType.get_shape(_select, {'fun_name': fun_name, 'entity': 'select'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
    )
    _alpha = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_n_py,),
    )
    _beta = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_n_py,),
    )
    _q = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'q'},
        q,
        copy=True,
    )
    _z = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'z'},
        z,
        copy=True,
    )
    _dif = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (2,),
    )
    _return_scalars = _b_lapackeig.ztgsen(
        ijob,
        wantq,
        wantz,
        _select,
        _n,
        _a,
        _b,
        _alpha,
        _beta,
        _q,
        _z,
        _dif,
    )
    _a_py = _a
    _b_py = _b
    _q_py = _q
    _z_py = _z
    _alpha_py = _alpha
    _beta_py = _beta
    _m_py = _return_scalars.m
    _pl_py = _return_scalars.pl
    _pr_py = _return_scalars.pr
    _dif_py = _dif
    _return_data = ZtgsenReturnData(
        _a_py,
        _b_py,
        _alpha_py,
        _beta_py,
        _q_py,
        _z_py,
        _m_py,
        _pl_py,
        _pr_py,
        _dif_py,
    )
    return _return_data

ZtgsylReturnData = _collections.namedtuple(
    'ZtgsylReturnData',
    ['c', 'f', 'scale', 'dif'],
)

def ztgsyl(trans, ijob, a, b, c, d, e, f):
    u"""
    Solves the complex generalized Sylvester equation.

    ``ztgsyl`` solves the generalized complex triangular Sylvester
    equations.

    For full information please refer to the NAG Library document for
    f08yv

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08yvf.html

    Parameters
    ----------
    trans : str, length 1
        If `trans` = 'N', solve the generalized Sylvester equation (1).

        If `trans` = 'C', solve the 'conjugate transposed' system (2).

    ijob : int
        Specifies what kind of functionality is to be performed when
        `trans` = 'N'.

        `ijob` = 0

            Solve (1) only.

        `ijob` = 1

            The functionality of `ijob` = 0 or 3.

        `ijob` = 2

            The functionality of `ijob` = 0 or 4.

        `ijob` = 3

            Only an estimate of Dif[(A,D),(B,E)] is computed based on
            the Frobenius norm.

        `ijob` = 4

            Only an estimate of Dif[(A,D),(B,E)] is computed based on
            the 1-norm.

        If `trans` = 'C', `ijob` is not referenced.

    a : complex, array-like, shape (m, m)
        The upper triangular matrix A.

    b : complex, array-like, shape (n, n)
        The upper triangular matrix B.

    c : complex, array-like, shape (m, n)
        Contains the right-hand-side matrix C.

    d : complex, array-like, shape (m, m)
        The upper triangular matrix D.

    e : complex, array-like, shape (n, n)
        The upper triangular matrix E.

    f : complex, array-like, shape (m, n)
        Contains the right-hand side matrix F.

    Returns
    -------
    c : complex, ndarray, shape (m, n)
        If `ijob` = 0, 1 or 2, `c` is overwritten by the solution matrix
        R.

        If `trans` = 'N' and `ijob` = 3 or 4, `c` holds R, the solution
        achieved during the computation of the Dif estimate.

    f : complex, ndarray, shape (m, n)
        If `ijob` = 0, 1 or 2, `f` is overwritten by the solution matrix
        L.

        If `trans` = 'N' and `ijob` = 3 or 4, `f` holds L, the solution
        achieved during the computation of the Dif estimate.

    scale : float
        alpha, the scaling factor in (1) and (2).

        If 0 < `scale` < 1, `c` and `f` hold the solutions R and L,
        respectively, to a slightly perturbed system but the input
        arrays `a`, `b`, `d` and `e` have not been changed.

        If `scale` = 0, `c` and `f` hold the solutions R and L,
        respectively, to the homogeneous system with C = F = 0.

        In this case `dif` is not referenced.

        Normally, `scale` = 1.

    dif : float
        The estimate of Dif. If `ijob` = 0, `dif` is not referenced.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `trans`.

            Constraint: `trans` = 'N' or 'C'.

        (`errno` -2)
            On entry, error in parameter `ijob`.

            Constraint: 0 <= `ijob` <= 4.

        (`errno` -3)
            On entry, error in parameter m.

            Constraint: m > 0.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n > 0.

        (`errno` i > 0)
            (A,D) and (B,E) have common or close eigenvalues and so no
            solution could be computed.

    Notes
    -----
    ``ztgsyl`` solves either the generalized complex Sylvester equations

        [table omitted]

    or the equations

        [table omitted]

    where the pair (A,D) are given m*m matrices in generalized Schur
    form, (B,E) are given n*n matrices in generalized Schur form and
    (C,F) are given m*n matrices.
    The pair (R,L) are the m*n solution matrices, and alpha is an output
    scaling factor determined by the function to avoid overflow in
    computing (R,L).

    Equations (1) are equivalent to equations of the form

        Zx = alpha b,

    where

        Z = (I⊗A-B^H⊗I;I⊗D-E^H⊗I)

    and ⊗ is the Kronecker product.
    Equations (2) are then equivalent to

        Z^Hy = alpha b.

    The pair (S,T) are in generalized Schur form if S and T are upper
    triangular as returned, for example, by ``zgges3``, or ``zhgeqz``
    with job = 'S'.

    Optionally, the function estimates Dif[(A,D),(B,E)], the separation
    between the matrix pairs (A,D) and (B,E), which is the smallest
    singular value of Z.
    The estimate can be based on either the Frobenius norm, or the
    1-norm.
    The 1-norm estimate can be three to ten times more expensive than
    the Frobenius norm estimate, but makes the condition estimation
    uniform with the nonsymmetric eigenproblem.
    The Frobenius norm estimate provides a low cost, but equally
    reliable estimate.
    For more information see Sections 2.4.8.3 and 4.11.1.3 of Anderson
    `et al.` (1999) and Kågström and Poromaa (1996).

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Kågström, B, 1994, `A perturbation analysis of the generalized
    Sylvester equation (AR-LB,DR-LE) = (c,F)`, SIAM J. Matrix Anal.
    Appl. (15), 1045--1060

    Kågström, B and Poromaa, P, 1996, `LAPACK-style algorithms and
    software for solving the generalized Sylvester equation and
    estimating the separation between regular matrix pairs`, ACM Trans.
    Math. Software (22), 78--103
    """

    fun_name = 'naginterfaces.library.lapackeig.ztgsyl'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=False,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=True,
    )
    _d = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'd'},
        d,
        copy=False,
    )
    _e = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'e'},
        e,
        copy=False,
    )
    _f = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'f'},
        f,
        copy=True,
    )
    _return_scalars = _b_lapackeig.ztgsyl(
        trans,
        ijob,
        _m,
        _n,
        _a,
        _b,
        _c,
        _d,
        _e,
        _f,
    )
    _c_py = _c
    _f_py = _f
    _scale_py = _return_scalars.scale
    _dif_py = _return_scalars.dif
    _return_data = ZtgsylReturnData(
        _c_py,
        _f_py,
        _scale_py,
        _dif_py,
    )
    return _return_data

ZtgevcReturnData = _collections.namedtuple(
    'ZtgevcReturnData',
    ['vl', 'vr', 'm'],
)

def ztgevc(side, howmny, a, b, select=None, vl=None, vr=None):
    u"""
    Computes left and right eigenvectors of a pair of complex upper
    triangular matrices.

    ``ztgevc`` computes some or all of the right and/or left generalized
    eigenvectors of a pair of complex upper triangular matrices (A,B).

    For full information please refer to the NAG Library document for
    f08yx

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08yxf.html

    Parameters
    ----------
    side : str, length 1
        Specifies the required sets of generalized eigenvectors.

        `side` = 'R'

            Only right eigenvectors are computed.

        `side` = 'L'

            Only left eigenvectors are computed.

        `side` = 'B'

            Both left and right eigenvectors are computed.

    howmny : str, length 1
        Specifies further details of the required generalized
        eigenvectors.

        `howmny` = 'A'

            All right and/or left eigenvectors are computed.

        `howmny` = 'B'

            All right and/or left eigenvectors are computed; they are
            backtransformed using the input matrices supplied in arrays
            `vr` and/or `vl`.

        `howmny` = 'S'

            Selected right and/or left eigenvectors, defined by the
            array `select`, are computed.

    a : complex, array-like, shape (n, n)
        The matrix A must be in upper triangular form. Usually, this is
        the matrix A returned by ``zhgeqz``.

    b : complex, array-like, shape (n, n)
        The matrix B must be in upper triangular form with non-negative
        real diagonal elements. Usually, this is the matrix B returned
        by ``zhgeqz``.

    select : None or bool, array-like, shape (:), optional
        Note: the required length for this argument is determined as
        follows: if `howmny`='S': n; otherwise: 1.

        Specifies the eigenvectors to be computed if `howmny` = 'S'. To
        select the generalized eigenvector corresponding to the jth
        generalized eigenvalue, the jth element of `select` should be
        set to True.

    vl : None or complex, array-like, shape (:, :), optional
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `side` in ('L', 'B'): n; if
        `side`='R': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `side` in ('L', 'B'): mm; if
        `side`='R': 1; otherwise: 0.

        If `howmny` = 'B' and `side` = 'L' or 'B', `vl` must be
        initialized to an n*n matrix Q. Usually, this is the unitary
        matrix Q of left Schur vectors returned by ``zhgeqz``.

    vr : None or complex, array-like, shape (:, :), optional
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `side` in ('R', 'B'): n; if
        `side`='L': 1; otherwise: 0.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `side` in ('R', 'B'): mm; if
        `side`='L': 1; otherwise: 0.

        If `howmny` = 'B' and `side` = 'R' or 'B', `vr` must be
        initialized to an n*n matrix Z. Usually, this is the unitary
        matrix Z of right Schur vectors returned by ``dhgeqz``.

    Returns
    -------
    vl : None or complex, ndarray, shape (:, :)
        If `side` = 'L' or 'B', `vl` contains:

            if `howmny` = 'A', the matrix Y of left eigenvectors of
            (A,B);

            if `howmny` = 'B', the matrix QY;

            if `howmny` = 'S', the left eigenvectors of (A,B) specified
            by `select`, stored consecutively in the columns of the
            array `vl`, in the same order as their corresponding
            eigenvalues.

    vr : None or complex, ndarray, shape (:, :)
        If `side` = 'R' or 'B', `vr` contains:

            if `howmny` = 'A', the matrix X of right eigenvectors of
            (A,B);

            if `howmny` = 'B', the matrix ZX;

            if `howmny` = 'S', the right eigenvectors of (A,B) specified
            by `select`, stored consecutively in the columns of the
            array `vr`, in the same order as their corresponding
            eigenvalues.

    m : int
        The number of columns in the arrays `vl` and/or `vr` actually
        used to store the eigenvectors. If `howmny` = 'A' or 'B', `m` is
        set to n. Each selected eigenvector occupies one column.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `side`.

            Constraint: `side` = 'B', 'L' or 'R'.

        (`errno` -2)
            On entry, error in parameter `howmny`.

            Constraint: `howmny` = 'A', 'B' or 'S'.

        (`errno` -3)
            On entry, error in parameter `select`.

            Constraint: `select`[j] = True or False, for j = 0,...,n-1.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -7)
            On entry, error in parameter `b`.

        (`errno` -13)
            On entry, error in parameter mm.

    Notes
    -----
    ``ztgevc`` computes some or all of the right and/or left generalized
    eigenvectors of the matrix pair (A,B) which is assumed to be in
    upper triangular form.
    If the matrix pair (A,B) is not upper triangular then the function
    ``zhgeqz`` should be called before invoking ``ztgevc``.

    The right generalized eigenvector x and the left generalized
    eigenvector y of (A,B) corresponding to a generalized eigenvalue
    lambda are defined by

        (A-lambda B)x = 0

    and

        y^H(A-lambda B) = 0.

    If a generalized eigenvalue is determined as 0/0, which is due to
    zero diagonal elements at the same locations in both A and B, a unit
    vector is returned as the corresponding eigenvector.

    Note that the generalized eigenvalues are computed using ``zhgeqz``
    but ``ztgevc`` does not explicitly require the generalized
    eigenvalues to compute eigenvectors.
    The ordering of the eigenvectors is based on the ordering of the
    eigenvalues as computed by ``ztgevc``.

    If all eigenvectors are requested, the function may either return
    the matrices X and/or Y of right or left eigenvectors of (A,B), or
    the products ZX and/or QY, where Z and Q are two matrices supplied
    by you.
    Usually, Q and Z are chosen as the unitary matrices returned by
    ``zhgeqz``.
    Equivalently, Q and Z are the left and right Schur vectors of the
    matrix pair supplied to ``zhgeqz``.
    In that case, QY and ZX are the left and right generalized
    eigenvectors, respectively, of the matrix pair supplied to
    ``zhgeqz``.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia

    Golub, G H and Van Loan, C F, 1996, `Matrix Computations`, (3rd
    Edition), Johns Hopkins University Press, Baltimore

    Moler, C B and Stewart, G W, 1973, `An algorithm for generalized
    matrix eigenproblems`, SIAM J. Numer. Anal. (10), 241--256

    Stewart, G W and Sun, J-G, 1990, `Matrix Perturbation Theory`,
    Academic Press, London
    """

    fun_name = 'naginterfaces.library.lapackeig.ztgevc'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
    )
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=False,
    )
    _vl = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'vl'},
        vl,
        copy=True,
        allow_none=True,
    )
    _vr = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'vr'},
        vr,
        copy=True,
        allow_none=True,
    )
    _m_sorder = _b_utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((_a, 'a'), (_b, 'b'), (_vl, 'vl'), (_vr, 'vr')),
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _select = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'select'},
        select,
        copy=False,
        allow_none=True,
    )
    _vl_m_py = 0
    for _vl_j_py in range(1, _n_py+1):
        if _b_utils._EngineLogicalArrayType.get_element(_select, {'fun_name': fun_name, 'entity': 'select'}, (_vl_j_py-1,)):
            _vl_m_py = _vl_m_py + 1
    if _b_utils._EngineComplex128ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[0] >= _n_py:
        _vl_ilbbad_py = False
        for _vl_j_py in range(1, _n_py+1):
            if _b_utils._EngineComplex128ArrayType.get_element(_b, {'fun_name': fun_name, 'entity': 'b'}, (_vl_j_py-1, _vl_j_py-1)).imag != 0:
                _vl_ilbbad_py = True
                break
        _vl_ilbbad = _b_utils._EngineLogicalScalarType(
            _vl_ilbbad_py, {'fun_name': fun_name, 'entity': '_vl_ilbbad_py'},
        ).to_ctypes()
    else:
        _vl_ilbbad_py = False
    _howmny = _b_utils._EngineCharScalarType(
        howmny, {'fun_name': fun_name, 'entity': 'howmny'}, exp_chrlen=1,
    ).to_ctypes()
    _howmny_py = _b_utils._EngineCharScalarType.to_py(_howmny)
    _side = _b_utils._EngineCharScalarType(
        side, {'fun_name': fun_name, 'entity': 'side'}, exp_chrlen=1,
    ).to_ctypes()
    _side_py = _b_utils._EngineCharScalarType.to_py(_side)
    if (_howmny_py in ('B', 'b')) and _side_py not in ('R', 'r'):
        if vl is None:
            raise _b_utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'vl'},
                'must not be None when (howmny in (\'B\', \'b\')) and side not in (\'R\', \'r\')',
            )
    if _howmny_py in ('S', 's'):
        _mm = _b_utils._EngineIntScalarType(
            _vl_m_py, {'fun_name': fun_name, 'entity': '_mm'},
        ).to_ctypes()
        _mm_py = _b_utils._EngineIntScalarType.to_py(_mm)
    else:
        _mm = _b_utils._EngineIntScalarType(
            _n_py, {'fun_name': fun_name, 'entity': '_mm'},
        ).to_ctypes()
        _mm_py = _b_utils._EngineIntScalarType.to_py(_mm)
    if vl is None:
        if _side_py in ('L', 'l', 'B', 'b'):
            _vl_dim1 = _n_py
        elif _side_py in ('R', 'r'):
            _vl_dim1 = 1
        else:
            _vl_dim1 = 0
        if _side_py in ('L', 'l', 'B', 'b'):
            _vl_dim2 = _mm_py
        elif _side_py in ('R', 'r'):
            _vl_dim2 = 1
        else:
            _vl_dim2 = 0
        exp_shape_vl = (_vl_dim1, _vl_dim2)
        _vl = _b_utils._EngineComplex128ArrayType.empty_ndarray(
            exp_shape_vl,
            sorder=_m_sorder,
        )
    if (_howmny_py in ('B', 'b')) and _side_py not in ('L', 'l'):
        if vr is None:
            raise _b_utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'vr'},
                'must not be None when (howmny in (\'B\', \'b\')) and side not in (\'L\', \'l\')',
            )
    if vr is None:
        if _side_py in ('R', 'r', 'B', 'b'):
            _vr_dim1 = _n_py
        elif _side_py in ('L', 'l'):
            _vr_dim1 = 1
        else:
            _vr_dim1 = 0
        if _side_py in ('R', 'r', 'B', 'b'):
            _vr_dim2 = _mm_py
        elif _side_py in ('L', 'l'):
            _vr_dim2 = 1
        else:
            _vr_dim2 = 0
        exp_shape_vr = (_vr_dim1, _vr_dim2)
        _vr = _b_utils._EngineComplex128ArrayType.empty_ndarray(
            exp_shape_vr,
            sorder=_m_sorder,
        )
    _return_scalar = _b_lapackeig.ztgevc(
        side,
        howmny,
        _select,
        _n,
        _a,
        _b,
        _vl,
        _vr,
        _mm,
    )
    if _side_py not in ('R', 'r'):
        _vl_py = _vl
    else:
        _vl_py = None
    if _side_py not in ('L', 'l'):
        _vr_py = _vr
    else:
        _vr_py = None
    _m_py = _return_scalar
    _return_data = ZtgevcReturnData(
        _vl_py,
        _vr_py,
        _m_py,
    )
    return _return_data

ZtgsnaReturnData = _collections.namedtuple(
    'ZtgsnaReturnData',
    ['s', 'dif', 'm'],
)

def ztgsna(job, howmny, a, b, mm, select=None, vl=None, vr=None):
    u"""
    Estimates reciprocal condition numbers for specified eigenvalues
    and/or eigenvectors of a complex matrix pair in generalized Schur
    canonical form.

    ``ztgsna`` estimates condition numbers for specified eigenvalues
    and/or eigenvectors of a complex matrix pair in generalized Schur
    form.

    For full information please refer to the NAG Library document for
    f08yy

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08yyf.html

    Parameters
    ----------
    job : str, length 1
        Indicates whether condition numbers are required for eigenvalues
        and/or eigenvectors.

        `job` = 'E'

            Condition numbers for eigenvalues only are computed.

        `job` = 'V'

            Condition numbers for eigenvectors only are computed.

        `job` = 'B'

            Condition numbers for both eigenvalues and eigenvectors are
            computed.

    howmny : str, length 1
        Indicates how many condition numbers are to be computed.

        `howmny` = 'A'

            Condition numbers for all eigenpairs are computed.

        `howmny` = 'S'

            Condition numbers for selected eigenpairs (as specified by
            `select`) are computed.

    a : complex, array-like, shape (n, n)
        The upper triangular matrix S.

    b : complex, array-like, shape (n, n)
        The upper triangular matrix T.

    mm : int
        The number of elements in the arrays `s` and `dif`.

    select : None or bool, array-like, shape (:), optional
        Note: the required length for this argument is determined as
        follows: if `howmny`='S': n; otherwise: 1.

        Specifies the eigenpairs for which condition numbers are to be
        computed if `howmny` = 'S'. To select condition numbers for the
        eigenpair corresponding to the eigenvalue lambda_j,
        `select`[j-1] must be set to True.

        If `howmny` = 'A', `select` is not referenced.

    vl : None or complex, array-like, shape (:, :), optional
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `job` in ('E', 'B'): n; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `job` in ('E', 'B'): `mm`; otherwise:
        0.

        If `job` = 'E' or 'B', `vl` must contain left eigenvectors of
        (S,T), corresponding to the eigenpairs specified by `howmny` and
        `select`. The eigenvectors must be stored in consecutive columns
        of `vl`, as returned by ``zggev3`` or ``ztgevc``.

        If `job` = 'V', `vl` is not referenced.

    vr : None or complex, array-like, shape (:, :), optional
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `job` in ('E', 'B'): n; otherwise: 1.

        Note: the required extent for this argument in dimension 2 is
        determined as follows: if `job` in ('E', 'B'): `mm`; otherwise:
        0.

        If `job` = 'E' or 'B', `vr` must contain right eigenvectors of
        (S,T), corresponding to the eigenpairs specified by `howmny` and
        `select`. The eigenvectors must be stored in consecutive columns
        of `vr`, as returned by ``zggev3`` or ``ztgevc``.

        If `job` = 'V', `vr` is not referenced.

    Returns
    -------
    s : float, ndarray, shape (:)
        If `job` = 'E' or 'B', the reciprocal condition numbers of the
        selected eigenvalues, stored in consecutive elements of the
        array.

        If `job` = 'V', `s` is not referenced.

    dif : float, ndarray, shape (:)
        If `job` = 'V' or 'B', the estimated reciprocal condition
        numbers of the selected eigenvectors, stored in consecutive
        elements of the array. If the eigenvalues cannot be reordered to
        compute `dif`[j-1], `dif`[j-1] is set to 0; this can only occur
        when the true value would be very small anyway.

        If `job` = 'E', `dif` is not referenced.

    m : int
        The number of elements of the arrays `s` and `dif` used to store
        the specified condition numbers; for each selected eigenvalue
        one element is used.

        If `howmny` = 'A', `m` is set to n.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter `job`.

            Constraint: `job` = 'E', 'V' or 'B'.

        (`errno` -2)
            On entry, error in parameter `howmny`.

            Constraint: `howmny` = 'A' or 'S'.

        (`errno` -4)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -15)
            On entry, error in parameter `mm`.

            Constraint: `mm` >= n.

        (`errno` -15)
            On entry, error in parameter `mm`.

            Constraint: `mm` >= the number of selected eigenvalues.

    Notes
    -----
    ``ztgsna`` estimates condition numbers for specified eigenvalues
    and/or right eigenvectors of an n*n matrix pair (S,T) in generalized
    Schur form.
    The function actually returns estimates of the reciprocals of the
    condition numbers in order to avoid possible overflow.

    The pair (S,T) are in generalized Schur form if S and T are upper
    triangular as returned, for example, by ``zgges3`` or ``zggesx``, or
    ``zhgeqz`` with job = 'S'.
    The diagonal elements define the generalized eigenvalues
    (alpha_i,beta_i), for i = 1,2,...,n, of the pair (S,T) and the
    eigenvalues are given by

        lambda_i = alpha_i/beta_i,

    so that

        beta_i Sx_i = alpha_i Tx_i or Sx_i = lambda_i Tx_i,

    where x_i is the corresponding (right) eigenvector.

    If S and T are the result of a generalized Schur factorization of a
    matrix pair (A,B)

        A = QSZ^H, B = QTZ^H

    then the eigenvalues and condition numbers of the pair (S,T) are the
    same as those of the pair (A,B).

    Let (alpha,beta) != (0,0) be a simple generalized eigenvalue of
    (A,B).
    Then the reciprocal of the condition number of the eigenvalue lambda
    = alpha/beta is defined as

        s(lambda) = ((|y^HAx|^2+|y^HBx|^2)^(1/2))/((||x||_2||y||_2)),

    where x and y are the right and left eigenvectors of (A,B)
    corresponding to lambda.
    If both alpha and beta are zero, then (A,B) is singular and
    s(lambda) = -1 is returned.

    If U and V are unitary transformations such that

        U^H(A,B)V = (S,T) = [table omitted][table omitted],

    where S_22 and T_22 are (n-1)*(n-1) matrices, then the reciprocal
    condition number is given by

        Dif(x) = Dif(y) = Dif((alpha,beta),(S_22,T_22)) = sigma_min(Z),

    where sigma_min(Z) denotes the smallest singular value of the
    2*(n-1)*2*(n-1) matrix

        Z = [table omitted]

    and ⊗ is the Kronecker product.

    See Sections 2.4.8 and 4.11 of Anderson `et al.` (1999) and Kågström
    and Poromaa (1996) for further details and information.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Kågström, B and Poromaa, P, 1996, `LAPACK-style algorithms and
    software for solving the generalized Sylvester equation and
    estimating the separation between regular matrix pairs`, ACM Trans.
    Math. Software (22), 78--103
    """

    fun_name = 'naginterfaces.library.lapackeig.ztgsna'

    _select = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'select'},
        select,
        copy=False,
        allow_none=True,
    )
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=False,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=False,
    )
    _vl = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'vl'},
        vl,
        copy=False,
        allow_none=True,
    )
    _vr = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'vr'},
        vr,
        copy=False,
        allow_none=True,
    )
    _job = _b_utils._EngineCharScalarType(
        job, {'fun_name': fun_name, 'entity': 'job'}, exp_chrlen=1,
    ).to_ctypes()
    _job_py = _b_utils._EngineCharScalarType.to_py(_job)
    _mm = _b_utils._EngineIntScalarType(
        mm, {'fun_name': fun_name, 'entity': 'mm'},
    ).to_ctypes()
    _mm_py = _b_utils._EngineIntScalarType.to_py(_mm)
    if _job_py in ('E', 'e', 'B', 'b'):
        _s_dim1 = _mm_py
    else:
        _s_dim1 = 0
    exp_shape_s = (_s_dim1,)
    _s = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_s,
    )
    if _job_py in ('V', 'v', 'B', 'b'):
        _dif_dim1 = _mm_py
    else:
        _dif_dim1 = 0
    exp_shape_dif = (_dif_dim1,)
    _dif = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        exp_shape_dif,
    )
    _return_scalar = _b_lapackeig.ztgsna(
        job,
        howmny,
        _select,
        _n,
        _a,
        _b,
        _vl,
        _vr,
        _s,
        _dif,
        _mm,
    )
    _s_py = _s
    _dif_py = _dif
    _m_py = _return_scalar
    _return_data = ZtgsnaReturnData(
        _s_py,
        _dif_py,
        _m_py,
    )
    return _return_data

DgglseReturnData = _collections.namedtuple(
    'DgglseReturnData',
    ['a', 'b', 'c', 'd', 'x'],
)

def dgglse(n, a, b, c, d):
    u"""
    Solves the real linear equality-constrained least squares (LSE)
    problem.

    ``dgglse`` solves a real linear equality-constrained least squares
    problem.

    For full information please refer to the NAG Library document for
    f08za

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08zaf.html

    Parameters
    ----------
    n : int
        n, the number of columns of the matrices A and B.

    a : float, array-like, shape (m, `n`)
        The m*n matrix A.

    b : float, array-like, shape (p, `n`)
        The p*n matrix B.

    c : float, array-like, shape (m)
        The right-hand side vector c for the least squares part of the
        LSE problem.

    d : float, array-like, shape (p)
        The right-hand side vector d for the equality constraints.

    Returns
    -------
    a : float, ndarray, shape (m, `n`)
        `a` is overwritten.

    b : float, ndarray, shape (p, `n`)
        `b` is overwritten.

    c : float, ndarray, shape (m)
        The residual sum of squares for the solution vector x is given
        by the sum of squares of elements
        `c`[`n`-p],`c`[`n`-p+1],...,`c`[m-1]; the remaining elements are
        overwritten.

    d : float, ndarray, shape (p)
        `d` is overwritten.

    x : float, ndarray, shape (`n`)
        The solution vector x of the LSE problem.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter p.

            Constraint: 0 <= p <= `n` <= m+p.

        (`errno` 1)
            The upper triangular factor R associated with B in the
            generalized RQ factorization of the pair (B,A) is singular,
            so that rank(B) < p; the least squares solution could not be
            computed.

        (`errno` 2)
            The (N-P)*(N-P) part of the upper trapezoidal factor T
            associated with A in the generalized RQ factorization of the
            pair (B,A) is singular, so that the rank of the matrix (E)
            comprising the rows of A and B is less than n; the least
            squares solutions could not be computed.

    Notes
    -----
    ``dgglse`` solves the real linear equality-constrained least squares
    (LSE) problem

        minimize_x||c-Ax||_2 subject to Bx = d

    where A is an m*n matrix, B is a p*n matrix, c is an m element
    vector and d is a p element vector.
    It is assumed that p <= n <= m+p, rank(B) = p and rank(E) = n, where
    E = (A;B).
    These conditions ensure that the LSE problem has a unique solution,
    which is obtained using a generalized RQ factorization of the
    matrices B and A.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia

    Anderson, E, Bai, Z and Dongarra, J, 1992, `Generalized QR
    factorization and its applications`, Linear Algebra Appl. (Volume
    162--164), 243--271

    Eldèn, L, 1980, `Perturbation theory for the least squares problem
    with linear equality constraints`, SIAM J. Numer. Anal. (17),
    338--350
    """

    fun_name = 'naginterfaces.library.lapackeig.dgglse'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _p_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[0]
    _p = _b_utils._EngineIntScalarType(
        _p_py, {'fun_name': fun_name, 'entity': '_p'},
    ).to_ctypes()
    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _d = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'd'},
        d,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _x = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _b_lapackeig.dgglse(
        _m,
        _n,
        _p,
        _a,
        _b,
        _c,
        _d,
        _x,
    )
    _a_py = _a
    _b_py = _b
    _c_py = _c
    _d_py = _d
    _x_py = _x
    _return_data = DgglseReturnData(
        _a_py,
        _b_py,
        _c_py,
        _d_py,
        _x_py,
    )
    return _return_data

DggglmReturnData = _collections.namedtuple(
    'DggglmReturnData',
    ['a', 'b', 'd', 'x', 'y'],
)

def dggglm(p, a, b, d):
    u"""
    Solves a real general Gauss--Markov linear model (GLM) problem.

    ``dggglm`` solves a real general Gauss--Markov linear (least
    squares) model problem.

    For full information please refer to the NAG Library document for
    f08zb

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08zbf.html

    Parameters
    ----------
    p : int
        p, the number of columns of the matrix B.

    a : float, array-like, shape (m, n)
        The m*n matrix A.

    b : float, array-like, shape (m, `p`)
        The m*p matrix B.

    d : float, array-like, shape (m)
        The left-hand side vector d of the GLM equation.

    Returns
    -------
    a : float, ndarray, shape (m, n)
        `a` is overwritten.

    b : float, ndarray, shape (m, `p`)
        `b` is overwritten.

    d : float, ndarray, shape (m)
        `d` is overwritten.

    x : float, ndarray, shape (n)
        The solution vector x of the GLM problem.

    y : float, ndarray, shape (`p`)
        The solution vector y of the GLM problem.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: 0 <= n <= m.

        (`errno` -3)
            On entry, error in parameter `p`.

            Constraint: `p` >= m-n.

        (`errno` 1)
            The upper triangular factor R associated with A in the
            generalized RQ factorization of the pair (A,B) is singular,
            so that rank(A) < n; the least squares solution could not be
            computed.

        (`errno` 2)
            The bottom (m-n)*(m-n) part of the upper trapezoidal factor
            T associated with B in the generalized QR factorization of
            the pair (A,B) is singular, so that rank(A B) < n; the least
            squares solutions could not be computed.

    Notes
    -----
    ``dggglm`` solves the real general Gauss--Markov linear model (GLM)
    problem

        minimize_x||y||_2 subject to d = Ax+By

    where A is an m*n matrix, B is an m*p matrix and d is an m element
    vector.
    It is assumed that n <= m <= n+p, rank(A) = n and rank(E) = m, where
    E = (A B).
    Under these assumptions, the problem has a unique solution x and a
    minimal 2-norm solution y, which is obtained using a generalized QR
    factorization of the matrices A and B.

    In particular, if the matrix B is square and nonsingular, then the
    GLM problem is equivalent to the weighted linear least squares
    problem

        minimize_x||B^-1(d-Ax)||_2.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia

    Anderson, E, Bai, Z and Dongarra, J, 1992, `Generalized QR
    factorization and its applications`, Linear Algebra Appl. (Volume
    162--164), 243--271
    """

    fun_name = 'naginterfaces.library.lapackeig.dggglm'

    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _d = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'd'},
        d,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _x = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _p = _b_utils._EngineIntScalarType(
        p, {'fun_name': fun_name, 'entity': 'p'},
    ).to_ctypes()
    _p_py = _b_utils._EngineIntScalarType.to_py(_p)
    _y = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_p_py,),
    )
    _b_lapackeig.dggglm(
        _m,
        _n,
        _p,
        _a,
        _b,
        _d,
        _x,
        _y,
    )
    _a_py = _a
    _b_py = _b
    _d_py = _d
    _x_py = _x
    _y_py = _y
    _return_data = DggglmReturnData(
        _a_py,
        _b_py,
        _d_py,
        _x_py,
        _y_py,
    )
    return _return_data

DggqrfReturnData = _collections.namedtuple(
    'DggqrfReturnData',
    ['a', 'taua', 'b', 'taub'],
)

def dggqrf(a, b):
    u"""
    Computes a generalized QR factorization of a real matrix pair.

    ``dggqrf`` computes a generalized QR factorization of a real matrix
    pair (A,B), where A is an n*m matrix and B is an n*p matrix.

    For full information please refer to the NAG Library document for
    f08ze

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08zef.html

    Parameters
    ----------
    a : float, array-like, shape (n, m)
        The n*m matrix A.

    b : float, array-like, shape (n, p)
        The n*p matrix B.

    Returns
    -------
    a : float, ndarray, shape (n, m)
        The elements on and above the diagonal of the array contain the
        min(n, m)*m upper trapezoidal matrix R (R is upper triangular if
        n >= m); the elements below the diagonal, with the array `taua`,
        represent the orthogonal matrix Q as a product of min(n, m)
        elementary reflectors (see [the F08 Introduction]).

    taua : float, ndarray, shape (min(n, m))
        The scalar factors of the elementary reflectors which represent
        the orthogonal matrix Q.

    b : float, ndarray, shape (n, p)
        If n <= p, the upper triangle of the subarray `b`[0:n,p-n:p]
        contains the n*n upper triangular matrix T_12.

        If n > p, the elements on and above the (n-p)th subdiagonal
        contain the n*p upper trapezoidal matrix T; the remaining
        elements, with the array `taub`, represent the orthogonal matrix
        Z as a product of elementary reflectors (see [the F08
        Introduction]).

    taub : float, ndarray, shape (min(n, p))
        The scalar factors of the elementary reflectors which represent
        the orthogonal matrix Z.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -2)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -3)
            On entry, error in parameter p.

            Constraint: p >= 0.

    Notes
    -----
    ``dggqrf`` forms the generalized QR factorization of an n*m matrix A
    and an n*p matrix B

        A = QR, B = QTZ,

    where Q is an n*n orthogonal matrix, Z is a p*p orthogonal matrix
    and R and T are of the form

        R = { [table omitted], if n >= m; ; [table omitted], if n < m,

    with R_11 upper triangular,

        T = { [table omitted], if n <= p, ; [table omitted], if n > p,

    with T_12 or T_21 upper triangular.

    In particular, if B is square and nonsingular, the generalized QR
    factorization of A and B implicitly gives the QR factorization of
    B^-1A as

        B^-1A = Z^T(T^-1R).

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Anderson, E, Bai, Z and Dongarra, J, 1992, `Generalized QR
    factorization and its applications`, Linear Algebra Appl. (Volume
    162--164), 243--271

    Hammarling, S, 1987, `The numerical solution of the general
    Gauss-Markov linear model`, Mathematics in Signal Processing, (eds T
    S Durrani, J B Abbiss, J E Hudson, R N Madan, J G McWhirter and T A
    Moore), 441--456, Oxford University Press

    Paige, C C, 1990, `Some aspects of generalized QR factorizations`, .
    In Reliable Numerical Computation, (eds M G Cox and S Hammarling),
    73--91, Oxford University Press
    """

    fun_name = 'naginterfaces.library.lapackeig.dggqrf'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _p_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[1]
    _p = _b_utils._EngineIntScalarType(
        _p_py, {'fun_name': fun_name, 'entity': '_p'},
    ).to_ctypes()
    _taua = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (min(_n_py, _m_py),),
    )
    _taub = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (min(_n_py, _p_py),),
    )
    _b_lapackeig.dggqrf(
        _n,
        _m,
        _p,
        _a,
        _taua,
        _b,
        _taub,
    )
    _a_py = _a
    _b_py = _b
    _taua_py = _taua
    _taub_py = _taub
    _return_data = DggqrfReturnData(
        _a_py,
        _taua_py,
        _b_py,
        _taub_py,
    )
    return _return_data

DggrqfReturnData = _collections.namedtuple(
    'DggrqfReturnData',
    ['a', 'taua', 'b', 'taub'],
)

def dggrqf(a, b):
    u"""
    Computes a generalized RQ factorization of a real matrix pair.

    ``dggrqf`` computes a generalized RQ factorization of a real matrix
    pair (A,B), where A is an m*n matrix and B is a p*n matrix.

    For full information please refer to the NAG Library document for
    f08zf

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08zff.html

    Parameters
    ----------
    a : float, array-like, shape (m, n)
        The m*n matrix A.

    b : float, array-like, shape (p, n)
        The p*n matrix B.

    Returns
    -------
    a : float, ndarray, shape (m, n)
        If m <= n, the upper triangle of the subarray `a`[0:m,n-m:n]
        contains the m*m upper triangular matrix R_12.

        If m >= n, the elements on and above the (m-n)th subdiagonal
        contain the m*n upper trapezoidal matrix R; the remaining
        elements, with the array `taua`, represent the orthogonal matrix
        Q as a product of min(m, n) elementary reflectors (see [the F08
        Introduction]).

    taua : float, ndarray, shape (min(m, n))
        The scalar factors of the elementary reflectors which represent
        the orthogonal matrix Q.

    b : float, ndarray, shape (p, n)
        The elements on and above the diagonal of the array contain the
        min(p, n)*n upper trapezoidal matrix T (T is upper triangular if
        p >= n); the elements below the diagonal, with the array `taub`,
        represent the orthogonal matrix Z as a product of elementary
        reflectors (see [the F08 Introduction]).

    taub : float, ndarray, shape (min(p, n))
        The scalar factors of the elementary reflectors which represent
        the orthogonal matrix Z.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -2)
            On entry, error in parameter p.

            Constraint: p >= 0.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

    Notes
    -----
    ``dggrqf`` forms the generalized RQ factorization of an m*n matrix A
    and a p*n matrix B

        A = RQ, B = ZTQ,

    where Q is an n*n orthogonal matrix, Z is a p*p orthogonal matrix
    and R and T are of the form

        R = { [table omitted] ; if m <= n, ; [table omitted] ; if m > n,

    with R_12 or R_21 upper triangular,

        T = { [table omitted] ; if p >= n, ; [table omitted] ; if p < n,

    with T_11 upper triangular.

    In particular, if B is square and nonsingular, the generalized RQ
    factorization of A and B implicitly gives the RQ factorization of
    AB^-1 as

        AB^-1 = (RT^-1)Z^T.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Anderson, E, Bai, Z and Dongarra, J, 1992, `Generalized QR
    factorization and its applications`, Linear Algebra Appl. (Volume
    162--164), 243--271

    Hammarling, S, 1987, `The numerical solution of the general
    Gauss-Markov linear model`, Mathematics in Signal Processing, (eds T
    S Durrani, J B Abbiss, J E Hudson, R N Madan, J G McWhirter and T A
    Moore), 441--456, Oxford University Press

    Paige, C C, 1990, `Some aspects of generalized QR factorizations`, .
    In Reliable Numerical Computation, (eds M G Cox and S Hammarling),
    73--91, Oxford University Press
    """

    fun_name = 'naginterfaces.library.lapackeig.dggrqf'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _p_py = _b_utils._EngineFloat64ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[0]
    _p = _b_utils._EngineIntScalarType(
        _p_py, {'fun_name': fun_name, 'entity': '_p'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _taua = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (min(_m_py, _n_py),),
    )
    _taub = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (min(_p_py, _n_py),),
    )
    _b_lapackeig.dggrqf(
        _m,
        _p,
        _n,
        _a,
        _taua,
        _b,
        _taub,
    )
    _a_py = _a
    _b_py = _b
    _taua_py = _taua
    _taub_py = _taub
    _return_data = DggrqfReturnData(
        _a_py,
        _taua_py,
        _b_py,
        _taub_py,
    )
    return _return_data

ZgglseReturnData = _collections.namedtuple(
    'ZgglseReturnData',
    ['a', 'b', 'c', 'd', 'x'],
)

def zgglse(n, a, b, c, d):
    u"""
    Solves the complex linear equality-constrained least squares (LSE)
    problem.

    ``zgglse`` solves a complex linear equality-constrained least
    squares problem.

    For full information please refer to the NAG Library document for
    f08zn

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08znf.html

    Parameters
    ----------
    n : int
        n, the number of columns of the matrices A and B.

    a : complex, array-like, shape (m, `n`)
        The m*n matrix A.

    b : complex, array-like, shape (p, `n`)
        The p*n matrix B.

    c : complex, array-like, shape (m)
        The right-hand side vector c for the least squares part of the
        LSE problem.

    d : complex, array-like, shape (p)
        The right-hand side vector d for the equality constraints.

    Returns
    -------
    a : complex, ndarray, shape (m, `n`)
        `a` is overwritten.

    b : complex, ndarray, shape (p, `n`)
        `b` is overwritten.

    c : complex, ndarray, shape (m)
        The residual sum of squares for the solution vector x is given
        by the sum of squares of elements
        `c`[`n`-p],`c`[`n`-p+1],...,`c`[m-1]; the remaining elements are
        overwritten.

    d : complex, ndarray, shape (p)
        `d` is overwritten.

    x : complex, ndarray, shape (`n`)
        The solution vector x of the LSE problem.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -2)
            On entry, error in parameter `n`.

            Constraint: `n` >= 0.

        (`errno` -3)
            On entry, error in parameter p.

            Constraint: 0 <= p <= `n` <= m+p.

        (`errno` 1)
            The upper triangular factor R associated with B in the
            generalized RQ factorization of the pair (B,A) is singular,
            so that rank(B) < p; the least squares solution could not be
            computed.

        (`errno` 2)
            The (N-P)*(N-P) part of the upper trapezoidal factor T
            associated with A in the generalized RQ factorization of the
            pair (B,A) is singular, so that the rank of the matrix (E)
            comprising the rows of A and B is less than n; the least
            squares solutions could not be computed.

    Notes
    -----
    ``zgglse`` solves the complex linear equality-constrained least
    squares (LSE) problem

        minimize_x||c-Ax||_2 subject to Bx = d

    where A is an m*n matrix, B is a p*n matrix, c is an m element
    vector and d is a p element vector.
    It is assumed that p <= n <= m+p, rank(B) = p and rank(E) = n, where
    E = (A;B).
    These conditions ensure that the LSE problem has a unique solution,
    which is obtained using a generalized RQ factorization of the
    matrices B and A.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia

    Anderson, E, Bai, Z and Dongarra, J, 1992, `Generalized QR
    factorization and its applications`, Linear Algebra Appl. (Volume
    162--164), 243--271

    Eldèn, L, 1980, `Perturbation theory for the least squares problem
    with linear equality constraints`, SIAM J. Numer. Anal. (17),
    338--350
    """

    fun_name = 'naginterfaces.library.lapackeig.zgglse'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
    )
    _p_py = _b_utils._EngineComplex128ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[0]
    _p = _b_utils._EngineIntScalarType(
        _p_py, {'fun_name': fun_name, 'entity': '_p'},
    ).to_ctypes()
    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=True,
    )
    _d = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'd'},
        d,
        copy=True,
    )
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _x = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_n_py,),
    )
    _b_lapackeig.zgglse(
        _m,
        _n,
        _p,
        _a,
        _b,
        _c,
        _d,
        _x,
    )
    _a_py = _a
    _b_py = _b
    _c_py = _c
    _d_py = _d
    _x_py = _x
    _return_data = ZgglseReturnData(
        _a_py,
        _b_py,
        _c_py,
        _d_py,
        _x_py,
    )
    return _return_data

ZggglmReturnData = _collections.namedtuple(
    'ZggglmReturnData',
    ['a', 'b', 'd', 'x', 'y'],
)

def zggglm(p, a, b, d):
    u"""
    Solves a complex general Gauss--Markov linear model (GLM) problem.

    ``zggglm`` solves a complex general Gauss--Markov linear (least
    squares) model problem.

    For full information please refer to the NAG Library document for
    f08zp

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08zpf.html

    Parameters
    ----------
    p : int
        p, the number of columns of the matrix B.

    a : complex, array-like, shape (m, n)
        The m*n matrix A.

    b : complex, array-like, shape (m, `p`)
        The m*p matrix B.

    d : complex, array-like, shape (m)
        The left-hand side vector d of the GLM equation.

    Returns
    -------
    a : complex, ndarray, shape (m, n)
        `a` is overwritten.

    b : complex, ndarray, shape (m, `p`)
        `b` is overwritten.

    d : complex, ndarray, shape (m)
        `d` is overwritten.

    x : complex, ndarray, shape (n)
        The solution vector x of the GLM problem.

    y : complex, ndarray, shape (`p`)
        The solution vector y of the GLM problem.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -2)
            On entry, error in parameter n.

            Constraint: 0 <= n <= m.

        (`errno` -3)
            On entry, error in parameter `p`.

            Constraint: `p` >= m-n.

        (`errno` 1)
            The upper triangular factor R associated with A in the
            generalized RQ factorization of the pair (A,B) is singular,
            so that rank(A) < n; the least squares solution could not be
            computed.

        (`errno` 2)
            The bottom (m-n)*(m-n) part of the upper trapezoidal factor
            T associated with B in the generalized QR factorization of
            the pair (A,B) is singular, so that rank(A B) < n; the least
            squares solutions could not be computed.

    Notes
    -----
    ``zggglm`` solves the complex general Gauss--Markov linear model
    (GLM) problem

        minimize_x||y||_2 subject to d = Ax+By

    where A is an m*n matrix, B is an m*p matrix and d is an m element
    vector.
    It is assumed that n <= m <= n+p, rank(A) = n and rank(E) = m, where
    E = (A B).
    Under these assumptions, the problem has a unique solution x and a
    minimal 2-norm solution y, which is obtained using a generalized QR
    factorization of the matrices A and B.

    In particular, if the matrix B is square and nonsingular, then the
    GLM problem is equivalent to the weighted linear least squares
    problem

        minimize_x||B^-1(d-Ax)||_2.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia

    Anderson, E, Bai, Z and Dongarra, J, 1992, `Generalized QR
    factorization and its applications`, Linear Algebra Appl. (Volume
    162--164), 243--271
    """

    fun_name = 'naginterfaces.library.lapackeig.zggglm'

    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _d = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'd'},
        d,
        copy=True,
    )
    _x = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_n_py,),
    )
    _p = _b_utils._EngineIntScalarType(
        p, {'fun_name': fun_name, 'entity': 'p'},
    ).to_ctypes()
    _p_py = _b_utils._EngineIntScalarType.to_py(_p)
    _y = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (_p_py,),
    )
    _b_lapackeig.zggglm(
        _m,
        _n,
        _p,
        _a,
        _b,
        _d,
        _x,
        _y,
    )
    _a_py = _a
    _b_py = _b
    _d_py = _d
    _x_py = _x
    _y_py = _y
    _return_data = ZggglmReturnData(
        _a_py,
        _b_py,
        _d_py,
        _x_py,
        _y_py,
    )
    return _return_data

ZggqrfReturnData = _collections.namedtuple(
    'ZggqrfReturnData',
    ['a', 'taua', 'b', 'taub'],
)

def zggqrf(a, b):
    u"""
    Computes a generalized QR factorization of a complex matrix pair.

    ``zggqrf`` computes a generalized QR factorization of a complex
    matrix pair (A,B), where A is an n*m matrix and B is an n*p matrix.

    For full information please refer to the NAG Library document for
    f08zs

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08zsf.html

    Parameters
    ----------
    a : complex, array-like, shape (n, m)
        The n*m matrix A.

    b : complex, array-like, shape (n, p)
        The n*p matrix B.

    Returns
    -------
    a : complex, ndarray, shape (n, m)
        The elements on and above the diagonal of the array contain the
        min(n, m)*m upper trapezoidal matrix R (R is upper triangular if
        n >= m); the elements below the diagonal, with the array `taua`,
        represent the unitary matrix Q as a product of min(n, m)
        elementary reflectors (see [the F08 Introduction]).

    taua : complex, ndarray, shape (min(n, m))
        The scalar factors of the elementary reflectors which represent
        the unitary matrix Q.

    b : complex, ndarray, shape (n, p)
        If n <= p, the upper triangle of the subarray `b`[0:n,p-n:p]
        contains the n*n upper triangular matrix T_12.

        If n > p, the elements on and above the (n-p)th subdiagonal
        contain the n*p upper trapezoidal matrix T; the remaining
        elements, with the array `taub`, represent the unitary matrix Z
        as a product of elementary reflectors (see [the F08
        Introduction]).

    taub : complex, ndarray, shape (min(n, p))
        The scalar factors of the elementary reflectors which represent
        the unitary matrix Z.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter n.

            Constraint: n >= 0.

        (`errno` -2)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -3)
            On entry, error in parameter p.

            Constraint: p >= 0.

    Notes
    -----
    ``zggqrf`` forms the generalized QR factorization of an n*m matrix A
    and an n*p matrix B

        A = QR, B = QTZ,

    where Q is an n*n unitary matrix, Z is a p*p unitary matrix and R
    and T are of the form

        R = { [table omitted], if n >= m; ; [table omitted], if n < m,

    with R_11 upper triangular,

        T = { [table omitted], if n <= p, ; [table omitted], if n > p,

    with T_12 or T_21 upper triangular.

    In particular, if B is square and nonsingular, the generalized QR
    factorization of A and B implicitly gives the QR factorization of
    B^-1A as

        B^-1A = Z^H(T^-1R).

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Anderson, E, Bai, Z and Dongarra, J, 1992, `Generalized QR
    factorization and its applications`, Linear Algebra Appl. (Volume
    162--164), 243--271

    Hammarling, S, 1987, `The numerical solution of the general
    Gauss-Markov linear model`, Mathematics in Signal Processing, (eds T
    S Durrani, J B Abbiss, J E Hudson, R N Madan, J G McWhirter and T A
    Moore), 441--456, Oxford University Press

    Paige, C C, 1990, `Some aspects of generalized QR factorizations`, .
    In Reliable Numerical Computation, (eds M G Cox and S Hammarling),
    73--91, Oxford University Press
    """

    fun_name = 'naginterfaces.library.lapackeig.zggqrf'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
    )
    _p_py = _b_utils._EngineComplex128ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[1]
    _p = _b_utils._EngineIntScalarType(
        _p_py, {'fun_name': fun_name, 'entity': '_p'},
    ).to_ctypes()
    _taua = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (min(_n_py, _m_py),),
    )
    _taub = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (min(_n_py, _p_py),),
    )
    _b_lapackeig.zggqrf(
        _n,
        _m,
        _p,
        _a,
        _taua,
        _b,
        _taub,
    )
    _a_py = _a
    _b_py = _b
    _taua_py = _taua
    _taub_py = _taub
    _return_data = ZggqrfReturnData(
        _a_py,
        _taua_py,
        _b_py,
        _taub_py,
    )
    return _return_data

ZggrqfReturnData = _collections.namedtuple(
    'ZggrqfReturnData',
    ['a', 'taua', 'b', 'taub'],
)

def zggrqf(a, b):
    u"""
    Computes a generalized RQ factorization of a complex matrix pair.

    ``zggrqf`` computes a generalized RQ factorization of a complex
    matrix pair (A,B), where A is an m*n matrix and B is a p*n matrix.

    For full information please refer to the NAG Library document for
    f08zt

    https://www.nag.com/numeric/nl/nagdoc_27.3/flhtml/f08/f08ztf.html

    Parameters
    ----------
    a : complex, array-like, shape (m, n)
        The m*n matrix A.

    b : complex, array-like, shape (p, n)
        The p*n matrix B.

    Returns
    -------
    a : complex, ndarray, shape (m, n)
        If m <= n, the upper triangle of the subarray `a`[0:m,n-m:n]
        contains the m*m upper triangular matrix R_12.

        If m >= n, the elements on and above the (m-n)th subdiagonal
        contain the m*n upper trapezoidal matrix R; the remaining
        elements, with the array `taua`, represent the unitary matrix Q
        as a product of min(m, n) elementary reflectors (see [the F08
        Introduction]).

    taua : complex, ndarray, shape (min(m, n))
        The scalar factors of the elementary reflectors which represent
        the unitary matrix Q.

    b : complex, ndarray, shape (p, n)
        The elements on and above the diagonal of the array contain the
        min(p, n)*n upper trapezoidal matrix T (T is upper triangular if
        p >= n); the elements below the diagonal, with the array `taub`,
        represent the unitary matrix Z as a product of elementary
        reflectors (see [the F08 Introduction]).

    taub : complex, ndarray, shape (min(p, n))
        The scalar factors of the elementary reflectors which represent
        the unitary matrix Z.

    Raises
    ------
    NagValueError
        (`errno` -1)
            On entry, error in parameter m.

            Constraint: m >= 0.

        (`errno` -2)
            On entry, error in parameter p.

            Constraint: p >= 0.

        (`errno` -3)
            On entry, error in parameter n.

            Constraint: n >= 0.

    Notes
    -----
    ``zggrqf`` forms the generalized RQ factorization of an m*n matrix A
    and a p*n matrix B

        A = RQ, B = ZTQ,

    where Q is an n*n unitary matrix, Z is a p*p unitary matrix and R
    and T are of the form

        R = { [table omitted] ; if m <= n, ; [table omitted] ; if m > n,

    with R_12 or R_21 upper triangular,

        T = { [table omitted] ; if p >= n, ; [table omitted] ; if p < n,

    with T_11 upper triangular.

    In particular, if B is square and nonsingular, the generalized RQ
    factorization of A and B implicitly gives the RQ factorization of
    AB^-1 as

        AB^-1 = (RT^-1)Z^H.

    References
    ----------
    Anderson, E, Bai, Z, Bischof, C, Blackford, S, Demmel, J, Dongarra,
    J J, Du Croz, J J, Greenbaum, A, Hammarling, S, McKenney, A and
    Sorensen, D, 1999, `LAPACK Users' Guide`, (3rd Edition), SIAM,
    Philadelphia, https://www.netlib.org/lapack/lug

    Anderson, E, Bai, Z and Dongarra, J, 1992, `Generalized QR
    factorization and its applications`, Linear Algebra Appl. (Volume
    162--164), 243--271

    Hammarling, S, 1987, `The numerical solution of the general
    Gauss-Markov linear model`, Mathematics in Signal Processing, (eds T
    S Durrani, J B Abbiss, J E Hudson, R N Madan, J G McWhirter and T A
    Moore), 441--456, Oxford University Press

    Paige, C C, 1990, `Some aspects of generalized QR factorizations`, .
    In Reliable Numerical Computation, (eds M G Cox and S Hammarling),
    73--91, Oxford University Press
    """

    fun_name = 'naginterfaces.library.lapackeig.zggrqf'

    _a = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'a'},
        a,
        copy=True,
    )
    _m_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _b = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'b'},
        b,
        copy=True,
    )
    _p_py = _b_utils._EngineComplex128ArrayType.get_shape(_b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2)[0]
    _p = _b_utils._EngineIntScalarType(
        _p_py, {'fun_name': fun_name, 'entity': '_p'},
    ).to_ctypes()
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _taua = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (min(_m_py, _n_py),),
    )
    _taub = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        (min(_p_py, _n_py),),
    )
    _b_lapackeig.zggrqf(
        _m,
        _p,
        _n,
        _a,
        _taua,
        _b,
        _taub,
    )
    _a_py = _a
    _b_py = _b
    _taua_py = _taua
    _taub_py = _taub
    _return_data = ZggrqfReturnData(
        _a_py,
        _taua_py,
        _b_py,
        _taub_py,
    )
    return _return_data
